#
#  Copyright (C) 2008-2014 National Institute For Space Research (INPE) - Brazil.
#
#  This file is part of the TerraLib - a Framework for building GIS enabled applications.
#
#  TerraLib is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation, either version 3 of the License,
#  or (at your option) any later version.
#
#  TerraLib is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with TerraLib. See COPYING. If not, write to
#  TerraLib Team at <terralib-team@terralib.org>.
#
#
#  Description: Find Novile include directory and libraries.
#
#
# The following variables are set if novile is found:
#  NOVILE_FOUND        - Set to true when novile is found.
#  NOVILE_INCLUDE_DIR  - Include directories for novile
#  NOVILE_LIBRARY      - The novile libraries.
#
#  Author: Matheus Cavassan Zaglia <mzaglia@dpi.inpe.br>
#	

cmake_minimum_required(VERSION 2.8.8)
# Find library - - tries to find *.a,*.so,*.dylib in paths hard-coded by the script

find_path(NOVILE_INCLUDE_DIR NovileEditor
	HINTS  	/usr/include/ 
			/usr/local/include/ 
			/usr/local/novile/include/
    PATH_SUFFIXES include/novile
	              include
				  novile
)

if(UNIX)

 find_library(NOVILE_LIBRARY
   NAMES novile
   PATHS /usr/local/novile
         /usr/local
         /usr
   PATH_SUFFIXES lib 
                 lib/x86_64-linux-gnu
)

elseif(WIN32)  
  find_library(NOVILE_LIBRARY_RELEASE
		     NAMES novile
	  	     PATH_SUFFIXES lib 
  )
  
  find_library(NOVILE_LIBRARY_DEBUG
	           NAMES noviled				
  			   PATH_SUFFIXES lib 
  )
  if(NOVILE_LIBRARY_RELEASE AND NOVILE_LIBRARY_DEBUG)
    set(NOVILE_LIBRARY optimized ${NOVILE_LIBRARY_RELEASE} debug ${NOVILE_LIBRARY_DEBUG})
  elseif(NOVILE_LIBRARY_RELEASE)
    set(NOVILE_LIBRARY optimized ${NOVILE_LIBRARY_RELEASE} debug ${NOVILE_LIBRARY_RELEASE})
  elseif(NOVILE_LIBRARY_DEBUG)
    set(NOVILE_LIBRARY optimized ${NOVILE_LIBRARY_DEBUG} debug ${NOVILE_LIBRARY_DEBUG})
  endif()

endif()


# Export include and library path for linking with other libraries
if(NOVILE_INCLUDE_DIR AND NOVILE_LIBRARY)
	set(NOVILE_FOUND TRUE)
else()
	set(NOVILE_FOUND FALSE)
endif()


include(FindPackageHandleStandardArgs)

FIND_PACKAGE_HANDLE_STANDARD_ARGS(novile DEFAULT_MSG NOVILE_LIBRARY NOVILE_INCLUDE_DIR)

mark_as_advanced(NOVILE_LIBRARY NOVILE_INCLUDE_DIR)