#
#  Copyright (C) 2008-2014 National Institute For Space Research (INPE) - Brazil.
#
#  This file is part of the TerraLib - a Framework for building GIS enabled applications.
#
#  TerraLib is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation, either version 3 of the License,
#  or (at your option) any later version.
#
#  TerraLib is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with TerraLib. See COPYING. If not, write to
#  TerraLib Team at <terralib-team@terralib.org>.
#
#
#  Description: Find WTSS include directory and libraries.
#
#
# The following variables are set if WTSS is found:
#  WTSS_CXX_FOUND        - Set to true when WTSS is found.
#  WTSS_CXX_INCLUDE_DIR  - Include directories for WTSS
#  WTSS_CXX_LIBRARY      - The WTSS libraries.

#  Author: Matheus Cavassan Zaglia <mzaglia@dpi.inpe.br>
#	

cmake_minimum_required(VERSION 2.8.8)
# Find library - - tries to find *.a,*.so,*.dylib in paths hard-coded by the script

find_path(WTSS_CXX_INCLUDE_DIR wtss-cxx/wtss.hpp
	HINTS  	/usr/include/ 
			/usr/local/include/ 
			/usr/local/novile/include/
    PATH_SUFFIXES include/wtss-cxx
	              include
				  wtss-cxx
)

if(UNIX)

 find_library(WTSS_CXX_LIBRARY
   NAMES wtss_cxx_client
   PATHS /usr/local/novile
         /usr/local
         /usr
   PATH_SUFFIXES lib 
                 lib/x86_64-linux-gnu
)

elseif(WIN32)  
  find_library(WTSS_CXX_LIBRARY_RELEASE
		     NAMES wtss_cxx_client
	  	     PATH_SUFFIXES lib 
  )
  
  find_library(WTSS_CXX_LIBRARY_DEBUG
	           NAMES wtss_cxx_clientd				
  			   PATH_SUFFIXES lib 
  )
  if(WTSS_CXX_LIBRARY_RELEASE AND WTSS_CXX_LIBRARY_DEBUG)
    set(WTSS_CXX_LIBRARY optimized ${WTSS_CXX_LIBRARY_RELEASE} debug ${WTSS_CXX_LIBRARY_DEBUG})
  elseif(WTSS_CXX_LIBRARY_RELEASE)
    set(WTSS_CXX_LIBRARY optimized ${WTSS_CXX_LIBRARY_RELEASE} debug ${WTSS_CXX_LIBRARY_RELEASE})
  elseif(WTSS_CXX_LIBRARY_DEBUG)
    set(WTSS_CXX_LIBRARY optimized ${WTSS_CXX_LIBRARY_DEBUG} debug ${WTSS_CXX_LIBRARY_DEBUG})
  endif()

endif()


# Export include and library path for linking with other libraries
if(WTSS_CXX_INCLUDE_DIR AND WTSS_CXX_LIBRARY)
	set(WTSS_CXX_FOUND TRUE)
else()
	set(WTSS_CXX_FOUND FALSE)
endif()

mark_as_advanced(WTSS_CXX_LIBRARY WTSS_CXX_INCLUDE_DIR)
 
