#!/bin/bash

#
#  Copyright (C) 2008-2014 National Institute For Space Research (INPE) - Brazil.
#
#  This file is part of the TerraLib - a Framework for building GIS enabled applications.
#
#  TerraLib is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation, either version 3 of the License,
#  or (at your option) any later version.
#
#  TerraLib is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with TerraLib. See COPYING. If not, write to
#  TerraLib Team at <terralib-team@terralib.org>.
#
#  Description: It compiles TerraLib on Linux Ubuntu environment. Used also in CI server on Merge Request changes
#
#  Variables:
#  TL_CODEBASE - Path to TerraLib clone
#  TL_BUILD_DIR - Path to where generate makefiles
#

# Defining default value to TL_CODEBASE if no set
if [ -z "$TL_CODEBASE" ]; then
  export TL_CODEBASE=`pwd`/../../..
fi

# Defining default value to TL_BUILD_DIR if not set
if [ -z "$TL_BUILD_DIR" ]; then
  export TL_BUILD_DIR=$TL_CODEBASE/solution/build
fi

# Defining default value to TL_INSTALL_DIR if not set
if [ -z "$TL_INSTALL_DIR" ]; then
  export TL_INSTALL_DIR=$TL_CODEBASE/solution/install
fi

# Defining default value to TL_3RDPARTY if not set
if [ -z "$TL_3RDPARTY" ]; then
  export TL_3RDPARTY=$HOME/mydevel/terralib/3rdparty
fi

# Cleaning up
rm -rf $TL_BUILD_DIR
mkdir -p $TL_BUILD_DIR

cd $TL_BUILD_DIR

# Copying TerraLib cache to build dir
cp $TL_CODEBASE/build/scripts/linux/terralib.conf.cmake .

# Generating Makefiles
cmake -G "Unix Makefiles" -C terralib.conf.cmake -DCMAKE_BUILD_TYPE="Release" -DCMAKE_EXPORT_COMPILE_COMMANDS=ON $TL_CODEBASE/build/cmake

# Building..
make -j 4

# Installing
make install

# Returning exit code in order to jenkins be able to identify build status
exit $?
