::
::  Copyright (C) 2008-2014 National Institute For Space Research (INPE) - Brazil.
::
::  This file is part of the TerraLib - a Framework for building GIS enabled applications.
::
::  TerraLib is free software: you can redistribute it and/or modify
::  it under the terms of the GNU Lesser General Public License as published by
::  the Free Software Foundation, either version 3 of the License,
::  or (at your option) any later version.
::
::  TerraLib is distributed in the hope that it will be useful,
::  but WITHOUT ANY WARRANTY; without even the implied warranty of
::  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
::  GNU Lesser General Public License for more details.
::
::  You should have received a copy of the GNU Lesser General Public License
::  along with TerraLib. See COPYING. If not, write to
::  TerraLib Team at <terralib-team@terralib.org>.
::

::
:: Description: It compiles TerraLib on Windows 10 environment. Used also in CI server on Merge Request changes
::

:: Turn off system messages
@echo off
:: Clear screen
cls;

:: ====

:: TerraLib Paths 
IF NOT DEFINED TERRALIB_DEPENDENCIES_DIR (
set "TERRALIB_DEPENDENCIES_DIR=D:\mylibs"
)

IF NOT DEFINED TERRALIB_CODEBASE_PATH (
set "TERRALIB_CODEBASE_PATH=D:\mydevel\terralib\codebase"
)

IF NOT DEFINED TERRALIB_BUILD_PATH (
set "TERRALIB_BUILD_PATH=%TERRALIB_CODEBASE_PATH%\..\build-package"
)

IF NOT DEFINED TERRALIB_INSTALL_DIR (
set "TERRALIB_INSTALL_DIR=%TERRALIB_CODEBASE_PATH%\..\terralib-install"
)

:: Where is qmake.exe
IF NOT DEFINED QMAKE_FILEPATH (
set "QMAKE_FILEPATH=C:\Qt\5.10.1\msvc2017_64\bin"
)

:: Where is Visual Studio
set "VCVARS_FILEPATH=%PROGRAMFILES(x86)%\Microsoft Visual Studio\2017\Community\VC\Auxiliary\Build"

:: Where is cmake.exe
IF NOT DEFINED CMAKE_FILEPATH (
  set "CMAKE_FILEPATH=%PROGRAMFILES%\CMake\bin"
)


:: Where is script file
set "ROOT_DIR=%CD%"
:: Environment Paths
set "PATH=%CMAKE_FILEPATH%;%QMAKE_FILEPATH%;%PATH%"

::  Setting visual studio environment
::  ==================================

set _CMAKE_GENERATOR="Visual Studio 15 2017 Win64" 

echo | set /p="Configuring visual studio... "<nul

call "%VCVARS_FILEPATH%"\vcvarsall.bat x86_amd64 

echo done.
echo.

::  Creating and configuring build folder
::  ======================================

echo | set /p="Configuring build folder... "<nul

rmdir %TERRALIB_BUILD_PATH% /s /q
mkdir %TERRALIB_BUILD_PATH% >nul 2>nul

xcopy terralib.conf.cmake %TERRALIB_BUILD_PATH% /S /Y >nul 2>nul

echo done.
echo.

cd /d %TERRALIB_BUILD_PATH% >nul 2>nul

::  Configuring CMake and generating files
::  =======================================

echo | set /p="Configuring CMake... "<nul

cmake -G %_CMAKE_GENERATOR% -C ./terralib.conf.cmake -DCMAKE_BUILD_TYPE="Release" %TERRALIB_CODEBASE_PATH%\build\cmake

echo done.
echo.

::  Building VS2013 all Project
::  ============================

echo | set /p="Building project... "<nul

msbuild /m /t:clean ALL_BUILD.vcxproj /p:Configuration=Release
msbuild /m ALL_BUILD.vcxproj /p:Configuration=Release

set RES=%ERRORLEVEL%

echo done.
echo.

exit %ERRORLEVEL%