
// STL
#include <cstdlib>
#include <string>
#include <iostream>

//TerraLib
#include "../../src/terralib/ws/ogc/wms/client/XMLParser.h"
#include "../../src/terralib/common/TerraLib.h"
#include "../../src/terralib/qt/af/ApplicationController.h"
#include "../../src/terralib/ws/ogc/wms/client/WMSClient.h"

#include <QApplication>

int main(int argc, char *argv[])
{
  TerraLib::getInstance().initialize();

  QApplication app(argc, argv);

  te::qt::af::AppCtrlSingleton::getInstance().initialize();

  // Directory where temporary WMS data will be stored.
  std::string usrDataDir = te::qt::af::AppCtrlSingleton::getInstance().getUserDataDir().toUtf8().data();

  // Version that will be used on WMS Requests.
  std::string version = "1.3.0";

  // WMS server URL.
  std::string url = "http://terrabrasilis.info/terraamazon/ows";

  {
    std::cout << "GetCapabilities Example." << std::endl;

    te::ws::ogc::WMSClient client (usrDataDir, url, version);

    client.updateCapabilities();

    te::ws::ogc::wms::WMSCapabilities capabilities = client.getCapabilities();

    te::ws::ogc::wms::Layer rootLayer = capabilities.m_capability.m_layer;

    std::vector<te::ws::ogc::wms::Layer> childLayers = rootLayer.m_layers;

    for(size_t i = 0; i < childLayers.size(); i++)
    {
      te::ws::ogc::wms::Layer childLayer = childLayers[i];

      std::cout << "Layer Info" << std::endl;

      std::cout << "Layer Name: " << childLayer.m_name << std::endl;
      std::cout << "Layer Title: " << childLayer.m_title << std::endl;

      std::cout << "Layer Possible Bounding Boxes" << std::endl;

      for(size_t j = 0; j < childLayer.m_boundingBoxes.size(); j++)
      {
        te::ws::ogc::wms::BoundingBox bbox = childLayer.m_boundingBoxes[j];

        std::cout << "BoundingBox CRS: " << bbox.m_crs << std::endl;
        std::cout << "minx: " << bbox.m_minX << std::endl;
        std::cout << "maxx: " << bbox.m_maxX << std::endl;
        std::cout << "minx: " << bbox.m_minX << std::endl;
        std::cout << "maxy: " << bbox.m_maxY << std::endl;
      }

      for(size_t j = 0; j < childLayer.m_styles.size(); j++)
      {
        te::ws::ogc::wms::Style style = childLayer.m_styles[j];

        std::cout << "Style: " << style.m_name << std::endl;
        std::cout << "LegendURL: " << style.m_legendURL.m_onlineResource.m_href << std::endl;
      }

      std::cout << "============================" << std::endl;
    }
  }

  {
    std::cout << "GetMap Example." << std::endl;

    te::ws::ogc::WMSClient client (usrDataDir, url);

    client.updateCapabilities();

    te::ws::ogc::wms::WMSCapabilities capabilities = client.getCapabilities();

    te::ws::ogc::wms::Layer rootLayer = capabilities.m_capability.m_layer;

    std::vector<te::ws::ogc::wms::Layer> childLayers = rootLayer.m_layers;

    te::ws::ogc::wms::Layer layer = childLayers[0];

    te::ws::ogc::wms::BoundingBox bbox = layer.m_boundingBoxes[1];

    // GetMap request parameters
    te::ws::ogc::wms::WMSGetMapRequest request;
    request.m_layers.push_back(layer.m_name);
    request.m_srs = bbox.m_crs;
    request.m_boundingBox = bbox;
    request.m_format = "image/png";
    request.m_transparent = true;
    request.m_width = 500;
    request.m_height = 500;

    if(!layer.m_styles.empty())
    {
      request.m_styles.push_back(layer.m_styles[0].m_name);
    }

    //GetMap response with image buffer
    te::ws::ogc::wms::WMSGetMapResponse response = client.getMap(request);

    std::cout << "Buffer Size: " << response.m_size << std::endl;
    std::cout << "Buffer Format: " << response.m_format << std::endl;
  }

  std::cout << "============================" << std::endl;

  {
    std::cout << "GetFeatureInfo Example." << std::endl;

    te::ws::ogc::WMSClient client (usrDataDir, url);

    client.updateCapabilities();

    te::ws::ogc::wms::WMSCapabilities capabilities = client.getCapabilities();

    te::ws::ogc::wms::Layer rootLayer = capabilities.m_capability.m_layer;

    std::vector<te::ws::ogc::wms::Layer> childLayers = rootLayer.m_layers;

    te::ws::ogc::wms::Layer layer = childLayers[0];

    te::ws::ogc::wms::BoundingBox bbox = layer.m_boundingBoxes[1];

        // GetMap request parameters
    te::ws::ogc::wms::WMSGetMapRequest getMap;
    getMap.m_layers.push_back(layer.m_name);
    getMap.m_srs = bbox.m_crs;
    getMap.m_boundingBox = bbox;
    getMap.m_format = "image/png";
    getMap.m_transparent = true;
    getMap.m_width = 500;
    getMap.m_height = 500;

    if(!layer.m_styles.empty())
    {
      getMap.m_styles.push_back(layer.m_styles[0].m_name);
    }

    te::ws::ogc::wms::WMSGetFeatureInfoRequest request;
    request.m_queryLayers.push_back(layer.m_name);
    request.m_infoFormat = "text/plain";
    request.m_featureCount = 1;
    request.m_i = 250;
    request.m_j = 250;
    request.m_exceptions = "application/vnd.ogc.se_xml";
    request.m_getMap = getMap;
    
    te::ws::ogc::wms::WMSGetFeatureInfoResponse response = client.getFeatureInfo(request);

    std::cout << response.m_featureInfoContent << std::endl;
    std::cout << "Feature Info Format: " << response.m_infoFormat << std::endl;
  }

  TerraLib::getInstance().finalize();

  return EXIT_SUCCESS;
}
