/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*! 
  \file terralib/process/qt/CalcTaskProcess.h

  \brief Example of Tasks
*/

#ifndef __TERRALIB_PROCESS_QT_INTERNAL_CALCTASKPROCESS_H
#define __TERRALIB_PROCESS_QT_INTERNAL_CALCTASKPROCESS_H

#include "../src/terralib/datatype/Enums.h"
#include "../src/terralib/datatype/SimpleData.h"
#include "../src/terralib/process/Task.h"
#include "../src/terralib/process/TaskCapabilities.h"
#include "../src/terralib/process/SimpleDataParameterValue.h"


namespace te
{
  namespace process
  {
    namespace example
    {
      class SumIntTask : public te::process::Task
      {
      public:

        SumIntTask() : te::process::Task("Addition")
        {
          te::process::TaskCapabilities* capabilities = new te::process::TaskCapabilities();
          capabilities->addParameterDescriptor(te::process::ParameterDescriptor("number1", te::dt::INT32_TYPE, true));
          capabilities->addParameterDescriptor(te::process::ParameterDescriptor("number2", te::dt::INT32_TYPE, true));
          capabilities->addParameterDescriptor(te::process::ParameterDescriptor("result", te::dt::INT32_TYPE, false, te::process::ParameterDescriptor::Direction::OUTPUT));
          init(capabilities);
        }

      protected:

        void runImpl()
        {
          const te::process::ParameterValue* pValue1 = this->getParameter(0);
          const te::process::ParameterValue* pValue2 = this->getParameter(1);

          const te::dt::AbstractData* absData1 = (static_cast<const te::process::SimpleDataParameterValue*>(pValue1))->getValue();
          const te::dt::AbstractData* absData2 = (static_cast<const te::process::SimpleDataParameterValue*>(pValue2))->getValue();

          const te::dt::Int32* data1 = static_cast<const te::dt::Int32*>(absData1);
          const te::dt::Int32* data2 = static_cast<const te::dt::Int32*>(absData2);

          boost::int32_t sumResult = data1->getValue() + data2->getValue();
          te::dt::Int32* sumResultData = new te::dt::Int32(sumResult);

          te::process::SimpleDataParameterValue* sumResultValue = new te::process::SimpleDataParameterValue(sumResultData);
          this->setParameter(2, sumResultValue, true);
        };
      };

      class SumIntTaskFactory : public te::process::TaskFactory
      {
      public:

        SumIntTaskFactory(): TaskFactory("Addition", "Calc")
        {
        }

        ~SumIntTaskFactory() = default;

      protected:

        Task* build() override
        {
          return new SumIntTask();
        }
      };

      static SumIntTaskFactory  sumIntTaskFactoryInstance;

      class Sum3IntTask : public te::process::Task
      {
        public:

        Sum3IntTask() : te::process::Task("Addition 3")
        {
          te::process::TaskCapabilities* capabilities = new te::process::TaskCapabilities();
          capabilities->addParameterDescriptor(te::process::ParameterDescriptor("number1", te::dt::INT32_TYPE, true));
          capabilities->addParameterDescriptor(te::process::ParameterDescriptor("number2", te::dt::INT32_TYPE, true));
          capabilities->addParameterDescriptor(te::process::ParameterDescriptor("number3", te::dt::INT32_TYPE, true));
          capabilities->addParameterDescriptor(te::process::ParameterDescriptor("result", te::dt::INT32_TYPE, false, te::process::ParameterDescriptor::Direction::OUTPUT));
          init(capabilities);
        }

      protected:

        void runImpl()
        {
          const te::process::ParameterValue* pValue1 = this->getParameter(0);
          const te::process::ParameterValue* pValue2 = this->getParameter(1);
          const te::process::ParameterValue* pValue3 = this->getParameter(2);

          const te::dt::AbstractData* absData1 = (static_cast<const te::process::SimpleDataParameterValue*>(pValue1))->getValue();
          const te::dt::AbstractData* absData2 = (static_cast<const te::process::SimpleDataParameterValue*>(pValue2))->getValue();
          const te::dt::AbstractData* absData3 = (static_cast<const te::process::SimpleDataParameterValue*>(pValue3))->getValue();

          const te::dt::Int32* data1 = static_cast<const te::dt::Int32*>(absData1);
          const te::dt::Int32* data2 = static_cast<const te::dt::Int32*>(absData2);
          const te::dt::Int32* data3 = static_cast<const te::dt::Int32*>(absData3);

          boost::int32_t sumResult = data1->getValue() + data2->getValue() + data3->getValue();
          te::dt::Int32* sumResultData = new te::dt::Int32(sumResult);

          te::process::SimpleDataParameterValue* sumResultValue = new te::process::SimpleDataParameterValue(sumResultData);
          this->setParameter(2, sumResultValue, true);
        };
      };

      class Sum3IntTaskFactory : public te::process::TaskFactory
      {
      public:

        Sum3IntTaskFactory() : TaskFactory("Addition 3", "Calc")
        {
        }

        ~Sum3IntTaskFactory() = default;

      protected:

        Task* build() override
        {
          return new Sum3IntTask();
        }
      };

      static Sum3IntTaskFactory  sum3IntTaskFactoryInstance;

      class SubIntTask : public te::process::Task
      {
      public:

        SubIntTask() : te::process::Task("Subtraction")
        {
          te::process::TaskCapabilities* capabilities = new te::process::TaskCapabilities();
          capabilities->addParameterDescriptor(te::process::ParameterDescriptor("number1", te::dt::INT32_TYPE, true));
          capabilities->addParameterDescriptor(te::process::ParameterDescriptor("number2", te::dt::INT32_TYPE, true));
          capabilities->addParameterDescriptor(te::process::ParameterDescriptor("result", te::dt::INT32_TYPE, false, te::process::ParameterDescriptor::Direction::OUTPUT));
          init(capabilities);
        }

      protected:

        void runImpl()
        {
          const te::process::ParameterValue* pValue1 = this->getParameter(0);
          const te::process::ParameterValue* pValue2 = this->getParameter(1);

          const te::dt::AbstractData* absData1 = (static_cast<const te::process::SimpleDataParameterValue*>(pValue1))->getValue();
          const te::dt::AbstractData* absData2 = (static_cast<const te::process::SimpleDataParameterValue*>(pValue2))->getValue();

          const te::dt::Int32* data1 = static_cast<const te::dt::Int32*>(absData1);
          const te::dt::Int32* data2 = static_cast<const te::dt::Int32*>(absData2);

          boost::int32_t sumResult = data1->getValue() - data2->getValue();
          te::dt::Int32* sumResultData = new te::dt::Int32(sumResult);

          te::process::SimpleDataParameterValue* sumResultValue = new te::process::SimpleDataParameterValue(sumResultData);
          this->setParameter(2, sumResultValue, true);
        };
      };

      class SubIntTaskFactory : public te::process::TaskFactory
      {
      public:

        SubIntTaskFactory() : TaskFactory("Subtraction", "Calc")
        {
        }

        ~SubIntTaskFactory() = default;

      protected:

        Task* build() override
        {
          return new SubIntTask();
        }
      };

      static SubIntTaskFactory  subIntTaskFactoryInstance;

      class MultiIntTask : public te::process::Task
      {
      public:

        MultiIntTask() : te::process::Task("Multiplication")
        {
          te::process::TaskCapabilities* capabilities = new te::process::TaskCapabilities();
          capabilities->addParameterDescriptor(te::process::ParameterDescriptor("number1", te::dt::INT32_TYPE, true));
          capabilities->addParameterDescriptor(te::process::ParameterDescriptor("number2", te::dt::INT32_TYPE, true));
          capabilities->addParameterDescriptor(te::process::ParameterDescriptor("result", te::dt::INT32_TYPE, false, te::process::ParameterDescriptor::Direction::OUTPUT));
          init(capabilities);
        }

      protected:

        void runImpl()
        {
          const te::process::ParameterValue* pValue1 = this->getParameter(0);
          const te::process::ParameterValue* pValue2 = this->getParameter(1);

          const te::dt::AbstractData* absData1 = (static_cast<const te::process::SimpleDataParameterValue*>(pValue1))->getValue();
          const te::dt::AbstractData* absData2 = (static_cast<const te::process::SimpleDataParameterValue*>(pValue2))->getValue();

          const te::dt::Int32* data1 = static_cast<const te::dt::Int32*>(absData1);
          const te::dt::Int32* data2 = static_cast<const te::dt::Int32*>(absData2);

          boost::int32_t sumResult = data1->getValue() * data2->getValue();
          te::dt::Int32* sumResultData = new te::dt::Int32(sumResult);

          te::process::SimpleDataParameterValue* sumResultValue = new te::process::SimpleDataParameterValue(sumResultData);
          this->setParameter(2, sumResultValue, true);
        };
      };

      class MultiIntTaskFactory : public te::process::TaskFactory
      {
      public:

        MultiIntTaskFactory() : TaskFactory("Multiplication", "Calc")
        {
        }

        ~MultiIntTaskFactory() = default;

      protected:

        Task* build() override
        {
          return new MultiIntTask();
        }
      };

      static MultiIntTaskFactory  multiIntTaskFactoryInstance;

      class DivIntTask : public te::process::Task
      {
      public:

        DivIntTask() : te::process::Task("Division")
        {
          te::process::TaskCapabilities* capabilities = new te::process::TaskCapabilities();
          capabilities->addParameterDescriptor(te::process::ParameterDescriptor("number1", te::dt::INT32_TYPE, true));
          capabilities->addParameterDescriptor(te::process::ParameterDescriptor("number2", te::dt::INT32_TYPE, true));
          capabilities->addParameterDescriptor(te::process::ParameterDescriptor("result", te::dt::INT32_TYPE, false, te::process::ParameterDescriptor::Direction::OUTPUT));
          init(capabilities);
        }

      protected:

        void runImpl()
        {
          const te::process::ParameterValue* pValue1 = this->getParameter(0);
          const te::process::ParameterValue* pValue2 = this->getParameter(1);

          const te::dt::AbstractData* absData1 = (static_cast<const te::process::SimpleDataParameterValue*>(pValue1))->getValue();
          const te::dt::AbstractData* absData2 = (static_cast<const te::process::SimpleDataParameterValue*>(pValue2))->getValue();

          const te::dt::Int32* data1 = static_cast<const te::dt::Int32*>(absData1);
          const te::dt::Int32* data2 = static_cast<const te::dt::Int32*>(absData2);

          boost::int32_t sumResult = data1->getValue() / data2->getValue();
          te::dt::Int32* sumResultData = new te::dt::Int32(sumResult);

          te::process::SimpleDataParameterValue* sumResultValue = new te::process::SimpleDataParameterValue(sumResultData);
          this->setParameter(2, sumResultValue, true);
        };
      };

      class DivIntTaskFactory : public te::process::TaskFactory
      {
      public:

        DivIntTaskFactory() : TaskFactory("Division", "Calc")
        {
        }

        ~DivIntTaskFactory() = default;

      protected:

        Task* build() override
        {
          return new DivIntTask();
        }
      };

      static DivIntTaskFactory  divIntTaskFactoryInstance;
      
    }
  }
}

#endif // __TERRALIB_PROCESS_QT_INTERNAL_TASKPROCESSTREEGROUP_H
