/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*! 
  \file terralib/process/qt/CalcTaskProcessTreeGroup.h

  \brief A QTreeWidgetItem for Calc Task Group of Process Builder items.
*/

#ifndef __TERRALIB_PROCESS_QT_INTERNAL_CALCTASKPROCESSTREEGROUP_H
#define __TERRALIB_PROCESS_QT_INTERNAL_CALCTASKPROCESSTREEGROUP_H

#include "../../src/terralib/process/Task.h"
#include "../../src/terralib/process/qt/explorer/AbstractProcessTreeGroup.h"
#include "../../src/terralib/process/qt/explorer/TaskProcessTreeItem.h"

namespace te
{
  namespace process
  {
    namespace qt
    {
      class ProcessExplorerTree;
    }

    namespace example
    {
      /*!
      \class TaskProcessTreeGroup

      \brief A QTreeWidgetItem Task Group of Process Builder items.

      */
      class CalcTaskProcessTreeGroup : public te::process::qt::AbstractProcessTreeGroup
      {
      public:

        /*! \brief Default constructor. */
        CalcTaskProcessTreeGroup(te::process::qt::ProcessExplorerTree* parent) :
          AbstractProcessTreeGroup(parent, "Calc Tasks")
        {
          setupGroup();
        }

        /*! \brief Virtual destructor. */
        virtual ~CalcTaskProcessTreeGroup() = default;

      protected:

        /*!
          \brief Virtual function used to setup process explorer tree items into a group.
        */
        virtual void setupGroup()
        {
          std::vector<std::string> names = te::process::TaskFactory::getTasksFromCategory("Calc");

          for (std::size_t i = 0; i < names.size(); ++i)
          {
            te::process::qt::TaskProcessTreeItem* item = new te::process::qt::TaskProcessTreeItem(m_root, names[i]);
            item->setText(0, names[i].c_str());
            item->setIcon(0, QIcon::fromTheme("D:/Workspace/terralib_55/share/terralib/icons/terralib/scalable/gear.svg"));
            item->setData(0, Qt::UserRole, QVariant(names[i].c_str()));

            m_root->addChild(item);
            m_root->setExpanded(true);
          }
        }

      protected:
      };
    }
  }
}

#endif // __TERRALIB_PROCESS_QT_INTERNAL_CALCTASKPROCESSTREEGROUP_H
