#include "CalcTaskProcess.h"
#include "CalcTaskProcessTreeGroup.h"

// TerraLib
#include <terralib/common/TerraLib.h>
#include <terralib/core/utils/Platform.h>
#include <terralib/process/qt/ProcessBuilder.h>
#include <terralib/process/qt/explorer/ProcessExplorerTree.h>


// STL
#include <cstdlib>
#include <iostream>
#include <time.h>

// Qt
#include <QApplication>

void LoadModules();

void UnloadModules();

int main(int argc, char *argv[])
{
  QApplication app(argc, argv);

  try
  {
    //init components
    TerraLib::getInstance().initialize();

    //start plugins
    LoadModules();

    //start theme support
    QStringList ithemes = QIcon::themeSearchPaths();

    ithemes.push_back(te::core::FindInTerraLibPath("share/terralib/icons").c_str());

    QIcon::setThemeSearchPaths(ithemes);

    QIcon::setThemeName("terralib");

    //start builder
    te::process::qt::ProcessBuilder builder;

    te::process::qt::ProcessExplorerTree* tree = builder.getExplorerTree();

    te::process::example::CalcTaskProcessTreeGroup* calcTreeGroup = new te::process::example::CalcTaskProcessTreeGroup(tree);

    builder.showMaximized();

    app.exec();

    UnloadModules();

    TerraLib::getInstance().finalize();
  }
  catch(const std::exception& e)
  {
    std::cout << std::endl << "An exception has occurred in the Process examples: " << e.what() << std::endl;

    return EXIT_FAILURE;
  }
  catch(...)
  {
    std::cout << std::endl << "An unexpected exception has occurred in the Process examples!" << std::endl;

    return EXIT_FAILURE;
  }

  return EXIT_SUCCESS;
}
