/*
  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

  This file is part of the TerraLib - a Framework for building GIS enabled
  applications.

  TerraLib is free software: you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation, either version 3 of the License,
  or (at your option) any later version.

  TerraLib is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with TerraLib. See COPYING. If not, write to
  TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/examples/vm/main.cpp

  \brief Examples for the Terralib Virtual Machine Module

  \author Frederico Augusto Bedê Teotônio
*/

// TerraLib
#include <terralib/common.h>
#include <terralib/core.h>

#include <terralib/vm.h>

// STL
#include <iostream>
#include <string>

int main(int /*argc*/, char** /*argv*/)
{
  TerraLib::getInstance().initialize();

  try
  {
    // load Lua Virtual Machine
    std::string lvm_lua_manifest = te::core::FindInTerraLibPath(
        "share/terralib/plugins/te.vm.lua.teplg.json");
    std::string lvm_python_manifest = te::core::FindInTerraLibPath(
        "share/terralib/plugins/te.vm.python.teplg.json");
    te::core::PluginInfo lvm_lua_info(
        te::core::JSONPluginInfoSerializer(lvm_lua_manifest));
    te::core::PluginInfo lvm_python_info(
        te::core::JSONPluginInfoSerializer(lvm_python_manifest));
    te::core::PluginManager::instance().insert(lvm_lua_info);
    te::core::PluginManager::instance().load(lvm_lua_info.name);
    te::core::PluginManager::instance().insert(lvm_python_info);
    te::core::PluginManager::instance().load(lvm_python_info.name);

    std::cout << "=== LUA VIRTUAL MACHINE ===\n";

    std::string lua_script =
        te::core::FindInTerraLibPath("examples/vm/vm_example.lua");

    te::vm::core::VirtualMachine* vm_lua =
        te::vm::core::VirtualMachineManager::instance().get("lua");

    vm_lua->setFileName(lua_script.c_str());

    vm_lua->execute();

    std::cout << "=== PYTHON VIRTUAL MACHINE ===\n";

    std::string py_script =
        te::core::FindInTerraLibPath("examples/vm/vm_example.py");

    te::vm::core::VirtualMachine* vm_python =
        te::vm::core::VirtualMachineManager::instance().get("python");

    vm_python->setFileName(py_script.c_str());

    vm_python->execute();

    te::core::PluginManager::instance().clear();
  }
  catch(const te::vm::core::Exception& e)
  {
    if(const std::string* d = boost::get_error_info<te::ErrorDescription>(e))
    {
      std::cerr << *d << std::endl;
    }
  }

  TerraLib::getInstance().finalize();

  return EXIT_SUCCESS;
}
