#include "../Config.h"
#include <terralib/common.h>
#include <terralib/dataaccess/datasource/DataSourceFactory.h>
#include <terralib/dataaccess/datasource/DataSourceInfoManager.h>
#include <terralib/vp/ComplexData.h>
#include <terralib/vp/Identity.h>

// Boost
#include <boost/uuid/random_generator.hpp>
#include <boost/uuid/uuid_io.hpp>

// STL
#include <iostream>

//OGR to OGR
bool IdentityOGRToOGR()
{
  std::string data_dir = TERRALIB_DATA_DIR;

  std::string input_filename(data_dir + "/shape/munic_goias.shp");
  std::string srcInputInfo ("file://" + input_filename);

  std::string intersection_filename(data_dir +
                                    "/shape/munic_goias_selected.shp");
  std::string srcIntersectionInfo ("file://" + intersection_filename);


  te::da::DataSourcePtr srcInputDs(
      te::da::DataSourceFactory::make("OGR", srcInputInfo));
  srcInputDs->open();

  te::da::DataSourcePtr srcIntersectionDs(
      te::da::DataSourceFactory::make("OGR", srcIntersectionInfo));
  srcIntersectionDs->open();


  std::string input_DsetName = "munic_goias";
  std::string intersection_DsetName = "munic_goias_selected";

  if (!srcInputDs->dataSetExists(input_DsetName))
  {
    std::cout << "Input dataset not found: " << input_DsetName << std::endl;
    return false;
  }

  std::unique_ptr<te::da::DataSet> input_Dset =
      srcInputDs->getDataSet(input_DsetName);

  std::unique_ptr<te::da::DataSetType> input_DsetType =
      srcInputDs->getDataSetType(input_DsetName);

  if (!srcIntersectionDs->dataSetExists(intersection_DsetName))
  {
    std::cout << "Input dataset not found: " << intersection_DsetName
              << std::endl;
    return false;
  }

  std::unique_ptr<te::da::DataSet> intersection_Dset =
      srcIntersectionDs->getDataSet(intersection_DsetName);

  std::unique_ptr<te::da::DataSetType> intersection_DsetType =
      srcIntersectionDs->getDataSetType(intersection_DsetName);

  std::vector<std::string> attributes;
  attributes.push_back("nome");
  attributes.push_back("nomemeso");

  std::map<std::string, te::dt::AbstractData*> specificParams;
  specificParams["munic_goias"] =
      new te::vp::ComplexData<std::vector<std::string> >(attributes);
  specificParams["munic_goias_selected"] =
      new te::vp::ComplexData<std::vector<std::string> >(attributes);

  std::string output_filename(data_dir + "/shape/identity_shptoshp_ex.shp");
  std::string tgrInfo ("file://" + output_filename);

  te::da::DataSourcePtr trgDs(te::da::DataSourceFactory::make("OGR", tgrInfo));
  trgDs->open();

  std::string outDSName = "identity_shptoshp_ex";

  if (trgDs->dataSetExists(outDSName))
  {
    std::cout << "A dataset with the same requested output dataset name "
                 "already exists:" << outDSName << std::endl;
    return false;
  }

  std::vector<te::vp::InputParams> inputParamsVec;
  te::vp::InputParams structInputParams;
  te::vp::InputParams structIntersectionParams;

  structInputParams.m_inputDataSource = srcInputDs;
  structInputParams.m_inputDataSetType = input_DsetType.release();
  structInputParams.m_inputDataSet = input_Dset.release();

  structIntersectionParams.m_inputDataSource = srcIntersectionDs;
  structIntersectionParams.m_inputDataSetType = intersection_DsetType.release();
  structIntersectionParams.m_inputDataSet = intersection_Dset.release();

  inputParamsVec.push_back(structInputParams);
  inputParamsVec.push_back(structIntersectionParams);

  te::vp::AlgorithmParams* params = new te::vp::AlgorithmParams();
  params->setInputParams(inputParamsVec);
  params->setOutputDataSource(trgDs);
  params->setOutputDataSetName(outDSName);
  params->setSpecificParams(specificParams);

  te::vp::Identity identity;
  bool result = identity.executeMemory(params);

  delete params;

  return result;
}

//OGR to Postgis
bool IdentityOGRToPGIS()
{
  std::string data_dir = TERRALIB_DATA_DIR;

  std::string input_filename(data_dir + "/shape/munic_goias.shp");
  std::string srcInputInfo ("file://" + input_filename);

  std::string intersection_filename(data_dir +
                                    "/shape/munic_goias_selected.shp");
  std::string srcIntersectionInfo ("file://" + intersection_filename);


  te::da::DataSourcePtr srcInputDs(
      te::da::DataSourceFactory::make("OGR", srcInputInfo));
  srcInputDs->open();

  te::da::DataSourcePtr srcIntersectionDs(
      te::da::DataSourceFactory::make("OGR", srcIntersectionInfo));
  srcIntersectionDs->open();


  std::string input_DsetName = "munic_goias";
  std::string intersection_DsetName = "munic_goias_selected";

  if (!srcInputDs->dataSetExists(input_DsetName))
  {
    std::cout << "Input dataset not found: " << input_DsetName << std::endl;
    return false;
  }

  std::unique_ptr<te::da::DataSet> input_Dset =
      srcInputDs->getDataSet(input_DsetName);

  std::unique_ptr<te::da::DataSetType> input_DsetType =
      srcInputDs->getDataSetType(input_DsetName);

  if (!srcIntersectionDs->dataSetExists(intersection_DsetName))
  {
    std::cout << "Input dataset not found: " << intersection_DsetName
              << std::endl;
    return false;
  }

  std::unique_ptr<te::da::DataSet> intersection_Dset =
      srcIntersectionDs->getDataSet(intersection_DsetName);

  std::unique_ptr<te::da::DataSetType> intersection_DsetType =
      srcIntersectionDs->getDataSetType(intersection_DsetName);

  std::vector<std::string> attributes;
  attributes.push_back("nome");
  attributes.push_back("nomemeso");

  std::map<std::string, te::dt::AbstractData*> specificParams;
  specificParams["munic_goias"] =
      new te::vp::ComplexData<std::vector<std::string> >(attributes);
  specificParams["munic_goias_selected"] =
      new te::vp::ComplexData<std::vector<std::string> >(attributes);

  std::string connInfo(
      "ppgsql://postgres:postgres@atlas.dpi.inpe.br:5433/testPostGIS");

  te::da::DataSourcePtr trgDs(
      te::da::DataSourceFactory::make("POSTGIS", connInfo));
  trgDs->open();

  std::string outDSName = "identity_shptopgis_ex";

  if (trgDs->dataSetExists(outDSName))
  {
    std::cout << "A dataset with the same requested output dataset name "
                 "already exists:" << outDSName << std::endl;
    return false;
  }

  std::vector<te::vp::InputParams> inputParamsVec;
  te::vp::InputParams structInputParams;
  te::vp::InputParams structIntersectionParams;

  structInputParams.m_inputDataSource = srcInputDs;
  structInputParams.m_inputDataSetType = input_DsetType.release();
  structInputParams.m_inputDataSet = input_Dset.release();

  structIntersectionParams.m_inputDataSource = srcIntersectionDs;
  structIntersectionParams.m_inputDataSetType = intersection_DsetType.release();
  structIntersectionParams.m_inputDataSet = intersection_Dset.release();

  inputParamsVec.push_back(structInputParams);
  inputParamsVec.push_back(structIntersectionParams);

  te::vp::AlgorithmParams* params = new te::vp::AlgorithmParams();
  params->setInputParams(inputParamsVec);
  params->setOutputDataSource(trgDs);
  params->setOutputDataSetName(outDSName);
  params->setSpecificParams(specificParams);

  te::vp::Identity identity;
  bool result = identity.executeMemory(params);

  delete params;

  return result;
}

//Postgis to Postgis
bool IdentityPGISToPGIS()
{
  boost::uuids::basic_random_generator<boost::mt19937> gen;
  boost::uuids::uuid u = gen();
  std::string dsId = boost::uuids::to_string(u);

  std::string connInfo(
      "ppgsql://postgres:postgres@atlas.dpi.inpe.br:5433/testPostGIS");

  te::da::DataSourcePtr srcDs(
      te::da::DataSourceFactory::make("POSTGIS", connInfo));
  srcDs->setId(dsId);

  te::da::DataSourceInfoPtr dsInfoPtr(new te::da::DataSourceInfo());
  dsInfoPtr->setId(dsId);
  dsInfoPtr->setConnInfo(connInfo);
  dsInfoPtr->setType("POSTGIS");

  te::da::DataSourceInfoManager::getInstance().add(dsInfoPtr);

  srcDs->open();

  std::string input_DsetName = "munic_goias";
  std::string intersection_DsetName = "munic_goias_selected";

  if (!srcDs->dataSetExists(input_DsetName))
  {
    std::cout << "Input dataset not found: " << input_DsetName << std::endl;
    return false;
  }

  std::unique_ptr<te::da::DataSet> input_Dset =
      srcDs->getDataSet(input_DsetName);

  std::unique_ptr<te::da::DataSetType> input_DsetType =
      srcDs->getDataSetType(input_DsetName);

  if (!srcDs->dataSetExists(intersection_DsetName))
  {
    std::cout << "Input dataset not found: " << intersection_DsetName
              << std::endl;
    return false;
  }

  std::unique_ptr<te::da::DataSet> intersection_Dset =
      srcDs->getDataSet(intersection_DsetName);

  std::unique_ptr<te::da::DataSetType> intersection_DsetType =
      srcDs->getDataSetType(intersection_DsetName);

  std::vector<std::string> attributes;
  attributes.push_back("nome");
  attributes.push_back("nomemeso");

  std::map<std::string, te::dt::AbstractData*> specificParams;
  specificParams["public.munic_goias"] =
      new te::vp::ComplexData<std::vector<std::string> >(attributes);
  specificParams["public.munic_goias_selected"] =
      new te::vp::ComplexData<std::vector<std::string> >(attributes);

  te::da::DataSourcePtr trgDs(
      te::da::DataSourceFactory::make("POSTGIS", connInfo));
  trgDs->open();

  std::string outDSName = "identity_pgistopgis_ex";

  if (trgDs->dataSetExists(outDSName))
  {
    std::cout << "A dataset with the same requested output dataset name "
                 "already exists:" << outDSName << std::endl;
    return false;
  }

  std::vector<te::vp::InputParams> inputParamsVec;
  te::vp::InputParams structInputParams;
  te::vp::InputParams structIntersectionParams;

  structInputParams.m_inputDataSource = srcDs;
  structInputParams.m_inputDataSetType = input_DsetType.release();
  structInputParams.m_inputDataSet = input_Dset.release();

  structIntersectionParams.m_inputDataSource = srcDs;
  structIntersectionParams.m_inputDataSetType = intersection_DsetType.release();
  structIntersectionParams.m_inputDataSet = intersection_Dset.release();

  inputParamsVec.push_back(structInputParams);
  inputParamsVec.push_back(structIntersectionParams);

  te::vp::AlgorithmParams* params = new te::vp::AlgorithmParams();
  params->setInputParams(inputParamsVec);
  params->setOutputDataSource(trgDs);
  params->setOutputDataSetName(outDSName);
  params->setSpecificParams(specificParams);

  te::vp::Identity identity;
  bool result = identity.executeMemory(params);

  delete params;

  return result;
}

//Postgis to OGR
bool IdentityPGISToOGR()
{
  boost::uuids::basic_random_generator<boost::mt19937> gen;
  boost::uuids::uuid u = gen();
  std::string dsId = boost::uuids::to_string(u);

  std::string connInfo(
      "ppgsql://postgres:postgres@atlas.dpi.inpe.br:5433/testPostGIS");

  te::da::DataSourcePtr srcDs(
      te::da::DataSourceFactory::make("POSTGIS", connInfo));
  srcDs->setId(dsId);

  te::da::DataSourceInfoPtr dsInfoPtr(new te::da::DataSourceInfo());
  dsInfoPtr->setId(dsId);
  dsInfoPtr->setConnInfo(connInfo);
  dsInfoPtr->setType("POSTGIS");

  te::da::DataSourceInfoManager::getInstance().add(dsInfoPtr);

  srcDs->open();

  std::string input_DsetName = "munic_goias";
  std::string intersection_DsetName = "munic_goias_selected";

  if (!srcDs->dataSetExists(input_DsetName))
  {
    std::cout << "Input dataset not found: " << input_DsetName << std::endl;
    return false;
  }

  std::unique_ptr<te::da::DataSet> input_Dset =
      srcDs->getDataSet(input_DsetName);

  std::unique_ptr<te::da::DataSetType> input_DsetType =
      srcDs->getDataSetType(input_DsetName);


  if (!srcDs->dataSetExists(intersection_DsetName))
  {
    std::cout << "Input dataset not found: " << intersection_DsetName
              << std::endl;
    return false;
  }

  std::unique_ptr<te::da::DataSet> intersection_Dset =
      srcDs->getDataSet(intersection_DsetName);

  std::unique_ptr<te::da::DataSetType> intersection_DsetType =
      srcDs->getDataSetType(intersection_DsetName);

  std::vector<std::string> attributes;
  attributes.push_back("nome");
  attributes.push_back("nomemeso");

  std::map<std::string, te::dt::AbstractData*> specificParams;
  specificParams["public.munic_goias"] =
      new te::vp::ComplexData<std::vector<std::string> >(attributes);
  specificParams["public.munic_goias_selected"] =
      new te::vp::ComplexData<std::vector<std::string> >(attributes);

  std::string data_dir = TERRALIB_DATA_DIR;
  std::string output_filename(data_dir + "/shape/identity_pgistoshp_ex.shp");
  std::string tgrInfo ("file://" + output_filename);

  te::da::DataSourcePtr trgDs(te::da::DataSourceFactory::make("OGR", tgrInfo));
  trgDs->open();

  std::string outDSName = "identity_pgistoshp_ex";

  if (trgDs->dataSetExists(outDSName))
  {
    std::cout << "A dataset with the same requested output dataset name "
                 "already exists:" << outDSName << std::endl;
    return false;
  }

  std::vector<te::vp::InputParams> inputParamsVec;
  te::vp::InputParams structInputParams;
  te::vp::InputParams structIntersectionParams;

  structInputParams.m_inputDataSource = srcDs;
  structInputParams.m_inputDataSetType = input_DsetType.release();
  structInputParams.m_inputDataSet = input_Dset.release();

  structIntersectionParams.m_inputDataSource = srcDs;
  structIntersectionParams.m_inputDataSetType = intersection_DsetType.release();
  structIntersectionParams.m_inputDataSet = intersection_Dset.release();

  inputParamsVec.push_back(structInputParams);
  inputParamsVec.push_back(structIntersectionParams);

  te::vp::AlgorithmParams* params = new te::vp::AlgorithmParams();
  params->setInputParams(inputParamsVec);
  params->setOutputDataSource(trgDs);
  params->setOutputDataSetName(outDSName);
  params->setSpecificParams(specificParams);

  te::vp::Identity identity;
  bool result = identity.executeMemory(params);

  delete params;

  return result;
}


