#!/bin/bash
#
#  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.
#
#  This file is part of the TerraLib - a Framework for building GIS enabled applications.
#
#  TerraLib is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation, either version 3 of the License,
#  or (at your option) any later version.
#
#  TerraLib is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with TerraLib. See COPYING. If not, write to
#  TerraLib Team at <terralib-team@terralib.org>.
#
#
#  Description: Install all required software for TerraLib on Linux Ubuntu 18.04.
#
#  Author: TerraLib Team <terralib-team@terralib.org>
#
#
#  Set "TERRALIB_DEPENDENCIES_DIR" with the path to thirdparty debian packages.
#  Example:
#  $ TERRALIB_DEPENDENCIES_DIR="/home/user/packages" ./install-3rdparty-linux-ubuntu-18.04-deb.sh
#

echo "*****************************************************************"
echo "* TerraLib 3rd-party Libraries Installer for Linux Ubuntu 18.04 *"
echo "*****************************************************************"
echo ""
sleep 1s

#
# Valid parameter val or abort script
#
function valid()
{
  if [ $1 -ne 0 ]; then
    echo $2
    echo ""
    exit $1
  fi
}

#
# Update Ubuntu install list
#
sudo apt-get update
valid $? "Error: could not update apt-get database list"


#
# gcc
#
gcpp_test=`dpkg -s g++ | grep Status`

if [ "$gcpp_test" != "Status: install ok installed" ]; then
  sudo  apt-get -y install g++
  valid $? "Error: could not install g++! Please, install g++: sudo apt-get -y install g++"
  echo "g++ installed!"
else
  echo "g++ already installed!"
fi


#
# zlibdevel
#
zlibdevel_test=`dpkg -s zlib1g-dev | grep Status`

if [ "$zlibdevel_test" != "Status: install ok installed" ]; then
  sudo apt-get -y install zlib1g-dev
  valid $? "Error: could not install zlib1g-dev! Please, install g++: sudo apt-get -y install zlib1g-dev"
  echo "zlib1g-dev installed!"
else
  echo "zlib1g-dev already installed!"
fi


#
# libreadline-dev
#
readline_test=`dpkg -s libreadline-dev | grep Status`

if [ "$readline_test" != "Status: install ok installed" ]; then
  sudo apt-get -y install libreadline-dev
  valid $? "Error: could not install libreadline-dev! Please, install readline: sudo apt-get -y install libreadline-dev"
  echo "libreadline-dev installed!"
else
  echo "libreadline-dev already installed!"
fi


#
# python support
#
pysetup_test=`dpkg -s python-setuptools | grep Status`

if [ "$gcpp_test" != "Status: install ok installed" ]; then
  sudo apt-get -y install python-setuptools
  valid $? "Error: could not install python-setuptools! Please, install readline: sudo apt-get -y install python-setuptools"
  echo "python-setuptools installed!"
else
  echo "python-setuptools already installed!"
fi

pypip_test=`dpkg -s python-pip | grep Status`

if [ "$pypip_test" != "Status: install ok installed" ]; then
  sudo apt-get -y install python-pip
  valid $? "Error: could not install python-pip! Please, install readline: sudo apt-get -y install python-pip"
  echo "python-pip installed!"
else
  echo "python-pip already installed!"
fi

pydev_test=`dpkg -s python-dev | grep Status`

if [ "$pydev_test" != "Status: install ok installed" ]; then
  sudo apt-get -y install python-dev
  valid $? "Error: could not install python-dev! Please, install readline: sudo apt-get -y install python-dev"
  echo "python-dev installed!"
else
  echo "python-dev already installed!"
fi

numpy_test=`dpkg -s python-numpy | grep Status`

if [ "$numpy_test" != "Status: install ok installed" ]; then
  sudo apt-get -y install python-numpy
  valid $? "Error: could not install python-numpy! Please, install readline: sudo apt-get -y install python-numpy"
  echo "python-numpy installed!"
else
  echo "python-numpy already installed!"
fi


#
# autoconf
#
autoconf_test=`dpkg -s autoconf | grep Status`

if [ "$autoconf_test" != "Status: install ok installed" ]; then
  sudo apt-get -y install autoconf
  valid $? "Error: could not install autoconf! Please, install readline: sudo apt-get -y install autoconf"
  echo "autoconf installed!"
else
  echo "autoconf already installed!"
fi


#
# GNU gettext
#
gettext_test=`dpkg -s gettext | grep Status`

if [ "$gettext_test" != "Status: install ok installed" ]; then
  sudo apt-get -y install gettext
  valid $? "Error: could not install gettext! Please, install readline: sudo apt-get -y install gettext"
  echo "gettext installed!"
else
  echo "gettext already installed!"
fi


#
# flex
#
flex_test=`dpkg -s flex | grep Status`

if [ "$flex_test" != "Status: install ok installed" ]; then
  sudo apt-get -y install flex
  valid $? "Error: could not install flex! Please, install readline: sudo apt-get -y install flex"
  echo "flex installed!"
else
  echo "flex already installed!"
fi


#
# bison
#
bison_test=`dpkg -s bison | grep Status`

if [ "$bison_test" != "Status: install ok installed" ]; then
  sudo apt-get -y install bison
  valid $? "Error: could not install bison! Please, install readline: sudo apt-get -y install bison"
  echo "bison installed!"
else
  echo "bison already installed!"
fi


#
# CMake
#

# CMake from dpkg
cmake_version_first_dig=`dpkg -s cmake | grep Version | awk 'NR ==1{print substr($2,1,1)}'`
cmake_version_second_dig=`dpkg -s cmake | grep Version | awk 'NR ==1{print substr($2,3,1)}'`
cmake_test=`dpkg -s cmake | grep Status`

# CMake installed
cmake_installed_version_first_dig=`cmake --version | grep version | awk 'NR ==1{print substr($3,1,1)}'`
cmake_installed_version_second_dig=`cmake --version | grep version | awk 'NR ==1{print substr($3,3,2)}'`

# Verify the first digit in installed version
if [[ "$cmake_installed_version_first_dig" == *"."* ]]; then
  cmake_installed_version_first_dig=`cmake --version | grep version | awk 'NR ==1{print substr($3,1,2)}'`
fi
# Verify the second digit in installed version
if [[ "$cmake_installed_version_second_dig" == *"."* ]]; then
  cmake_installed_version_second_dig=`cmake --version | grep version | awk 'NR ==1{print substr($3,3,1)}'`
fi

# Verify the first digit in repository version
if [[ "$cmake_version_first_dig" == *"."* ]]; then
  cmake_version_first_dig=`dpkg -s cmake | grep Version | awk 'NR ==1{print substr($2,1,2)}'`
fi
# Verify the second digit in repository version
if [[ "$cmake_version_second_dig" == *"."* ]]; then
  cmake_version_second_dig=`dpkg -s cmake | grep Version | awk 'NR ==1{print substr($2,3,2)}'`
fi

# Checking if the CMake is already installed
echo "Checking if the CMake is already installed..."
if [ ! -z "$cmake_installed_version_first_dig" ]; then
   echo "Checking CMake version..."
     if [ "$cmake_installed_version_first_dig" -ge "3" ] ||
          [ "$cmake_installed_version_first_dig" -eq "3" ] &&
            [ "$cmake_installed_version_second_dig" -ge "11" ]; then
      echo "The version of CMake installed: $cmake_installed_version_first_dig.$cmake_installed_version_second_dig is compatible with TerraLib."
   else
    echo "The version of CMake installed: $cmake_installed_version_first_dig.$cmake_installed_version_second_dig is not compatible with current TerraLib version."
    echo "Minimum version required is 3.11*"
    echo ""
    exit
   fi
else
  echo "There is not CMake installed."
  echo "Trying to install CMake..."
  if [ ! -z "$cmake_version_first_dig" ]; then
     echo "Checking CMake version from repository..."
     if [ "$cmake_version_first_dig" -ge "3" ] ||
          [ "$cmake_version_first_dig" -eq "3" ] &&
            [ "$cmake_version_second_dig" -ge "11" ]; then
        echo "CMake version is compatible with TerraLib."
        sudo apt-get -y install cmake cmake-qt-gui
        valid $? "Error: could not install CMake! Please, install CMake: sudo apt-get -y install cmake"
        echo "CMake has been installed!"
     else
        echo "The CMake $cmake_version_first_dig.$cmake_version_second_dig from dpkg is not compatible with current terralib version."
        echo "Minimum version required is 3.11*"
        echo ""
        exit
     fi
  else
      echo "There is not CMake from dpkg."
      echo "Minimum version of CMake required is 3.11*"
      echo ""
      exit
  fi
fi

# Checking if the CMake is in PATH
echo "Checking if the CMake is in PATH..."
if [ ! command -v cmake --version >/dev/null 2>&1 ]; then
    valid 1 "CMake already installed but not found in PATH!"
    echo ""
    exit
else
  echo "CMake already in PATH!"
fi


#
# libkml
#
libkml_test=`dpkg -s libkml-dev | grep Status`

if [ "$libkml_test" != "Status: install ok installed" ]; then
  sudo apt-get -y install libkml-dev
  valid $? "Error: could not install libkml-dev! Please, install libkml-dev: sudo apt-get -y install libkml-dev"
  echo "libkml-dev installed!"
else
  echo "libkml-dev already installed!"
fi


#
# libssl
#
libssl_test=`dpkg -s libssl-dev | grep Status`

if [ "$libssl_test" != "Status: install ok installed" ]; then
  sudo apt-get -y install libssl-dev
  valid $? "Error: could not install libssl-dev! Please, install libssl-dev: sudo apt-get -y install libssl-dev"
  echo "libssl-dev installed!"
else
  echo "libssl-dev already installed!"
fi


#
# SWIG version 3.0.8
# Dependencies: pcre
#
swig_test=`dpkg -s swig3.0 | grep Status`

if [ "$swig_test" != "Status: install ok installed" ]; then
  sudo apt-get -y install swig3.0
  valid $? "Error: could not install swig! Please, install readline: sudo apt-get -y install swig3.0"
  echo "swig installed!"
else
  echo "swig already installed!"
fi


#
# FreeXL version 1.0.2
#
freexl_test=`dpkg -s libfreexl-dev | grep Status`

if [ "$freexl_test" != "Status: install ok installed" ]; then
  sudo apt-get -y install libfreexl-dev
  valid $? "Error: could not install freexl! Please, install readline: sudo apt-get -y install libfreexl-dev"
  echo "freexl installed!"
else
  echo "freexl already installed!"
fi


#
# OOSP-UUID version 1.6.2
#
uuid_test=`dpkg -s uuid | grep Status`

if [ "$uuid_test" != "Status: install ok installed" ]; then
  sudo apt-get -y install uuid
  valid $? "Error: could not install uuid! Please, install readline: sudo apt-get -y install uuid"
  echo "uuid installed!"
else
  echo "uuid already installed!"
fi



#
# BZIP2 version 1.0.6
#
bzip2_test=`dpkg -s libbz2-dev | grep Status`

if [ "$bzip2_test" != "Status: install ok installcd ..ed" ]; then
  #sudo apt-get -y install bzip2
  sudo apt-get -y install libbz2-dev
  valid $? "Error: could not install bzip2! Please, install readline: sudo apt-get -y install libbz2-dev"
  echo "bzip2 installed!"
else
  echo "bzip2 already installed!"
fi


#
# Proj4 version 4.9.1
#
proj4_test=`dpkg -s libproj-dev | grep Status`

if [ "$proj4_test" != "Status: install ok installed" ]; then
  sudo apt-get -y install libproj-dev
  valid $? "Error: could not install proj4! Please, install readline: sudo apt-get -y install libproj-dev"
  echo "proj4 installed!"
else
  echo "proj4 already installed!"
fi


#
# libPNG
#
png_test=`dpkg -s libpng-dev | grep Status`

if [ "$png_test" != "Status: install ok installed" ]; then
  sudo apt-get -y install libpng-dev
  valid $? "Error: could not install png! Please, install readline: sudo apt-get -y install libpng12-dev"
  echo "png installed!"
else
  echo "png already installed!"
fi


#
# GeoTIFF
# Dependencies: zlib, proj, libtiff, jpeg
#
geotiff_test=`dpkg -s libgeotiff-dev | grep Status`

if [ "$geotiff_test" != "Status: install ok installed" ]; then
  sudo apt-get -y install libgeotiff-dev
  valid $? "Error: could not install geotiff! Please, install readline: sudo apt-get -y install libgeotiff-dev"
  echo "geotiff installed!"
else
  echo "geotiff already installed!"
fi


#
# CURL
#
curl_test=`dpkg -s libcurl4-openssl-dev | grep Status`

if [ "$curl_test" != "Status: install ok installed" ]; then
  sudo apt-get -y install libcurl4-openssl-dev
  valid $? "Error: could not install curl! Please, install readline: sudo apt-get -y install libcurl4-openssl-dev"
  echo "curl installed!"
else
  echo "curl already installed!"
fi


#
# ICU version 55.1.7
#
icu_test=`dpkg -s libicu-dev | grep Status`

if [ "$icu_test" != "Status: install ok installed" ]; then
  sudo apt-get -y install libicu-dev
  valid $? "Error: could not install icu! Please, install readline: sudo apt-get -y install libicu-dev"
  echo "icu installed!"
else
  echo "icu already installed!"
fi


#
# Xerces-c version 3.1.3
# Dependencies: curl, icu
#
xerces_test=`dpkg -s libxerces-c-dev | grep Status`

if [ "$xerces_test" != "Status: install ok installed" ]; then
  sudo apt-get -y install libxerces-c-dev
  valid $? "Error: could not install xerces! Please, install readline: sudo apt-get -y install libxerces-c-dev"
  echo "xerces installed!"
else
  echo "xerces already installed!"
fi


#
# libxml2 version 2.9.3
# Dependencies: icu
#
xml2_test=`dpkg -s libxml2-dev | grep Status`

if [ "$xml2_test" != "Status: install ok installed" ]; then
  sudo apt-get -y install libxml2-dev
  valid $? "Error: could not install xml2! Please, install readline: sudo apt-get -y install libxml2-dev"
  echo "xml2 installed!"
else
  echo "xml2 already installed!"
fi


#
# libxslt version 1.1.28
# Dendencies: xml
#
xslt_test=`dpkg -s libxslt1-dev | grep Status`

if [ "$xslt_test" != "Status: install ok installed" ]; then
  sudo apt-get -y install libxslt1-dev
  valid $? "Error: could not install xslt! Please, install readline: sudo apt-get -y install libxslt1-dev"
  echo "xslt installed!"
else
  echo "xslt already installed!"
fi


#
# Lua version 5.3
#
lua_test=`dpkg -s liblua5.3-dev | grep Status`

if [ "$lua_test" != "Status: install ok installed" ]; then
  sudo apt-get -y install liblua5.3-dev
  valid $? "Error: could not install lua! Please, install readline: sudo apt-get -y install liblua5.2-dev"
  echo "lua installed!"
else
  echo "lua already installed!"
fi


#
# LibPQ
#
pgsql_test=`dpkg -s libpq-dev | grep Status`

if [ "$pgsql_test" != "Status: install ok installed" ]; then
  sudo apt-get -y install libpq-dev
  valid $? "Error: could not install pgsql! Please, install readline: sudo apt-get -y install libpq-dev"
  echo "pgsql installed!"
else
  echo "pgsql already installed!"
fi


#
# SQlite version 3.11
#
sqlite_test=`dpkg -s libsqlite3-dev | grep Status`

if [ "$sqlite_test" != "Status: install ok installed" ]; then
  sudo apt-get -y install libsqlite3-dev
  valid $? "Error: could not install sqlite! Please, install readline: sudo apt-get -y install libsqlite3-dev"
  echo "sqlite installed!"
else
  echo "sqlite already installed!"
fi


#
# Spatialite version 4.3
#
spatialite_test=`dpkg -s libspatialite-dev  | grep Status`

if [ "$spatialite_test" != "Status: install ok installed" ]; then
  sudo apt-get -y install libspatialite-dev
  #sudo apt-get -y install spatialite-bin
  valid $? "Error: could not install spatialite! Please, install readline: sudo apt-get -y install libspatialite-dev spatialite-bin"
  echo "spatialite installed!"
else
  echo "spatialite already installed!"
fi


#
# qt5-default qttools5-dev qttools5-dev-tools libqt5svg5-dev libqt5designer5
#
qt5_dev_test=`dpkg -s qt5-default qttools5-dev qttools5-dev-tools libqt5svg5-dev libqt5designer5 | grep Status`

if [ "$qt5_dev_test" != "Status: install ok installed" ]; then
  sudo apt-get -y install qt5-default qttools5-dev qttools5-dev-tools libqt5svg5-dev libqt5designer5
  valid $? "Error: could not install qt5-default! Please, install Qt 5 support: sudo apt-get -y install qt5-default qttools5-dev qttools5-dev-tools libqt5svg5-dev libqt5designer5"
  echo "qt5-dev-tools installed!"
else
  echo "qt5-dev-tools already installed!"
fi


#
# GEOS version 3.4.2
#
geos_test=`dpkg -s libgeos++-dev | grep Status`

if [ "$geos_test" != "Status: install ok installed" ]; then
  sudo apt-get -y install libgeos++-dev
  valid $? "Error: could not install geos! Please, install readline: sudo apt-get -y install libgeos++-dev"
  echo "geos installed!"
else
  echo "geos already installed!"
fi

#
# libhdf4 version 4.2.10
#
hdf4_test=`dpkg -s libhdf4-dev | grep Status`

if [ "$hdf4_test" != "Status: install ok installed" ]; then
  sudo apt-get -y install libhdf4-dev
  valid $? "Error: could not install hdf4! Please, install readline: sudo apt-get -y install libhdf4-dev"
  echo "hdf4 installed!"
else
  echo "hdf4 already installed!"
fi

#
# devscripts
#
devscripts_test=`dpkg -s devscripts | grep Status`

if [ "$devscripts_test" != "Status: install ok installed" ]; then
  sudo apt-get -y install devscripts
  valid $? "Error: could not install devscripts! Please, install readline: sudo apt-get -y install devscripts"
  echo "devscripts installed!"
else
  echo "devscripts already installed!"
fi

#
# debhelper
#
debhelper_test=`dpkg -s debhelper | grep Status`

if [ "$debhelper_test" != "Status: install ok installed" ]; then
  sudo apt-get -y install debhelper
  valid $? "Error: could not install debhelper! Please, install readline: sudo apt-get -y install debhelper"
  echo "debhelper installed!"
else
  echo "debhelper already installed!"
fi

#
# GeographicLib
#
geolib_test=`dpkg -s libgeographic-dev | grep Status`

if [ "$geolib_test" != "Status: install ok installed" ]; then
  sudo apt-get -y install libgeographic-dev
  valid $? "Error: could not install geographiclib! Please, install readline: sudo apt-get -y install libgeographic-dev"
  echo "geographiclib installed!"
else
  echo "geographiclib already installed!"
fi

#
# Install dpkg debian packages
#

if [ -z "$TERRALIB_DEPENDENCIES_DIR" ]; then

  echo ""
  echo "Install without terralib deb packages!"
  echo ""

else

  echo ""
  echo "Install with terralib deb packages:"
  echo ""

  UBUNTU_VERSION=`lsb_release -rs`
  export OUTPUT_PATH=$TERRALIB_DEPENDENCIES_DIR
  export LINUX_SCRIPTS_DIR=`pwd`/../packages/linux

  if [ -z "$TE_VERSION" ]; then
  export TE_VERSION=5.3.0
  fi

  echo ""
  echo "Downloading Sources..."
  echo ""

  if [ ! -d "terralib-3rdparty-linux-ubuntu-${UBUNTU_VERSION}" ]; then
    cd $TERRALIB_DEPENDENCIES_DIR
    wget -N http://www.dpi.inpe.br/terralib5-devel/3rdparty/src/terralib-3rdparty-linux-ubuntu-${UBUNTU_VERSION}.tar.gz
    tar xzvf terralib-3rdparty-linux-ubuntu-${UBUNTU_VERSION}.tar.gz
    cd terralib-3rdparty-linux-ubuntu-${UBUNTU_VERSION}
    mv boost* gdal* qtproperty* qwt* wtss* QScintilla* libqtlua* ${LINUX_SCRIPTS_DIR}
    cd ..
  fi

  cd ${LINUX_SCRIPTS_DIR}

  echo ""
  echo "Packaging Boost..."
  echo ""
  sleep 1s

  ./deb-boost.sh
  export BOOST_VERSION=$DEBVERSION
  sudo dpkg -i ${TERRALIB_DEPENDENCIES_DIR}/packages/terralib-boost*
  valid $? "Error: could not install Boost"

  echo ""
  echo "Packaging Gdal..."
  echo ""
  sleep 1s

  ./deb-gdal.sh
  export GDAL_VERSION=$DEBVERSION
  sudo dpkg -i ${TERRALIB_DEPENDENCIES_DIR}/packages/terralib-gdal*
  valid $? "Error: could not install Gdal"

  echo ""
  echo "Packaging Qt Property Browser..."
  echo ""
  sleep 1s

  ./deb-qtpb.sh
  export QTPB_VERSION=$DEBVERSION
  sudo dpkg -i ${TERRALIB_DEPENDENCIES_DIR}/packages/terralib-qtpb*
  valid $? "Error: could not install Qt Property Browser"

  echo ""
  echo "Packaging Qwt..."
  echo ""
  sleep 1s

  ./deb-qwt.sh
  export QWT_VERSION=$DEBVERSION
  sudo dpkg -i ${TERRALIB_DEPENDENCIES_DIR}/packages/terralib-qwt*
  valid $? "Error: could not install Qwt"

  echo ""
  echo "Packaging Wtss-Cxx..."
  echo ""
  sleep 1s

  ./deb-wtss-cxx.sh
  export WTSS_VERSION=$DEBVERSION
  sudo dpkg -i ${TERRALIB_DEPENDENCIES_DIR}/packages/terralib-wtss-cxx*
  valid $? "Error: could not install Wtss-Cxx"

  echo ""
  echo "Packaging Qscintilla..."
  echo ""
  sleep 1s

  ./deb-qscintilla.sh
  export QSCI_VERSION=$DEBVERSION
  sudo dpkg -i ${TERRALIB_DEPENDENCIES_DIR}/packages/terralib-qscintilla*
  valid $? "Error: could not install Qscintilla"

  echo ""
  echo "Packaging QtLua..."
  echo ""
  sleep 1s

  ./deb-qtlua.sh
  export QTLUA_VERSION=$DEBVERSION
  sudo dpkg -i ${TERRALIB_DEPENDENCIES_DIR}/packages/terralib-qtlua*
  valid $? "Error: could not install QtLua"

  rm boost* gdal* qtproperty* qwt* wtss* QScintilla* libqtlua*

fi

#
# Finished!
#
echo "*****************************************************************"
echo "* TerraLib 3rd-party Libraries Installer for Linux Ubuntu 18.04 *"
echo "*****************************************************************"
echo ""
echo "finished successfully!"
