set "ROOT_DIR=%CD%"

set "BUILD_LOG=%ROOT_DIR%\..\build.log"
set "CONFIG_LOG=%ROOT_DIR%\..\config.log"
set "FAILURES_LOG="%ROOT_DIR%\..\failures.log"

::  Setting visual studio environment
::  =================================
IF DEFINED TERRALIB_X64 (
set "_config=x64"
set "_X86=_x64"
set "PATH=%CMAKE_FILEPATH%;%QMAKE_FILEPATH%;%ROOT_DIR%\icu-59.1-vs2017\bin64;%PATH%"
set _CMAKE_GENERATOR="Visual Studio 15 2017 Win64"
set _MSVC_PLATFORM=x64
) ELSE (
set "_config=x86"
set "_X86=_x86"
set "PATH=%CMAKE_FILEPATH%;%QMAKE_FILEPATH%;%ROOT_DIR%\icu-59.1-vs2017\bin;%PATH%"
set _CMAKE_GENERATOR="Visual Studio 15 2017"
set _MSVC_PLATFORM=Win32
set "Platform=Win32"
)

set "CMAKE_BUILD_PARALLEL_LEVEL=%NUMBER_OF_PROCESSORS%"
set MSVC_FOLDER=msvc15
set MSVC_VERSION=1910
set MSVC_PLATFORM_TOOL=v141

:: Definition of cmake build and install commands
set "_CMAKE_BUILD_RELEASE=cmake --build . --config Release --clean-first"
set "_CMAKE_INSTALL_RELEASE=cmake --build . --config Release --target INSTALL"
set "_CMAKE_BUILD_DEBUG=cmake --build . --config Debug --clean-first"
set "_CMAKE_INSTALL_DEBUG=cmake --build . --config Debug --target INSTALL"


echo | set /p="Configuring visual studio... "<nul
echo.

call "%VCVARS_FILEPATH%"\vcvarsall.bat %_config%

cd %VCVARS_FILEPATH% >nul 2>nul

set VS_VERSION=%VisualStudioVersion%
echo "Visual Studio Version: %VS_VERSION%"
set MSVC_SDK_VERSION=%UCRTVersion%
echo "MSVC SDK Version: %MSVC_SDK_VERSION%"
echo "MSVC Platform: %_MSVC_PLATFORM%"
echo "Available processors: %NUMBER_OF_PROCESSORS%"

cd %ROOT_DIR%

echo done.
echo.

::  ================================

set "INCLUDE=%INCLUDE%"

goto begin_libs

:: Building libraries
::  ==================

:append_log_begin
SETLOCAL
echo ********************** >> %CONFIG_LOG% 2>nul
echo %1 >> %CONFIG_LOG% 2>nul
echo ********************** >> %CONFIG_LOG% 2>nul

echo ********************** >> %BUILD_LOG% 2>nul
echo %1 >> %BUILD_LOG% 2>nul
echo ********************** >> %BUILD_LOG% 2>nul
goto :EOF
ENDLOCAL


:append_log_end
SETLOCAL
echo. >> %CONFIG_LOG% 2>nul
echo ********************** >> %CONFIG_LOG% 2>nul
echo. >> %CONFIG_LOG% 2>nul
echo. >> %CONFIG_LOG% 2>nul

echo. >> %BUILD_LOG% 2>nul
echo ********************** >> %BUILD_LOG% 2>nul
echo. >> %BUILD_LOG% 2>nul
echo. >> %BUILD_LOG% 2>nul

goto :EOF
ENDLOCAL

:skip_build
SETLOCAL
echo skip.

goto :EOF
ENDLOCAL

:remove_lib
SETLOCAL
del %TERRALIB_DEPENDENCIES_DIR%\lib\*%1* /S /Q >nul 2>nul

goto :EOF
ENDLOCAL

:: 1-Nome da biblioteca
:: 2-Mensagem de erro simples.
:: 3-Desvio, nome do atalho para a próxima biblioteca

:buildFailLog
SETLOCAL
set FAIL=1
echo fail on %2.
echo %1: fail on %2. >>%FAILURES_LOG%

goto :EOF
ENDLOCAL


:begin_libs

set LIBS_DIR=%TERRALIB_DEPENDENCIES_DIR%\lib
set BINS_DIR=%TERRALIB_DEPENDENCIES_DIR%\bin

:: libraries not linked against TerraLib 5 (NOT fully installed)
:: -------------------------------------------------------------
set EXPAT=%LIBS_DIR%\libexpat.dll
set EAY=%LIBS_DIR%\libcrypto-1_1-x64.dll
set EAYD=%LIBS_DIR%\libcrypto-1_1-x64.dll
set SSL=%LIBS_DIR%\libssl-1_1-x64.dll
set SSLD=%LIBS_DIR%\libssl-1_1-x64.dll
set ZLIB=%LIBS_DIR%\zlib.dll
set ZLIBD=%LIBS_DIR%\zlibd.dll
set READLINE=%LIBS_DIR%\readline-32.dll
set READLINED=%LIBS_DIR%\readline-32d.dll
set PCRE=%LIBS_DIR%\pcre.dll
set PCRED=%LIBS_DIR%\pcred.dll
set FREEXL=%LIBS_DIR%\freexl.dll
set FREEXLD=%LIBS_DIR%\freexld.dll
set PNG=%LIBS_DIR%\libpng15.dll
set HDF4=%LIBS_DIR%\hdf.dll
set TIFF=%LIBS_DIR%\tiff.dll
set GEOTIFF=%LIBS_DIR%\geotiff.dll
set ICU=%LIBS_DIR%\icuuc59.dll
set XML2=%LIBS_DIR%\libxml2.dll
set NETCDF=%LIBS_DIR%\netcdf.dll
set OPENJPEG=%LIBS_DIR%\openjp2.dll

set LIBKML=%ROOT_DIR%\libkml-master\build%_X86%\Release\libkml.lib
set BZIP=%ROOT_DIR%\bzip2-1.0.6\lib%_X86%\libbz2.lib
set JPEG=%ROOT_DIR%\jpeg-9a\build%_X86%\libjpeg.lib
set MINIZIP=%ROOT_DIR%\unzip101e\build%_X86%\Release\minizip.lib

set "_aux=_x86"
IF DEFINED TERRALIB_X64 set "_aux=_x64"
set URIPARSER=%ROOT_DIR%\uriparser-0.8.4\win32\build%_aux%\uriparser.lib

:: libraries linked against TerraLib 5 (fully installed)
:: -------------------------------------------------------------
set ICONV=%LIBS_DIR%\iconv.lib
set PROJ=%LIBS_DIR%\proj.lib
set GEOS=%LIBS_DIR%\geos.lib
set XERCES=%LIBS_DIR%\xerces-c_3.lib
set BOOST=%LIBS_DIR%\boost_system-mt-gd-x64.lib
set CPPRESTSDK=%LIBS_DIR%\cpprest141_2_10.lib
set PGIS=%LIBS_DIR%\libpqdll.lib
set SQLITE=%LIBS_DIR%\sqlite3.lib
set SPATIALITE=%LIBS_DIR%\spatialite_i.lib
set GDAL=%LIBS_DIR%\gdal_i.lib
set PROPERTYBROWSER=%LIBS_DIR%\qt_property_browser.lib
set QWT=%LIBS_DIR%\qwt.lib
set LUA=%LIBS_DIR%\lua.lib
set QTLUA=%LIBS_DIR%\qtlua.lib
set QTLUAE=%LIBS_DIR%\qtluae.lib
set TERRALIB4=%LIBS_DIR%\terralib.lib
set CURL=%LIBS_DIR%\libcurl_imp.lib
set QSCINTILLA=%LIBS_DIR%\qscintilla2.lib

del %ROOT_DIR%\..\*.log /S /Q >nul 2>nul

IF NOT EXIST %LIBS_DIR% (
  mkdir %TERRALIB_DEPENDENCIES_DIR%\include >nul 2>nul
  mkdir %TERRALIB_DEPENDENCIES_DIR%\lib >nul 2>nul
)

:begin_build

:gettext

:: ====
:: Gettext
set GETTEXT_COPY_DIR=%ROOT_DIR%\gettext0.19.8.1-iconv1.14-static-64
set GETTEXT_DIR=%TERRALIB_DEPENDENCIES_DIR%\bin

:: Check dependencies
goto end_gettext_deps
:gettext_deps
goto geolib_deps
:end_gettext_deps

echo | set /p="Installing gettext... "<nul

IF EXIST %LIBS_DIR%\..\bin\xgettext.exe call :skip_build && goto geolib

call :append_log_begin gettext

:begin_gettext

mkdir %GETTEXT_DIR% >nul 2>nul

xcopy %GETTEXT_COPY_DIR%\bin\xgettext.exe %GETTEXT_DIR% /S /Y >nul 2>nul
xcopy %GETTEXT_COPY_DIR%\bin\msgfmt.exe %GETTEXT_DIR% /S /Y >nul 2>nul
xcopy %GETTEXT_COPY_DIR%\bin\msgmerge.exe %GETTEXT_DIR% /S /Y >nul 2>nul
xcopy %GETTEXT_COPY_DIR%\bin\msgexec.exe %GETTEXT_DIR% /S /Y >nul 2>nul
xcopy %GETTEXT_COPY_DIR%\bin\msgexec.exe %GETTEXT_DIR% /S /Y >nul 2>nul
xcopy %GETTEXT_COPY_DIR%\bin\msginit.exe %GETTEXT_DIR% /S /Y >nul 2>nul

call :append_log_end gettext

:end_gettext

echo done.

cd %ROOT_DIR%
:: ====

:geolib

:: ====
:: Geographic Lib
set GEOLIB_DIR=%ROOT_DIR%\GeographicLib-1.48
set GEOLIB_INCLUDE_DIR=%TERRALIB_DEPENDENCIES_DIR%\include
set GEOLIB_LIBRARY=%TERRALIB_DEPENDENCIES_DIR%\lib\Geographic.lib

:: Check dependencies
goto end_geolib_deps
:geolib_deps
goto openjpeg_deps
:end_geolib_deps

echo | set /p="Installing geographic lib... "<nul

IF EXIST %GEOLIB_LIBRARY% call :skip_build && goto openjpeg

call :append_log_begin geolib

:begin_geolib

cd %GEOLIB_DIR% >nul 2>nul

IF EXIST building%_X86% del /s /Q building%_X86% >nul 2>nul

mkdir building%_X86% >nul 2>nul

cd building%_X86% >nul 2>nul

:: Runs cmake to create the build project
( cmake -G %_CMAKE_GENERATOR% ^
-DCMAKE_INSTALL_PREFIX=%TERRALIB_DEPENDENCIES_DIR% ^
-DCMAKE_DEBUG_POSTFIX=d ^
%GEOLIB_DIR% >>%CONFIG_LOG% 2>nul ) || call :buildFailLog geolib "configuring geolib" && goto openjpeg

:: Build and install both Debug and Release
( %_CMAKE_BUILD_RELEASE% >>%BUILD_LOG% 2>nul ) || call :buildFailLog geolib "build release" && goto openjpeg
( %_CMAKE_INSTALL_RELEASE%  >>%BUILD_LOG% 2>nul ) || call :buildFailLog geolib "build release" && goto openjpeg
( %_CMAKE_BUILD_DEBUG% >>%BUILD_LOG% 2>nul ) || call :buildFailLog geolib "build debug" && goto openjpeg
( %_CMAKE_INSTALL_DEBUG% >>%BUILD_LOG% 2>nul ) || call :buildFailLog geolib "build debug" && goto openjpeg

call :append_log_end geolib

:end_geolib

echo done.

cd %ROOT_DIR%
:: ====

:openjpeg

:: ====
:: OpenJPEG Lib
set OPENJPEGLIB_DIR=%ROOT_DIR%\openjpeg-2.3.1
set OPENJPEGLIB_INCLUDE_DIR=%TERRALIB_DEPENDENCIES_DIR%\include\openjpeg-2.3
set OPENJPEGLIB_LIBRARY=%TERRALIB_DEPENDENCIES_DIR%\lib\openjp2.lib
set OPENJPEGLIBD_LIBRARY=%TERRALIB_DEPENDENCIES_DIR%\lib\openjp2d.lib

:: Check dependencies
goto end_openjpeg_deps
:openjpeg_deps
goto swig_deps
:end_openjpeg_deps

echo | set /p="Installing openjpeg lib... "<nul

IF EXIST %OPENJPEGLIB_LIBRARY% call :skip_build && goto swig

call :append_log_begin openjpeg

:begin_openjpeg

cd %OPENJPEGLIB_DIR%

del building%_X86% /S /Q >nul 2>nul

del binaries%_X86% /S /Q >nul 2>nul

mkdir building%_X86% >nul 2>nul

cd building%_X86% >nul 2>nul

:: Runs cmake to create the build project
( cmake -G %_CMAKE_GENERATOR% ^
-DCMAKE_INSTALL_PREFIX=%TERRALIB_DEPENDENCIES_DIR% ^
-DCMAKE_DEBUG_POSTFIX=d ^
%OPENJPEGLIB_DIR% >>%CONFIG_LOG% 2>nul ) || call :buildFailLog openjpeg "configuring openjpeg" && goto swig

:: Build and install both Debug and Release
( %_CMAKE_BUILD_RELEASE% >>%BUILD_LOG% 2>nul ) || call :buildFailLog openjpeg "build release" && goto swig
( %_CMAKE_INSTALL_RELEASE%  >>%BUILD_LOG% 2>nul ) || call :buildFailLog openjpeg "build release" && goto swig
( %_CMAKE_BUILD_DEBUG% >>%BUILD_LOG% 2>nul ) || call :buildFailLog openjpeg "build debug" && goto swig
( %_CMAKE_INSTALL_DEBUG% >>%BUILD_LOG% 2>nul ) || call :buildFailLog openjpeg "build debug" && goto swig

xcopy %BINS_DIR%\openjp2*.dll %LIBS_DIR% /Y >nul 2>nul

del %BINS_DIR%\openjp2*.dll /S /Q >nul 2>nul

call :append_log_end openjpeg

:end_openjpeg

echo done.

cd %ROOT_DIR%
:: ====


:swig

:: ====
:: Swig
set SWIG_COPY_DIR=%ROOT_DIR%\swigwin-3.0.12
set SWIG_DIR=%TERRALIB_DEPENDENCIES_DIR%\swig\

:: Check dependencies
goto end_swig_deps
:swig_deps
goto gmock_deps
:end_swig_deps

echo | set /p="Installing swig... "<nul

IF EXIST %SWIG_DIR% call :skip_build && goto gmock

call :append_log_begin swig

:begin_swig

mkdir %SWIG_DIR% >nul 2>nul

xcopy %SWIG_COPY_DIR% %SWIG_DIR% /S /Y >nul 2>nul

call :append_log_end swig

:end_swig

echo done.

cd %ROOT_DIR%
:: ====

:gmock

:: ====
:: GTest and GMock
set GMOCK_DIR=%ROOT_DIR%\googletest-master\googlemock
set GMOCK_INCLUDE_DIR=%TERRALIB_DEPENDENCIES_DIR%\include
set GMOCK_LIBRARY=%TERRALIB_DEPENDENCIES_DIR%\lib\gmock.lib

:: Check dependencies
goto end_gmock_deps
:gmock_deps
goto iconv_deps
:end_gmock_deps

echo | set /p="Installing gtest and gmock... "<nul

IF EXIST %LIBS_DIR%\gmock.lib call :skip_build && goto iconv

call :append_log_begin gmock

:begin_gmock

cd %GMOCK_DIR% >nul 2>nul

IF EXIST building%_X86% del /s /Q building%_X86% >nul 2>nul

mkdir building%_X86% >nul 2>nul

cd building%_X86% >nul 2>nul

:: Runs cmake to create the build project
( cmake -G %_CMAKE_GENERATOR% -DCMAKE_INSTALL_PREFIX=%GMOCK_DIR% %GMOCK_DIR% -Dgtest_force_shared_crt=ON >>%CONFIG_LOG% 2>nul ) || call :buildFailLog gmock "configuring gmock" && goto iconv

:: Build and install both Debug and Release
( %_CMAKE_BUILD_RELEASE% >>%BUILD_LOG% 2>nul ) || call :buildFailLog gmock "build release" && goto iconv
( %_CMAKE_BUILD_DEBUG% >>%BUILD_LOG% 2>nul ) || call :buildFailLog gmock "build debug" && goto iconv


xcopy %GMOCK_DIR%\..\googletest\include %GMOCK_INCLUDE_DIR% /S /Y >nul 2>nul

xcopy %GMOCK_DIR%\include %GMOCK_INCLUDE_DIR% /S /Y >nul 2>nul

xcopy %GMOCK_DIR%\building%_X86%\Release\* %LIBS_DIR% /Y >nul 2>nul

xcopy %GMOCK_DIR%\building%_X86%\Debug\* %LIBS_DIR% /Y >nul 2>nul

xcopy %GMOCK_DIR%\building%_X86%\gtest\Release\* %LIBS_DIR% /Y >nul 2>nul

xcopy %GMOCK_DIR%\building%_X86%\gtest\Debug\* %LIBS_DIR% /Y >nul 2>nul

call :append_log_end gmock

:end_gmock

echo done.

cd %ROOT_DIR%
:: ====

:iconv

::  ====
::  Iconv
set ICONV_DIR=%ROOT_DIR%\iconv\%MSVC_FOLDER%
set ICONV_INCLUDE_DIR=%TERRALIB_DEPENDENCIES_DIR%\include
set ICONV_LIBRARY=%ICONV%
set ICONVD_LIBRARY=%LIBS_DIR%\iconvd.lib
set ICONV_LIBRARIES=debug;%ICONVD_LIBRARY%;optimized;%ICONV_LIBRARY%

:: Check dependencies
goto end_iconv_deps
:iconv_deps
goto expat_deps
:end_iconv_deps

echo | set /p="Installing iconv... "<nul

IF EXIST %ICONV_LIBRARY% call :skip_build && goto expat

call :append_log_begin iconv

:begin_iconv

cd %ICONV_DIR% >nul 2>nul

( msbuild /m /p:Configuration=Debug /p:Platform=%_MSVC_PLATFORM% /t:clean myIconv.sln >>%BUILD_LOG% 2>nul ) || call :buildFailLog iconv  "clean debug" && goto expat

( msbuild /m /p:Configuration=Release /p:Platform=%_MSVC_PLATFORM% /t:clean myIconv.sln >>%BUILD_LOG% 2>nul ) || call :buildFailLog iconv  "clean release" && goto expat

( msbuild /m /p:PlatformToolset=%MSVC_PLATFORM_TOOL%;ToolsVersion=%VS_VERSION%;WindowsTargetPlatformVersion=%MSVC_SDK_VERSION%;Configuration=Debug /p:Platform=%_MSVC_PLATFORM% myIconv.sln >>%BUILD_LOG% 2>nul ) || call :buildFailLog iconv "build debug" && goto expat

( msbuild /m /p:PlatformToolset=%MSVC_PLATFORM_TOOL%;ToolsVersion=%VS_VERSION%;WindowsTargetPlatformVersion=%MSVC_SDK_VERSION%;Configuration=Release /p:Platform=%_MSVC_PLATFORM% myIconv.sln >>%BUILD_LOG% 2>nul ) || call :buildFailLog iconv "build release" && goto expat

xcopy %ICONV_DIR%\myIconv\include\iconv.h %ICONV_INCLUDE_DIR% /Y >nul 2>nul

IF DEFINED TERRALIB_X64 (
  set "_libF=x64"
) ELSE set "_libF=x86"

xcopy %ICONV_DIR%\%_libF%\Release\*.lib %LIBS_DIR% /Y  >nul 2>nul
xcopy %ICONV_DIR%\%_libF%\Release\*.dll %LIBS_DIR% /Y  >nul 2>nul

xcopy %ICONV_DIR%\%_libF%\Debug\*.lib %LIBS_DIR% /Y  >nul 2>nul
xcopy %ICONV_DIR%\%_libF%\Debug\*.dll %LIBS_DIR% /Y  >nul 2>nul

call :append_log_end iconv

:end_iconv

echo done.

cd %ROOT_DIR%
::  ========

:expat

:: Expat version 2.2.9
set EXPAT_DIR=%ROOT_DIR%\expat-2.2.9
set EXPAT_INCLUDE_DIR=%EXPAT_DIR%\binaries%_X86%\include
set EXPAT_LIBRARY=%EXPAT_DIR%\binaries%_X86%\lib\libexpat.lib
set EXPATD_LIBRARY=%EXPAT_DIR%\binaries%_X86%\lib\libexpatd.lib

:: Check dependencies
goto end_expat_deps
:expat_deps
goto zlib_deps
:end_expat_deps

echo | set /p="Installing expat... "<nul

IF EXIST %EXPAT% call :skip_build && goto zlib

call :append_log_begin expat

:begin_expat

cd %EXPAT_DIR%

del build%_X86% /S /Q >nul 2>nul

del binaries%_X86% /S /Q >nul 2>nul

mkdir build%_X86% >nul 2>nul

cd build%_X86% >nul 2>nul

:: Runs cmake to create the build project
( cmake -G %_CMAKE_GENERATOR% ^
-DCMAKE_INSTALL_PREFIX=%EXPAT_DIR%\binaries%_X86% ^
-DCMAKE_DEBUG_POSTFIX=d ^
%EXPAT_DIR% >>%CONFIG_LOG% 2>nul ) || call :buildFailLog expat "configuring" && goto zlib

:: Build and install both Debug and Release
( %_CMAKE_BUILD_RELEASE% >>%BUILD_LOG% 2>nul ) || call :buildFailLog expat "build release" && goto zlib
( %_CMAKE_INSTALL_RELEASE%  >>%BUILD_LOG% 2>nul ) || call :buildFailLog expat "build release" && goto zlib
( %_CMAKE_BUILD_DEBUG% >>%BUILD_LOG% 2>nul ) || call :buildFailLog expat "build debug" && goto zlib
( %_CMAKE_INSTALL_DEBUG% >>%BUILD_LOG% 2>nul ) || call :buildFailLog expat "build debug" && goto zlib

:: Now we install just the files that are really needed
xcopy %EXPAT_DIR%\binaries%_X86%\bin\*.dll %LIBS_DIR% /Y >nul 2>nul

call :append_log_end expat

:end_expat

echo done.

cd %ROOT_DIR%
:: ====

:zlib

:: ZLIB 1.2.11
::  ====
set ZL_DIR=%ROOT_DIR%\zlib-1.2.11
set ZL_INCLUDE_DIR=%ZL_DIR%;%ZL_DIR%\build%_X86%
set ZL_LIBRARY=%ZL_DIR%\build%_X86%\Release\zlib.lib
set ZLD_LIBRARY=%ZL_DIR%\build%_X86%\Debug\zlibd.lib
set ZL_LIBRARIES=debug;%ZLD_LIBRARY%;optimized;%ZL_LIBRARY%

:: Check dependencies
goto end_zlib_deps
:zlib_deps
goto openssl_deps
:end_zlib_deps

echo | set /p="Installing zlib... "<nul

IF EXIST %ZLIB% call :skip_build && goto openssl

call :append_log_begin zlib

:begin_zlib

cd %ZL_DIR%

del build%_X86% /S /Q >nul 2>nul

mkdir build%_X86% >nul 2>nul

cd build%_X86% >nul 2>nul

:: Runs cmake to create the build project
( cmake -G %_CMAKE_GENERATOR% ^
-DCMAKE_INSTALL_PREFIX=%TERRALIB_DEPENDENCIES_DIR% ^
%ZL_DIR% >>%CONFIG_LOG% 2>nul ) || call :buildFailLog  zlib "configuring" && goto openssl

:: Build and install both Debug and Release
( %_CMAKE_BUILD_RELEASE% >>%BUILD_LOG% 2>nul ) || call :buildFailLog zlib "build release" && goto openssl
( %_CMAKE_INSTALL_RELEASE%  >>%BUILD_LOG% 2>nul ) || call :buildFailLog zlib "build release" && goto openssl
( %_CMAKE_BUILD_DEBUG% >>%BUILD_LOG% 2>nul ) || call :buildFailLog zlib "build debug" && goto openssl
( %_CMAKE_INSTALL_DEBUG% >>%BUILD_LOG% 2>nul ) || call :buildFailLog zlib "build debug" && goto openssl

:: Now we install just the files that are really needed
xcopy %ZL_DIR%\zutil.h %TERRALIB_DEPENDENCIES_DIR%\include /Y >nul 2>nul
xcopy %TERRALIB_DEPENDENCIES_DIR%\bin\zlib*.dll %TERRALIB_DEPENDENCIES_DIR%\lib >nul 2>nul
del /Q %TERRALIB_DEPENDENCIES_DIR%\bin\zlib*.dll >nul 2>nul

call :append_log_end zlib

:end_zlib

echo done.

cd %ROOT_DIR%
::  ================================

:openssl

IF NOT DEFINED TERRALIB_X64 (
set "_x86_=.x86"
set "_out_=32"
set "_openssl_build_type=VC-WIN32"
) ELSE (
set "_out_=64"
set "_x86_=.x64"
set "_openssl_build_type=VC-WIN64A"
)

::  OPENSSL 1.1.1g
set SSL_DIR=%ROOT_DIR%\openssl-1.1.1g
set SSL_INCLUDE_DIR=%SSL_DIR%\include
set SSL_LIBRARY=%SSL_DIR%\libssl.lib
set SSLD_LIBRARY=%SSL_DIR%\libssl.lib
set EAY_LIBRARY=%SSL_DIR%\libcrypto.lib
set EAYD_LIBRARY=%SSL_DIR%\libcrypto.lib

:: Check dependencies
goto end_openssl_deps
:openssl_deps
IF NOT EXIST %ZLIB% call :remove_lib %SSL% && goto bzip_deps
goto bzip_deps
:end_openssl_deps

echo | set /p="Installing openssl... "<nul

IF EXIST %TERRALIB_DEPENDENCIES_DIR%\lib\libssl.lib call :skip_build && goto bzip

call :append_log_begin openssl

:begin_openssl

del %SSL_DIR%\build%_X86% /S /Q >nul 2>nul

mkdir %SSL_DIR%\build%_X86% >nul 2>nul

cd %SSL_DIR%

( %PERL_EXE% configure  %_openssl_build_type% --release shared zlib-dynamic no-asm --prefix=%SSL_DIR%\build%_X86% --openssldir=ssl --with-zlib-include=%TERRALIB_DEPENDENCIES_DIR%\include >>%BUILD_LOG% 2>nul ) || call :buildFailLog openssl "build release" && goto bzip

( nmake clean >>%BUILD_LOG% 2>nul ) || call :buildFailLog openssl "cleaning" && goto bzip

( nmake >>%BUILD_LOG% 2>nul ) || call :buildFailLog openssl "build release" && goto bzip

( nmake install >>%BUILD_LOG% 2>nul ) || call :buildFailLog openssl "install release" && goto bzip

:: Now we install just the files that are really needed
xcopy %SSL_DIR%\build%_X86%\bin\*.dll %TERRALIB_DEPENDENCIES_DIR%\lib /Y >nul 2>nul
xcopy %SSL_DIR%\build%_X86%\lib\*.lib %TERRALIB_DEPENDENCIES_DIR%\lib /Y >nul 2>nul
xcopy %SSL_DIR%\build%_X86%\include %TERRALIB_DEPENDENCIES_DIR%\include /Y /S >nul 2>nul

call :append_log_end openssl

:end_openssl

echo done.

cd %ROOT_DIR%
::  ====

:bzip

:: BZIP2 version 1.0.6
::  =========================================
set BZIP2_DIR=%ROOT_DIR%\bzip2-1.0.6
set BZIP2_INCLUDE_DIR=%BZIP2_DIR%
set BZIP2_LIBRARY=%BZIP2_DIR%\lib%_X86%\libbz2.lib
set BZIP2D_LIBRARY=%BZIP2_DIR%\lib%_X86%\libbz2d.lib

:: Check dependencies
goto end_bzip_deps
:bzip_deps
goto readline_deps
:end_bzip_deps

echo | set /p="Installing bzip2... "<nul

IF EXIST %BZIP2_LIBRARY% call :skip_build && goto readline

call :append_log_begin bzip

:begin_bzip

cd %BZIP2_DIR% >nul 2>nul

del *.lib /Q >nul 2>nul

del *.pdb /Q >nul 2>nul

IF NOT EXIST lib%_X86% mkdir lib%_X86% >nul 2>nul

( nmake /f makefile.msc lib >>%BUILD_LOG% 2>nul ) || call :buildFailLog bzip "build release" && goto readline

xcopy *.lib lib%_X86% /Y >nul 2>nul

( nmake /f makefile.msc clean >>%BUILD_LOG% 2>nul ) || call :buildFailLog bzip  "clean release" && goto readline

( nmake /f makefile.msc DEBUG=1 lib >>%BUILD_LOG% 2>nul ) || call :buildFailLog bzip "build debug" && goto readline

xcopy *.lib lib%_X86% /Y >nul 2>nul

( nmake /f makefile.msc DEBUG=1 clean >>%BUILD_LOG% 2>nul ) || call :buildFailLog bzip  "clean debug" && goto readline

call :append_log_end bzip

:end_bzip

echo done.

cd %ROOT_DIR%
::  ================================

:readline

IF NOT DEFINED TERRALIB_X64 (
  set "_x86_=x86"
) ELSE set "_x86_=x64"

:: ReadLine
set READLINE_DIR=%ROOT_DIR%\readline-32\readline-32\%MSVC_FOLDER%
set READLINE_INCLUDE_DIR=%READLINE_DIR%\..\src
set READLINE_LIBRARY=%READLINE_DIR%\Release\%_x86_%\readline-32.lib
set READLINED_LIBRARY=%READLINE_DIR%\Debug\%_x86_%\readline-32d.lib
set READLINE_LIBRARIES=debug;%READLINED_LIBRARY%;optimized;%READLINE_LIBRARY%

:: Check dependencies
goto end_readline_deps
:readline_deps
goto pcre_deps
:end_readline_deps

echo | set /p="Installing readline... "<nul

IF EXIST %READLINE% call :skip_build && goto pcre

call :append_log_begin readline

:begin_readline

cd %READLINE_DIR% >nul 2>nul

( msbuild /m /p:Configuration=Release /t:clean /p:Platform=%_MSVC_PLATFORM% >>%BUILD_LOG% 2>nul ) || call :buildFailLog readline  "clean release" && goto pcre

( msbuild /m /t:clean /p:Platform=%_MSVC_PLATFORM% >>%BUILD_LOG% 2>nul ) || call :buildFailLog readline  "clean debug" && goto pcre

( msbuild /m /p:PlatformToolset=%MSVC_PLATFORM_TOOL%;ToolsVersion=%VS_VERSION%;WindowsTargetPlatformVersion=%MSVC_SDK_VERSION%;Configuration=Release /p:Platform=%_MSVC_PLATFORM% >>%BUILD_LOG% 2>nul ) || call :buildFailLog readline "build release" && goto pcre

( msbuild /m /p:PlatformToolset=%MSVC_PLATFORM_TOOL%;ToolsVersion=%VS_VERSION%;WindowsTargetPlatformVersion=%MSVC_SDK_VERSION% /p:Platform=%_MSVC_PLATFORM% >>%BUILD_LOG% 2>nul ) || call :buildFailLog readline "build debug" && goto pcre

xcopy %READLINE_DIR%\Release\%_x86_%\*.dll %LIBS_DIR% /Y >nul 2>nul

xcopy %READLINE_DIR%\Debug\%_x86_%\*.dll %LIBS_DIR% /Y >nul 2>nul

call :append_log_end readline

:end_readline

echo done.

cd %ROOT_DIR%
::  ========

:pcre

::  ====
:: PCRE version 8.37
set PCRE_DIR=%ROOT_DIR%\pcre-8.37
set "PCRE_INCLUDE_DIR= /I%PCRE_DIR% /I%PCRE_DIR%\sljit /I%PCRE_DIR%\build%_X86%"
set PCRE_LIBRARY=%PCRE_DIR%\build%_X86%\Release\pcre.lib
set PCRED_LIBRARY=%PCRE_DIR%\build%_X86%\Debug\pcred.lib
set PCRE_LIBRARIES=debug;%PCRED_LIBRARY%;optimized;%PCRE_LIBRARY%

:: Check dependencies
goto end_pcre_deps
:pcre_deps
IF NOT EXIST %BZIP% call :remove_lib pcre && goto freexl_deps
IF NOT EXIST %ZLIB% call :remove_lib pcre && goto freexl_deps
IF NOT EXIST %READLINE% call :remove_lib pcre && goto freexl_deps
goto freexl_deps
:end_pcre_deps

echo | set /p="Installing pcre... "<nul

IF EXIST %PCRE% call :skip_build && goto freexl

call :append_log_begin pcre

:begin_pcre

cd %PCRE_DIR% >nul 2>nul

del build%_X86% /S /Q >nul 2>nul

mkdir build%_X86% >nul 2>nul

cd build%_X86% >nul 2>nul

:: Runs cmake to create the build project
( cmake -G %_CMAKE_GENERATOR% ^
-DBZIP2_INCLUDE_DIR="%BZIP2_INCLUDE_DIR%" ^
-DBZIP2_LIBRARY_RELEASE="%BZIP2_LIBRARY%" ^
-DBZIP2_LIBRARY_DEBUG="%BZIP2D_LIBRARY%" ^
-DZLIB_INCLUDE_DIR="%ZL_INCLUDE_DIR%" ^
-DZLIB_LIBRARY:STRING="%ZL_LIBRARIES%" ^
-DREADLINE_INCLUDE_DIR="%READLINE_INCLUDE_DIR%" ^
-DREADLINE_LIBRARY:STRING="%READLINE_LIBRARIES%" ^
-DPCRE_BUILD_TESTS:BOOL=OFF ^
-DBUILD_SHARED_LIBS=ON ^
%PCRE_DIR% >>%CONFIG_LOG% 2>nul ) || call :buildFailLog pcre "configuring" && goto freexl

:: Build and install both Debug and Release
( %_CMAKE_BUILD_RELEASE% >>%BUILD_LOG% 2>nul ) || call :buildFailLog pcre "build release" && goto freexl
( %_CMAKE_BUILD_DEBUG% >>%BUILD_LOG% 2>nul ) || call :buildFailLog pcre "build debug" && goto freexl

:: Now we install just the files that are really needed
xcopy %PCRE_DIR%\build%_X86%\Release\pcre.dll %LIBS_DIR% /Y >nul 2>nul
xcopy %PCRE_DIR%\build%_X86%\Debug\pcred.dll %LIBS_DIR% /Y >nul 2>nul

call :append_log_end pcre

:end_pcre

echo done.

cd %ROOT_DIR%
::  ========


:freexl

::  FreeXL version 1.0.1
set FREEXL_DIR=%ROOT_DIR%\freexl-1.0.1
set FREEXL_INCLUDE_DIR=%FREEXL_DIR%\headers
set FREEXL_LIBRARY=%FREEXL_DIR%\lib%_X86%\freexl.lib
set FREEXLD_LIBRARY=%FREEXL_DIR%\lib%_X86%\freexld.lib
set FREEXL_LIBRARIES=debug;%FREEXLD_LIBRARY%;optimized;%FREEXL_LIBRARY%

:: Check dependencies
goto end_freexl_deps
:freexl_deps
IF NOT EXIST %ICONV% call :remove_lib freexl && goto proj_deps
goto proj_deps
:end_freexl_deps

echo | set /p="Installing freexl... "<nul

IF EXIST %FREEXL% call :skip_build && goto sqlite

call :append_log_begin freexl

:begin_freexl

cd %FREEXL_DIR% >nul 2>nul

del lib%_X86% /S /Q >nul 2>nul

mkdir lib%_X86% >nul 2>nul

del *.ilk /S /Q >nul 2>nul

( nmake /f makefile.vc >>%BUILD_LOG% 2>nul ) || call :buildFailLog freexl "build release" && goto sqlite

xcopy *.lib lib%_X86% /Y >nul 2>nul

xcopy *.dll lib%_X86% /Y >nul 2>nul

( nmake /f makefile.vc clean >>%BUILD_LOG% 2>nul ) || call :buildFailLog freexl  "clean release" && goto sqlite

( nmake /f makefile.vc DEBUG=1 >>%BUILD_LOG% 2>nul ) || call :buildFailLog freexl "build debug" && goto sqlite

xcopy *.lib lib%_X86% /Y >nul 2>nul

xcopy *.dll lib%_X86% /Y >nul 2>nul

xcopy lib%_X86%\*.dll %LIBS_DIR% /Y >nul 2>nul

( nmake /f makefile.vc DEBUG=1 clean >>%BUILD_LOG% 2>nul ) || call :buildFailLog freexl  "clean debug" && goto sqlite

call :append_log_end freexl

:end_freexl

echo done.

cd %ROOT_DIR%
::  ========

:sqlite

:: SQLite version 3.32.3
set SQLITE_DIR=%ROOT_DIR%\sqlite-amalgamation-3320300
set SQLITE_INCLUDE_DIR=%TERRALIB_DEPENDENCIES_DIR%\include
set SQLITE_LIBRARY=%LIBS_DIR%\sqlite3.lib
set SQLITED_LIBRARY=%LIBS_DIR%\sqlite3d.lib
set SQLITE_EXECUTABLE=%BINS_DIR%\sqlite3.exe

:: Check dependencies
goto end_sqlite_deps
:sqlite_deps
goto tiff
:end_sqlite_deps

echo | set /p="Installing sqlite... "<nul

IF EXIST %SQLITE% call :skip_build && goto tiff

call :append_log_begin sqlite

:begin_sqlite

cd %SQLITE_DIR% >nul 2>nul

cl sqlite3.c /DSQLITE_API=__declspec(dllexport) /DSQLITE_ENABLE_COLUMN_METADATA /DSQLITE_ENABLE_RTREE=1 /DSQLITE_THREADSAFE=2 -link -dll -out:sqlite3.dll >>%BUILD_LOG% 2>nul || call :buildFailLog sqlite "build release" && goto tiff

cl sqlite3.c shell.c /DSQLITE_ENABLE_COLUMN_METADATA /DSQLITE_ENABLE_RTREE=1 /DSQLITE_THREADSAFE=2 -link -exe -out:sqlite3.exe >>%BUILD_LOG% 2>nul || call :buildFailLog sqlite "build release" && goto tiff

cl sqlite3.c /DSQLITE_API=__declspec(dllexport) /DSQLITE_ENABLE_COLUMN_METADATA /DSQLITE_ENABLE_RTREE=1 /DSQLITE_THREADSAFE=2 /DSQLITE_DEBUG -link -dll -out:sqlite3d.dll >>%BUILD_LOG% 2>nul || call :buildFailLog sqlite "build debug" && goto tiff

xcopy *.h %TERRALIB_DEPENDENCIES_DIR%\include /Y >nul 2>nul

xcopy *.lib %LIBS_DIR% /Y >nul 2>nul

xcopy *.dll %LIBS_DIR% /Y >nul 2>nul

xcopy *.exe %TERRALIB_DEPENDENCIES_DIR%\bin /Y >nul 2>nul

echo done.

call :append_log_end sqlite

:end_sqlite

cd %ROOT_DIR%
:: ====

:tiff

::  TIFF version 4.1.0
set TIFF_DIR=%ROOT_DIR%\tiff-4.1.0
set TIFF_INCLUDE_DIR=%TIFF_DIR%\libtiff
set TIFF_LIBRARY=%TERRALIB_DEPENDENCIES_DIR%\lib\tiff.lib
set TIFFD_LIBRARY=%TERRALIB_DEPENDENCIES_DIR%\lib\tiffd.lib
set TIFF_INSTALL=%TIFF_DIR%\install%_X86%

:: Check dependencies
goto end_tiff_deps
:tiff_deps
IF NOT EXIST %JPEG% call :remove_lib tiff && goto curl_deps
IF NOT EXIST %ZLIB% call :remove_lib tiff && goto curl_deps
goto curl_deps
:end_tiff_deps

echo | set /p="Installing tiff... "<nul

IF EXIST %TIFF% call :skip_build && goto curl

call :append_log_begin tiff

:begin_tiff

cd %TIFF_DIR% >nul 2>nul

RD /S /Q build%_X86% >nul 2>nul

mkdir build%_X86% >nul 2>nul

cd build%_X86% >nul 2>nul

:: Runs cmake to create the build project
( cmake -G %_CMAKE_GENERATOR% ^
-DCMAKE_INSTALL_PREFIX=%TERRALIB_DEPENDENCIES_DIR% ^
-DCMAKE_PREFIX_PATH=%TERRALIB_DEPENDENCIES_DIR% ^
-DCMAKE_DEBUG_POSTFIX=d ^
%TIFF_DIR% >>%CONFIG_LOG% 2>nul ) || call :buildFailLog tiff "configuring" && goto curl

:: Build and install both Debug and Release
( %_CMAKE_BUILD_RELEASE% >>%BUILD_LOG% 2>nul ) || call :buildFailLog tiff "build release" && goto curl
( %_CMAKE_INSTALL_RELEASE%  >>%BUILD_LOG% 2>nul ) || call :buildFailLog tiff "build release" && goto curl
( %_CMAKE_BUILD_DEBUG% >>%BUILD_LOG% 2>nul ) || call :buildFailLog tiff "build debug" && goto curl
( %_CMAKE_INSTALL_DEBUG% >>%BUILD_LOG% 2>nul ) || call :buildFailLog tiff "build debug" && goto curl

xcopy %TERRALIB_DEPENDENCIES_DIR%\bin\tiff*.dll %TERRALIB_DEPENDENCIES_DIR%\lib >nul 2>nul
del /Q %TERRALIB_DEPENDENCIES_DIR%\bin\tiff*.dll >nul 2>nul

echo done.

call :append_log_end tiff

:end_tiff

cd %ROOT_DIR%
:: ====

:curl

set "_bits=i386"

IF DEFINED TERRALIB_X64 set "_bits=x64"

::  CURL version 7.71.1
set CURL_DIR=%ROOT_DIR%\curl-7.71.1
set CURL_INCLUDE_DIR=%TERRALIB_DEPENDENCIES_DIR%\include
set CURL_LIBRARY=%LIBS_DIR%\libcurl_imp.lib
set CURLD_LIBRARY=%LIBS_DIR%\libcurld_imp.lib
set CURL_EXE=%BINS_DIR%\curl.exe

:: Check dependencies
goto end_curl_deps
:curl_deps
IF NOT EXIST %ZLIB% call :remove_lib curl && goto proj_deps
IF NOT EXIST %SSL% call :remove_lib curl && goto proj_deps
goto proj_deps
:end_curl_deps

echo | set /p="Installing curl... "<nul

IF EXIST %CURL% call :skip_build && goto proj

call :append_log_begin curl

:begin_curl

cd %CURL_DIR% >nul 2>nul

RD /S /Q build%_X86% >nul 2>nul

mkdir build%_X86% >nul 2>nul

cd build%_X86% >nul 2>nul

:: Runs cmake to create the build project for DYNAMIC LIBRARY
( cmake -G %_CMAKE_GENERATOR% ^
-DCMAKE_INSTALL_PREFIX=%TERRALIB_DEPENDENCIES_DIR% ^
-DCMAKE_PREFIX_PATH=%TERRALIB_DEPENDENCIES_DIR% ^
-DCMAKE_USE_OPENSSL:BOOL=ON ^
-DCMAKE_DEBUG_POSTFIX=d ^
-DCMAKE_INSTALL_BINDIR:PATH=lib ^
-DBUILD_SHARED_LIBS:BOOL=ON ^
-DBUILD_TESTING:BOOL=OFF ^
%CURL_DIR% >>%CONFIG_LOG% 2>nul ) || call :buildFailLog curl "configuring" && goto proj

:: Build and install both Debug and Release
( %_CMAKE_BUILD_RELEASE% >>%BUILD_LOG% 2>nul ) || call :buildFailLog curl "build release" && goto proj
( %_CMAKE_INSTALL_RELEASE%  >>%BUILD_LOG% 2>nul ) || call :buildFailLog curl "build release" && goto proj
( %_CMAKE_BUILD_DEBUG% >>%BUILD_LOG% 2>nul ) || call :buildFailLog curl "build debug" && goto proj
( %_CMAKE_INSTALL_DEBUG% >>%BUILD_LOG% 2>nul ) || call :buildFailLog curl "build debug" && goto proj


RD /S /Q "%TERRALIB_DEPENDENCIES_DIR%\lib\cmake\CURL" >>%BUILD_LOG% 2>nul

echo done.

call :append_log_end curl

:end_curl

cd %ROOT_DIR%
:: ====


:proj

::  Proj4 version 7.1.0
set PJ_DIR=%ROOT_DIR%\proj-7.1.0
set PJ_INCLUDE_DIR=%TERRALIB_DEPENDENCIES_DIR%\include
set PJ_LIBRARY=%TERRALIB_DEPENDENCIES_DIR%\lib\proj.lib
set PJD_LIBRARY=%TERRALIB_DEPENDENCIES_DIR%\lib\proj_d.lib
set PJ_LIBRARIES=debug;%PJD_LIBRARY%;optimized;%PJ_LIBRARY%

:: Check dependencies
goto end_proj_deps
:proj_deps
goto png_deps
:end_proj_deps

echo | set /p="Installing proj... "<nul

IF EXIST %PROJ% call :skip_build && goto png

call :append_log_begin proj

:begin_proj

cd %PJ_DIR% >nul 2>nul

del *.ilk /S /Q >nul 2>nul

mkdir build%_X86% >nul 2>nul

cd build%_X86% >nul 2>nul

:: Runs cmake to create the build project
( cmake -G %_CMAKE_GENERATOR% ^
-DBUILD_LIBPROJ_SHARED=ON ^
-DCMAKE_DEBUG_POSTFIX=d ^
-DBUILD_SHARED_LIBS=ON ^
-DBINDIR:PATH=lib ^
-DCMAKE_INSTALL_BINDIR=lib ^
-DSQLITE3_INCLUDE_DIR=%SQLITE_INCLUDE_DIR% ^
-DSQLITE3_LIBRARY=%SQLITE_LIBRARY% ^
-DEXE_SQLITE3=%SQLITE_EXECUTABLE% ^
-DCURL_INCLUDE_DIR=%CURL_INCLUDE_DIR% ^
-DCURL_LIBRARY_DEBUG=%CURLD_LIBRARY% ^
-DCURL_LIBRARY_RELEASE=%CURL_LIBRARY% ^
-DCMAKE_INSTALL_PREFIX="%TERRALIB_DEPENDENCIES_DIR%" ^
%PJ_DIR% >>%CONFIG_LOG% 2>nul ) || call :buildFailLog proj "configuring" && goto png

:: Build and install both Debug and Release
( %_CMAKE_BUILD_RELEASE% >>%BUILD_LOG% 2>nul ) || call :buildFailLog proj "build release" && goto png
( %_CMAKE_INSTALL_RELEASE%  >>%BUILD_LOG% 2>nul ) || call :buildFailLog proj "build release" && goto png
( %_CMAKE_BUILD_DEBUG% >>%BUILD_LOG% 2>nul ) || call :buildFailLog proj "build debug" && goto png
( %_CMAKE_INSTALL_DEBUG% >>%BUILD_LOG% 2>nul ) || call :buildFailLog proj "build debug" && goto png

:: Now we install just the files that are really needed
xcopy %LIBS_DIR%\proj*.exe %BINS_DIR% /Y >nul 2>nul

call :append_log_end proj

:end_proj

echo done.

cd %ROOT_DIR%
::  ====

:png

::  LibPNG version 1.5.17
set PNG_DIR=%ROOT_DIR%\libpng-1.5.17
set PNG_INCLUDE_DIR=%PNG_DIR%\deploy%_X86%\include
set PNG_LIBRARY=%PNG_DIR%\deploy%_X86%\lib\libpng15.lib
set PNGD_LIBRARY=%PNG_DIR%\deploy%_X86%\lib\libpng15d.lib

:: Check dependencies
goto end_png_deps
:png_deps
IF NOT EXIST %ZLIB% call :remove_lib png && goto geos_deps
goto geos_deps
:end_png_deps

echo | set /p="Installing libpng... "<nul

IF EXIST %PNG% call :skip_build && goto geos

call :append_log_begin png

:begin_png

cd %PNG_DIR% >nul 2>nul

del deploy%_X86% /S /Q >nul 2>nul

del build%_X86% /S /Q >nul 2>nul

mkdir build%_X86% >nul 2>nul

cd build%_X86% >nul 2>nul

:: Runs cmake to create the build project
( cmake -G %_CMAKE_GENERATOR% -DCMAKE_INSTALL_PREFIX=%PNG_DIR%\deploy%_X86% ^
-DPNG_STATIC=OFF ^
-DPNG_TESTS=OFF ^
-DZLIB_INCLUDE_DIR:STRING="%ZL_INCLUDE_DIR%" ^
-DZLIB_LIBRARY:STRING="%ZL_LIBRARIES%" %PNG_DIR% >>%CONFIG_LOG% 2>nul ) || call :buildFailLog png "configuring" && goto geos

:: Now we install just the files that are really needed
( %_CMAKE_BUILD_RELEASE% >>%BUILD_LOG% 2>nul ) || call :buildFailLog png "build release" && goto geos
( %_CMAKE_INSTALL_RELEASE%  >>%BUILD_LOG% 2>nul ) || call :buildFailLog png "build release" && goto geos
( %_CMAKE_BUILD_DEBUG% >>%BUILD_LOG% 2>nul ) || call :buildFailLog png "build debug" && goto geos
( %_CMAKE_INSTALL_DEBUG% >>%BUILD_LOG% 2>nul ) || call :buildFailLog png "build debug" && goto geos

:: Now we install just the files that are really needed
xcopy %PNG_DIR%\deploy%_X86%\bin\*.dll %LIBS_DIR% /Y >nul 2>nul

call :append_log_end png

:end_png

echo done.

cd %ROOT_DIR%
:: ====

:geos

::  GEOS version 3.8.1
set G_DIR=%ROOT_DIR%\geos-3.8.1
set GEOS_INCLUDE_DIR=%TERRALIB_DEPENDENCIES_DIR%\include
set GEOS_LIBRARY=%TERRALIB_DEPENDENCIES_DIR%\lib\geos.lib
set GEOSD_LIBRARY=%TERRALIB_DEPENDENCIES_DIR%\lib\geosd.lib
set GEOSC_LIBRARY=%TERRALIB_DEPENDENCIES_DIR%\lib\geos_c.lib
set GEOSCD_LIBRARY=%TERRALIB_DEPENDENCIES_DIR%\lib\geos_cd.lib

:: Check dependencies
goto end_geos_deps
:geos_deps
goto jpeg_deps
:end_geos_deps

echo | set /p="Installing geos... "<nul

IF EXIST %GEOS% call :skip_build && goto jpeg

call :append_log_begin geos

:begin_geos

cd %G_DIR% >nul 2>nul

IF NOT EXIST capi\geos_c.h autogen.bat >nul 2>nul

IF NOT EXIST geos_svn_revision.h echo #define GEOS_SVN_REVISION "1.2.1" > geos_svn_revision.h 2>nul

mkdir build%_X86% >nul 2>nul

cd build%_X86% >nul 2>nul

:: Runs cmake to create the build project
( cmake -G %_CMAKE_GENERATOR% ^
-DCMAKE_INSTALL_PREFIX="%TERRALIB_DEPENDENCIES_DIR%" ^
-DBUILD_TESTING:BOOL=ON ^
-DCMAKE_DEBUG_POSTFIX=d ^
%G_DIR% >>%CONFIG_LOG% 2>nul ) || call :buildFailLog geos "configuring" && goto jpeg

:: Build and install both Debug and Release
( %_CMAKE_BUILD_RELEASE% >>%BUILD_LOG% 2>nul ) || call :buildFailLog geos "build release" && goto jpeg
( %_CMAKE_INSTALL_RELEASE%  >>%BUILD_LOG% 2>nul ) || call :buildFailLog geos "build release" && goto jpeg
( %_CMAKE_BUILD_DEBUG% >>%BUILD_LOG% 2>nul ) || call :buildFailLog geos "build debug" && goto jpeg
( %_CMAKE_INSTALL_DEBUG% >>%BUILD_LOG% 2>nul ) || call :buildFailLog geos "build debug" && goto jpeg

:: IMPORTANT: Get rid of io.h deprecated file. This can cause build error on gdal
del /Q %TERRALIB_DEPENDENCIES_DIR%\include\geos\io.h

:: Now we install just the files that are really needed
xcopy %BINS_DIR%\geos* %LIBS_DIR% /Y >nul 2>nul
del /Q %BINS_DIR%\geos*

call :append_log_end geos

:end_geos

echo done.

cd %ROOT_DIR%
::  ====

:jpeg

:: JPEG version 9a
set JPG_DIR=%ROOT_DIR%\jpeg-9a
set JPG_INCLUDE_DIR=%JPG_DIR%
set JPG_LIBRARY=%JPG_DIR%\build%_X86%\libjpeg.lib
set JPGD_LIBRARY=%JPG_DIR%\build%_X86%\libjpegd.lib

:: Check dependencies
goto end_jpeg_deps
:jpeg_deps
goto hdf4_deps
:end_jpeg_deps

echo | set /p="Installing jpeg... "<nul

IF EXIST %JPEG% call :skip_build && goto hdf4

call :append_log_begin jpeg

:begin_jpeg

cd %JPG_DIR% >nul 2>nul

del build%_X86% /S /Q >nul 2>nul

mkdir build%_X86% >nul 2>nul

( nmake /f makefile.vc libjpeg.lib >>%BUILD_LOG% 2>nul ) || call :buildFailLog jpeg "build release" && goto hdf4

xcopy *.lib build%_X86% /Y >nul 2>nul

( nmake /f makefile.vc clean >>%BUILD_LOG% 2>nul ) || call :buildFailLog jpeg  "clean release" && goto hdf4

( nmake /f makefile.vc DEBUG=1 libjpeg.lib >>%BUILD_LOG% 2>nul ) || call :buildFailLog jpeg "build debug" && goto hdf4

xcopy *.lib build%_X86% /Y >nul 2>nul

( nmake /f makefile.vc DEBUG=1 clean >>%BUILD_LOG% 2>nul ) || call :buildFailLog jpeg  "clean debug" && goto hdf4

call :append_log_end jpeg

:end_jpeg

echo done.

cd %ROOT_DIR%
::  ====

:hdf4

:: HDF4 version 4.2.15
set HDF4C_DIR=%ROOT_DIR%\hdf-4.2.15
set HDF4_INCLUDE_DIR=%HDF4C_DIR%\binaries%_X86%\include
set HDF4_LIBRARY=%HDF4C_DIR%\binaries%_X86%\lib\hdf.lib
set HDF4D_LIBRARY=%HDF4C_DIR%\binaries%_X86%\lib\hdfd.lib
set MFHDF_LIBRARY=%HDF4C_DIR%\binaries%_X86%\lib\mfhdf.lib
set MFHDFD_LIBRARY=%HDF4C_DIR%\binaries%_X86%\lib\mfhdfd.lib

:: Check dependencies
goto end_hdf4_deps
:hdf4_deps
IF NOT EXIST %JPEG% call :remove_lib hdf4 && goto geotiff_deps
IF NOT EXIST %ZLIB% call :remove_lib hdf4 && goto geotiff_deps
goto geotiff_deps
:end_hdf4_deps

echo | set /p="Installing hdf4... "<nul

IF EXIST %HDF4% call :skip_build && goto geotiff

call :append_log_begin hdf4

:begin_hdf4

cd %HDF4C_DIR% >nul 2>nul

del binaries%_X86% /S /Q >nul 2>nul

del build%_X86% /S /Q >nul 2>nul

mkdir build%_X86% >nul 2>nul

cd build%_X86% >nul 2>nul

:: Runs cmake to create the build project
( cmake -G %_CMAKE_GENERATOR% ^
-DCMAKE_INSTALL_PREFIX=%HDF4C_DIR%\binaries%_X86% ^
-DCMAKE_PREFIX_PATH=%JPG_DIR%;%TERRALIB_DEPENDENCIES_DIR% ^
-DCMAKE_DEBUG_POSTFIX=d ^
-DBUILD_SHARED_LIBS:BOOL=ON ^
-DBUILD_STATIC_LIBS:BOOL=OFF ^
-DBUILD_TESTING:BOOL=OFF ^
-DHDF4_DISABLE_COMPILER_WARNINGS=ON ^
-DHDF4_ENABLE_COVERAGE=ON ^
-DHDF4_ENABLE_NETCDF=OFF ^
-DHDF4_BUILD_FORTRAN=OFF ^
-DJPEG_INCLUDE_DIR=%JPG_DIR% ^
-DJPEG_LIBRARY_RELEASE=%JPG_LIBRARY% ^
-DJPEG_LIBRARY_DEBUG=%JPGD_LIBRARY% ^
%HDF4C_DIR% >>%CONFIG_LOG% 2>nul ) || call :buildFailLog hdf4 "configuring" && goto geotiff

:: Build and install both Debug and Release
( %_CMAKE_BUILD_RELEASE% >>%BUILD_LOG% 2>nul ) || call :buildFailLog hdf4 "build release" && goto geotiff
( %_CMAKE_INSTALL_RELEASE%  >>%BUILD_LOG% 2>nul ) || call :buildFailLog hdf4 "build release" && goto geotiff
( %_CMAKE_BUILD_DEBUG% >>%BUILD_LOG% 2>nul ) || call :buildFailLog hdf4 "build debug" && goto geotiff
( %_CMAKE_INSTALL_DEBUG% >>%BUILD_LOG% 2>nul ) || call :buildFailLog hdf4 "build debug" && goto geotiff

:: Now we install just the files that are really needed
xcopy %HDF4C_DIR%\binaries%_X86%\bin\hdf* %LIBS_DIR% /Y >nul 2>nul
xcopy %HDF4C_DIR%\binaries%_X86%\bin\mfhdf* %LIBS_DIR% /Y >nul 2>nul
xcopy %HDF4C_DIR%\binaries%_X86%\bin\xdr* %LIBS_DIR% /Y >nul 2>nul

echo done.

call :append_log_end hdf4

:end_hdf4

cd %ROOT_DIR%
:: ====

:geotiff

::  GEOTIFF version 1.6.0
set GTIFF_DIR=%ROOT_DIR%\libgeotiff-1.6.0
set GTIFF_INCLUDE_DIR= /I%GTIFF_DIR% /I%GTIFF_DIR%/libxtiff
set GTIFF_LIBRARY=%TERRALIB_DEPENDENCIES_DIR%\lib\geotiff_i.lib
set GTIFFD_LIBRARY=%TERRALIB_DEPENDENCIES_DIR%\lib\geotiff_d_i.lib
set GTIFF_INSTALL=%GTIFF_DIR%\install%_X86%


:: Check dependencies
goto end_geotiff_deps
:geotiff_deps
IF NOT EXIST %PROJ% call :remove_lib geotiff && goto icu_deps
IF NOT EXIST %TIFF% call :remove_lib geotiff && goto icu_deps
goto icu_deps
:end_geotiff_deps

echo | set /p="Installing geotiff... "<nul

IF EXIST %GEOTIFF% call :skip_build && goto icu

call :append_log_begin geotiff

:begin_geotiff

cd %GTIFF_DIR% >nul 2>nul

RD /S /Q build%_X86% >nul 2>nul

mkdir build%_X86% >nul 2>nul

cd build%_X86% >nul 2>nul

:: Runs cmake to create the build project
( cmake -G %_CMAKE_GENERATOR% ^
-DCMAKE_INSTALL_PREFIX=%TERRALIB_DEPENDENCIES_DIR% ^
-DCMAKE_PREFIX_PATH=%TERRALIB_DEPENDENCIES_DIR% ^
-DCMAKE_DEBUG_POSTFIX=d ^
-DBUILD_SHARED_LIBS:BOOL=ON ^
-DPROJ_INCLUDE_DIR=%PJ_INCLUDE_DIR%; ^
-DPROJ_LIBRARY=%PJ_LIBRARY%; ^
%GTIFF_DIR% >>%CONFIG_LOG% 2>nul ) || call :buildFailLog geotiff "configuring" && goto icu

:: Build and install both Debug and Release
( %_CMAKE_BUILD_RELEASE% >>%BUILD_LOG% 2>nul ) || call :buildFailLog geotiff "build release" && goto icu
( %_CMAKE_INSTALL_RELEASE%  >>%BUILD_LOG% 2>nul ) || call :buildFailLog geotiff "build release" && goto icu
( %_CMAKE_BUILD_DEBUG% >>%BUILD_LOG% 2>nul ) || call :buildFailLog geotiff "build debug" && goto icu
( %_CMAKE_INSTALL_DEBUG% >>%BUILD_LOG% 2>nul ) || call :buildFailLog geotiff "build debug" && goto icu

xcopy %TERRALIB_DEPENDENCIES_DIR%\bin\geotiff*.dll %TERRALIB_DEPENDENCIES_DIR%\lib >nul 2>nul
del /Q %TERRALIB_DEPENDENCIES_DIR%\bin\geotiff*.dll >nul 2>nul

echo done.

call :append_log_end geotiff

:end_geotiff

cd %ROOT_DIR%
:: ====

:icu

set "_libF="

IF DEFINED TERRALIB_X64 set "_libF=64"

::  ICU version 58.2
set ICU_DIR=%ROOT_DIR%\icu-59.1-vs2017
set ICU_INCLUDE_DIR=%ICU_DIR%\include
set ICU_LIBRARY=%ICU_DIR%\lib%_libF%\icuuc.lib
set ICUD_LIBRARY=%ICU_DIR%\lib%_libF%\icuucd.lib
set ICU_BIN=%ICU_DIR%\lib%_libF%

:: Check dependencies
goto end_icu_deps
:icu_deps
goto xerces_deps
:end_icu_deps

echo | set /p="Installing icu... "<nul

IF EXIST %ICU% call :skip_build && goto xerces

call :append_log_begin icu

:begin_icu

cd %ICU_DIR%\source\allinone >nul 2>nul

xcopy %ICU_DIR%\bin%_libF%\icuuc*.dll %LIBS_DIR% /Y >nul 2>nul

xcopy %ICU_DIR%\bin%_libF%\icuin*.dll %LIBS_DIR% /Y >nul 2>nul

xcopy %ICU_DIR%\bin%_libF%\icudt*.dll %LIBS_DIR% /Y >nul 2>nul

echo done.

call :append_log_end icu

:end_icu

cd %ROOT_DIR%
:: ====

:xerces

set "_libF=x86"

IF DEFINED TERRALIB_X64 set "_libF=x64"

::  XERCES version 3.1.4
set XERCES_DIR=%TERRALIB_DEPENDENCIES_DIR%
set XERCES_INCLUDE_DIR=%XERCES_DIR%\include
set XERCES_LIBRARY=%LIBS_DIR%\xerces-c_3.lib
set XERCESD_LIBRARY=%LIBS_DIR%\xerces-c_3D.lib
set XERCESCROOT=%ROOT_DIR%\xerces-c-3.1.4
set XERCES_ICU_LIBRARY=%XERCESCROOT%\src\xercesc\util\MsgLoaders\ICU\resources\%_libF%\xercesc_messages_3_1.lib

:: Check dependencies
goto end_xerces_deps
:xerces_deps
IF NOT EXIST %ICU% call :remove_lib xerces && goto xml2_deps
goto xml2_deps
:end_xerces_deps

echo | set /p="Installing xerces... "<nul

IF EXIST %XERCES% call :skip_build && goto xml2

call :append_log_begin xerces

:begin_xerces

set "_xerces_output_dir=Win32"
IF DEFINED TERRALIB_X64 set "_xerces_output_dir=Win64"

cd %XERCESCROOT%\src\xercesc\util\MsgLoaders\ICU\resources >nul 2>nul

:: Generating message files
genrb root.txt 2> %XERCESCROOT%\..\build.log >nul 2>nul

pkgdata --name xercesc_messages_3_1 --mode dll -O %ICU_DIR% -c -d . -e xercesc_messages_3_1 res-file-list.txt -r _1 >nul 2>nul

IF NOT EXIST %_libF% mkdir %_libF% >nul 2>nul

xcopy *.dll %_libF% /Y >nul 2>nul

xcopy *.lib  %_libF% /Y >nul 2>nul

del *.dll *.lib *.exp *.obj /Q >nul 2>nul

cd %XERCESCROOT%\projects\Win32\VC15\xerces-all >nul 2>nul

set "INCLUDE=%ICU_INCLUDE_DIR%;%CURL_INCLUDE_DIR%;%additional_include_path%;%INCLUDE%"
set "LIB=%ICU_LIBRARY%;%CURL_LIBRARY%;%ICUD_LIBRARY%;%CURLD_LIBRARY%;%XERCES_ICU_LIBRARY%;%additional_lib_path%;%LIB%"
set UseEnv=true

( msbuild /m /t:XercesLib /t:clean /p:PlatformToolset=%MSVC_PLATFORM_TOOL%;ToolsVersion=%VS_VERSION%;WindowsTargetPlatformVersion=%MSVC_SDK_VERSION%;Configuration="ICU Release" /p:Platform=%_MSVC_PLATFORM% xerces-all.sln >>%BUILD_LOG% 2>nul ) || call :buildFailLog xerces  "clean release" && goto xml2

( msbuild /m /t:XercesLib /t:clean /p:PlatformToolset=%MSVC_PLATFORM_TOOL%;ToolsVersion=%VS_VERSION%;WindowsTargetPlatformVersion=%MSVC_SDK_VERSION%;Configuration="ICU Debug" /p:Platform=%_MSVC_PLATFORM% xerces-all.sln >>%BUILD_LOG% 2>nul ) || call :buildFailLog xerces  "clean debug" && goto xml2

( msbuild /m /t:XercesLib /p:PlatformToolset=%MSVC_PLATFORM_TOOL%;ToolsVersion=%VS_VERSION%;WindowsTargetPlatformVersion=%MSVC_SDK_VERSION%;Configuration="ICU Release" /p:Platform=%_MSVC_PLATFORM% xerces-all.sln >>%BUILD_LOG% 2>nul ) || call :buildFailLog xerces "build release" && goto xml2

( msbuild /m /t:XercesLib /p:PlatformToolset=%MSVC_PLATFORM_TOOL%;ToolsVersion=%VS_VERSION%;WindowsTargetPlatformVersion=%MSVC_SDK_VERSION%;Configuration="ICU Debug" /p:Platform=%_MSVC_PLATFORM% xerces-all.sln >>%BUILD_LOG% 2>nul ) || call :buildFailLog xerces "build debug" && goto xml2

:: Install
cd %XERCESCROOT% >nul 2>nul

IF NOT EXIST %TERRALIB_DEPENDENCIES_DIR%\include\xercesc mkdir %TERRALIB_DEPENDENCIES_DIR%\include\xercesc >nul 2>nul

xcopy "%XERCESCROOT%\Build\%_xerces_output_dir%\VC14\ICU Release\*.dll" %LIBS_DIR% /Y >nul 2>nul

xcopy "%XERCESCROOT%\Build\%_xerces_output_dir%\VC14\ICU Debug\*.dll" %LIBS_DIR% /Y >nul 2>nul

xcopy "%XERCESCROOT%\Build\%_xerces_output_dir%\VC14\ICU Release\*.lib" %LIBS_DIR% /Y >nul 2>nul

xcopy "%XERCESCROOT%\Build\%_xerces_output_dir%\VC14\ICU Debug\*.lib" %LIBS_DIR% /Y >nul 2>nul

xcopy %XERCESCROOT%\src\xercesc\util\MsgLoaders\ICU\resources\%_libF%\*.dll %LIBS_DIR% /Y >nul 2>nul

xcopy %XERCESCROOT%\src\xercesc\*.h* %TERRALIB_DEPENDENCIES_DIR%\include\xercesc /S /Y >nul 2>nul

xcopy %XERCESCROOT%\src\xercesc\framework\psvi\*.c %TERRALIB_DEPENDENCIES_DIR%\include\xercesc\framework\psvi /S /Y >nul 2>nul

xcopy %XERCESCROOT%\src\xercesc\util\*.c %TERRALIB_DEPENDENCIES_DIR%\include\xercesc\util /S /Y >nul 2>nul

echo done.

call :append_log_end xerces

:end_xerces

cd %ROOT_DIR%
:: ====

:xml2

::  LibXML2 version 2.9.1
set XML2_DIR=%ROOT_DIR%\libxml2-2.9.1
set XML2_INCLUDE_DIR=%XML2_DIR%\include
set XML2_LIBRARY=%XML2_DIR%\win32\bin%_X86%.msvc\libxml2.lib
set XML2D_LIBRARY=%XML2_DIR%\win32\bin%_X86%.dbg.msvc\libxml2d.lib

:: Check dependencies
goto end_xml2_deps
:xml2_deps
IF NOT EXIST %ICONV% call :remove_lib xml2 && goto libboost_deps
IF NOT EXIST %ICU% call :remove_lib xml2 && goto libboost_deps
IF NOT EXIST %ZLIB% call :remove_lib xml2 && goto libboost_deps
goto libboost_deps
:end_xml2_deps

echo | set /p="Installing xml2... "<nul

IF EXIST %XML2% call :skip_build && goto libboost

call :append_log_begin xml2

:begin_xml2

set "_bits=_x86"
IF DEFINED TERRALIB_X64 (
  set "_bits=_x64"
  set "_win64=WIN64=1"
)

cd %XML2_DIR%\win32 >nul 2>nul

del bin%_bits%.msvc /S /Q >nul 2>nul

del bin%_bits%.dbg.msvc /S /Q >nul 2>nul

del int%_bits%.msvc /S /Q >nul 2>nul

del int%_bits%.dbg.msvc /S /Q >nul 2>nul

( nmake /f Makefile.msvc %_win64% libxml >>%BUILD_LOG% 2>nul ) || call :buildFailLog xml2 "build release" && goto libboost

( nmake /f Makefile.msvc %_win64% DEBUG=1 libxml >>%BUILD_LOG% 2>nul ) || call :buildFailLog xml2 "build debug" && goto libboost

xcopy %XML2_DIR%\win32\bin%_bits%.msvc\*.dll %LIBS_DIR% /Y >nul 2>nul

xcopy %XML2_DIR%\win32\bin%_bits%.dbg.msvc\*.dll %LIBS_DIR% /Y >nul 2>nul

echo done.

call :append_log_end xml2

:end_xml2

cd %ROOT_DIR%
:: ====

:libboost

:: BOOST version 1.74.0
set B_DIR=%ROOT_DIR%\boost_1_74_0
set BOOST_INCLUDE_DIR=%TERRALIB_DEPENDENCIES_DIR%\include
set BOOST_BUILD_PATH=%B_DIR%
set BOOST_BUILD_DIR=%B_DIR%\build%_X86%

:: Check dependencies
goto end_libboost_deps
:libboost_deps
IF NOT EXIST %BZIP% call :remove_lib boost && goto pgis_deps
IF NOT EXIST %ICU% call :remove_lib boost && goto pgis_deps
IF NOT EXIST %ZLIB% call :remove_lib boost && goto pgis_deps
goto pgis_deps
:end_libboost_deps

echo | set /p="Installing boost... "<nul

IF EXIST %BOOST% call :skip_build && goto websocketpp

call :append_log_begin libboost

:begin_libboost

cd %B_DIR% >nul 2>nul

set "_b2_setup=setup x86"
set "_am=address-model=32"

IF DEFINED TERRALIB_X64 (
  set "_b2_setup=setup amd64"
  set "_am=address-model=64"
)

set "BOOST_REGEX_NO_LIB=1"

call :remove_lib boost

( call bootstrap.bat vc141 %_b2_setup% >>%CONFIG_LOG% 2>nul ) || call :buildFailLog libboost "configuring" && goto websocketpp

set ZLIB_SOURCE=%ZL_DIR%
set ZLIB_INCLUDE=%ZL_DIR%\build%_X86%
set ZLIB_LIBPATH=%ZL_DIR%\build%_X86%\Release\
set ZLIB_BINARY=zlib

set BZIP2_INCLUDE=%BZIP2_DIR%
set BZIP2_LIBPATH=%BZIP2_DIR%\lib%_X86%\
set BZIP2_BINARY=libbz2

:: --layout=versioned
( b2 --reconfigure ^
toolset=msvc-14.1 %_am% ^
variant=debug,release ^
link=shared ^
threading=multi ^
runtime-link=shared ^
include=%ZL_DIR%\build%_X86% ^
--abbreviate-paths ^
--build-dir=%BOOST_BUILD_DIR% ^
--prefix=%TERRALIB_DEPENDENCIES_DIR% ^
--with-random ^
--with-chrono ^
--with-date_time ^
--with-filesystem ^
--with-system ^
--with-thread ^
--with-timer ^
--with-locale ^
--with-iostreams ^
--with-regex ^
--with-test ^
--with-exception ^
--with-log ^
--with-python ^
--with-program_options ^
--with-stacktrace ^
--layout=tagged ^
-sICU_PATH=%ICU_DIR% ^
install -j %NUMBER_OF_PROCESSORS% >>%BUILD_LOG% 2>nul ) || call :buildFailLog libboost "building %_build_type%" && goto websocketpp

echo done.

call :append_log_end libboost

:end_libboost

cd %ROOT_DIR%
:: ====

:websocketpp

:: WebSocket 0.8.2
::  ====
set WEBSOCKETPP_DIR=%ROOT_DIR%\websocketpp-0.8.2
set WEBSOCKETPP_INSTALL=%WEBSOCKETPP_DIR%\install%_X86%

:websocketpp_deps

:end_websocketpp_deps

echo | set /p="Installing WebSocketPP... "<nul

IF EXIST %WEBSOCKETPP_INSTALL% call :skip_build && goto cpprestsdk

:begin_websocketpp

cd %WEBSOCKETPP_DIR% >nul 2>nul

RD /S /Q build%_X86% >nul 2>nul

mkdir build%_X86% >nul 2>nul

cd build%_X86% >nul 2>nul

:: Runs cmake to create the build project

( cmake -G %_CMAKE_GENERATOR% ^
-DCMAKE_INSTALL_PREFIX=%WEBSOCKETPP_INSTALL% ^
-DCMAKE_PREFIX_PATH=%TERRALIB_DEPENDENCIES_DIR% ^
-DCMAKE_DEBUG_POSTFIX=d ^
-DBOOST_ROOT:PATH=%TERRALIB_DEPENDENCIES_DIR% ^
-DENABLE_CPP11:BOOL=ON ^
-DBUILD_TESTS:BOOL=OFF ^
-DBUILD_EXAMPLES:BOOL=OFF ^
%WEBSOCKETPP_DIR% >>%CONFIG_LOG% 2>nul ) || call :buildFailLog websocketpp "configuring" && goto cpprestsdk

:: No build is needed. We must just install the headers and the cmake files
( %_CMAKE_INSTALL_RELEASE% >>%BUILD_LOG% 2>nul ) || call :buildFailLog websocketpp "build release" && goto cpprestsdk

echo done.

call :append_log_end websocketpp

:end_websocketpp

cd %ROOT_DIR%
:: ====

:cpprestsdk

:: CppRestSDK 2.10.16
::  ====
set CPPRESTSDK_DIR=%ROOT_DIR%\cpprestsdk-2.10.16
set CPPRESTSDK_INSTALL=%CPPRESTSDK_DIR%\install%_X86%

:cpprestsdk_deps

:end_cpprestsdk_deps

echo | set /p="Installing CppRestSDK... "<nul

IF EXIST %CPPRESTSDK% call :skip_build && goto minizip

:begin_cpprestsdk

cd %CPPRESTSDK_DIR% >nul 2>nul

RD /S /Q build%_X86% >nul 2>nul

mkdir build%_X86% >nul 2>nul

cd build%_X86% >nul 2>nul

:: Runs cmake to create the build project
( cmake -G %_CMAKE_GENERATOR% ^
-DCMAKE_INSTALL_PREFIX=%TERRALIB_DEPENDENCIES_DIR% ^
-DCMAKE_PREFIX_PATH=%TERRALIB_DEPENDENCIES_DIR%;%WEBSOCKETPP_INSTALL% ^
-DCMAKE_DEBUG_POSTFIX=d ^
-DCMAKE_INSTALL_BINDIR:PATH=lib ^
-DBOOST_ROOT:PATH=%TERRALIB_DEPENDENCIES_DIR% ^
-DBUILD_SHARED_LIBS:BOOL=ON ^
-DBUILD_TESTS:BOOL=OFF ^
-DBUILD_SAMPLES:BOOL=OFF ^
%CPPRESTSDK_DIR% >>%CONFIG_LOG% 2>nul ) || call :buildFailLog cpprestsdk "configuring" && goto minizip

:: Build and install both Debug and Release
( %_CMAKE_BUILD_RELEASE% >>%BUILD_LOG% 2>nul ) || call :buildFailLog cpprestsdk "build release" && goto minizip
( %_CMAKE_INSTALL_RELEASE%  >>%BUILD_LOG% 2>nul ) || call :buildFailLog cpprestsdk "build release" && goto minizip
( %_CMAKE_BUILD_DEBUG% >>%BUILD_LOG% 2>nul ) || call :buildFailLog cpprestsdk "build debug" && goto minizip
( %_CMAKE_INSTALL_DEBUG% >>%BUILD_LOG% 2>nul ) || call :buildFailLog cpprestsdk "build debug" && goto minizip


echo done.

call :append_log_end cpprestsdk

:end_cpprestsdk

cd %ROOT_DIR%
:: ====

:minizip

:: Minizip
::  ====
set MINIZIP_DIR=%ROOT_DIR%\unzip101e
set MZ_INCLUDE_DIR=%MINIZIP_DIR%
set MINIZIP_LIBRARY=%MINIZIP_DIR%\build%_X86%\Release\minizip.lib
set MINIZIPD_LIBRARY=%MINIZIP_DIR%\build%_X86%\Debug\minizipd.lib
set MINIZIP_LIBRARIES="debug;%MINIZIPD_LIBRARY%;optimized;%MINIZIP_LIBRARY%"

:: Check dependencies
goto end_minizip_deps
:minizip_deps
IF NOT EXIST %ZLIB% del %MINIZIP% /Q >nul 2>nul && goto uriparser_deps
goto uriparser_deps
:end_minizip_deps

echo | set /p="Installing minizip... "<nul

IF EXIST %MINIZIP% call :skip_build && goto uriparser

call :append_log_begin minizip

:begin_minizip

cd %MINIZIP_DIR% >nul 2>nul

RD /S /Q build%_X86% >nul 2>nul

mkdir build%_X86% >nul 2>nul

cd build%_X86% >nul 2>nul

:: Runs cmake to create the build project
( cmake -G %_CMAKE_GENERATOR% ^
-DCMAKE_DEBUG_POSTFIX=d ^
-DZLIB_INCLUDE_DIR="%ZL_INCLUDE_DIR%" ^
-DZLIB_LIBRARY:STRING="%ZL_LIBRARIES%" ^
%MINIZIP_DIR% >>%CONFIG_LOG% 2>nul ) || call :buildFailLog minizip "configuring" && goto uriparser

:: Build and install both Debug and Release
( %_CMAKE_BUILD_RELEASE% >>%BUILD_LOG% 2>nul ) || call :buildFailLog minizip "build release" && goto uriparser
( %_CMAKE_BUILD_DEBUG% >>%BUILD_LOG% 2>nul ) || call :buildFailLog minizip "build debug" && goto uriparser

echo done.

call :append_log_end minizip

:end_minizip

cd %ROOT_DIR%
:: ====

:uriparser

set "_bits=_x64"
IF NOT DEFINED TERRALIB_X64 set "_bits=_x86"

::  URIParser
set URIPARSER_DIR=%ROOT_DIR%\uriparser-0.8.4
set UR_INCLUDE_DIR=%URIPARSER_DIR%\include
set URIPARSER_LIBRARY=%URIPARSER_DIR%\win32\build%_bits%\uriparser.lib
set URIPARSERD_LIBRARY=%URIPARSER_DIR%\win32\build%_bits%\uriparserd.lib
set URIPARSER_LIBRARIES=debug;%URIPARSERD_LIBRARY%;optimized;%URIPARSER_LIBRARY%

:: Check dependencies
goto end_uriparser_deps
:uriparser_deps
goto libkml_deps
:end_uriparser_deps

echo | set /p="Installing uriparser... "<nul

IF EXIST %URIPARSER% call :skip_build && goto libkml

call :append_log_begin uriparser

:begin_uriparser

cd %URIPARSER_DIR%\win32\%MSVC_FOLDER% >nul 2>nul

( msbuild /m /p:Configuration=Release uriparser.vcxproj /t:clean >>%BUILD_LOG% 2>nul ) || call :buildFailLog uriparser "clean release" && goto libkml

( msbuild /m uriparser.vcxproj /t:clean >>%BUILD_LOG% 2>nul ) || call :buildFailLog uriparser "clean debug" && goto libkml

( msbuild /m /p:PlatformToolset=%MSVC_PLATFORM_TOOL%;ToolsVersion=%VS_VERSION%;WindowsTargetPlatformVersion=%MSVC_SDK_VERSION%;Configuration=Release /p:Platform=%_MSVC_PLATFORM% uriparser.vcxproj >>%BUILD_LOG% 2>nul ) || call :buildFailLog uriparser "build release" && goto libkml

( msbuild /m /p:PlatformToolset=%MSVC_PLATFORM_TOOL%;ToolsVersion=%VS_VERSION%;WindowsTargetPlatformVersion=%MSVC_SDK_VERSION% /p:Platform=%_MSVC_PLATFORM% uriparser.vcxproj >>%BUILD_LOG% 2>nul ) || call :buildFailLog uriparser "build debug" && goto libkml

echo done.

call :append_log_end uriparser

:end_uriparser

cd %ROOT_DIR%
:: ====

:libkml

:: LibKML
::  ====
set LIBKML_DIR=%ROOT_DIR%\libkml-master
set LIBKML_INCLUDE_DIR=%LIBKML_DIR%\src
set LIBKML_LIBRARY=%LIBKML_DIR%\build%_X86%\Release\libkml.lib
set LIBKMLD_LIBRARY=%LIBKML_DIR%\build%_X86%\Debug\libkmld.lib

:: Check dependencies
goto end_libkml_deps
:libkml_deps
IF NOT EXIST %BOOST% del %LIBKML% /Q >nul 2>nul && goto pgis_deps
IF NOT EXIST %EXPAT% del %LIBKML% /Q >nul 2>nul && goto pgis_deps
IF NOT EXIST %MINIZIP% del %LIBKML% /Q >nul 2>nul && goto pgis_deps
IF NOT EXIST %URIPARSER% del %LIBKML% /Q >nul 2>nul && goto pgis_deps
IF NOT EXIST %ZLIB% del %LIBKML% /Q >nul 2>nul && goto pgis_deps
goto pgis_deps
:end_libkml_deps

echo | set /p="Installing libkml... "<nul

IF EXIST %LIBKML% call :skip_build && goto pgis

call :append_log_begin libkml

:begin_libkml

cd %LIBKML_DIR% >nul 2>nul

del build%_X86% /S /Q >nul 2>nul

mkdir build%_X86% >nul 2>nul

cd build%_X86% >nul 2>nul

:: Runs cmake to create the build project
( cmake -G %_CMAKE_GENERATOR% ^
-DCMAKE_DEBUG_POSTFIX=d ^
-DCMAKE_PREFIX_PATH="%TERRALIB_DEPENDENCIES_DIR%;%ZL_DIR%;%EXPAT_DIR%\binaries%_X86%;%MINIZIP_DIR%;%URIPARSER_DIR%" ^
%LIBKML_DIR% >>%CONFIG_LOG% 2>nul ) || call :buildFailLog libkml "configuring" && goto pgis

:: Build and install both Debug and Release
( %_CMAKE_BUILD_RELEASE% >>%BUILD_LOG% 2>nul ) || call :buildFailLog libkml "build release" && goto pgis
( %_CMAKE_BUILD_DEBUG% >>%BUILD_LOG% 2>nul ) || call :buildFailLog libkml "build debug" && goto pgis

echo done.

call :append_log_end libkml

:end_libkml

cd %ROOT_DIR%

:pgis

:: PostgreSQL version 9.4.1
set PGis_DIR=%ROOT_DIR%\postgresql-9.4.1
set PG_INCLUDE_DIR=%TERRALIB_DEPENDENCIES_DIR%\include
set PG_LIBRARY=%PGIS%
set PGD_LIBRARY=%LIBS_DIR%\libpqddll.lib

:: Check dependencies
goto end_pgis_deps
:pgis_deps
IF NOT EXIST %SSL% call :remove_lib pgis && goto netcdf_deps
goto netcdf_deps
:end_pgis_deps

echo | set /p="Installing libpq... "<nul

IF EXIST %PGIS% call :skip_build && goto netcdf

call :append_log_begin pgis

:begin_pgis

cd %PGis_DIR%\src\interfaces\libpq >nul 2>nul

set "_cpu=i386"
set "ARC_DIR=x86"
set "ADD_DEFINES=/D _WIN32_WINNT=0x0600"

IF DEFINED TERRALIB_X64 (
set "_cpu=AMD64"
set "ARC_DIR=x64"
set "ADD_DEFINES="
)

del %ARC_DIR% /S /Q >nul 2>nul

( nmake /f win32.mak CPU="%_cpu%" USE_SSL=1 >>%BUILD_LOG% 2>nul ) || call :buildFailLog pgis "build release" && goto netcdf

( nmake /f win32.mak CPU="%_cpu%" USE_SSL=1 DEBUG=1 >>%BUILD_LOG% 2>nul ) || call :buildFailLog pgis "build debug" && goto netcdf

xcopy libpq-fe.h %TERRALIB_DEPENDENCIES_DIR%\include /Y >nul 2>nul

xcopy %PGis_DIR%\src\include\pg_config_ext.h %TERRALIB_DEPENDENCIES_DIR%\include /Y >nul 2>nul

xcopy %PGis_DIR%\src\include\postgres_ext.h %TERRALIB_DEPENDENCIES_DIR%\include /Y >nul 2>nul

xcopy %ARC_DIR%\Release\libpqdll.lib %LIBS_DIR% /Y >nul 2>nul

xcopy %ARC_DIR%\Release\*.dll %LIBS_DIR% /Y >nul 2>nul

xcopy %ARC_DIR%\Debug\libpqddll.lib %LIBS_DIR% /Y >nul 2>nul

xcopy %ARC_DIR%\Debug\*.dll %LIBS_DIR% /Y >nul 2>nul

echo done.

call :append_log_end pgis

:end_pgis

cd %ROOT_DIR%
:: ====

:netcdf

:: netcdf version 4.7.4
set NETCDF_DIR=%ROOT_DIR%\netcdf-c-4.7.4
set NETCDF_INCLUDE_DIR=%NETCDF_DIR%\binaries%_X86%\include
set NETCDF_LIBRARY=%NETCDF_DIR%\binaries%_X86%\lib\netcdf.lib
set NETCDFD_LIBRARY=%NETCDF_DIR%\binaries%_X86%\lib\netcdfd.lib

:: Check dependencies
goto end_netcdf_deps
:netcdf_deps
IF NOT EXIST %JPEG% call :remove_lib netcdf && goto spatialite
IF NOT EXIST %CURL% call :remove_lib netcdf && goto spatialite
IF NOT EXIST %HDF4% call :remove_lib netcdf && goto spatialite
goto sqlite_deps
:end_netcdf_deps

echo | set /p="Installing netcdf... "<nul

IF EXIST %NETCDF% call :skip_build && goto spatialite

call :append_log_begin netcdf

:begin_netcdf

cd %NETCDF_DIR% >nul 2>nul

IF EXIST binaries%_X86% RD /S /Q binaries%_X86% >nul 2>nul

IF EXIST build%_X86% RD /S /Q build%_X86% >nul 2>nul

mkdir build%_X86% >nul 2>nul

cd build%_X86% >nul 2>nul

:: Runs cmake to create the build project
( cmake -G %_CMAKE_GENERATOR% ^
-DCMAKE_INSTALL_PREFIX=%NETCDF_DIR%\binaries%_X86% ^
-DCMAKE_PREFIX_PATH="%TERRALIB_DEPENDENCIES_DIR%" ^
-DCMAKE_DEBUG_POSTFIX=d ^
-DENABLE_DLL=ON ^
-DBUILD_TESTING=OFF ^
-DBUILD_UTILITIES=OFF ^
-DENABLE_TESTS=OFF ^
-DBUILD_SHARED_LIBS=ON ^
-DENABLE_HDF4=ON ^
-DUSE_HDF4=ON ^
-DUSE_HDF5=OFF ^
-DHDF4_DF_LIB=%HDF4_LIBRARY% ^
-DHDF4_MFHDF_LIB=%MFHDF_LIBRARY% ^
-DMFHDF_H_INCLUDE_DIR=%HDF4_INCLUDE_DIR% ^
-DJPEG_LIB=%JPG_LIBRARY% ^
-DJPEGLIB_H_INCLUDE_DIR=%JPG_DIR% ^
-DCURL_INCLUDE_DIR=%CURL_INCLUDE_DIR% ^
-DCURL_LIBRARY_DEBUG=%CURLD_LIBRARY% ^
-DCURL_LIBRARY_RELEASE=%CURL_LIBRARY% ^
-DPROG_CURL=%CURL_EXE% ^
-DENABLE_NETCDF_4=OFF %NETCDF_DIR% >>%CONFIG_LOG% 2>nul ) || call :buildFailLog netcdf "configuring" && goto spatialite

:: Build and install both Debug and Release
( %_CMAKE_BUILD_RELEASE% >>%BUILD_LOG% 2>nul ) || call :buildFailLog netcdf "build release" && goto spatialite
( %_CMAKE_INSTALL_RELEASE%  >>%BUILD_LOG% 2>nul ) || call :buildFailLog netcdf "build release" && goto spatialite
( %_CMAKE_BUILD_DEBUG% >>%BUILD_LOG% 2>nul ) || call :buildFailLog netcdf "build debug" && goto spatialite
( %_CMAKE_INSTALL_DEBUG% >>%BUILD_LOG% 2>nul ) || call :buildFailLog netcdf "build debug" && goto spatialite

xcopy %NETCDF_DIR%\binaries%_X86%\bin\net* %LIBS_DIR% /Y >nul 2>nul

echo done.

call :append_log_end netcdf

:end_netcdf

cd %ROOT_DIR%
:: ====

:spatialite

:: SpatialLite version 4.3.0a
set SPLITE_DIR=%ROOT_DIR%\libspatialite-4.3.0a
set SPLITE_INCLUDE_DIR=%TERRALIB_DEPENDENCIES_DIR%\include
set SPLITE_LIBRARY=%SPATIALITE%
set SPLITED_LIBRARY=%LIBS_DIR%\spatialite_id.lib

:: Check dependencies
goto end_spatialite_deps
:spatialite_deps
IF NOT EXIST %PROJ% call :remove_lib spatialite && goto gdal_deps
IF NOT EXIST %GEOS% call :remove_lib spatialite && goto gdal_deps
IF NOT EXIST %ICONV% call :remove_lib spatialite && goto gdal_deps
IF NOT EXIST %SQLITE% call :remove_lib spatialite && goto gdal_deps
IF NOT EXIST %FREEXL% call :remove_lib spatialite && goto gdal_deps
IF NOT EXIST %ZLIB% call :remove_lib spatialite && goto gdal_deps
IF NOT EXIST %XML2% call :remove_lib spatialite && goto gdal_deps
goto gdal_deps
:end_spatialite_deps

echo | set /p="Installing spatialite... "<nul

IF EXIST %SPATIALITE% call :skip_build && goto gdal

call :append_log_begin spatialite

:begin_spatialite

cd %SPLITE_DIR% >nul 2>nul

del *.obj /S /Q >nul 2>nul

( nmake /f makefile.vc install >>%BUILD_LOG% 2>nul ) || call :buildFailLog spatialite "build release" && goto gdal

( nmake /f makefile.vc clean >>%BUILD_LOG% 2>nul ) || call :buildFailLog spatialite  "clean release" && goto gdal

( nmake /f makefile.vc DEBUG=1 install >>%BUILD_LOG% 2>nul ) || call :buildFailLog spatialite "build debug" && goto gdal

( nmake /f makefile.vc DEBUG=1 clean >>%BUILD_LOG% 2>nul ) || call :buildFailLog spatialite  "clean debug" && goto gdal

echo done.

call :append_log_end spatialite

:end_spatialite

cd %ROOT_DIR%
:: ====

:gdal

:: GDAL version 3.1.2
set GDAL_DIR=%ROOT_DIR%\gdal-3.1.2
set GDAL_INCLUDE_DIR=%TERRALIB_DEPENDENCIES_DIR%\include
set GDAL_LIBRARY=%GDAL%
set GDALD_LIBRARY=%LIBS_DIR%\gdal_id.lib

:: Check dependencies
goto end_gdal_deps
:gdal_deps
IF NOT EXIST %XERCES% call :remove_lib gdal && goto property_browser_deps
IF NOT EXIST %PROJ% call :remove_lib gdal && goto && goto property_browser_deps
IF NOT EXIST %SQLITE% call :remove_lib gdal && goto property_browser_deps
IF NOT EXIST %SPATIALITE% call :remove_lib gdal && goto property_browser_deps
IF NOT EXIST %GEOS% call :remove_lib gdal && goto property_browser_deps
IF NOT EXIST %ICONV% call :remove_lib gdal && goto property_browser_deps
IF NOT EXIST %EXPAT% call :remove_lib gdal && goto property_browser_deps
IF NOT EXIST %HDF4% call :remove_lib gdal && goto property_browser_deps
IF NOT EXIST %NETCDF% call :remove_lib gdal && goto property_browser_deps
IF NOT EXIST %GEOTIFF% call :remove_lib gdal && goto property_browser_deps
IF NOT EXIST %TIFF% call :remove_lib gdal && goto property_browser_deps
IF NOT EXIST %CURL% call :remove_lib gdal && goto property_browser_deps
IF NOT EXIST %PNG% call :remove_lib gdal && goto property_browser_deps
IF NOT EXIST %FREEXL% call :remove_lib gdal && goto property_browser_deps
IF NOT EXIST %XML2% call :remove_lib gdal && goto property_browser_deps
IF NOT EXIST %PCRE% call :remove_lib gdal && goto property_browser_deps
IF NOT EXIST %LIBKML% call :remove_lib gdal && goto property_browser_deps
goto property_browser_deps
:end_gdal_deps

echo | set /p="Installing gdal... "<nul

IF EXIST %GDAL% call :skip_build && goto property_browser

call :append_log_begin gdal

:begin_gdal

cd %GDAL_DIR% >nul 2>nul

del *.lib /S /Q >nul 2>nul

del *.exp /S /Q >nul 2>nul

del *.obj /S /Q >nul 2>nul

SET GDAL_WIN64=""

IF DEFINED TERRALIB_X64 (
	SET GDAL_WIN64="WIN64=1"
)

copy nmake.local.release.in nmake.local /Y >nul 2>nul

( nmake -f makefile.vc MSVC_VER=%MSVC_VERSION% DLLBUILD=1 %GDAL_WIN64% VERSION=312 clean >>%BUILD_LOG% 2>nul ) || call :buildFailLog gdal  "clean release" && goto property_browser

( nmake -f makefile.vc MSVC_VER=%MSVC_VERSION% DLLBUILD=1 %GDAL_WIN64% VERSION=312 >>%BUILD_LOG% 2>nul ) || call :buildFailLog gdal "build release" && goto property_browser

( nmake -f makefile.vc MSVC_VER=%MSVC_VERSION% DLLBUILD=1 %GDAL_WIN64% VERSION=312 devinstall >>%BUILD_LOG% 2>nul ) || call :buildFailLog gdal "install" property_browser

xcopy %LIBS_DIR%/ogr*.exe %BINS_DIR% /Y >nul 2>nul
xcopy %LIBS_DIR%/gdal*.exe %BINS_DIR% /Y >nul 2>nul

( nmake -f makefile.vc MSVC_VER=%MSVC_VERSION% DLLBUILD=1 %GDAL_WIN64% VERSION=312 clean >>%BUILD_LOG% 2>nul ) || call :buildFailLog gdal  "clean release" && goto property_browser

copy nmake.local.debug.in nmake.local /Y >nul 2>nul

( nmake -f makefile.vc MSVC_VER=%MSVC_VERSION% DLLBUILD=1 DEBUG=1 %GDAL_WIN64% VERSION=312 >>%BUILD_LOG% 2>nul ) || call :buildFailLog gdal "build debug" && goto property_browser

xcopy *.dll %LIBS_DIR% /Y >nul 2>nul

xcopy gdal_id.lib %LIBS_DIR% /Y >nul 2>nul

( nmake -f makefile.vc MSVC_VER=%MSVC_VERSION% DLLBUILD=1 DEBUG=1 %GDAL_WIN64% VERSION=312 clean >>%BUILD_LOG% 2>nul ) || call :buildFailLog gdal  "clean debug" && goto property_browser

echo done.

call :append_log_end gdal

:end_gdal

cd %ROOT_DIR%
:: ====

:property_browser

:: QtPropertyBrowser
set QBROWSER_DIR=%ROOT_DIR%\qtpropertybrowser
set QBROWSER_INCLUDE_DIR=%TERRALIB_DEPENDENCIES_DIR%\include\qtpropertybrowser
set QBROWSER_LIBRARY=%PROPERTYBROWSER%
set QBROWSERD_LIBRARY=%LIBS_DIR%\qt_property_browserd.lib

:: Check dependencies
goto end_property_browser_deps
:property_browser_deps
goto qwt_deps
:end_property_browser_deps

echo | set /p="Installing property_browser... "<nul

IF EXIST %PROPERTYBROWSER% call :skip_build && goto qwt

call :append_log_begin property_browser

:begin_property_browser

cd %QBROWSER_DIR%\buildlib >nul 2>nul

( qmake "TERRALIB_DIR=%TERRALIB_DEPENDENCIES_DIR%" >>%CONFIG_LOG% 2>nul ) || call :buildFailLog property_browser "configuring" && goto qwt

( nmake clean >>%BUILD_LOG% 2>nul ) || call :buildFailLog property_browser "cleaning" && goto qwt

( nmake Makefile release install >>%BUILD_LOG% 2>nul ) || call :buildFailLog property_browser "building" && goto qwt

( nmake Makefile debug >>%BUILD_LOG% 2>nul ) || call :buildFailLog property_browser "installing" && goto qwt

copy ..\lib\qt_property_browserd.dll %LIBS_DIR% /Y >nul 2>nul

copy ..\lib\qt_property_browserd.lib %LIBS_DIR% /Y >nul 2>nul

echo done.

call :append_log_end property_browser

:end_property_browser

cd %ROOT_DIR%
:: ====

:qwt

:: QWT version 6.1.2
set QWT_PATH=%ROOT_DIR%\qwt-6.1.2

:: Check dependencies
goto end_qwt_deps
:qwt_deps
goto lua_deps
:end_qwt_deps

echo | set /p="Installing qwt... "<nul

IF EXIST %QWT% call :skip_build && goto lua

call :append_log_begin qwt

:begin_qwt

cd %QWT_PATH% >nul 2>nul

( qmake qwt.pro -r "QWT_INSTALL_PREFIX_TARGET=%TERRALIB_DEPENDENCIES_DIR%" >>%CONFIG_LOG% 2>nul ) || call :buildFailLog qwt "configuring" && goto lua

( nmake clean >>%BUILD_LOG% 2>nul ) || call :buildFailLog  qwt "cleaning" && goto lua

( nmake >>%BUILD_LOG% 2>nul ) || call :buildFailLog qwt "building" && goto lua

( nmake install >>%BUILD_LOG% 2>nul ) || call :buildFailLog qwt "installing" && goto lua

echo done.

call :append_log_end qwt

:end_qwt

cd %ROOT_DIR%
:: ====

:lua

:: Lua version 5.3.4
set LUAC_DIR=%ROOT_DIR%\lua-5.3.4
set LUAC_INCLUDE_DIR=%TERRALIB_DEPENDENCIES_DIR%\include\lua
set LUAC_LIBRARY=%LUA%

:: Check dependencies
goto end_lua_deps
:lua_deps
IF NOT EXIST %READLINE% call :remove_lib lua && goto qtlua_deps
goto qtlua_deps
:end_lua_deps

echo | set /p="Installing lua... "<nul

IF EXIST %LUA% call :skip_build && goto qtlua

call :append_log_begin lua

:begin_lua

cd %LUAC_DIR% >nul 2>nul

del build%_X86% /S /Q >nul 2>nul

mkdir build%_X86% >nul 2>nul

cd build%_X86% >nul 2>nul

:: Runs cmake to create the build project
( cmake -G %_CMAKE_GENERATOR% ^
-DCMAKE_INSTALL_PREFIX="%TERRALIB_DEPENDENCIES_DIR%" ^
-DINSTALL_INC="%LUAC_INCLUDE_DIR%" "%LUAC_DIR%" >>%CONFIG_LOG% 2>nul ) || call :buildFailLog lua  "configuring" && goto qscintilla

:: Build and install both Debug and Release
( %_CMAKE_BUILD_RELEASE% >>%BUILD_LOG% 2>nul ) || call :buildFailLog lua "build release" && goto qscintilla
( %_CMAKE_INSTALL_RELEASE%  >>%BUILD_LOG% 2>nul ) || call :buildFailLog lua "build release" && goto qscintilla
( %_CMAKE_BUILD_DEBUG% >>%BUILD_LOG% 2>nul ) || call :buildFailLog lua "build debug" && goto qscintilla
( %_CMAKE_INSTALL_DEBUG% >>%BUILD_LOG% 2>nul ) || call :buildFailLog lua "build debug" && goto qscintilla

echo done.

call :append_log_end lua

:end_lua

cd %ROOT_DIR%
:: ====

:qtlua

:: Check dependencies
goto end_qtlua_deps
:qtlua_deps
IF NOT EXIST %LUA% call :remove_lib qtlua && goto qtlua-extras_deps
goto qtlua-extras_deps
:end_qtlua_deps

:: QtLua version 2.0
set QTLUA_DIR=%ROOT_DIR%\libqtlua
set Qt5_DIR=%QMAKE_FILEPATH%\..
set LUAC_INCLUDE_DIR=%TERRALIB_DEPENDENCIES_DIR%\include\lua
set LUACD_LIBRARY=%LIBS_DIR%\luad.lib

echo | set /p="Installing qtlua... "<nul

IF EXIST %QTLUA% call :skip_build && goto qtlua-extras

call :append_log_begin qtlua

:begin_qtlua

cd %QTLUA_DIR% >nul 2>nul

IF EXIST build%_X86% RD /S /Q build%_X86% >nul 2>nul

mkdir build%_X86% >nul 2>nul

cd build%_X86% >nul 2>nul

:: Runs cmake to create the build project
( cmake -G %_CMAKE_GENERATOR% -DCMAKE_INSTALL_PREFIX="%TERRALIB_DEPENDENCIES_DIR%" ^
-DCMAKE_PREFIX_PATH="%Qt5_DIR%/bin;%Qt5_DIR%/lib/cmake;" ^
-DLUA_LIBRARY="%LUA%" -DLUA_INCLUDE_DIR="%LUAC_INCLUDE_DIR%" "%QTLUA_DIR%" >>%CONFIG_LOG% 2>nul ) || call :buildFailLog qtlua  "release configuring" && goto qtlua-extras

:: Build and install Release
( %_CMAKE_BUILD_RELEASE% >>%BUILD_LOG% 2>nul ) || call :buildFailLog qtlua "build release" && goto qtlua-extras
( %_CMAKE_INSTALL_RELEASE%  >>%BUILD_LOG% 2>nul ) || call :buildFailLog qtlua "build release" && goto qtlua-extras

cd %QTLUA_DIR% >nul 2>nul

IF EXIST build%_X86% RD /S /Q build%_X86% >nul 2>nul

mkdir build%_X86% >nul 2>nul

cd build%_X86% >nul 2>nul

:: Runs cmake to create the build project
( cmake -G %_CMAKE_GENERATOR% -DCMAKE_INSTALL_PREFIX="%TERRALIB_DEPENDENCIES_DIR%" ^
-DCMAKE_PREFIX_PATH="%Qt5_DIR%/bin;%Qt5_DIR%/lib/cmake;" ^
-DLUA_LIBRARY="%LUACD_LIBRARY%" -DLUA_INCLUDE_DIR="%LUAC_INCLUDE_DIR%" "%QTLUA_DIR%" >>%CONFIG_LOG% 2>nul ) || call :buildFailLog qtlua  "debug configuring" && goto qtlua-extras

:: Build and install Debug
( %_CMAKE_BUILD_DEBUG% >>%BUILD_LOG% 2>nul ) || call :buildFailLog qtlua "build debug" && goto qtlua-extras
( %_CMAKE_INSTALL_DEBUG% >>%BUILD_LOG% 2>nul ) || call :buildFailLog qtlua "build debug" && goto qtlua-extras

echo done.

call :append_log_end qtlua

:end_qtlua

cd %ROOT_DIR%
:: ====

:qtlua-extras

:: Check dependencies
goto end_qtlua-extras_deps
:qtlua-extras_deps
IF NOT EXIST %QTLUA% call :remove_lib qtlua-extras && goto qscintilla_deps
goto qscintilla_deps
:end_qtlua-extras_deps

:: QtLua Extras
set QTLUAE_DIR=%ROOT_DIR%\qtluae
set Qt5_DIR=%QMAKE_FILEPATH%\..
set LUAC_INCLUDE_DIR=%TERRALIB_DEPENDENCIES_DIR%\include\lua
set LUACD_LIBRARY=%LIBS_DIR%\luad.lib
set QTLUA_INCLUDE_DIR=%TERRALIB_DEPENDENCIES_DIR%\include
set QTLUAD_LIBRARY=%LIBS_DIR%\qtluad.lib

echo | set /p="Installing qtlua-extras... "<nul

IF EXIST %QTLUAE% call :skip_build && goto qscintilla

call :append_log_begin qtlua-extras

:begin_qtlua-extras

cd %QTLUAE_DIR% >nul 2>nul

del build%_X86% /S /Q >nul 2>nul

mkdir build%_X86% >nul 2>nul

cd build%_X86% >nul 2>nul

:: Runs cmake to create the build project
( cmake -G %_CMAKE_GENERATOR% -DCMAKE_INSTALL_PREFIX="%TERRALIB_DEPENDENCIES_DIR%" ^
-DCMAKE_PREFIX_PATH="%Qt5_DIR%/bin;%Qt5_DIR%/lib/cmake;" ^
-DLUA_LIBRARY="%LUA%" -DLUA_INCLUDE_DIR="%LUAC_INCLUDE_DIR%" ^
-DQTLUA_LIBRARY="%QTLUA%" -DQTLUA_INCLUDE_DIR="%QTLUA_INCLUDE_DIR%" "%QTLUAE_DIR%\build\cmake" >>%CONFIG_LOG% 2>nul ) || call :buildFailLog qtlua-extras  "release configuring" && goto qscintilla

:: Build and install Release
( %_CMAKE_BUILD_RELEASE% >>%BUILD_LOG% 2>nul ) || call :buildFailLog qtlua-extras "build release" && goto qscintilla
( %_CMAKE_INSTALL_RELEASE%  >>%BUILD_LOG% 2>nul ) || call :buildFailLog qtlua-extras "build release" && goto qscintilla

:: Runs cmake to create the build project
( cmake -G %_CMAKE_GENERATOR% -DCMAKE_INSTALL_PREFIX="%TERRALIB_DEPENDENCIES_DIR%" ^
-DCMAKE_PREFIX_PATH="%Qt5_DIR%/bin;%Qt5_DIR%/lib/cmake;" ^
-DLUA_LIBRARY="%LUACD_LIBRARY%" -DLUA_INCLUDE_DIR="%LUAC_INCLUDE_DIR%" ^
-DQTLUA_LIBRARY="%QTLUAD_LIBRARY%" -DQTLUA_INCLUDE_DIR="%QTLUA_INCLUDE_DIR%" "%QTLUAE_DIR%\build\cmake" >>%CONFIG_LOG% 2>nul ) || call :buildFailLog qtlua-extras  "debug configuring" && goto qscintilla

:: Build and install Debug
( %_CMAKE_BUILD_DEBUG% >>%BUILD_LOG% 2>nul ) || call :buildFailLog qtlua-extras "build debug" && goto qscintilla
( %_CMAKE_INSTALL_DEBUG% >>%BUILD_LOG% 2>nul ) || call :buildFailLog qtlua-extras "build debug" && goto qscintilla

echo done.

call :append_log_end qtlua-extras

:end_qtlua-extras

cd %ROOT_DIR%
:: ====

:qscintilla

:: Qscintilla version 2.10
set QSCINTILLA_PATH=%ROOT_DIR%\QScintilla_gpl-2.10

:: Check dependencies
goto end_qscintilla_deps
:qscintilla_deps
goto wtss_deps

:end_qscintilla_deps

echo | set /p="Installing qscintilla... "<nul

IF EXIST %QSCINTILLA% call :skip_build && goto wtss-cxx

call :append_log_begin qscintilla

:begin_qscintilla

cd %QSCINTILLA_PATH%\Qt4Qt5 >nul 2>nul

( qmake qscintilla.pro -r "TERRALIB_DIR=%TERRALIB_DEPENDENCIES_DIR%" >>%CONFIG_LOG% 2>nul ) || call :buildFailLog qscintilla "configuring" && goto wtss-cxx

( nmake clean >>%BUILD_LOG% 2>nul ) || call :buildFailLog  qscintilla "clean" && goto wtss-cxx

( nmake /f Makefile release install >>%BUILD_LOG% 2>nul ) || call :buildFailLog qscintilla "build release" && goto wtss-cxx

( nmake /f Makefile debug >>%BUILD_LOG% 2>nul ) || call :buildFailLog qscintilla "build debug" && goto wtss-cxx

copy debug\qscintilla2d.dll %LIBS_DIR%\qscintilla2d.dll /Y >nul 2>nul

copy debug\qscintilla2d.lib %LIBS_DIR%\qscintilla2d.lib /Y >nul 2>nul

echo done.

call :append_log_end qscintilla

:end_qscintilla

cd %ROOT_DIR%
:: ====

:wtss-cxx

:: wtss-cxx
set WTSS_DIR=%ROOT_DIR%\wtss-cxx
set WTSS_LIBRARY=%TERRALIB_DEPENDENCIES_DIR%\lib\wtss_cxx_client.lib

:: Check dependencies
goto end_wtss_deps
:wtss_deps
IF NOT EXIST %CURL% call :remove_lib wtss-cxx && goto terralib4_deps
IF NOT EXIST %BOOST% call :remove_lib wtss-cxx && goto terralib4_deps
goto terralib4_deps
:end_wtss_deps

echo | set /p="Installing wtss-cxx... "<nul

IF EXIST %LIBS_DIR%\wtss_cxx_client.lib call :skip_build && goto terralib4

call :append_log_begin wtss-cxx

:begin_wtss

cd %WTSS_DIR% >nul 2>nul

del build%_X86% /S /Q >nul 2>nul

mkdir build%_X86% >nul 2>nul

cd build%_X86% >nul 2>nul

:: Runs cmake to create the build project
( cmake -G %_CMAKE_GENERATOR% ^
-DCMAKE_INSTALL_PREFIX=%TERRALIB_DEPENDENCIES_DIR% ^
-DCMAKE_PREFIX_PATH=%TERRALIB_DEPENDENCIES_DIR% ^
%WTSS_DIR%\build\cmake >>%CONFIG_LOG% 2>nul ) || call :buildFailLog wtss-cxx  "configuring" && goto terralib4

:: Build and install both Debug and Release
( %_CMAKE_BUILD_RELEASE% >>%BUILD_LOG% 2>nul ) || call :buildFailLog wtss-cxx "build release" && goto terralib4
( %_CMAKE_INSTALL_RELEASE%  >>%BUILD_LOG% 2>nul ) || call :buildFailLog wtss-cxx "build release" && goto terralib4
( %_CMAKE_BUILD_DEBUG% >>%BUILD_LOG% 2>nul ) || call :buildFailLog wtss-cxx "build debug" && goto terralib4
( %_CMAKE_INSTALL_DEBUG% >>%BUILD_LOG% 2>nul ) || call :buildFailLog wtss-cxx "build debug" && goto terralib4

echo done.

call :append_log_end wtss-cxx

:end_wtss

cd %ROOT_DIR%
:: ====

:terralib4

:: TerraLib version 4.2.2
set TERRALIB4_DIR=%ROOT_DIR%\terralib4
set TERRALIB4_INCLUDE_DIR=%TERRALIB_DEPENDENCIES_DIR%\include
set TERRALIB4_LIBRARY=%LIBS_DIR%\terralib.lib
set TERRALIB4D_LIBRARY=%LIBS_DIR%\terralibd.lib

:: Check dependencies
goto end_terralib4_deps
:terralib4_deps
IF NOT EXIST %TIFF% call :remove_lib terralib && goto begin_build
IF NOT EXIST %GEOTIFF% call :remove_lib terralib && goto begin_build
IF NOT EXIST %ZLIB% call :remove_lib terralib && goto begin_build
IF NOT EXIST %JPEG% call :remove_lib terralib && goto begin_build
goto begin_build
:remove_terralib4
del %LIBS_DIR%\terralib* /Q >nul 2>nul
goto begin_build
:end_terralib4_deps

echo | set /p="Installing terralib4... "<nul

IF EXIST %TERRALIB4% call :skip_build && goto clean_third_directory

call :append_log_begin terralib4

:begin_terralib4

cd %TERRALIB4_DIR% >nul 2>nul

del building%_X86% /S /Q >nul 2>nul

mkdir building%_X86% >nul 2>nul

cd building%_X86% >nul 2>nul

xcopy ..\terralib.conf.cmake . /Y >nul 2>nul

set CL=/MP1

( cmake -G %_CMAKE_GENERATOR% ^
-DCMAKE_INSTALL_PREFIX="%TERRALIB_DEPENDENCIES_DIR%" ^
-DCMAKE_PREFIX_PATH="%TERRALIB_DEPENDENCIES_DIR%" ^
-DTIFF_INCLUDE_DIR:STRING="%TERRALIB_DEPENDENCIES_DIR%/include" ^
-DTIFF_LIBRARY:STRING="debug;%TIFFD_LIBRARY%;optimized;%TIFF_LIBRARY%" ^
-DGeoTIFF_INCLUDE_DIR:STRING="%TERRALIB_DEPENDENCIES_DIR%/include" ^
-DGeoTIFF_LIBRARY:STRING="debug;%GTIFFD_LIBRARY%;optimized;%GTIFF_LIBRARY%" ^
-DJPEG_INCLUDE_DIR="%JPG_DIR%" ^
-DJPEG_LIBRARY:STRING="debug;%JPGD_LIBRARY%;optimized;%JPG_LIBRARY%" ^
-DZLIB_INCLUDE_DIR:STRING="%ZL_INCLUDE_DIR%" ^
-DZLIB_LIBRARIES:STRING="%ZL_LIBRARIES%" ^
-C terralib.conf.cmake ^
%TERRALIB4_DIR%\build\cmake >>%CONFIG_LOG% 2>nul ) || call :buildFailLog terralib4 "configuring" && goto clean_third_directory

:: Build and install both Debug and Release
( %_CMAKE_BUILD_RELEASE% >>%BUILD_LOG% 2>nul ) || call :buildFailLog terralib4 "build release" && goto clean_third_directory
( %_CMAKE_INSTALL_RELEASE%  >>%BUILD_LOG% 2>nul ) || call :buildFailLog terralib4 "build release" && goto clean_third_directory
( %_CMAKE_BUILD_DEBUG% >>%BUILD_LOG% 2>nul ) || call :buildFailLog terralib4 "build debug" && goto clean_third_directory
( %_CMAKE_INSTALL_DEBUG% >>%BUILD_LOG% 2>nul ) || call :buildFailLog terralib4 "build debug" && goto clean_third_directory

echo done.

call :append_log_end terralib4

:end_terralib4

cd %ROOT_DIR%
:: ====

:clean_third_directory

mkdir -p %TERRALIB_DEPENDENCIES_DIR%\bin >nul 2>nul

xcopy bin %TERRALIB_DEPENDENCIES_DIR%\bin /S /Y >nul 2>nul

cd %TERRALIB_DEPENDENCIES_DIR% >nul 2>nul

RD /S /Q doc >nul 2>nul

RD /S /Q features >nul 2>nul

RD /S /Q plugins >nul 2>nul

del lib\*.exe /S /Q >nul 2>nul

del *.pdb /S /Q >nul 2>nul

IF DEFINED FAIL exit /b 1

cd %ROOT_DIR%
::exit /b 0
