/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

 This file is part of the TerraLib - a Framework for building GIS enabled applications.

 TerraLib is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation, either version 3 of the License,
 or (at your option) any later version.

 TerraLib is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with TerraLib. See COPYING. If not, write to
 TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
 \file terralib/itest/raster/ITsDataTilling.cpp

 \brief A integration test suit for the TerraLib VP Module.

 \author Mario Rocco Pettinati
 */

// TerraLib
#include <terralib/common/STLUtils.h>
#include <terralib/geometry/CommonDataStructures.h>
#include <terralib/geometry/Utils.h>
#include <terralib/vp/Feature.h>
#include <terralib/vp/Utils.h>

#include "TestUtils.h"

// GTest and GMock
#include <gmock/gmock.h>
#include <gtest/gtest.h>

//STL include files
#include <thread>

namespace
{
  class ItsDataTilling : public ::testing::Test
  {
    virtual void SetUp()
    {
    }

    virtual void TearDown()
    {
    }
  };

  TEST_F(ItsDataTilling, SequencialTilling)
  {
    te::gtest::DataInfo redDataInfo = te::gtest::GetRedDataInfo();

    te::vp::FeatureSet featureSet;
    te::gtest::GetFeaturesFromShapefile(redDataInfo.m_uri, featureSet);

    ASSERT_FALSE(featureSet.empty());

    te::gm::GeometryVectorConst inputGeometryVector = te::vp::GetGeometriesFromFeatureSet(featureSet);

    te::gtest::SimpleSaveGeometriesToShapefile(te::gtest::GetOutputDir(), redDataInfo.m_datasetName + "_from_sequencial_tilling", inputGeometryVector);

    te::common::FreeContents(featureSet);
  }
}
 
