/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

 This file is part of the TerraLib - a Framework for building GIS enabled applications.

 TerraLib is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation, either version 3 of the License,
 or (at your option) any later version.

 TerraLib is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with TerraLib. See COPYING. If not, write to
 TerraLib Team at <terralib-team@terralib.org>.
 */

#include <memory>
#include <string>
#include <vector>

#include <terralib/core/filesystem/FileSystem.h>
#include <terralib/geometry/CommonDataStructures.h>
#include <terralib/geometry/Enums.h>
#include <terralib/vp/CommonDataStructures.h>


namespace te
{
  namespace da
  {
    class DataSet;
    class DataSetType;
  }

  namespace gm
  {
    class Feature;
  }

  namespace vp
  {
    class FeatureSet;
  }

  namespace gtest
  {
    struct DataToSave
    {
      DataToSave(const std::string& attributeName)
        : m_attributeName(attributeName)
      {

      }

      std::string m_attributeName;
      std::vector<std::string> m_vecValues;
    };

    struct DataInfo
    {
      DataInfo(const std::string& fileName)
      {
        m_uri = "file://" + fileName;
        m_path = te::core::FileSystem::parentPath(fileName);
        m_fileName = fileName;
        m_datasetName = te::core::FileSystem::stem(fileName);
      }

      std::string m_uri;
      std::string m_path;
      std::string m_fileName;
      std::string m_datasetName;
    };

    //!< Gets all the features from a shapefile
    void GetFeaturesFromShapefile(const std::string& uri, te::vp::FeatureSet& vecFeatures);

    //!< Creates an output shapeFile and saves all geometries to it
    void SimpleSaveGeometriesToShapefile(const std::string& outputPath, const std::string& dataSetName, const te::gm::GeometryVectorConst& geometryVector, const std::vector<DataToSave>& dataToAppend = std::vector<DataToSave>());

    //!< Creates an output shapeFile and saves all features to it
    void SimpleSaveGeometriesToShapefile(const std::string& outputPath, te::da::DataSetType* outputDataSetType, const te::vp::FeatureSet& featureSet);

    //!< Creates an Output Shapefile
    void CreateShapeFile(const std::string& uri, te::da::DataSetType* dataSetType);

    //!< Gets the red data info
    te::gtest::DataInfo GetRedDataInfo();

    //!< Gets the blue data info
    te::gtest::DataInfo GetBlueDataInfo();

    //<! Returns the test output dir
    std::string GetOutputDir();

    //!< Reads a wkt from the given filename
    std::string GetWktFromFile(const std::string& fileName);
  }
}



