#!/bin/bash
#
#  Copyright (C) 2008-2014 National Institute For Space Research (INPE) - Brazil.
#
#  This file is part of the TerraLib - a Framework for building GIS enabled applications.
#
#  TerraLib is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation, either version 3 of the License,
#  or (at your option) any later version.
#
#  TerraLib is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with TerraLib. See COPYING. If not, write to
#  TerraLib Team at <terralib-team@terralib.org>.
#
#  Description: Gdal Package for Linux Ubuntu environment.
#
#  Before start, please read the "PACKAGE-INSTRUCTIONS".
#

#
# Set package info:
#
DEBNAME=terralib-gdal
DEBVERSION=3.1.2
DEBARC=amd64
LIBNAME=gdal
FILENAME=gdal-${DEBVERSION}.tar.gz
FOLDERNAME=gdal-${DEBVERSION}
if [ -z "$OUTPUT_PATH" ]; then
  OUTPUT_PATH=`pwd`/../../../3rdparty
fi

#
# Check if the package already exist in the "OUTPUT_PATH":
#
if [ ! -f "${OUTPUT_PATH}/packages/${DEBNAME}_${DEBVERSION}_${DEBARC}.deb" ]; then
#
# Unpack source:
#
if [ ! -d "${FOLDERNAME}" ]; then
    tar xzvf ${FILENAME}
    if [ $? -ne 0 ]; then
        echo "Fail: tar"
        exit 1 
    fi      
fi
#
# Create debian folder:
#
cd ${FOLDERNAME}
rm -rf debian

mkdir -p debian
if [ $? -ne 0 ]; then
    echo "Fail: mkdir"
    exit 1 
fi

#
# Create the copyright file:
#
cat > debian/copyright <<EOF
Format: http://www.debian.org/doc/packaging-manuals/copyright-format/1.0/
Upstream-Name: TerraLib ${LIBNAME} Library
Upstream-Contact: TerraLib Team <terralib-team@terralib.org>
Source: https://gitlab.dpi.inpe.br/terralib/terralib

Files: *
Copyright: Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil
License: LGPL-3.0
  TerraLib is free software, you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation, either version 3 of the License, or 
  (at your option) any later version.

  TerraLib is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU General Public License  
  along with this program. If not, write to TerraLib Team at <terralib-team@terralib.org>.
EOF

#
# Create the changelog (no messages needed):
#

dch --create -v $DEBVERSION --distribution unstable --package ${DEBNAME} ""
if [ $? -ne 0 ]; then
    echo "Fail: dhc"
    exit 1 
fi

#
# Create control file:
#
cat > debian/control <<EOF
Source: ${DEBNAME}
Maintainer: TerraLib Team <terralib-team@terralib.org>
Section: misc
Priority: optional
Standards-Version: 3.9.2
Build-Depends: debhelper (>= 7)

Package: ${DEBNAME}
Architecture: ${DEBARC}
Depends: \${shlibs:Depends}, \${misc:Depends}, ${DEBNAME} (= ${DEBVERSION})
Description: TerraLib ${LIBNAME} Library, version ${DEBVERSION}
Replaces:${DEBNAME} (>= 2.0.1)
EOF

#
# Create rules file:
#

cat > debian/rules <<EOF 
#!/usr/bin/make -f
%:
	dh \$@
override_dh_auto_configure:
	mkdir -p debian/${DEBNAME}/opt/terralib/$TE_VERSION/3rdparty && PATH="$PATH" LD_LIBRARY_PATH="$LD_LIBRARY_PATH:$TERRALIB_DEPENDENCIES_DIR/lib" TERRALIB_DEPENDENCIES_DIR="$TERRALIB_DEPENDENCIES_DIR" PKG_CONFIG_PATH="$TERRALIB_DEPENDENCIES_DIR/pgsql/lib/pkgconfig/:$TERRALIB_DEPENDENCIES_DIR/lib/pkgconfig/" CPPFLAGS="-I$TERRALIB_DEPENDENCIES_DIR/include -I$TERRALIB_DEPENDENCIES_DIR/include/libxml2" LDFLAGS="-L$TERRALIB_DEPENDENCIES_DIR/lib -Wl,-rpath,$TERRALIB_DEPENDENCIES_DIR/pgsql/lib" ./configure --with-pg=yes --with-png=$TERRALIB_DEPENDENCIES_DIR --with-libtiff=$TERRALIB_DEPENDENCIES_DIR --with-geotiff=$TERRALIB_DEPENDENCIES_DIR --with-jpeg=$TERRALIB_DEPENDENCIES_DIR  --with-gif --with-ecw=yes --with-xerces=$TERRALIB_DEPENDENCIES_DIR --with-expat=yes --with-openjpeg-2.0=$TERRALIB_DEPENDENCIES_DIR --with-curl=$TERRALIB_DEPENDENCIES_DIR/bin/curl-config --with-sqlite3=$TERRALIB_DEPENDENCIES_DIR --with-geos=$TERRALIB_DEPENDENCIES_DIR/bin/geos-config --with-threads --with-spatialite=$TERRALIB_DEPENDENCIES_DIR --with-freexl=$TERRALIB_DEPENDENCIES_DIR --without-python --prefix=`pwd`/debian/${DEBNAME}/opt/terralib/$TE_VERSION/3rdparty --with-xml2=yes  --with-libkml --with-hdf4=$TERRALIB_DEPENDENCIES_DIR --without-netcdfd
override_dh_auto_build:
	LD_LIBRARY_PATH="$LD_LIBRARY_PATH:$TERRALIB_DEPENDENCIES_DIR/lib" make -j 8
override_dh_auto_test:
override_dh_auto_clean:
override_dh_auto_install:
	make install
override_dh_shlibdeps:
	dh_shlibdeps --dpkg-shlibdeps-params=--ignore-missing-info
EOF

#PATH="$PATH" CPPFLAGS="$CPPFLAGS -I/usr/include -I/usr/include/libxml2" LDFLAGS="$LDFLAGS -L/usr/lib -Wl,-rpath,/usr/lib" ./configure --prefix=`pwd`/debian/${DEBNAME}/opt/terralib/$TE_VERSION/3rdparty --with-pg --with-png --with-libtiff=internal --with-geotiff=internal --with-jpeg --with-gif --with-ecw=yes --with-xerces=no --with-expat=yes --with-curl --with-sqlite3=/opt/terralib/$TE_VERSION/3rdparty --with-geos --with-threads --with-spatialite --with-freexl --without-python --with-xml2 --with-libkml --with-hdf4 --without-netcdfd
#PATH="$PATH" CPPFLAGS="$CPPFLAGS -I/usr/include -I/usr/include/libxml2" LDFLAGS="$LDFLAGS -L/usr/lib -Wl,-rpath,/usr/lib" PREFIX=`pwd`/debian/${DEBNAME}/opt/terralib/$TE_VERSION/3rdparty make -s
#PATH="$PATH" CPPFLAGS="$CPPFLAGS -I/usr/include -I/usr/include/libxml2" LDFLAGS="$LDFLAGS -L/usr/lib -Wl,-rpath,/usr/lib" PREFIX=`pwd`/debian/${DEBNAME}/opt/terralib/$TE_VERSION/3rdparty make -j install

#
# Create some misc files:
#

echo "8" > debian/compat

mkdir -p debian/source
if [ $? -ne 0 ]; then
    echo "Fail: mkdir"
    exit 1 
fi

echo "3.0 (quilt)" > debian/source/format

#
# Configure Gdal 2:
#
#PATH="$PATH" CPPFLAGS="$CPPFLAGS" LDFLAGS="$LDFLAGS" ./configure --prefix=`pwd`/debian/${DEBNAME}/opt/terralib/$TE_VERSION/3rdparty --with-pg --with-png --with-libtiff=internal --with-geotiff=internal --with-jpeg --with-gif --with-ecw=yes --with-xerces=yes --with-expat=yes --with-curl --with-sqlite3 --with-geos --with-threads --with-spatialite --with-freexl --without-python --with-xml2 --with-libkml --with-hdf4 --without-netcdfd

#
# Build the package:
#
# To add a GnuPG key just uncomment the "-k" and change the following key.

debuild -us -uc -b #-kFBC36213
if [ $? -ne 0 ]; then
    echo "Fail: debuild"
    exit 1 
fi

#Clean debian-package directory:

if [ ! -f "${DEBNAME}_${DEBVERSION}_${DEBARC}.deb" ]; then
  cd ..
  rm -rf ${FOLDERNAME}
  
  mkdir -p ${OUTPUT_PATH}/packages ${OUTPUT_PATH}/build-changes
  if [ $? -ne 0 ]; then
    echo "Fail: mkdir"
    exit 1 
  fi  
  
  mv ${DEBNAME}_${DEBVERSION}_${DEBARC}.deb ${OUTPUT_PATH}/packages
  if [ $? -ne 0 ]; then
      echo "Fail: mv"
      exit 1 
  fi  
  
  mv ${DEBNAME}_${DEBVERSION}_${DEBARC}.changes ${OUTPUT_PATH}/build-changes
  if [ $? -ne 0 ]; then
      echo "Fail: mv"
      exit 1 
  fi  
  
  mv ${DEBNAME}_${DEBVERSION}_${DEBARC}.build ${OUTPUT_PATH}/build-changes
  if [ $? -ne 0 ]; then
      echo "Fail: mv"
      exit 1 
  fi    
fi

fi
