#!/bin/bash

#
#  Copyright (C) 2008-2014 National Institute For Space Research (INPE) - Brazil.
#
#  This file is part of the TerraLib - a Framework for building GIS enabled applications.
#
#  TerraLib is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation, either version 3 of the License,
#  or (at your option) any later version.
#
#  TerraLib is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with TerraLib. See COPYING. If not, write to
#  TerraLib Team at <terralib-team@terralib.org>.
#
#  Description: TerraLib installation package for MacOS environment.
#
#  Before start, please read the "PACKAGE-INSTRUCTIONS".
#

set

USER_DIR=$HOME

CODEBASE=`pwd`/../..

# Checking if the QT Directory if not set.
if [ -z "${QT_DIR}" ]; then
  echo "Missing variable: QT_DIR"
  exit 1
fi

# Checking if default TerraLib 3rdparty directory if not set.
if [ -z "${TE_THIRD_DIR}" ]; then
  echo "Missing variable: TE_THIRD_DIR"
  exit 1
fi

if [ -z "${PYTHON_INCLUDE_DIR}" ]; then
  echo "Missing variable: PYTHON_INCLUDE_DIR"
  exit 1
fi

if [ -z "${PYTHON_LIBRARY}" ]; then
  echo "Missing variable: PYTHON_LIBRARY"
  exit 1
fi

# Defining environment variables in PATH.
export PATH=$PATH:${QT_DIR}/bin:/Applications/CMake.app/Contents/bin

echo "****************************************"
echo "* TerraLib Package Installer for MacOS *"
echo "****************************************"
echo ""
sleep 1s

if [ -z "${THREADS}" ]; then
  echo "Missing variable: THREADS"
  exit 1
fi

# Checking if the default directory for TerraLib build artifacts if not set
if [ -z "${TE_OUT_DIR}" ]; then
  echo "Missing variable: TE_OUT_DIR"
  exit 1
fi

# Checking if the default directory for TerraLib installation if not set
if [ -z "${TE_INSTALL_PATH}" ]; then
  echo "Missing variable: TE_INSTALL_PATH"
  exit 1
fi

# Checking if the default directory for TerraLib Layout installation if not set
if [ -z "${TE_LAYOUT_PATH}" ]; then
  echo "Missing variable: TE_INSTALL_PATH"
  exit 1
fi

export BUILD_TYPE="PACKAGE"

mkdir -p ${TE_OUT_DIR}
cp terralib.conf.cmake ${TE_OUT_DIR}
cd ${TE_OUT_DIR}

cmake --version

# Generating Makefiles
cmake -G "Unix Makefiles" -C ./terralib.conf.cmake -DQT_QCOLLECTIONGENERATOR_EXECUTABLE="$QTDIR/bin/qcollectiongenerator" -DTERRALIB_BUILD_EXAMPLES_ENABLED:BOOL=OFF -DTERRALIB_BUILD_UNITTEST_ENABLED:BOOL=OFF -DTERRALIB_MOD_BINDING_JAVA_ENABLED:BOOL=OFF -DTERRALIB_MOD_BINDING_PYTHON_ENABLED:BOOL=OFF -DTERRALIB_MOD_BINDING_LUA_ENABLED:BOOL=OFF -DTERRALIB_MOD_VM_PYTHON_ENABLED:BOOL=OFF -DTERRALIB_MOD_VM_LUA_ENABLED:BOOL=OFF -DPYTHON_INCLUDE_DIR="$PYTHON_INCLUDE_DIR" -DPYTHON_LIBRARY="$PYTHON_LIBRARY" -DCMAKE_BUILD_TYPE="Release" ${CODEBASE}/build/cmake
_ECODE_CMAKE=$?

# Building TerraLib
make terralib_qtranslations terralib_qhelp package -j $THREADS
_ECODE_MAKE=$?

exit $(( ${_ECODE_CMAKE} | ${_ECODE_MAKE} ))
