#!/bin/bash
#
#  Copyright (C) 2008-2014 National Institute For Space Research (INPE) - Brazil.
#
#  This file is part of the TerraLib - a Framework for building GIS enabled applications.
#
#  TerraLib is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation, either version 3 of the License,
#  or (at your option) any later version.
#
#  TerraLib is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with TerraLib. See COPYING. If not, write to
#  TerraLib Team at <terralib-team@terralib.org>.
#
#  Description: TerraLib installation package for MacOS environment.
#
#  Before start, please read the "PACKAGE-INSTRUCTIONS".
#

USER_DIR=$HOME

CODEBASE=`pwd`/../..

# Defining default QT Directory if not set. (Jenkins will inject a defined-path)
if [ -z "$QT_DIR" ]; then
  export QT_DIR=${USER_DIR}/Qt5.4.1/5.4/clang_64
fi

# Defining default TerraLib 3rdparty directory if not set.
if [ -z "$TE_THIRD_DIR" ]; then
  export TE_THIRD_DIR=${USER_DIR}/mylibs
fi

# Defining default directory for TerraLib Layout installation if not set
if [ -z "$TE_LAYOUT_PATH" ]; then
  export TE_LAYOUT_PATH=${CODEBASE}/terraprint-install
fi

# Defining environment variables in PATH.
export PATH=$PATH:${QT_DIR}/bin:/Applications/CMake.app/Contents/bin

# Building thirdparty script (If the sources are in the same directory of this script)
TERRALIB_DEPENDENCIES_DIR=${TE_THIRD_DIR} ${CODEBASE}/install/install-3rdparty-macos-sierra.sh

clear
echo ""
echo "***********************************"
echo "* TerraLib Installation for MacOS *"
echo "***********************************"
echo ""
sleep 1s

# Defining default directory for TerraLib build artifacts if not set
if [ -z "$TE_OUT_DIR" ]; then
  export TE_OUT_DIR=${CODEBASE}/../build-install
fi

# Defining default directory for TerraLib installation if not set
if [ -z "$TE_INSTALL_PATH" ]; then
  export TE_INSTALL_PATH=${TE_THIRD_DIR}/terralib-install
fi

export BUILD_TYPE="INSTALL"

mkdir -p ${TE_OUT_DIR}
cp terralib.conf.cmake ${TE_OUT_DIR}
cd ${TE_OUT_DIR}

cmake --version

# Generating Makefiles
cmake -G "Unix Makefiles" -C ./terralib.conf.cmake -DCMAKE_BUILD_TYPE="Release" ${CODEBASE}/build/cmake
_ECODE_CMAKE=$?

# Building TerraLib
make install -j 8
_ECODE_MAKE=$?

exit $(( ${_ECODE_CMAKE} | ${_ECODE_MAKE} ))