#!/bin/bash
#
#  Copyright (C) 2008-2014 National Institute For Space Research (INPE) - Brazil.
#
#  This file is part of the TerraLib - a Framework for building GIS enabled applications.
#
#  TerraLib is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation, either version 3 of the License,
#  or (at your option) any later version.
#
#  TerraLib is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with TerraLib. See COPYING. If not, write to
#  TerraLib Team at <terralib-team@terralib.org>.
#
#  Description: TerraPrint installation for MacOS environment.
#

USER_DIR=$HOME
CODEBASE=`pwd`/../../../terraprint-codebase

# Defining default QT Directory if not set.
if [ -z "$QT_DIR" ]; then
  export QT_DIR=${USER_DIR}/Qt5.4.1/5.4/clang_64
fi

# Defining default TerraLib 3rdparty directory if not set.
if [ -z "$TE_THIRD_DIR" ]; then
  export TE_THIRD_DIR=${USER_DIR}/mylibs
fi

# Defining the default branch for terraprint
if [ -z "$TERRAPRINT_BRANCH" ]; then
  export TERRAPRINT_BRANCH=release-1.6
fi

# Defining environment variables in PATH.
export PATH=$PATH:${QT_DIR}/bin:/Applications/CMake.app/Contents/bin

clear
echo ""
echo "*************************************"
echo "* TerraPrint Installation for MacOS *"
echo "*************************************"
echo ""
sleep 1s

rm -rf ${CODEBASE}
mkdir -p ${CODEBASE}
cd ${CODEBASE}
git clone --branch ${TERRAPRINT_BRANCH} https://bdc-gitlab.dpi.inpe.br/terralib/terraprint.git .

# Defining default directory for build artifacts if not set
if [ -z "$TP_OUT_DIR" ]; then
  export TP_OUT_DIR=${CODEBASE}/../build-install-terraprint
fi

# Defining default directory for installation if not set
if [ -z "$TP_INSTALL_PATH" ]; then
  export TP_INSTALL_PATH=${TE_THIRD_DIR}/terraprint-install
fi

mkdir -p ${TP_OUT_DIR}
cd ${TP_OUT_DIR}

cmake --version

# Generating Makefiles
cmake -G "Unix Makefiles" -DCMAKE_BUILD_TYPE="Release" \
                          -DCMAKE_INSTALL_PREFIX="${TP_INSTALL_PATH}" \
                          -DCMAKE_PREFIX_PATH="${TE_THIRD_DIR};${TE_INSTALL_PATH}" \
                          -DTERRALIB_BUILD_EXAMPLES_ENABLED:BOOL=OFF \
                          -DTERRALIB_BUILD_LAYOUT_TEST_ENABLED:BOOL=OFF \
                          ${CODEBASE}/build/cmake
_ECODE_CMAKE=$?

# Building TerraPrint
make install -j 8
_ECODE_MAKE=$?

exit $(( ${_ECODE_CMAKE} | ${_ECODE_MAKE} ))
