#
#  Copyright (C) 2008-2014 National Institute For Space Research (INPE) - Brazil.
#
#  This file is part of the TerraLib - a Framework for building GIS enabled applications.
#
#  TerraLib is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation, either version 3 of the License,
#  or (at your option) any later version.
#
#  TerraLib is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with TerraLib. See COPYING. If not, write to
#  TerraLib Team at <terralib-team@terralib.org>.
#
#  Description: TerraLib CMake cache options for building TerraLib on MacOS environments.
#
#  Before start, please read the "PACKAGE-INSTRUCTIONS".
#

# Define CMake variables:
set (CMAKE_INSTALL_PREFIX "$ENV{TE_INSTALL_PATH}" CACHE PATH "Where to install TerraLib?" FORCE)
set (CMAKE_PREFIX_PATH "$ENV{TE_THIRD_DIR};$ENV{TE_LAYOUT_PATH}" CACHE PATH "Where are the dependencies of TerraLib?" FORCE)
set (CMAKE_INCLUDE_PATH "$ENV{TE_THIRD_DIR}/pgsql/include" CACHE PATH "PGSQL include dir" FORCE)
set (CMAKE_LIBRARY_PATH "$ENV{TE_THIRD_DIR}/pgsql/lib" CACHE PATH "PGSQL lib dir" FORCE)

# Set 3rdparty paths
set (Qt5_DIR "$ENV{QT_DIR}/lib/cmake/Qt5" CACHE PATH "Where to find file Qt5Config.cmake?" FORCE)
set (QT_QCOLLECTIONGENERATOR_EXECUTABLE "${Qt5_DIR}/../../../bin/qcollectiongenerator" CACHE FILEPATH "Where to find file qt collection generator executable?" FORCE)
set (SWIG_EXECUTABLE "$ENV{TE_THIRD_DIR}/bin/swig" CACHE FILEPATH "Where to find swig.exe?" FORCE)
set (Qt5LinguistTools_DIR ${Qt5_DIR}/../Qt5LinguistTools CACHE PATH "Where to find qt linguist tools?" FORCE)
set (GDAL_DATA_DIR $ENV{TE_THIRD_DIR}/share/gdal CACHE PATH "GDAL data dir" FORCE)

# Set the package type to be generated:
set (CPACK_BINARY_DRAGNDROP ON CACHE BOOL "Use DMG?" FORCE)
set (CPACK_BINARY_7Z OFF CACHE BOOL "Use 7zip for package?" FORCE)
set (CPACK_BINARY_IFW OFF CACHE BOOL "Use IFW for package?" FORCE)
set (CPACK_BINARY_NSIS OFF CACHE BOOL "Use NSIS for package?" FORCE)
set (CPACK_BINARY_WIX OFF CACHE BOOL "Use Wix for package?" FORCE)
set (CPACK_BINARY_ZIP OFF CACHE BOOL "Use Zip for package?" FORCE)
set (CPACK_BINARY_STGZ OFF CACHE BOOL "Use STGZ?" FORCE)
set (CPACK_BINARY_TGZ OFF CACHE BOOL "Use TGZ?" FORCE)
set (CPACK_SOURCE_TBZ2 OFF CACHE BOOL "Use TBZ2?" FORCE)
set (CPACK_SOURCE_TXZ OFF CACHE BOOL "Use TXZ?" FORCE)
set (CPACK_SOURCE_TZ OFF CACHE BOOL "Use TZ?" FORCE)
set (CPACK_SOURCE_RPM OFF CACHE BOOL "Use RPM?" FORCE)


# Turn off TerraLib CMake variables:
set (TERRALIB_BUILD_EXAMPLES_ENABLED OFF CACHE BOOL "Build the examples?" FORCE)
set (TERRALIB_BUILD_UNITTEST_ENABLED OFF CACHE BOOL "Build the unit tests?" FORCE)
set (TERRALIB_DOXYGEN_ENABLED OFF CACHE BOOL "Generate API documentation?" FORCE)
set (TERRALIB_MOD_BINDING_JAVA_ENABLED OFF CACHE BOOL "Build Java binding?" FORCE)

# Turn on TerraLib CMake variables:
set (TERRALIB_MOD_BINDING_PYTHON_ENABLED ON CACHE BOOL "Build Python binding?" FORCE)
set (TERRALIB_MOD_BINDING_LUA_ENABLED ON CACHE BOOL "Build Lua binding?" FORCE)
set (TERRALIB_MOD_EDIT_CORE_ENABLED ON CACHE BOOL "Build Edit?" FORCE)
set (TERRALIB_MOD_QT_PLUGINS_WTSS_ENABLED ON CACHE BOOL "Build Wtss-Cxx?" FORCE)
set (TERRALIB_MOD_CEDITOR_ENABLED ON CACHE BOOL "Build CEdiror plugin?" FORCE)
set (TERRALIB_MOD_VM_ENABLED ON CACHE BOOL "Build VM Core?" FORCE)
set (TERRALIB_MOD_VM_PYTHON_ENABLED ON CACHE BOOL "Build Python VM?" FORCE)
set (TERRALIB_MOD_VM_LUA_ENABLED ON CACHE BOOL "Build Lua VM?" FORCE)

if($ENV{BUILD_TYPE} MATCHES "PACKAGE")
  set (TERRALIB_TRACK_3RDPARTY_DEPENDENCIES ON CACHE BOOL "Track the 3rd-parties on instalation?" FORCE)
  set (TERRALIB_BUILD_AS_BUNDLE ON CACHE BOOL "Bundle?" FORCE)
elseif($ENV{BUILD_TYPE} MATCHES "INSTALL")
  set (TERRALIB_TRACK_3RDPARTY_DEPENDENCIES OFF CACHE BOOL "Track the 3rd-parties on instalation?" FORCE)
  set (TERRALIB_BUILD_AS_BUNDLE OFF CACHE BOOL "Bundle?" FORCE)
endif()