:: Turn off system messages
:: @echo off
:: Clear screen
cls;
:: Welcome message
echo.
echo ********************************************
echo * GeoDMA Installer for Windows 64-bits *
echo ********************************************
echo.
:: ====

IF NOT DEFINED _CMAKE_GENERATOR (
  echo Error : Undefined variable _CMAKE_GENERATOR
  exit 1
)

IF NOT DEFINED WORKSPACE (
  echo Error : Undefined variable WORKSPACE
  exit 1
)

IF NOT DEFINED TERRALIB_DEPENDENCIES_DIR (
  echo Error : Undefined variable TERRALIB_DEPENDENCIES_DIR
  exit 1
)

IF NOT DEFINED TERRALIB_INSTALL_DIR (
  echo Error : Undefined variable TERRALIB_INSTALL_DIR
  exit 1
)

IF NOT DEFINED TERRALIB_BUILD_PATH (
  echo Error : Undefined variable TERRALIB_BUILD_PATH
  exit 1
)

IF NOT DEFINED TERRALIB_CODEBASE_PATH (
  echo Error : Undefined variable TERRALIB_CODEBASE_PATH
  exit 1
)

IF NOT DEFINED GEODMA_CODEBASE_PATH (
  echo Error : Undefined variable GEODMA_CODEBASE_PATH
  exit 1
)

IF NOT DEFINED GEODMA_BUILD_PATH (
  echo Error : Undefined variable GEODMA_BUILD_PATH
  exit 1
)

IF NOT DEFINED GEODMA_BRANCH (
  echo Error : Undefined variable GEODMA_BRANCH
  exit 1
)

IF NOT DEFINED GEODMA_REPO_URL (
  echo Error : Undefined variable GEODMA_REPO_URL
  exit 1
)

:: Where is Dependencies
IF NOT DEFINED QMAKE_FILEPATH (
  echo Error : Undefined variable QMAKE_FILEPATH
  exit 1
)

IF NOT DEFINED VCVARS_FILEPATH (
  echo Error : Undefined variable VCVARS_FILEPATH
  exit 1
)

:: Where is cmake.exe
IF NOT DEFINED CMAKE_FILEPATH (
  echo Error : Undefined variable CMAKE_FILEPATH
  exit 1
)

IF NOT DEFINED CGAL_DIR (
  echo Error : Undefined variable CGAL_DIR
  exit 1
)

IF NOT DEFINED QTDIR (
  echo Error : Undefined variable QTDIR
  exit 1
)


IF NOT DEFINED C50_BUILD_PATH (
  echo Error : Undefined variable C50_BUILD_PATH
  exit 1
)

IF NOT DEFINED C50_INSTALL_PATH (
  echo Error : Undefined variable C50_INSTALL_PATH
  exit 1
)

:: Environment Paths
set "PATH=%CMAKE_FILEPATH%;%QMAKE_FILEPATH%;%PATH%"

::  Cloning GeoDMA source
::  ==========================

IF NOT EXIST %GEODMA_CODEBASE_PATH% (
  echo Cloning GeoDMA source

  mkdir %GEODMA_CODEBASE_PATH% >nul 2>nul
  if %errorlevel% neq 0 exit /b %errorlevel%

  cd %GEODMA_CODEBASE_PATH%
  if %errorlevel% neq 0 exit /b %errorlevel%

  git -c http.sslVerify=false clone --branch %GEODMA_BRANCH% %GEODMA_REPO_URL% .
  if %errorlevel% neq 0 exit /b %errorlevel%

  echo done.
  echo.
)

::  Setting visual studio environment
::  ==================================

echo | set /p="Configuring visual studio... "<nul

call "%VCVARS_FILEPATH%"\vcvarsall.bat x64
if %errorlevel% neq 0 exit /b %errorlevel%

echo done.
echo.

::  Building C50
::  =======================================

IF NOT EXIST %C50_BUILD_PATH% (
  mkdir %C50_BUILD_PATH% >nul 2>nul
  if %errorlevel% neq 0 exit /b %errorlevel%

  cd /d %C50_BUILD_PATH% >nul 2>nul
  if %errorlevel% neq 0 exit /b %errorlevel%

  echo | set /p="Configuring C50 CMake... "<nul

  cmake -G %_CMAKE_GENERATOR% ^
  -DCMAKE_BUILD_TYPE="Release" ^
  -DCMAKE_VERBOSE_MAKEFILE:BOOL="ON" ^
  -DCMAKE_INSTALL_PREFIX="%C50_INSTALL_PATH%" ^
  %GEODMA_CODEBASE_PATH%\3rdparty\c50\build\cmake
  if %errorlevel% neq 0 exit /b %errorlevel%

  echo | set /p="Building C50 CMake... "<nul

  cmake --build . --target install --config Release
  if %errorlevel% neq 0 exit /b %errorlevel%

  echo done.
  echo.
)

::  Building GeoDMA
::  =======================================

IF NOT EXIST %GEODMA_BUILD_PATH% (
  mkdir %GEODMA_BUILD_PATH% >nul 2>nul
  if %errorlevel% neq 0 exit /b %errorlevel%

  cd /d %GEODMA_BUILD_PATH% >nul 2>nul
  if %errorlevel% neq 0 exit /b %errorlevel%

  echo | set /p="Configuring GeoDMA CMake... "<nul

  cmake -G "Visual Studio 15 2017 Win64" ^
  -DCMAKE_BUILD_TYPE="Release" ^
  -DCMAKE_PREFIX_PATH="%TERRALIB_DEPENDENCIES_DIR%\include;%TERRALIB_DEPENDENCIES_DIR%\lib;%TERRALIB_INSTALL_DIR%" ^
  -DC50_LIBRARY:PATH="%C50_INSTALL_PATH%\lib\static\c50lib.lib" ^
  -DC50_INCLUDE_DIR:PATH="%C50_INSTALL_PATH%\include" ^
  -DCMAKE_VERBOSE_MAKEFILE:BOOL="ON" ^
  -DGEODMA_SOURCE_VERIFY_ENABLED:BOOL="OFF" ^
  -DBoost_DIR:PATH="%TERRALIB_DEPENDENCIES_DIR%" ^
  -DBoost_INCLUDE_DIR:PATH="%TERRALIB_DEPENDENCIES_DIR%\include" ^
  -DCGAL_DIR:PATH="%CGAL_DIR%" ^
  -DCGAL_Boost_USE_STATIC_LIBS:BOOL="ON" ^
  -DQt5_DIR="%QTDIR%\lib\cmake\Qt5" ^
  %GEODMA_CODEBASE_PATH%\build\cmake
  if %errorlevel% neq 0 exit /b %errorlevel%

  cmake --build . --config Release --target geodma_qhelp
  if %errorlevel% neq 0 exit /b %errorlevel%

  cmake --build . --config Release
  if %errorlevel% neq 0 exit /b %errorlevel%

  echo done.
  echo.
)

::  Installing GeoDMA
::  =======================================

xcopy "%GEODMA_BUILD_PATH%\Release\*.lib" "%TERRALIB_BUILD_PATH%\Release" /f /i /e /y
xcopy "%GEODMA_BUILD_PATH%\Release\*.lib" "%TERRALIB_INSTALL_DIR%\lib" /f /i /e /y

xcopy "%GEODMA_BUILD_PATH%\Release\*.dll" "%TERRALIB_BUILD_PATH%\Release" /f /i /e /y
xcopy "%GEODMA_BUILD_PATH%\Release\*.dll" "%TERRALIB_INSTALL_DIR%\bin" /f /i /e /y

xcopy "%GEODMA_CODEBASE_PATH%\share\plugin\*" "%TERRALIB_CODEBASE_PATH%\share\terralib\plugins" /f /i /e /y
xcopy "%GEODMA_CODEBASE_PATH%\share\plugin\*" "%TERRALIB_INSTALL_DIR%\share\terralib\plugins" /f /i /e /y

xcopy "%GEODMA_CODEBASE_PATH%\share\*" "%TERRALIB_CODEBASE_PATH%\share" /f /i /e /y
xcopy "%GEODMA_CODEBASE_PATH%\share\*" "%TERRALIB_INSTALL_DIR%\share" /f /i /e /y

:: ====

echo.
echo.
echo ******************************************************
echo * GeoDMA Installer for Windows 64-bits finished! *
echo ******************************************************
echo Finished!
echo.

