::
:: Before start, please read the "PACKAGE-INSTRUCTIONS"
:: file located in packages/windows folder.
::

:: Turn off system messages
@echo off
:: Clear screen
cls;
:: Welcome message
echo.
echo **************************************************
echo * TerraLib Package Installer for Windows 64-bits *
echo **************************************************
echo.
:: ====

:: Where is script file
set "ROOT_DIR=%CD%"

:: TerraLib Paths
IF NOT DEFINED TERRALIB_DEPENDENCIES_DIR (
  set "TERRALIB_DEPENDENCIES_DIR=C:\MyDevel\terralib\3rdparty\terralib5-3rdparty-msvc-2017-win64"
)

IF NOT DEFINED TERRALIB_CODEBASE_PATH (
  set "TERRALIB_CODEBASE_PATH=C:\tview"
)

IF NOT DEFINED TERRALIB_BUILD_PATH (
  set "TERRALIB_BUILD_PATH=%TERRALIB_CODEBASE_PATH%\solution\build"
)

IF NOT DEFINED TERRALIB_INSTALL_PATH (
  set "TERRALIB_INSTALL_PATH=%TERRALIB_CODEBASE_PATH%\solution\install"
)

:: Where is qmake.exe
IF NOT DEFINED QMAKE_FILEPATH (
  set "QMAKE_FILEPATH=C:\Qt\Qt5.12.3\5.12.3\msvc2017_64\bin"
)
:: Where is vcvars
set "VCVARS_FILEPATH=%PROGRAMFILES(x86)%\Microsoft Visual Studio\2017\Community\VC\Auxiliary\Build"

:: Where is cmake.exe
IF NOT DEFINED CMAKE_FILEPATH (
  set "CMAKE_FILEPATH=C:\Program Files\CMake\bin"
)

:: Environment Paths
set "PATH=%CMAKE_FILEPATH%;%QMAKE_FILEPATH%;%PATH%"
:: Build Type
set "BUILD_TYPE=PACKAGE"

::  Setting visual studio environment
::  ==================================

set _CMAKE_GENERATOR="Visual Studio 15 2017 Win64"

echo | set /p="Configuring visual studio... "<nul

call "%VCVARS_FILEPATH%"\vcvarsall.bat x64

echo done.
echo.

::  Creating and configuring build folder
::  ======================================

echo | set /p="Configuring build folder... "<nul

rmdir %TERRALIB_BUILD_PATH% /s /q
mkdir %TERRALIB_BUILD_PATH% >nul 2>nul

xcopy %TERRALIB_CODEBASE_PATH%\packages\windows\terralib.conf.cmake %TERRALIB_BUILD_PATH% /S /Y >nul 2>nul
if %errorlevel% neq 0 ( echo "xcopy check failed" & exit /b 1 )
echo "xcopy ok"

echo done.
echo.

cd /d %TERRALIB_BUILD_PATH% >nul 2>nul

::  Configuring CMake and generating files
::  =======================================

echo | set /p="Configuring CMake... "<nul

cmake -G %_CMAKE_GENERATOR% -C ./terralib.conf.cmake -DCMAKE_BUILD_TYPE="Release" %TERRALIB_CODEBASE_PATH%\build\cmake
if %errorlevel% neq 0 ( echo "Cmake failed" & exit /b 1 )
echo "Cmake ok"

echo done.
echo.

::  Building VS2017 all Project
::  ============================

echo | set /p="Building project... "<nul

msbuild /m ALL_BUILD.vcxproj /p:Configuration=Release
if %errorlevel% neq 0 ( echo "Build ALL failed" & exit /b 1 )
echo "Build ALL ok"

echo done.
echo.

::  Installing TerraLib in dependencies dir
::  ========================================

echo | set /p="Installing... "<nul

msbuild /m INSTALL.vcxproj /p:Configuration=Release
if %errorlevel% neq 0 ( echo "Install failed" & exit /b 1 )
echo "Install ok"

echo done.
echo.

::  Build and Installing TerraPrint in dependencies dir
::  ========================================

IF DEFINED TERRALIB_WITH_TERRAPRINT (

  echo "Building and Installing TerraPrint..."

  cd /d "%TERRALIB_CODEBASE_PATH%\packages\windows" >nul 2>nul
  
  call install-terraprint.bat
  
  echo "Updating Terralib CMake project files..."
  
  cd /d %TERRALIB_BUILD_PATH% 
  
  cmake -G %_CMAKE_GENERATOR% -C ./terralib.conf.cmake -DCMAKE_BUILD_TYPE="Release" %TERRALIB_CODEBASE_PATH%\build\cmake
  if %errorlevel% neq 0 ( echo "Cmake failed" & exit /b 1 )
  echo "Cmake ok"
  
  echo done.
  echo.
)

::  Build and Installing GeoDMA
::  ========================================

IF DEFINED TERRALIB_WITH_GEODMA (
  call "%TERRALIB_CODEBASE_PATH%\packages\windows\install-geodma.bat"

  cd /d %TERRALIB_BUILD_PATH%
  cmake -G %_CMAKE_GENERATOR% -C ./terralib.conf.cmake -DCMAKE_BUILD_TYPE="Release" %TERRALIB_CODEBASE_PATH%\build\cmake
  if %errorlevel% neq 0 ( echo "Cmake failed" & exit /b 1 )
  echo "Cmake ok"
)

::  Generating TerraLib NSIS package
::  =================================

echo | set /p="Packaging... "<nul

msbuild /m PACKAGE.vcxproj /p:Configuration=Release
if %errorlevel% neq 0 ( echo "Package failed" & exit /b 1 )
echo "Package ok"

echo done.
echo.

:: ====

echo.
echo.
echo ************************************************************
echo * TerraLib Package Installer for Windows 64-bits finished! *
echo ************************************************************
echo Finished!
echo.

exit /b 0
