
::
::  Copyright (C) 2008-2014 National Institute For Space Research (INPE) - Brazil.
::
::  This file is part of the TerraLib - a Framework for building GIS enabled applications.
::
::  TerraLib is free software: you can redistribute it and/or modify
::  it under the terms of the GNU Lesser General Public License as published by
::  the Free Software Foundation, either version 3 of the License,
::  or (at your option) any later version.
::
::  TerraLib is distributed in the hope that it will be useful,
::  but WITHOUT ANY WARRANTY; without even the implied warranty of
::  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
::  GNU Lesser General Public License for more details.
::
::  You should have received a copy of the GNU Lesser General Public License
::  along with TerraLib. See COPYING. If not, write to
::  TerraLib Team at <terralib-team@terralib.org>.
::
::  Description: TerraLib installation package for Windows 10 environment.
::
::  Before start, please read the "PACKAGE-INSTRUCTIONS".
::

:: Turn off system messages
@echo off
:: Clear screen
cls;
:: Welcome message
echo.
echo **************************************************
echo * TerraLib Package Installer for Windows 64-bits *
echo **************************************************
echo.
:: ====

set "WORKSPACE=%CD%\..\.."
:: TerraLib Paths
IF NOT DEFINED TERRALIB_DEPENDENCIES_DIR (
  set "TERRALIB_DEPENDENCIES_DIR=C:\MyDevel\terralib\3rdparty\release-5.3"
)

IF NOT DEFINED TERRALIB_CODEBASE_PATH (
  set "TERRALIB_CODEBASE_PATH=%WORKSPACE%"
)

IF NOT DEFINED TERRALIB_BUILD_PATH (
  set "TERRALIB_BUILD_PATH=%WORKSPACE%\build-terralib"
)

IF NOT DEFINED TERRALIB_INSTALL_PATH (
  set "TERRALIB_INSTALL_PATH=%WORKSPACE%\terralib-install"
)

IF NOT DEFINED TERRAPRINT_DIR (
  set "TERRAPRINT_DIR=%WORKSPACE%\terraprint-install"
)

:: Where is qmake.exe
IF NOT DEFINED QMAKE_FILEPATH (
  set "QMAKE_FILEPATH=C:\Qt5.4.1\5.4\msvc2013_64\bin"
)
:: Where is Visual Studio
set "VCVARS_FILEPATH=%PROGRAMFILES(x86)%\Microsoft Visual Studio 12.0\VC"
:: Where is cmake.exe
set "CMAKE_FILEPATH=%PROGRAMFILES(x86)%\CMake\bin"
:: Where is script file
set "ROOT_DIR=%CD%"
:: Environment Paths
set "PATH=%CMAKE_FILEPATH%;%QMAKE_FILEPATH%;%PATH%"

::  Setting visual studio environment
::  ==================================

set _CMAKE_GENERATOR="Visual Studio 12 2013 Win64"

echo | set /p="Configuring visual studio... "<nul

call "%VCVARS_FILEPATH%"\vcvarsall.bat %_config%

echo done.
echo.

::  Creating and configuring build folder
::  ======================================

echo | set /p="Configuring build folder... "<nul

rmdir %TERRALIB_BUILD_PATH% /s /q
mkdir %TERRALIB_BUILD_PATH% >nul 2>nul

xcopy %WORKSPACE%\packages\windows\terralib.conf.cmake %TERRALIB_BUILD_PATH% /S /Y >nul 2>nul

echo done.
echo.

cd /d %TERRALIB_BUILD_PATH% >nul 2>nul

::  Configuring CMake and generating files
::  =======================================

echo | set /p="Configuring CMake... "<nul

cmake -G %_CMAKE_GENERATOR% -C ./terralib.conf.cmake -DCMAKE_BUILD_TYPE="Release" %TERRALIB_CODEBASE_PATH%\build\cmake

echo done.
echo.

::  Building VS2013 all Project
::  ============================

echo | set /p="Building project... "<nul

msbuild /m terralib.sln /p:Configuration=Release /maxcpucount:4

msbuild /m terralib_qhelp\terralib_qhelp.vcxproj /p:Configuration=Release /maxcpucount:4

msbuild /m terralib_qtranslations\terralib_qtranslations.vcxproj /p:Configuration=Release /maxcpucount:4

echo done.
echo.

::  Installing TerraLib in dependencies dir
::  ========================================

echo | set /p="Installing... "<nul

msbuild /m INSTALL.vcxproj /p:Configuration=Release

echo done.
echo.

::  Generating TerraLib NSIS package
::  =================================

echo | set /p="Packaging... "<nul

msbuild /m PACKAGE.vcxproj /p:Configuration=Release
set _BUILD_STATUS=%errorlevel%

echo done.
echo.

:: ====

echo.
echo.
echo ************************************************************
echo * TerraLib Package Installer for Windows 64-bits finished! *
echo ************************************************************
echo Finished!
echo.

exit /b %_BUILD_STATUS%