/*
 * File: Geometry.i
 *
 */

#define TEGEOMEXPORT

%import "terralib/common/BaseVisitable.h"

%ignore accept;

#ifdef SWIGLUA
%ignore te::gm::PerpendicularDistance;
%ignore te::gm::Validation::MakeValid(const te::gm::Geometry*);
%ignore te::gm::Validation::MakeValid(const te::gm::Geometry*, std::string&);
%ignore te::gm::Validation::SnapToSelf(const te::gm::Geometry* geometry);
%ignore te::gm::Validation::SnapToSelf(const te::gm::Geometry* geometry, double, bool);
%ignore te::gm::Validation::MakeValidWithSnap(const te::gm::Geometry*);
%ignore te::gm::CreatePolygon(te::gm::Curve const *);
%ignore te::gm::CreateCollectionFromGeometryVector(te::gm::GeometryVector const &);
#endif

UNIQUE_PTR_TYPEMAPS(te::gm::Point)
UNIQUE_PTR_TYPEMAPS(te::gm::Geometry)

%{
#include "terralib/geometry/Config.h"
#include "terralib/geometry/Envelope.h"
#include "terralib/geometry/Enums.h"
#include "terralib/geometry/CircularString.h"
#include "terralib/geometry/CompoundCurve.h"
#include "terralib/geometry/Coord2D.h"
#include "terralib/geometry/Curve.h"
#include "terralib/geometry/CurvePolygon.h"
#include "terralib/geometry/Geometry.h"
#include "terralib/geometry/Line.h"
#include "terralib/geometry/LinearRing.h"
#include "terralib/geometry/LineString.h"
#include "terralib/geometry/MultiCurve.h"
#include "terralib/geometry/MultiLineString.h"
#include "terralib/geometry/MultiPoint.h"
#include "terralib/geometry/MultiPolygon.h"
#include "terralib/geometry/MultiSurface.h"
#include "terralib/geometry/Point.h"
#include "terralib/geometry/Polygon.h"
#include "terralib/geometry/PolyhedralSurface.h"
#include "terralib/geometry/Surface.h"
#include "terralib/geometry/GeometryProperty.h"
#include "terralib/geometry/TIN.h"
#include "terralib/geometry/Triangle.h"
#include "terralib/geometry/Utils.h"
#include "terralib/geometry/Validation.h"
%}

%{
  namespace te
  {
    namespace gm
	{
      te::gm::Point ClosestPointInGeometryFromPoint(te::gm::Point* point, Geometry* geom)
      {
        te::gm::Coord2D coordGeom, coordPoint;
	    te::gm::ClosestPoints(geom, point, coordGeom, coordPoint);
	    return te::gm::Point(coordGeom.getX(), coordGeom.getY(), point->getSRID());
      }
	}
  }
%}

%nspace te::gm::GeomType;
%nspace te::gm::SpatialRelation;
%nspace te::gm::Dimensionality;
%nspace te::gm::BufferCapStyle;
%nspace te::gm::Geometry;
%nspace te::gm::Coord2D;
%nspace te::gm::Point;
%nspace te::gm::Curve;
%nspace te::gm::CircularString;
%nspace te::gm::CompoundCurve;
%nspace te::gm::Envelope;
%nspace te::gm::LineString;
%nspace te::gm::Line;
%nspace te::gm::LinearRing;
%nspace te::gm::Surface;
%nspace te::gm::CurvePolygon;
%nspace te::gm::GeometryCollection;
%nspace te::gm::MultiCurve;
%nspace te::gm::MultiLineString;
%nspace te::gm::MultiPoint;
%nspace te::gm::MultiSurface;
%nspace te::gm::MultiPolygon;
%nspace te::gm::Polygon;
%nspace te::gm::PolyhedralSurface;
%nspace te::gm::GeometryProperty;

AUTO_PTR_TYPEMAPS(te::gm::Point)
UNIQUE_PTR_TYPEMAPS(te::gm::Point)

%include "terralib/geometry/Enums.h"
%include "terralib/geometry/Geometry.h"
%include "terralib/geometry/Coord2D.h"
%include "terralib/geometry/Point.h"
%include "terralib/geometry/Envelope.h"
%include "terralib/geometry/Curve.h"
%include "terralib/geometry/CircularString.h"
%include "terralib/geometry/CompoundCurve.h"
%include "terralib/geometry/LineString.h"
%include "terralib/geometry/Line.h"
%include "terralib/geometry/LinearRing.h"
%include "terralib/geometry/Surface.h"
%include "terralib/geometry/CurvePolygon.h"
%include "terralib/geometry/GeometryCollection.h"
%include "terralib/geometry/MultiCurve.h"
%include "terralib/geometry/MultiLineString.h"
%include "terralib/geometry/MultiPoint.h"
%include "terralib/geometry/MultiSurface.h"
%include "terralib/geometry/MultiPolygon.h"
%include "terralib/geometry/Polygon.h"
%include "terralib/geometry/PolyhedralSurface.h"
%include "terralib/geometry/GeometryProperty.h"
%include "terralib/geometry/Utils.h"
%include "terralib/geometry/Validation.h"
