#define TERPEXPORT

%{

// TerraLib
#include <terralib/raster.h>
#include <terralib/dataaccess.h>
#include <terralib/geometry.h>
#include <terralib/memory.h>
#include <terralib/rp.h>

// STL
#include <map>
#include <string>
#include <vector>

void SegmenterRegionGrowing(std::string inputFilePath, std::string outputFilePath, int minSegmentSize, double segmentsSimilarityThreshold)
{
  try
  {
    std::map<std::string, std::string> rinfo;
    rinfo["URI"] = inputFilePath;
  
    te::rst::Raster* rin = te::rst::RasterFactory::open(rinfo);

    std::map<std::string, std::string> orinfo;
    orinfo["URI"] = outputFilePath;
   
    te::rp::Segmenter::InputParameters algoInputParameters;
    algoInputParameters.m_inputRasterPtr = rin;
    algoInputParameters.m_inputRasterBands.push_back( 0 );
    algoInputParameters.m_inputRasterBands.push_back( 1 );
    algoInputParameters.m_inputRasterBands.push_back( 2 );
    algoInputParameters.m_enableBlockProcessing = false;
  
  
    te::rp::SegmenterRegionGrowingMeanStrategy::Parameters segparameters;
    segparameters.m_minSegmentSize = minSegmentSize;
    segparameters.m_segmentsSimilarityThreshold = segmentsSimilarityThreshold;

    algoInputParameters.m_strategyName = "RegionGrowingMean";
    algoInputParameters.setSegStrategyParams(segparameters);
  
    te::rp::Segmenter::OutputParameters algoOutputParameters;
    algoOutputParameters.m_rInfo = orinfo;
    algoOutputParameters.m_rType = "GDAL";
  
    te::rp::Segmenter seginstance;
  

    seginstance.initialize(algoInputParameters);
    seginstance.execute(algoOutputParameters);
  }
  catch(std::exception e)
  {
  }
}

void ArithmeticOp(const char* arithmeticString, const char* outPath, std::vector<te::rst::Raster*> inputRasters )
{
  try
  {
    te::rp::ArithmeticOperations::InputParameters inputParams;
    inputParams.m_arithmeticString = arithmeticString;
    inputParams.m_normalize = false;
    inputParams.m_inputRasters = inputRasters;
    
// create output raster info
    std::map<std::string, std::string> orinfo;
    orinfo["URI"] = outPath;

// defining output parameters
    te::rp::ArithmeticOperations::OutputParameters outputParams;
    outputParams.m_rInfo = orinfo;
    outputParams.m_rType = "GDAL";

    te::rp::ArithmeticOperations algorithmInstance;
    
    if(!algorithmInstance.initialize(inputParams))
      throw;
      
    if(!algorithmInstance.execute(outputParams))
      throw;

    outputParams.reset();

// open output raster
    std::auto_ptr<te::rst::Raster> rout (te::rst::RasterFactory::open(orinfo));
    std::cout << rout->toString();
  }
  catch(const std::exception& e)
  {
    std::cout << std::endl << "An exception has occurred in ArithmeticOperations(): " << e.what() << std::endl;
  }
  catch(...)
  {
    std::cout << std::endl << "An unexpected exception has occurred in ArithmeticOperations()!" << std::endl;
  }
}

void ArithmeticOp(const char* arithmeticString, const char* outPath, const std::vector<std::string>& inputStrings )
{
  std::vector<te::rst::Raster*> rasters;
  
  std::vector<std::string>::const_iterator it;
  
  for(it = inputStrings.cbegin(); it != inputStrings.cend(); ++it)
  {
    std::map<std::string, std::string> rinfo;
    rinfo["URI"] = *it;
    te::rst::Raster* rin = te::rst::RasterFactory::open(rinfo);
    rasters.push_back(rin);
  }

  ArithmeticOp(arithmeticString, outPath, rasters);
}

%}

//Wrapping functions

void SegmenterRegionGrowing(std::string inputFilePath, std::string outputFilePath, int minSegmentSize, double segmentsSimilarityThreshold);

void ArithmeticOp(const char* arithmeticString, const char* outPath, const std::vector<std::string>& inputStrings);

%{
#include "terralib/rp/RasterHandler.h"
#include "terralib/rp/Functions.h"
%}

%ignore te::rp::GetSpectralBandMin;
%ignore te::rp::GetSpectralBandMax;

#ifndef SWIGPYTHON
%nspace te::rp::RasterHandler;
#endif //SWIGPYTHON

%include "terralib/rp/RasterHandler.h"
%include "terralib/rp/Functions.h"