/*
 * File: Raster.i
 *
 * Raster module
 */

#define TERASTEREXPORT

UNIQUE_PTR_TYPEMAPS(te::rst::TileIndexer)
UNIQUE_PTR_TYPEMAPS(te::gm::Polygon)

%{
namespace te
{
  namespace rst
  {
    std::unique_ptr<te::da::DataSetType> CreateDataSetTypeToPoligonizer(std::string dataSetName, int srid)
    {
      std::unique_ptr<te::da::DataSetType> dsType(new te::da::DataSetType(dataSetName));

      //create id property
      te::dt::SimpleProperty* idProperty = new te::dt::SimpleProperty("id", te::dt::INT32_TYPE);
      dsType->add(idProperty);

      te::dt::SimpleProperty* valueProperty = new te::dt::SimpleProperty("value", te::dt::DOUBLE_TYPE, true);
      dsType->add(valueProperty);

      //create geometry property
      te::gm::GeometryProperty* geomProperty = new te::gm::GeometryProperty("geom", srid, te::gm::PolygonType);
      dsType->add(geomProperty);

      //create primary key
      std::string pkName = "pk_id";
                  pkName+= "_" + dataSetName;
      te::da::PrimaryKey* pk = new te::da::PrimaryKey(pkName, dsType.get());
      pk->add(idProperty);

      return dsType;
    }

    std::unique_ptr<te::mem::DataSet> CreateDataSetToPoligonizer(te::da::DataSetType* dsType,
	    													   std::vector<te::gm::Geometry*>& geoms,
		    												   std::vector< double >& geomsValues,
															   int geomsSrid)
    {
      std::unique_ptr<te::mem::DataSet> ds(new te::mem::DataSet(dsType));

      for(std::size_t t = 0; t < geoms.size(); ++t)
      {
        //create dataset item
        te::mem::DataSetItem* item = new te::mem::DataSetItem(ds.get());

        //set id
        item->setInt32(0, (int)t);

        //set Value
        item->setDouble( 1, geomsValues[ t ] );

        //set geometry
		geoms[t]->setSRID(geomsSrid);
        item->setGeometry(2, geoms[t]);

        ds->add(item);
      }

      return ds;
    }
  }
}
%}

%{
#include "terralib/raster/Band.h"
#include "terralib/raster/BandIterator.h"
#include "terralib/raster/BandIteratorWindow.h"
#include "terralib/raster/BandProperty.h"
#include "terralib/raster/BandSummary.h"
#include "terralib/raster/Enums.h"
#include "terralib/raster/Grid.h"
#include "terralib/raster/Interpolator.h"
#include "terralib/raster/PositionIterator.h"
#include "terralib/raster/ProxyRaster.h"
#include "terralib/raster/Raster.h"
#include "terralib/raster/RasterFactory.h"
#include "terralib/raster/RasterIterator.h"
#include "terralib/raster/RasterProperty.h"
#include "terralib/raster/RasterSummary.h"
#include "terralib/raster/RasterSummaryManager.h"
#include "terralib/raster/RasterSynchronizer.h"
#include "terralib/raster/Reprojection.h"
#include "terralib/raster/SynchronizedBand.h"
#include "terralib/raster/SynchronizedBandBlocksManager.h"
#include "terralib/raster/SynchronizedRaster.h"
#include "terralib/raster/TileIndexer.h"
#include "terralib/raster/Vectorizer.h"
#include "terralib/raster/VectorizerPolygonStructure.h"
%}

%ignore te::rst::Band::callOperator;
%ignore te::rst::Raster::callOperator;

//Python doesn't work with namespaces
#ifndef SWIGPYTHON

%nspace te::rst::SummaryTypes;
%nspace te::rst::ColorInterp;
%nspace te::rst::PaletteInterpretation;
%nspace te::rst::Band;
%nspace te::rst::BandIterator;
%nspace te::rst::BandIteratorWindow;
%nspace te::rst::BandProperty;
%nspace te::rst::BandSummary;
%nspace te::rst::Grid;
%nspace te::rst::Interpolator;
%nspace te::rst::PositionIterator;
%nspace te::rst::Raster;
%nspace te::rst::ProxyRaster;
%nspace te::rst::RasterFactory;
%nspace te::rst::RasterIterator;
%nspace te::rst::RasterProperty;
%nspace te::rst::RasterSummary;
%nspace te::rst::RasterSummaryManager;
%nspace te::rst::RasterSynchronizer;
%nspace te::rst::Reprojection;
%nspace te::rst::SynchronizedBand;
%nspace te::rst::SynchronizedBandBlocksManager;
%nspace te::rst::SynchronizedRaster;
%nspace te::rst::TileIndexer;
%nspace te::rst::Vectorizer;
%nspace te::rst::VectorizerPolygonStructure;

#endif //SWIGPYTHON

%include "terralib/raster/Enums.h"
%include "terralib/raster/Band.h"
%include "terralib/raster/BandIterator.h"
%include "terralib/raster/BandIteratorWindow.h"
%include "terralib/raster/BandProperty.h"
%include "terralib/raster/BandSummary.h"
%include "terralib/raster/Grid.h"
%include "terralib/raster/Interpolator.h"
%include "terralib/raster/PositionIterator.h"
%include "terralib/raster/Raster.h"
%include "terralib/raster/ProxyRaster.h"
%include "terralib/raster/RasterFactory.h"
%include "terralib/raster/RasterIterator.h"
%include "terralib/raster/RasterProperty.h"
%include "terralib/raster/RasterSummary.h"
%include "terralib/raster/RasterSummaryManager.h"
%include "terralib/raster/RasterSynchronizer.h"
%include "terralib/raster/Reprojection.h"
%include "terralib/raster/SynchronizedBand.h"
%include "terralib/raster/SynchronizedBandBlocksManager.h"
%include "terralib/raster/SynchronizedRaster.h"
%include "terralib/raster/TileIndexer.h"
%include "terralib/raster/Vectorizer.h"
%include "terralib/raster/VectorizerPolygonStructure.h"
