/*
 * File: AttributeFill.i
 *
 * AttributeFill module specific to Lua
 */

%extend te::attributefill::VectorToVectorOp {
  void setParams(const std::string& fromAttribute, te::attributefill::OperationType operation, te::da::DataSetType toDst)
  {
    std::map<std::string, std::vector<te::attributefill::OperationType> > options;
	std::vector<te::attributefill::OperationType> ops;
	std::vector<std::string> toLayerProps;

	std::vector<te::dt::Property*> props = toDst.getProperties();

    for(std::size_t i = 0; i < props.size(); ++i)
    {
	  toLayerProps.push_back(props[i]->getName());
    }

	ops.push_back(operation);
	options.insert(std::make_pair(fromAttribute, ops));

    $self->setParams(options, toLayerProps);
  }

  void setInput(te::map::AbstractLayer* fromLayer, te::map::AbstractLayer* toLayer)
  {
    te::map::AbstractLayerPtr fl(fromLayer);
    te::map::AbstractLayerPtr tl(toLayer);

	$self->setInput(fl, tl);
  }

  te::da::DataSource* createAndSetOutput(const std::string& dsName, const std::string& dsType,
										const std::string& connInfo)
  {
    te::da::DataSourcePtr outSource(te::da::DataSourceFactory::make(dsType, connInfo).release());
    outSource->open();

	$self->setOutput(outSource, dsName);

	return outSource.get();
  }

  te::da::DataSource* createAndSetOutput(const std::string& dsName, const std::string& dsType,
										const te::core::URI& connInfo)
  {
    te::da::DataSourcePtr outSource(te::da::DataSourceFactory::make(dsType, connInfo).release());
    outSource->open();

	$self->setOutput(outSource, dsName);

	return outSource.get();
  }

  void setOutput(te::da::DataSource* outDs, std::string dsName)
  {
	te::da::DataSourcePtr ds(outDs);
    $self->setOutput(ds, dsName);
  }
};

%extend te::attributefill::VectorToVectorMemory
{
  std::string exec()
  {
	try
	{
      $self->run();
	}
	catch (const te::common::Exception& e)
	{
	  return e.what();
	}
	catch (const std::exception& e)
	{
	  return e.what();
	}

	return "";
  }
};


%extend te::attributefill::RasterToVector {
  void setInput(te::rst::Raster* raster, te::map::DataSetLayer* toLayer)
  {
	te::da::DataSourcePtr vDs = te::da::GetDataSource(toLayer->getDataSourceId(), true);
	std::string fileNameVector(toLayer->getDataSetName());
    std::unique_ptr<te::da::DataSetType> dsTypeVector = vDs->getDataSetType(fileNameVector);
    std::unique_ptr<te::da::DataSetTypeConverter> converterVector(new te::da::DataSetTypeConverter(
									dsTypeVector.get(), vDs->getCapabilities(), vDs->getEncoding()));
    te::da::AssociateDataSetTypeConverterSRID(converterVector.get(), toLayer->getSRID());

	$self->setInput(raster, vDs, fileNameVector, converterVector.release(), toLayer->getSelected());
  }

  void setParams(int band, te::attributefill::OperationType operation, bool iteratorByBox, bool texture, bool readAll)
  {
	std::vector<unsigned int> bands;
	bands.push_back(band);

	std::vector<te::stat::StatisticalSummary> stats;

    switch(operation)
    {
      case te::attributefill::MIN_VALUE:
        stats.push_back(te::stat::MIN_VALUE);
        break;
      case te::attributefill::MAX_VALUE:
        stats.push_back(te::stat::MAX_VALUE);
        break;
      case te::attributefill::MEAN:
        stats.push_back(te::stat::MEAN);
        break;
      case te::attributefill::SUM:
        stats.push_back(te::stat::SUM);
        break;
      case te::attributefill::COUNT:
        stats.push_back(te::stat::COUNT);
        break;
        case te::attributefill::VALID_COUNT:
        stats.push_back(te::stat::VALID_COUNT);
        break;
      case te::attributefill::STANDARD_DEVIATION:
        stats.push_back(te::stat::STANDARD_DEVIATION);
        break;
        case te::attributefill::VARIANCE:
        stats.push_back(te::stat::VARIANCE);
        break;
      case te::attributefill::SKEWNESS:
        stats.push_back(te::stat::SKEWNESS);
        break;
      case te::attributefill::KURTOSIS:
        stats.push_back(te::stat::KURTOSIS);
        break;
      case te::attributefill::AMPLITUDE:
        stats.push_back(te::stat::AMPLITUDE);
        break;
      case te::attributefill::MEDIAN:
        stats.push_back(te::stat::MEDIAN);
        break;
      case te::attributefill::VAR_COEFF:
        stats.push_back(te::stat::VAR_COEFF);
        break;
      case te::attributefill::MODE:
        stats.push_back(te::stat::MODE);
        break;
      case te::attributefill::PERCENT_EACH_CLASS:
        stats.push_back(te::stat::PERCENT_EACH_CLASS_BY_AREA);
        break;
      case te::attributefill::TOTAL_AREA_BY_CLASS:
        stats.push_back(te::stat::TOTAL_AREA_BY_CLASS);
        break;
      default:
        break;
    }

	$self->setParams(bands, stats, iteratorByBox, texture);
  }

  te::da::DataSource* createAndSetOutput(const std::string& dsName, const std::string& dsType,
										const std::string& connInfo)
  {
    te::da::DataSourcePtr outSource(te::da::DataSourceFactory::make(dsType, connInfo).release());
    outSource->open();

	$self->setOutput(outSource, dsName);

	return outSource.get();
  }

  te::da::DataSource* createAndSetOutput(const std::string& dsName, const std::string& dsType,
										const te::core::URI& connInfo)
  {
    te::da::DataSourcePtr outSource(te::da::DataSourceFactory::make(dsType, connInfo).release());
    outSource->open();

	$self->setOutput(outSource, dsName);

	return outSource.get();
  }

  void setOutput(te::da::DataSource* ds, const std::string& dseName)
  {
    te::da::DataSourcePtr dsptr(ds);
	$self->setOutput(dsptr, dseName);
  }

  std::string exec()
  {
	try
	{
      $self->run();
	}
	catch (const te::common::Exception& e)
	{
	  return e.what();
	}
	catch (const std::exception& e)
	{
	  return e.what();
	}

	return "";
  }
};