/*
 * File: DataAccess.i
 *
 * Data Access especific to Lua
 */

%{
  class DataSetTable{};
  class RasterTable{};

  static DataSetTable getDataSetAsTable(te::da::DataSet* dset, std::string missing)
  {
    return DataSetTable();
  }

  static RasterTable getRasterAsTable(te::da::DataSet* dset)
  {
    return RasterTable();
  }
%}

static DataSetTable getDataSetAsTable(te::da::DataSet* dset, std::string missing);
static RasterTable getRasterAsTable(te::da::DataSet* dset);


%{
te::da::DataSource* GetDs(const std::string& dsId, const bool opened = true)
{
  te::da::DataSourcePtr ds = te::da::GetDataSource(dsId, opened);

  return ds.get();
}

// TODO: why 'const std::vector<std::string>&' typemap does not work in %extend?
void UpdateDs(te::da::DataSource* ds, const std::string& dseName,
			    te::mem::DataSet* dseUpdated, const std::vector<std::string>& propsUpdated)
{
  std::unique_ptr<te::da::DataSetType> dst = ds->getDataSetType(dseName);
  std::vector<int> propsUpdatedPos;

  for (size_t i = 0; i < propsUpdated.size(); i++)
  {
    std::string propName(propsUpdated[i]);
    propsUpdatedPos.push_back((int)dst->getPropertyPosition(propName));
  }

  te::da::PrimaryKey* pk = dst->getPrimaryKey();
  size_t numPks = pk->getProperties().size();
  std::vector<std::size_t> ids;

  for (size_t i = 0; i < numPks; ++i)
  {
    std::string pkName = pk->getProperties()[i]->getName();
    ids.push_back(dst->getPropertyPosition(pkName));
  }

  std::vector< std::set<int> > propsUpPerLine;

  dseUpdated->moveBeforeFirst();
  while (dseUpdated->moveNext())
  {
    std::set<int> propsUpInLine;

    for (size_t i = 0; i < propsUpdated.size(); i++)
    {
	  std::string propName(propsUpdated[i]);
	  int propPos = propsUpdatedPos[i];
	  propsUpInLine.insert(propPos);
    }

    if (propsUpInLine.size() > 0)
    {
	  propsUpPerLine.push_back(propsUpInLine);
    }
  }

  if (propsUpPerLine.size() > 0)
  {
    ds->update(dseName, dseUpdated, propsUpPerLine, ids);
  }
}

std::string CheckName(const std::string& name)
{
	std::string msg = "";
	te::da::IsValidName(name, msg);
	return msg;
}
%}

te::da::DataSource* GetDs(const std::string& dsId, bool opened);

void UpdateDs(te::da::DataSource* ds, const std::string& dseName,
			    te::mem::DataSet* dseUpdated, const std::vector<std::string>& propsUpdated);

std::string CheckName(const std::string& name);

%extend te::da::DataSourceManager {
  void insert(te::da::DataSource* source)
  {
    te::da::DataSourcePtr src(source);

    $self->insert(src);
  }

  void open(const te::da::DataSourceInfo& info)
  {
    $self->open(info.getId(), info.getType(), info.getConnInfo());
  }

  te::da::DataSource* search(const std::string& id)
  {
	return ($self->find(id)).get();
  }

  te::da::DataSource* detach(te::da::DataSource* ds)
  {
    te::da::DataSourcePtr src = $self->detach(ds->getId());
	return src.get();
  }
};

%extend te::da::DataSourceInfoManager {
  bool add(te::da::DataSourceInfo source)
  {
	te::da::DataSourceInfoPtr ds;
    ds.reset(new te::da::DataSourceInfo);
	*ds = source;
	bool res = $self->add(ds);

	ds.reset();

	return res;
  }

  te::da::DataSourceInfo getDsInfo(const std::string& id)
  {
	te::da::DataSourceInfoPtr ds;
    ds = $self->get(id);

	if (ds.get() == 0)
		return te::da::DataSourceInfo();

    return *ds.get();
  }

  te::da::DataSourceInfo getDsInfoByConnInfo(const std::string& connInfoAsString)
  {
	te::da::DataSourceInfoPtr ds;
    ds = $self->getByConnInfo(connInfoAsString);

	if (ds.get() == 0)
		return te::da::DataSourceInfo();

    return *ds.get();
  }

  void show()
  {
    te::da::DataSourceInfoManager::iterator it;
    te::da::DataSourceInfoManager::iterator beg = te::da::DataSourceInfoManager::getInstance().begin();
    te::da::DataSourceInfoManager::iterator end = te::da::DataSourceInfoManager::getInstance().end();

    for(it = beg; it != end; ++it)
	{
      std::cout << "------DataSource Info------" << std::endl;
      std::cout << "id:\t" << it->second.get()->getId() << std::endl;
      std::cout << "type:\t" << it->second.get()->getType() << std::endl;
      std::cout << "access driver:\t" << it->second.get()->getAccessDriver() << std::endl;
      std::cout << "title:\t" << it->second.get()->getTitle() << std::endl;
      std::cout << "description:\t" << it->second.get()->getDescription() << std::endl;
      std::cout << "conn info:\t" << it->second.get()->getConnInfoAsString() << std::endl;
	  te::core::URI uri(it->second.get()->getConnInfo());
	  std::cout << "scheme:\t" << uri.scheme() << std::endl;
	  std::cout << "user:\t" << uri.user() << std::endl;
	  std::cout << "password:\t" << uri.password() << std::endl;
	  std::cout << "host:\t" << uri.host() << std::endl;
	  std::cout << "port:\t" << uri.port() << std::endl;
	  std::cout << "path:\t" << uri.path() << std::endl;
	  std::cout << "query:\t" << uri.query() << std::endl;
	  std::cout << "fragment:\t" << uri.fragment() << std::endl;
      std::cout << "\n\n" << std::endl;
	}
  }
};

%extend te::da::DataSource {
  static std::string Check(te::da::DataSource* ds)
  {
    try
	{
	  if (ds->exists(ds->getType(), ds->getConnectionInfo().uri()))
		  return "";
	}
	catch (const std::exception& e)
	{
	  return e.what();
	}

	return "";
  }

  void createDataSet(DataSetType* dst)
  {
	std::map<std::string, std::string> options;
	$self->createDataSet(dst, options);
  }

  void add(const std::string& dstName, DataSet* dse)
  {
	std::map<std::string, std::string> options;
	$self->add(dstName, dse, options);
  }

  void updateNullValues(const std::string& dseName, const std::string& propName, const std::string& data)
  {
    std::unique_ptr<te::da::DataSet>	dse = $self->getDataSet(dseName);
	std::unique_ptr<te::da::DataSetType> dst = $self->getDataSetType(dseName);

	unsigned int propPos = (unsigned int)dst->getPropertyPosition(propName);
	te::da::PrimaryKey* pk = dst->getPrimaryKey();
    size_t numPks = pk->getProperties().size();
    std::vector<std::size_t> ids;

    for (size_t i = 0; i < numPks; ++i)
    {
      std::string pkName = pk->getProperties()[i]->getName();
      ids.push_back(dst->getPropertyPosition(pkName));
    }

	te::mem::DataSet* newDse = new te::mem::DataSet(dst.get());
	std::vector< std::set<int> > fields;

	size_t numProps = dse->getNumProperties();

	while (dse->moveNext())
	{
      te::mem::DataSetItem* item = new te::mem::DataSetItem(newDse);
	  std::set<int> cols;

	  for (unsigned int i = 0; i < numProps; i++)
	  {
	    if (dse->isNull(i) && (i == propPos))
	    {
          switch(dse->getPropertyDataType(i))
          {
            case te::dt::INT16_TYPE:
               item->setValue(i, new te::dt::Int16(stoi(data)));
            break;

            case te::dt::INT32_TYPE:
               item->setValue(i, new te::dt::Int32(stoi(data)));
            break;

            case te::dt::INT64_TYPE:
               item->setValue(i, new te::dt::Int64(stoi(data)));
            break;

            case te::dt::FLOAT_TYPE:
               item->setValue(i, new te::dt::Float(stof(data)));
            break;

            case te::dt::DOUBLE_TYPE:
               item->setValue(i, new te::dt::Double(stod(data)));
            break;

            case te::dt::NUMERIC_TYPE:
               item->setValue(i, new te::dt::Numeric(data));
            break;

            case te::dt::STRING_TYPE:
               item->setValue(i, new te::dt::String(data));
            break;

            default:
              break;
		  }

		  cols.insert(propPos);
	    }
	    else
	    {
		  item->setValue(i, dse->getValue(i).release());
	    }
	  }

	  if (cols.size() > 0)
	  {
		newDse->add(item);
		fields.push_back(cols);
	  }
	}

	if (fields.size() > 0)
	{
		$self->update(dseName, newDse, fields, ids);
	}
  }

  te::da::DataSetType* cloneDataSetType(const std::string& dsetName)
  {
	std::unique_ptr<te::da::DataSetType> dst = $self->getDataSetType(dsetName);
    return (te::da::DataSetType*)dst->clone();
  }
};

%extend te::da::PrimaryKey {
  size_t getNumProperties()
  {
    return $self->getProperties().size();
  }

  void setPropertyName(int i, const std::string& name)
  {
    $self->getProperties()[i]->setName(name);
  }

  std::string getPropertyName(int i)
  {
    return $self->getProperties()[i]->getName();
  }

  void setName(const std::string& name)
  {
    $self->setName(name);
  }

  const std::string& getName()
  {
    return $self->getName();
  }

  te::dt::Property* getProperty(int i)
  {
    return $self->getProperties()[i];
  }
};

%extend te::da::DataSet {
  te::dt::AbstractData* getValue(int i)
  {
	return $self->getValue(i).release();
  }

  te::gm::Geometry* getGeom(int i)
  {
    return $self->getGeometry(i).release();
  }

  int getInt(int i)
  {
	switch($self->getPropertyDataType(i))
    {
      case te::dt::INT16_TYPE:
        return boost::numeric_cast<int>($self->getInt16(i));

      case te::dt::UINT16_TYPE:
        return boost::numeric_cast<unsigned int>($self->getInt16(i));

      case te::dt::INT32_TYPE:
        return boost::numeric_cast<int>($self->getInt32(i));

      case te::dt::UINT32_TYPE:
        return boost::numeric_cast<unsigned int>($self->getInt32(i));

      case te::dt::INT64_TYPE:
        return boost::numeric_cast<int>($self->getInt64(i));

      case te::dt::UINT64_TYPE:
        return boost::numeric_cast<unsigned int>($self->getInt64(i));

      default:
        return boost::numeric_cast<int>($self->getInt32(i));
	}
  }
};

%extend te::da::DataSetType {
  void add(const std::string& name, bool isPk, int datatype, bool isRequired)
  {
    te::dt::SimpleProperty* prop;

	switch(datatype)
    {
	  case te::dt::NUMERIC_TYPE:
		prop = new te::dt::NumericProperty(name, 10, 3, isRequired);
		break;
	  case te::dt::STRING_TYPE:
	    prop = new te::dt::StringProperty(name);
	    break;
	  case te::dt::DATETIME_TYPE:
	    prop = new te::dt::DateTimeProperty(name, te::dt::DATE, isRequired);
		break;
	  default:
	    prop = new te::dt::SimpleProperty(name, datatype, isRequired);
        break;
	}

	$self->add(prop);

    if (isPk)
    {
	  te::da::PrimaryKey* pk = new te::da::PrimaryKey($self->getName() + "_pk_" + name, $self);
	  pk->add(prop);
	}
  }

  void add(const std::string& name, int srid, te::gm::GeomType subtype, bool isRequired)
  {
	te::gm::GeometryProperty* prop = new te::gm::GeometryProperty(name, srid, subtype, isRequired);
	$self->add(prop);
  }

  void removeIndex(const std::string& name)
  {
    $self->remove($self->getIndex(name));
  }
};

%extend te::da::DataSourceTransactor {
  void add(const std::string& datasetName, te::da::DataSetAdapter* dset, std::size_t limit = 0)
  {
	std::map<std::string,std::string> nopt;
    $self->add(datasetName, dset, nopt);
  }
};