/*
 * File: Geometry.i
 *
 * Geometry module specific to Lua
 */

%extend te::gm::Geometry {
  static te::gm::MultiLineString* toMultiLineString(Geometry* geom)
  {
    return dynamic_cast<te::gm::MultiLineString*>(geom);
  }

  static te::gm::LineString* toLineString(Geometry* geom)
  {
    return dynamic_cast<te::gm::LineString*>(geom);
  }

  static te::gm::MultiPolygon* toMultiPolygon(Geometry* geom)
  {
    return dynamic_cast<te::gm::MultiPolygon*>(geom);
  }

  static te::gm::Line* toLine(Geometry* geom)
  {
    return dynamic_cast<te::gm::Line*>(geom);
  }

  static te::gm::Point* toPoint(Geometry* geom)
  {
    return dynamic_cast<te::gm::Point*>(geom);
  }

  static te::gm::Polygon* toPolygon(Geometry* geom)
  {
    return dynamic_cast<te::gm::Polygon*>(geom);
  }

  static te::gm::MultiPoint* toMultiPoint(Geometry* geom)
  {
    return dynamic_cast<te::gm::MultiPoint*>(geom);
  }

  static te::gm::CircularString* toCircularString(Geometry* geom)
  {
    return dynamic_cast<te::gm::CircularString*>(geom);
  }

  static te::gm::CompoundCurve* toCompoundCurve(Geometry* geom)
  {
    return dynamic_cast<te::gm::CompoundCurve*>(geom);
  }

  static te::gm::CurvePolygon* toCurvePolygon(Geometry* geom)
  {
    return dynamic_cast<te::gm::CurvePolygon*>(geom);
  }

  static te::gm::GeometryCollection* toGeometryCollection(Geometry* geom)
  {
    return dynamic_cast<te::gm::GeometryCollection*>(geom);
  }

  static te::gm::MultiSurface* toMultiSurface(Geometry* geom)
  {
    return dynamic_cast<te::gm::MultiSurface*>(geom);
  }

  static te::gm::PolyhedralSurface* toPolyhedralSurface(Geometry* geom)
  {
    return dynamic_cast<te::gm::PolyhedralSurface*>(geom);
  }

  static te::gm::TIN* toTIN(Geometry* geom)
  {
    return dynamic_cast<te::gm::TIN*>(geom);
  }

  static te::gm::Triangle* toTriangle(Geometry* geom)
  {
    return dynamic_cast<te::gm::Triangle*>(geom);
  }

  double getLength()
  {
	if ($self->getGeometryType() == "MultiLineString")
	{
      return dynamic_cast<te::gm::MultiLineString*>($self)->getLength();
	}

	if ($self->getGeometryType() == "LineString")
	{
      return dynamic_cast<te::gm::LineString*>($self)->getLength();
	}

    throw std::runtime_error("Use a LineString or MultiLineString to get the length.");
  }

  double getArea()
  {
	if ($self->getGeometryType() == "Polygon")
	{
      return dynamic_cast<te::gm::Polygon*>($self)->getArea();
	}

	if ($self->getGeometryType() == "MultiPolygon")
	{
      return dynamic_cast<te::gm::MultiPolygon*>($self)->getArea();
	}

	throw std::runtime_error("Use a Polygon or MultiPolygon to get the area.");
  }
};

%extend te::gm::MultiLineString {
  te::gm::MultiLineString* clone()
  {
    return dynamic_cast<te::gm::MultiLineString*>($self->clone());
  }

  te::gm::LineString* getGeometryN(int i)
  {
    return dynamic_cast<te::gm::LineString*>($self->getGeometryN(i));
  }
};

%extend te::gm::LineString {
  te::gm::LineString* clone()
  {
    return dynamic_cast<te::gm::LineString*>($self->clone());
  }

  te::gm::Point* getPointN(int i)
  {
    return dynamic_cast<te::gm::Point*>($self->getPointN(i)->clone());
  }

  std::string getPointAsTextAt(int i)
  {
    return $self->getPointN(i)->asText();
  }

  te::gm::Point* getStartPoint()
  {
    return dynamic_cast<te::gm::Point*>($self->getStartPoint()->clone());
  }

  te::gm::Point* getEndPoint()
  {
    return dynamic_cast<te::gm::Point*>($self->getEndPoint()->clone());
  }
};

%extend te::gm::Point {
  te::gm::Point* clone()
  {
    return dynamic_cast<te::gm::Point*>($self->clone());
  }

  te::gm::Point closestPoint(Geometry* geom)
  {
    return te::gm::ClosestPointInGeometryFromPoint($self, geom);
  }

  std::string getClosestPointOfLineAsText(te::gm::LineString* line)
  {
    te::gm::Point closestPoint = te::gm::ClosestPointInGeometryFromPoint($self, line);

    int resultPointPos = 0;
    double minDistance = std::numeric_limits<double>::max();
    for(unsigned int i = 0; i < line->getNPoints(); i++)
    {
      double distance = closestPoint.distance(line->getPointN(i).get());
      if(distance < minDistance)
      {
        minDistance = distance;
        resultPointPos = i;
      }
    }
    return line->getPointN(resultPointPos)->asText();
  }
};

%extend te::gm::MultiPoint {
  te::gm::Point* getGeometryN(int i)
  {
    return dynamic_cast<te::gm::Point*>($self->getGeometryN(i));
  }
};

%extend te::gm::MultiPolygon {
  te::gm::Polygon* getGeometryN(int i)
  {
    return dynamic_cast<te::gm::Polygon*>($self->getGeometryN(i));
  }
};

%extend te::gm::Polygon {
  te::gm::LineString* getExteriorRing()
  {
    return dynamic_cast<te::gm::LineString*>($self->getExteriorRing());
  }
};
