/*
 * File: Maptools.i
 *
 * Map Tools module specific to Lua
 */

%{
struct GeometryErrorInfo
{
  std::string propertyName;
  std::string propertyValue;
  std::string errorMsg;
  double coordX;
  double coordY;
};

struct GeometryError
{
  std::vector<GeometryErrorInfo*> problems;
  te::da::ObjectIdSet* objectIdSet;

  std::vector<GeometryErrorInfo*> getProblems()
  {
    return problems;
  }
};

static GeometryError CheckLayerGeometries(te::map::AbstractLayer* layer)
{
  std::unique_ptr<te::da::DataSetType> dst = layer->getSchema();
  std::unique_ptr<te::da::DataSet> dse = layer->getData();
  te::da::PrimaryKey* pk = dst->getPrimaryKey();
  std::string pkName;
  GeometryError geomError;

  if (pk)
  {
    pkName = pk->getProperties()[0]->getName();
    std::size_t pkPos = dst->getPropertyPosition(pkName);
    int pkType = pk->getProperties()[0]->getType();
	geomError.objectIdSet = new te::da::ObjectIdSet();
    geomError.objectIdSet->addProperty(pkName, pkPos, pkType);

    std::size_t geomPos = te::da::GetFirstSpatialPropertyPos(dse.get());
    te::common::TaskProgress task;
    task.setTotalSteps((int)dse->size());

    dse->moveBeforeFirst();
    while(dse->moveNext())
    {
      std::unique_ptr<te::gm::Geometry> geom = dse->getGeometry(geomPos);
      te::gm::TopologyValidationError err;

      if (!te::gm::Validation::CheckValidity(geom.get(), err))
      {
        te::da::ObjectId* oid = new te::da::ObjectId;
        oid->addValue(dse->getValue(pkName)->clone());
        geomError.objectIdSet->add(oid);
        GeometryErrorInfo* geomErrorInfo = new GeometryErrorInfo();
        geomErrorInfo->propertyName = pkName;
        geomErrorInfo->propertyValue = dse->getAsString(pkName);
        geomErrorInfo->errorMsg = err.m_message;
        geomErrorInfo->coordX = err.m_coordinate.getX();
        geomErrorInfo->coordY = err.m_coordinate.getY();
        geomError.problems.push_back(geomErrorInfo);
      }
	
	  task.pulse();
    }
  }

  return geomError;
}
%}

struct GeometryErrorInfo
{
  std::string propertyName;
  std::string propertyValue;
  std::string errorMsg;
  double coordX;
  double coordY;
};

struct GeometryError
{
  te::da::ObjectIdSet* objectIdSet;
  std::vector<GeometryErrorInfo*> getProblems();
};

static GeometryError CheckLayerGeometries(te::map::AbstractLayer* layer);

%extend te::map::DataSetLayer {
  static DataSetLayer* toDataSetLayer(te::map::AbstractLayer* layer)
  {
    return dynamic_cast<te::map::DataSetLayer*>(layer);
  }
};
