/*
 * File: Raster.i
 *
 * Raster module specific to Lua
 */

%extend te::rst::Raster{
  double getValue(int c, int r, int b)
  {
    double value;
    $self->getValue(c, r, value, b);
    return value;
  }

  void setValue(int c, int r, double value, int b)
  {
    $self->setValue(c, r, value, b);
  }

  void polygonize(int band, const te::da::DataSourceInfo& outInfo, int outSrid)
  {
    //output parameters
    std::vector<te::gm::Geometry*> geomVec;
    std::vector< double > geomValues;
    int maxGeom = 0; //<- what is this?

	//run operation
    $self->vectorize(geomVec, band, maxGeom, &geomValues);

    //save data
	std::string outName(outInfo.getTitle());
    std::unique_ptr<te::da::DataSetType> dst = te::rst::CreateDataSetTypeToPoligonizer(outName, outSrid);
    std::unique_ptr<te::mem::DataSet> dstMem = te::rst::CreateDataSetToPoligonizer(dst.get(), geomVec, geomValues, outSrid);
	te::da::DataSourcePtr outDs(te::da::DataSourceFactory::make(outInfo.getType(), outInfo.getConnInfo()).release());
	outDs->open();

    //save dataset
    dstMem->moveBeforeFirst();
    std::map<std::string, std::string> options;
    outDs->createDataSet(dst.get(), options);
    outDs->add(outName, dstMem.get(), options);
  }
};

%extend te::rst::Band{
  double getValue(int c, int r)
  {
    double value;
    $self->getValue(c, r, value);
    return value;
  }
};

%ignore te::rst::VectorizerPolygonStructure::operator=;
