/*
 * File: SAM.i
 *
 * SAM module
 */

%{
#include <boost/geometry.hpp>
#include <boost/geometry/geometries/point.hpp>
#include <boost/geometry/geometries/box.hpp>
#include <boost/geometry/index/rtree.hpp>

class RTree
{
typedef boost::geometry::model::point<double, 2, boost::geometry::cs::cartesian> Point;
typedef boost::geometry::model::box<Point> Box;
typedef std::pair<Box, long long> Value;

public:
  RTree(){}
  ~RTree(){}
  
  void insert(te::gm::Geometry* geom, long long index)
  {
	te::gm::Envelope mbr = *geom->getMBR();
	rstar.insert(std::make_pair(
                              Box(
                                Point(mbr.getLowerLeftX(), mbr.getLowerLeftY()),
                                Point(mbr.getUpperRightX(), mbr.getUpperRightY())), 
                              index));
  }
  
  std::vector<long long> intersects(te::gm::Point* point)
  {
    std::vector<long long> results;
	std::vector<Value> intersects;
    rstar.query(boost::geometry::index::intersects(
                                       Point(point->getX(), point->getY())), 
                                       std::back_inserter(intersects));
	for (const auto &i : intersects)
    {
      results.push_back(i.second);
    }
    return results;    
  }
  
  std::vector<long long> nearest(te::gm::Point* point, int numOfResults)
  {
    std::vector<long long> results;
    std::vector<Value> nearest;
    rstar.query(boost::geometry::index::nearest(
                                             Point(point->getX(), point->getY()), numOfResults), 
                                             std::back_inserter(nearest));
    for (const auto &n : nearest)
    {
      results.push_back(n.second);
    }
    return results;
  }	
  
private:
  boost::geometry::index::rtree<Value, boost::geometry::index::rstar<16> > rstar;
};
%}

class RTree
{
public:
  RTree();
  ~RTree();
  void insert(te::gm::Geometry* geom, long long index);
  std::vector<long long> nearest(te::gm::Point* point, int numOfResults);
  std::vector<long long> intersects(te::gm::Point* point);
};