/*
 * File: WMS.i
 *
 * WMS module especific to Lua
 */

%extend te::ws::ogc::wms::Layer
{
  void show()
  {
	std::vector<te::ws::ogc::wms::Layer> childLayers = $self->m_layers;

    for(size_t i = 0; i < childLayers.size(); i++)
    {
      te::ws::ogc::wms::Layer childLayer = childLayers[i];

      std::cout << "Layer Info" << std::endl;

      std::cout << "Layer Name: " << childLayer.m_name << std::endl;
      std::cout << "Layer Title: " << childLayer.m_title << std::endl;

      std::cout << "Layer Possible Bounding Boxes" << std::endl;

      for(size_t j = 0; j < childLayer.m_boundingBoxes.size(); j++)
      {
        te::ws::ogc::wms::BoundingBox bbox = childLayer.m_boundingBoxes[j];

        std::cout << "BoundingBox CRS: " << bbox.m_crs << std::endl;
        std::cout << "minx: " << bbox.m_minX << std::endl;
        std::cout << "maxx: " << bbox.m_maxX << std::endl;
        std::cout << "minx: " << bbox.m_minX << std::endl;
        std::cout << "maxy: " << bbox.m_maxY << std::endl;
      }
    }
  }

  te::ws::ogc::wms::Layer getLayerByTitle(const std::string& title)
  {
	std::vector<te::ws::ogc::wms::Layer> childLayers = $self->m_layers;

    for(size_t i = 0; i < childLayers.size(); i++)
    {
	  if (childLayers[i].m_title == title)
		return childLayers[i];
    }

	return te::ws::ogc::wms::Layer();
  }

  te::ws::ogc::wms::Layer getLayerByDataSetName(const std::string& name)
  {
	std::vector<te::ws::ogc::wms::Layer> childLayers = $self->m_layers;

    for(size_t i = 0; i < childLayers.size(); i++)
    {
	  if (childLayers[i].m_name == name)
		return childLayers[i];
    }

	return te::ws::ogc::wms::Layer();
  }

  te::ws::ogc::wms::WMSGetMapRequest createMapRequest()
  {
    te::ws::ogc::wms::WMSGetMapRequest request = te::ws::ogc::wms::WMSGetMapRequest();

    request.m_layers.push_back($self->m_name);

    if($self->m_styles.size() > 0)
    {
      request.m_styles.push_back($self->m_styles[0].m_name);
    }

    for(size_t i = 0; i < $self->m_boundingBoxes.size(); i++)
    {
      if($self->m_boundingBoxes[i].m_crs.find("EPSG:") == std::string::npos)
      {
        continue;
      }
      else
      {
        te::ws::ogc::wms::BoundingBox bbox = $self->m_boundingBoxes[i];
        request.m_srs = bbox.m_crs;
        request.m_boundingBox = bbox;
        break;
      }
    }

    return request;
  }
};

%extend te::ws::ogc::wms::WMSLayer
{
  static te::ws::ogc::wms::WMSLayer* toWmsLayer(te::map::AbstractLayer* layer)
  {
    return dynamic_cast<te::ws::ogc::wms::WMSLayer*>(layer);
  }
}


