/*
 * Defining a typemap for std::auto_ptr
 */
%include std_vector.i
%include std_map.i

%define AUTO_PTR_TYPEMAPS(TYPE...)

%typemap(out) (std::auto_ptr< TYPE >) %{
  SWIG_NewPointerObj(L, (void *) $result.release(), $descriptor(TYPE *), 1);
  SWIG_arg++;
%}

%template () std::auto_ptr< TYPE >;

%enddef

%define WEAK_PTR_TYPEMAPS(TYPE...)

%typemap(out) (std::weak_ptr< TYPE >) %{
  SWIG_NewPointerObj(L, (void *) $result.release(), $descriptor(TYPE *), 1);
  SWIG_arg++;
%}

%template () std::weak_ptr< TYPE >;

%enddef

%define UNIQUE_PTR_TYPEMAPS(TYPE...)

%typemap(out) (std::unique_ptr< TYPE >) %{
  SWIG_NewPointerObj(L, (void *) $result.release(), $descriptor(TYPE *), 1);
  SWIG_arg++;
%}

%typemap (memberin) std::unique_ptr<TYPE>%{
	$1.reset($input->release());
%}

%typemap (in) std::unique_ptr<TYPE>%{
  if(lua_isuserdata(L, $input))
  {
    std::unique_ptr<TYPE>* aux = static_cast< std::unique_ptr<TYPE>* >(lua_touserdata(L, $input));
    $1.reset(aux->release());
  }
%}

%template () std::unique_ptr<TYPE>;

%enddef

%define SHARED_PTR_TYPEMAPS(TYPE...)

%typemap(out) (std::shared_ptr< TYPE >) %{
  SWIG_NewPointerObj(L, (void *) $result.release(), $descriptor(TYPE *), 1);
  SWIG_arg++;
%}

%typemap (memberin) std::shared_ptr<TYPE>%{
	$1.reset($input->release());
%}

%typemap (in) std::shared_ptr<TYPE>%{
  if(lua_isuserdata(L, $input))
  {
    std::shared_ptr<TYPE>* aux = static_cast< std::shared_ptr<TYPE>* >(lua_touserdata(L, $input));
    $1.reset(aux->release());
  }
%}

%template () std::unique_ptr<TYPE>;

%enddef

namespace std
{
  %template (StrVector) vector <string>;
  %template (LongLongVector) vector <long long>;
  %template (StrStrMap) map <string, string>; 
}

/*
 * Defining a typemap for std::map< std::string, std::string >
 */
%typemap(in) std::map< std::string, std::string >, const std::map< std::string, std::string > & (std::map< std::string, std::string > aux)
{
  lua_pushnil(L);

  while(lua_next(L, -2) != 0)
  {
    if(lua_isstring(L, -1) && lua_isstring(L, -2))
    {
      std::string key(lua_tostring(L, -2)),
                  value(lua_tostring(L, -1));

      aux[key] = value;
    }

    lua_pop(L, 1);
  }

  $1 = &aux;
}

%typemap(out) const std::map< std::string, std::string >&
{
  lua_newtable(L);

  std::map<std::string, std::string>::const_iterator it;

  for(it = $result->begin(); it != $result->end(); ++it)
  {
    const char* key = it->first.c_str();
    const char* value = it->second.c_str();

    lua_pushstring(L, key);
    lua_pushstring(L, value);
    lua_settable(L, -3);
  }

  return 1;
}

%typemap(out) std::map< std::string, std::string >
{
  lua_newtable(L);

  std::map<std::string, std::string>::const_iterator it;

  for(it = $result.begin(); it != $result.end(); ++it)
  {
    const char* key = it->first.c_str();
    const char* value = it->second.c_str();

    lua_pushstring(L, key);
    lua_pushstring(L, value);
    lua_settable(L, -3);
  }

  return 1;
}

%typemap(out) std::map< std::string, std::string >&
{
  lua_newtable(L);

  std::map<std::string, std::string>::const_iterator it;

  for(it = $result->begin(); it != $result->end(); ++it)
  {
    const char* key = it->first.c_str();
    const char* value = it->second.c_str();

    lua_pushstring(L, key);
    lua_pushstring(L, value);
    lua_settable(L, -3);
  }

  return 1;
}

/*
 * Defining a typemap for std::vector< std::string >
 */
%typemap(in) std::vector< std::string >, const std::vector< std::string > & (std::vector< std::string > aux)
{
  lua_pushnil(L);

  while(lua_next(L, -2) != 0)
  {
    if(lua_isstring(L, -1))
    {
      std::string value(lua_tostring(L, -1));
      aux.push_back(value);
    }

    lua_pop(L, 1);
  }

  $1 = &aux;
}

%typemap(out) std::vector<std::string>
{
  int s = (int)$result.size();

  int sI = static_cast<int>(s);

  lua_createtable(L, s, 0);
  int newTable = lua_gettop(L);

  for(int i = 0; i < sI; i++)
  {
    lua_pushstring(L, $result[i].c_str());
	lua_rawseti(L, newTable, i);
  }

  return 1;
}

%typemap(in) std::vector< double > (std::vector< double > aux)
{
  lua_pushnil(L);

  while(lua_next(L, -2) != 0)
  {
    if(lua_isnumber(L, -1))
    {
      double value(lua_tonumber(L, -1));
      aux.push_back(value);
    }

    lua_pop(L, 1);
  }

  $1 = aux;
}

%typemap(in) const std::vector< double >& (std::vector< double > aux)
{
  lua_pushnil(L);

  while(lua_next(L, -2) != 0)
  {
    if(lua_isnumber(L, -1))
    {
      double value(lua_tonumber(L, -1));
      aux.push_back(value);
    }

    lua_pop(L, 1);
  }

  $1 = &aux;
}

%typemap(out) std::vector<long long>
{
  long long s = (long long)$result.size();
  long long sI = static_cast<int>(s);
  lua_createtable(L, s, 0);
  int newTable = lua_gettop(L);

  for(long long i = 0; i < sI; i++)
  {
    lua_pushinteger(L, $result[i]);
	lua_rawseti(L, newTable, i);
  }

  return 1;
}

namespace std
{
  template<class T> class auto_ptr { };
  template<class T> class unique_ptr { };
}

/*
 * Defining a typemap for std::vector< te::map::AbstractLayer* >
 */
%typemap(in) std::vector< te::map::AbstractLayer* >, const std::vector< te::map::AbstractLayer* > & (std::vector< te::map::AbstractLayer* > aux)
{
  lua_pushnil(L);

  while(lua_next(L, -2) != 0)
  {
    if(lua_isuserdata(L, -1))
    {
	  swig_lua_userdata* usr = (swig_lua_userdata*)lua_touserdata(L, -1);
	  te::map::AbstractLayer* l = (te::map::AbstractLayer*)usr->ptr;
      aux.push_back(l);
    }

    lua_pop(L, 1);
  }

  $1 = &aux;
}

%typemap(out) std::vector< te::map::AbstractLayer* >
{
  std::vector<te::map::AbstractLayer*> layers = (std::vector<te::map::AbstractLayer*>)$result;
  int s = (int)layers.size();
  int sI = static_cast<int>(s);

  lua_createtable(L, s, 0);
  int newTable = lua_gettop(L);

  for(int i = 0; i < sI; i++)
  {
	SWIG_NewPointerObj(L, layers[i], $descriptor(te::map::AbstractLayer*), 0);
	lua_rawseti(L, newTable, i);
  }

  return 1;
}

/*
 * Defining a typemap for std::vector< QgisLayerInfo* >
 */
%typemap(in) std::vector< QgisLayerInfo* >, const std::vector< QgisLayerInfo* > & (std::vector< QgisLayerInfo* > aux)
{
  lua_pushnil(L);

  while(lua_next(L, -2) != 0)
  {
    if(lua_isuserdata(L, -1))
    {
	  swig_lua_userdata usr = reinterpret_cast<swig_lua_userdata*>(lua_touserdata(L, -1));
	  QgisLayerInfo* l = reinterpret_cast<QgisLayerInfo*>(usr->ptr);
      aux.push_back(l);
    }

    lua_pop(L, 1);
  }

  $1 = &aux;
}

%typemap(out) std::vector< QgisLayerInfo* >
{
  std::vector<QgisLayerInfo*> layers = (std::vector<QgisLayerInfo*>)$result;
  int s = (int)layers.size();
  int sI = static_cast<int>(s);

  lua_createtable(L, s, 0);
  int newTable = lua_gettop(L);

  for(int i = 0; i < sI; i++)
  {
	SWIG_NewPointerObj(L, layers[i], $descriptor(QgisLayerInfo*), 0);
	lua_rawseti(L, newTable, i);
  }

  return 1;
}

%typemap(out) std::vector< GeometryErrorInfo* >
{
  std::vector<GeometryErrorInfo*> errors = (std::vector<GeometryErrorInfo*>)$result;
  int size = (int)errors.size();
  lua_createtable(L, size, 0);
  int errorsTable = lua_gettop(L);

  for(int i = 0; i < size; i++)
  {
    SWIG_NewPointerObj(L, (void*) errors.at(i), $descriptor(GeometryErrorInfo*), 1);
    SWIG_arg++;
    lua_rawseti(L, errorsTable, i);
  }

  return 1;
}

%typemap(out) double&
{
  lua_pushnumber(L, (lua_Number) *$1); SWIG_arg++;
}

%typemap(out) DataSetTable
{
  size_t numProps = arg1->getNumProperties();
  unsigned int count = 0;
  int s = (int)arg1->size();
  lua_createtable(L, s, 0);
  int newTable = lua_gettop(L);

  while(arg1->moveNext())
  {
    lua_newtable(L);
    for(unsigned int i = 0; i < numProps; i++)
    {
      if(arg1->isNull(i))
      {
        if(arg1->getPropertyDataType(i) == te::dt::GEOMETRY_TYPE)
        {
		  return 0;
		}
        else if(!arg2.empty())
        {
          lua_pushstring(L, arg1->getPropertyName(i).c_str());
          lua_pushnumber(L, std::stod(arg2));
        }
        else
        {
          return 0;
        }
      }
      else
      {
        switch(arg1->getPropertyDataType(i))
        {
          case te::dt::INT16_TYPE:
            lua_pushstring(L, arg1->getPropertyName(i).c_str());
            lua_pushinteger(L, boost::numeric_cast<int>(arg1->getInt16(i)));
		    break;
          case te::dt::UINT16_TYPE:
            lua_pushstring(L, arg1->getPropertyName(i).c_str());
            lua_pushinteger(L, boost::numeric_cast<unsigned int>(arg1->getInt16(i)));
		    break;
          case te::dt::INT32_TYPE:
            lua_pushstring(L, arg1->getPropertyName(i).c_str());
            lua_pushinteger(L, boost::numeric_cast<int>(arg1->getInt32(i)));
		    break;
          case te::dt::UINT32_TYPE:
            lua_pushstring(L, arg1->getPropertyName(i).c_str());
            lua_pushinteger(L, boost::numeric_cast<unsigned int>(arg1->getInt32(i)));
		    break;
          case te::dt::INT64_TYPE:
            lua_pushstring(L, arg1->getPropertyName(i).c_str());
            lua_pushinteger(L, boost::numeric_cast<int>(arg1->getInt64(i)));
            break;
          case te::dt::UINT64_TYPE:
            lua_pushstring(L, arg1->getPropertyName(i).c_str());
            lua_pushinteger(L, boost::numeric_cast<unsigned int>(arg1->getInt64(i)));
            break;
		  case te::dt::CHAR_TYPE:
		  case te::dt::UCHAR_TYPE:
          case te::dt::STRING_TYPE:
          case te::dt::NUMERIC_TYPE:
            lua_pushstring(L, arg1->getPropertyName(i).c_str());
            lua_pushstring(L, arg1->getAsString(i).c_str());
            break;
          case te::dt::BOOLEAN_TYPE:
            lua_pushstring(L, arg1->getPropertyName(i).c_str());
            lua_pushboolean(L, arg1->getBool(i));
            break;
          case te::dt::FLOAT_TYPE:
		    lua_pushstring(L, arg1->getPropertyName(i).c_str());
            lua_pushnumber(L, arg1->getFloat(i));
            break;
          case te::dt::DOUBLE_TYPE:
            lua_pushstring(L, arg1->getPropertyName(i).c_str());
            lua_pushnumber(L, arg1->getDouble(i));
            break;
          case te::dt::GEOMETRY_TYPE:
            lua_pushstring(L, arg1->getPropertyName(i).c_str());
            SWIG_NewPointerObj(L, arg1->getGeometry(i).release(), $descriptor(te::gm::Geometry*), 0);
            break;
          default:
            lua_pushstring(L, arg1->getPropertyName(i).c_str());
            lua_pushstring(L, arg1->getAsString(i).c_str());
            break;
        }
      }
      lua_settable(L, -3);
    }
    lua_rawseti(L, newTable, count);
    count++;
  }

  return 1;
}

%typemap(out) RasterTable
{
  int rpos = (int)te::da::GetFirstPropertyPos(arg1, te::dt::RASTER_TYPE);
  std::unique_ptr<te::rst::Raster> raster = arg1->getRaster(rpos);
  int rows = raster->getNumberOfRows();
  int columns = raster->getNumberOfColumns();
  int bands = (int)raster->getNumberOfBands();
  int s = rows * columns;
  lua_createtable(L, s, 0);
  int newTable = lua_gettop(L);
  int count = 0;
  std::vector<std::string> bandNames;

  for(int b = 0; b < bands; b++)
  {
    bandNames.push_back(std::string("b") + std::to_string(b));
  }

  for(int r = 0; r < rows; r++)
  {
    for(int c = 0; c < columns; c++)
    {
	  lua_newtable(L);
	  for(int b = 0; b < bands; b++)
	  {
	    lua_pushstring(L, bandNames.at(b).c_str());
	    double value;
	    raster->getValue(c, r, value, b);
	    lua_pushnumber(L, value);
	    lua_settable(L, -3);
	  }
	  lua_pushstring(L, "row");
	  lua_pushinteger(L, r);
	  lua_settable(L, -3);
	  lua_pushstring(L, "col");
	  lua_pushinteger(L, c);
	  lua_settable(L, -3);
	  lua_rawseti(L, newTable, count);
	  count++;
    }
  }

  return 1;
}
