#define TERPEXPORT
#define TECOMMONEXPORT
#define TERASTEREXPORT

%include Config.i
%module(package="terralib") rp

%import Common.i

%feature("flatnested");
%feature("kwargs");

// C++ includes and declarations
%{
#include "terralib/rp/AbstractFeeder.h"
#include "terralib/rp/Algorithm.h"
#include "terralib/rp/AlgorithmInputParameters.h"
#include "terralib/rp/AlgorithmOutputParameters.h"
#include "terralib/rp/ArithmeticOperations.h"
#include "terralib/rp/Blender.h"
#include "terralib/rp/ClassifierDummyStrategy.h"
#include "terralib/rp/ClassifierEMStrategy.h"
#include "terralib/rp/Classifier.h"
#include "terralib/rp/ClassifierISOSegStrategy.h"
#include "terralib/rp/ClassifierKMeansStrategy.h"
#include "terralib/rp/ClassifierMAPStrategy.h"
#include "terralib/rp/ClassifierEDStrategy.h"
#include "terralib/rp/ClassifierSAMStrategy.h"
#include "terralib/rp/ClassifierStrategyFactory.h"
#include "terralib/rp/ClassifierStrategyParameters.h"
#include "terralib/rp/ClassifierStrategy.h"
#include "terralib/rp/Contrast.h"
#include "terralib/rp/FeedersRaster.h"
#include "terralib/rp/Filter.h"
#include "terralib/rp/Functions.h"
#include "terralib/rp/GeoMosaic.h"
#include "terralib/rp/IHSFusion.h"
#include "terralib/rp/Matrix.h"
#include "terralib/rp/MixtureModel.h"
#include "terralib/rp/MixtureModelLinearStrategy.h"
#include "terralib/rp/MixtureModelPCAStrategy.h"
#include "terralib/rp/MixtureModelStrategyFactory.h"
#include "terralib/rp/MixtureModelStrategy.h"
#include "terralib/rp/Module.h"
#include "terralib/rp/PCAFusion.h"
#include "terralib/rp/RasterAttributes.h"
#include "terralib/rp/RasterHandler.h"
#include "terralib/rp/Register.h"
#include "terralib/rp/SegmenterDummyStrategy.h"
#include "terralib/rp/Segmenter.h"
#include "terralib/rp/SegmenterIdsManager.h"
#include "terralib/rp/SegmenterRegionGrowingSegment.h"
#include "terralib/rp/SegmenterRegionGrowingSegmentsPool.h"
#include "terralib/rp/SegmenterRegionGrowingBaatzStrategy.h"
#include "terralib/rp/SegmenterRegionGrowingMeanStrategy.h"
#include "terralib/rp/SegmenterSegmentsBlock.h"
#include "terralib/rp/SegmenterStrategyFactory.h"
#include "terralib/rp/SegmenterStrategy.h"
#include "terralib/rp/SegmenterStrategyParameters.h"
#include "terralib/rp/SequenceMosaic.h"
#include "terralib/rp/Skeleton.h"
#include "terralib/rp/SpectralResponseFunctions.h"
#include "terralib/rp/StrategyParameters.h"
#include "terralib/rp/Texture.h"
#include "terralib/rp/TiePointsLocator.h"
#include "terralib/rp/TiePointsLocatorStrategyParameters.h"
#include "terralib/rp/TiePointsLocatorInputParameters.h"
#include "terralib/rp/TiePointsLocatorMoravecStrategy.h"
#include "terralib/rp/TiePointsLocatorStrategy.h"
#include "terralib/rp/TiePointsLocatorSURFStrategy.h"
#include "terralib/rp/TiePointsLocatorStrategyFactory.h"
#include "terralib/rp/TiePointsMosaic.h"
#include "terralib/rp/WisperFusion.h"
#include "terralib/rp/radar/RadarFunctions.h"
#include "terralib/raster/Raster.h"
#include "terralib/raster/BandProperty.h"
#include "terralib/common/progress/ConsoleProgressViewer.h"
#include "terralib/common/progress/ProgressManager.h"
#include "terralib/core/uri/URI.h"
#include "terralib/core/uri/Utils.h"

using namespace te::rp;
using namespace te::common;
typedef te::rp::SegmenterStrategy::BlocksMergingMethod BlocksMergingMethod;
%}

//typemap fixes
%typemap(in) const SegmenterStrategyParameters&(void * temp = 0)
{
  SWIG_ConvertPtr($input, &temp, $descriptor(te::rp::SegmenterStrategyParameter *), 0 |  0);
  $1 = (te::rp::SegmenterStrategyParameters * )temp;
}

%include <std_complex.i>

//STL Mapping
%template() std::vector< const te::rst::Raster *>;
%template() std::vector<double>;
%template() std::map< std::string, std::vector<double> >;
%template() std::vector< std::complex< double > >;

//implementation of auxilary function
%inline{
void ComposeBands(std::vector< const te::rst::Raster * > rasters,
    const std::vector< unsigned int >& inputRasterBands,
    te::rst::InterpolationMethod interpMethod,
    const std::string& outputPath)
{
  std::unique_ptr<te::rst::Raster> raster;
  std::map<std::string, std::string> rInfo;
  rInfo["URI"] = outputPath;
  te::rp::ComposeBands(rasters, inputRasterBands, interpMethod, rInfo, "GDAL", 0, false, false, raster);
}

void ComposeBandsFromURI(std::vector< const te::rst::Raster * > rasters,
    const std::vector< unsigned int >& inputRasterBands,
    te::rst::InterpolationMethod interpMethod,
    const te::core::URI& uri)
{
  ComposeBands(rasters,
               inputRasterBands,
               interpMethod,
               uri.path());
}

void SimpleComposeBandsFromURI(std::vector< const te::rst::Raster * > rasters, const te::core::URI& uri)
{
  ComposeBands(rasters, {0,0}, te::rst::InterpolationMethod::NearestNeighbor, uri.path());
}

void Threshold(const te::rst::Raster* rst, int band, double min, double max,
            const std::string& outPath)
{
  const te::rst::Band* _band = rst->getBand(band);
  te::rst::BandProperty* _bp = new te::rst::BandProperty(*_band->getProperty());

  std::vector <  te::rst::BandProperty* > _bprops;
  _bprops.push_back(_bp);

  std::map< std::string, std::string > rInfo;
  rInfo["URI"] = outPath;
  te::rst::Raster* _out = te::rst::RasterFactory::make(
      "GDAL", new te::rst::Grid(*(rst->getGrid())), _bprops, rInfo, 0, 0);

  const unsigned int nRows = rst->getNumberOfRows();
  const unsigned int nCols = rst->getNumberOfColumns();

  for (unsigned int row = 0; row < nRows; ++row)
  {
    for (unsigned int col = 0; col < nCols; ++col)
    {
      double val;
      _band->getValue(col, row, val);
      if (val >= min && val <= max)
        _out->setValue(col, row, 0);
      else
        _out->setValue(col, row, 255);
     }
  }
  delete _out;
}

void ThresholdFromURI(const te::rst::Raster* rst, int band, double min, double max,
            const te::core::URI& uri)
{
  Threshold(rst, band, min, max, uri.path());
}

void SimpleThresholdFromURI(const te::rst::Raster* rst, const te::core::URI& uri)
{
  Threshold(rst, 1, 1, 50, uri.path());
}

void SegmenterRegionGrowingBaatz(const te::rst::Raster* rst,
  std::vector<unsigned int> bands, const std::string& output,
  unsigned int minSegmentSize = 100, double segmentsSimilarityThreshold = 0.03,
  unsigned int segmentsSimIncreaseSteps = 2, bool enableLocalMutualBestFitting = false,
  bool enableSameIterationMerges = false,
  std::vector< double > bandsWeights = std::vector<double>(),
  double colorWeight = 0.9, double compactnessWeight = 0.5, bool enableThreadedProcessing = true,
  unsigned int maxSegThreads = 0, bool enableBlockProcessing = true, unsigned int maxBlockSize = 0,
  unsigned char blocksOverlapPercent = 10, bool enableRasterCache = true)
{
  te::rp::SegmenterRegionGrowingBaatzStrategy::Parameters parameters;
  parameters.m_minSegmentSize = minSegmentSize;
  parameters.m_segmentsSimilarityThreshold = segmentsSimilarityThreshold;
  parameters.m_segmentsSimIncreaseSteps = segmentsSimIncreaseSteps;
  parameters.m_enableLocalMutualBestFitting = enableLocalMutualBestFitting;
  parameters.m_bandsWeights = bandsWeights;
  parameters.m_colorWeight = colorWeight;
  parameters.m_compactnessWeight = compactnessWeight;

  te::rp::Segmenter::InputParameters input_params;
  input_params.m_inputRasterPtr = rst;
  input_params.m_inputRasterBands = bands;
  input_params.m_strategyName = "RegionGrowingBaatz";
  input_params.m_enableThreadedProcessing = enableThreadedProcessing;
  input_params.m_maxSegThreads = maxSegThreads;
  input_params.m_enableBlockProcessing = enableBlockProcessing;
  input_params.m_maxBlockSize = maxBlockSize;
  input_params.m_blocksOverlapPercent = blocksOverlapPercent;
  input_params.m_enableRasterCache = enableRasterCache;
  input_params.setSegStrategyParams(parameters);

  std::map<std::string, std::string> rOut;
  rOut["URI"] = output;
  te::rp::Segmenter::OutputParameters output_params;
  output_params.m_rInfo = rOut;
  output_params.m_rType = "GDAL";

  te::rp::Segmenter algorithmInstance;
  algorithmInstance.initialize(input_params);
  algorithmInstance.execute(output_params);
}

void SegmenterRegionGrowingBaatzFromURI(const te::rst::Raster* rst,
  std::vector<unsigned int> bands, const te::core::URI& uri,
  unsigned int minSegmentSize = 100, double segmentsSimilarityThreshold = 0.03,
  unsigned int segmentsSimIncreaseSteps = 2, bool enableLocalMutualBestFitting = false,
  bool enableSameIterationMerges = false,
  std::vector< double > bandsWeights = std::vector<double>(),
  double colorWeight = 0.9, double compactnessWeight = 0.5, bool enableThreadedProcessing = true,
  unsigned int maxSegThreads = 0, bool enableBlockProcessing = true, unsigned int maxBlockSize = 0,
  unsigned char blocksOverlapPercent = 10, bool enableRasterCache = true)
{
  SegmenterRegionGrowingBaatz(rst,
                              bands, uri.path(),
                              minSegmentSize, segmentsSimilarityThreshold,
                              segmentsSimIncreaseSteps, enableLocalMutualBestFitting,
                              enableSameIterationMerges,
                              bandsWeights,
                              colorWeight, compactnessWeight, enableThreadedProcessing,
                              maxSegThreads, enableBlockProcessing, maxBlockSize,
                              blocksOverlapPercent, enableRasterCache);
}

void SimpleSegmenterRegionGrowingBaatzFromURI(const te::rst::Raster* rst,
  const te::core::URI& uri, unsigned int minSegmentSize = 100, double segmentsSimilarityThreshold = 0.03,
  unsigned int segmentsSimIncreaseSteps = 2, bool enableLocalMutualBestFitting = false,
  bool enableSameIterationMerges = false,
  std::vector< double > bandsWeights = std::vector<double>(),
  double colorWeight = 0.9, double compactnessWeight = 0.5, bool enableThreadedProcessing = true,
  unsigned int maxSegThreads = 0, bool enableBlockProcessing = true, unsigned int maxBlockSize = 0,
  unsigned char blocksOverlapPercent = 10, bool enableRasterCache = true)
{

  SegmenterRegionGrowingBaatz(rst,
                              {0}, uri.path(),
                              minSegmentSize, segmentsSimilarityThreshold,
                              segmentsSimIncreaseSteps, enableLocalMutualBestFitting,
                              enableSameIterationMerges,
                              bandsWeights,
                              colorWeight, compactnessWeight, enableThreadedProcessing,
                              maxSegThreads, enableBlockProcessing, maxBlockSize,
                              blocksOverlapPercent, enableRasterCache);
}

void SegmenterRegionGrowingMean(const te::rst::Raster* rst,
  std::vector<unsigned int> bands, const std::string& output,
  unsigned int minSegmentSize = 100, double segmentsSimilarityThreshold = 0.03,
  unsigned int segmentsSimIncreaseSteps = 2, bool enableLocalMutualBestFitting = false,
  bool enableSameIterationMerges = false, bool enableThreadedProcessing = true,
  unsigned int maxSegThreads = 0, bool enableBlockProcessing = true, unsigned int maxBlockSize = 0,
  unsigned char blocksOverlapPercent = 10, bool enableRasterCache = true)
{
  te::rp::SegmenterRegionGrowingMeanStrategy::Parameters parameters;
  parameters.m_minSegmentSize = minSegmentSize;
  parameters.m_segmentsSimilarityThreshold = segmentsSimilarityThreshold;
  parameters.m_segmentsSimIncreaseSteps = segmentsSimIncreaseSteps;
  parameters.m_enableLocalMutualBestFitting = enableLocalMutualBestFitting;

  te::rp::Segmenter::InputParameters input_params;
  input_params.m_inputRasterPtr = rst;
  input_params.m_inputRasterBands = bands;
  input_params.m_strategyName = "RegionGrowingMean";
  input_params.m_enableThreadedProcessing = enableThreadedProcessing;
  input_params.m_maxSegThreads = maxSegThreads;
  input_params.m_enableBlockProcessing = enableBlockProcessing;
  input_params.m_maxBlockSize = maxBlockSize;
  input_params.m_blocksOverlapPercent = blocksOverlapPercent;
  input_params.m_enableRasterCache = enableRasterCache;
  input_params.setSegStrategyParams(parameters);

  std::map<std::string, std::string> rOut;
  rOut["URI"] = output;
  te::rp::Segmenter::OutputParameters output_params;
  output_params.m_rInfo = rOut;
  output_params.m_rType = "GDAL";

  te::rp::Segmenter algorithmInstance;
  algorithmInstance.initialize(input_params);
  algorithmInstance.execute(output_params);
}

void SimpleSegmenterRegionGrowingMeanFromURI(const te::rst::Raster* rst,
  const te::core::URI& uri, unsigned int minSegmentSize = 100, double segmentsSimilarityThreshold = 0.03,
  unsigned int segmentsSimIncreaseSteps = 2, bool enableLocalMutualBestFitting = false,
  bool enableSameIterationMerges = false, bool enableThreadedProcessing = true,
  unsigned int maxSegThreads = 0, bool enableBlockProcessing = true, unsigned int maxBlockSize = 0,
  unsigned char blocksOverlapPercent = 10, bool enableRasterCache = true)
{
  SegmenterRegionGrowingMean(rst,
                             {0}, uri.path(),
                             minSegmentSize, segmentsSimilarityThreshold,
                             segmentsSimIncreaseSteps, enableLocalMutualBestFitting,
                             enableSameIterationMerges, enableThreadedProcessing,
                             maxSegThreads, enableBlockProcessing, maxBlockSize,
                             blocksOverlapPercent, enableRasterCache);
}

void SegmenterRegionGrowingMeanFromURI(const te::rst::Raster* rst,
  std::vector<unsigned int> bands, const te::core::URI& uri,
  unsigned int minSegmentSize = 100, double segmentsSimilarityThreshold = 0.03,
  unsigned int segmentsSimIncreaseSteps = 2, bool enableLocalMutualBestFitting = false,
  bool enableSameIterationMerges = false, bool enableThreadedProcessing = true,
  unsigned int maxSegThreads = 0, bool enableBlockProcessing = true, unsigned int maxBlockSize = 0,
  unsigned char blocksOverlapPercent = 10, bool enableRasterCache = true)
{
  SegmenterRegionGrowingMean(rst,
                             bands, uri.path(),
                             minSegmentSize, segmentsSimilarityThreshold,
                             segmentsSimIncreaseSteps, enableLocalMutualBestFitting,
                             enableSameIterationMerges, enableThreadedProcessing,
                             maxSegThreads, enableBlockProcessing, maxBlockSize,
                             blocksOverlapPercent, enableRasterCache);
}

void MixtureModelLinear(const te::rst::Raster* rst, const std::string& output,
  std::vector<unsigned int> bands,
  std::map< std::string, std::vector<double> > components, bool createErrorRaster, bool normalize,
  std::vector<std::string> inputSensorBands = std::vector<std::string>())
{
  te::rp::MixtureModelLinearStrategy::Parameters parameters;

  te::rp::MixtureModel::InputParameters input_params;
  input_params.m_inputRasterPtr = rst;
  input_params.m_inputRasterBands = bands;
  input_params.m_components = components;
  input_params.m_inputSensorBands = inputSensorBands;
  input_params.m_strategyName = "linear";
  input_params.setMixtureModelStrategyParams(parameters);

  std::map<std::string, std::string> rOut;
  rOut["URI"] = output;
  te::rp::MixtureModel::OutputParameters output_params;
  output_params.m_rInfo = rOut;
  output_params.m_rType = "GDAL";
  output_params.m_createErrorRaster = createErrorRaster;
  output_params.m_normalizeOutput = normalize;
  output_params.m_normalMin = 0;
  output_params.m_normalMax = 255;

  te::rp::MixtureModel algorithmInstance;
  algorithmInstance.initialize(input_params);
  algorithmInstance.execute(output_params);
}

void MixtureModelLinearFromURI(const te::rst::Raster* rst, const te::core::URI& uri,
  std::vector<unsigned int> bands,
  std::map< std::string, std::vector<double> > components, bool createErrorRaster, bool normalize,
  std::vector<std::string> inputSensorBands = std::vector<std::string>())
{
  MixtureModelLinear(rst, uri.path(),
                     bands,
                     components, createErrorRaster, normalize,
                     inputSensorBands);
}

void SimpleMixtureModelLinearFromURI(const te::rst::Raster* rst, const te::core::URI& uri,
  std::vector<unsigned int> bands,
  std::map< std::string, std::vector<double> > components)
{
  std::vector<std::string> inputSensorBands = {"AWIFS_GREEN","AWIFS_GREEN","AWIFS_GREEN"};

  MixtureModelLinear(rst, uri.path(),
                     bands,
                     components,
                     false,
                     false,
                     inputSensorBands);
}

void MixtureModelPCA(const te::rst::Raster* rst, const std::string& output,
  std::vector<unsigned int> bands,
  std::map< std::string, std::vector<double> > components, bool createErrorRaster,
  bool normalizeOutput,
  std::vector<std::string> inputSensorBands = std::vector<std::string>())
{
  te::rp::MixtureModelPCAStrategy::Parameters specificParameters;
  te::rp::MixtureModel::InputParameters input_params;

  input_params.m_strategyName = "pca";
  input_params.setMixtureModelStrategyParams(specificParameters);
  input_params.m_inputRasterBands = bands;
  input_params.m_inputSensorBands = inputSensorBands;
  input_params.m_components = components;

  input_params.m_inputRasterPtr = rst;

  std::map<std::string, std::string> rOut;
  rOut["URI"] = output;

  te::rp::MixtureModel::OutputParameters output_params;

  output_params.m_createErrorRaster = createErrorRaster;
  output_params.m_normalizeOutput = normalizeOutput;
  output_params.m_normalMin = 0;
  output_params.m_normalMax = 255;
  output_params.m_rInfo = rOut;
  output_params.m_rType = "GDAL";

  te::rp::MixtureModel algorithmInstance;

  algorithmInstance.initialize(input_params);
  algorithmInstance.execute(output_params);
}

void MixtureModelPCAFromURI(const te::rst::Raster* rst, const te::core::URI& uri,
  std::vector<unsigned int> bands,
  std::map< std::string, std::vector<double> > components, bool createErrorRaster,
  bool normalizeOutput,
  std::vector<std::string> inputSensorBands = std::vector<std::string>())
{
  MixtureModelPCA(rst, uri.path(),
                  bands,
                  components, createErrorRaster,
                  normalizeOutput,
                  inputSensorBands);
}

void SimpleMixtureModelPCAFromURI(const te::rst::Raster* rst, const te::core::URI& uri,
  std::vector<unsigned int> bands,
  std::map< std::string, std::vector<double> > components)
{
  std::vector<std::string> inputSensorBands = {"AWIFS_GREEN","AWIFS_GREEN","AWIFS_GREEN"};

  MixtureModelPCA(rst,
                  uri.path(),
                  bands,
                  components,
                  false,
                  false,
                  inputSensorBands);
}

}

//includes of classes to be exposed and import for let SWIG know about the given
//header without exposing to the wrapper
// NOTE: includes cannot be alphabetically ordered, because base classes have to
// be include first
%import "terralib/raster/Raster.h"
%import "terralib/raster/Enums.h"
