/*
 * Defining a typemap for std::auto_ptr and std::unique_ptr
 */

/*
    The typemaps here allow to handle functions returning std::auto_ptr<>,
    which is the most common use of this type. If you have functions taking it
    as parameter, these typemaps can't be used for them and you need to do
    something else (e.g. use shared_ptr<> which SWIG supports fully).
 */

%define %unique_ptr(TYPE)
%typemap (out) std::unique_ptr<TYPE > %{
   %set_output(SWIG_NewPointerObj($1.release(), $descriptor(TYPE *), SWIG_POINTER_OWN | %newpointer_flags));
%}
%template() std::unique_ptr<TYPE >;
%enddef

%define %weak_ptr(TYPE)
%typemap (out) std::weak_ptr<TYPE > %{
   %set_output(SWIG_NewPointerObj($1.release(), $descriptor(TYPE *), SWIG_POINTER_OWN | %newpointer_flags));
%}
%template() std::weak_ptr<TYPE >;
%enddef

%define %unique_member(TYPE)
%typemap (memberin) std::unique_ptr<TYPE>%{
	$1.reset($input->release());
%}
%enddef

%define %auto_ptr(TYPE)
%typemap (out) std::auto_ptr<TYPE > %{
   %set_output(SWIG_NewPointerObj(%as_voidptr($1.release()), $descriptor(TYPE *), SWIG_POINTER_OWN | %newpointer_flags));
%}
%template() std::auto_ptr<TYPE >;
%enddef

namespace std {
   template <class T> class unique_ptr {};
   template <class T> class auto_ptr {};
   template <class T> class weak_ptr {};
}

%template() std::vector<std::string>;
%template() std::vector<unsigned int>;
%template() std::map<std::string, std::string>;
