/*
  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

  This file is part of the TerraLib - a Framework for building GIS enabled
  applications.

  TerraLib is free software: you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation, either version 3 of the License,
  or (at your option) any later version.

  TerraLib is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with TerraLib. See COPYING. If not, write to
  TerraLib Team at <terralib-team@terralib.org>.
*/
#ifndef __TERRALIB_CEDITOR_CODEEDITORDIALOG_H__
#define __TERRALIB_CEDITOR_CODEEDITORDIALOG_H__

// Terralib
#include "../qt/widgets/ceditor/IOWidget.h"
#include "../vm/core/VirtualMachine.h"
#include "Config.h"

// STL
#include <memory>

// Qt
#include <QDialog>
#include <QFutureWatcher>
#include <QTime>

namespace Ui
{
  class CodeEditorDialogForm;
}

namespace te
{
  namespace ce
  {
    class TECEDITOREXPORT CodeEditorDialog : public QDialog
    {
      Q_OBJECT
     public:
      CodeEditorDialog(QWidget* parent = 0, Qt::WindowFlags f = 0);
      ~CodeEditorDialog();

     protected slots:
      void onSaveButtonClicked();
      void onSaveAsButtonClicked();
      void onOpenButtonClicked();
      void onNewButtonClicked();
      void onRunButtonClicked();
      void onCloseTabClicked(int index);
      void onRunFinished();
      void onTextChanged();
      void onCancelPushButtonClicked();

     private:
      void save(bool saveAs);

     private:
      QTime m_runtime;
      QFutureWatcher<void> m_watcher;
      te::qt::widgets::IOWidget* m_io;
      te::vm::core::VirtualMachine* m_vm;
      std::unique_ptr<Ui::CodeEditorDialogForm> m_ui;
    };
  }
}
#endif  //__TERRALIB_CEDITOR_CODEEDITORDIALOG_H__
