/*
  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

  This file is part of the TerraLib - a Framework for building GIS enabled
  applications.

  TerraLib is free software: you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation, either version 3 of the License,
  or (at your option) any later version.

  TerraLib is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with TerraLib. See COPYING. If not, write to
  TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/ceditor/Plugin.cpp

  \brief Plugin interface for dynamic loading of the Code Editor.
*/
// TerraLib
#include "../core/plugin/CppPlugin.h"
#include "../qt/af/ApplicationController.h"
#include "CodeEditorDialog.h"
#include "Plugin.h"

//qt
#include <QAction>
#include <QIcon>
#include <QMenu>

te::ce::Plugin::Plugin(const te::core::PluginInfo &pluginInfo)
    : te::core::CppPlugin(pluginInfo),
      m_codeEditorAction(nullptr),
      m_codeEditorDialog(nullptr)
{
}
te::ce::Plugin::~Plugin() = default;

void te::ce::Plugin::startup()
{
  // if initialized, return
  if (m_initialized)
    return;

  // creates a new QAction for the plugin
  m_codeEditorAction = new QAction(this);
  m_codeEditorAction->setText("Code Editor");
  m_codeEditorAction->setIcon(QIcon::fromTheme("terminal"));
  m_codeEditorAction->setObjectName("CEditor");

  // inserts the action into the Plugins menu
  te::qt::af::AppCtrlSingleton::getInstance().getMenu("Plugins")->addAction(
      m_codeEditorAction);

  // connects the action to the slot
  connect(m_codeEditorAction, SIGNAL(triggered(bool)), this,
          SLOT(onCodeEditorActionClicked(bool)));
  
  // initialized is now true
  m_initialized = true;
}

void te::ce::Plugin::shutdown()
{
  if (!m_initialized)
    return;

  delete m_codeEditorDialog;
  delete m_codeEditorAction;

  m_initialized = false;
}

void te::ce::Plugin::onCodeEditorActionClicked(bool)
{
  // creates a new CodeEditorDialog when the actions is triggered
  m_codeEditorDialog = new te::ce::CodeEditorDialog(
      te::qt::af::AppCtrlSingleton::getInstance().getMainWindow());

  // shows the CodeEditorDialog
  m_codeEditorDialog->show();
}

TERRALIB_PLUGIN_CALL_BACK_IMPL(te::ce::Plugin)
