/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/color/Utils.h

  \brief This file contains utility functions for custom color bar.
*/

#ifndef __TERRALIB_COLOR_INTERNAL_UTILS_H
#define __TERRALIB_COLOR_INTERNAL_UTILS_H

// TerraLib
#include "Config.h"
#include "ColorSchemeCatalog.h"
#include "RGBAColor.h"

// STL
#include <vector>

namespace te
{
  namespace color
  {
    /*!
      \brief This file contains utility functions for color bar.

      \ingroup color
    */

        /*!
       \brief 
       This methods reads the JSON file for color schemes definitions.
       \exception te::srs::Exception if the JSON file is not well formed.
       */
    TECOLOREXPORT te::color::ColorSchemeCatalog* loadColorBar(const std::string& jsonfile, const std::string& catalogName);
        
        /*! \brief This method save the custom color bar in a JSON file. */
    TECOLOREXPORT bool saveCustomColorBar(const std::string& groupName, const std::string& schemeName, const std::vector<te::color::RGBAColor> rgbaVec);

    TECOLOREXPORT te::color::ColorSchemeCatalog* GetSchemeCatalog(std::string catalogName);

  }   // end namespace color
}     // end namespace te

#endif  // __TERRALIB_COLOR_INTERNAL_UTILS_H

