/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/common/Module.cpp

  \brief This singleton defines an entry in the Platform for the TerraLib Common Runtime module.
*/

// TerraLib
#include "Enums.h"
#include "../core/lib/LibraryManager.h"
#include "../core/logger/Logger.h"
#include "Module.h"
#include "PlatformUtils.h"
#include "TerraLib.h"
#include "../core/translator/Translator.h"
#include "UnitsOfMeasureManager.h"

// STL
#include <cstdlib>
#include <ctime>

#ifdef TERRALIB_LEAK_DETECTOR_ENABLED
  #include <vld.h>
#endif


const te::common::Module& sm_module = te::common::Module::getInstance();

te::common::Module::Module()
{
  m_module.reset(new TerraLib::Module(TE_COMMON_MODULE_NAME,
                         te::common::Module::initialize,
                         te::common::Module::finalize
                       ));

// initialize TerraLib singleton
  TerraLib::getInstance().add(m_module);

// let's start all singletons in the right order and other stuffs that must be in the static initialization!

// initialize the translator singleton and the common runtime multilanguage support
//  TE_ADD_TEXT_DOMAIN("terralib_mod_common");

// initialize the singleton LibraryManager
//  LibraryManager::getInstance();

// initialize the singleton UnitsOfMeasureManager
  UnitsOfMeasureManager::getInstance();
}

te::common::Module::~Module() = default;

void te::common::Module::initialize()
{
  /* initialize random seed to be used by std::rand calls */
  std::srand(static_cast<unsigned int>(std::time(nullptr)));

#ifdef TERRALIB_AUTOMATIC_INITIALIZATION
  UnitsOfMeasureManager::getInstance().init();
#endif

  TE_LOG_TRACE(TE_TR("TerraLib Common Runtime initialized!"));
}

void te::common::Module::finalize()
{
  TE_LOG_TRACE(TE_TR("TerraLib Common Runtime finalized!"));

#ifdef TERRALIB_AUTOMATIC_INITIALIZATION
  UnitsOfMeasureManager::getInstance().clear();
#endif

  TerraLib::getInstance().remove(TE_COMMON_MODULE_NAME);
}


