/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/common/StringUtils.cpp

  \brief Utility functions for dealing with strings.

  \ingroup common
*/

#include "StringUtils.h"

#include <sstream>
#include <boost/algorithm/string/predicate.hpp>
#include <boost/uuid/random_generator.hpp>
#include <boost/uuid/uuid_io.hpp>

std::vector<std::string> te::common::SplitString(const std::string& str, const char& delimiter)
{
  std::stringstream ss (str);
  std::string item;

  std::vector<std::string> elements;

  while (std::getline(ss, item, delimiter))
  {
    elements.push_back(item);
  }

  return elements;
}

std::string te::common::GetUniqueID()
{
  static boost::uuids::basic_random_generator<boost::mt19937> gen;
  boost::uuids::uuid u = gen();
  std::string id = boost::uuids::to_string(u);

  return id;
}

bool te::common::IsStringEqual(const std::string& str1, const std::string& str2, bool caseSensitive)
{
  if (caseSensitive == true)
  {
    return boost::equals(str1, str2);
  }
  else
  {
    return boost::iequals(str1, str2);
  }
}

std::string te::common::Convert2String(const std::vector<std::string>& vecStrings, const std::string& separator)
{
  std::string result;
  if (vecStrings.empty() == true)
  {
    return result;
  }

  result += vecStrings.at(0);
  for (std::size_t i = 1; i < vecStrings.size(); ++i)
  {
    result += separator + vecStrings.at(i);
  }

  return result;
}
