/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

 /*!
   \file terralib/common/ThreadGroup.cpp

   \brief This class represents a thread group

   \ingroup common
 */

#include "ThreadGroup.h"

//boost include files
#include <boost/bind/bind.hpp>

te::common::ThreadGroup::ThreadGroup(std::size_t threadCount)
  : m_threadCount(threadCount)
  , m_ioService(new boost::asio::io_service())
  , m_threadGroup(new boost::thread_group())
{
  m_work.reset(new boost::asio::io_service::work(*m_ioService.get()));

  //the we add the threads to the pool
  if (m_threadCount == 0)
  {
    m_threadCount = static_cast<std::size_t>(boost::thread::hardware_concurrency());
  }
}

te::common::ThreadGroup::~ThreadGroup()
{
  waitToFinish();
}

std::size_t te::common::ThreadGroup::getThreadCount()
{
  return m_threadCount;
}

void te::common::ThreadGroup::waitToFinish()
{
  if (m_ioService->stopped() == true)
  {
    return;
  }

  //we MUST add the threads AFTER all the jobs are added
  //this ensures that all of them will be executed
  run();

  //wait for the jobs to be finished
  m_work.reset();
  m_threadGroup->join_all();
}

void te::common::ThreadGroup::run()
{
  for (std::size_t i = 0; i < m_threadCount; ++i)
  {
    m_threadGroup->create_thread(boost::bind(&boost::asio::io_service::run, m_ioService.get()));
  }
}

