/*
  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

  This file is part of the TerraLib - a Framework for building GIS enabled
  applications.

  TerraLib is free software: you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation, either version 3 of the License,
  or (at your option) any later version.

  TerraLib is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with TerraLib. See COPYING. If not, write to
  TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/core/filesystem/FileWatcher.h

  \brief This class is designed to watch a file.

  \author Matheus Cavassan Zaglia
*/

#ifndef __TERRALIB_CORE_FILESYSTEM_FILEWATCHER_H__
#define __TERRALIB_CORE_FILESYSTEM_FILEWATCHER_H__

// TerraLib
#include "../Config.h"

// STL
#include <string>
#include <functional>
#include <future>
#include <thread>

namespace te
{
  namespace core
  {
    class TECOREEXPORT FileWatcher
    {
      /*!
        \class FileWatcher

        \brief This class is designed to watch a file.
       */
     public:
      /*!
        \brief It initializes a new FileWatcher from a given path and callback
        function

        \param path A UTF-8 String with a path to a file.

        \param callback A function with zero parameters and void return that
        will be called when the file is changed.
       */
      FileWatcher(const std::string& path,
                  const std::function<void()>& callback);

      /*!
       \brief Destructor.
      */
      ~FileWatcher();

      /*!
       \brief Starts a thread that will monitor the given file and call the
       previously given callback function.
      */
      void watch();
      /*!
       \brief Stops the thread watching the file.
      */
      void unwatch();

     private:
      std::mutex m_mutex;
      std::string m_file;
      std::function<void()> m_callback;
      std::thread m_watcher;
      bool m_running;
    };
  }  // end namespace core
}  // end namespace te
#endif  //__TERRALIB_CORE_FILESYSTEM_FILEWATCHER_H__
