#include "Proxy.h"

#include "ProxyAuthenticationHandler.h"

void te::core::Proxy::setProxyInfo(const te::core::ProxyInfo & proxyInfo)
{
  m_proxyInfo = proxyInfo;
}

const te::core::ProxyInfo& te::core::Proxy::getProxyInfo() const
{
  return m_proxyInfo;
}

const te::core::ProxyInfo& te::core::Proxy::getAuthenticatedProxyInfo() const
{
  if (m_proxyInfo.isEnabled() == false)
  {
    return m_proxyInfo;
  }

  if (m_proxyInfo.isAuthenticated() == false && m_proxyAuthenticationHandler != nullptr)
  {
    std::string userName;
    std::string password;
    if (m_proxyAuthenticationHandler->getAuthentication(userName, password) == true)
    {
      m_proxyInfo.setUserName(userName);
      m_proxyInfo.setPassword(password);
    }
  }
  return m_proxyInfo;
}

void te::core::Proxy::setAuthenticationHandler(ProxyAuthenticationHandler* proxyAuthenticationHandler)
{
  if (m_proxyAuthenticationHandler != nullptr)
  {
    delete m_proxyAuthenticationHandler;
  }
  m_proxyAuthenticationHandler = proxyAuthenticationHandler;
}

void te::core::Proxy::resetAuthentication()
{
  m_proxyInfo.setUserName("");
  m_proxyInfo.setPassword("");
}

te::core::Proxy& te::core::Proxy::instance()
{
  static te::core::Proxy proxy;
  return proxy;
}

te::core::Proxy::Proxy()
  : m_proxyAuthenticationHandler(nullptr)
{
}

te::core::Proxy::~Proxy()
{
  delete m_proxyAuthenticationHandler;
}
