/*
  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

  This file is part of the TerraLib - a Framework for building GIS enabled
  applications.

  TerraLib is free software: you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation, either version 3 of the License,
  or (at your option) any later version.

  TerraLib is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with TerraLib. See COPYING. If not, write to
  TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/core/uri/Proxy.h

  \brief A singleton class to hold the ProxyInfo class.

  \author Mario Rocco Pettinati
 */

#ifndef __TERRALIB_CORE_URI_PROXY_H__
#define __TERRALIB_CORE_URI_PROXY_H__

// TerraLib
#include "../Config.h"

#include "ProxyInfo.h"

// STL
#include <string>

namespace te
{
  namespace core
  {
    class ProxyAuthenticationHandler;

    /*!
      \class URI

      \brief A class to store the proxy information that must be used to access
      data located in URIs.
     */
    class TECOREEXPORT Proxy
    {
    public:

      /*! \brief Sets the proxy info to be used. */
      void setProxyInfo(const te::core::ProxyInfo& proxyInfo);

      /*! \brief Gets the current proxy info. */
      const te::core::ProxyInfo& getProxyInfo() const;

      /*! \brief Gets the current proxy info. */
      const te::core::ProxyInfo& getAuthenticatedProxyInfo() const;

      /*! \brief Sets the authentication handler to be used by proxy. */
      void setAuthenticationHandler(ProxyAuthenticationHandler* proxyAuthenticationHandler);

      /*! \brief Resets the authentication information (userName and password). */
      void resetAuthentication();

      /*!
          \brief It returns a reference to the singleton instance.

          \return A reference to the singleton instance.
         */
      static Proxy& instance();

    protected:

      /*! \brief Default constructor. */
      Proxy();

      /*! Default destructor. */
      ~Proxy();

      /*! \brief Copy constructor. */
      Proxy(const Proxy& other) = delete;

      /*! \brief Move constructor. */
      Proxy(const Proxy&& other) = delete;

    protected:

      te::core::ProxyAuthenticationHandler* m_proxyAuthenticationHandler;
      mutable te::core::ProxyInfo m_proxyInfo;

    };

  }  // end namespace core
}  // end namespace te

#endif  // __TERRALIB_CORE_URI_PROXY_H__
