/*
  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

  This file is part of the TerraLib - a Framework for building GIS enabled
  applications.

  TerraLib is free software: you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation, either version 3 of the License,
  or (at your option) any later version.

  TerraLib is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with TerraLib. See COPYING. If not, write to
  TerraLib Team at <terralib-team@terralib.org>.
 */

 /*!
   \file terralib/core/uri/ProxyAuthenticationHandler.h

   \brief A class to handle the proxy authentication information.

   \author Mario Rocco Pettinati
  */

#ifndef __TERRALIB_CORE_URI_PROXYAUTHENTICATIONHANDLER_H__
#define __TERRALIB_CORE_URI_PROXYAUTHENTICATIONHANDLER_H__

  // TerraLib
#include "../Config.h"

namespace te
{
  namespace core
  {
    class ProxyInfo;

    /*!
      \class URI

      \brief A class to handle the proxy authentication information.
     */
    class TECOREEXPORT ProxyAuthenticationHandler
    {
    public:

      ProxyAuthenticationHandler() = default;

      virtual ~ProxyAuthenticationHandler() = default;

      /*! \brief Gets the current proxy authentication information. 
          \param userName (output) The user name to be used in the proxy authentication
          \param password (output) The password to be used in the proxy authentication
          \return TRUE if both parameters had been successfully retrived. FALSE otherwise
      */
      virtual bool getAuthentication(std::string& userName, std::string& password) const = 0;
    };

  }  // end namespace core
}  // end namespace te

#endif  // __TERRALIB_CORE_URI_PROXYAUTHENTICATIONHANDLER_H__
