/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/core/utils/Utils.cpp
*/

// TerraLib
#include "../uri/URI.h"
#include "../uri/Utils.h"
#include "Utils.h"

// STL
#include <cstring>
#include <memory>

// Boost
#include <boost/algorithm/string/case_conv.hpp>
#include <boost/format.hpp>
#include <boost/lexical_cast.hpp>
#include <boost/algorithm/string/replace.hpp>
#include <boost/algorithm/hex.hpp>
#include <boost/filesystem.hpp>
#include <boost/archive/iterators/binary_from_base64.hpp>
#include <boost/archive/iterators/base64_from_binary.hpp>
#include <boost/archive/iterators/transform_width.hpp>
#include <boost/algorithm/string.hpp>

#define flag "Q29weXJpZ2h0LUlOUEUtQlI"

bool isCipherURIPassword(const std::string uriPassword);

std::string te::core::ASCIIToHex(const std::string& text)
{
  std::string aSCIIToHex = boost::algorithm::hex(text);

  return aSCIIToHex;
}

std::string te::core::HexToASCII(const std::string& text)
{
  std::string hexToASCII = boost::algorithm::unhex(text);

  return hexToASCII;
}

std::string te::core::URIConvertASCIIToHex(const std::string &connInfo)
{
  te::core::URI uri(connInfo);

  if(!uri.password().empty())
  {
    if(isCipherURIPassword(uri.password()))
       return connInfo;

    std::string ASCIIToHex = te::core::ASCIIToHex(uri.password());

    std::string newConnInfo = uri.scheme() + "://" +
                              uri.user() + ":" +
                              flag + ASCIIToHex + "@" +
                              uri.host() + ":" +
                              uri.port() +
                              uri.path() + "?" +
                              uri.query();
    return newConnInfo;
  }

  return connInfo;
}

std::string te::core::URIConvertHexToASCII(const std::string &connInfo)
{
  te::core::URI uri(te::core::URIDecode(connInfo));

  if(!uri.password().empty())
  {
    if(!isCipherURIPassword(uri.password()))
       return te::core::URIDecode(connInfo);

    std::string password = uri.password();

    std::string hexToASCII = te::core::HexToASCII((password.substr(23, (password.size() - 1))));

    std::string newConnInfo = uri.scheme() + "://" +
                              uri.user() + ":" +
                              hexToASCII + "@" +
                              uri.host() + ":" +
                              uri.port() +
                              uri.path() + "?" +
                              uri.query();
    return newConnInfo;
  }

  return te::core::URIDecode(connInfo);
}

bool isCipherURIPassword(const std::string uriPassword)
{
  return uriPassword.substr(0,23) == flag;
}

std::string te::core::decodeBase64(const std::string &val)
{
  using namespace boost::archive::iterators;

  using It = transform_width<binary_from_base64<std::string::const_iterator>, 8, 6>;

  return boost::algorithm::trim_right_copy_if(std::string(It(std::begin(val)), It(std::end(val))), [](char c) {
      return c == '\0';
  });
}

std::string te::core::encodeBase64(const std::string &val)
{
  using namespace boost::archive::iterators;

  using It = base64_from_binary<transform_width<std::string::const_iterator, 6, 8>>;

  auto tmp = std::string(It(std::begin(val)), It(std::end(val)));

  return tmp.append((3 - val.size() % 3) % 3, '=');
}
