/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/dataaccess/query/GeometryType.cpp

  \brief Spatial geometry type.
*/

// TerraLib
#include "../../geometry/Geometry.h"
#include "LiteralGeom.h"
#include "LiteralBool.h"
#include "PropertyName.h"
#include "GeometryType.h"

te::da::GeometryType::GeometryType(const te::gm::Geometry& g)
  : Function("geometrytype")
{
  LiteralGeom* geom = new LiteralGeom(g);
  m_args.push_back(geom);
}

te::da::GeometryType::GeometryType(const PropertyName& prop)
  : Function("geometrytype")
{
  m_args.push_back(prop.clone());
}

te::da::GeometryType::GeometryType(const Expression& e)
  : Function("geometrytype")
{
  m_args.push_back(e.clone());
}

te::da::GeometryType::~GeometryType()
{
}

te::da::GeometryType& te::da::GeometryType::operator=(const GeometryType& rhs)
{
  Function::operator=(rhs);
  return *this;
}

te::da::Expression* te::da::GeometryType::clone() const
{
  return new GeometryType(*this);
}

