/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/dataaccess/query/SetSridToLiteralGeomVisitor.cpp

  \brief A visitor for building adjusting the SRID of literal geometries. This is particularly useful after reading a previously serialized data
*/

// TerraLib
#include "SetSridToLiteralGeomVisitor.h"

#include "../../common/StringUtils.h"
#include "../../core/translator/Translator.h"
#include "../../datatype/AbstractData.h"
#include "../../datatype/SimpleData.h"
#include "../../datatype/Enums.h"
#include "../Exception.h"
#include "../utils/Utils.h"

#include "Function.h"
#include "LiteralGeom.h"
#include "Select.h"
#include "Where.h"


te::da::SetSridToLiteralGeomVisitor::SetSridToLiteralGeomVisitor(int srid, bool adjustOnlyUnknown)
  : QueryVisitor()
  , m_srid(srid)
  , m_adjustOnlyUnknown(adjustOnlyUnknown)
{

}

te::da::SetSridToLiteralGeomVisitor::~SetSridToLiteralGeomVisitor() = default;

void te::da::SetSridToLiteralGeomVisitor::visit(const Function& visited)
{
  for (std::size_t i = 0; i < visited.getNumArgs(); ++i)
  {
    visited.getArg(i)->accept(*this);
  }
}

void te::da::SetSridToLiteralGeomVisitor::visit(const LiteralGeom & visited)
{
  te::dt::AbstractData* absData = visited.getValue();
  if (absData == nullptr)
  {
    return;
  }

  te::gm::Geometry* geometry = dynamic_cast<te::gm::Geometry*>(absData);
  if (geometry != nullptr)
  {
    if (geometry->getSRID() == TE_UNKNOWN_SRS)
    {
      geometry->setSRID(m_srid);
    }
    else if (m_adjustOnlyUnknown == false)
    {
      geometry->setSRID(m_srid);
    }
  }
}

void te::da::SetSridToLiteralGeomVisitor::visit(const Select& visited)
{
  if (visited.getWhere())
  {
    visited.getWhere()->getExp()->accept(*this);
  }
}
