/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

 /*!
   \file terralib/dataaccess/utils/FieldNameValidator.h

   \brief This class is responsible for validating field names. It can handle size limits, fix special characters changes the form 'tableName.propertyName' into the form 'tableName_propertyName'
 */

#ifndef __TERRALIB_DATAACCESS_INTERNAL_FIELDNAMEVALIDATOR_H
#define __TERRALIB_DATAACCESS_INTERNAL_FIELDNAMEVALIDATOR_H

#include "../Config.h"

#include <set>
#include <string>
#include <vector>

namespace te
{
  namespace da
  {
    /*!
      \class FieldNameValidator

      \brief This class is responsible for validating field names. It can handle size limits, fix special characters changes the form 'tableName.propertyName' into the form 'tableName_propertyName'

    */
    class TEDATAACCESSEXPORT FieldNameValidator
    {
    public:

      FieldNameValidator(int fieldNameMaxSize, bool replaceDotForUnderline, bool replaceSpecialChars);

      static std::string adjustFieldNameMaxSize(const std::string& fieldName, int fieldNameMaxSize);
      static std::string replaceDotForUnderline(const std::string& fieldName);
      static std::string replaceSpecialCharacters(const std::string& fieldName);

      bool addFieldName(const std::string& fieldName);
      bool addFieldName(const std::string& fieldName, std::string& adjustedFieldName);

      bool exists(const std::string& fieldName) const;

      const std::vector<std::string>& getAdjustedNames() const;

    protected:

      int m_fieldNameMaxSize; //!< The field name maximum size. -1 means no limit
      bool m_replaceDotForUnderline; //!< If TRUE, all 'dots' characters present in a "table.property" format will be replaced by an underline character
      bool m_replaceSpecialChars; //!< If TRUE, the special chars will be replaced
      std::vector<std::string> m_vecAdjustedNames; //!< Internal vector containing the ajusted (or the original if no adjustment hasbeen needed) names
      std::set<std::string> m_setAdjustedNames; //!< Internal set used to make a fast check to make sure there arent repeated field names
    };
  } //end namespace ogr
} //end namespace te

#endif // __TERRALIB_DATAACCESS_INTERNAL_FIELDNAMEVALIDATOR_H
