/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/datatype/IntegerConverters.cpp

  \brief A set of function that convert an Integer type to other types.
*/

// TerraLib
#include "DoubleConverters.h"

#include "../core/translator/Translator.h"
#include "Enums.h"
#include "Exception.h"
#include "SimpleData.h"

// Boost
#include <boost/lexical_cast.hpp>

// Double
te::dt::AbstractData* te::dt::DoubleToCharConverter(te::dt::AbstractData* d) 
{
  AssertDouble(d);
  return new Char(boost::lexical_cast<char>(static_cast<const Double*>(d)->getValue()));
}

te::dt::AbstractData* te::dt::DoubleToUCharConverter(te::dt::AbstractData* d) 
{
  AssertDouble(d);
  return new UChar(boost::lexical_cast<unsigned char>(static_cast<const Double*>(d)->getValue()));
}

te::dt::AbstractData* te::dt::DoubleToInt16Converter(te::dt::AbstractData* d) 
{
  AssertDouble(d);
  return new Int16(boost::lexical_cast<int16_t>(static_cast<const Double*>(d)->getValue()));
}

te::dt::AbstractData* te::dt::DoubleToUInt16Converter(te::dt::AbstractData* d) 
{
  AssertDouble(d);
  return new UInt16(boost::lexical_cast<uint16_t>(static_cast<const Double*>(d)->getValue()));
}

te::dt::AbstractData* te::dt::DoubleToInt32Converter(te::dt::AbstractData* d) 
{
  AssertDouble(d);
  return new Int32(static_cast<int32_t>(static_cast<const Double*>(d)->getValue()));
}

te::dt::AbstractData* te::dt::DoubleToUInt32Converter(te::dt::AbstractData* d) 
{
  AssertDouble(d);
  return new UInt32(static_cast<uint32_t>(static_cast<const Double*>(d)->getValue()));
}

te::dt::AbstractData* te::dt::DoubleToInt64Converter(te::dt::AbstractData* d) 
{
  AssertDouble(d);
  return new Int64(boost::lexical_cast<int64_t>(static_cast<const Double*>(d)->getValue()));
}

te::dt::AbstractData* te::dt::DoubleToUInt64Converter(te::dt::AbstractData* d) 
{
  AssertDouble(d);
  return new UInt64(static_cast<uint64_t>(static_cast<const Double*>(d)->getValue()));
}

te::dt::AbstractData* te::dt::DoubleToBooleanConverter(te::dt::AbstractData* d) 
{
  AssertDouble(d);
  
  bool nval = true;
  if(static_cast<const Double*>(d)->getValue() == 0.)
    nval = false;

  return new Boolean(nval);
}

te::dt::AbstractData* te::dt::DoubleToFloatConverter(te::dt::AbstractData* d) 
{
  AssertDouble(d);
  return new Float(boost::lexical_cast<float>(static_cast<const Double*>(d)->getValue()));
}

te::dt::AbstractData* te::dt::DoubleToDoubleConverter(te::dt::AbstractData* d) 
{
  AssertDouble(d);
  return new Double(boost::lexical_cast<double>(static_cast<const Double*>(d)->getValue()));
}

te::dt::AbstractData* te::dt::DoubleToNumericConverter(te::dt::AbstractData* d) 
{
  AssertDouble(d);
  return new Numeric(boost::lexical_cast<std::string>(static_cast<const Double*>(d)->getValue()));
}

te::dt::AbstractData* te::dt::DoubleToStringConverter(te::dt::AbstractData* d) 
{
  AssertDouble(d);
  return new String(boost::lexical_cast<std::string>(static_cast<const Double*>(d)->getValue()));
}

void te::dt::AssertDouble(AbstractData* d) 
{
  if(d->getTypeCode() != DOUBLE_TYPE)
    throw Exception(TE_TR("The input data type is not a Double!"));
}
