/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/edit/Feature.h
   
  \brief This class represents a geographic feature.
*/

#ifndef __TERRALIB_EDIT_INTERNAL_FEATURE_H
#define __TERRALIB_EDIT_INTERNAL_FEATURE_H

// TerraLib
#include "Config.h"
#include "Enums.h"
#include "Utils.h"

// STL
#include <map>

namespace te
{
// Forward declarations
  namespace da
  {
    class ObjectId;
  }

  namespace dt
  {
    class AbstractData;
  }

  namespace gm
  {
    class Geometry;
  }

  namespace edit
  {
    class TEEDITEXPORT Feature
    {
      public:

        Feature();

        Feature(te::da::ObjectId* id);

        Feature(te::da::ObjectId* id, te::gm::Geometry* geom);

        ~Feature();

        void set(te::da::ObjectId* id, te::gm::Geometry* geom);

        void setId(te::da::ObjectId* id);

        void setGeometry(te::gm::Geometry* geom);

        void setData(const std::map<std::string, te::dt::AbstractData*>& data);

        void setFillColor(const te::color::RGBAColor& fillColor);

        void setContourColor(const te::color::RGBAColor& contourColor);

        void setContourDashStyle(te::map::LineDashStyle dashStyle);

        void setContourWidth(int width);

        void setOperationTypeId(const te::edit::OperationType& currentOperationType);

        void setEditable(const bool& editable);

        te::da::ObjectId* getId() const;

        te::gm::Geometry* getGeometry() const;

        const std::map<std::string, te::dt::AbstractData*>& getData() const;

        te::color::RGBAColor getFillColor() const;

        te::color::RGBAColor getContourColor() const;

        te::map::LineDashStyle getContourDashStyle() const;

        int getContourWidth() const;

        te::edit::OperationType getOperationTypeId() const;

        bool hasColorChanged();

        bool isEquals(te::da::ObjectId* id);

        bool isEditable() const;

        Feature* clone() const;

      private:

        te::da::ObjectId* m_id;
        te::gm::Geometry* m_geom;
        std::map<std::string, te::dt::AbstractData*> m_data;

        te::color::RGBAColor m_fillColor;
        te::color::RGBAColor m_contourColor;
        te::map::LineDashStyle m_contourDashStyle;
        int m_contourWidth;
        bool m_colorChanged;

        te::edit::OperationType m_currentOperationType;
        bool m_isEditable;

    };

  } // end namespace edit
}   // end namespace te

#endif  // __TERRALIB_EDIT_INTERNAL_FEATURE_H
