/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See them_initialPosition
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
*/

/*!
\file GeometriesUpdateTool.cpp

\brief

*/

// TerraLib
#include "../../../qt/widgets/canvas/MapDisplay.h"
#include "../../../dataaccess/dataset/ObjectId.h"
#include "../../../geometry/Geometry.h"
#include "../../../maptools/AbstractLayer.h"
#include "../../Feature.h"
#include "../../RepositoryManager.h"
#include "../core/UndoStackManager.h"
#include "GeometriesUpdateTool.h"

te::edit::GeometriesUpdateTool::GeometriesUpdateTool(te::qt::widgets::MapDisplay* display, te::map::AbstractLayer* layer, QObject* parent) :
  te::qt::widgets::AbstractTool(display, parent),
  m_isInUse(true),
  m_feature(nullptr),
  m_layer(layer)
{}

te::edit::GeometriesUpdateTool::~GeometriesUpdateTool()
{
  delete m_feature;

  emit toolDeleted();
}

void te::edit::GeometriesUpdateTool::setLayer(te::map::AbstractLayer* layer)
{
  m_layer = layer;
}

std::string te::edit::GeometriesUpdateTool::getLayerId()
{
  return m_layer->getId();
}

bool te::edit::GeometriesUpdateTool::isInUse() const
{
  return m_isInUse;
}

void te::edit::GeometriesUpdateTool::setInUse(const bool& status)
{
  m_isInUse = status;
}

void te::edit::GeometriesUpdateTool::resetVisualizationTool()
{
  delete m_feature;
  m_feature = nullptr;
}

bool te::edit::GeometriesUpdateTool::eventFilter(QObject* watched, QEvent* e)
{
  if (e->type() == m_mEvent)
    return false;

  return te::qt::widgets::AbstractTool::eventFilter(watched, e);
}

void te::edit::GeometriesUpdateTool::disableMouseEvent(QMouseEvent::Type mEvent)
{
  m_mEvent = mEvent;
}
