/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

This file is part of the TerraLib - a Framework for building GIS enabled applications.

TerraLib is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License,
or (at your option) any later version.

TerraLib is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with TerraLib. See COPYING. If not, write to
TerraLib Team at <terralib-team@terralib.org>.
*/

/*!
\file Utils.cpp

\brief This file has util functions for this module.
*/

// TerraLib
#include "../fe.h"
#include "Utils.h"


te::fe::Filter* te::fe::CreateFilterByUniqueValue(const std::string& attrName, const std::string& value)
{
  te::fe::PropertyName* propName = new te::fe::PropertyName(attrName);
  te::fe::Literal* propValue = new te::fe::Literal(value);
  te::fe::BinaryComparisonOp* equal = new te::fe::BinaryComparisonOp(te::fe::Globals::sm_propertyIsEqualTo, propName, propValue);

  te::fe::Filter* filter = new te::fe::Filter;
  filter->setOp(equal);

  return filter;
}

void te::fe::GetFilterUniqueValue(const te::fe::Filter* filter, std::string& value)
{
  if (!filter)
    return;

  const te::fe::BinaryComparisonOp* equal = dynamic_cast<te::fe::BinaryComparisonOp*>(filter->getOp());
  
  if (equal)
  {
    const te::fe::Literal* propValue = dynamic_cast<te::fe::Literal*>(equal->getSecond());

    if (propValue)
    {
      value = propValue->getValue();
    }
  }
}

te::fe::Filter* te::fe::CreateFilterByStep(const std::string& attrName, const std::string& minValue, const std::string& maxValue)
{
  te::fe::PropertyName* propMinName = new te::fe::PropertyName(attrName);
  te::fe::Literal* propMinValue = new te::fe::Literal(minValue);
  te::fe::BinaryComparisonOp* greater = new te::fe::BinaryComparisonOp(te::fe::Globals::sm_propertyIsGreaterThanOrEqualTo, propMinName, propMinValue);

  te::fe::PropertyName* propMaxName = new te::fe::PropertyName(attrName);
  te::fe::Literal* propMaxValue = new te::fe::Literal(maxValue);
  te::fe::BinaryComparisonOp* less = new te::fe::BinaryComparisonOp(te::fe::Globals::sm_propertyIsLessThan, propMaxName, propMaxValue);

  te::fe::BinaryLogicOp* andOp = new te::fe::BinaryLogicOp(te::fe::Globals::sm_and, greater, less);

  te::fe::Filter* filter = new te::fe::Filter;
  filter->setOp(andOp);

  return filter;
}

void te::fe::GetFilterStepValues(const te::fe::Filter* filter, std::string& valueMin, std::string& valueMax)
{
  if (!filter)
    return;

  te::fe::BinaryLogicOp* andOp = dynamic_cast<te::fe::BinaryLogicOp*>(filter->getOp());

  if (andOp)
  {
    te::fe::BinaryComparisonOp* greater = dynamic_cast<te::fe::BinaryComparisonOp*>(andOp->getOp(0));

    if (greater)
    {
      const te::fe::Literal* propValue = dynamic_cast<te::fe::Literal*>(greater->getSecond());

      if (propValue)
      {
        valueMin = propValue->getValue();
      }
    }

    te::fe::BinaryComparisonOp* less = dynamic_cast<te::fe::BinaryComparisonOp*>(andOp->getOp(1));

    if (less)
    {
      const te::fe::Literal* propValue = dynamic_cast<te::fe::Literal*>(less->getSecond());

      if (propValue)
      {
        valueMax = propValue->getValue();
      }
    }
  }
}
