/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
 \file terralib/dataaccess/gdal/ScopedDataSetHandle.h
 
 \brief This encapsulates a GDAL dataSet and closes it when it runs out of scope. It also creates a wrapper to the dataSet in the cases which the file has a RPC metadata.
 */

#ifndef __TERRALIB_DATAACCES_GDAL_INTERNAL_SCOPEDDATASETHANDLE_H
#define __TERRALIB_DATAACCES_GDAL_INTERNAL_SCOPEDDATASETHANDLE_H

#include "Config.h"

//boost include files
#include <boost/noncopyable.hpp>

//STL include files
#include <memory>

class GDALDataset;

namespace te
{
  namespace gdal
  {
    /*!
     \class ScopedDataSetHandle
     
     \brief This encapsulates a GDAL dataSet and closes it when it runs out of scope. It also creates a wrapper to the dataSet in the cases which the file has a RPC metadata.
     */
    class TEGDALEXPORT ScopedDataSetHandle : protected boost::noncopyable
    {
      public :
        
        /*!
         \brief Default constructor.
         */
        ScopedDataSetHandle(GDALDataset* gdalDataSetHandle);
        
        ~ScopedDataSetHandle();

        GDALDataset* getGDALDataSetHandle() const;

    private:

      GDALDataset* m_gdalDataSetHandle;
      GDALDataset* m_gdalDataSetWrappedHandle;
    };

    using ScopedDataSetHandlePtr = std::unique_ptr<ScopedDataSetHandle>;

  } // end namespace gdal
}   // end namespace te

#endif  // __TERRALIB_DATAACCES_GDAL_INTERNAL_SCOPEDDATASETHANDLE_H
