#include "FixOrientation.h"

#include "../CommonDataStructures.h"
#include "../Coord2D.h"
#include "../Geometry.h"
#include "../GeometryFunctions.h"
#include "../LineString.h"
#include "../LinearRing.h"
#include "../Polygon.h"
#include "../Utils.h"

#include <vector>

te::gm::FixOrientation::FixOrientation(bool outerRingisCCW)
  : m_outerRingisCCW(outerRingisCCW)
{

}

te::gm::LineString* reverse(const te::gm::LineString* line)
{
  te::gm::LineString* outputLineString(static_cast<te::gm::LineString*>(line->clone()));

  for (std::size_t i = 0; i < line->size(); ++i)
  {
    std::size_t reversedIndex = line->size() - 1 - i;
    outputLineString->setX(reversedIndex, line->getX(i));
    outputLineString->setY(reversedIndex, line->getY(i));
  }

  return outputLineString;
}

te::gm::GeometryPtr te::gm::FixOrientation::validatePolygon(const te::gm::Polygon* polygon, bool& wasChanged) const
{
  //the OGC SFS definition states that the outer ring must be counter-clockwise and the inner rings must be clockwise
  bool outerRingIsCCW = m_outerRingisCCW;
  bool innerRingIsCCW = !m_outerRingisCCW;
  std::unique_ptr<te::gm::Polygon> outputPolygon(new Polygon(0, polygon->getGeomTypeId(), polygon->getSRID()));
  for (std::size_t i = 0; i < polygon->getNumRings(); ++i)
  {
    const te::gm::LinearRing* currentRing = static_cast<const te::gm::LinearRing*>(polygon->getRingN(i));
    if (i == 0 && te::gm::isCCW(currentRing) != outerRingIsCCW)
    {
      te::gm::LineString* reversedLine = reverse(currentRing);
      outputPolygon->add(static_cast<te::gm::Curve*>(reversedLine));
    }
    else if (i > 0 && te::gm::isCCW(currentRing) != innerRingIsCCW)
    {
      te::gm::LineString* reversedLine = reverse(currentRing);
      outputPolygon->add(static_cast<te::gm::Curve*>(reversedLine));
    }
    else
    {
      outputPolygon->add(static_cast<te::gm::Curve*>(currentRing->clone()));
    }
  }

  te::gm::GeometryPtr result(outputPolygon.release());
  return result;
}

te::gm::GeometryPtr te::gm::FixOrientation::validateLine(const te::gm::LineString* lineString, bool& wasChanged) const
{
  te::gm::GeometryPtr result(static_cast<te::gm::Geometry*>(lineString->clone()));
  return result.release();
}
