#include "FixRepeatedCoordinates.h"

#include "../CommonDataStructures.h"
#include "../Coord2D.h"
#include "../Geometry.h"
#include "../GeometryFunctions.h"
#include "../LineString.h"
#include "../LinearRing.h"
#include "../Utils.h"

#include <vector>

te::gm::GeometryPtr te::gm::FixRepeatedCoordinates::validateLine(const te::gm::LineString* lineString, bool& wasChanged) const
{
  wasChanged = false;
  if (te::gm::IsNullOrEmpty(lineString) == true)
  {
    return nullptr;
  }

  std::vector<te::gm::Coord2D> vecCoords;
  vecCoords.reserve(lineString->size());

  for (std::size_t i = 0; i < lineString->size(); ++i)
  {
    te::gm::Coord2D coord(lineString->getX(i), lineString->getY(i));

    if (vecCoords.empty() == false)
    {
      te::gm::Coord2D lastCoord = vecCoords.at(vecCoords.size() - 1);

      bool isEqual = false;
      if (m_precision == 0)
      {
        isEqual = te::gm::Distance::IsEqual(coord, lastCoord);
      }
      else
      {
        isEqual = te::gm::Distance::IsWithinDistance(coord, lastCoord, m_precision);
      }
      if (isEqual == true)
      {
        wasChanged = true;
        continue;
      }
    }

    vecCoords.push_back(coord);
  }

  if (wasChanged == false)
  {
    te::gm::GeometryPtr result(static_cast<te::gm::Geometry*>(lineString->clone()));
    return result;
  }

  std::unique_ptr<te::gm::LineString> outputLineString;

  bool isRing = (lineString->isClosed() && dynamic_cast<const te::gm::LinearRing*>(lineString) != nullptr);
  if (isRing == true && vecCoords.size() >= 4 && te::gm::Distance::IsEqual(vecCoords.at(0), vecCoords.at(vecCoords.size() - 1)) == true)
  {
    outputLineString.reset(static_cast<te::gm::LineString*>(lineString->clone()));
  }
  else if (isRing == false && vecCoords.size() >= 2)
  {
    outputLineString.reset(static_cast<te::gm::LineString*>(lineString->clone()));
  }

  if (outputLineString != nullptr)
  {
    outputLineString->setNumCoordinates(vecCoords.size());
    for (std::size_t i = 0; i < vecCoords.size(); ++i)
    {
      const te::gm::Coord2D& coord = vecCoords.at(i);
      outputLineString->setX(i, coord.x);
      outputLineString->setY(i, coord.y);
    }
  }

  te::gm::GeometryPtr result(outputLineString.release());
  return result.release();
}
