/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

 /*!
   \file terralib/geometry/validation/RemoveCollapsedSegments.h

   \brief An abstract class to represent and algorithm that validates an geometry
 */

#ifndef __TERRALIB_GEOMETRY_INTERNAL_VALIDATION_REMOVECOLLAPSEDSEGMENTS_H
#define __TERRALIB_GEOMETRY_INTERNAL_VALIDATION_REMOVECOLLAPSEDSEGMENTS_H

 // TerraLib
#include "AbstractValidator.h"

#include "../CommonDataStructures.h"
#include "../Config.h"

#include <map>
#include <vector>

namespace te
{
  namespace gm
  {
    struct Coord2D;
    class LineString;

    using VecCoords = std::vector<te::gm::Coord2D>;
    using VecFragments = std::vector<VecCoords>;
    using MapFragments = std::map<te::gm::Coord2D, VecFragments>;

    class TEGEOMEXPORT RemoveCollapsedSegments : public AbstractValidator
    {
    public:

      RemoveCollapsedSegments() = default;

      virtual ~RemoveCollapsedSegments() = default;

    protected:

      virtual te::gm::GeometryPtr validateLine(const te::gm::LineString* lineString, bool& wasChanged) const override;

      bool removeCollapsedSegmentsAndGetFragments(const te::gm::LineString* lineString, VecFragments& fragments) const;

      VecCoords joinFragments(const VecFragments& vecFragments) const;
    };

  } // end namespace gm
}   // end namespace te

#endif  // __TERRALIB_GEOMETRY_INTERNAL_VALIDATION_REMOVECOLLAPSEDSEGMENTS_H
