/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/postgis/ScopedConnection.cpp

  \brief A class that control the use of the connection to a PostgreSQL database.
*/

// TerraLib
#include "Connection.h" 
#include "ConnectionPool.h" 
#include "ScopedConnection.h"

te::pgis::ScopedConnection::ScopedConnection(ConnectionPool* pool, int id)
  : m_connectionPool(pool){
  m_connection = m_connectionPool->getConnection(id);
}

te::pgis::ScopedConnection::~ScopedConnection(){
  release();
}

void te::pgis::ScopedConnection::release(){
  if (m_connection != nullptr)
  {
    m_connectionPool->release(m_connection);
    m_connection = nullptr;
  }
}

te::pgis::Connection* te::pgis::ScopedConnection::operator->() const{
  return m_connection;
}
