/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/postgis/ScopedConnection.h

  \brief A class that control the use of connection to a PostgreSQL database.
*/

#ifndef __TERRALIB_POSTGIS_INTERNAL_SCOPEDCONNECTION_H
#define __TERRALIB_POSTGIS_INTERNAL_SCOPEDCONNECTION_H

// TerraLib
#include "Config.h"

namespace te
{
  namespace pgis
  {
    class Connection;
    class ConnectionPool;
    
    /*!
      \class ScopedConnection

      \brief A class that control the use of the connection to a PostgreSQL database.

      This class control the use of the connection and release the connection to the connectionPool.

      \sa Connection
    */
    class TEPGISEXPORT ScopedConnection
    {
      public:

        /*! \brief Constructor. */
        ScopedConnection(ConnectionPool* pool, int id = -1);
        
        /*! \brief Destructor. */
        ~ScopedConnection();

        /*! \brief Release connection. */
        void release();

        /*! Member access operator.  */
        Connection* operator->() const;

      protected:
        
        ScopedConnection(const ScopedConnection&);

        ScopedConnection& operator=(const ScopedConnection&);

      private:

        Connection* m_connection;           //!< An available connection that will be released when the scopedConnection is deleted.
        ConnectionPool* m_connectionPool;   //!< The connection pool associated to the connection.

    };

  } // end namespace pgis
}   // end namespace te

#endif  // __TERRALIB_POSTGIS_INTERNAL_SCOPEDCONNECTION_H
