#include "DataAccessTaskProcess.h"

#include "Task.h"
#include "TaskCapabilities.h"
#include "SimpleDataParameterValue.h"

#include "../core/uri/URI.h"
#include "../datatype/Enums.h"
#include "../datatype/SimpleData.h"
#include "../dataaccess/utils/DataAccess.h"

te::process::DataAccessTask::DataAccessTask() : te::process::Task("DataAccessTask")
{
  te::process::TaskCapabilities* capabilities = new te::process::TaskCapabilities();
  capabilities->addParameterDescriptor(te::process::ParameterDescriptor("Data Source URI", te::dt::STRING_TYPE, true));
  capabilities->addParameterDescriptor(te::process::ParameterDescriptor("Data Set Name", te::dt::STRING_TYPE, true));
  capabilities->addParameterDescriptor(te::process::ParameterDescriptor("Data Access", te::dt::DATAACCESS_TYPE, false, te::process::ParameterDescriptor::Direction::OUTPUT));
  init(capabilities);
}

void te::process::DataAccessTask::setDataSourceType(const std::string& type)
{
  m_driver = type;
}

std::string te::process::DataAccessTask::getDataSourceType()
{
  return m_driver;
}


void te::process::DataAccessTask::runImpl()
{
  const te::process::ParameterValue* pValue1 = this->getParameter(0);
  const te::process::ParameterValue* pValue2 = this->getParameter(1);

  const te::dt::AbstractData* absData1 = (static_cast<const te::process::SimpleDataParameterValue*>(pValue1))->getValue();
  const te::dt::AbstractData* absData2 = (static_cast<const te::process::SimpleDataParameterValue*>(pValue2))->getValue();

  const te::dt::String* data1 = static_cast<const te::dt::String*>(absData1);
  const te::dt::String* data2 = static_cast<const te::dt::String*>(absData2);

  std::string uriStr = data1->getValue();
  std::string dataSetName =  data2->getValue();

  te::core::URI uri(uriStr);

  te::da::DataAccess* dataAccess = new te::da::DataAccess(uri, m_driver, dataSetName);

  te::process::SimpleDataParameterValue* resultValue = new te::process::SimpleDataParameterValue(dataAccess);
  this->setParameter(2, resultValue, true);
};

te::process::DataAccessTaskFactory::DataAccessTaskFactory(): te::process::TaskFactory("DataAccessTask", "DataAccess")
{
}

te::process::DataAccessTaskFactory::~DataAccessTaskFactory() = default;

te::process::Task* te::process::DataAccessTaskFactory::build()
{
  return new DataAccessTask();
}
