/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/dataaccess/Module.cpp
   
  \brief This singleton defines the TerraLib Data Access module entry.
*/

#include "Module.h"

#include "Task.h"
#include "DataAccessTaskProcess.h"
#include "DataStoreTaskProcess.h"

// TerraLib
#include "../core/logger/Logger.h"
#include "../core/translator/Translator.h"
#include "../common/TerraLib.h"

#ifdef TERRALIB_LEAK_DETECTOR_ENABLED
  #include <vld.h>
#endif

const te::process::Module& sm_module = te::process::Module::getInstance();

//Initializing static map from TaskFactory
std::map<std::string, std::vector<std::string> > te::process::TaskFactory::m_mapCategories;

static te::process::DataAccessTaskFactory  dataAccessTaskFactoryInstance;
static te::process::DataStoreTaskFactory  dataStoreTaskFactoryInstance;

te::process::Module::Module()
{
  m_module.reset(new TerraLib::Module("process", te::process::Module::initialize, te::process::Module::finalize));

// initialize TerraLib singleton
  TerraLib::getInstance().add(m_module);
}

te::process::Module::~Module() = default;

void te::process::Module::initialize()
{
  TE_LOG_TRACE(TE_TR("TerraLib Data Access module initialized!"));
}

void te::process::Module::finalize()
{
  TE_LOG_TRACE(TE_TR("TerraLib Data Access module finalized!"));

  TerraLib::getInstance().remove("process");
}

