/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

 /*!
   \file terralib/process/ParameterDescriptor.cpp

   \brief This class represents the description of a parameter
 */

#include "ParameterDescriptor.h"

#include "../datatype/Enums.h"
#include "ParameterValue.h"


te::process::ParameterDescriptor::ParameterDescriptor(const std::string& name, int dataType, bool isRequired, Direction direction)
  : m_name(name)
  , m_dataType(dataType)
  , m_isRequired(isRequired)
  , m_direction(direction)
{

}

te::process::ParameterDescriptor::ParameterDescriptor(const te::process::ParameterDescriptor& rhs)
{
  *this = rhs;
}

te::process::ParameterDescriptor::~ParameterDescriptor() = default;

te::process::ParameterDescriptor& te::process::ParameterDescriptor::operator=(const ParameterDescriptor& rhs)
{
  this->m_name = rhs.m_name;
  this->m_dataType = rhs.m_dataType;
  this->m_isRequired = rhs.m_isRequired;
  this->m_direction = rhs.m_direction;

  ParameterValue* defaultValue = nullptr;
  if (rhs.m_defaultValue != nullptr)
  {
    defaultValue = rhs.m_defaultValue->clone();
  }

  this->m_defaultValue.reset(defaultValue);

  return *this;
}

const std::string te::process::ParameterDescriptor::getName() const
{
  return m_name;
}

void te::process::ParameterDescriptor::setName(const std::string& name)
{
  m_name = name;
}

const std::string te::process::ParameterDescriptor::getDescription() const
{
  return m_description;
}

void te::process::ParameterDescriptor::setDescription(const std::string& description)
{
  m_description = description;
}

int te::process::ParameterDescriptor::getDataType() const
{
  return m_dataType;
}

void te::process::ParameterDescriptor::setDataType(int dataType)
{
  m_dataType = dataType;
}

bool te::process::ParameterDescriptor::isRequired() const
{
  return m_isRequired;
}

void te::process::ParameterDescriptor::setRequired(bool required)
{
  m_isRequired = required;
}

te::process::ParameterDescriptor::Direction te::process::ParameterDescriptor::getDirection() const
{
  return m_direction;
}

void te::process::ParameterDescriptor::setDirection(te::process::ParameterDescriptor::Direction direction)
{
  m_direction = direction;
}

const te::process::ParameterValue* te::process::ParameterDescriptor::getDefaultValue() const
{
  return m_defaultValue.get();
}

void te::process::ParameterDescriptor::setDefaultValue(te::process::ParameterValue* defaultValue)
{
  m_defaultValue.reset(defaultValue);
}

bool te::process::ParameterDescriptor::isValueValid(const te::process::ParameterValue* value) const
{
  if (value == nullptr)
  {
    if (m_isRequired == true)
    {
      return false;
    }
    else 
    {
      return true;
    }
  }

  if (value->getDataType() != m_dataType)
  {
    return false;
  }

  return true;
}
