/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

 /*!
   \file terralib/process/ParameterValue.h

   \brief This class represents the value of a parameter
 */

#ifndef __TERRALIB_PROCESS_INTERNAL_PARAMETERVALUE_H
#define __TERRALIB_PROCESS_INTERNAL_PARAMETERVALUE_H

 // TerraLib
#include "Config.h"

#include "../common/AbstractFactory.h"

//STL include files
#include <string>

namespace te
{
  namespace process
  {
    /*!
      \class Task

      \brief This class represents the value of a parameter.
    */
    class TEPROCESSEXPORT ParameterValue
    {
    public:

      //!< Constructor
      ParameterValue(int dataType);

      //!< Constructor
      ParameterValue(const ParameterValue& rhs) = delete;

      /*! \brief Virtual destructor. */
      virtual ~ParameterValue();

      //!< Gets the parameter data type
      virtual int getDataType() const;

      //!< Clones the object
      virtual ParameterValue* clone() const = 0;

      //!< Returns the string representation of the parameter value
      virtual std::string toString() const = 0;

      //!< Decodes the given string representation and sets the value into the parameter value object
      virtual void fromString(const std::string& strValue) = 0;

    protected:

      int m_dataType; //!< The data type of the parameter value
    };


    /*!
      \class ParameterValueFactory

      \brief Abstract factory used to create ParameterValue.
    */
    class TEPROCESSEXPORT ParameterValueFactory : public te::common::AbstractFactory < te::process::ParameterValue, int >
    {
    protected:

      /*!
        \brief Constructor.

        \param factoryKey The key that identifies the factory.
      */
      ParameterValueFactory(int dataType);

      //!< Destructor
      virtual ~ParameterValueFactory();
    };

    //!< Wrapper function used to create taks. Used to avoid C++ to create multiple instances of the factory singleton on Windows
    TEPROCESSEXPORT ParameterValue* CreateParameterValue(int dataType);

  } // end namespace process
}   // end namespace te

#endif  // __TERRALIB_PROCESS_INTERNAL_PARAMETERVALUE_H

