/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

 /*!
   \file terralib/process/ProcessListener.h

   \brief This abstract class a listener to the process flow
 */

#ifndef __TERRALIB_PROCESS_INTERNAL_PROCESSLISTENER_H
#define __TERRALIB_PROCESS_INTERNAL_PROCESSLISTENER_H

 // TerraLib
#include "Config.h"

//STL
#include <string>

namespace te
{
  namespace process
  {
    class TaskSharedPtr;

    /*!
      \class ProcessListener

      \brief This abstract class a listener to the process flow
    */
    class TEPROCESSEXPORT ProcessListener
    {
    public:

      virtual void onProcessStarted() {};

      virtual void onProcessFinished() {};

      virtual void onTaskStarted(const te::process::TaskSharedPtr& task) {};

      virtual void onTaskFinished(const te::process::TaskSharedPtr& task) {};

      virtual void onProcessAborted(const std::string& errorMessage) {};

    };
  } // end namespace process
}   // end namespace te

#endif  // __TERRALIB_PROCESS_INTERNAL_PROCESSLISTENER_H
