/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

 /*!
   \file terralib/process/TaskCapabilities.cpp

   \brief Represents the capabilities of a task
 */

#include "TaskCapabilities.h"

#include "../common/Exception.h"
#include "../core/translator/Translator.h"

te::process::TaskCapabilities::TaskCapabilities()
{

}

te::process::TaskCapabilities::~TaskCapabilities() = default;

void te::process::TaskCapabilities::clear()
{
  m_vecParameterDescriptors.clear();
}

std::size_t te::process::TaskCapabilities::size() const
{
  return m_vecParameterDescriptors.size();
}

void te::process::TaskCapabilities::addParameterDescriptor(const te::process::ParameterDescriptor& parameter)
{
  m_vecParameterDescriptors.push_back(parameter);
  
}

const te::process::ParameterDescriptor& te::process::TaskCapabilities::getParameterDescriptor(std::size_t index) const
{
  if (index >= m_vecParameterDescriptors.size())
  {
    throw te::common::Exception(TE_TR("Invalid value index in") + std::string(": te::process::TaskCapabilities::getParameterDescriptor"));
  }

  return m_vecParameterDescriptors.at(index);
}

const std::vector<te::process::ParameterDescriptor>& te::process::TaskCapabilities::getParameterDescriptors() const
{
  return m_vecParameterDescriptors;
}

std::vector<te::process::ParameterDescriptor> te::process::TaskCapabilities::filter(te::process::ParameterDescriptor::Direction direction) const
{
  std::vector<te::process::ParameterDescriptor> filterResult;
  for (std::size_t i = 0; i < m_vecParameterDescriptors.size(); ++i)
  {
    const te::process::ParameterDescriptor& descriptor = m_vecParameterDescriptors.at(i);
    if (descriptor.getDirection() == direction)
    {
      //add a copy to the filter result
      filterResult.push_back(descriptor);
    }
  }

  return filterResult;
}

std::size_t te::process::TaskCapabilities::getParameterIndex(const std::string& name) const
{
  for (std::size_t i = 0; i < m_vecParameterDescriptors.size(); ++i)
  {
    const te::process::ParameterDescriptor& descriptor = m_vecParameterDescriptors.at(i);
    if (descriptor.getName() == name)
    {
      return i;
    }
  }

  return std::string::npos;
}
