/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

 /*!
   \file terralib/process/TaskCapabilities.h

   \brief Represents the capabilities of a task
 */

#ifndef __TERRALIB_PROCESS_INTERNAL_TASKCAPABILITIES_H
#define __TERRALIB_PROCESS_INTERNAL_TASKCAPABILITIES_H

 // TerraLib
#include "Config.h"
#include "ParameterDescriptor.h"

//STL include files
#include <string>
#include <vector>

namespace te
{
  namespace process
  {
    /*!
      \class TaskCapabilities

      \brief Represents the capabilities of a task
    */
    class TEPROCESSEXPORT TaskCapabilities
    {
    public:

      /*!
        \brief Constructor
      */
      TaskCapabilities();

      /*! \brief Virtual destructor. */
      virtual ~TaskCapabilities();

      //!< Clears the capabilities
      void clear();

      //!< Gets the number of parameter descriptors
      std::size_t size() const;

      //!< Adds an input parameter descriptor
      void addParameterDescriptor(const ParameterDescriptor& parameter);

      //!< Gets the index-th parameter descriptors
      const ParameterDescriptor& getParameterDescriptor(std::size_t index) const;

      //!< Gets the parameters descriptors
      const std::vector<ParameterDescriptor>& getParameterDescriptors() const;

      //!< Filters the parameters list based on the given direction
      std::vector<ParameterDescriptor> filter(te::process::ParameterDescriptor::Direction direction) const;

      //!< Gets the parameter index given a parameter name
      std::size_t getParameterIndex(const std::string& name) const;

    protected:

      std::vector<ParameterDescriptor> m_vecParameterDescriptors;
    };
  } // end namespace process
}   // end namespace te

#endif  // __TERRALIB_PROCESS_INTERNAL_TASKCAPABILITIES_H

