/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

 /*!
   \file terralib/process/TaskParameters.h

   \brief This class is a holder for all the task input and output parameters
 */

#ifndef __TERRALIB_PROCESS_INTERNAL_TASKPARAMETERS_H
#define __TERRALIB_PROCESS_INTERNAL_TASKPARAMETERS_H

 // TerraLib
#include "Config.h"

#include "TaskCapabilities.h"

//STL include files
#include <vector>

namespace te
{
  namespace process
  {
    class ParameterValue;

    /*!
      \class Task

      \brief This class is a holder for all the task input and output parameters
    */
    class TEPROCESSEXPORT TaskParameters
    {
    public:

      //!< Constructor
      TaskParameters(const te::process::TaskCapabilities& taskCapabilities);

      //!<  Copy constructor
      TaskParameters(const TaskParameters& rhs);

      /*! \brief Virtual destructor. */
      virtual ~TaskParameters();

      //!< Assigment operator
      virtual TaskParameters& operator=(const TaskParameters& rhs);

      //!< Gets the number of parameters
      virtual std::size_t size() const;

      //!< Checks if the parameters are valid
      bool isValid() const;

      //!< Gets the 'index' value. If not set, returns null
      const te::process::ParameterValue* getValue(std::size_t index) const;

      //!< Sets the 'index' value. Also sets the information related to the owner of the pointer
      void setValue(std::size_t index, te::process::ParameterValue* value, bool takeOwnership);

    protected:

      //!< Clears the task parameters
      void clear();

    protected:

      std::vector<te::process::ParameterValue*> m_vecValues;
      std::vector<bool> m_vecOwnership;
      TaskCapabilities m_capabilities;

    };
  } // end namespace process
}   // end namespace te

#endif  // __TERRALIB_PROCESS_INTERNAL_TASKPARAMETERS_H

