/*
  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

  This file is part of the TerraLib - a Framework for building GIS enabled applications.

  TerraLib is free software: you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation, either version 3 of the License,
  or (at your option) any later version.

  TerraLib is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with TerraLib. See COPYING. If not, write to
  TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/process/qt/Config.h

  \brief Configuration file for TerraLib Qt Process.
*/

#ifndef __TERRALIB_PROCESS_QT_CONFIG_H__
#define __TERRALIB_PROCESS_QT_CONFIG_H__

// TerraLib
#include "capabilities/DataAccessTaskCapabilities.h"
#include "capabilities/DataStoreTaskCapabilities.h"
#include "capabilities/DefaultTaskCapabilities.h"

#include "property/DataAccessTaskPropertyHandler.h"
#include "property/DataStoreTaskPropertyHandler.h"
#include "property/DefaultTaskPropertyHandler.h"

void RegisterFactories()
{
  static te::process::qt::DataAccessTaskCapabilitiesFactory  dataAccessTaskCapabilitiesFactory;
  static te::process::qt::DataStoreTaskCapabilitiesFactory  dataStoreTaskCapabilitiesFactory;
  static te::process::qt::DefaultTaskCapabilitiesFactory  defaultTaskCapabilitiesFactory;

  static te::process::qt::DefaultTaskPropertyHandlerFactory  defaultTaskPropertyHandlerInstance;
  static te::process::qt::DataAccessTaskPropertyHandlerFactory  dataAccessTaskPropertyHandlerInstance;
  static te::process::qt::DataStoreTaskPropertyHandlerFactory  dataStoreTaskPropertyHandlerInstance;
}

#endif  // __TERRALIB_PROCESS_QT_CONFIG_H__
