/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*! 
  \file terralib/process/qt/ProcessBuilder.h

  \brief A QMainWindow for Process Builder Application.
*/

#ifndef __TERRALIB_PROCESS_QT_INTERNAL_PROCESSBUILDER_H
#define __TERRALIB_PROCESS_QT_INTERNAL_PROCESSBUILDER_H

#include "../Config.h"

// Qt Includes
#include <QDockWidget>
#include <QMainWindow>

namespace Ui {
  class ProcessBuilderForm;
}

namespace te
{
  namespace process
  {
    namespace qt
    {
      class ConnectorsProperty;
      class ProcessExplorerTree;
      class ProcessGraphicsScene;
      class ProcessInspector;
      class ProcessManager;
      class ProcessRunner;
      class TaskProperty;

      /*!
      \class ProcessBuilder

      \brief A QMainWindow for Process Builder Application.

      */
      class TEPROCESSEXPORT ProcessBuilder : public QMainWindow
      {
        Q_OBJECT

      public:

        /*! \brief Default constructor. */
        ProcessBuilder(QWidget* parent = 0);

        /*! \brief Virtual destructor. */
        virtual ~ProcessBuilder();

      public:

        /*! \brief Gets process explorer tree qt component. */
        te::process::qt::ProcessExplorerTree* getExplorerTree();

      protected:

        /*!
          \brief Function used to setup the process builder main window.
        */
        void setupUI();

        /*!
          \brief Function used to setup the menus from main window.
        */
        void setupMenus();

        /*!
          \brief Function used create a new dock widget in application
        */
        void createDock(QWidget* widget, QDockWidget*& dockWidget, Qt::DockWidgetArea area, const std::string& dockTitle);

      protected slots:

        void sceneitemSelectionChanged();


        void onProjectRunTriggered();

        void onProjectValidateTriggered();

        void onProjectResetTriggered();

        void onFormatLeftAlignTriggered();

        void onFormatRightAlignTriggered();

        void onFormatTopAlignTriggered();

        void onFormatBottomAlignTriggered();

        void onFormatHCenterAlignTriggered();

        void onFormatVCenterAlignTriggered();

        void onWindowExplorerDockVisibilityChanged(bool state);

        void onWindowInspectorDockVisibilityChanged(bool state);

        void onWindowTaskDockVisibilityChanged(bool state);

        void onWindowConnectorDockVisibilityChanged(bool state);

        void onWindowRunnerDockVisibilityChanged(bool state);

        void onHelpAboutTriggered();

      protected:

        ConnectorsProperty* m_connectorsProperty;   //!< Qt component to represents a task connectors explorer.
        ProcessGraphicsScene* m_scene;              //!< Qt component use to represents the process items.
        ProcessExplorerTree* m_explorerTree;        //!< Qt Component to explore process items.
        ProcessInspector* m_processInspector;       //!< Qt component to inspect each process item.
        ProcessManager* m_manager;                  //!< Qt class used to handle with process management.
        ProcessRunner* m_processRunner;             //!< Qt component to run process.
        TaskProperty* m_taskProperty;               //!< Qt component to represents a task property explorer.

        QDockWidget* m_explorerDock;
        QDockWidget* m_inspectorDock;
        QDockWidget* m_taskPropertyDock;
        QDockWidget* m_connectorPropertyDock;
        QDockWidget* m_processRunnerDock;

        QAction* m_windowExplorerDock;
        QAction* m_windowInspectorDock;
        QAction* m_windowTaskDock;
        QAction* m_windowConnectorDock;
        QAction* m_windowRunnerDock;
  
      private:
        Ui::ProcessBuilderForm* m_ui;
      };
    }
  }
}

#endif // __TERRALIB_PROCESS_QT_INTERNAL_PROCESSBUILDER_H
