/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*! 
  \file terralib/process/qt/ProcessManager.h

  \brief A QObject class to handle with process management
*/

#ifndef __TERRALIB_PROCESS_QT_INTERNAL_PROCESSMANAGER_H
#define __TERRALIB_PROCESS_QT_INTERNAL_PROCESSMANAGER_H


#include "capabilities/TaskGraphicalCapabilities.h"

#include "../Config.h"
#include "../Task.h"

// Qt Includes
#include <QObject>

// STL Includes
#include <map>
#include <string>

namespace te
{
  namespace process
  {
    class Process;

    namespace qt
    {
      /*!
      \class ProcessManager

      \brief A QObject class to handle with process management

      */
      class TEPROCESSEXPORT ProcessManager : public QObject
      {
        Q_OBJECT

      public:

        /*! \brief Default constructor. */
        ProcessManager(QObject *parent = nullptr);

        /*! \brief Virtual destructor. */
        ~ProcessManager();

      public:

        void start();

        void reset();

        std::string addTask(te::process::TaskSharedPtr task);

        std::string createTask(const std::string& taskType);

        void removeTask(const std::string& id);

        void resetTaskId(const std::string& id, const std::string& newId);

        std::string createConnector(const std::string& taskFrom, std::size_t paramIndexFrom, const std::string& taskTo, std::size_t paramIndexTo);

        void removeConnector(const std::string& id);

        Process* getProcess() const;

        /*! \brief Function used to check if a name is a valid item name. */
        bool isProcessItemNameValid(const std::string& itemType, const std::string& itemName);

        std::unique_ptr<TaskGraphicalCapabilities> getTaskGraphicalCapabilities(const std::string& taskType);

      protected:

        /*! \brief Function used to generate a valid item name. */
        std::string createProcessItemName(const std::string& itemType);

      signals:

        void createdTask(const std::string& id);

        void removedTask(const std::string& id);

        void resetedTaskId(const std::string& id, const std::string& newId);

        void createdConnector(const std::string& id);

        void removedConnector(const std::string& id);

      protected:

        Process* m_process;

        std::map<std::string, int> m_processItemMapCount;
      };
    }
  }
}

#endif // __TERRALIB_PROCESS_QT_INTERNAL_PROCESSMANAGER_H
