#include "ProcessRunner.h"

#include "../Connector.h"
#include "../Process.h"
#include "../Task.h"

Q_DECLARE_METATYPE(te::process::TaskSharedPtr)

te::process::qt::ProcessRunner::ProcessRunner(QWidget *parent) :
  QWidget(parent)
{
  qRegisterMetaType<te::process::TaskSharedPtr>();

  m_processLogTextEdit = new QTextEdit(this);
  m_processLogTextEdit->setReadOnly(true);

  //connects
  connect(this, SIGNAL(onProcessStartedSignal()), this, SLOT(onProcessStartedSlot()));
  connect(this, SIGNAL(onProcessFinishedSignal()), this, SLOT(onProcessFinishedSlot()));
  connect(this, SIGNAL(onTaskStartedSignal(const te::process::TaskSharedPtr&)), this, SLOT(onTaskStartedSlot(const te::process::TaskSharedPtr&)));
  connect(this, SIGNAL(onTaskFinishedSignal(const te::process::TaskSharedPtr&)), this, SLOT(onTaskFinishedSlot(const te::process::TaskSharedPtr&)));
  connect(this, SIGNAL(onProcessAbortedSignal(const std::string&)), this, SLOT(onProcessAbortedSlot(const std::string&)));
}

te::process::qt::ProcessRunner::~ProcessRunner() = default;

void te::process::qt::ProcessRunner::validate(te::process::Process* process)
{
  m_processLogTextEdit->clear();

  process->isValid(this);
}


void te::process::qt::ProcessRunner::run(te::process::Process* process)
{
  m_processLogTextEdit->clear();

  process->run(this);
}

void te::process::qt::ProcessRunner::reset(te::process::Process* process)
{
  m_processLogTextEdit->append("TODO");
}

QTextEdit* te::process::qt::ProcessRunner::getLoggerWidget()
{
  return m_processLogTextEdit;
}

void te::process::qt::ProcessRunner::onProcessStarted()
{
  emit onProcessStartedSignal();
}

void te::process::qt::ProcessRunner::onProcessFinished()
{
  emit onProcessFinishedSignal();
}

void te::process::qt::ProcessRunner::onTaskStarted(const te::process::TaskSharedPtr& task)
{
  emit onTaskStartedSignal(task);
}

void te::process::qt::ProcessRunner::onTaskFinished(const te::process::TaskSharedPtr& task)
{
  emit onTaskFinishedSignal(task);
}

void te::process::qt::ProcessRunner::onProcessAborted(const std::string& errorMessage)
{
  emit onProcessAbortedSignal(errorMessage);
}

void te::process::qt::ProcessRunner::onProcessStartedSlot()
{
  m_processLogTextEdit->append("START");
}

void te::process::qt::ProcessRunner::onProcessFinishedSlot()
{
  m_processLogTextEdit->append("END");
}

void te::process::qt::ProcessRunner::onTaskStartedSlot(const te::process::TaskSharedPtr& task)
{
  m_processLogTextEdit->append("TASK STARTED: " + QString(task->getId().c_str()));
}

void te::process::qt::ProcessRunner::onTaskFinishedSlot(const te::process::TaskSharedPtr& task)
{
  m_processLogTextEdit->append("TASK FINISHED: " + QString(task->getId().c_str()));
}

void te::process::qt::ProcessRunner::onProcessAbortedSlot(const std::string& errorMessage)
{
  m_processLogTextEdit->append("ERROR: " + QString(errorMessage.c_str()));
}