/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*! 
  \file terralib/process/qt/ProcessRunner.h

  \brief A QWidget to run process 
*/

#ifndef __TERRALIB_PROCESS_QT_INTERNAL_PROCESSRUNNER_H
#define __TERRALIB_PROCESS_QT_INTERNAL_PROCESSRUNNER_H

#include "../Config.h"
#include "../ProcessListener.h"

// Qt Includes
#include <QTextEdit>
#include <QWidget>

// STL
#include <map>

namespace te
{
  namespace process
  {
    class Process;
    class TaskSharedPtr;

    namespace qt
    {
      /*!
      \class ProcessRunner

      \brief A QWidget to run process 

      */
      class TEPROCESSEXPORT ProcessRunner : public QWidget, public te::process::ProcessListener
      {
        Q_OBJECT

      public:

        /*! \brief Default constructor. */
        ProcessRunner(QWidget *parent = nullptr);

        /*! \brief Virtual destructor. */
        ~ProcessRunner();

      public:

        void validate(te::process::Process* process);

        void run(te::process::Process* process);

        void reset(te::process::Process* process);

        QTextEdit* getLoggerWidget();

      protected:

        virtual void onProcessStarted() override;

        virtual void onProcessFinished() override;

        virtual void onTaskStarted(const te::process::TaskSharedPtr& task) override;

        virtual void onTaskFinished(const te::process::TaskSharedPtr& task) override;

        virtual void onProcessAborted(const std::string& errorMessage) override;

      signals:

        void onProcessStartedSignal();

        void onProcessFinishedSignal();

        void onTaskStartedSignal(const te::process::TaskSharedPtr& task);

        void onTaskFinishedSignal(const te::process::TaskSharedPtr& task);

        void onProcessAbortedSignal(const std::string& errorMessage);


      protected slots:

        void onProcessStartedSlot();

        void onProcessFinishedSlot();

        void onTaskStartedSlot(const te::process::TaskSharedPtr& task);

        void onTaskFinishedSlot(const te::process::TaskSharedPtr& task);

        void onProcessAbortedSlot(const std::string& errorMessage);

      protected:

        QTextEdit* m_processLogTextEdit;
      };
    }
  }
}

#endif // __TERRALIB_PROCESS_QT_INTERNAL_PROCESSRUNNER_H
