#include "DefaultTaskCapabilities.h"

#include "../../ParameterDescriptor.h"
#include "../../Task.h"
#include "../../TaskCapabilities.h"

// TerraLib Includes
#include "../../../qt/widgets/Utils.h"

te::process::qt::DefaultTaskCapabilities::DefaultTaskCapabilities()
  : TaskGraphicalCapabilities( "DefaultTask", "DeFault Task", "gear", "share/terralib/icons/terralib/scalable/gear.svg")
{
  m_brushColor = Qt::lightGray;
}

te::process::qt::DefaultTaskCapabilities::~DefaultTaskCapabilities() = default;

void te::process::qt::DefaultTaskCapabilities::setTaskType(const std::string& taskType)
{
  m_operationName = taskType;
}

QPainterPath te::process::qt::DefaultTaskCapabilities::getShape(QList<QPointF>& inHotPoints, QList<QPointF>& outHotPoints) const
{
  std::unique_ptr<te::process::Task> task(te::process::CreateTask(m_operationName));
  const te::process::TaskCapabilities* taskCap = task->getCapabilities();

  std::vector<te::process::ParameterDescriptor> inputParams = taskCap->filter(te::process::ParameterDescriptor::Direction::INPUT);
  std::vector<te::process::ParameterDescriptor> outputParams = taskCap->filter(te::process::ParameterDescriptor::Direction::OUTPUT);

  //set main shape
  QPainterPath path;
  path.addRoundedRect(10, 10, 40, 40, 10, 10);

  //set shape for input parameters
  double startHeight = 10.;
  double height = 40.;
  double space = height / (inputParams.size() + 1);

  for (std::size_t i = 0; i < inputParams.size(); ++i)
  {
    startHeight += space;

    path.moveTo(0, startHeight);
    path.lineTo(10, startHeight);

    inHotPoints.append(QPointF(3, startHeight));
  }

  //set shape for output parameters
  startHeight = 10.;
  height = 40.;
  space = height / (outputParams.size() + 1);

  for (std::size_t i = 0; i < outputParams.size(); ++i)
  {
    startHeight += space;

    path.moveTo(60, startHeight);
    path.lineTo(50, startHeight);

    outHotPoints.append(QPointF(57, startHeight));
  }

  return path;
}

QRectF te::process::qt::DefaultTaskCapabilities::shapeBoundingRect() const
{
  return QRectF(0, 0, 60, 60);
}

te::process::qt::DefaultTaskCapabilitiesFactory::DefaultTaskCapabilitiesFactory()
  : TaskGraphicalCapabilitiesFactory("DefaultTask")
{

}

te::process::qt::DefaultTaskCapabilitiesFactory::~DefaultTaskCapabilitiesFactory() = default;

te::process::qt::TaskGraphicalCapabilities* te::process::qt::DefaultTaskCapabilitiesFactory::build()
{
  return new te::process::qt::DefaultTaskCapabilities();
}