/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

This file is part of the TerraLib - a Framework for building GIS enabled applications.

TerraLib is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License,
or (at your option) any later version.

TerraLib is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with TerraLib. See COPYING. If not, write to
TerraLib Team at <terralib-team@terralib.org>.
*/

/*!
 * \file src/terralib/process/qt/DefaultTaskCapabilities.h
 *
 * \brief Defines a default task capabilities of a task used in graphical components
 */

#ifndef __TERRALIB_PROCESS_QT_INTERNAL_DEFAULTTASKCAPABILITIES_H
#define __TERRALIB_PROCESS_QT_INTERNAL_DEFAULTTASKCAPABILITIES_H

#include "../../Config.h"

#include "TaskGraphicalCapabilities.h"

namespace te
{
  namespace process
  {
    namespace qt
    {
      /*!
        \class DefaultTaskCapabilities

        \brief Defines a default task capabilities of a task used in graphical components
      */
      class TEPROCESSEXPORT DefaultTaskCapabilities : public TaskGraphicalCapabilities
      {
      public:
        DefaultTaskCapabilities();

        //!< Destructor
        virtual ~DefaultTaskCapabilities();

        void setTaskType(const std::string& taskType);

        virtual QPainterPath getShape(QList<QPointF>& inHotPoints, QList<QPointF>& outHotPoints) const override;

        virtual QRectF shapeBoundingRect() const override;
      };


      /*!
        \class DefaultTaskCapabilitiesFactory

        \brief Factory used to create DefaultTaskCapabilities.
      */
      class TEPROCESSEXPORT DefaultTaskCapabilitiesFactory : public TaskGraphicalCapabilitiesFactory
      {
      public:

        //!< brief Constructor
        DefaultTaskCapabilitiesFactory();

        //!< Destructor
        virtual ~DefaultTaskCapabilitiesFactory();

      protected:

        virtual TaskGraphicalCapabilities* build() override;
      };

    } //end namespace qt
  }//end namespace process
}//end namespace te


#endif //__TERRALIB_PROCESS_QT_INTERNAL_DEFAULTTASKCAPABILITIES_H
