#include "AbstractProcessTreeGroup.h"

#include "ProcessExplorerTree.h"
#include "ProcessTreeItem.h"

te::process::qt::AbstractProcessTreeGroup::AbstractProcessTreeGroup(ProcessExplorerTree* parent, const std::string& groupName) : 
  m_tree(parent),
  m_groupName(groupName)
{
  buildRootItem();
}

te::process::qt::AbstractProcessTreeGroup::~AbstractProcessTreeGroup() = default;

void te::process::qt::AbstractProcessTreeGroup::buildRootItem()
{
  m_root = new QTreeWidgetItem(m_tree, te::process::qt::ProcessTreeItem::ROOT_TREEITEMTYPE);
  m_root->setTextAlignment(0, Qt::AlignHCenter);
  m_root->setBackgroundColor(0, QColor(Qt::lightGray));
  m_root->setText(0, m_groupName.c_str());
  m_root->setFlags(Qt::ItemIsEnabled);

  m_tree->addTopLevelItem(m_root);
}

te::process::qt::ProcessTreeItem* te::process::qt::AbstractProcessTreeGroup::addGroupItem(const std::string& itemName, const std::string& iconName)
{
  ProcessTreeItem* item = new ProcessTreeItem(m_root, te::process::qt::ProcessTreeItem::ITEM_TREEITEMTYPE);
  item->setText(0, itemName.c_str());
  item->setIcon(0, QIcon::fromTheme(iconName.c_str()));
  item->setData(0, Qt::UserRole, QVariant(itemName.c_str()));

  m_root->addChild(item);
  m_root->setExpanded(true);

  return item;
}
