/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*! 
  \file terralib/process/qt/AbstractProcessTreeGroup.h

  \brief A QTreeWidgetItem Group of Process Builder items.
*/

#ifndef __TERRALIB_PROCESS_QT_INTERNAL_ABSTRACTPROCESSTREEGROUP_H
#define __TERRALIB_PROCESS_QT_INTERNAL_ABSTRACTPROCESSTREEGROUP_H

#include "../../Config.h"

// Qt Includes
#include <QObject>
#include <QTreeWidgetItem>

namespace te
{
  namespace process
  {
    namespace qt
    {
      class ProcessExplorerTree;
      class ProcessTreeItem;

      /*!
      \class AbstractProcessTreeGroup

      \brief A Abstract QTreeWidgetItem Group of Process Builder items.

      */
      class TEPROCESSEXPORT AbstractProcessTreeGroup : public QObject
      {
        Q_OBJECT

      public:

        /*! \brief Default constructor. */
        AbstractProcessTreeGroup(ProcessExplorerTree* parent, const std::string& groupName);

        /*! \brief Virtual destructor. */
        virtual ~AbstractProcessTreeGroup();

      protected:

        /*!
          \brief Function used to build and add the main item in process explorer tree.
        */
        void buildRootItem();

        /*!
          \brief Add a new item into group

          \param itemName Item name

          \param iconName Item icon path/name

          \return ProcessTreeItem that represents a process item.
        */
        ProcessTreeItem* addGroupItem(const std::string& itemName, const std::string& iconName);

        /*!
          \brief Virtual function used to setup process explorer tree items into a group.
        */
        virtual void setupGroup() = 0;

      protected:

        ProcessExplorerTree* m_tree;    //!< Group parent.

        std::string m_groupName;        //!< The group name.

        QTreeWidgetItem* m_root;        //!< Main item that represents the group.
      };
    }
  }
}

#endif // __TERRALIB_PROCESS_QT_INTERNAL_ABSTRACTPROCESSTREEGROUP_H
