/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*! 
  \file terralib/process/qt/ProcessExplorerTree.h

  \brief A QTreeWidget to view Process Builder items in a tree.
*/

#ifndef __TERRALIB_PROCESS_QT_INTERNAL_PROCESSEXPLORERTREE_H
#define __TERRALIB_PROCESS_QT_INTERNAL_PROCESSEXPLORERTREE_H

#include "../../Config.h"

// Qt Includes
#include <QTreeWidget>

namespace te
{
  namespace process
  {
    namespace qt
    {
      /*!
      \class ProcessExplorerTree

      \brief A QTreeWidget to view Process Builder items in a tree.

      */
      class TEPROCESSEXPORT ProcessExplorerTree : public QTreeWidget
      {
        Q_OBJECT

      public:

        /*! \brief Default constructor. */
        ProcessExplorerTree(QWidget *parent = nullptr);

        /*! \brief Virtual destructor. */
        ~ProcessExplorerTree();

      public:

        /*!
          \brief Virtual function used to setup process explorer tree.
        */
        void setupTree();

      protected:

        /*!
          \brief Virtual function used get a mimedata from selected process tree item.
        */
        virtual QMimeData *mimeData(const QList<QTreeWidgetItem*> items) const;

      };
    }
  }
}

#endif // __TERRALIB_PROCESS_QT_INTERNAL_PROCESSEXPLORERTREE_H
