/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*! 
  \file terralib/process/qt/ProcessTreeItem.h

  \brief A QTreeWidgetItem to represents a Process Builder items in a tree.
*/

#ifndef __TERRALIB_PROCESS_QT_INTERNAL_PROCESSTREEITEM_H
#define __TERRALIB_PROCESS_QT_INTERNAL_PROCESSTREEITEM_H

#include "../../Config.h"

// Qt Includes
#include <QTreeWidgetItem>

namespace te
{
  namespace process
  {
    namespace qt
    {
      /*!
      \class ProcessTreeItem

      \brief A QTreeWidgetItem to represents a Process Builder items in a tree.

      */
      class TEPROCESSEXPORT ProcessTreeItem : public QTreeWidgetItem
      {
      public:

        enum ProcessTreeItemType
        {
          ROOT_TREEITEMTYPE,      //!< Top level group item
          ITEM_TREEITEMTYPE       //!< Group item 
        };

        /*! \brief Default constructor. */
        ProcessTreeItem(QTreeWidgetItem* parent, int type = Type);

        /*! \brief Virtual destructor. */
        ~ProcessTreeItem();

      public:

        virtual QMimeData* getMimeData();

      };
    }
  }
}

#endif // __TERRALIB_PROCESS_QT_INTERNAL_PROCESSTREEITEM_H
