/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*! 
  \file terralib/process/qt/TaskMimeData.h

  \brief A Mime Data to represents a Task from process.
*/

#ifndef __TERRALIB_PROCESS_QT_INTERNAL_TASKMIMEDATA_H
#define __TERRALIB_PROCESS_QT_INTERNAL_TASKMIMEDATA_H

#include "../../Config.h"

#include "../../Task.h"

// Qt Includes
#include <QMimeData>

namespace te
{
  namespace process
  {
    namespace qt
    {
      /*!
      \class TaskMimeData

      \brief A Mime Data to represents a Task from process.

      */
      class TEPROCESSEXPORT TaskMimeData : public QMimeData
      {
      public:

        /*! \brief Default constructor. */
        TaskMimeData(TaskSharedPtr task);

        /*! \brief Virtual destructor. */
        ~TaskMimeData();


        void setTask(TaskSharedPtr task);

        TaskSharedPtr getTask() const;

      protected:

        TaskSharedPtr m_task;
      };
    }
  }
}

#endif // __TERRALIB_PROCESS_QT_INTERNAL_TASKMIMEDATA_H
