#include "AbstractGraphicsItem.h"

// TerraLib Includes
#include "../../../common/STLUtils.h"

// Qt Includes
#include <QBrush>
#include <QPainter>

#include <QGraphicsSceneMouseEvent>

te::process::qt::AbstractGraphicsItem::AbstractGraphicsItem() :
  QGraphicsObject(), 
  m_drawHotPoints(false)
{
  setFlag(ItemIsSelectable);
  setFlag(ItemIsMovable);
  setFlag(ItemSendsScenePositionChanges);
  setAcceptHoverEvents(true);
}

te::process::qt::AbstractGraphicsItem::~AbstractGraphicsItem()
{
  te::common::FreeContents(m_hotPoints);
  m_hotPoints.clear();
}

void te::process::qt::AbstractGraphicsItem::paint(QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget)
{
  if (isSelected())
  {
    //draw extent
    QRectF rec = boundingRect();

    QPen pen(Qt::green);
    pen.setStyle(Qt::DashLine);
    pen.setWidth(1);

    painter->setPen(pen);
    painter->drawRect(rec);
  }

  if (m_drawHotPoints)
  {
    //draw hotpoints
    for (std::size_t i = 0; i < m_hotPoints.size(); ++i)
    {
      m_hotPoints[i]->paint(painter, option, widget);
    }
  }
}

void te::process::qt::AbstractGraphicsItem::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
{
  m_drawHotPoints = true;

  QGraphicsObject::hoverEnterEvent(event);
}

void te::process::qt::AbstractGraphicsItem::hoverMoveEvent(QGraphicsSceneHoverEvent *event)
{
  QGraphicsObject::hoverMoveEvent(event);
}

void te::process::qt::AbstractGraphicsItem::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
{
  m_drawHotPoints = false;

  QGraphicsObject::hoverLeaveEvent(event);
}

void te::process::qt::AbstractGraphicsItem::mousePressEvent(QGraphicsSceneMouseEvent *event)
{
  HotPointGraphicsItem* hotPoint = getHotPoint(event->pos());

  if (hotPoint)
  {
    hotPointPressed(hotPoint);
  }

  QGraphicsObject::mousePressEvent(event);
}

void te::process::qt::AbstractGraphicsItem::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
{
  QGraphicsObject::mouseMoveEvent(event);
}

void te::process::qt::AbstractGraphicsItem::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
{
  QGraphicsObject::mouseReleaseEvent(event);
}

QPen te::process::qt::AbstractGraphicsItem::getPen()
{
  return m_pen;
}

QBrush te::process::qt::AbstractGraphicsItem::getBrush()
{
  return m_brush;
}

QFont te::process::qt::AbstractGraphicsItem::getFont()
{
  return m_font;
}

void te::process::qt::AbstractGraphicsItem::setPenColor(QColor color)
{
  m_pen.setColor(color);
}

void te::process::qt::AbstractGraphicsItem::setPenWidth(int width)
{
  m_pen.setWidth(width);
}

void te::process::qt::AbstractGraphicsItem::setPenStyle(Qt::PenStyle style)
{
  m_pen.setStyle(style);
}

void te::process::qt::AbstractGraphicsItem::setBrushColor(QColor color)
{
  m_brush.setColor(color);
}

void te::process::qt::AbstractGraphicsItem::setBrushStyle(Qt::BrushStyle style)
{
  m_brush.setStyle(style);
}

void te::process::qt::AbstractGraphicsItem::setFont(QFont font)
{
  m_font = font;
}

te::process::qt::HotPointGraphicsItem* te::process::qt::AbstractGraphicsItem::getHotPoint(const QPointF& pos)
{
  QPointF pixelOffset(4.0, 4.0);

  QRectF rect(pos - pixelOffset, pos + pixelOffset);

  for (int i = 0; i < m_hotPoints.size(); ++i)
  {
    if (rect.contains(m_hotPoints[i]->getHotPointPos()))
    {
      return m_hotPoints[i];
    }
  }

  return nullptr;
}