#include "ConnectorGraphicsItem.h"

#include "HotPointGraphicsItem.h"
#include "TaskGraphicsItem.h"

#include "../../Connector.h"

// Qt Includes
#include <QBrush>
#include <QGraphicsScene>
#include <QGraphicsView>
#include <QLineF>
#include <QPainter>
#include <QRectF>

//STL include files
#include <cmath>

te::process::qt::ConnectorGraphicsItem::ConnectorGraphicsItem(const std::string& connectorId, TaskGraphicsItem* taskFrom, HotPointGraphicsItem* hotPointFrom, TaskGraphicsItem* taskTo, HotPointGraphicsItem* hotPointTo) :
  QGraphicsObject(),
  m_connectorId(connectorId),
  m_taskFrom(taskFrom),
  m_hotPointFrom(hotPointFrom),
  m_taskTo(taskTo),
  m_hotPointTo(hotPointTo)
{
  m_hotPointFrom->setHotPointColor(Qt::blue);
  m_hotPointTo->setHotPointColor(Qt::blue);
}

te::process::qt::ConnectorGraphicsItem::~ConnectorGraphicsItem() = default;

const std::string te::process::qt::ConnectorGraphicsItem::getConnectorId() const
{
  return m_connectorId;
}

void te::process::qt::ConnectorGraphicsItem::removed()
{
  m_hotPointFrom->setHotPointColor(Qt::red);
  m_hotPointTo->setHotPointColor(Qt::red);
}

QRectF te::process::qt::ConnectorGraphicsItem::boundingRect() const
{
  QPainterPath path = getConnectorPainterPath();

  return path.boundingRect();
}

void te::process::qt::ConnectorGraphicsItem::paint(QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget)
{
  QPainterPath path = getConnectorPainterPath();
  QPainterPath arrowPath = getConnectorArrowPainterPath();

  QPen pointPen(Qt::blue);
  pointPen.setCapStyle(Qt::RoundCap);
  pointPen.setWidth(1);

  painter->setPen(pointPen);
  painter->drawPath(path);
  painter->fillPath(arrowPath, QBrush(Qt::blue));
}

QPainterPath te::process::qt::ConnectorGraphicsItem::getConnectorPainterPath() const
{
  QPointF pointA;
  QPointF pointB;

  if (m_hotPointFrom == nullptr || m_hotPointTo == nullptr)
  {
    pointA = QPointF(0, 0);
    pointB = QPointF(0, 0);
  }
  else
  {
    pointA = m_taskFrom->mapToScene(m_hotPointFrom->getHotPointPos());
    pointB = m_taskTo->mapToScene(m_hotPointTo->getHotPointPos());
  }

  QPointF lineCenter = QRectF(pointA, pointB).normalized().center();

  QPainterPath path;
  path.moveTo(pointA);
  //path.addEllipse(pointA, 3, 3);
  path.lineTo(lineCenter.x(), pointA.y());
  path.lineTo(lineCenter.x(), pointB.y());
  path.lineTo(pointB);

  return path;
}

QPainterPath te::process::qt::ConnectorGraphicsItem::getConnectorArrowPainterPath() const
{
  QPointF pointA;
  QPointF pointB;

  if (m_hotPointFrom == nullptr || m_hotPointTo == nullptr)
  {
    pointA = QPointF(0, 0);
    pointB = QPointF(0, 0);
  }
  else
  {
    pointA = m_taskFrom->mapToScene(m_hotPointFrom->getHotPointPos());
    pointB = m_taskTo->mapToScene(m_hotPointTo->getHotPointPos());
  }


  qreal Pi = 3.14;
  qreal arrowSize = 7;

  QPointF lineCenter = QRectF(pointA, pointB).normalized().center();

  QLineF line(QPointF(lineCenter.x(), pointB.y()), pointB);

   double angle = acos(line.dx() / line.length());

  if (line.dy() >= 0)
  {
    angle = (Pi * 2) - angle;
  }

  QPointF arrowP1 = line.p2() - QPointF(sin(angle + Pi / 3) * arrowSize, cos(angle + Pi / 3) * arrowSize);
  QPointF arrowP2 = line.p2() - QPointF(sin(angle + Pi - Pi / 3) * arrowSize, cos(angle + Pi - Pi / 3) * arrowSize);

  QPainterPath path;
  path.moveTo(line.p2());
  path.lineTo(arrowP1);
  path.lineTo(arrowP2);
  path.lineTo(line.p2());

  return path;
}

