/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*! 
  \file terralib/process/qt/HotPointMimeData.h

  \brief A Mime Data to represents a hot point from Graphic Item.
*/

#ifndef __TERRALIB_PROCESS_QT_INTERNAL_HOTPOINTMIMEDATA_H
#define __TERRALIB_PROCESS_QT_INTERNAL_HOTPOINTMIMEDATA_H

#include "../../Config.h"

// Qt Includes
#include <QMimeData>

namespace te
{
  namespace process
  {
    namespace qt
    {
      class HotPointGraphicsItem;
      class TaskGraphicsItem;

      /*!
      \class HotPointMimeData

      \brief A Mime Data to represents a hot point from Graphic Item.

      */
      class TEPROCESSEXPORT HotPointMimeData : public QMimeData
      {
      public:

        /*! \brief Default constructor. */
        HotPointMimeData(TaskGraphicsItem* task, HotPointGraphicsItem* hotPoint);

        /*! \brief Virtual destructor. */
        ~HotPointMimeData();

        /*!
          \brief Function used to get the hotpoint for this mime data

          \return A valid pointer to a HotPointGraphicsItem object.
        */
        HotPointGraphicsItem* getHotPoint() const;

        /*!
          \brief Function used to get the task for this mime data

          \return A valid pointer to a TaskGraphicsItem object.
        */
        TaskGraphicsItem* getTask() const;

      protected:

        HotPointGraphicsItem* m_hotPoint;   //!< Pointer to hotpoint graphic item.

        TaskGraphicsItem* m_task;           //!< Pointer to task graphic item.
      };
    }
  }
}

#endif // __TERRALIB_PROCESS_QT_INTERNAL_HOTPOINTMIMEDATA_H
