/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*! 
  \file terralib/process/qt/ProcessGraphicsScene.h

  \brief A QGraphicsScebe to display a Process Builder items.
*/

#ifndef __TERRALIB_PROCESS_QT_INTERNAL_PROCESSGRAPHICSSCENE_H
#define __TERRALIB_PROCESS_QT_INTERNAL_PROCESSGRAPHICSSCENE_H

#include "../../Config.h"

#include "../../Task.h"

// Qt Includes
#include <QGraphicsScene>

// STL
#include <string>

namespace te
{
  namespace process
  {
    namespace qt
    {
      class ProcessManager;

      class ConnectorGraphicsItem;
      class HotPointGraphicsItem;
      class TaskGraphicsItem;
      
      /*!
      \class ProcessGraphicsScene

      \brief A Abstract QTreeWidgetItem Group of Process Builder items.

      */
      class TEPROCESSEXPORT ProcessGraphicsScene : public QGraphicsScene
      {
        Q_OBJECT

      public:

        /*! \brief Default constructor. */
        ProcessGraphicsScene(ProcessManager* manager, QObject *parent = nullptr);

        /*! \brief Virtual destructor. */
        ~ProcessGraphicsScene();

      public:

        void createTaskItem(te::process::TaskSharedPtr task, QPointF pos = QPointF());

        void createConnectorItem(te::process::qt::TaskGraphicsItem* taskFromItem, HotPointGraphicsItem* hotPointFromItem, 
                                 te::process::qt::TaskGraphicsItem* taskToItem, HotPointGraphicsItem* hotPointToItem);

        void alignSelecteds(Qt::Alignment alignment);

      public slots:

        void onRemovedTask(const std::string& taskId);

        void onResetedTaskId(const std::string& id, const std::string& newId);

        void onRemovedConnector(const std::string& connectorId);

      protected:

        /*!
          \brief Virtual function to reimplement qt event.
        */
        virtual void dragEnterEvent(QGraphicsSceneDragDropEvent* event) override;

        /*!
          \brief Virtual function to reimplement qt event.
        */
        virtual void dragMoveEvent(QGraphicsSceneDragDropEvent *event) override;

        /*!
          \brief Virtual function to reimplement qt event.
        */
        virtual void dragLeaveEvent(QGraphicsSceneDragDropEvent *event) override;

        /*!
          \brief Virtual function to reimplement qt event.
        */
        virtual void dropEvent(QGraphicsSceneDragDropEvent* event) override;

        /*!
          \brief Virtual function to reimplement qt event.
        */
        virtual void keyPressEvent(QKeyEvent* event) override;

      protected:

        ProcessManager* m_manager;
      };
    }
  }
}

#endif // __TERRALIB_PROCESS_QT_INTERNAL_PROCESSGRAPHICSSCENE_H
