/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*! 
  \file terralib/process/qt/ProcessGraphicsView.h

  \brief A QGraphicsVIew to display a Process Builder items.
*/

#ifndef __TERRALIB_PROCESS_QT_INTERNAL_PROCESSGRAPHICSVIEW_H
#define __TERRALIB_PROCESS_QT_INTERNAL_PROCESSGRAPHICSVIEW_H

#include "../../Config.h"

// Qt Includes
#include <QGraphicsScene>
#include <QGraphicsView>

namespace te
{
  namespace process
  {
    namespace qt
    {
      /*!
      \class ProcessGraphicsView

      \brief A QGraphicsVIew to display a Process Builder items.

      */
      class TEPROCESSEXPORT ProcessGraphicsView : public QGraphicsView
      {
        Q_OBJECT

      public:

        /*! \brief Default constructor. */
        ProcessGraphicsView(QGraphicsScene* scene, QWidget* parent = nullptr);

        /*! \brief Virtual destructor. */
        ~ProcessGraphicsView();

      protected:

        /*! \brief Function used to controll the view scale. Used to apply zoom to graphics view. */
        void wheelEvent(QWheelEvent *event) override;
      };
    }
  }
}

#endif // __TERRALIB_PROCESS_QT_INTERNAL_PROCESSGRAPHICSVIEW_H
