#include "ConnectorsProperty.h"

#include "../ProcessManager.h"

// TerraLib
#include "../../Connector.h"
#include "../../Process.h"
#include "../../Task.h"
#include "../../TaskCapabilities.h"

#include "../../../qt/widgets/Utils.h"

// Qt
#include <QFrame>
#include <QGridLayout>
#include <QHeaderView>
#include <QLabel>
#include <QTableWidget>
#include <QToolButton>

te::process::qt::ConnectorsProperty::ConnectorsProperty(ProcessManager* manager, QWidget *parent) :
  QWidget(parent),
  m_manager(manager)
{
  this->setMinimumWidth(400);

  //create connectors table
  QStringList headerLabels;

  m_connectorsTable = new QTableWidget(this);
  m_connectorsTable->setColumnCount(6);
  m_connectorsTable->setAlternatingRowColors(true);
  m_connectorsTable->setSelectionMode(QAbstractItemView::NoSelection);
  m_connectorsTable->verticalHeader()->setVisible(false);
  m_connectorsTable->horizontalHeader()->setVisible(true);
  m_connectorsTable->resizeColumnsToContents();
  m_connectorsTable->horizontalHeader()->setSectionResizeMode(QHeaderView::Stretch);

  QTableWidgetItem* itemTool = new QTableWidgetItem(QIcon::fromTheme("preferences-system"), "");
  m_connectorsTable->setHorizontalHeaderItem(0, itemTool);

  QTableWidgetItem* itemDir = new QTableWidgetItem(QIcon::fromTheme("connector_direction"), "");
  m_connectorsTable->setHorizontalHeaderItem(1, itemDir);

  QTableWidgetItem* itemSender = new QTableWidgetItem(QIcon::fromTheme("connector_sender"), "");
  m_connectorsTable->setHorizontalHeaderItem(2, itemSender);

  QTableWidgetItem* itemParamSender = new QTableWidgetItem(QIcon::fromTheme("parameter_sender"), "");
  m_connectorsTable->setHorizontalHeaderItem(3, itemParamSender);

  QTableWidgetItem* itemReceiver = new QTableWidgetItem(QIcon::fromTheme("connector_receiver"), "");
  m_connectorsTable->setHorizontalHeaderItem(4, itemReceiver);

  QTableWidgetItem* itemParamReceiver = new QTableWidgetItem(QIcon::fromTheme("parameter_receiver"), "");
  m_connectorsTable->setHorizontalHeaderItem(5, itemParamReceiver);


  //create main layout
  QGridLayout* layout = new QGridLayout(this);
  layout->setContentsMargins(0, 0, 0, 0);
  layout->setVerticalSpacing(2);
  setLayout(layout);

  layout->addWidget(m_connectorsTable, 0, 0);
}

te::process::qt::ConnectorsProperty::~ConnectorsProperty() = default;

void te::process::qt::ConnectorsProperty::setTask(const std::string& taskId)
{
  m_taskId = taskId;
  
  listConnectors();
}

void te::process::qt::ConnectorsProperty::listConnectors()
{
  //clear table
  m_connectorsTable->setRowCount(0);

  te::process::TaskSharedPtr task = m_manager->getProcess()->getTask(m_taskId);

  // list
  std::vector<te::process::Connector*> inptuConn = task->getInputConnectors();
  listConnectorsVec(inptuConn, true);

  std::vector<te::process::Connector*> outputConn = task->getOutputConnectors();
  listConnectorsVec(outputConn, false);
}

void te::process::qt::ConnectorsProperty::listConnectorsVec(std::vector<te::process::Connector*>& connectors, bool inputConnectors)
{
  std::string directionInfo = "connector_sender";

  if (inputConnectors)
  {
    directionInfo = "connector_receiver";
  }

  for (std::size_t i = 0; i < connectors.size(); ++i)
  {
    int newrow = m_connectorsTable->rowCount();
    m_connectorsTable->insertRow(newrow);

    Connector* conn = connectors[i];

    //A ToolButton used to remove a scene from a project
    QToolButton* removeButton = new QToolButton(m_connectorsTable);
    removeButton->setAutoRaise(true);
    removeButton->setIcon(QIcon::fromTheme("list-remove"));

    connect
    (
      removeButton, &QToolButton::clicked, [this, conn]()
      {
        removeConnector(conn);
      }
    );

    m_connectorsTable->setCellWidget(newrow, 0, removeButton);

    QTableWidgetItem* itemDir = new QTableWidgetItem(QIcon::fromTheme(directionInfo.c_str()), "");
    m_connectorsTable->setItem(newrow, 1, itemDir);


    QTableWidgetItem* itemTaskFrom = new QTableWidgetItem(te::qt::widgets::Convert2QString(conn->getTaskFrom()->getId()));
    m_connectorsTable->setItem(newrow, 2, itemTaskFrom);


    te::process::ParameterDescriptor paramDescFrom = conn->getTaskFrom()->getCapabilities()->getParameterDescriptor(conn->getParamIndexFrom());

    QTableWidgetItem* itemParamFrom = new QTableWidgetItem(te::qt::widgets::Convert2QString(paramDescFrom.getName()));
    m_connectorsTable->setItem(newrow, 3, itemParamFrom);

    QTableWidgetItem* itemTaskTo = new QTableWidgetItem(te::qt::widgets::Convert2QString(conn->getTaskTo()->getId()));
    m_connectorsTable->setItem(newrow, 4, itemTaskTo);

    te::process::ParameterDescriptor paramDescTo = conn->getTaskTo()->getCapabilities()->getParameterDescriptor(conn->getParamIndexTo());

    QTableWidgetItem* itemParamTo = new QTableWidgetItem(te::qt::widgets::Convert2QString(paramDescTo.getName()));
    m_connectorsTable->setItem(newrow, 5, itemParamTo);
  }
}

void te::process::qt::ConnectorsProperty::removeConnector(te::process::Connector* connector)
{
  m_manager->removeConnector(connector->getId());

  listConnectors();
}
