/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*! 
  \file terralib/process/qt/ConnectorsProperty.h

  \brief A QWidget to explore connectors from a task.
*/

#ifndef __TERRALIB_PROCESS_QT_INTERNAL_CONNECTORSPROPERTY_H
#define __TERRALIB_PROCESS_QT_INTERNAL_CONNECTORSPROPERTY_H

#include "../../Config.h"

// STL Includes
#include <string>

// Qt Includes
#include <QTableWidget>
#include <QWidget>

namespace te
{
  namespace process
  {
    class Connector;

    namespace qt
    {
      class ProcessManager;

      /*!
      \class ConnectorsProperty

      \brief A QWidget to explore connectors from a task.

      */
      class TEPROCESSEXPORT ConnectorsProperty : public QWidget
      {
        Q_OBJECT

      public:

        /*! \brief Default constructor. */
        ConnectorsProperty(ProcessManager* manager, QWidget *parent = nullptr);

        /*! \brief Virtual destructor. */
        ~ConnectorsProperty();

        /*! \brief Function used to set the Task. */
        void setTask(const std::string& taskId);

      protected:

        /*! \brief Function used to list connectors associated to internal task. */
        void listConnectors();

        /*! \brief Function used to list connectors associated to internal task. */
        void listConnectorsVec(std::vector<te::process::Connector*>& connectors, bool inputConnectors);

        /*! \brief Function used to remove a connector. */
        void removeConnector(te::process::Connector* connector);

      protected:

        ProcessManager* m_manager;

        std::string m_taskId;               //!< Task object id.

        QTableWidget* m_connectorsTable;    //!< Qt table component used to list the task connectors.
      };
    }
  }
}

#endif // __TERRALIB_PROCESS_QT_INTERNAL_CONNECTORSPROPERTY_H
