#include "DataAccessTaskPropertyHandler.h"

#include "../ProcessManager.h"

#include "../../Connector.h"
#include "../../DataAccessTaskProcess.h"
#include "../../Process.h"
#include "../../SimpleDataParameterValue.h"
#include "../../Task.h"
#include "../../TaskCapabilities.h"

// TerraLib Includes
#include "../../../datatype/SimpleData.h"
#include "../../../datatype/Utils.h"
#include "../../../dataaccess/datasource/DataSourceInfoManager.h"
#include "../../../dataaccess/utils/DataAccess.h"
#include "../../../qt/widgets/Utils.h"
#include "../../../qt/widgets/dataset/selector/DataSetSelectorDialog.h"
#include "../../../qt/widgets/datasource/core/DataSourceType.h"
#include "../../../qt/widgets/datasource/core/DataSourceTypeManager.h"
#include "../../../qt/widgets/datasource/connector/AbstractDataSourceConnector.h"
#include "../../../qt/widgets/propertybrowser/AbstractPropertyManager.h"

te::process::qt::DataAccessTaskPropertyHandler::DataAccessTaskPropertyHandler()
  : TaskPropertyHandler("")
  , m_dataSourceConnectorProperty(nullptr)
  , m_dataSetConnectorProperty(nullptr)
  , m_dataAccessProperty(nullptr)
{
  connect(te::qt::widgets::AbstractPropertyManager::getInstance().m_dlgFactory, SIGNAL(internalDlg(QWidget*, QtProperty*)), this, SLOT(setDlg(QWidget*, QtProperty*)));
}

te::process::qt::DataAccessTaskPropertyHandler::~DataAccessTaskPropertyHandler() = default;

void te::process::qt::DataAccessTaskPropertyHandler::setDefaultProperties(QtProperty* parent)
{
  TaskPropertyHandler::setDefaultProperties(parent);

  te::process::TaskSharedPtr task = m_manager->getProcess()->getTask(m_taskId);

  te::process::DataAccessTask* daTask = dynamic_cast<te::process::DataAccessTask*>(task.get());

  if (daTask)
  {
    QtProperty* type = te::qt::widgets::AbstractPropertyManager::getInstance().m_stringManager->addProperty(te::qt::widgets::Convert2QString("DataSource Type"));
    type->setEnabled(false);
    parent->addSubProperty(type);

    te::qt::widgets::AbstractPropertyManager::getInstance().m_stringManager->setValue(type, te::qt::widgets::Convert2QString(daTask->getDataSourceType())); //type
  }
}

QtProperty* te::process::qt::DataAccessTaskPropertyHandler::createProperty(te::process::TaskSharedPtr task, te::process::ParameterDescriptor& paramDesc)
{
  QtProperty* prop = nullptr;

  if (paramDesc.getName() == "Data Source URI")
  {
    m_dataSourceConnectorProperty = te::qt::widgets::AbstractPropertyManager::getInstance().m_strDlgManager->addProperty(te::qt::widgets::Convert2QString(paramDesc.getName()));

    prop = m_dataSourceConnectorProperty;
  }
  else if (paramDesc.getName() == "Data Set Name")
  {
    m_dataSetConnectorProperty = te::qt::widgets::AbstractPropertyManager::getInstance().m_strDlgManager->addProperty(te::qt::widgets::Convert2QString(paramDesc.getName()));

    prop = m_dataSetConnectorProperty;
  }
  else if (paramDesc.getName() == "Data Access")
  {
    m_dataAccessProperty = te::qt::widgets::AbstractPropertyManager::getInstance().m_stringManager->addProperty(te::qt::widgets::Convert2QString(paramDesc.getName()));

    prop = m_dataAccessProperty;
  }

  return prop;
}

void te::process::qt::DataAccessTaskPropertyHandler::setPropertyData(const te::dt::AbstractData* data, QtProperty* property, int dataType)
{
  if (property == m_dataSourceConnectorProperty)
  {
    const te::dt::SimpleData<std::string, te::dt::STRING_TYPE>* castData = dynamic_cast<const te::dt::SimpleData<std::string, te::dt::STRING_TYPE>*>(data);

    std::string val = castData->getValue();

    te::qt::widgets::AbstractPropertyManager::getInstance().m_strDlgManager->setValue(property, te::qt::widgets::Convert2QString(val));

    if (m_dsInfo.get() == nullptr)
    {
      m_dsInfo = te::da::DataSourceInfoManager::getInstance().getByConnInfo(val);
    }
  }
  else if (property == m_dataSetConnectorProperty)
  {
    const te::dt::SimpleData<std::string, te::dt::STRING_TYPE>* castData = dynamic_cast<const te::dt::SimpleData<std::string, te::dt::STRING_TYPE>*>(data);

    std::string val = castData->getValue();

    te::qt::widgets::AbstractPropertyManager::getInstance().m_strDlgManager->setValue(property, te::qt::widgets::Convert2QString(val));
  }
  else if (property == m_dataAccessProperty)
  {
    const te::da::DataAccess* da = dynamic_cast<const te::da::DataAccess*>(data);

    if (da)
    {
      std::string daStr = da->toString();

      te::qt::widgets::AbstractPropertyManager::getInstance().m_stringManager->setValue(property, te::qt::widgets::Convert2QString(daStr));
    }
    else
    {
      te::qt::widgets::AbstractPropertyManager::getInstance().m_stringManager->clear();
    }
  }
}

void te::process::qt::DataAccessTaskPropertyHandler::setDlg(QWidget* parent, QtProperty* prop)
{
  if (prop == m_dataSourceConnectorProperty)
  {
    connect(parent, SIGNAL(showDlg()), this, SLOT(showDataSourceConnector()));
  }
  else if (prop == m_dataSetConnectorProperty)
  {
    connect(parent, SIGNAL(showDlg()), this, SLOT(showDataSetSelector()));
  }
}

void te::process::qt::DataAccessTaskPropertyHandler::showDataSourceConnector()
{
  te::process::TaskSharedPtr task = m_manager->getProcess()->getTask(m_taskId);

  te::process::DataAccessTask* daTask = dynamic_cast<te::process::DataAccessTask*>(task.get());

  const te::qt::widgets::DataSourceType* dsType = te::qt::widgets::DataSourceTypeManager::getInstance().get(daTask->getDataSourceType().c_str());

  std::unique_ptr<QWidget> connectorw(dsType->getWidget(te::qt::widgets::DataSourceType::WIDGET_DATASOURCE_CONNECTOR, nullptr));

  te::qt::widgets::AbstractDataSourceConnector* connector = dynamic_cast<te::qt::widgets::AbstractDataSourceConnector*>(connectorw.get());

  std::list<te::da::DataSourceInfoPtr> datasources;

  connector->connect(datasources);

  if (datasources.size() == 1)
  {
    m_dsInfo = *datasources.begin();

    std::string uri = m_dsInfo->getConnInfoAsString();

    te::qt::widgets::AbstractPropertyManager::getInstance().m_strDlgManager->setValue(m_dataSourceConnectorProperty, uri.c_str());
  }
}

void te::process::qt::DataAccessTaskPropertyHandler::showDataSetSelector()
{
  if (m_dsInfo.get() == nullptr)
  {
    return;
  }

  std::unique_ptr<te::qt::widgets::DataSetSelectorDialog> ldialog(new te::qt::widgets::DataSetSelectorDialog(nullptr));

  ldialog->set(m_dsInfo, true);

  int retval = ldialog->exec();

  if (retval == QDialog::Accepted)
  {
    std::list<te::da::DataSetTypePtr> datasets = ldialog->getCheckedDataSets();

    if (datasets.empty() == false)
    {
      te::da::DataSetTypePtr dsType = *datasets.begin();

      te::qt::widgets::AbstractPropertyManager::getInstance().m_strDlgManager->setValue(m_dataSetConnectorProperty, dsType->getName().c_str());
    }
  }
}

te::process::qt::DataAccessTaskPropertyHandlerFactory::DataAccessTaskPropertyHandlerFactory() : te::process::qt::TaskPropertyHandlerFactory("DataAccessTask")
{
}

te::process::qt::DataAccessTaskPropertyHandlerFactory::~DataAccessTaskPropertyHandlerFactory() = default;

te::process::qt::TaskPropertyHandler* te::process::qt::DataAccessTaskPropertyHandlerFactory::build()
{
  return new DataAccessTaskPropertyHandler();
}