/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*! 
  \file terralib/process/qt/DataAccessTaskPropertyHandler.h

  \brief A data access handler to list task properties into a property browser
*/

#ifndef __TERRALIB_PROCESS_QT_INTERNAL_DATAACCESSTASKPROPERTYHANDLER_H
#define __TERRALIB_PROCESS_QT_INTERNAL_DATAACCESSTASKPROPERTYHANDLER_H

#include "../../Config.h"

#include "TaskPropertyHandler.h"

#include "../../../dataaccess/datasource/DataSourceInfo.h"

namespace te
{
  namespace dt
  {
    class AbstractData;
  }

  namespace process
  {
    class TaskSharedPtr;

    namespace qt
    {
      class ProcessManager;

      /*!
      \class DataAccessTaskPropertyHandler

      \brief A data access handler to list task properties into a property browser

      */
      class TEPROCESSEXPORT DataAccessTaskPropertyHandler : public TaskPropertyHandler
      {
        Q_OBJECT

      public:

        /*! \brief Default constructor. */
        DataAccessTaskPropertyHandler();

        /*! \brief Virtual destructor. */
        virtual ~DataAccessTaskPropertyHandler();

        void setDefaultProperties(QtProperty* parent) override;

      protected:

        /*! \brief Function used to create the right property type given a param descriptor.

          \param paramDesc  Param descriptor used to create the property

          \return QtProperty object.

        */
        virtual QtProperty* createProperty(te::process::TaskSharedPtr task, te::process::ParameterDescriptor& paramDesc) override;

        /*! \brief Function used to set into qt property a terralib data.

          \param data       TerraLib object data
          \param property   Qt property object
          \param dataType   TerraLib object data type

        */
        virtual void setPropertyData(const te::dt::AbstractData* data, QtProperty* property, int dataType) override;

      public slots:

        virtual void setDlg(QWidget* parent, QtProperty* prop);

        void showDataSourceConnector();

        void showDataSetSelector();

      protected:

        QtProperty* m_dataSourceConnectorProperty;
        QtProperty* m_dataSetConnectorProperty;
        QtProperty* m_dataAccessProperty;

        te::da::DataSourceInfoPtr m_dsInfo;
      };

      /*!
        \class TaskPropertyHandlerFactory

        \brief Abstract factory used to create Tasks property handlers.
      */
      class TEPROCESSEXPORT DataAccessTaskPropertyHandlerFactory : public TaskPropertyHandlerFactory
      {
      public:

        /*!
          \brief Constructor.

          \param factoryKey The key that identifies the factory.
        */
        DataAccessTaskPropertyHandlerFactory();

        //!< Destructor
        virtual ~DataAccessTaskPropertyHandlerFactory();

      protected:

        TaskPropertyHandler* build() override;

      };
    }
  }
}

#endif // __TERRALIB_PROCESS_QT_INTERNAL_DATAACCESSTASKPROPERTYHANDLER_H
