#include "DefaultTaskPropertyHandler.h"

#include "../../Connector.h"
#include "../../Process.h"
#include "../../SimpleDataParameterValue.h"
#include "../../Task.h"
#include "../../TaskCapabilities.h"

#include "../ProcessManager.h"

#include "../../../datatype/SimpleData.h"
#include "../../../datatype/Utils.h"
#include "../../../qt/widgets/Utils.h"
#include "../../../qt/widgets/propertybrowser/AbstractPropertyManager.h"


te::process::qt::DefaultTaskPropertyHandler::DefaultTaskPropertyHandler()
  : TaskPropertyHandler("")
{

}

te::process::qt::DefaultTaskPropertyHandler::~DefaultTaskPropertyHandler() = default;

QtProperty* te::process::qt::DefaultTaskPropertyHandler::createProperty(te::process::TaskSharedPtr task, te::process::ParameterDescriptor& paramDesc)
{
  QtProperty* prop = nullptr;

  int dataType = paramDesc.getDataType();

  switch (dataType)
  {
    case te::dt::DOUBLE_TYPE:
    {
      prop = te::qt::widgets::AbstractPropertyManager::getInstance().m_doubleManager->addProperty(te::qt::widgets::Convert2QString(paramDesc.getName()));
      break;
    }
    case te::dt::STRING_TYPE:
    {
      prop = te::qt::widgets::AbstractPropertyManager::getInstance().m_stringManager->addProperty(te::qt::widgets::Convert2QString(paramDesc.getName()));
      break;
    }
    case te::dt::INT16_TYPE:
    case te::dt::INT32_TYPE:
    case te::dt::INT64_TYPE:
    {
      prop = te::qt::widgets::AbstractPropertyManager::getInstance().m_intManager->addProperty(te::qt::widgets::Convert2QString(paramDesc.getName()));
      break;
    }
    case te::dt::UINT16_TYPE:
    case te::dt::UINT32_TYPE:
    case te::dt::UINT64_TYPE:
    {
      prop = te::qt::widgets::AbstractPropertyManager::getInstance().m_intManager->addProperty(te::qt::widgets::Convert2QString(paramDesc.getName()));
      break;
    }
    default:
    {
      throw te::common::Exception(TE_TR("Unexpected data type!"));
    }
  }

  return prop;
}

void te::process::qt::DefaultTaskPropertyHandler::setPropertyData(const te::dt::AbstractData* data, QtProperty* property, int dataType)
{
  switch (dataType)
  {
    case te::dt::DOUBLE_TYPE:
    {
      const te::dt::SimpleData<double, te::dt::DOUBLE_TYPE>* castData = dynamic_cast<const te::dt::SimpleData<double, te::dt::DOUBLE_TYPE>*>(data);

      double val = castData->getValue();

      te::qt::widgets::AbstractPropertyManager::getInstance().m_doubleManager->setValue(property, val);

      break;
    }
    case te::dt::STRING_TYPE:
    {
      const te::dt::SimpleData<std::string, te::dt::STRING_TYPE>* castData = dynamic_cast<const te::dt::SimpleData<std::string, te::dt::STRING_TYPE>*>(data);

      std::string val = castData->getValue();

      te::qt::widgets::AbstractPropertyManager::getInstance().m_stringManager->setValue(property, te::qt::widgets::Convert2QString(val));

      break;
    }
    case te::dt::INT16_TYPE:
    case te::dt::INT32_TYPE:
    case te::dt::INT64_TYPE:
    {
      const te::dt::SimpleData<int, te::dt::INT32_TYPE>* castData = dynamic_cast<const te::dt::SimpleData<int, te::dt::INT32_TYPE>*>(data);

      int val = castData->getValue();

      te::qt::widgets::AbstractPropertyManager::getInstance().m_intManager->setValue(property, val);

      break;
    }
    case te::dt::UINT16_TYPE:
    case te::dt::UINT32_TYPE:
    case te::dt::UINT64_TYPE:
    {
      const te::dt::SimpleData<int, te::dt::INT32_TYPE>* castData = dynamic_cast<const te::dt::SimpleData<int, te::dt::INT32_TYPE>*>(data);

      int val = castData->getValue();

      te::qt::widgets::AbstractPropertyManager::getInstance().m_intManager->setValue(property, val);

      break;
    }
    default:
    {
      throw te::common::Exception(TE_TR("Unexpected data type!"));
    }
  }
}

te::process::qt::DefaultTaskPropertyHandlerFactory::DefaultTaskPropertyHandlerFactory() : te::process::qt::TaskPropertyHandlerFactory("DefaultTaskPropertyHandler")
{

}

te::process::qt::DefaultTaskPropertyHandlerFactory::~DefaultTaskPropertyHandlerFactory() = default;

te::process::qt::TaskPropertyHandler* te::process::qt::DefaultTaskPropertyHandlerFactory::build()
{
  return new DefaultTaskPropertyHandler();
}