
#include "GraphicsPropertyItem.h"

// TerraLib Includes
#include "../../../qt/widgets/propertybrowser/AbstractPropertyManager.h"
#include "../../../qt/widgets/Utils.h"

// Qt Includes
#include <QPainter>

te::process::qt::GraphicsPropertyItem::GraphicsPropertyItem(QtTreePropertyBrowser* pb, QColor c)
  : te::qt::widgets::AbstractPropertyItem(pb, c)
  , m_item(nullptr)
{
  m_penStyleMap[Qt::NoPen] = "No Pen";
  m_penStyleMap[Qt::SolidLine] = "Solid Line";
  m_penStyleMap[Qt::DotLine] = "Dot Line";
  m_penStyleMap[Qt::DashLine] = "Dash Line";
  m_penStyleMap[Qt::DashDotLine] = "Dash Dot Line";
  m_penStyleMap[Qt::DashDotDotLine] = "Dash Dot Dot Line";

  m_brushStyleMap[Qt::NoBrush] = "No Brush";
  m_brushStyleMap[Qt::SolidPattern] = "Solid";
  m_brushStyleMap[Qt::Dense1Pattern] = "Extremely dense brush pattern";
  m_brushStyleMap[Qt::Dense2Pattern] = "Very dense brush pattern";
  m_brushStyleMap[Qt::Dense3Pattern] = "Somewhat dense brush pattern";
  m_brushStyleMap[Qt::Dense4Pattern] = "Half dense brush pattern";
  m_brushStyleMap[Qt::Dense5Pattern] = "Somewhat sparse brush pattern";
  m_brushStyleMap[Qt::Dense6Pattern] = "Very sparse brush pattern";
  m_brushStyleMap[Qt::Dense7Pattern] = "Extremely sparse brush pattern";
  m_brushStyleMap[Qt::HorPattern] = "Horizontal lines";
  m_brushStyleMap[Qt::VerPattern] = "Vertical lines";
  m_brushStyleMap[Qt::CrossPattern] = "Crossing horizontal and vertical lines";
  m_brushStyleMap[Qt::BDiagPattern] = "Backward diagonal lines";
  m_brushStyleMap[Qt::FDiagPattern] = "Forward diagonal lines";
  m_brushStyleMap[Qt::DiagCrossPattern] = "Crossing diagonal lines";
  
  buildProperties();

  //connects
  //connect(m_manager, SIGNAL(removedTask(const std::string&)), this, SLOT(onRemovedTask(const std::string&)));
}

te::process::qt::GraphicsPropertyItem::~GraphicsPropertyItem() = default;

void te::process::qt::GraphicsPropertyItem::setItem(AbstractGraphicsItem* item)
{
  m_item = item;

  m_enableListeners = false;

  //clearProperties();

  QPen pen = item->getPen();
  QBrush brush = item->getBrush();
  QFont font = item->getFont();

  //pen
  te::qt::widgets::AbstractPropertyManager::getInstance().m_colorManager->setValue(m_colorPenProperty, pen.color());
  te::qt::widgets::AbstractPropertyManager::getInstance().m_intManager->setValue(m_widthPenProperty, pen.width());
  AbstractPropertyItem::setValueToEnumProperty(m_stylePenProperty, m_penStyleMap[pen.style()].c_str());

  //brush
  te::qt::widgets::AbstractPropertyManager::getInstance().m_colorManager->setValue(m_colorBrushProperty, brush.color());
  AbstractPropertyItem::setValueToEnumProperty(m_styleBrushProperty, m_brushStyleMap[brush.style()].c_str());

  //font
  te::qt::widgets::AbstractPropertyManager::getInstance().m_fontManager->setValue(m_fontProperty, font);


  m_enableListeners = true;
}

void te::process::qt::GraphicsPropertyItem::buildProperties()
{
  //pen
  m_penItemProperty = te::qt::widgets::AbstractPropertyManager::getInstance().m_groupManager->addProperty(tr("Graphic Pen"));
  addProperty(m_penItemProperty, QColor(179, 226, 221));

  m_widthPenProperty = te::qt::widgets::AbstractPropertyManager::getInstance().m_intManager->addProperty(tr("Width"));
  te::qt::widgets::AbstractPropertyManager::getInstance().m_intManager->setMinimum(m_widthPenProperty, 0);
  te::qt::widgets::AbstractPropertyManager::getInstance().m_intManager->setMaximum(m_widthPenProperty, 10);
  te::qt::widgets::AbstractPropertyManager::getInstance().m_intManager->setSingleStep(m_widthPenProperty, 1);

  m_colorPenProperty = te::qt::widgets::AbstractPropertyManager::getInstance().m_colorManager->addProperty(tr("Color"));

  m_stylePenProperty = te::qt::widgets::AbstractPropertyManager::getInstance().m_enumManager->addProperty(tr("Style"));
  te::qt::widgets::AbstractPropertyManager::getInstance().m_enumManager->setEnumNames(m_stylePenProperty, getStylePenNames());
  te::qt::widgets::AbstractPropertyManager::getInstance().m_enumManager->setEnumIcons(m_stylePenProperty, getStylePenIcons());
  
  m_penItemProperty->addSubProperty(m_colorPenProperty);
  m_penItemProperty->addSubProperty(m_widthPenProperty);
  m_penItemProperty->addSubProperty(m_stylePenProperty);

  //brush
  m_brushItemProperty = te::qt::widgets::AbstractPropertyManager::getInstance().m_groupManager->addProperty(tr("Graphic Brush"));
  addProperty(m_brushItemProperty, QColor(191, 213, 232));

  m_colorBrushProperty = te::qt::widgets::AbstractPropertyManager::getInstance().m_colorManager->addProperty(tr("Color"));

  m_styleBrushProperty = te::qt::widgets::AbstractPropertyManager::getInstance().m_enumManager->addProperty(tr("Style"));
  te::qt::widgets::AbstractPropertyManager::getInstance().m_enumManager->setEnumNames(m_styleBrushProperty, getStyleBrushNames());
  te::qt::widgets::AbstractPropertyManager::getInstance().m_enumManager->setEnumIcons(m_styleBrushProperty, getStyleBrushIcons());

  m_brushItemProperty->addSubProperty(m_colorBrushProperty);
  m_brushItemProperty->addSubProperty(m_styleBrushProperty);

  //text
  m_textItemProperty = te::qt::widgets::AbstractPropertyManager::getInstance().m_groupManager->addProperty(tr("Graphic Text"));
  addProperty(m_textItemProperty, QColor(253, 222, 238));

  m_fontProperty = te::qt::widgets::AbstractPropertyManager::getInstance().m_fontManager->addProperty(tr("Font"));
  m_textItemProperty->addSubProperty(m_fontProperty);
}

void te::process::qt::GraphicsPropertyItem::clearProperties()
{
  m_propertyBrowser->removeProperty(m_penItemProperty);
  m_propertyBrowser->removeProperty(m_brushItemProperty);
  m_propertyBrowser->removeProperty(m_textItemProperty);

  delete m_penItemProperty;
  delete m_brushItemProperty;
  delete m_textItemProperty;

  buildProperties();
}

QStringList te::process::qt::GraphicsPropertyItem::getStylePenNames()
{
  QStringList list;
  
  std::map<Qt::PenStyle, std::string>::iterator it = m_penStyleMap.begin();

  while (it != m_penStyleMap.end())
  {
    list.append(it->second.c_str());

    ++it;
  }

  return list;
}

QMap<int, QIcon> te::process::qt::GraphicsPropertyItem::getStylePenIcons()
{
  QMap<int, QIcon> iconMap;

  QSize size(32, 16);
  QLine line(0, size.height() * 0.5, size.width(), size.height() * 0.5);
  QPen pen;
  pen.setWidth(2);

  std::map<Qt::PenStyle, std::string>::iterator it = m_penStyleMap.begin();

  int count = 0;

  while (it != m_penStyleMap.end())
  {
    pen.setStyle(it->first);

    QPixmap pixmap(size);
    pixmap.fill(Qt::transparent);

    QPainter painter(&pixmap);
    painter.setPen(pen);
    painter.drawLine(line);

    QIcon icon(pixmap);
    iconMap[count] = icon;

    ++count;
    ++it;
  }

  return iconMap;
}

QStringList te::process::qt::GraphicsPropertyItem::getStyleBrushNames()
{
  QStringList list;

  std::map<Qt::BrushStyle, std::string>::iterator it = m_brushStyleMap.begin();

  while (it != m_brushStyleMap.end())
  {
    list.append(it->second.c_str());

    ++it;
  }

  return list;
}

QMap<int, QIcon> te::process::qt::GraphicsPropertyItem::getStyleBrushIcons()
{
  QMap<int, QIcon> iconMap;

  QSize size(32, 16);

  std::map<Qt::BrushStyle, std::string>::iterator it = m_brushStyleMap.begin();

  int count = 0;

  while (it != m_brushStyleMap.end())
  {
    QPixmap pixmap(size);
    pixmap.fill(Qt::transparent);

    QPainter painter(&pixmap);
    painter.setBrush(it->first);
    painter.drawRect(0, 0, size.width() -1, size.height() -1);

    QIcon icon(pixmap);
    iconMap[count] = icon;

    ++count;
    ++it;
  }

  return iconMap;
}

void te::process::qt::GraphicsPropertyItem::valueChanged(QtProperty* qProperty, const QVariant& qVariant)
{
  if (m_enableListeners == false)
  {
    return;
  }

  m_enableListeners = false;

  if (qProperty == m_colorPenProperty)
  {
    QColor color = qVariant.value<QColor>();
    m_item->setPenColor(color);
  }
  else if (qProperty == m_widthPenProperty)
  {
    int width = qVariant.toInt();
    m_item->setPenWidth(width);
  }
  else if (qProperty == m_stylePenProperty)
  {
    QString penStyle = AbstractPropertyItem::getValueFromEnumProperty(m_stylePenProperty);
    std::string penStyleStr = te::qt::widgets::Convert2StdString(penStyle);

    std::map<Qt::PenStyle, std::string>::iterator it = m_penStyleMap.begin();

    while (it != m_penStyleMap.end())
    {
      if (it->second == penStyleStr)
      {
        m_item->setPenStyle(it->first);
        break;
      }

      ++it;
    }
  }
  else if (qProperty == m_colorBrushProperty)
  {
    QColor color = qVariant.value<QColor>();
    m_item->setBrushColor(color);
  }
  else if (qProperty == m_styleBrushProperty)
  {
    QString brushStyle = AbstractPropertyItem::getValueFromEnumProperty(m_styleBrushProperty);
    std::string brushStyleStr = te::qt::widgets::Convert2StdString(brushStyle);

    std::map<Qt::BrushStyle, std::string>::iterator it = m_brushStyleMap.begin();

    while (it != m_brushStyleMap.end())
    {
      if (it->second == brushStyleStr)
      {
        m_item->setBrushStyle(it->first);
        break;
      }

      ++it;
    }
  }
  else if (qProperty == m_fontProperty)
  {
    QFont font = qVariant.value<QFont>();
    m_item->setFont(font);
  }

  m_item->update();

  m_enableListeners = true;
}

void te::process::qt::GraphicsPropertyItem::onRemovedItem(AbstractGraphicsItem* item)
{
  if (item == m_item)
  {
    clearProperties();
  }
}
