/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

 /*!
   \file terralib/process/qt/property/GraphicsPropertyItem.h

   \brief A base item to represent the graphical task item properties items in property tree.
 */

#ifndef __TERRALIB_PROCESS_QT_INTERNAL_GRAPHICSPROPERTYITEM_H
#define __TERRALIB_PROCESS_QT_INTERNAL_GRAPHICSPROPERTYITEM_H

#include "../../Config.h"
#include "../graphics/AbstractGraphicsItem.h"

 // TerraLib
#include "../../../qt/widgets/propertybrowser/AbstractPropertyItem.h"

// Qt
#include <QColor>
#include <QIcon>
#include <QVariant>

// QtPropertyBrowser
#include <QtPropertyBrowser/QtProperty>



namespace te
{
  namespace process
  {
    namespace qt
    {
      /*!
        \class GraphicsPropertyItem

        \brief A base item to represent the graphical task item properties items in property tree.
      */

      class TEPROCESSEXPORT GraphicsPropertyItem : public te::qt::widgets::AbstractPropertyItem
      {
        Q_OBJECT

      public:

          /*! \brief Constructor */
        GraphicsPropertyItem(QtTreePropertyBrowser* pb, QColor c = QColor());

        /*! \brief Destructor. */
        virtual ~GraphicsPropertyItem();

        /*! \brief Function used to set the Task. */
        void setItem(AbstractGraphicsItem* item);
      
        void addListenerToProperty(QtProperty* p);

        bool propertyHasListener(QtProperty* p);

      protected:

        /*! \brief Function used build toplevel task params properties . */
        void buildProperties();

        /*! \brief Function used clear task params properties . */
        void clearProperties();

        QStringList getStylePenNames();

        QMap<int, QIcon> getStylePenIcons();

        QStringList getStyleBrushNames();

        QMap<int, QIcon> getStyleBrushIcons();

      public slots:

        virtual void valueChanged(QtProperty* p, const QVariant& qVariant) override;

        void onRemovedItem(AbstractGraphicsItem* item);

      protected:

        AbstractGraphicsItem* m_item;                //!< Terralib process graphical item object.

        QtProperty* m_penItemProperty;               //!< A qt main item for graphical pen properties
        QtProperty* m_widthPenProperty;
        QtProperty* m_colorPenProperty;
        QtProperty* m_stylePenProperty;
        
        QtProperty* m_brushItemProperty;             //!< A qt main item for graphical brush properties
        QtProperty* m_colorBrushProperty;
        QtProperty* m_styleBrushProperty;

        QtProperty* m_textItemProperty;              //!< A qt main item for graphical text properties
        QtProperty* m_fontProperty;


        std::map<Qt::PenStyle, std::string> m_penStyleMap;
        std::map<Qt::BrushStyle, std::string> m_brushStyleMap;
      };
    }
  }
}

#endif //__TERRALIB_PROCESS_QT_INTERNAL_GRAPHICSPROPERTYITEM_H
