/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*! 
  \file terralib/process/qt/ProcessInspector.h

  \brief A QWidget to inspect each process items added to process
*/

#ifndef __TERRALIB_PROCESS_QT_INTERNAL_PROCESSINSPECTOR_H
#define __TERRALIB_PROCESS_QT_INTERNAL_PROCESSINSPECTOR_H

#include "../../Config.h"
#include "../../Connector.h"
#include "../../Task.h"

// Qt Includes
#include <QTreeWidget>
#include <QTreeWidgetItem>
#include <QWidget>

// STL
#include <map>

namespace te
{
  namespace process
  {
    class Process;

    namespace qt
    {
      class ProcessManager;

      /*!
      \class ProcessInspector

      \brief A QWidget to inspect each process items added to process

      */
      class TEPROCESSEXPORT ProcessInspector : public QWidget
      {
        Q_OBJECT

      public:

        /*! \brief Default constructor. */
        ProcessInspector(ProcessManager* manager, QWidget *parent = nullptr);

        /*! \brief Virtual destructor. */
        ~ProcessInspector();

        void setProcess(te::process::Process* process);

      public slots:

        void onAddedTask(const std::string& taskId);

        void onAddedConnector(const std::string& connectorId);

        void onResetedTaskId(const std::string& id, const std::string& newId);

        void onRemovedTask(const std::string& taskId);

        void onRemovedConnector(const std::string& connectorId);

      protected:

        void listParameters(QTreeWidgetItem* taskItemParent, const te::process::Task* task);

        void listConnectors(QTreeWidgetItem* taskItemParent, const te::process::Task* task);

        void addConnector(QTreeWidgetItem* taskItemParent, te::process::Connector* connector, bool inputConnectors);

      protected:

        ProcessManager* m_manager;

        QTreeWidget* m_processTree;

        QTreeWidgetItem* m_processTreeItem;

        std::map<std::string, QTreeWidgetItem*> m_taskItemMap;
      };
    }
  }
}

#endif // __TERRALIB_PROCESS_QT_INTERNAL_PROCESSINSPECTOR_H
