/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*! 
  \file terralib/process/qt/TaskProperty.h

  \brief A QWidget to explore task properties
*/

#ifndef __TERRALIB_PROCESS_QT_INTERNAL_TASKPROPERTY_H
#define __TERRALIB_PROCESS_QT_INTERNAL_TASKPROPERTY_H

#include "../../Config.h"

#include "TaskPropertyItemListener.h"

// STL Includes
#include <string>

// Qt Includes
#include <QWidget>

namespace te
{
  namespace process
  {
    namespace qt
    {
      class AbstractGraphicsItem;
      class GraphicsPropertyItem;
      class ProcessManager;
      class TaskPropertyItem;

      /*!
      \class TaskProperty

      \brief A QWidget to explore task properties

      */
      class TEPROCESSEXPORT TaskProperty : public QWidget, public TaskPropertyItemListener
      {
        Q_OBJECT

      public:

        /*! \brief Default constructor. */
        TaskProperty(ProcessManager* manager, QWidget *parent = nullptr);

        /*! \brief Virtual destructor. */
        ~TaskProperty();

        /*! \brief Function used to set the Task. */
        void setTask(const std::string& taskId);

        void setTaskItem(AbstractGraphicsItem* item);

        virtual void handleValueChanged(QtProperty* qProperty, const QVariant& qVariant) override;

      protected:

        te::process::qt::GraphicsPropertyItem* m_graphicsPropertyItem;  // A property browser for graphical task properties
        te::process::qt::TaskPropertyItem* m_traskPropertyItem;         // A property browser for task properties
      };
    }
  }
}

#endif // __TERRALIB_PROCESS_QT_INTERNAL_TASKPROPERTY_H
