
#include "TaskPropertyItem.h"
#include "TaskPropertyHandler.h"
#include "TaskPropertyItemListener.h"

#include "../ProcessManager.h"

#include "../../Connector.h"
#include "../../Process.h"
#include "../../SimpleDataParameterValue.h"
#include "../../Task.h"
#include "../../TaskCapabilities.h"

// TerraLib
#include "../../../src/terralib/datatype/AbstractData.h"
#include "../../../src/terralib/datatype/SimpleData.h"
#include "../../../src/terralib/datatype/Utils.h"
#include "../../../src/terralib/qt/widgets/propertybrowser/AbstractPropertyManager.h"
#include "../../../src/terralib/qt/widgets/Utils.h"

te::process::qt::TaskPropertyItem::TaskPropertyItem(QtTreePropertyBrowser* pb, TaskPropertyItemListener* listener, ProcessManager* manager, QColor c)
  : te::qt::widgets::AbstractPropertyItem(pb, c)
  , m_manager(manager)
  , m_handler(nullptr)
  , m_listener(listener)
  
{
  buildProperties();

  //connects
  connect(m_manager, SIGNAL(removedTask(const std::string&)), this, SLOT(onRemovedTask(const std::string&)));
}

te::process::qt::TaskPropertyItem::~TaskPropertyItem()
{
  m_vecProperties.clear();

  delete m_handler;
}

void te::process::qt::TaskPropertyItem::setTask(const std::string& taskId)
{
  m_taskId = taskId;

  m_enableListeners = false;

  clearProperties();

  if (m_handler)
  {
    delete m_handler;
    m_handler = nullptr;
  }

  te::process::TaskSharedPtr task = m_manager->getProcess()->getTask(m_taskId);

  m_handler = te::process::qt::CreateTaskPropertyHandler(task->getType(), m_taskId, m_manager);

  if (m_handler)
  {
    m_handler->setDefaultProperties(m_taskItemProperty);

    m_handler->setInputProperties(m_inItemProperty);

    m_handler->setOutputProperties(m_outItemProperty);
  }

  m_enableListeners = true;
}

void te::process::qt::TaskPropertyItem::addListenerToProperty(QtProperty* p)
{
  m_vecProperties.push_back(p);
}

bool te::process::qt::TaskPropertyItem::propertyHasListener(QtProperty* p)
{
  std::vector<QtProperty*>::iterator it = std::find(m_vecProperties.begin(), m_vecProperties.end(), p);

  if (it == m_vecProperties.end())
  {
    return false;
  }

  return true;
}

void te::process::qt::TaskPropertyItem::buildProperties()
{
  m_taskItemProperty = te::qt::widgets::AbstractPropertyManager::getInstance().m_groupManager->addProperty(tr("Task Parameters"));
  addProperty(m_taskItemProperty, QColor(240, 232, 205));

  m_inItemProperty = te::qt::widgets::AbstractPropertyManager::getInstance().m_groupManager->addProperty(tr("Input Parameters"));
  addProperty(m_inItemProperty, QColor(255, 255, 176));

  m_outItemProperty = te::qt::widgets::AbstractPropertyManager::getInstance().m_groupManager->addProperty(tr("Output Parameters"));
  addProperty(m_outItemProperty, QColor(207, 236, 207));
}

void te::process::qt::TaskPropertyItem::clearProperties()
{
  //clear commun sub properties
  QList<QtProperty*> taskProperties = m_taskItemProperty->subProperties();

  for (int i = 0; i < taskProperties.size(); ++i)
  {
    m_taskItemProperty->removeSubProperty(taskProperties.at(i));
  }
  
  //clear in sub properties
  QList<QtProperty*> inProperties = m_inItemProperty->subProperties();

  for (int i = 0; i < inProperties.size(); ++i)
  {
    m_inItemProperty->removeSubProperty(inProperties.at(i));
  }

  //clear out sub properties
  QList<QtProperty*> outProperties = m_outItemProperty->subProperties();

  for (int i = 0; i < outProperties.size(); ++i)
  {
    m_outItemProperty->removeSubProperty(outProperties.at(i));
  }
}

void te::process::qt::TaskPropertyItem::valueChanged(QtProperty* qProperty, const QVariant& qVariant)
{
  if (m_listener == nullptr)
  {
    return;
  }

  if (m_enableListeners == false)
  {
    return;
  }

  m_enableListeners = false;

  m_handler->valueChanged(qProperty, qVariant);
  
  m_enableListeners = true;

  if (propertyHasListener(qProperty) )
  {
    m_listener->handleValueChanged(qProperty, qVariant);
  }
}

void te::process::qt::TaskPropertyItem::onRemovedTask(const std::string& taskId)
{
  if (m_taskId == taskId)
  {
    m_propertyBrowser->clear();

    buildProperties();
  }
}
