/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

 /*!
   \file terralib/process/qt/property/TaskPropertyItem.h

   \brief A base item to represent the process task properties items in property tree.
 */

#ifndef __TERRALIB_PROCESS_QT_INTERNAL_TASKPROPERTYITEM_H
#define __TERRALIB_PROCESS_QT_INTERNAL_TASKPROPERTYITEM_H

#include "../../Config.h"
#include "../../ParameterDescriptor.h"

 // TerraLib
#include "../../../qt/widgets/propertybrowser/AbstractPropertyItem.h"

// Qt
#include <QColor>
#include <QVariant>

// QtPropertyBrowser
#include <QtPropertyBrowser/QtProperty>

namespace te
{
  namespace dt
  {
    class AbstractData;
  }

  namespace process
  {
    namespace qt
    {
      class ProcessManager;
      class TaskPropertyItemListener;
      class TaskPropertyHandler;

      /*!
        \class TaskPropertyItem

        \brief A base item to represent the process task properties items in property tree.
      */

      class TEPROCESSEXPORT TaskPropertyItem : public te::qt::widgets::AbstractPropertyItem
      {
        Q_OBJECT

      public:

          /*! \brief Constructor */
        TaskPropertyItem(QtTreePropertyBrowser* pb, TaskPropertyItemListener* listener, ProcessManager* manager, QColor c = QColor());

        /*! \brief Destructor. */
        virtual ~TaskPropertyItem();

        /*! \brief Function used to set the Task. */
        void setTask(const std::string& taskId);
      
        void addListenerToProperty(QtProperty* p);

        bool propertyHasListener(QtProperty* p);

      protected:

        /*! \brief Function used build toplevel task params properties . */
        void buildProperties();

        /*! \brief Function used clear task params properties . */
        void clearProperties();

      public slots:

        virtual void valueChanged(QtProperty* p, const QVariant& qVariant) override;

        void onRemovedTask(const std::string& taskId);

      protected:

        ProcessManager* m_manager;

        TaskPropertyHandler* m_handler;

        TaskPropertyItemListener* m_listener;
        std::vector<QtProperty*> m_vecProperties;

        std::string m_taskId;                         //!< Terralib Task object id.

        QtProperty* m_taskItemProperty;               //!< A qt main item for default task properties
        QtProperty* m_inItemProperty;                 //!< A qt main item for input paramters from task
        QtProperty* m_outItemProperty;                //!< A qt main item for output paramters from task
      };
    }
  }
}

#endif //__TERRALIB_PROCESS_QT_INTERNAL_TASKPROPERTYITEM_H
