/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

 /*!
   \file terralib/qt/af/ApplicationDocker.h

   \brief The base API for docker controllers of TerraLib applications.
 */

#ifndef __TERRALIB_QT_AF_INTERNAL_APPLICATIONDOCKER_H
#define __TERRALIB_QT_AF_INTERNAL_APPLICATIONDOCKER_H

 // Terralib
#include "Config.h"

// STL
#include <map>


// Qt
#include <QMainWindow>

// Forward Declaration Qt
class QAction;
class QDockWidget;
class QToolBar;

namespace te
{
  namespace qt
  {
    namespace af
    {
      struct ApplicationDockerInfo
      {
        QDockWidget* m_dockWidget;
        QIcon m_icon;
        QString m_dockName;
        bool m_isCloseable;

        ApplicationDockerInfo()
        {
          m_dockWidget = nullptr;
          m_isCloseable = false;
        }
      };

      /*!
        \class ApplicationDocker

        \brief The base API for docker controllers of TerraLib applications.

        \ingroup af
      */
      class TEQTAFEXPORT ApplicationDocker : public QMainWindow
      {
        Q_OBJECT

        public:

          /*!
            \brief Constructor.

            \param parent The parent object.
          */
          ApplicationDocker(QWidget* parent, Qt::ToolBarArea area);

          /*!
            \brief Destructor.
          */
          ~ApplicationDocker();

        public:

          bool eventFilter(QObject* watched, QEvent* e);

          QSize sizeHint() const;

          QSize minimumSizeHint() const;

          bool hasActiveDock();

          bool hasDockWidget(QDockWidget* dock);

          void raiseDockWidget(QDockWidget* dock);
          
          void addDock(const ApplicationDockerInfo& dockInfo, Qt::DockWidgetArea area);

          void takeDock(ApplicationDockerInfo& dockInfo);

          void removeDock(QDockWidget* dock);

          bool isCloseable();

          int getMinimumWidth();

        protected:

          void computeMinimumSize();

          int computeGlobalMinSize();

          void resizeEvent(QResizeEvent *event);

        protected slots:

          void onDockActionVisibilityChanged(bool state);

          void onCloseDock();

        signals:

          void dockSelected(QString dockName);

          void clearDockSelected();

        protected:

          QToolBar* m_sideBar;
          
          std::map<QAction*, ApplicationDockerInfo> m_dockInfoActionMap;

          QAction* m_currentAction;

          QSize m_sizeHint;

          int m_minimumWidth;
      };
    } // end namespace af
  }   // end namespace qt
}     // end namespace te

#endif // __TERRALIB_QT_AF_INTERNAL_APPLICATIONDOCKER_H
