/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

 /*!
   \file terralib/qt/af/ApplicationDockerController.h

   \brief The base API for docker controllers of TerraLib applications.
 */

#ifndef __TERRALIB_QT_AF_INTERNAL_APPLICATIONDOCKERCONTROLLER_H
#define __TERRALIB_QT_AF_INTERNAL_APPLICATIONDOCKERCONTROLLER_H

 // Terralib
#include "Config.h"

#include "ApplicationDocker.h"
#include "ApplicationDockerTitleBar.h"

// Qt
#include <QDockWidget>
#include <QMainWindow>
#include <QMenu>
#include <QObject>

namespace te
{
  namespace qt
  {
    namespace af
    {
      /*!
        \class ApplicationDockerController

        \brief The base API for docker controllers of TerraLib applications.

        \ingroup af
      */
      class TEQTAFEXPORT ApplicationDockerController : public QObject
      {
        Q_OBJECT

        public:

          /*!
            \brief Constructor.

            \param parent The parent object.
          */
          ApplicationDockerController(QMainWindow* mainWindow);

          /*!
            \brief Destructor.
          */
          ~ApplicationDockerController();

        public:

          void addDockWidget(const ApplicationDockerInfo& dockInfo, Qt::DockWidgetArea area);

          void removeDockWidget(QDockWidget* dock);

          void addDockerVisibilityActions(QMenu* menu);

          void raiseDockWidget(QDockWidget* dock);

          static te::qt::af::ApplicationDockerInfo createAppDockInfo(QDockWidget* dock, const QIcon& icon, const QString& dockName, bool isCloseable);

        protected slots:

          void onDockSelected(QString dockName);

          void onClearDockSelected();

          void onPinDock();

          void onMoveDock();

          void onDockVisibilityChanged(bool state);

        protected:

          void startDocker(QDockWidget*& dockWidget, Qt::DockWidgetArea area, ApplicationDocker*& appDocker, ApplicationDockerTitleBar*& appDockerTitleBar, QAction*& action, QString title);

        protected:

          QMainWindow* m_mainWindow;             //!< The application main window pointer.

          ApplicationDocker* m_leftAppDocker;    //!< The left application docker

          ApplicationDocker* m_rightAppDocker;   //!< The right application docker

          ApplicationDockerTitleBar* m_leftAppDockerTitleBar;   //!< The left docker title bar widget

          ApplicationDockerTitleBar* m_rightAppDockerTitleBar;  //!< The right docker title bar widget

          QDockWidget* m_leftDockWidget;         //!< The left Qt Dock Widget

          QDockWidget* m_rightDockWidget;        //!< The right Qt Dock Widget

          QAction* m_leftDockVisibleAction;     //!< Action to controll the left dock visibility

          QAction* m_rightDockVisibleAction;    //!< Action to controll the right dock visibility
      };
    } // end namespace af
  }   // end namespace qt
}     // end namespace te

#endif // __TERRALIB_QT_AF_INTERNAL_APPLICATIONDOCKERCONTROLLER_H
