/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

 /*!
   \file terralib/qt/af/ApplicationDockerTitleBar.h

   \brief The base API for docker title bar for TerraLib Docker.
 */

#ifndef __TERRALIB_QT_AF_INTERNAL_APPLICATIONDOCKERTITLEBAR_H
#define __TERRALIB_QT_AF_INTERNAL_APPLICATIONDOCKERTITLEBAR_H

 // Terralib
#include "Config.h"

// Qt
#include <QLabel>
#include <QToolButton>
#include <QWidget>


namespace te
{
  namespace qt
  {
    namespace af
    {
      /*!
        \class ApplicationDockerTitleBar

        \brief The base API for docker title bar for TerraLib Docker.

        \ingroup af
      */
      class TEQTAFEXPORT ApplicationDockerTitleBar : public QWidget
      {
        Q_OBJECT

        public:

          /*!
            \brief Constructor.

            \param parent The parent object.
          */
          ApplicationDockerTitleBar(QWidget* parent, Qt::DockWidgetArea area);

          /*!
            \brief Destructor.
          */
          ~ApplicationDockerTitleBar();

        public:

          QSize sizeHint() const;

          QSize minimumSizeHint() const;

          void setDockInfo(QString title, bool isCloseable, int minWidth);

          void reset();

        signals:

          void pinTriggered();

          void moveTriggered();

          void closeTriggered();

        protected:

          QToolButton* m_moveToolButton;
          QToolButton* m_closeToolButton;

          QLabel* m_titleLabel;

          int m_minWidth;

      };
    } // end namespace af
  }   // end namespace qt
}     // end namespace te

#endif // __TERRALIB_QT_AF_INTERNAL_APPLICATIONDOCKER_H
