/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

This file is part of the TerraLib - a Framework for building GIS enabled applications.

TerraLib is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License,
or (at your option) any later version.

TerraLib is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with TerraLib. See COPYING. If not, write to
TerraLib Team at <terralib-team@terralib.org>.
*/

/*!
\file terralib/qt/af/ThirdPartyInfoWidget.cpp

\brief A Widget used to inform about third party versions
*/

#include "ThirdPartyInfoWidget.h"

// TerraLib Includes
#include "ThirdPartyInfo.h"
#include "../widgets/Utils.h"

// Qt Includes
#include <QGridLayout>
#include <QHeaderView>


te::qt::af::ThirdPartyInfoWidget::ThirdPartyInfoWidget(QWidget* parent, Qt::WindowFlags f) : QWidget(parent, f)
{
  buildUi();

  getThirdPartyInfo();
}

te::qt::af::ThirdPartyInfoWidget::~ThirdPartyInfoWidget() = default;


void te::qt::af::ThirdPartyInfoWidget::buildUi()
{
  QGridLayout* layout = new QGridLayout(this);

  m_thirdPartyTableWidget = new QTableWidget(this);

  layout->addWidget(m_thirdPartyTableWidget, 0, 0);

  m_thirdPartyTableWidget->setColumnCount(2);
  m_thirdPartyTableWidget->setAlternatingRowColors(true);
  m_thirdPartyTableWidget->setSelectionMode(QAbstractItemView::NoSelection);
  m_thirdPartyTableWidget->verticalHeader()->setVisible(false);

  QTableWidgetItem* libraryTableWidgetitem = new QTableWidgetItem(tr("Library"));
  m_thirdPartyTableWidget->setHorizontalHeaderItem(0, libraryTableWidgetitem);

  QTableWidgetItem* versionTableWidgetitem = new QTableWidgetItem(tr("Version"));
  m_thirdPartyTableWidget->setHorizontalHeaderItem(1, versionTableWidgetitem);
}

void te::qt::af::ThirdPartyInfoWidget::getThirdPartyInfo()
{
  //clear table
  m_thirdPartyTableWidget->setRowCount(0);

  //check qt version
  addThirdPartyInfo("Qt", te::qt::af::ThirdPartyInfo::getQtVersion());

  //check gdal version
  addThirdPartyInfo("GDAL/OGR", te::qt::af::ThirdPartyInfo::getGDALVersion());
  
  //check boost version
  addThirdPartyInfo("Boost", te::qt::af::ThirdPartyInfo::getBoostVersion());

  //check geos version
  addThirdPartyInfo("Geos", te::qt::af::ThirdPartyInfo::getGeosVersion());

  //check proj version
  addThirdPartyInfo("Proj", te::qt::af::ThirdPartyInfo::getProjVersion());

  //check qwt version
  addThirdPartyInfo("Qwt", te::qt::af::ThirdPartyInfo::getQwtVersion());

  //check SQLite version
  addThirdPartyInfo("SQLite", te::qt::af::ThirdPartyInfo::getSQLiteVersion());

  //Check PostgreSQL LibPQ version
  addThirdPartyInfo("PostgreSQL (LibPQ)", te::qt::af::ThirdPartyInfo::getPostgreSQLVersion());

  //adjust contents
  m_thirdPartyTableWidget->resizeColumnsToContents();
#if (QT_VERSION >= 0x050000)
  m_thirdPartyTableWidget->horizontalHeader()->setSectionResizeMode(QHeaderView::Stretch);
#else
  m_thirdPartyTableWidget->horizontalHeader()->setResizeMode(QHeaderView::Stretch);
#endif
}

void te::qt::af::ThirdPartyInfoWidget::addThirdPartyInfo(std::string name, std::string version)
{
  int nRows = m_thirdPartyTableWidget->rowCount();

  m_thirdPartyTableWidget->insertRow(nRows);

  QTableWidgetItem* itemName = new QTableWidgetItem(te::qt::widgets::Convert2QString(name));
  m_thirdPartyTableWidget->setItem(nRows, 0, itemName);

  QTableWidgetItem* itemVersion = new QTableWidgetItem(te::qt::widgets::Convert2QString(version));
  m_thirdPartyTableWidget->setItem(nRows, 1, itemVersion);
}
