/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of TerraView - A GIS Application.

    TerraView is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraView is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TerraLib Code Editor. See COPYING. If not, write to
    TerraLib Team at <terralib-team@dpi.inpe.br>.
 */

 /*!
   \file terralib/qt/af/connectors/ApplicationDockerConnector.cpp

   \brief A connector for the te::qt::af::ApplicationDockerController class to the Application Framework.
 */

#include "ApplicationDockerConnector.h"

// TerraLib
#include "../events/ApplicationEvents.h"
#include "../events/DockerEvents.h"
#include "../ApplicationController.h"
#include "../ApplicationDockerController.h"

// STL
#include <cassert>

te::qt::af::ApplicationDockerConnector::ApplicationDockerConnector(te::qt::af::ApplicationDockerController* controller, te::qt::af::ApplicationController* app, QObject* parent)
  : QObject(parent),
    m_app(app),
    m_controller(controller),
    m_internalEvent(false)
{
  assert(controller);

}

te::qt::af::ApplicationDockerConnector::~ApplicationDockerConnector() = default;

te::qt::af::ApplicationDockerController* te::qt::af::ApplicationDockerConnector::getController() const
{
  return m_controller;
}

void te::qt::af::ApplicationDockerConnector::onApplicationTriggered(te::qt::af::evt::Event* evt)
{
  switch(evt->m_id)
  {
    case te::qt::af::evt::ADD_DOCK:
    {
      te::qt::af::evt::AddDock* e = static_cast<te::qt::af::evt::AddDock*>(evt);

      te::qt::af::ApplicationDockerInfo appDockInfo = m_controller->createAppDockInfo(e->m_dockWidget, e->m_icon, e->m_dockName, e->m_isCloseable);
      m_controller->addDockWidget(appDockInfo, e->m_area);
    }
    break;

    case te::qt::af::evt::RAISE_DOCK:
    {
      te::qt::af::evt::RaiseDock* e = static_cast<te::qt::af::evt::RaiseDock*>(evt);

      m_controller->raiseDockWidget(e->m_dockWidget);
    }
    break;

    default:
      return;
  }
}

