/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of TerraView - A GIS Application.

    TerraView is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraView is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TerraLib Code Editor. See COPYING. If not, write to
    TerraLib Team at <terralib-team@dpi.inpe.br>.
 */

/*! 
  \file terralib/qt/af/connectors/ApplicationDockerConnector.h

  \brief A connector for the te::qt::af::ApplicationDockerController class to the Application Framework.
*/

#ifndef __TERRALIB_QT_AF_CONNECTORS_INTERNAL_APPLICATIONDOCKERCONNECTOR_H
#define __TERRALIB_QT_AF_CONNECTORS_INTERNAL_APPLICATIONDOCKERCONNECTOR_H

// Terralib
#include "../Config.h"

// Qt
#include <QtCore/QObject>

namespace te
{
  namespace qt
  {
    namespace af
    {
      class ApplicationController;
      class ApplicationDockerController;

      namespace evt
      {
        // Forward declarations
        struct Event;
      }

      /*!
        \class ApplicationDockerConnector

        \brief A connector for the te::qt::af::ApplicationDockerController class to the Application Framework.

        It is used to listen events sent by the application framework.
      */
      class TEQTAFEXPORT ApplicationDockerConnector : public QObject
      {
        Q_OBJECT

        public:

          /*!
            \brief Constructor.

            \param controller te::qt::af::ApplicationDockerController to be listened.
          */
          ApplicationDockerConnector(te::qt::af::ApplicationDockerController* controller, te::qt::af::ApplicationController* app, QObject* parent = 0);

          /*! \brief Destructor. */
          ~ApplicationDockerConnector();

          te::qt::af::ApplicationDockerController* getController() const;

        public slots:

          /*!
            \brief Listener to the application framewrork events.

            \param evt An event sent by framework.
          */
          void onApplicationTriggered(te::qt::af::evt::Event* evt);


        signals:

          void triggered(te::qt::af::evt::Event* e);

        protected:

          te::qt::af::ApplicationController* m_app;                //!< Pointer to applicatin controller;

          te::qt::af::ApplicationDockerController* m_controller;   //!< Pointer to a component te::qt::af::ApplicationDockerController.

          bool m_internalEvent;                                    //!< Flag used for internal sent events
      };

    } // end namespace af
  }   // end namespace qt
}     // end namespace te

#endif // __TERRALIB_QT_AF_CONNECTORS_INTERNAL_APPLICATIONDOCKERCONNECTOR_H

