/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file DockerEvents.h

  \brief Contains a list of docker events.
*/

#ifndef __TERRALIB_QT_AF_EVENTS_INTERNAL_DOCKEREVENTS_H
#define __TERRALIB_QT_AF_EVENTS_INTERNAL_DOCKEREVENTS_H

#include "Event.h"
#include "Enums.h"

// Qt Include
#include <QDockWidget>
#include <QIcon>

// STL
#include <memory>

namespace te
{
  namespace qt
  {
    namespace af
    {
      namespace evt
      {
        /*!
          \struct AddDock

          \brief This event signals that a new dock must be added
         */
        struct AddDock : public Event
        {
          /*!
            \brief Constructor.

            \param dock         The Qt component dock widget to be added
            \param area         The Qt info to define the dock area
            \param icon         The Qt component icon for dockwidget
            \param dockName     String dock name
            \param isCloseable  Flag to indicate if this component can be closed
          */
          AddDock(QDockWidget* dock, Qt::DockWidgetArea area, const QIcon& icon, const QString& dockName, bool isCloseable) : Event(ADD_DOCK),
            m_dockWidget(dock),
            m_area(area),
            m_icon(icon),
            m_dockName(dockName),
            m_isCloseable(isCloseable)
          {
          }

          QDockWidget* m_dockWidget;
          Qt::DockWidgetArea m_area;
          QIcon m_icon;
          QString m_dockName;
          bool m_isCloseable;
        };

        /*!
          \struct RaiseDock

          \brief This event signals that a dock must be raised
         */
        struct RaiseDock : public Event
        {
          /*!
            \brief Constructor.

            \param dock       The Qt component dock widget to be raised
          */
          RaiseDock(QDockWidget* dock) : Event(RAISE_DOCK),
            m_dockWidget(dock)
          {
          }

          QDockWidget* m_dockWidget;
        };
      }
    }
  }
}

#endif //__TERRALIB_QT_AF_EVENTS_INTERNAL_DOCKEREVENTS_H
