/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
\file terralib/qt/plugins/datasource/pgis/PostGISDataSetSelectorDialog.h

\brief A dialog window for showing the PostGIS data set selector widget.
*/

#ifndef __TERRALIB_QT_PLUGINS_DATASOURCE_PGISRASTER_INTERNAL_POSTGISDATASETSELECTORDIALOG_H
#define __TERRALIB_QT_PLUGINS_DATASOURCE_PGISRASTER_INTERNAL_POSTGISDATASETSELECTORDIALOG_H

// TerraLib
#ifndef Q_MOC_RUN
#include "../../../../dataaccess/dataset/DataSetType.h"
#include "../../../../dataaccess/datasource/DataSourceInfo.h"
#endif

#include "RasterPropertiesInfo.h"
#include "Utils.h"

// Qt
#include <QDialog>
#include <QGridLayout>
#include <QTimer>
#include <QTreeWidgetItem>

// QtPropertyBrowser
#include <QtPropertyBrowser/QtTreePropertyBrowser>

namespace Ui { class PostGISDataSetSelectorDialogForm; }

namespace te
{
  namespace rst
  {
    class Raster;
  }

  namespace qt
  {
    namespace widgets
    {
      class DataSetDisplay;
    }

    namespace plugins
    {
      namespace pgisRaster
      {

        /*!
          \class PostGISDataSetSelectorDialog

          \brief ....

          \ingroup plugins
          */
        class PostGISDataSetSelectorDialog : public QDialog
        {
          Q_OBJECT

          enum ItemType
          {
            ROOT_ITEM,
            TABLE_ITEM
          };

          public:

            PostGISDataSetSelectorDialog(QWidget* parent = 0, Qt::WindowFlags f = 0);

            ~PostGISDataSetSelectorDialog();

            void set(const te::da::DataSourceInfoPtr& ds);

            void getSelecteds(PostGISRasterInfoVec& vec);

          protected:

            void listDataSets();

            void drawDataSet(std::string dataSetName, std::string schema);

            void getDataSetInfo(std::string dataSetName, std::string schema);

            void buildDataSetInfoProperties(te::rst::Raster* raster);

            void resizeEvent(QResizeEvent* e);

            std::string getGDALURI(std::string dataSetName, std::string schema);

          protected slots:

            void onMapPreviewToggled(bool on);

            void onItemPressed(QTreeWidgetItem* item, int column);

            void onResizeTimeout();

          private:

            std::unique_ptr<Ui::PostGISDataSetSelectorDialogForm> m_ui;
            std::unique_ptr<te::qt::widgets::DataSetDisplay> m_mapPreview;

            te::da::DataSourceInfoPtr m_datasource;

            QTreeWidgetItem* m_rootItem;

            QtTreePropertyBrowser* m_propertyBrowser;
            RasterPropertiesInfo* m_rasterPropertyInfo;
            QGridLayout* m_propertyLayout;

            QTimer* m_timer;             //!< Timer to execute redraw on resize action.
        };

      } // end namespace pgis
    }   // end namespace plugins
  }     // end namespace qt
}       // end namespace te

#endif  // __TERRALIB_QT_PLUGINS_DATASOURCE_PGISRASTER_INTERNAL_POSTGISDATASETSELECTORDIALOG_H

