/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/plugins/datasource/pgis/PostGISType.cpp

  \brief PostGIS data source type.
*/

// TerraLib
#include "../../../../core/translator/Translator.h"
#include "../../../widgets/layer/selector/DataSetLayerSelector.h"
#include "PostGISConnector.h"
#include "PostGISType.h"

// STL
#include <cassert>

te::qt::plugins::pgisRaster::PostGISType::PostGISType() = default;

te::qt::plugins::pgisRaster::PostGISType::~PostGISType() = default;

bool te::qt::plugins::pgisRaster::PostGISType::hasDatabaseSupport() const
{
  return false;
}

bool te::qt::plugins::pgisRaster::PostGISType::hasFileSupport() const
{
  return true;
}

bool te::qt::plugins::pgisRaster::PostGISType::hasRasterSupport() const
{
  return true;
}

bool te::qt::plugins::pgisRaster::PostGISType::hasVectorialSupport() const
{
  return false;
}

std::string te::qt::plugins::pgisRaster::PostGISType::getName() const
{
  return "POSTGISRASTER";
}

std::string te::qt::plugins::pgisRaster::PostGISType::getTitle() const
{
  return TE_TR("PostGIS Raster");
}

std::string te::qt::plugins::pgisRaster::PostGISType::getDescription() const
{
  return TE_TR("Access raster data in a PostgreSQL database with the geospatial extension PostGIS");
}

QWidget* te::qt::plugins::pgisRaster::PostGISType::getWidget(int widgetType, QWidget* parent, Qt::WindowFlags f) const
{
  switch(widgetType)
  {
    case te::qt::widgets::DataSourceType::WIDGET_DATASOURCE_CONNECTOR:
      return new PostGISConnector(parent, f);

    case DataSourceType::WIDGET_LAYER_SELECTOR:
      return new te::qt::widgets::DataSetLayerSelector(parent, f);

    default:
      return nullptr;
  }
}

QIcon te::qt::plugins::pgisRaster::PostGISType::getIcon(int iconType) const
{
  switch(iconType)
  {
    case te::qt::widgets::DataSourceType::ICON_DATASOURCE_SMALL:
      return QIcon::fromTheme("datasource-postgis");

    case te::qt::widgets::DataSourceType::ICON_DATASOURCE_CONNECTOR:
      return QIcon::fromTheme("datasource-postgis");

    default:
      return QIcon::fromTheme("unknown-icon");
  }
}


