/*  Copyright (C) 2001-2013 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

#include "WFSItemDelegate.h"

// TerraLib
#include "../../../../wfs/Utils.h"
#include "../../../widgets/layer/explorer/LayerItem.h"

te::qt::plugins::wfs::WFSItemDelegate::WFSItemDelegate(QStyledItemDelegate* decorated, QObject* parent) :
  te::common::Decorator<QStyledItemDelegate>(decorated, false)
{
  setParent(parent);

  m_icon = QIcon::fromTheme("datasource-wfs");
}

te::qt::plugins::wfs::WFSItemDelegate::~WFSItemDelegate() = default;

void te::qt::plugins::wfs::WFSItemDelegate::paint(QPainter * painter, const QStyleOptionViewItem & option, const QModelIndex & index) const
{
  te::qt::widgets::TreeItem* item = static_cast<te::qt::widgets::TreeItem*>(index.internalPointer());

  if(item->getType() == "LAYER")
  {
    te::qt::widgets::LayerItem* li = (te::qt::widgets::LayerItem*)item;

    if(te::wfs::IsWFSLayer(*li->getLayer()))
    {
#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
      QStyleOptionViewItem opt(option);
      opt.features |= QStyleOptionViewItem::HasDecoration;
#else
      QStyleOptionViewItemV4 opt(option);
      opt.features |= QStyleOptionViewItemV4::HasDecoration;
#endif
      opt.decorationSize = QSize(20, 20);
      opt.icon = m_icon;

      QStyledItemDelegate::paint(painter, opt, index);

      return;
    }
  }

  if(m_decorated != nullptr)
    m_decorated->paint(painter, option, index);
  else
    QStyledItemDelegate::paint(painter, option, index);
}
