
#include "GeometryFile.h"
#include "Stasher.h"
#include "../../../edit/Enums.h"
#include "../../../edit/Utils.h"
#include "../../../maptools/AbstractLayer.h"

// Qt
#include <QDir>
#if QT_VERSION >= 0x050000
#include <QStandardPaths>
#endif
#if QT_VERSION < 0x050000
#include <QDesktopServices>
#endif

void StashGeometries(const te::map::AbstractLayer* layer, const std::map<std::string, te::gm::Geometry*>& geoms, const std::map<std::string, int>& ops)
{
  QString userDataDir;
  // find user data directory
#if QT_VERSION >= 0x050000
  userDataDir = QStandardPaths::writableLocation(QStandardPaths::DataLocation);
#else
  userDataDir = QDesktopServices::storageLocation(QDesktopServices::DataLocation);
#endif

  QDir dir(userDataDir + "/edition/" +layer->getTitle().c_str());

  if(!dir.exists())
    dir.mkpath(dir.path());

  QString f = dir.path() + "/" + QString::fromUtf8(layer->getTitle().c_str());
  QString f2 = f + "_r";

  std::vector<te::gm::Geometry*> gs;
  std::vector<te::gm::Geometry*> gsR;

  std::string fName = (f + ".ids").toUtf8().data();

  std::ofstream idsFile(fName.c_str(), std::ios::out);
  std::ofstream idsRFile((fName + "r").c_str(), std::ios::out);

  for(std::map<std::string, te::gm::Geometry*>::const_iterator it = geoms.begin(); it != geoms.end(); ++it)
  {
    std::string id = it->first;

    if(ops.at(id) == te::edit::DELETE)
    {
      gsR.push_back(it->second);
      idsRFile << id << "\n";
    }
    else
    {
      gs.push_back(it->second);
      idsFile << id << "\n";
    }
  }

  idsFile.close();
  idsRFile.close();

  GeometryFile::writeGeometries(f.toUtf8().data(), gs);
  GeometryFile::writeGeometries(f2.toUtf8().data(), gsR);
}


void GetStashedGeometries(const te::map::AbstractLayer* layer, std::map<std::string, te::gm::Geometry*>& geoms, std::map<std::string, int>& ops)
{
  QString userDataDir;
  // find user data directory
#if QT_VERSION >= 0x050000
  userDataDir = QStandardPaths::writableLocation(QStandardPaths::DataLocation);
#else
  userDataDir = QDesktopServices::storageLocation(QDesktopServices::DataLocation);
#endif

  QDir dir(userDataDir + "/edition/" + QString::fromUtf8(layer->getTitle().c_str()));

  if(dir.exists())
  {
    GeometryFile f;

    QString fName = dir.path() + "/" + QString::fromUtf8(layer->getTitle().c_str()) + ".geom";

    f.openFile(fName.toUtf8().data());

    QFile qf(dir.path() + "/" + QString::fromUtf8(layer->getTitle().c_str()) + ".ids");

    if(!qf.open(QIODevice::ReadOnly | QIODevice::Text))
      return;

    while(te::gm::Geometry* gm = f.next())
    {
      QString id(qf.readLine().data());

      if(id.isEmpty())
        continue;

      id.remove("\n");

      bool ok;

      id.toInt(&ok);
      
      ops[id.toUtf8().data()] = ((ok) ? te::edit::UPDATE : te::edit::INSERT);

      gm->setSRID(layer->getSRID());

      geoms[id.toUtf8().data()] = gm;
    }

    fName = dir.path() + "/" + QString::fromUtf8(layer->getTitle().c_str()) + "_r.geom";

    GeometryFile f2;

    f2.openFile(fName.toUtf8().data());

    QFile qf2(dir.path() + "/" + QString::fromUtf8(layer->getTitle().c_str()) + ".idsr");

    if(!qf2.open(QIODevice::ReadOnly | QIODevice::Text))
      return;

    while(te::gm::Geometry* gm = f2.next())
    {
      QString id(qf2.readLine().data());

      if(id.isEmpty())
        continue;

      id.remove("\n");

      bool ok;

      id.toInt(&ok);

      ops[id.toUtf8().data()] = te::edit::DELETE;

      gm->setSRID(layer->getSRID());

      geoms[id.toUtf8().data()] = gm;
    }
  }
}

std::set<std::string> GetStashedLayers()
{
  std::set<std::string> res;

  QString userDataDir;
  // find user data directory
#if QT_VERSION >= 0x050000
  userDataDir = QStandardPaths::writableLocation(QStandardPaths::DataLocation);
#else
  userDataDir = QDesktopServices::storageLocation(QDesktopServices::DataLocation);
#endif

  QDir dir(userDataDir + "/edition");

  QStringList lst = dir.entryList(QDir::Dirs);

  for(QStringList::iterator it = lst.begin(); it != lst.end(); ++it)
    res.insert((*it).toUtf8().data());

  return res;
}

void RemoveStash(const te::map::AbstractLayer* layer)
{
  QString userDataDir;
  // find user data directory
#if QT_VERSION >= 0x050000
  userDataDir = QStandardPaths::writableLocation(QStandardPaths::DataLocation);
#else
  userDataDir = QDesktopServices::storageLocation(QDesktopServices::DataLocation);
#endif

  QDir dir(userDataDir + "/edition/" + QString::fromUtf8(layer->getTitle().c_str()));

  if (!RemoveDir(dir.absolutePath()))
    throw;
}

bool RemoveDir(const QString& dirName)
{
  bool result = true;
  QDir dir(dirName);

  if (dir.exists(dirName)) {
    Q_FOREACH(QFileInfo info, dir.entryInfoList(QDir::NoDotAndDotDot | QDir::System | QDir::Hidden | QDir::AllDirs | QDir::Files, QDir::DirsFirst)) {
      if (info.isDir()) {
        result = RemoveDir(info.absoluteFilePath());
      }
      else {
        result = QFile::remove(info.absoluteFilePath());
      }

      if (!result) {
        return result;
      }
}
    result = dir.rmdir(dirName);
  }
  return result;
}

