/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/plugins/photolayer/PhotoLayerBuilder.h

  \brief Class used to build a vector layer of points from a set of images with EXIF GPS metadata. 
*/

#ifndef __TE_QT_PLUGINS_PHOTOLAYER_INTERNAL_PHOTOLAYERBUILDER_H
#define __TE_QT_PLUGINS_PHOTOLAYER_INTERNAL_PHOTOLAYERBUILDER_H

// TerraLib
#include <terralib/dataaccess/datasource/DataSource.h>
#include <terralib/raster/Raster.h>

// STL
#include <string>

namespace te
{
  //forward declarations
  namespace da { class DataSetType; }
  namespace mem { class DataSet; }

  namespace qt
  {
    namespace plugins
    {
      namespace photolayer
      {
        /*!
          \class PhotoLayerBuilder

          \brief Class used to build a vector layer of points from a set of images with EXIF GPS metadata. 

        */
        class PhotoLayerBuilder
        {
          public:

            /*!
              \brief Constructor.

              \param r The raster to get gps information.
            */
            PhotoLayerBuilder(te::da::DataSourcePtr ds, const std::string& dataSetName, const std::string& path);

            /*! \brief Destructor. */
            ~PhotoLayerBuilder();

          public:

            void execute();

            static std::size_t GetValidFilesCount(const std::string& path);

          protected:

            /*! Function used to create the output dataset type */
            std::unique_ptr<te::da::DataSetType> createDataSetType();

            /*! Function used to save the output dataset */
            void saveDataSet(te::mem::DataSet* dataSet, te::da::DataSetType* dsType);

          protected:

            te::da::DataSourcePtr m_dataSource;

            std::string m_dataSetName;

            std::string m_path;

        };
      } // end namespace photolayer
    } // end namespace plugins
  } // end namespace qt
} // end namespace te

#endif  // __TE_QT_PLUGINS_PHOTOLAYER_INTERNAL_PHOTOLAYERBUILDER_H
