/*  Copyright (C) 2011-2012 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/plugins/photolayer/PhotoLayerDialog.h

  \brief This interface is used to create new layer operation from photos with geo location information.
*/

#ifndef __TE_QT_PLUGINS_PHOTOLAYER_INTERNAL_PHOTOLAYERDIALOG_H
#define __TE_QT_PLUGINS_PHOTOLAYER_INTERNAL_PHOTOLAYERDIALOG_H

// TerraLib
#ifndef Q_MOC_RUN
#include <terralib/dataaccess/datasource/DataSourceInfo.h>
#include <terralib/maptools/AbstractLayer.h>
#endif
#include "Config.h"

// STL
#include <memory>

// Qt
#include <QDialog>

namespace Ui { class PhotoLayerDialogForm; }

namespace te
{
  namespace qt
  {
    namespace plugins
    {
      namespace photolayer
      {
        /*!
          \class PhotoLayerDialog

          \brief This interface is used to create new layer operation from photos with geo location information.
        */
        class PhotoLayerDialog : public QDialog
        {
          Q_OBJECT

        public:

          PhotoLayerDialog(QWidget* parent = 0, Qt::WindowFlags f = 0);

          ~PhotoLayerDialog();

        public:

          te::map::AbstractLayerPtr getLayer();

        protected slots:

          void onInputFileToolButtonPressed();

          void onTargetDatasourceToolButtonPressed();

          void onTargetFileToolButtonPressed();

          void onOkPushButtonClicked();

        private:

          te::da::DataSourceInfoPtr m_outputDatasource;

          te::map::AbstractLayerPtr m_outputLayer;        //!< Generated Layer.

          bool m_toFile;

          std::unique_ptr<Ui::PhotoLayerDialogForm> m_ui;
        };
      }   // end namesapce photolayer
    }     // end namespace plugins
  }       // end namespace qt
}         // end namespace te

#endif  // __TE_QT_PLUGINS_PHOTOLAYER_INTERNAL_PHOTOLAYERDIALOG_H

