/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/plugins/photolayer/PhotoViewerDialog.h

  \brief This file has the PhotoViewerDialog class.
*/

#ifndef __TE_QT_PLUGINS_PHOTOLAYER_INTERNAL_PHOTOVIEWERDIALOG_H
#define __TE_QT_PLUGINS_PHOTOLAYER_INTERNAL_PHOTOVIEWERDIALOG_H

// TerraLib
#include "Config.h"

// STL
#include <memory>

// Qt
#include <QDialog>
#include <QListWidgetItem>

namespace Ui { class PhotoViewerDialogForm; }

namespace te
{
  namespace qt
  {
    namespace widgets
    {
      class MultiThreadMapDisplay;
      class PanMiddleClick;
      class ZoomWheel;
    }

    namespace plugins
    {
      namespace photolayer
      {
        /*!
          \class DataViewerDialog

          \brief This class is used to view a geographic data.
        */
        class PhotoViewerDialog : public QDialog
        {
          Q_OBJECT

          public:

            PhotoViewerDialog(QWidget* parent = 0, Qt::WindowFlags f = 0);

            ~PhotoViewerDialog();

          public:

            void setListOfFiles(std::vector<std::string>& files);

          protected:

            void setURI(const std::string& uri);

          protected slots:

            void onItemActivated(QListWidgetItem *item);

          protected:

            void resizeEvent(QResizeEvent *) override;

          private:

            te::qt::widgets::MultiThreadMapDisplay* m_mapDisplay;
            te::qt::widgets::PanMiddleClick* m_panTool;               //!< Pan tool 
            te::qt::widgets::ZoomWheel* m_zoomTool;                   //!< Zoom tool

            std::unique_ptr<Ui::PhotoViewerDialogForm> m_ui;
        };

      } // end namespace photolayer
    }   // end namespace plugins
  }     // end namespace qt
}       // end namespace te

#endif  // __TE_QT_PLUGINS_PHOTOLAYER_INTERNAL_PHOTOVIEWERDIALOG_H

