/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/plugins/photolayer/Plugin.h

  \brief Plugin implementation for the TerraLib Photo Layer Qt Plugin.
*/

#ifndef __TE_QT_PLUGINS_PHOTOLAYER_INTERNAL_PLUGIN_H
#define __TE_QT_PLUGINS_PHOTOLAYER_INTERNAL_PLUGIN_H

// TerraLib
#include "../../widgets/tools/PointPicker.h"
#include "../../../core/plugin/CppPlugin.h"
#include "Config.h"

// Qt
#include <QAction>
#include <QMenu>
#include <QToolBar>

namespace te
{
  namespace qt
  {
    namespace af
    {
      namespace evt
      {
        struct Event;
      }
    }
    namespace plugins
    {
      namespace photolayer
      {
        class Plugin : public QObject, public te::core::CppPlugin
        {
          Q_OBJECT

          public:

            Plugin(const te::core::PluginInfo& pluginInfo);

            ~Plugin();

            void startup();

            void shutdown();

          protected slots:

            /*!
              \brief Slot function used when a action was selected.

              \param checked Flag used in case a toggle action.
              */
            void onActionActivated(bool);

            /*!
            \brief Slot function used to set the toolbar visibility

            \param checked Flag used in case a toggle action.
            */
            void onToolVisibleActionActivated(bool state);

            /*!
              \brief Slot function used when a tool action was selected.

              \param checked Flag used in case a toggle action.
              */
            void onToolActionActivated(bool state);

            void onGetPointCoordinate(QPointF& point);

          Q_SIGNALS:

            void triggered(te::qt::af::evt::Event* e);

          protected:

            QMenu* m_menu;                    //!< Photo Layer plugin menu
            QAction* m_action;                //!< Action used to call the process
            QAction* m_toolVisibleAction;     //!< Action used to start tool
            QAction* m_toolAction;            //!< Action used to start tool

            QToolBar* m_toolBar;      //!< Tool Bar

            te::qt::widgets::PointPicker* m_pointPickerTool;
        };

      } // end namespace photolayer
    }   // end namespace plugins
  }     // end namespace qt
}       // end namespace te

#endif //__TE_QT_PLUGINS_PHOTOLAYER_INTERNAL_PLUGIN_H
