/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/plugins/cellspace/CreateCellLayerAction.cpp

  \brief This file defines the CreateCellLayer class
*/

// Terralib
#include "../../../qt/widgets/rp/CloudDetectionDialog.h"
#include "../../af/ApplicationController.h"
#include "../../af/BaseApplication.h"
#include "../../af/events/LayerEvents.h"
#include "CloudDetectionAction.h"

// Qt
#include <QObject>
#include <QMessageBox>
#include <QActionGroup>

// STL
#include <memory>

te::qt::plugins::rp::CloudDetectionAction::CloudDetectionAction(QMenu* menu, QMenu* popupMenu):te::qt::plugins::rp::AbstractAction(menu, popupMenu)
{
  createAction(tr("Cloud Detection...").toUtf8().data(), "cloudDetection");
  m_action->setObjectName("Processing.Raster Processing.Cloud Detection");
}

te::qt::plugins::rp::CloudDetectionAction::~CloudDetectionAction() = default;

void te::qt::plugins::rp::CloudDetectionAction::onActionActivated(
    bool /*checked*/)
{
  te::qt::af::AppCtrlSingleton::getInstance().addListener(this,te::qt::af::RECEIVER);

  m_dlg.reset(new te::qt::widgets::CloudDetectionDialog(te::qt::af::AppCtrlSingleton::getInstance().getMainWindow()));

  connect(m_dlg.get(), SIGNAL(addLayer(te::map::AbstractLayerPtr)), this, SLOT(addLayer(te::map::AbstractLayerPtr)));
  connect(m_dlg.get(), SIGNAL(closeTool()), this, SLOT(closeTool()));

  te::qt::af::BaseApplication* ba = dynamic_cast<te::qt::af::BaseApplication*>(te::qt::af::AppCtrlSingleton::getInstance().getMainWindow());
  
  QActionGroup* mapEditionTools = te::qt::af::AppCtrlSingleton::getInstance().findActionGroup("Map.ToolsGroup");
  assert(mapEditionTools);

  std::list<te::map::AbstractLayerPtr> layersList = getLayers();

  m_dlg->setMapDisplay(ba->getMapDisplay());
  m_dlg->setActionGroup(mapEditionTools);
  m_dlg->setModal(false);
  m_dlg->setLayers(layersList);
  m_dlg->setUserPath(te::qt::af::AppCtrlSingleton::getInstance().getUserDataDir().toUtf8().data());
  m_dlg->show();
}

void te::qt::plugins::rp::CloudDetectionAction::addLayer(te::map::AbstractLayerPtr outputLayer)
{
  //add new layer
  addNewLayer(outputLayer);
}

void te::qt::plugins::rp::CloudDetectionAction::closeTool()
{
  te::qt::af::AppCtrlSingleton::getInstance().removeListener(this);
  m_dlg.release();
}
