/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/plugins/vp/CloudDetectionAction.h

  \brief This file defines the cloud detection class
*/

#ifndef __TE_QT_PLUGINS_CLOUDDETECTION_INTERNAL_CLOUDDETECTIONACTION_H
#define __TE_QT_PLUGINS_CLOUDDETECTION_INTERNAL_CLOUDDETECTIONACTION_H

// TerraLib
#ifndef Q_MOC_RUN
#include "AbstractAction.h"
#include "../../../maptools/AbstractLayer.h"
#include "../../../qt/widgets/rp/CloudDetectionDialog.h"
#endif
#include "Config.h"

//STL
#include <memory>

// Qt
#include <QAction>
#include <QMenu>
#include <QObject>

namespace te
{
  namespace qt
  {
    namespace af
    {
      namespace evt
      {
        struct Event;
      }
    }
    namespace plugins
    {
      namespace rp
      {
        /*!
          \class CloudDetectionAction

          \brief This class register the action into Cloud Detection Plugin.

        */
        class CloudDetectionAction : public te::qt::plugins::rp::AbstractAction
        {
          Q_OBJECT

          public:

            CloudDetectionAction(QMenu* menu, QMenu* popupMenu);

            virtual ~CloudDetectionAction();

          protected slots:

            virtual void onActionActivated(bool checked);

            void addLayer(te::map::AbstractLayerPtr outputLayer);

            void closeTool();

          protected:

            std::unique_ptr<te::qt::widgets::CloudDetectionDialog> m_dlg;
        };

      } // end namespace clouddetection
    }   // end namespace plugins
  }     // end namespace qt
}       // end namespace te

#endif //__TE_QT_PLUGINS_RP_INTERNAL_CLOUDDETECTIONACTION_H
