/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/plugins/rp/ColorTransformAction.cpp

  \brief This file defines the ColorTransform class
*/

// Terralib
#include "../../../qt/widgets/rp/ColorTransformDialog.h"
#include "../../af/ApplicationController.h"
#include "../../af/BaseApplication.h"
#include "ColorTransformAction.h"

// Qt
#include <QtCore/QObject>
#include <QMessageBox>

// STL
#include <memory>

te::qt::plugins::rp::ColorTransformAction::ColorTransformAction(QMenu* menu, QMenu* popupMenu):te::qt::plugins::rp::AbstractAction(menu, popupMenu)
{
  createAction(tr("Color Transform...").toUtf8().data(), "colorTransform");
  m_action->setObjectName("Processing.Raster Processing.Color Transform");
}

te::qt::plugins::rp::ColorTransformAction::~ColorTransformAction() = default;

void te::qt::plugins::rp::ColorTransformAction::onActionActivated(
    bool /*checked*/)
{
  m_colorTransfDlg.reset(new te::qt::widgets::ColorTransformDialog(te::qt::af::AppCtrlSingleton::getInstance().getMainWindow()));

  connect(m_colorTransfDlg.get(), SIGNAL(addLayer(te::map::AbstractLayerPtr)), this, SLOT(addLayer(te::map::AbstractLayerPtr)));
  connect(m_colorTransfDlg.get(), SIGNAL(closeTool()), this, SLOT(closeTool()));

  te::qt::af::BaseApplication* ba = dynamic_cast<te::qt::af::BaseApplication*>(te::qt::af::AppCtrlSingleton::getInstance().getMainWindow());

  std::list<te::map::AbstractLayerPtr> layersList = getLayers();

  m_colorTransfDlg->setModal(false);
  m_colorTransfDlg->setMapDisplay(ba->getMapDisplay());
  m_colorTransfDlg->setList(layersList);

  m_colorTransfDlg->show();
}

void te::qt::plugins::rp::ColorTransformAction::addLayer(te::map::AbstractLayerPtr outputLayer)
{
  //add new layer
  addNewLayer(outputLayer);
}

void te::qt::plugins::rp::ColorTransformAction::closeTool()
{
  m_colorTransfDlg.reset(nullptr);
}
