/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/plugins/rp/PCAAction.cpp

  \brief This file defines the PCA class
*/

// Terralib
#include "../../../qt/widgets/rp/PCAWizard.h"
#include "../../../raster/Raster.h"
#include "../../af/ApplicationController.h"
#include "../../af/BaseApplication.h"
#include "../../af/events/LayerEvents.h"
#include "PCAAction.h"

// Qt
#include <QObject>
#include <QMessageBox>

// STL
#include <memory>

te::qt::plugins::rp::PCAAction::PCAAction(QMenu* menu, QMenu* popupMenu):te::qt::plugins::rp::AbstractAction(menu, popupMenu)
{
  createAction(tr("PCA...").toUtf8().data(), "pca");
  m_action->setObjectName("Processing.Raster Processing.PCA");

  createPopUpAction(tr("PCA...").toUtf8().data(), "Principal Components Analysis");
}

te::qt::plugins::rp::PCAAction::~PCAAction() = default;

void te::qt::plugins::rp::PCAAction::onActionActivated(bool /*checked*/)
{
  te::qt::af::AppCtrlSingleton::getInstance().addListener(this, te::qt::af::RECEIVER);

  m_Wizard.reset(new te::qt::widgets::PCAWizard(te::qt::af::AppCtrlSingleton::getInstance().getMainWindow()));

/// -- ?????????
// -> te::qt::af::BaseApplication* ba = dynamic_cast<te::qt::af::BaseApplication*>(te::qt::af::AppCtrlSingleton::getInstance().getMainWindow());
//  m_Wizard->setMapDisplay(ba->getMapDisplay());

  connect(m_Wizard.get(), SIGNAL(addLayer(te::map::AbstractLayerPtr)), this, SLOT(addLayerSlot(te::map::AbstractLayerPtr)));

  connect(m_Wizard.get(), SIGNAL(finished(int)), this, SLOT(onPCAWizardFinished()));

//  m_Wizard->setActionGroup(mapEditionTools);

  std::list<te::map::AbstractLayerPtr> layersList = getLayers();

  m_Wizard->setAttribute(Qt::WA_DeleteOnClose);

  m_Wizard->setList(layersList);

  m_Wizard->setModal(false);

  m_Wizard->show();
}

void te::qt::plugins::rp::PCAAction::addLayerSlot(te::map::AbstractLayerPtr layer)
{
  addNewLayer(layer);
}

void te::qt::plugins::rp::PCAAction::onPCAWizardFinished()
{
  te::qt::af::AppCtrlSingleton::getInstance().removeListener(this);

  m_Wizard.release();
}

void te::qt::plugins::rp::PCAAction::onPopUpActionActivated(bool /*checked*/)
{
  te::map::AbstractLayerPtr layer = getCurrentLayer();

  if(layer.get())
  {
    te::qt::widgets::PCAWizard dlg(te::qt::af::AppCtrlSingleton::getInstance().getMainWindow());

    dlg.setLayer(layer);

    if(dlg.exec() == QDialog::Accepted)
    {
      addNewLayer( dlg.getOutputLayer() );
    }
  }
  else
  {
    QMessageBox::warning(te::qt::af::AppCtrlSingleton::getInstance().getMainWindow(), tr("Warning"), tr("The layer selected is invalid or does not have an raster representation."));
  }
}

void te::qt::plugins::rp::PCAAction::onApplicationTriggered(te::qt::af::evt::Event*)
{
}
