/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/plugins/rp/PaletteAction.cpp

  \brief This file defines the Palette class
*/

// Terralib
#include "../../../qt/widgets/rp/PaletteDialog.h"
#include "../../af/ApplicationController.h"
#include "PaletteAction.h"

// Qt
#include <QMessageBox>
#include <QObject>

// STL
#include <memory>

te::qt::plugins::rp::PaletteAction::PaletteAction(QMenu* menu, QMenu* popupMenu):te::qt::plugins::rp::AbstractAction(menu, popupMenu)
{
  createAction(tr("Converts to palette raster...").toUtf8().data(), "palette");
  m_action->setObjectName("Processing.Raster Processing.Converts To Palette Raster");
}

te::qt::plugins::rp::PaletteAction::~PaletteAction() = default;

void te::qt::plugins::rp::PaletteAction::onActionActivated(
  bool /*checked*/)
{
  try
  {
 
    te::map::AbstractLayerPtr layer = getCurrentLayer();
    if (!layer.get())
    {
      throw te::common::Exception(tr("There is no selected  layer.").toStdString());
    }

    te::qt::widgets::PaletteDialog palleteDlg(te::qt::af::AppCtrlSingleton::getInstance().getMainWindow());

    palleteDlg.setWindowTitle(palleteDlg.windowTitle() + " (" + tr("Layer") + ":" + layer->getTitle().c_str() + ")");

    palleteDlg.setParameters(layer);

    if (palleteDlg.exec() != QDialog::Accepted)
      return;

    te::map::AbstractLayerPtr outlayer = palleteDlg.getLayer();

    if (!outlayer)
      return;

    int reply = QMessageBox::question(nullptr, tr("Palette Raster"), tr("The operation was concluded successfully. Would you like to add the layer to the project?"), QMessageBox::No, QMessageBox::Yes);

    if (reply == QMessageBox::Yes)
      addNewLayer(outlayer);
  }
  catch (const te::common::Exception& e)
  {
    QMessageBox::warning(nullptr, tr("Palette Raster"), e.what());
    return;
  }
}

