/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/plugins/rp/Plugin.h

  \brief Plugin implementation for the RP Qt Plugin widget.
*/

#ifndef __TE_QT_PLUGINS_RP_INTERNAL_PLUGIN_H
#define __TE_QT_PLUGINS_RP_INTERNAL_PLUGIN_H

// TerraLib
#include "../../../core/plugin/CppPlugin.h"
#include "Config.h"

// Qt
#include <QAction>
#include <QMenu>

namespace te
{
  namespace qt
  {
    namespace af
    {
      namespace evt
      {
        struct Event;
      }
    }
    namespace plugins
    {
      namespace rp
      {
        class ArithmeticOpAction;
        class BoundaryExtractionAction;
        class ClassifierAction;
        class ClippingAction;
        class CloudDetectionAction;
        class ColorTransformAction;
        class ComposeBandsAction;
        class ContrastAction;
        class FilterAction;
        class FusionAction;
        class MajorityFilterAction;
        class MixtureModelAction;
        class MosaicAction;
        class PaletteAction;
        class PCAAction;
        class PostClassificationAction;
        class RasterEditAction;
        class RasterSlicingAction;
        class RasterizationAction;
        class SegmenterAction;
        class RegisterAction;
        class VectorizationAction;
        class RasterStatisticsAction;
        class RasterTransformAction;
        class RestorationAction;
        class MultiSourceClassifierAction;

        class Plugin : public QObject, public te::core::CppPlugin
        {
          Q_OBJECT

          public:

            Plugin(const te::core::PluginInfo& pluginInfo);

            ~Plugin();

            void startup();

            void shutdown();

          protected:

            /*!
              \brief Function used to register all raster processing actions.

            */
            void registerActions();

            /*!
              \brief Function used to unregister all raster processing actions.

            */
            void unRegisterActions();

          Q_SIGNALS:

            void triggered(te::qt::af::evt::Event* e);

          protected:

            QMenu* m_rpMenu;                                              //!< RP Main Menu registered.
            QMenu* m_rpPopupMenu;                                         //!< RP Main Menu registered.
            QAction* m_popupActionRP;                                       //!< RP pop up action registered.

            QMenu* m_rpRadarMenu;                                              //!< RP Main Menu registered.
            QMenu* m_rpRadarPopupMenu;                                         //!< RP Main Menu registered.
            QAction* m_popupActionRPRadar;                                       //!< RP pop up action registered.

            te::qt::plugins::rp::ArithmeticOpAction* m_arithmeticOp;      //!< Arithmetic Operation Process Action
            te::qt::plugins::rp::ClassifierAction* m_classifier;          //!< Classifier Process Action
            te::qt::plugins::rp::ClippingAction* m_clipping;              //!< Clipping Process Action
            te::qt::plugins::rp::CloudDetectionAction* m_cloudDetection;  //!< Cloud Detection Process Action
            te::qt::plugins::rp::ColorTransformAction* m_colorTrans;      //!< Color Transform Process Action
            te::qt::plugins::rp::ComposeBandsAction* m_composeBands;      //!< Compose Bands Process Action
            te::qt::plugins::rp::ContrastAction* m_contrast;              //!< Contrast Process Action
            te::qt::plugins::rp::FilterAction* m_filter;                  //!< Filter Process Action
            te::qt::plugins::rp::FusionAction* m_fusion;                  //!< Fusion Process Action
            te::qt::plugins::rp::MajorityFilterAction* m_majorityFilter;  //!< Majority Filter Process Action
            te::qt::plugins::rp::MixtureModelAction* m_mixtureModel;      //!< Mixture Model Process Action
            te::qt::plugins::rp::MosaicAction* m_mosaic;                  //!< Mosaic Process Action
            te::qt::plugins::rp::PaletteAction* m_palette;                //!< Convertion to Palette Raster Action
            te::qt::plugins::rp::PCAAction* m_PCA;                        //!< PCA Action
            te::qt::plugins::rp::PostClassificationAction* m_postClass;   //!< Post Classification Process Action
            te::qt::plugins::rp::RasterEditAction* m_rasterEdit;          //!< Raster edit Action
            te::qt::plugins::rp::SegmenterAction* m_segmenter;            //!< Segmenter Process Action
            te::qt::plugins::rp::RasterSlicingAction* m_rasterSlicing;    //!< Raster slicing Process Action
            te::qt::plugins::rp::RasterizationAction* m_raster;           //!< Rasterization Process Action
            te::qt::plugins::rp::RegisterAction* m_register;              //!< Register Process Action
            te::qt::plugins::rp::VectorizationAction* m_vector;           //!< Vectorization Process Action
            te::qt::plugins::rp::BoundaryExtractionAction *m_boundary;    //!< Raster Boundary Extraction Process Action
            te::qt::plugins::rp::RasterStatisticsAction *m_rasterStats; //!< Raster Statistics Process Action
            te::qt::plugins::rp::RasterTransformAction *m_rasterTransf; //!< Raster Transform Process Action
            te::qt::plugins::rp::RestorationAction *m_restoration; //!< Raster restoration action.
            te::qt::plugins::rp::MultiSourceClassifierAction *m_multiSourceClassifier; //!< Multi Source Classifier action.
        };

      } // end namespace rp
    }   // end namespace plugins
  }     // end namespace qt
}       // end namespace te

#endif //__TE_QT_PLUGINS_RP_INTERNAL_PLUGIN_H
