/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/plugins/rp/PostClassificationAction.cpp

  \brief This file defines the Post Classification class
*/

// Terralib
#include "../../../qt/widgets/rp/PostClassificationDialogForm.h"
#include "../../af/ApplicationController.h"
#include "PostClassificationAction.h"

// Qt
#include <QtCore/QObject>

// STL
#include <memory>

te::qt::plugins::rp::PostClassificationAction::PostClassificationAction(QMenu* menu, QMenu* popupMenu) :te::qt::plugins::rp::AbstractAction(menu, popupMenu)
{
  createAction(tr("Post Classification...").toUtf8().data(), "post-classifier");
  m_action->setObjectName("Processing.Raster Processing.Post Classification");
}

te::qt::plugins::rp::PostClassificationAction::~PostClassificationAction() =
    default;

void te::qt::plugins::rp::PostClassificationAction::onActionActivated(
    bool /*checked*/)
{
  m_postClassDlg.reset(new te::qt::widgets::PostClassificationDialogForm(te::qt::af::AppCtrlSingleton::getInstance().getMainWindow()));

  connect(m_postClassDlg.get(), SIGNAL(addLayer(te::map::AbstractLayerPtr)), this, SLOT(addLayer(te::map::AbstractLayerPtr)));
  connect(m_postClassDlg.get(), SIGNAL(closeTool()), this, SLOT(closeTool()));

  std::list<te::map::AbstractLayerPtr> layersList = getLayers();

  m_postClassDlg->setModal(false);
  m_postClassDlg->setList(layersList);
  m_postClassDlg->show();
}

void te::qt::plugins::rp::PostClassificationAction::addLayer(te::map::AbstractLayerPtr outputLayer)
{
  //add new layer
  addNewLayer(outputLayer);
}

void te::qt::plugins::rp::PostClassificationAction::closeTool()
{
  m_postClassDlg.reset(nullptr);
}
