/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file RasterConfigWidget.h

  \brief A frame for setting raster processing options.
*/

#ifndef __TERRALIB_QT_PLUGINS_RP_RASTERCONFIGWIDGET_H
#define __TERRALIB_QT_PLUGINS_RP_RASTERCONFIGWIDGET_H

// TerraLib
#include "../../af/Config.h"

// Qt
#include "../../af/settings/AbstractSettingWidget.h"

// STL
#include <memory>

// Forward declaration
namespace Ui
{
  class RasterConfigWidgetForm;
}

namespace te
{
  namespace qt
  {
    namespace plugins
    {
     namespace rp
     {
       /*!
         \class DisplayWidget

         \brief A frame for setting raster processing options.
       */
       class RasterConfigWidget : public te::qt::af::AbstractSettingWidget
       {
         Q_OBJECT

         public:

           RasterConfigWidget(QWidget* parent = 0);

           ~RasterConfigWidget();

           void saveChanges();

           void resetState();

           void getHelpInformations(QString& ns, QString& helpFile);

         private:

           void initialize();

           void fillInterpolator();

         private slots:

           void onInterpolatorComboBoxChanged(int index);

           void onLevelChanged(int value);

           void onQuestionChecked(bool checked);

           void onSetMultiResolutionDefaultChecked(bool checked);

         private:

           std::unique_ptr<Ui::RasterConfigWidgetForm> m_ui;

       };
      } // end namespace rp
    }   // end namespace plugins
  }     // end namespace qt
}       // end namespace te

#endif  // __TERRALIB_QT_PLUGINS_RP_RASTERCONFIGWIDGET_H
