/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/plugins/rp/RasterConfigWidgetFactory.cpp

  \brief A factory to build the raster processing config frame object.
*/

#include "RasterConfigWidget.h"
#include "RasterConfigWidgetFactory.h"

te::qt::plugins::rp::RasterConfigWidgetFactory* te::qt::plugins::rp::RasterConfigWidgetFactory::sm_factory(nullptr);

void te::qt::plugins::rp::RasterConfigWidgetFactory::initialize()
{
  finalize();
  sm_factory = new RasterConfigWidgetFactory;
}

void te::qt::plugins::rp::RasterConfigWidgetFactory::finalize()
{
  delete sm_factory;
  sm_factory = nullptr;
}

te::qt::plugins::rp::RasterConfigWidgetFactory::~RasterConfigWidgetFactory() =
    default;

te::qt::af::AbstractSettingWidget* te::qt::plugins::rp::RasterConfigWidgetFactory::build()
{
  return new RasterConfigWidget;
}

te::qt::plugins::rp::RasterConfigWidgetFactory::RasterConfigWidgetFactory() :
te::qt::af::SettingsWidgetsFactory(QObject::tr("Raster processing").toUtf8().data())
{

}

te::qt::af::AbstractSettingWidget* te::qt::plugins::rp::RasterConfigWidgetFactory::create(QWidget* parent)
{
  return new RasterConfigWidget(parent);
}
