/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/plugins/rp/RasterEditAction.cpp

  \brief This file defines the RasterEdit class
*/

// Terralib
#include "../../../qt/widgets/rp/RasterEditWizard.h"
#include "../../../raster/Raster.h"
#include "../../af/ApplicationController.h"
#include "../../af/BaseApplication.h"
#include "../../af/events/LayerEvents.h"
#include "../../widgets/canvas/MapDisplay.h"
#include "RasterEditAction.h"

// Qt
#include <QObject>
#include <QMessageBox>

// STL
#include <memory>

te::qt::plugins::rp::RasterEditAction::RasterEditAction(QMenu* menu, QMenu* popupMenu):te::qt::plugins::rp::AbstractAction(menu, popupMenu)
{
  createAction(tr("Raster Edit...").toUtf8().data(), "rasteredit");
  m_action->setObjectName("Processing.Raster Processing.RasterEdit");
}

te::qt::plugins::rp::RasterEditAction::~RasterEditAction() = default;

void te::qt::plugins::rp::RasterEditAction::onActionActivated(
    bool /*checked*/)
{
  te::qt::af::AppCtrlSingleton::getInstance().addListener(this, te::qt::af::RECEIVER);

  te::qt::af::BaseApplication* ba = dynamic_cast<te::qt::af::BaseApplication*>(
    te::qt::af::AppCtrlSingleton::getInstance().getMainWindow());

  m_rasterEditWizard.reset( new te::qt::widgets::RasterEditWizard(
    ba, ba->getMapDisplay(), ba->getLayerExplorer() ) );

  connect(m_rasterEditWizard.get(), SIGNAL(addLayer(te::map::AbstractLayerPtr)), this, SLOT(addLayerSlot(te::map::AbstractLayerPtr)));
  connect(m_rasterEditWizard.get(), SIGNAL(finished(int)), SLOT(onRasterEditFinished()));

  QActionGroup* mapEditionTools = te::qt::af::AppCtrlSingleton::getInstance().findActionGroup("Map.ToolsGroup");
  assert(mapEditionTools);

  m_rasterEditWizard->setActionGroup(mapEditionTools);

  std::list<te::map::AbstractLayerPtr> layersList = getLayers();

  m_rasterEditWizard->setAttribute(Qt::WA_DeleteOnClose);

  m_rasterEditWizard->setList(layersList);

  m_rasterEditWizard->setModal(false);

  m_rasterEditWizard->show();
}

void te::qt::plugins::rp::RasterEditAction::onPopUpActionActivated(
    bool /*checked*/)
{
  te::map::AbstractLayerPtr layer = getCurrentLayer();

  if(layer.get())
  {
    te::qt::af::BaseApplication* ba = dynamic_cast<te::qt::af::BaseApplication*>(
      te::qt::af::AppCtrlSingleton::getInstance().getMainWindow());

    te::qt::widgets::RasterEditWizard dlg( ba, ba->getMapDisplay(),
      ba->getLayerExplorer() );

    if(dlg.exec() == QDialog::Accepted)
    {
      //add new layer
      addNewLayer(dlg.getOutputLayer());
    }
  }
  else
  {
    QMessageBox::warning(te::qt::af::AppCtrlSingleton::getInstance().getMainWindow(), tr("Warning"), tr("The layer selected is invalid or does not have an raster representation."));
  }
}

void te::qt::plugins::rp::RasterEditAction::onRasterEditFinished()
{
  te::qt::af::AppCtrlSingleton::getInstance().removeListener(this);

  m_rasterEditWizard.release();
}

void te::qt::plugins::rp::RasterEditAction::addLayerSlot(te::map::AbstractLayerPtr layer)
{
  addNewLayer(layer);
}

void te::qt::plugins::rp::RasterEditAction::onApplicationTriggered(te::qt::af::evt::Event* e)
{
  switch(e->m_id)
  {
    case te::qt::af::evt::DRAWING_FINISHED:
    {
      m_rasterEditWizard->onMapDisplayExtentChanged();
    }
    break;

    default:
     break;
  }
}
