/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.
  
    This file is part of the TerraLib - a Framework for building GIS enabled applications.
    
    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.
    
    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.
    
    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */


#include "RasterStatisticsAction.h"

#include "../../../qt/af/ApplicationController.h"
#include "../../af/BaseApplication.h"

te::qt::plugins::rp::RasterStatisticsAction::RasterStatisticsAction(QMenu *menu, QMenu *popupMenu) :
  AbstractAction(menu, popupMenu)
{
  createAction(tr("Raster Statistics...").toUtf8().data(), "histogram");
  m_action->setObjectName("Processing.Raster Processing.Raster Statistics");
  
  m_wizardPtr.reset( new te::qt::widgets::RasterStatisticsWizard(
    te::qt::af::AppCtrlSingleton::getInstance().getMainWindow()) );
}

te::qt::plugins::rp::RasterStatisticsAction::~RasterStatisticsAction() = default;

void te::qt::plugins::rp::RasterStatisticsAction::onActionActivated(
    bool /*checked*/)
{
  te::qt::af::BaseApplication* ba = dynamic_cast<te::qt::af::BaseApplication*>(
    te::qt::af::AppCtrlSingleton::getInstance().getMainWindow());
  std::list<te::map::AbstractLayerPtr> layersList = getLayers();  
  
  QActionGroup* mapEditionTools = te::qt::af::AppCtrlSingleton::getInstance().findActionGroup("Map.ToolsGroup");
  assert(mapEditionTools);  

  m_wizardPtr->setModal(false);
  m_wizardPtr->setList( layersList );
  m_wizardPtr->setMapDisplay(ba->getMapDisplay());
  m_wizardPtr->setActionGroup(mapEditionTools);
  m_wizardPtr->setModal(false);
  m_wizardPtr->show();
}

void te::qt::plugins::rp::RasterStatisticsAction::onPopUpActionActivated(
    bool /*checked*/)
{
}


