/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.
  
    This file is part of the TerraLib - a Framework for building GIS enabled applications.
    
    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.
    
    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.
    
    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */


#include "RasterTransformAction.h"
#include "../../../qt/widgets/rp/RasterTransformWizard.h"
#include "../../../qt/af/ApplicationController.h"
#include "../../af/BaseApplication.h"

te::qt::plugins::rp::RasterTransformAction::RasterTransformAction(QMenu *menu, QMenu *popupMenu) :
  AbstractAction(menu, popupMenu)
{
  createAction(tr("Raster Transform...").toUtf8().data(), "file-raster");
  m_action->setObjectName("Processing.Raster Processing.Raster Transform");
}

te::qt::plugins::rp::RasterTransformAction::~RasterTransformAction() = default;

void te::qt::plugins::rp::RasterTransformAction::onActionActivated(
    bool /*checked*/)
{
  te::qt::af::BaseApplication* ba = dynamic_cast<te::qt::af::BaseApplication*>(
    te::qt::af::AppCtrlSingleton::getInstance().getMainWindow());
  std::list<te::map::AbstractLayerPtr> layersList = getLayers();  

  te::qt::widgets::RasterTransformWizard* m_wizardPtr = 
    new te::qt::widgets::RasterTransformWizard(
    te::qt::af::AppCtrlSingleton::getInstance().getMainWindow());
  m_wizardPtr->setModal(true);
  m_wizardPtr->setAttribute(Qt::WA_DeleteOnClose);
  m_wizardPtr->setList( layersList );
  connect(m_wizardPtr, SIGNAL(addLayer(te::map::AbstractLayerPtr)), 
    this, SLOT(addLayerSlot(te::map::AbstractLayerPtr)));  
  m_wizardPtr->show();
}

void te::qt::plugins::rp::RasterTransformAction::onPopUpActionActivated(
    bool /*checked*/)
{
}

void te::qt::plugins::rp::RasterTransformAction::addLayerSlot(te::map::AbstractLayerPtr layer)
{
  addNewLayer(layer);
}

