/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/plugins/rp/MultiSourceClassifierAction.cpp

  \brief This file defines the Post Classification class
*/

// Terralib
#include "../../../widgets/rp/radar/MultiSourceClassifierFirstDialog.h"
#include "../../../af/ApplicationController.h"
#include "MultiSourceClassifierAction.h"

// Qt
#include <QtCore/QObject>

// STL
#include <memory>

te::qt::plugins::rp::MultiSourceClassifierAction::MultiSourceClassifierAction(QMenu* menu, QMenu* popupMenu) :te::qt::plugins::rp::AbstractAction(menu, popupMenu)
{
  createAction(tr("Multi Source Classifier...").toUtf8().data(),
    "classifier");
  m_action->setObjectName("Processing.Radar Raster Processing.Multi Source Classifier");
}

te::qt::plugins::rp::MultiSourceClassifierAction::~MultiSourceClassifierAction() =
    default;

void te::qt::plugins::rp::MultiSourceClassifierAction::onActionActivated(
    bool /*checked*/)
{
  m_Dlg.reset(new te::qt::widgets::MultiSourceClassifierFirstDialog(
    te::qt::af::AppCtrlSingleton::getInstance().getMainWindow()));

  connect(m_Dlg.get(), SIGNAL(processingFinishedOK()),
    this, SLOT(processingFinishedOK()));
  connect(m_Dlg.get(), SIGNAL(closeTool()), this, SLOT(closeTool()));

  std::list<te::map::AbstractLayerPtr> layersList = getLayers();

  m_Dlg->setModal(false);
  m_Dlg->setList(layersList);
  m_Dlg->show();
}

void te::qt::plugins::rp::MultiSourceClassifierAction::processingFinishedOK()
{
  if( m_Dlg.get() )
  {
    te::map::AbstractLayerPtr classImageLayer;
    te::map::AbstractLayerPtr mapLayer;
    te::map::AbstractLayerPtr mapPValueLayer;

    m_Dlg->getOutputLayers( classImageLayer, mapLayer, mapPValueLayer );

    if( classImageLayer.get() ) addNewLayer(classImageLayer);
    if( mapLayer.get() ) addNewLayer(mapLayer);
    if( mapPValueLayer.get() ) addNewLayer(mapPValueLayer);
  }
}

void te::qt::plugins::rp::MultiSourceClassifierAction::closeTool()
{
  m_Dlg.reset(nullptr);
}
