/*
  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

  This file is part of the TerraLib - a Framework for building GIS enabled
  applications.

  TerraLib is free software: you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation, either version 3 of the License,
  or (at your option) any later version.

  TerraLib is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with TerraLib. See COPYING. If not, write to
  TerraLib Team at <terralib-team@terralib.org>.
*/

/*!
  \file terralib/qt/plugins/timeviewer/plugin.cpp

  \brief Add a plugin interface for dynamic loading of the TimeViewer plugin.

  \author Matheus Cavassan Zaglia
 */

// Terralib
#include "../../af/ApplicationController.h"

// TimeViewer
#include "Plugin.h"
#include  "TimeViewerDockWidget.h"

// QT
#include <QMenu>

#include <iostream>

te::qt::plugins::tv::Plugin::Plugin(const te::core::PluginInfo& pluginInfo)
    : QObject(nullptr),
      te::core::CppPlugin(pluginInfo),
      m_timeViewerAction(nullptr),
      m_tvDock(nullptr)
{
}

te::qt::plugins::tv::Plugin::~Plugin() = default;

void te::qt::plugins::tv::Plugin::startup()
{
  if(m_initialized)
    return;

  m_timeViewerAction = new QAction(this);
  m_timeViewerAction->setText("Time Viewer");
  m_timeViewerAction->setObjectName("TimeViewer");
  m_timeViewerAction->setIcon(QIcon::fromTheme("time-viewer"));

  connect(m_timeViewerAction, SIGNAL(triggered()), this,SLOT(onTimeViewerActionTriggered()));

  te::qt::af::AppCtrlSingleton::getInstance().getMenu("Plugins")->addAction(m_timeViewerAction);

  m_initialized = true;
}

void te::qt::plugins::tv::Plugin::shutdown()
{
  if(!m_initialized)
    return;

  delete m_tvDock;
  delete m_timeViewerAction;

  te::qt::af::AppCtrlSingleton::getInstance().removeListener(this);

  m_initialized = false;
}

void te::qt::plugins::tv::Plugin::onTimeViewerActionTriggered()
{
  if(!m_tvDock){
    m_tvDock = new te::qt::plugins::tv::TimeViewerDockWidget();
  }

  m_tvDock->show();
  m_tvDock->raise();
}

void te::qt::plugins::tv::Plugin::onTimeDockTriggered(te::qt::af::evt::Event* e)
{
  emit triggered(e);
}

TERRALIB_PLUGIN_CALL_BACK_IMPL(te::qt::plugins::tv::Plugin)
