/*
  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

  This file is part of the TerraLib - a Framework for building GIS enabled
  applications.

  TerraLib is free software: you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation, either version 3 of the License,
  or (at your option) any later version.

  TerraLib is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with TerraLib. See COPYING. If not, write to
  TerraLib Team at <terralib-team@terralib.org>.
*/

/*!
  \file terralib/qt/plugins/timeviewer/Plugin.h

  \brief Add a plugin interface for dynamic loading of the TimeViewer plugin.

  \author Carolina Galvão dos Santos
  \author Matheus Cavassan Zaglia
  \author Fabiana Zioti

 */

#ifndef __TERRALIB_QT_PLUGINS_TIMEVIEWER_PLUGIN_H__
#define __TERRALIB_QT_PLUGINS_TIMEVIEWER_PLUGIN_H__

// TerraLib
#include "../../../core/plugin/CppPlugin.h"

// Timeviewer
#include "Config.h"

// QT
#include <QAction>

namespace te
{
  namespace qt
  {
    namespace af
    {
      namespace evt
      {
        struct Event;
      }
    }

    namespace plugins
    {
      namespace tv
      {
        class TimeViewerDockWidget;  // foward declaration

        class Plugin : public QObject, public te::core::CppPlugin
        {
          Q_OBJECT

         public:
          Plugin(const te::core::PluginInfo &pluginInfo);

          ~Plugin();

          void startup();

          void shutdown();

         Q_SIGNALS:

          void triggered(te::qt::af::evt::Event *e);

         protected slots:

          void onTimeViewerActionTriggered();
          void onTimeDockTriggered(te::qt::af::evt::Event *e);

         protected:
          QAction *m_timeViewerAction;
          te::qt::plugins::tv::TimeViewerDockWidget *m_tvDock;
        };
      }  // end namespace tv
    }    // end namespace plugins
  }      // end namespace qt
}  // end namespace te

#endif  //__TERRALIB_QT_PLUGINS_TIMEVIEWER_PLUGIN_H__
