/*
  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

  This file is part of the TerraLib - a Framework for building GIS enabled
  applications.

  TerraLib is free software: you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation, either version 3 of the License,
  or (at your option) any later version.

  TerraLib is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with TerraLib. See COPYING. If not, write to
  TerraLib Team at <terralib-team@terralib.org>.
*/

/*!
  \file terralib/qt/plugins/timeviewer/Plugin.h

  \brief Add a plugin interface for dynamic loading of the TimeViewer plugin.

  \author Carolina Galvão dos Santos
  \author Matheus Cavassan Zaglia
  \author Fabiana Zioti

 */

#ifndef __TERRALIB_QT_PLUGINS_TIMEVIEWERDOCKWIDGET_H__
#define __TERRALIB_QT_PLUGINS_TIMEVIEWERDOCKWIDGET_H__

// TerraLib
#include "../../../color/RGBAColor.h"
#include "../../../dataaccess/datasource/DataSource.h"
#include "../../../maptools/QueryLayer.h"
#include "../../af/BaseApplication.h"
#include "../../widgets/colorbar/ColorCatalogWidget.h"
#include "Config.h"
#include "TimeViewerEditAccumulateDialog.h"
#include "TimeViewerEditLegendDialog.h"

// QT
#include <QtConcurrent/qtconcurrentrun.h>
#include <QAction>
#include <QDockWidget>
#include <QEventLoop>
#include <QList>
#include <QTimer>

// STL
#include <future>

namespace Ui
{
  class TimeViewerWidget;
}

class QWidget;
class QDateTime;

namespace te
{
  // Forward declarations
  namespace color
  {
    class ColorBar;
  }

  namespace se
  {
    class Rule;
    class Style;
  }

  namespace map
  {
    class Grouping;
    class GroupingItem;
  }

  namespace qt
  {
    namespace af
    {
      namespace evt
      {
        struct Event;
      }
      class BaseApplication;
    }

    namespace plugins
    {
      namespace tv
      {
        class TimeViewerDockWidget : public QDockWidget
        {
          Q_OBJECT

         public:
          TimeViewerDockWidget(QWidget* parent = nullptr);

          ~TimeViewerDockWidget();

         protected slots:

          void onLayerCBoxSelected(int index);
          void onApplyButtonClicked();
          void onPlayButtonClicked();
          void onStopButtonClicked();
          void onPauseButtonClicked();
          void onPassButtonClicked();
          void onBackButtonClicked();
          void onSliderChanged();
          void onLegendButtoClicked();
          void onAccumulateButtonClicked();
          void onEditLegendApplyClicked();
          void onAccumulateOkClicked();
          void onAccumulateSelect();

          void onApplicationTriggered(te::qt::af::evt::Event* e);

         private:
          void setupSlider();
          void setupLayerCombo();
          void setupLayer();
          void setupLayerInt();
          void drawLayer();
          void setupPlayer();
          void buildSymbolizer(std::string date);
          void getDataAsString(std::vector<std::string>& vec, const std::string& attrName, int& nullValues);

         Q_SIGNALS:

          void triggered(te::qt::af::evt::Event* e);

         private:
          Ui::TimeViewerWidget* m_ui;

          std::string m_dsId;
          std::string m_start;
          std::string m_end;
          std::string m_datasetName;
          std::string m_class;
          std::string m_label;
          QString m_accumColor = "#dddddd";

          bool m_accumulate;
          bool m_sliderThr;

          QFuture<void> m_future;

          QEventLoop* m_drawingLoop;
          std::atomic<bool> m_drawing;

          te::da::DataSourcePtr m_dsPtr;
          std::list<te::map::AbstractLayerPtr> m_layerList;
          te::map::QueryLayerPtr m_layer;

          std::vector<te::se::Rule*> m_rules;

          std::unique_ptr<te::qt::plugins::tv::TimeViewerEditLegendDialog> m_editLegendDialog;
          std::unique_ptr<te::qt::plugins::tv::TimeViewerEditAccumulateDialog> m_editAccumulateDialog;

          te::qt::af::BaseApplication* m_app;
          std::vector<std::string> m_dates;
          std::vector<std::string> vec;
          QWidget* m_mainWidget;
        };

      }  // end namespace tv
    }    // end namespace plugins
  }      // end namespace qt
}  // end namespace te

#endif  //__TERRALIB_QT_PLUGINS_TIMEVIEWERDOCKWIDGET_H__
