/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled
   applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/timeviewer/TimeViewEditLegendDialog.cpp

  \brief A dialog used to edit legend.
*/

// TimeViewer
#include "TimeViewerEditAccumulateDialog.h"
#include "ui/ui_TimeViewerEditAccumulateDialog.h"

// Qt
#include <QColorDialog>
#include <QDialogButtonBox>
#include <QFileDialog>
#include <QPainter>

te::qt::plugins::tv::TimeViewerEditAccumulateDialog::
    TimeViewerEditAccumulateDialog(QWidget* parent, Qt::WindowFlags f)
    : QDialog(parent, f), m_ui(new Ui::TimeViewerEditAccumulateDialog)
{
  m_ui->setupUi(this);

  connect(m_ui->m_selectColorPushButton, SIGNAL(clicked()), this, SLOT(onSelectColorPushButtonClicked()));
  connect(m_ui->m_cancelPushButton, SIGNAL(clicked()), this, SLOT(onCancelPushButtonClicked()));
  connect(m_ui->m_okPushButton, SIGNAL(clicked()), this, SLOT(onOkPushButtonClicked()));
}

te::qt::plugins::tv::TimeViewerEditAccumulateDialog::~TimeViewerEditAccumulateDialog() = default;

void te::qt::plugins::tv::TimeViewerEditAccumulateDialog::setColors(
    QString accumColor)
{
  m_accumColor = accumColor;

  m_ui->m_colorLabel->setStyleSheet("QLabel {background-color:" + m_accumColor +  " ; }");
}

QString te::qt::plugins::tv::TimeViewerEditAccumulateDialog::getAccumColor()
{
  return m_accumColor;
}

QString te::qt::plugins::tv::TimeViewerEditAccumulateDialog::colorPick()
{
  QColor color = QColorDialog::getColor(Qt::yellow, this);
  QString colcode;
  QVariant variant;

  if(color.isValid())
  {
    variant = color;
    colcode = variant.toString();
  }

  return colcode;
}

void te::qt::plugins::tv::TimeViewerEditAccumulateDialog::
    onSelectColorPushButtonClicked()
{
  QString setColor = colorPick();

  if(!setColor.isEmpty())
  {
    m_accumColor = setColor;
    m_ui->m_colorLabel->setStyleSheet("QLabel { background-color :" + m_accumColor + " ;}");
  }
}

void te::qt::plugins::tv::TimeViewerEditAccumulateDialog::
    onCancelPushButtonClicked()
{
  reject();
}

void te::qt::plugins::tv::TimeViewerEditAccumulateDialog::
    onOkPushButtonClicked()
{
  accept();

  emit okAccPushButtonClicked();
}
