/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled
   applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/plugins/timeviewer/TimeViewerEditAccumulateDialog.h

  \brief A dialog used to edit Accumulate Legend.

  \author Fabiana Zioti
*/

#ifndef __TERRALIB_QT_PLUGINS_TIMEVIEWER_TIMEVIEWEREDITACCUMULATEDIALOG_H__
#define __TERRALIB_QT_PLUGINS_TIMEVIEWER_TIMEVIEWEREDITACCUMULATEDIALOG_H__

// TerraLib
#ifndef Q_MOC_RUN
#endif

#include "../../../dataaccess/datasource/DataSourceInfo.h"
#include "../../../maptools/AbstractLayer.h"
#include "Config.h"

// Qt
#include <QDialog>

// STL
#include <QString>

// Forward declaraion
namespace Ui
{
  class TimeViewerEditAccumulateDialog;
}

namespace te
{
  namespace color
  {
    class ColorBar;
  }

  namespace se
  {
    class Rule;
  }

  namespace qt
  {
    namespace widgets
    {
      // Forward declarations
      class ColorCatalogWidget;
    }
    namespace plugins
    {
      namespace tv
      {
        /*!
          \class TimeViewerEditAccumulateDialog

          \brief A dialog used to edit legend.

        */
        class TimeViewerEditAccumulateDialog : public QDialog
        {
          Q_OBJECT

         public:
          /** @name Initializer Methods
          *  Methods related to instantiation and destruction.
          */
          //@{

          /*! \brief Constructs a basic fill dialog which is a child of parent,
           * with widget flags set to f. */
          TimeViewerEditAccumulateDialog(QWidget* parent = 0, Qt::WindowFlags f = 0);

          /*! \brief Destructor. */
          ~TimeViewerEditAccumulateDialog();

          //@}

          void setColors(QString accumColor);

          QString getAccumColor();

         private:

          QString colorPick();

         protected slots:

          void onSelectColorPushButtonClicked();

          void onCancelPushButtonClicked();

          void onOkPushButtonClicked();

         signals:

          void okAccPushButtonClicked();

         private:

          std::unique_ptr<Ui::TimeViewerEditAccumulateDialog> m_ui;  //!< Dialog form.
          QString m_accumColor;
        };
      }  // end namespace tv
    }  // end namespace plugins
  }  // end namespace qt
}  // end namespace te

#endif  //  __TERRALIB_QT_PLUGINS_TIMEVIEWER_EDITACCUMULATEDIALOG_H_
