/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled
   applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/plugins/timeviewer/TimeViewerEditLegendDialog.h

  \brief A dialog used to edit legend.
*/

#ifndef __TERRALIB_QT_PLUGINS_TIMEVIEWER_TIMEVIEWEREEDITLEGEND_H__
#define __TERRALIB_QT_PLUGINS_TIMEVIEWER_TIMEVIEWEREEDITLEGEND_H__

// TerraLib
#ifndef Q_MOC_RUN
#endif

#include "../../../dataaccess/datasource/DataSourceInfo.h"
#include "../../../maptools/AbstractLayer.h"
#include "Config.h"

// Qt
#include <QDialog>
#include <QTableWidgetItem>

// STL
#include <memory>

// Forward declaraion
namespace Ui
{
  class TimeViewerEditLegendDialog;
}

namespace te
{
  namespace color
  {
    class ColorBar;
  }

  namespace se
  {
    class Rule;
  }

  namespace qt
  {
    namespace widgets
    {
      // Forward declarations
      class ColorCatalogWidget;
    }

    namespace plugins
    {
      namespace tv
      {
        /*!
          \class TimeViewerEditLegendDialog

          \brief A dialog used to edit legend.

        */
        class TimeViewerEditLegendDialog : public QDialog
        {
          Q_OBJECT

         public:
          /** @name Initializer Methods
          *  Methods related to instantiation and destruction.
          */
          //@{

          /*! \brief Constructs a basic fill dialog which is a child of parent,
           * with widget flags set to f. */
          TimeViewerEditLegendDialog(QWidget* parent = 0, Qt::WindowFlags f = 0);

          /*! \brief Destructor. */
          ~TimeViewerEditLegendDialog();

          //@}

          void setLayers(te::map::AbstractLayerPtr layer);

          const std::vector<se::Rule*>& getRules() const;

          void saveLegend(const std::string& path);

         private:

          void getDataAsString(std::vector<std::string>& vec, const std::string& attrName, int& nullValues);

          void buildSymbolizer(std::string meanTitle = "");

          void listAtrributes();

          int getGeometryType();

          void updateUi(bool loadColorBar = false);

         protected:

          void initialize();

         protected slots:

          void onApplyPushButtonClicked();

          void onColorBarChanged();

          void onCancelPushButtonClicked();

          void onOkPushButtonClicked();

          void onLoadPushButtonClicked();

          void onSavePushButtonClicked();

          void onSelectAllPushButtonClicked();

          void onUnselectAllPushButtonClicked();

         signals:

          void applyPushButtonClicked();

          void okPushButtonClicked();

         private:

          std::unique_ptr<Ui::TimeViewerEditLegendDialog> m_ui;  //!< Dialog form.

          te::map::AbstractLayerPtr m_layer;  //!< TerraLib layer auto ptr

          std::vector<te::se::Rule*> m_legend;  //!< Rule items

          te::color::ColorBar* m_cb;  //!< Terralib color bar objetc

          te::qt::widgets::ColorCatalogWidget* m_colorBar;  //!< Widget used to pick a color.

          bool m_manual;
        };
      }  // end namespace tv
    }  // end namespace plugins
  }  // end namespace qt
}  // end namespace te

#endif  //__TERRALIB_QT_PLUGINS_TIMEVIEWER_TIMEVIEWEREEDITLEGEND_H__
