/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/plugins/vp/CheckGeomValidityAction.cpp

  \brief This file defines the CheckGeomValidity class
*/

// Terralib
#include "../../../vp/qt/CheckGeomValidityDialog.h"
#include "../../af/ApplicationController.h"
#include "../../af/BaseApplication.h"
#include "../../af/events/LayerEvents.h"
#include "CheckGeomValidityAction.h"

// Qt
#include <QObject>
#include <QMessageBox>

// STL
#include <memory>

te::qt::plugins::vp::CheckGeomValidityAction::CheckGeomValidityAction(QMenu* menu)
  : te::qt::plugins::vp::AbstractAction(menu)
{
  createAction(tr("Check Geometry Validity...").toUtf8().data(), "tools-check-geom-validity");
  m_action->setObjectName("Tools.Check Geometry Validity");
}

te::qt::plugins::vp::CheckGeomValidityAction::~CheckGeomValidityAction() =
    default;

void te::qt::plugins::vp::CheckGeomValidityAction::onActionActivated(bool)
{
  te::qt::af::evt::GetAvailableLayers e;
  te::qt::af::evt::GetLayerSelected ed;

  emit triggered(&e);
  emit triggered(&ed);

  std::list<te::map::AbstractLayerPtr> layers = e.m_layers;
  te::map::AbstractLayerPtr selectedlayer = ed.m_layer;

  QWidget* parent = te::qt::af::AppCtrlSingleton::getInstance().getMainWindow();

  te::vp::CheckGeomValidityDialog* dlg = new te::vp::CheckGeomValidityDialog(parent);

  te::qt::af::BaseApplication* ba = dynamic_cast<te::qt::af::BaseApplication*>(
      te::qt::af::AppCtrlSingleton::getInstance().getMainWindow());

  dlg->setAttribute(Qt::WA_DeleteOnClose);

  dlg->setLayers(layers, selectedlayer);

  dlg->setMapDisplay(ba->getMapDisplay());

  dlg->show();
}
