/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/plugins/vp/Plugin.h

  \brief Plugin implementation for the VP Qt Plugin widget.
*/

#ifndef __TE_QT_PLUGINS_VP_INTERNAL_PLUGIN_H
#define __TE_QT_PLUGINS_VP_INTERNAL_PLUGIN_H

// TerraLib
#include "../../../core/plugin/CppPlugin.h"
#include "Config.h"

// Qt
#include <QMenu>

namespace te
{
  namespace qt
  {
    namespace af
    {
      namespace evt
      {
        struct Event;
      }
    }

    namespace plugins
    {
      namespace vp
      {
        class BufferAction;
        class CheckGeomValidityAction;
        class DifferenceAction;
        class DissolveAction;
        class GeometricOpAction;
        class IdentityAction;
        class IntersectionAction;
        class LineToPolygonAction;
        class MergeAction;
        class MultipartToSinglepartAction;
        class PolygonToLineAction;
        class UnionAction;
        class RegisterAction;

        class Plugin : public QObject, public te::core::CppPlugin
        {
          Q_OBJECT

        public:

            Plugin(const te::core::PluginInfo& pluginInfo);

            ~Plugin();

            void startup();

            void shutdown();

          protected:

            /*!
              \brief Function used to register all raster processing actions.

            */
            void registerActions();

            /*!
              \brief Function used to unregister all raster processing actions.

            */
            void unRegisterActions();

          Q_SIGNALS:

            void triggered(te::qt::af::evt::Event* e);


          protected:

            QMenu* m_vpMenu;                                      //!< VP Main Menu registered.

            BufferAction* m_buffer;                               //!< Buffer Operation Action
            AbstractAction* m_checkGeomValidity;                  //!< Check Geometry Validity Operation Action
            AbstractAction* m_difference;                         //!< Difference Operation Action
            DissolveAction* m_dissolveWithAggregation;            //!< The older dissolve operation that contains aggregation of data
            AbstractAction* m_dissolve;                           //!< Dissolve Operation Action
            AbstractAction* m_dissolveSubdivision;                //!< Dissolve Subdivision Operation Action
            GeometricOpAction* m_geometricOp;                     //!< Geometric Operation Action
            AbstractAction* m_identity;                           //!< Identity Operation Action
            AbstractAction* m_intersection;                       //!< Intersection Operation Action
            LineToPolygonAction* m_lineToPolygon;                 //!< Line to Polygon Operation Action
            MergeAction* m_merge;                                 //!< Merge Operation Action
            MultipartToSinglepartAction* m_multipart2singlepart;  //!< Multipart to Singlepart Operation Action
            PolygonToLineAction* m_polygonToLine;                 //!< Polygon to Line Operation Action
            AbstractAction* m_union;                              //!< Union Operation Action
            AbstractAction* m_subdivide;                          //!< Subdivides the geometries from the given layer
            AbstractAction* m_makeValid;                          //!< Fix all the geometries from the given layer
            AbstractAction* m_register;                          //!< Register action
        };

      } // end namespace vp
    }   // end namespace plugins
  }     // end namespace qt
}       // end namespace te

#endif //__TE_QT_PLUGINS_VP_INTERNAL_PLUGIN_H
