/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file VectorProcessingConfigWidget.cpp

  \brief A frame for setting vector processing options.
*/

// TerraLib
#include "../../../common/SystemApplicationSettings.h"
#include "../../../common/UserApplicationSettings.h"
#include "../../../core/utils/Platform.h"
#include "../../../srs/Config.h"
#include "../../af/ApplicationController.h"
#include "../../af/Utils.h"
#include "VectorProcessingConfigWidget.h"
#include "ui/ui_VectorProcessingConfigWidgetForm.h"


te::qt::plugins::vp::VectorProcessingConfigWidget::VectorProcessingConfigWidget(QWidget* parent)
  : AbstractSettingWidget(parent),
    m_ui(new Ui::VectorProcessingConfigWidgetForm)
{
  m_resumeText = tr("Changes the vector processing configurations of the application.");

  m_ui->setupUi(this);

  configSettings();

  initialize();
}

te::qt::plugins::vp::VectorProcessingConfigWidget::
    ~VectorProcessingConfigWidget() = default;

void te::qt::plugins::vp::VectorProcessingConfigWidget::saveChanges()
{
  QSettings settings(QSettings::IniFormat, QSettings::UserScope, qApp->organizationName(), qApp->applicationName());

  settings.setValue("vp_config/default_question", m_ui->m_questionCheckBox->isChecked());
  settings.setValue("vp_config/spatial_index_as_default", m_ui->m_defaultCheckBox->isChecked());

  changeApplyButtonState(false);
}

void te::qt::plugins::vp::VectorProcessingConfigWidget::resetState()
{
}

void te::qt::plugins::vp::VectorProcessingConfigWidget::getHelpInformations(QString& ns, QString& helpFile)
{
  ns = "dpi.inpe.br.apf";
  helpFile = "";
}

void te::qt::plugins::vp::VectorProcessingConfigWidget::configSettings()
{
  // Get user settings
  QSettings settings(QSettings::IniFormat,
                     QSettings::UserScope,
                     QApplication::instance()->organizationName(),
                     QApplication::instance()->applicationName());

  bool contains = settings.contains(QString("vp_config/default_question"));
  if (contains == false)
  {
    settings.beginGroup("vp_config");
    settings.endGroup();

    settings.setValue("vp_config/default_question", true);
    settings.setValue("vp_config/spatial_index_as_default", false);
  }
}

void te::qt::plugins::vp::VectorProcessingConfigWidget::initialize()
{
  QSettings set(QSettings::IniFormat, QSettings::UserScope, qApp->organizationName(), qApp->applicationName());

  set.beginGroup("vp_config");
  set.endGroup();

  // Get user settings
  QSettings settings(QSettings::IniFormat, QSettings::UserScope, QApplication::instance()->organizationName(), QApplication::instance()->applicationName());

  bool question = settings.value("vp_config/default_question", te::common::SystemApplicationSettings::getInstance().getValue("Application.vectorProcessingDefaultQuestion").c_str()).toBool();
  bool option = settings.value("vp_config/spatial_index_as_default", te::common::SystemApplicationSettings::getInstance().getValue("Application.vectorProcessingSpatialIndexAsDefault").c_str()).toBool();

  m_ui->m_questionCheckBox->setChecked(question);
  m_ui->m_defaultCheckBox->setChecked(option);

  // Signals & slots
  connect(m_ui->m_questionCheckBox, SIGNAL(toggled(bool)), SLOT(onQuestionChecked(bool)));
  connect(m_ui->m_defaultCheckBox, SIGNAL(toggled(bool)), SLOT(onSetSpatialIndexAsDefaultChecked(bool)));
}

void te::qt::plugins::vp::VectorProcessingConfigWidget::onQuestionChecked(bool checked)
{
  if(checked)
    m_ui->m_defaultCheckBox->setChecked(false);

  changeApplyButtonState(true);
}

void te::qt::plugins::vp::VectorProcessingConfigWidget::onSetSpatialIndexAsDefaultChecked(bool checked)
{
  if(checked)
    m_ui->m_questionCheckBox->setChecked(false);

  changeApplyButtonState(true);
}
