/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/plugins/vp/VectorProcessingConfigWidgetFactory.cpp

  \brief A factory to build the raster processing config frame object.
*/

#include "VectorProcessingConfigWidget.h"
#include "VectorProcessingConfigWidgetFactory.h"

te::qt::plugins::vp::VectorProcessingConfigWidgetFactory* te::qt::plugins::vp::VectorProcessingConfigWidgetFactory::sm_factory(nullptr);

void te::qt::plugins::vp::VectorProcessingConfigWidgetFactory::initialize()
{
  finalize();
  sm_factory = new VectorProcessingConfigWidgetFactory;
}

void te::qt::plugins::vp::VectorProcessingConfigWidgetFactory::finalize()
{
  delete sm_factory;
  sm_factory = nullptr;
}

te::qt::plugins::vp::VectorProcessingConfigWidgetFactory::
    ~VectorProcessingConfigWidgetFactory() = default;

te::qt::af::AbstractSettingWidget* te::qt::plugins::vp::VectorProcessingConfigWidgetFactory::build()
{
  return new VectorProcessingConfigWidget;
}

te::qt::plugins::vp::VectorProcessingConfigWidgetFactory::VectorProcessingConfigWidgetFactory() :
te::qt::af::SettingsWidgetsFactory(QObject::tr("Vector processing").toUtf8().data())
{

}

te::qt::af::AbstractSettingWidget* te::qt::plugins::vp::VectorProcessingConfigWidgetFactory::create(QWidget* parent)
{
  return new VectorProcessingConfigWidget(parent);
}
