/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file VectorProcessingConfigWidgetFactory.h

  \brief A factory to build the vector processing config frame object.
*/

#ifndef __TERRALIB_QT_PLUGINS_VP_VECTORPROCESSINGCONFIGWIDGETFACTORY_H
#define __TERRALIB_QT_PLUGINS_VP_VECTORPROCESSINGCONFIGWIDGETFACTORY_H

// TerraLib
#include "../../af/Config.h"
#include "../../af/settings/SettingsWidgetsFactory.h"

namespace te
{
  namespace qt
  {
    namespace af
    {
     class AbstractSettingWidget;
    }
    namespace plugins
    {
     namespace vp
     {
       /*!
         \class VectorProcessingConfigWidgetFactory

         \brief A factory to build the vector processing config frame object.
       */
       class VectorProcessingConfigWidgetFactory : public te::qt::af::SettingsWidgetsFactory
       {
         public:

           static void initialize();

           static void finalize();

           ~VectorProcessingConfigWidgetFactory();

         protected:

           te::qt::af::AbstractSettingWidget* build();

           VectorProcessingConfigWidgetFactory();

           virtual te::qt::af::AbstractSettingWidget* create(QWidget* parent = 0);

         private:

           static VectorProcessingConfigWidgetFactory* sm_factory;
       };
      }  // end namespace vp
    }    // end namespace plugins
  }      // end namespace qt
}        // end namespace te
#endif // __TERRALIB_QT_PLUGINS_VP_VECTORPROCESSINGCONFIGWIDGETFACTORY_H
