/*
  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

  This file is part of the TerraLib - a Framework for building GIS enabled
  applications.

  TerraLib is free software: you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation, either version 3 of the License,
  or (at your option) any later version.

  TerraLib is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with TerraLib. See COPYING. If not, write to
  TerraLib Team at <terralib-team@terralib.org>.
*/

/*!
  \file terralib/qt/plugins/wtss/plugin.h

  \brief Add a plugin interface for dynamic loading of the Web Time Series Data
  Service plugin.

  \author Matheus Cavassan Zaglia
 */

#ifndef __TERRALIB_QT_PLUGINS_WTSS_PLUGIN_H
#define __TERRALIB_QT_PLUGINS_WTSS_PLUGIN_H

// TerraLib
#include "../../../core/plugin/CppPlugin.h"

// QT
#include <QAction>
#include <QMenu>
#include <QObject>
#include <QToolBar>

// STL
#include <vector>

// wtss.tl
#include "Config.h"
#include "WtssDialog.h"

namespace te
{
  namespace qt
  {
    namespace af
    {
      namespace evt
      {
        struct Event;
      }
    }

    namespace plugins
    {
      namespace wtss
      {
        class Plugin : public QObject, public te::core::CppPlugin
        {
          Q_OBJECT

         public:

          Plugin(const te::core::PluginInfo& pluginInfo);

          ~Plugin();

          void startup();

          void shutdown();

         protected:

          void registerActions();

          void unregisterActions();

         Q_SIGNALS:

          void triggered(te::qt::af::evt::Event* e);

         protected:

          QMenu* m_menu;
          QAction* m_wtssAction;
          QAction* m_timeSeriesAction;
          QToolBar* m_wtssToolBar;

         protected slots:

          void onActionActivated(bool);

          void onActionQueryToggled();

          void onCloseTool();

         private:

          std::unique_ptr<te::qt::plugins::wtss::WtssDialog> m_wtssDlg;

        };
      }
    }
  }
}

#endif  //__TERRALIB_QT_PLUGINS_WTSS_PLUGIN_H
