/*
  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

  This file is part of the TerraLib - a Framework for building GIS enabled
  applications.

  TerraLib is free software: you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation, either version 3 of the License,
  or (at your option) any later version.

  TerraLib is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with TerraLib. See COPYING. If not, write to
  TerraLib Team at <terralib-team@terralib.org>.
*/

/*!
  \file terralib/qt/plugins/wtss/ServerManager.h

  \brief Manage the JSON file for services configuration of the Web Time Series
  Services plugin.

  \author Matheus Cavassan Zaglia
 */

#ifndef __TERRALIB_QT_PLUGINS_WTSS_SERVERMANAGER_H
#define __TERRALIB_QT_PLUGINS_WTSS_SERVERMANAGER_H

// QT
#include <QJsonDocument>

// STL
#include <string>

namespace te
{
  namespace qt
  {
    namespace plugins
    {
      namespace wtss
      {
        class ServerManager
        {
         public:

          static ServerManager &getInstance()
          {
            static ServerManager instance;
            return instance;
          }

         public:

          QJsonDocument loadSettings();

          void addServer(const QString &server_uri);
          void removeServer(const QString &server_uri);
          void refreshServer(const QString &server_uri);

          void addDateFilter(const QString startDate, const QString endDate);

          QJsonObject getDateFilter();

          QJsonObject getAttribute(const QString &server_uri,
                                   const QString &cv_name,
                                   const QString &attribute);

          void changeStatusServer(const QString &server_uri);

          void changeStatusCoverage(const QString &server_uri,
                                    const QString &cv_name);

          void changeStatusAttribute(const QString &server_uri,
                                     const QString &cv_name,
                                     const QString &attribute);

         private:

          ServerManager()
          {
          }
          ~ServerManager()
          {
          }
          // no copy allowed
          ServerManager(ServerManager const &) = delete;
          ServerManager &operator=(ServerManager const &) = delete;

          void saveSettings(QJsonDocument j_doc);

        };
      }
    }
  }  // end namespace tl
}  // end namespace wtss
#endif  // __TERRALIB_QT_PLUGINS_WTSS_SERVERMANAGER_H
