/*
  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

  This file is part of the TerraLib - a Framework for building GIS enabled
  applications.

  TerraLib is free software: you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation, either version 3 of the License,
  or (at your option) any later version.

  TerraLib is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with TerraLib. See COPYING. If not, write to
  TerraLib Team at <terralib-team@terralib.org>.
*/

/*!
  \file terralib/qt/plugins/wtss/WtssDialog.h

  \brief Configuration dialog for Web Time Series Services plugin.

  \author Matheus Cavassan Zaglia
 */

#ifndef __TERRALIB_QT_PLUGINS_WTSS_WTSSDIALOG_H
#define __TERRALIB_QT_PLUGINS_WTSS_WTSSDIALOG_H

// TerraLib
#ifndef Q_MOC_RUN
#include "../../../common.h"
#include "../../../datatype/Date.h"
#include "../../../se/Mark.h"
#include "../../../st/core/timeseries/TimeSeries.h"
#include "../../widgets/canvas/Canvas.h"
#include "../../widgets/canvas/MapDisplay.h"
#include "../../widgets/charts/ChartDisplay.h"
#include "../../widgets/charts/ChartDisplayWidget.h"
#include "../../widgets/charts/ChartStyle.h"
#include "../../widgets/charts/TimeSeriesChart.h"
#endif

// QT
#include <QColor>
#include <QDialog>
#include <QGridLayout>
#include <QListWidgetItem>
#include <QObject>
#include <QTreeWidget>
#include <QtGui>
#include <QActionGroup>

// STL
#include <memory>
#include <string>

// wtss.cxx
#include <wtss-cxx/data_types.hpp>

namespace cxx = wtss::cxx;

namespace Ui
{
  class WtssDialogForm;
}

namespace te
{
  namespace qt
  {
    namespace plugins
    {
      namespace wtss
      {
        enum treeItemType
        {
          server,
          coverage,
          attribute
        };

        class WtssDialog : public QDialog
        {
          Q_OBJECT

         public:

          WtssDialog(QWidget* parent = 0, Qt::WindowFlags f = 0);

          ~WtssDialog();

          void setMapDisplay(te::qt::widgets::MapDisplay* mapDisplay);

          void setActionGroup(QActionGroup* actionGroup);

         protected slots:

          void onServerAddButtonClicked();

          void onServerRemoveButtonClicked();

          void onServerRefreshButtonClicked();

          void onItemChecked(QTreeWidgetItem* item);

          void onHideButtonClicked();

          void onImportGraphClicked();

          void onExportGraphClicked();

          void onQueryButtonClicked();

          void onAddCoordToList(QListWidgetItem* coordSelected);

          void onRemoveCoordButtonClicked();

          void onGetPointCoordinate(QPointF& coord);

          void onPointPickerToggled(bool checked);

         signals:

          void close();

         private:

          void loadSettings();

          void addServer(QString server);

          void addCoverage(QTreeWidgetItem* serverItem, QJsonObject j_server);

          void addAtributes(QTreeWidgetItem* coverageItem,
                             QJsonObject j_coverage);

          bool validateQuery();

          void doTimeseriesQuery(cxx::timeseries_query_t query);

          void convertToTimeSeries(cxx::timeseries_query_result_t result);

          void plotTimeSeries();

          void enablePointPicker();

          void addMarker(double x, double y);

          QColor randomColor();

          void addLocation(double x, double y);

          void clearCanvas();

          void closeEvent(QCloseEvent* e);

         private:

          std::unique_ptr<Ui::WtssDialogForm> m_ui;

          bool dirty;

          bool m_checkServer;

          bool m_checkCoverage;

          bool m_checkAttribute;

          QJsonObject j_config;

          cxx::timeseries_query_result_t m_result;

          std::string m_lastQueriedServer;

          te::qt::widgets::MapDisplay* m_mapDisplay;

          te::qt::widgets::ChartDisplay* m_chartDisplay;

          QVector<te::qt::widgets::TimeSeriesChart*> m_timeSeriesChartVec;

          std::vector<te::st::TimeSeries*> m_timeSeriesVec;

          te::color::RGBAColor** m_rgbaMarker;

          QActionGroup* m_actionGroup;

          QAction* m_pickerAction;
        };
      }
    }
  }  // end namespace tl
}  // end namespace wtss

#endif  // __TERRALIB_QT_PLUGINS_WTSS_WTSSDIALOG_H
