/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

 /*!
   \file terralib/qt/widgets/canvas/AuxiliaryMapDisplayDockWidget.cpp

   \brief This file has the AuxiliaryMapDisplayDockWidget class.
 */

// TerraLib
#include "AuxiliaryMapDisplayDockWidget.h"

// Qt
#include <QSplitter>

te::qt::widgets::AuxiliaryMapDisplayDockWidget::AuxiliaryMapDisplayDockWidget(te::qt::widgets::MapDisplay* mapParent, QWidget* parent, Qt::WindowFlags f)
  : QDockWidget(parent, f)
{
  QSplitter* splitter = new QSplitter(Qt::Vertical, this);

  m_eyeBird = new te::qt::widgets::EyeBirdMapDisplayWidget(mapParent, splitter);

  m_eyeBird->setMinimumWidth(300);
  m_eyeBird->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);

  m_zoomIn = new te::qt::widgets::ZoomInMapDisplayWidget(mapParent, splitter);

  m_zoomIn->setMinimumWidth(300);
  m_zoomIn->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);

  splitter->addWidget(m_eyeBird);
  splitter->addWidget(m_zoomIn);

  QList<int> heights;
  heights.push_back(300);
  heights.push_back(300);

  splitter->setSizes(heights);

  setWidget(splitter);

  connect(this, SIGNAL(visibilityChanged(bool)), this, SLOT(onVisibilityChanged(bool)));

  onVisibilityChanged(false);
}

te::qt::widgets::AuxiliaryMapDisplayDockWidget::~AuxiliaryMapDisplayDockWidget() = default;

te::qt::widgets::EyeBirdMapDisplayWidget* te::qt::widgets::AuxiliaryMapDisplayDockWidget::getEyeBird()
{
  return m_eyeBird;
}

te::qt::widgets::ZoomInMapDisplayWidget* te::qt::widgets::AuxiliaryMapDisplayDockWidget::getZoomIn()
{
  return m_zoomIn;
}

void te::qt::widgets::AuxiliaryMapDisplayDockWidget::onVisibilityChanged(bool visible)
{
  m_zoomIn->setEnabled(visible);
  m_eyeBird->setEnabled(visible);
}
