/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/widgets/canvas/AuxiliaryMapDisplayDockWidget.h

  \brief This file has the AuxiliaryMapDisplayDockWidget class.
*/

#ifndef __TERRALIB_QT_WIDGETS_CANVAS_INTERNAL_AUXILIARYMAPDISPLAYDOCKWIDGET_H
#define __TERRALIB_QT_WIDGETS_CANVAS_INTERNAL_AUXILIARYMAPDISPLAYDOCKWIDGET_H

// TerraLib
#ifndef Q_MOC_RUN
#include "EyeBirdMapDisplayWidget.h"
#include "MapDisplay.h"
#include "ZoomInMapDisplayWidget.h"
#endif

#include "../Config.h"

// Qt
#include <QDockWidget>

namespace te
{
  namespace qt
  {
    namespace widgets
    {
      /*!
        \class AuxiliaryMapDisplayDockWidget

        \brief This class defines a dock widget with map display ZoomIn and eye bird to help the display viewer.
      */
      class TEQTWIDGETSEXPORT AuxiliaryMapDisplayDockWidget : public QDockWidget
      {
        Q_OBJECT

        public:

          AuxiliaryMapDisplayDockWidget(te::qt::widgets::MapDisplay* mapDisplay, QWidget* parent = 0, Qt::WindowFlags f = 0);

          ~AuxiliaryMapDisplayDockWidget();

        public:

          te::qt::widgets::EyeBirdMapDisplayWidget* getEyeBird();

          te::qt::widgets::ZoomInMapDisplayWidget* getZoomIn();

        protected slots:

          void onVisibilityChanged(bool visible);

        protected:

          te::qt::widgets::EyeBirdMapDisplayWidget* m_eyeBird;

          te::qt::widgets::ZoomInMapDisplayWidget* m_zoomIn;
      };

    } // end namespace widgets
  }   // end namespace qt
}     // end namespace te

#endif  // __TERRALIB_QT_WIDGETS_CANVAS_INTERNAL_AUXILIARYMAPDISPLAYDOCKWIDGET_H

