#include "../../../common/PlatformUtils.h"
#include "DrawThread.h"
#include "ThreadManager.h"

// Qt
#include <QThreadPool>
#include <QTimer>

te::qt::widgets::ThreadManager::ThreadManager(
    const std::vector<QRunnable*>& threads, int interval)
    :

      m_threads(threads),
      m_pool(nullptr),
      m_interval(interval)
{
  m_timer = new QTimer(this);
  connect(m_timer, SIGNAL(timeout()), SIGNAL(showFeedback()));
}

te::qt::widgets::ThreadManager::~ThreadManager() = default;

void te::qt::widgets::ThreadManager::run( const bool enableMultiThread )
{
  m_timer->start(m_interval);

  if( enableMultiThread == false )
  {
    m_pool = nullptr;

    for (std::vector<QRunnable*>::iterator it = m_threads.begin(); it != m_threads.end(); ++it)
    {
      ((DrawThread*)*it)->run();
    }

    m_timer->stop();

    emit finished();
  }
  else
  {
    m_pool = new QThreadPool;

    for (std::vector<QRunnable*>::iterator it = m_threads.begin(); it != m_threads.end(); ++it)
    {
      connect((DrawThread*)*it, SIGNAL(finished()), SLOT(onThreadFinished()));
      m_pool->start(*it);
    }
  }
}

void te::qt::widgets::ThreadManager::stopProccess()
{
  if(m_pool == nullptr)
    return;

  for(std::vector<QRunnable*>::iterator it = m_threads.begin(); it != m_threads.end(); ++it)
    ((DrawThread*)*it)->m_cancel = true;

  m_pool->waitForDone();
  onThreadFinished();
}

void te::qt::widgets::ThreadManager::onThreadFinished()
{
  if(m_pool == nullptr)
    return;

  if(m_pool->activeThreadCount() == 0)
  {
    m_timer->stop();

    delete m_pool;
    m_pool = nullptr;

    emit finished();
  }
}
