/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

This file is part of the TerraLib - a Framework for building GIS enabled applications.

TerraLib is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License,
or (at your option) any later version.

TerraLib is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with TerraLib. See COPYING. If not, write to
TerraLib Team at <terralib-team@terralib.org>.
*/

/*!
\file  terralib/qt/widgets/charts/TimeScaleDraw.h

\brief A class that adjusts the labels that based on temporal data on a chart.
*/

#ifndef __TERRALIB_QT_WIDGETS_INTERNAL_TIMESCALEDRAW_H
#define __TERRALIB_QT_WIDGETS_INTERNAL_TIMESCALEDRAW_H

//TerraLib
#include "../Config.h"

//QWT
#include <qwt_date_scale_draw.h>

//STL
#include <memory>

namespace te
{
  namespace qt
  {
    namespace widgets
    {
      /*!
      \class TimeScaleDraw

      \brief A class that adjusts the labels on a chart.
      */
      class TimeScaleDraw : public QwtDateScaleDraw
      {
      public:
        /*!
        \brief Constructor

        It constructs a TimeScaleDraw that has the appropriate granularity for the data that the label is displaying.

        The default format strings are:

        - Millisecond :hh:mm:ss:zzz\nddd dd MMM yyyy
        - Second: hh:mm:ss\nddd dd MMM yyyy
        - Minute: hh:mm\nddd dd MMM yyyy
        - Hour:   hh:mm\nddd dd MMM yyyy
        - Day:    ddd dd MMM yyyy
        - Week    Www yyyy
        - Month:  MMM yyyy
        - Year:   yyyy
        */
        TimeScaleDraw(std::string dateFormat = "dd MMM yyyy");

        /*! \brief Destructor. */
        ~TimeScaleDraw();

        virtual QwtText label(double v) const;

      private:
        std::string m_dateFormat;
      };
    } // end namespace widgets
  }   // end namespace qt
}     // end namespace te

#endif //__TERRALIB_QT_WIDGETS_INTERNAL_TIMESCALEDRAW_H