/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

This file is part of the TerraLib - a Framework for building GIS enabled applications.

TerraLib is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License,
or (at your option) any later version.

TerraLib is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with TerraLib. See COPYING. If not, write to
TerraLib Team at <terralib-team@terralib.org>.
*/

/*!
 * \file CustomColorCatalogDialog.h
 *
 * \brief Implements a dialog used to save a custom color catalog.
 */

#ifndef __TERRALIB_QT_WIDGETS_COLORBAR_INTERNAL_CUSTOMCOLORCATALOGDIALOG_H
#define __TERRALIB_QT_WIDGETS_COLORBAR_INTERNAL_CUSTOMCOLORCATALOGDIALOG_H

//TerraLib
#include "../Config.h"

// STL
#include <memory>

// Qt
#include <QDialog>

namespace Ui
{
  // Forward declaration
  class CustomColorCatalogDialogForm;
}

namespace te
{
  // Forward declarations
  namespace color { class ColorBar; }

  namespace qt
  {
    namespace widgets
    {
      // Forward declarations
      namespace colorbar { class ColorBar; }

      /*!
       * \class CustomColorCatalogDialog
       *
       * \brief A dialog that saves the custom color catalog.
       */
      class TEQTWIDGETSEXPORT CustomColorCatalogDialog : public QDialog
      {
        Q_OBJECT

      public:

        /*!
         * \brief Constructor.
         */
        CustomColorCatalogDialog(QWidget* parent = 0, Qt::WindowFlags f = 0);

        /*!
         * \brief Destructor.
         */
        ~CustomColorCatalogDialog();

        void setColorBar(te::color::ColorBar* colorBar);

      protected slots:

        void onOkPushButtonClicked();

      protected:

        std::unique_ptr<Ui::CustomColorCatalogDialogForm> m_ui;  //!< Pointer for Qt structure
        te::color::ColorBar* m_colorBar;
      };
    }
  }
}

#endif // __TERRALIB_QT_WIDGETS_COLORBAR_INTERNAL_CUSTOMCOLORCATALOGDIALOG_H
