/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

 /*!
   \file terralib/qt/widgets/dataset/utils/DataSetTypeFilter.h

   \brief Custom widget used filter a dataset type, creating a DataSetTypeConverter.
 */

#ifndef __TERRALIB_QT_WIDGETS_DATASET_UTILS_INTERNAL_DATASETTYPEFILTER_H
#define __TERRALIB_QT_WIDGETS_DATASET_UTILS_INTERNAL_DATASETTYPEFILTER_H

 // TerrLib
#include "../../Config.h"

// Qt
#include <QGridLayout>
#include <QWidget>

// STL
#include <memory>

namespace te
{
  namespace da
  {
    class DataAccess;
    class DataSetType;
    class DataSetTypeConverter;
    class DataSourceCapabilities;
  }

  namespace qt
  {
    namespace widgets
    {
      class DoubleListWidget;

      /*!
      \class DataSetTypeFilter

      \brief Custom widget used filter a dataset type, creating a DataSetTypeConverter.

      \note If input DataSetType has primary key, the output will keep the same pk.

      \note If input DataSetType has geom prop, the output will keep the geom prop.


    */
      class TEQTWIDGETSEXPORT DataSetTypeFilter : public QWidget
      {
        Q_OBJECT

        public:

          /*! \brief Default constructor. */
          DataSetTypeFilter(QWidget* parent = 0, Qt::WindowFlags f = 0);

          /*! \brief Virtual destructor. */
          ~DataSetTypeFilter();

        public:

          /*!
            \brief Inits the component based on the given dataSetType

            \param dataSetType A valid dataSetType. This class will not take the ownership of the pointer
            \param selectAllAttrs A flag used to select all attributes from input dataset type to output converter
          */
          void init(const te::da::DataSetType* dataSetType, bool selectAllAttrs = true);

          /*!
            \brief Applies the selected attributes filter to the given Converter

            \return The converter to be adapter
          */
          void applyFilterToConverter(te::da::DataSetTypeConverter* converter) const;

          /*!
            \brief Applies the selected attributes filter to the given DataAccess

            \return The dataAccess to be adapter
          */
          void applyFilterToDataAccess(te::da::DataAccess* dataAccess) const;

          /*!
            \brief Creates a new converter and applies the selected attributes filter

            \return The new converter
          */
          te::da::DataSetTypeConverter* getConverter(const te::da::DataSetType* dataSetType, const te::da::DataSourceCapabilities& capabilities, const std::string& resultDataSetName) const;

      private:

          std::unique_ptr<te::qt::widgets::DoubleListWidget> m_doubleListWidget;
      };
    }
  }
}

#endif //__TERRALIB_QT_WIDGETS_DATASET_UTILS_INTERNAL_DATASETTYPEFILTER_H
