/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/widgets/datasource/explorer/DataSourceExplorer.cpp

  \brief A widget designed to explore the data sources of an application.
*/

// TerraLib
#include "DataSourceExplorer.h"

#include "../core/DataSourceTypeManager.h"

// Qt
#include <QTabWidget>


te::qt::widgets::DataSourceExplorer::DataSourceExplorer(QWidget* parent, Qt::WindowFlags flags)
  : QDockWidget(tr("Data Source Explorer"), parent, flags)
{
  QTabWidget* tabWidget = new QTabWidget(this);
  tabWidget->setTabPosition(QTabWidget::South);

  m_fileSytemTreeView = new FileSystemTreeView(tabWidget);
  getDataSourceTypesSupportedExtensions();
  tabWidget->addTab(m_fileSytemTreeView, QString("File System"));
  
  m_dataSourceTreeWidget = new DataSourceTreeWidget(tabWidget);
  getDataSourceTypes();
  tabWidget->addTab(m_dataSourceTreeWidget, QString("DataSource"));

  setWidget(tabWidget);

  //connects
  connect(m_fileSytemTreeView, SIGNAL(layersCreated(const std::list<te::map::AbstractLayerPtr>&)), this, SLOT(onCreatedLayers(const std::list<te::map::AbstractLayerPtr>&)));
  connect(m_dataSourceTreeWidget, SIGNAL(layersCreated(const std::list<te::map::AbstractLayerPtr>&)), this, SLOT(onCreatedLayers(const std::list<te::map::AbstractLayerPtr>&)));
}

te::qt::widgets::DataSourceExplorer::~DataSourceExplorer() = default;

void te::qt::widgets::DataSourceExplorer::getDataSourceTypesSupportedExtensions()
{
  DataSourceTypeManager::const_iterator it = DataSourceTypeManager::getInstance().begin();
  DataSourceTypeManager::const_iterator itend = DataSourceTypeManager::getInstance().end();

  std::vector<std::string> extVec;
  std::vector<std::string> dsTypes;
  

  while (it != itend)
  {
    std::string driver;

    it->second->getSupportedExtensions(driver, extVec);

    if (driver.empty() == false)
    {
      dsTypes.push_back(driver);
    }

    ++it;
  }

  QStringList stringList;

  for (std::size_t i = 0; i < extVec.size(); ++i)
  {
    stringList.append(extVec[i].c_str());
  }

  m_fileSytemTreeView->setNameFilters(stringList);
  m_fileSytemTreeView->setDataSourceTypesNames(dsTypes);
}

void te::qt::widgets::DataSourceExplorer::getDataSourceTypes()
{
  DataSourceTypeManager::const_iterator it = DataSourceTypeManager::getInstance().begin();
  DataSourceTypeManager::const_iterator itend = DataSourceTypeManager::getInstance().end();

  std::vector<te::qt::widgets::DataSourceType*> dataSourcesType;

  while (it != itend)
  {
    dataSourcesType.push_back(it->second);

    ++it;
  }

  m_dataSourceTreeWidget->setDataSourceTypes(dataSourcesType);
}

void te::qt::widgets::DataSourceExplorer::onCreatedLayers(const std::list<te::map::AbstractLayerPtr>& layers)
{
  if (layers.empty())
  {
    return;
  }

  emit addLayers(layers);
}
