/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/widgets/datasource/explorer/DataSourceExplorer.h

  \brief A widget designed to explore the data sources of an application.
*/

#ifndef __TERRALIB_QT_WIDGETS_INTERNAL_DATASOURCEEXPLORER_H
#define __TERRALIB_QT_WIDGETS_INTERNAL_DATASOURCEEXPLORER_H

// TerraLib
#include "DataSourceTreeWidget.h"
#include "FileSystemTreeView.h"
#include "../core/DataSourceType.h"
#include "../../Config.h"
#include "../../../../maptools/AbstractLayer.h"

// Qt
#include <QDockWidget>

namespace te
{
  namespace qt
  {
    namespace widgets
    {
      /*!
        \class DataSourceExplorer

        \brief A widget designed to explore the data sources of an application.

        \ingroup widgets
      */
      class TEQTWIDGETSEXPORT DataSourceExplorer : public QDockWidget
      {
        Q_OBJECT

        public:

          DataSourceExplorer(QWidget* parent = 0, Qt::WindowFlags flags = 0);

          ~DataSourceExplorer();

        protected:

          void getDataSourceTypesSupportedExtensions();

          void getDataSourceTypes();

        signals:

          void addLayers(const std::list<te::map::AbstractLayerPtr>& layers);

        protected slots:

          void onCreatedLayers(const std::list<te::map::AbstractLayerPtr>& layers);

        protected:

          FileSystemTreeView* m_fileSytemTreeView;
          DataSourceTreeWidget* m_dataSourceTreeWidget;
      }; 
    } // end namespace widgets
  }   // end namespace qt
}     // end namespace te

#endif  // __TERRALIB_QT_WIDGETS_INTERNAL_DATASOURCEEXPLORER_H
