/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

 /*!
   \file DataSourceTreeWidget.h

   \brief A tree widget for the data sources of an application.
 */

#ifndef __TERRALIB_QT_WIDGETS_INTERNAL_DATASOURCETREEWIDGET_H
#define __TERRALIB_QT_WIDGETS_INTERNAL_DATASOURCETREEWIDGET_H

 // TerraLib
#include "../core/DataSourceType.h"
#include "../../Config.h"
#include "../../../../maptools/AbstractLayer.h"

// Qt
#include <QMenu>
#include <QTreeWidget>

namespace te
{
  namespace qt
  {
    namespace widgets
    {
      /*!
        \class DataSourceTreeWidget

        \brief A tree widget for the data sources of an application.

      */
      class TEQTWIDGETSEXPORT DataSourceTreeWidget : public QTreeWidget
      {
        Q_OBJECT

        /** DataSourceTreeWidget Item Type enum.
        *  Possible ways for DataSourceTreeWidget items.
        */
        enum DataSourceTreeWidgetItemType
        {
          DATASOURCETYPE, 
          DATASOURCE,
          DATASET
        };

        public:

          DataSourceTreeWidget(QWidget* parent = 0);

          ~DataSourceTreeWidget();

        public:

          void setDataSourceTypes(std::vector<te::qt::widgets::DataSourceType*> dsTypes);

        protected:

          void initMenus();

          void loadDataSources(QTreeWidgetItem* item);

          void loadDataSets(QTreeWidgetItem* item);

        signals:

          void layersCreated(const std::list<te::map::AbstractLayerPtr>& layers);

        protected slots:

          void slotItemDoubleClicked(QTreeWidgetItem* item, int column);

          void slotCustomContextMenu(const QPoint &point);

          void newConnection();

          void dataSourceTypeUpdate();

          void dataSourceUpdate();

          void createLayer();

        protected:

          QMenu* m_dataSourceTypeMenu;
          QMenu* m_dataSourceMenu;
          QMenu* m_dataSetMenu;

          std::vector<te::qt::widgets::DataSourceType*> m_dsTypes;

      };
    } // end namespace widgets
  }   // end namespace qt
}     // end namespace te

#endif  // __TERRALIB_QT_WIDGETS_INTERNAL_DATASOURCETREEWIDGET_H
