/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file FileSystemTreeView.h

  \brief A tree view for the file system of an application.
*/

#ifndef __TERRALIB_QT_WIDGETS_INTERNAL_FILESYSTEMTREEVIEW_H
#define __TERRALIB_QT_WIDGETS_INTERNAL_FILESYSTEMTREEVIEW_H

// TerraLib
#include "../../Config.h"
#include "../../../../maptools/AbstractLayer.h"

// Qt
#include <QEvent>
#include <QFileInfo>
#include <QFileSystemModel>
#include <QTreeView>

namespace te
{
  namespace qt
  {
    namespace widgets
    {
      /*!
        \class FileSystemTreeView

        \brief A tree view for the file system of an application.

      */
      class TEQTWIDGETSEXPORT FileSystemTreeView : public QTreeView
      {
        Q_OBJECT

        public:

          FileSystemTreeView(QWidget* parent = 0);

          ~FileSystemTreeView();

        public:

          void setNameFilters(QStringList list);

          void setDataSourceTypesNames(std::vector<std::string> dsTypesNames);

        signals:

          void clicked(QFileInfo info);

          void layersCreated(const std::list<te::map::AbstractLayerPtr>& layers);

        protected slots:

          void itemClicked(const QModelIndex & index);

          void itemDoubleClicked(const QModelIndex & index);

          void slotCustomContextMenu(const QPoint &point);

        protected:

          std::list<te::map::AbstractLayerPtr> createLayer(const QFileInfo& file);

        protected:

          QFileSystemModel* m_model;

          std::vector<std::string> m_dataSourceTypesNames;
      }; 
    } // end namespace widgets
  }   // end namespace qt
}     // end namespace te

#endif  // __TERRALIB_QT_WIDGETS_INTERNAL_FILESYSTEMTREEVIEW_H
