#include "FolderItem.h"
#include "LayerItem.h"

te::qt::widgets::VISIBLE GetVisibility(const te::qt::widgets::TreeItem* item)
{
  size_t c = item->getChildrenCount("");
  size_t nv = 0;

  if(c == 0)
    return te::qt::widgets::NONE;

  for(size_t i = 0; i < c; i++)
  {
    if(item->getChild(i)->isVisible() == te::qt::widgets::NONE)
      nv++;
  }

  if(nv == c)
    return te::qt::widgets::NONE;

  if(nv == 0)
    return te::qt::widgets::TOTALLY;

  return te::qt::widgets::PARTIALLY;
}

void CheckLayer(te::map::AbstractLayerPtr folderLayer, te::map::AbstractLayerPtr newItemLayer)
{
  std::size_t count = folderLayer->getChildrenCount();

  for (std::size_t i = 0; i < count; ++i)
  {
    te::common::TreeItemPtr curItemPtr = folderLayer->getChild(i);

    te::map::AbstractLayer* layer = dynamic_cast<te::map::AbstractLayer*>(curItemPtr.get());

    if (layer && layer->getId() == newItemLayer->getId())
    {
      folderLayer->remove(i);
      break;
    }
  }
}

te::map::AbstractLayerPtr GetLayer(te::qt::widgets::TreeItem* item)
{
  te::map::AbstractLayerPtr child = nullptr;

  if (item->getType() == "FOLDER")
  {
    te::qt::widgets::FolderItem* layerItem = dynamic_cast<te::qt::widgets::FolderItem*>(item);
    child = layerItem->getLayer();
  }
  else if (item->getType() == "LAYER")
  {
    te::qt::widgets::LayerItem* layerItem = dynamic_cast<te::qt::widgets::LayerItem*>(item);
    child = layerItem->getLayer();
  }

  return child;
}

te::qt::widgets::FolderItem::FolderItem(te::map::AbstractLayerPtr layer):
  TreeItem("FOLDER"),
  m_layer(layer)
{

}

te::qt::widgets::FolderItem::~FolderItem()
{
  if (m_layer->getParent() != nullptr)
  {
    m_layer->getParent()->remove(m_layer->getIndex());
  }
}

std::string te::qt::widgets::FolderItem::getAsString() const
{
  return m_layer->getTitle();
}

te::map::AbstractLayerPtr te::qt::widgets::FolderItem::getLayer() const
{
  return m_layer;
}

te::qt::widgets::VISIBLE te::qt::widgets::FolderItem::isVisible() const
{
  return GetVisibility(this);
}

void te::qt::widgets::FolderItem::setVisible(const VISIBLE& visible, const bool& updateAncestors, const bool& updateDescendents)
{
  if(m_parent == nullptr)
    return;

  if(updateDescendents)
  {
    unsigned int c = (unsigned int)getChildrenCount("");

    for(unsigned int i = 0; i < c; i++)
      getChild(i)->setVisible(visible, false, true);
  }

  te::qt::widgets::VISIBLE v = GetVisibility(this);

  if(updateAncestors)
    m_parent->setVisible(v, true);
}

Qt::ItemFlags te::qt::widgets::FolderItem::flags()
{
  return TreeItem::flags() | Qt::ItemIsEditable | Qt::ItemIsDragEnabled | Qt::ItemIsUserCheckable;
}

void te::qt::widgets::FolderItem::setTitle(const std::string& title)
{
  m_layer->setTitle(title);
}

void te::qt::widgets::FolderItem::addChild(TreeItem* item)
{
  te::qt::widgets::TreeItem::addChild(item);

  te::map::AbstractLayerPtr child = GetLayer(item);

  if (child)
  {
    //check if folder alreay has this layer and then remove
    CheckLayer(m_layer, child);

    m_layer->add(child);
  }
}

void te::qt::widgets::FolderItem::insertChild(TreeItem* item, const size_t& pos) 
{
  te::qt::widgets::TreeItem::insertChild(item, pos);

  te::map::AbstractLayerPtr child = GetLayer(item);

  if (child)
  {
    //check if folder alreay has this layer and then remove
    CheckLayer(m_layer, child);

    m_layer->insert(pos, child);
  }
}