﻿#include "LayerViewDelegate.h"
#include "LayerItem.h"
#include "LegendItem.h"
#include "ColorMapSliceItem.h"
#include "ChartSliceItem.h"
#include "RuleItem.h"

#include "../../../../maptools/AbstractLayer.h"
#include "../../../../srs/Config.h"

te::qt::widgets::LayerViewDelegate::LayerViewDelegate(QStyledItemDelegate* decorated, QObject* parent):
te::common::Decorator<QStyledItemDelegate>(decorated)
{
  setParent(parent);

  m_layer = QIcon::fromTheme("dataset-layer");
  m_folder = QIcon::fromTheme("folder");
  m_style = QIcon::fromTheme("style");
  m_colorMap = QIcon::fromTheme("grouping");
  m_chart = QIcon::fromTheme("chart-pie");
  m_queryLayer = QIcon::fromTheme("query-layer");
  m_tabularLayer = QIcon::fromTheme("dataset-layer-tabular");
  m_invalidLayer = QIcon::fromTheme("dataset-layer-invalid");
  m_warningLayer = QIcon::fromTheme("dataset-layer-warning");
}

te::qt::widgets::LayerViewDelegate::~LayerViewDelegate() = default;

void te::qt::widgets::LayerViewDelegate::paint(QPainter * painter, const QStyleOptionViewItem & option, const QModelIndex & index) const
{
  if (!index.isValid())
    return;

#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
  QStyleOptionViewItem opt(option);
  opt.features |= QStyleOptionViewItem::HasDecoration;
#else
  QStyleOptionViewItemV4 opt(option);
  opt.features |= QStyleOptionViewItemV4::HasDecoration;
#endif
  
  
  
  TreeItem* item = static_cast<TreeItem*>(index.internalPointer());

  QIcon ic = (item->getType() == "LEGEND") ? ((LegendItem*)item)->getIcon() : 
    (item->getType() == "RULE") ? ((RuleItem*)item)->getIcon() :
    (item->getType() == "STYLE") ? m_style :
    (item->getType() == "COLORMAP" || item->getType() == "GROUPING") ? m_colorMap :
    (item->getType() == "COLORMAPSLICE") ? ((ColorMapSliceItem*)item)->getIcon() :
    (item->getType() == "CHART") ? m_chart :
    (item->getType() == "CHARTSLICE") ? ((ChartSliceItem*)item)->getIcon() :
    (item->getType() == "FOLDER") ? m_folder : getIcon(index);

  QSize s(20, 20);

  if(opt.icon.isNull())
    opt.icon = ic;

  opt.decorationSize = s;

  if(m_decorated != nullptr)
  {
    m_decorated->paint(painter, opt, index);
    return;
  }

  QStyledItemDelegate::paint(painter, opt, index);
}

QIcon te::qt::widgets::LayerViewDelegate::getIcon(const QModelIndex & index) const
{
  if(!index.isValid())
    throw;

  TreeItem* item = static_cast<TreeItem*>(index.internalPointer());
  
  if(item->getType() != "LAYER")
    throw;

  te::map::AbstractLayerPtr l = ((LayerItem*)item)->getLayer();

  if(!l->getLastValidatedValue())
    return m_invalidLayer;

  if(l->getType() == "QUERYLAYER")
    return m_queryLayer;

  std::unique_ptr<te::da::DataSetType> schema = l->getSchema();
  
  if(schema.get())
  {
    if ((!schema->hasGeom() && !schema->hasRaster()) || l->getType() == "DATASETADAPTERLAYER")
      return m_tabularLayer;

    if (l->getType() == "DATASETLAYER")
    {
      // check PK
      if (schema->hasRaster() == false && schema->getPrimaryKey() == nullptr)
      {
        return m_warningLayer;
      }

      // check SRID
      if (l->getSRID() == TE_UNKNOWN_SRS)
      {
        return m_warningLayer;
      }
    }
  }

  return m_layer;
}
