/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

This file is part of the TerraLib - a Framework for building GIS enabled applications.

TerraLib is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License,
or (at your option) any later version.

TerraLib is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with TerraLib. See COPYING. If not, write to
TerraLib Team at <terralib-team@terralib.org>.
*/

/*!
* \file layer/explorer/StyleItem.cpp
*/
#include "StyleItem.h"

// TerraLib
#include "../../../../color/RGBAColor.h"
#include "../../../../maptools/AbstractLayer.h"
#include "../../../../se/Description.h"
#include "../../../../se/Rule.h"
#include "../../../../se/Symbolizer.h"
#include "../../../../se/Style.h"

#include "LayerItem.h"
#include "RuleItem.h"

// Qt
#include <QObject>

void AddRuleItems(te::qt::widgets::StyleItem* item, const te::se::Style* style)
{
  std::vector<te::se::Rule*> rules = style->getRules();

  for (std::size_t i = 0; i != rules.size(); ++i)
  {
    te::qt::widgets::RuleItem* ruleItem = new  te::qt::widgets::RuleItem(rules[i]);

    item->addChild(ruleItem);
  }
}

te::qt::widgets::StyleItem::StyleItem(te::se::Style* style) :
  TreeItem("STYLE"),
  m_style(style)
{
  m_label = (QObject::tr("Style")).toUtf8().data();

  if (m_style->getName())
    m_label = *m_style->getName();
  else
    m_label = QObject::tr("Default Style").toUtf8().data();

  AddRuleItems(this, m_style);
}

te::qt::widgets::StyleItem::~StyleItem() = default;

std::string te::qt::widgets::StyleItem::getAsString() const
{
  return m_label;
}

Qt::ItemFlags te::qt::widgets::StyleItem::flags()
{
  return TreeItem::flags();
}

std::string te::qt::widgets::StyleItem::getToolTip() const
{
  std::string toolTip;

  if (m_style && m_style->getDescription())
    toolTip = m_style->getDescription()->getTitle();

  return toolTip;
}

te::se::Style* te::qt::widgets::StyleItem::getStyle()
{
  return m_style;
}
