/*  Copyright (C) 2017 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/widgets/layer/info/AbstractFeatureInfoMaker.h

  \brief AbstractFeatureInfoMaker is an abstraction to construct a QTreeWidgetItem to be
         used on Info tool screen based on TerraLib layers types.
*/

#ifndef __TERRALIB_QT_WIDGETS_LAYER_INTERNAL_ABSTRACTFEATUREINFOMAKER_H
#define __TERRALIB_QT_WIDGETS_LAYER_INTERNAL_ABSTRACTFEATUREINFOMAKER_H

#include "../../Config.h"
#include "../../../../maptools/AbstractLayer.h"

// Qt
#include <QTreeWidgetItem>

namespace te
{
  namespace gm
  {
    class Envelope;
  }

  namespace map
  {
    class MapDisplay;
  }

  namespace qt
  {
    namespace widgets
    {
    // Forward declarations
    class MapDisplay;

      class TEQTWIDGETSEXPORT AbstractFeatureInfoMaker
      {
        public:

          /*! \brief Constructor */
          AbstractFeatureInfoMaker();

          /*! \brief Destructor. */
          virtual ~AbstractFeatureInfoMaker();

        public:

          /*!
           * \brief This function is an abstraction to get feature information.
           *        The information retrieved must be embedded in QTreeWidgetItem.
           *
           * \param layerItem QTreeWidgetItem that will be populated.
           * \param display MapDisplay that will be passed by Info tool.
           * \param layer AbstractLayerPtr to retrieve the feature information.
           * \param e Envelope with the box based on user's click.
           */
          virtual void getInfo(QTreeWidgetItem* layerItem, MapDisplay* display, const te::map::AbstractLayerPtr& layer, const te::gm::Envelope& e) = 0;
      };
    }
  }
}

#endif  // __TERRALIB_QT_WIDGETS_LAYER_INTERNAL_ABSTRACTFEATUREINFOMAKER_H
