/*  Copyright (C) 2017 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/widgets/layer/info/FeatureInfoMakerFactory.cpp

  \brief FeatureInfoMakerFactory is the base implementation of a factory to make
  AbstractFeatureInfoMaker objects.
*/

#include "FeatureInfoMakerFactory.h"


te::qt::widgets::FeatureInfoMakerFactory::FeatureInfoMakerFactory(const std::string &factoryKey)
  : te::common::AbstractFactory<AbstractFeatureInfoMaker, std::string>(factoryKey)
{
}

te::qt::widgets::AbstractFeatureInfoMaker* te::qt::widgets::FeatureInfoMakerFactory::make()
{
  return te::common::AbstractFactory<AbstractFeatureInfoMaker, std::string>::make("DEFAULT_FEATURE_INFO");
}

te::qt::widgets::AbstractFeatureInfoMaker* te::qt::widgets::FeatureInfoMakerFactory::make(const std::string &makerKey)
{
  if(!getDictionary().find(makerKey))
  {
    return FeatureInfoMakerFactory::make();
  }

  return te::common::AbstractFactory<AbstractFeatureInfoMaker, std::string>::make(makerKey);
}

te::qt::widgets::FeatureInfoMakerFactory::~FeatureInfoMakerFactory() = default;
