/*  Copyright (C) 2017 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/widgets/layer/info/FeatureInfoMakerFactory.h

  \brief FeatureInfoMakerFactory is the base implementation of a factory to make
  AbstractFeatureInfoMaker objects.
*/

#ifndef __TERRALIB_QT_WIDGETS_LAYER_INTERNAL_FEATUREINFOMAKERFACTORY_H
#define __TERRALIB_QT_WIDGETS_LAYER_INTERNAL_FEATUREINFOMAKERFACTORY_H

//TerraLib
#include "../../Config.h"
#include "../../../../common/AbstractFactory.h"

#include "AbstractFeatureInfoMaker.h"

namespace te
{
  namespace qt
  {
    namespace widgets
    {

    class TEQTWIDGETSEXPORT FeatureInfoMakerFactory : public te::common::AbstractFactory<AbstractFeatureInfoMaker, std::string>
    {

    public:

      /*!
       * \brief It creates and returns the default AbstractFeatureInfoMaker
       *
       * \return A default AbstractFeatureInfoMaker
       *
       * \note The caller will take the ownership of the returned pointer
       */
      static AbstractFeatureInfoMaker* make();

      /*!
       * \brief It creates and returns the specific AbstractFeatureInfoMaker
       *
       * \return A specific AbstractFeatureInfoMaker
       *
       * \param makerKey The name of specific feature info maker
       *
       * \note The caller will take the ownership of the returned pointer
       */
      static AbstractFeatureInfoMaker* make(const std::string& makerKey);

      /*! \brief Destructor. */
      virtual ~FeatureInfoMakerFactory();

    protected:

      /*!
        \brief Constructor.

        \param factoryKey The key that identifies the factory.
      */
      FeatureInfoMakerFactory(const std::string& factoryKey);

    };

    }
  }
}

#endif // __TERRALIB_QT_WIDGETS_LAYER_INTERNAL_FEATUREINFOMAKERFACTORY_H
