/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/widgets/tools/InfoDockWidget.cpp

  \brief A dock widget used control the geographic data style using SE elements and a property browser to show its properties.
*/

// TerraLib
#include <terralib/dataaccess/utils/Utils.h>
#include <terralib/maptools/AbstractLayer.h>
#include <terralib/maptools/Utils.h>
#include "InfoDockWidget.h"
#include <terralib/qt/widgets/tools/Info.h>
#include <terralib/qt/widgets/Utils.h>

// Qt
#include <QLayout>
#include <QScrollArea>
#include <QTabWidget>
#include <QVBoxLayout>
#include <QComboBox>
#include <QSpacerItem>
#include <QLabel>
#include <QTreeWidget>

te::qt::widgets::InfoDockWidget::InfoDockWidget(QWidget* parent, Qt::WindowFlags flags)
  : QDockWidget(tr("Information"), parent, flags),
    m_cmbMode(nullptr),
    m_infoTreeWidget(nullptr)
{
  setContextMenuPolicy(Qt::CustomContextMenu);
  setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);
    
  buildUi();
}

te::qt::widgets::InfoDockWidget::~InfoDockWidget()
{

}

void te::qt::widgets::InfoDockWidget::layerSelected(te::map::AbstractLayerPtr layer)
{

}

void te::qt::widgets::InfoDockWidget::layerAdded(te::map::AbstractLayerPtr layer)
{

}

void te::qt::widgets::InfoDockWidget::layerRemoved(te::map::AbstractLayerPtr layer)
{

}

void te::qt::widgets::InfoDockWidget::updateUi()
{

}

void te::qt::widgets::InfoDockWidget::buildUi()
{
  QWidget* w = new QWidget(this);
  w->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);

  QGridLayout* layout = new QGridLayout(w);

  buildTree();
    
  QWidget* widgetComponents = new QWidget(w);
  widgetComponents->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);

  QVBoxLayout* vLayout = new QVBoxLayout(widgetComponents);

  QHBoxLayout* hLayout = new QHBoxLayout(); 
  QSpacerItem* spacerItemAfter = new QSpacerItem(10, 10, QSizePolicy::Expanding, QSizePolicy::Preferred);
  QSpacerItem* spacerItemBefore = new QSpacerItem(10, 10, QSizePolicy::Expanding, QSizePolicy::Preferred);

  QLabel* lblMode = new QLabel(widgetComponents);
  lblMode->setObjectName("m_lblMode");
  lblMode->setText(tr("Mode"));

  QStringList list;
  list.append(tr("Visible layers top to bottom"));
  list.append(tr("Selected layers"));
  
  m_cmbMode = new QComboBox(widgetComponents);
  m_cmbMode->setObjectName("m_cmbMode");
  
  m_cmbMode->addItems(list);
  m_cmbMode->setCurrentIndex(0);
  
  hLayout->addSpacerItem(spacerItemAfter);
  hLayout->addWidget(lblMode);
  hLayout->addWidget(m_cmbMode);

  vLayout->addWidget(m_infoTreeWidget);
  vLayout->addLayout(hLayout);
  vLayout->addSpacerItem(spacerItemBefore);

  QScrollArea* sa = new QScrollArea;
  sa->setWidgetResizable(true);
  layout->addWidget(sa, 0, 0);

  sa->setWidget(widgetComponents);

  setWidget(w);
}

void te::qt::widgets::InfoDockWidget::buildTree()
{
  // Setup the widget that will be used to show the informations
  
  m_infoTreeWidget = new QTreeWidget(this);

  m_infoTreeWidget->setContextMenuPolicy(Qt::CustomContextMenu);
  m_infoTreeWidget->setWindowTitle(tr("Information"));
  m_infoTreeWidget->setWindowFlags(Qt::Tool);
  m_infoTreeWidget->setAlternatingRowColors(true);
  m_infoTreeWidget->setMinimumSize(300, 300);
  m_infoTreeWidget->setColumnCount(2);

  QStringList labels;
  labels << tr("Property") << tr("Value");
  m_infoTreeWidget->setHeaderLabels(labels);
}

void te::qt::widgets::InfoDockWidget::closeEvent(QCloseEvent* event)
{
  m_infoTreeWidget->clear();
}

void te::qt::widgets::InfoDockWidget::onGeometrySelected()
{
  // Active tool but the dock is closed
  setVisible(true);
  raise();
}

QTreeWidget* te::qt::widgets::InfoDockWidget::getInfoTree()
{
  return m_infoTreeWidget;
}

QComboBox* te::qt::widgets::InfoDockWidget::getModeCombo()
{
  return m_cmbMode;
}
