/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/widgets/tools/InfoDockWidget.h

  \brief A dock widget used control the geographic data style using SE elements and a property browser to show its properties.
*/

#ifndef __TERRALIB_QT_WIDGETS_SE_INTERNAL_INFODOCKWIDGET_H
#define __TERRALIB_QT_WIDGETS_SE_INTERNAL_INFODOCKWIDGET_H

// TerraLib
#ifndef Q_MOC_RUN
#include <terralib/qt/widgets/InterfaceController.h>
#endif
#include <terralib/qt/widgets/Config.h>

// STL
#include <list>

// Qt
#include <QDockWidget>
#include <QCursor>

class QTabWidget;
class QComboBox;
class QTreeWidget;
class QCloseEvent;

namespace te
{
  namespace map 
  {
    class AbstractLayer; 
  }

  namespace qt
  {
    namespace widgets
    {
      // Forward declarations
      class Info;
      class MapDisplay;

      /*!
        \class InfoDockWidget

        \brief A dock widget used control the geographic data style using SE elements and a property browser to show its properties.
      */
      class TEQTWIDGETSEXPORT InfoDockWidget : public QDockWidget, public InterfaceController
      {
        Q_OBJECT

        public:

          /** @name Initializer Methods
           *  Methods related to instantiation and destruction.
           */
          //@{

          /*! \brief Constructs a InfoDockWidget. */
          InfoDockWidget(QWidget* parent = 0, Qt::WindowFlags flags = 0);

          /*! \brief Destructor. */
          virtual ~InfoDockWidget();

          /*!
            \brief This method is used to set current layer

          */
          virtual void layerSelected(te::map::AbstractLayerPtr layer) override;

          /*!
            \brief This method is used to add a new layer

          */
          virtual void layerAdded(te::map::AbstractLayerPtr layer) override;

          /*!
            \brief This method is used to remove a layer

          */
          virtual void layerRemoved(te::map::AbstractLayerPtr layer) override;

          QTreeWidget* getInfoTree();

          QComboBox* getModeCombo();
                      
        protected:

          void updateUi();

          void buildUi();

          void buildTree();

          virtual void closeEvent(QCloseEvent* event);
          
        protected slots:  
                    
          void onGeometrySelected();
                              
        private:
                             
          QComboBox* m_cmbMode;
          QTreeWidget* m_infoTreeWidget; //!< Widget used to show the informations.
      };

    } // end namespace widgets
  }   // end namespace qt
}     // end namespace te

#endif  // __TERRALIB_QT_WIDGETS_SE_INTERNAL_INFODOCKWIDGET_H
