/*
  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

  This file is part of the TerraLib - a Framework for building GIS enabled
  applications.

  TerraLib is free software: you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation, either version 3 of the License,
  or (at your option) any later version.

  TerraLib is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with TerraLib. See COPYING. If not, write to
  TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/widgets/ceditor/LogTextEdit.h

  \brief A TextEdit that can be used to watch a log file.
*/

#ifndef __TERRALIB_QTWIDGETS_LOG_LOGTEXTEDIT_H__
#define __TERRALIB_QTWIDGETS_LOG_LOGTEXTEDIT_H__

// TerraLib
#include "../Config.h"
#include "../../../core/filesystem/FileWatcher.h"

// STL
#include <memory>

// QT
#include <QPlainTextEdit>

namespace te
{

  namespace qt
  {
    namespace widgets
    {
      class TEQTWIDGETSEXPORT LogTextEdit : public QPlainTextEdit
      {
       Q_OBJECT
       public:
        LogTextEdit(QWidget *parent = 0, const QString& fileName = "");
        ~LogTextEdit();

       public slots:
        void findAll(const QString &str);

       private slots:
        void textReady();
        void read();
                         
       signals:
        void textPending();

       private:
        QFile m_file;
        std::unique_ptr<te::core::FileWatcher> m_watcher;
        qint64 m_pos;
      };
    }  // end namespace widgets
  }    // end namespace qt
}  // end namespace te
#endif  //__TERRALIB_QTWIDGETS_LOG_LOGTEXTEDIT_H__
