/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/widgets/query/QueryBuilderDialog.h

  \brief This file defines a class for a Query Builder Dialog.
*/

#ifndef __TERRALIB_QT_WIDGETS_QUERY_INTERNAL_QUERYBUILDERDIALOG_H
#define __TERRALIB_QT_WIDGETS_QUERY_INTERNAL_QUERYBUILDERDIALOG_H

// TerraLib
#ifndef Q_MOC_RUN
#include "../../../dataaccess/dataset/DataSetType.h"
#include "../../../dataaccess/datasource/DataSource.h"
#include "../../../dataaccess/query/Expression.h"
#include "../../../dataaccess/query/Where.h"
#include "../../../maptools/AbstractLayer.h"
#include "QueryBuilderWidget.h"
#include "../Config.h"
#endif

// STL
#include <memory>

// Qt
#include <QDialog>

namespace Ui { class QueryBuilderDialogForm; }

namespace te
{
  namespace qt
  {
    namespace widgets
    {
      /*!
        \class QueryBuilderDialog

        \brief This file defines a class for a Query Builder Dialog.
      */
      class TEQTWIDGETSEXPORT QueryBuilderDialog : public QDialog
      {
        Q_OBJECT

        public:

          QueryBuilderDialog(QWidget* parent = 0, Qt::WindowFlags f = 0);

          ~QueryBuilderDialog();

        public:

          /*!
          \brief This method is used to set the list of layers

          */
          void setLayerList(std::list<te::map::AbstractLayerPtr>& layerList);

          /*!
          \brief This method is used to set current layer

          */
          void setCurrentLayer(te::map::AbstractLayerPtr layer);

          void startWhereClause();

          void setWhereClause(te::da::Where* where);

          te::da::Where* getWhereClause();

          std::string getWhereString();

        protected slots:

          void onCreateLayerPushButtonClicked();

          void onInputLayerActivated(QString value);

          void onApplyPushButtonClicked();

          void onStrWhereClause(std::string str);

        signals:

          /*! This signal is emitted when the layer selection changed. */
          void layerSelectedObjectsChanged(const te::map::AbstractLayerPtr& layer);

          void createLayer(te::map::AbstractLayerPtr layer);

        private:

          std::unique_ptr<Ui::QueryBuilderDialogForm> m_ui;

          te::qt::widgets::QueryBuilderWidget* m_queryWidget;
      };

    } // end namespace widgets
  }   // end namespace qt
}     // end namespace te

#endif  // __TERRALIB_QT_WIDGETS_QUERY_INTERNAL_QUERYBUILDERDIALOG_H
