/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

 /*!
 \file terralib/qt/widgets/query/QueryBuilderTreeWidget.cpp

 \brief This file has the Query Builder Tree Widget class.
 */

// TerraLib
#include "QueryBuilderTreeWidget.h"

// Qt
#include <QVBoxLayout>

te::qt::widgets::QueryBuilderTreeWidgetItemKeeper::QueryBuilderTreeWidgetItemKeeper(QWidget* child)
{
  m_child = child;

  QVBoxLayout* layoutAux = new QVBoxLayout(this);
  layoutAux->setContentsMargins(0, 0, 0, 0);
  layoutAux->addWidget(m_child);
}

te::qt::widgets::QueryBuilderTreeWidgetItemKeeper::~QueryBuilderTreeWidgetItemKeeper()
{
  if (m_child && m_child->parent() == this)
  {
    m_child->hide();
    m_child->setParent(0);
  }
}


te::qt::widgets::QueryBuilderTreeWidget::QueryBuilderTreeWidget(QWidget* parent) :
  QTreeWidget(parent)
{
  connect(model(), SIGNAL(rowsInserted(QModelIndex, int, int)), this, SLOT(onRowsInserted(QModelIndex, int, int)));
}

void te::qt::widgets::QueryBuilderTreeWidget::setItemWidget(QTreeWidgetItem* item, int column, QWidget* widget) 
{
  QTreeWidget::setItemWidget(item, column, new QueryBuilderTreeWidgetItemKeeper(widget));

  item->setData(column, Qt::UserRole, m_allWidgets.count());

  m_allWidgets << widget;
}

void te::qt::widgets::QueryBuilderTreeWidget::onRowsInserted(QModelIndex parent, int start, int end)
{
  for (int column = 0; column < model()->columnCount(parent); column++) 
  {
    for (int row = start; row <= end; row++) 
    {
      QModelIndex index = model()->index(row, column, parent);

      QVariant data = model()->data(index, Qt::UserRole);

      if (data.type() == QVariant::Int) 
      {
        int i = data.toInt();

        QTreeWidgetItem* item = itemFromIndex(index);

        if (item && i >= 0 && i < m_allWidgets.count())
        {
          if (m_allWidgets[i])
          {
            setItemWidget(item, column, m_allWidgets[i]);

            m_allWidgets[i]->show();
          }
        }
      }
    }
  }
}
