/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/widgets/query/QueryBuilderTreeWidget.h

  \brief This file has the Query Builder Tree Widget class.
*/

#ifndef __TERRALIB_QT_WIDGETS_QUERY_INTERNAL_QUERYBUILDERDTREEWIDGET_H
#define __TERRALIB_QT_WIDGETS_QUERY_INTERNAL_QUERYBUILDERDTREEWIDGET_H

// TerraLib
#ifndef Q_MOC_RUN
#include "../Config.h"
#endif

// STL
#include <memory>

// Qt
#include <QTreeWidget>

namespace te
{
  namespace qt
  {
    namespace widgets
    {

      class TEQTWIDGETSEXPORT QueryBuilderTreeWidgetItemKeeper : public QWidget 
      {
        Q_OBJECT

        public:
          QueryBuilderTreeWidgetItemKeeper(QWidget* child);

          ~QueryBuilderTreeWidgetItemKeeper();

        private:

          QWidget * m_child;

      };

      /*!
        \class QueryBuilderTreeWidget

        \brief This class is used to define the Query Builder Tree Widget.
      */
      class TEQTWIDGETSEXPORT QueryBuilderTreeWidget : public QTreeWidget
      {
        Q_OBJECT

        public:

          QueryBuilderTreeWidget(QWidget* parent);

          //~QueryBuilderTreeWidget();

          void setItemWidget(QTreeWidgetItem* item, int column, QWidget* widget);

        private slots:

          void onRowsInserted(QModelIndex parent, int start, int end);

        private:

          QWidgetList m_allWidgets;
      }; 

    } // end namespace widgets
  }   // end namespace qt
}     // end namespace te

#endif  // __TERRALIB_QT_WIDGETS_QUERY_INTERNAL_QUERYBUILDERDTREEWIDGET_H