/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/widgets/query/QueryBuilderWidget.h

  \brief This file has the QueryBuilderWidget class.
*/

#ifndef __TERRALIB_QT_WIDGETS_QUERY_INTERNAL_QUERYBUILDERWIDGET_H
#define __TERRALIB_QT_WIDGETS_QUERY_INTERNAL_QUERYBUILDERWIDGET_H

// TerraLib
#ifndef Q_MOC_RUN
#include "../../../dataaccess/dataset/DataSetType.h"
#include "../../../dataaccess/datasource/DataSource.h"
#include "../../../dataaccess/query/Expression.h"
#include "../../../dataaccess/query/Where.h"
#include "../Config.h"
#endif

// STL
#include <memory>
#include <map>

// Qt
#include <QWidget>
#include <QTreeWidgetItem>


namespace Ui { class QueryBuilderWidgetForm; }

namespace te
{
  namespace qt
  {
    namespace widgets
    {
      class QueryBuilderTreeWidget;
      class QueryConnectorWidget;
      class QueryConditionWidget;

      /*!
        \class QueryBuilderWidget

        \brief This class is used to define the Where Clause object of select operation.
      */
      class TEQTWIDGETSEXPORT QueryBuilderWidget : public QWidget
      {
        Q_OBJECT

        public:

          QueryBuilderWidget(QWidget* parent = 0, Qt::WindowFlags f = 0);

          ~QueryBuilderWidget();

          void setInputParameters(te::da::DataSourcePtr dataSource, std::string dataSetName, std::unique_ptr<te::da::DataSetType> dsType);

          void startWhereClause();

          void setWhereClause(te::da::Where* where);

          te::da::Where* getWhereClause();

          std::string getWhereString();

        protected:

          QWidget * createCondition(bool startCondition, QWidget* parent, bool right);

          QWidget* createConnector(bool startCondition, QWidget* owner, QWidget* parent, bool autoCreateCondition);

          void removeCondition(te::qt::widgets::QueryConditionWidget* condition);

          void removeConnector(te::qt::widgets::QueryConnectorWidget* connector);

          void rebuildTree(QTreeWidgetItem* item);

          void buildTree(te::da::Expression* exp, QWidget* parent, bool right);

          bool functionsIsAConnector(te::da::Function* func);

          bool functionsIsACondition(te::da::Function* func);

          protected slots:

          void onAddConnectorClicked(QWidget* owner, QWidget* parent);

          void onRemoveConnectorClicked(QWidget* owner, QWidget* parent);

          void onChanged();

        signals:

          void strWhereClause(std::string str);


        protected:

          std::unique_ptr<Ui::QueryBuilderWidgetForm> m_ui;

          QueryBuilderTreeWidget* m_treeWidget;

          QWidget* m_starterWidget;

          te::da::DataSourcePtr m_dataSource;

          std::string m_dataSetName;

          std::unique_ptr<te::da::DataSetType> m_dsType;

          std::map<QTreeWidgetItem*, QWidget*> m_widgetMap;

      }; 

    } // end namespace widgets
  }   // end namespace qt
}     // end namespace te

#endif  // __TERRALIB_QT_WIDGETS_QUERY_INTERNAL_QUERYBUILDERWIDGET_H