/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/widgets/query/QueryConditionWidget.h

  \brief This file has the Query Condition Widget class.
*/

#ifndef __TERRALIB_QT_WIDGETS_QUERY_INTERNAL_QUERYCONDITIONWIDGET_H
#define __TERRALIB_QT_WIDGETS_QUERY_INTERNAL_QUERYCONDITIONWIDGET_H

// TerraLib
#ifndef Q_MOC_RUN
#include "../../../dataaccess/dataset/DataSetType.h"
#include "../../../dataaccess/datasource/DataSource.h"
#include "../../../dataaccess/query/Function.h"
#include "../Config.h"
#endif

// STL
#include <memory>

// Qt
#include <QTreeWidgetItem>
#include <QWidget>

namespace Ui { class QueryConditionWidgetForm; }

namespace te
{
  namespace qt
  {
    namespace widgets
    {
      class QueryConnectorWidget;
      /*!
        \class QueryConditionWidget

        \brief This class is used to define the Query Condition for a where clause.
      */
      class TEQTWIDGETSEXPORT QueryConditionWidget : public QWidget
      {
        Q_OBJECT

        public:

          QueryConditionWidget(te::da::DataSourcePtr dataSource, std::string dataSetName, te::da::DataSetType* dsType, QWidget* parent = 0, Qt::WindowFlags f = 0);

          ~QueryConditionWidget();

          void setCurrentCondition(te::da::Function* func);

          te::da::Expression* getExpression();

          void setConnectorParent(te::qt::widgets::QueryConnectorWidget* parent);

          te::qt::widgets::QueryConnectorWidget* getConnectorParent();

        protected:

          void setOperatorsList();

          void setAttributeList();

          void fillValuesComboBox(QStringList& list);

          std::string getAliasName(std::string fullName);

        signals:

          void addConnectorClicked(QWidget* owner, QWidget* parent);

          void removeConnectorClicked(QWidget* owner, QWidget* parent);

          void conditionChanged();

        protected slots:

          void onAddToolButtonClicked();

          void onRemoveToolButtonClicked();

          void onLoadAttrsToolButtonClicked();

        private:

          std::unique_ptr<Ui::QueryConditionWidgetForm> m_ui;

          te::qt::widgets::QueryConnectorWidget* m_parent;

          te::da::DataSourcePtr m_dataSource;

          std::string m_dataSetName;

          std::unique_ptr<te::da::DataSetType> m_dsType;

        public:

          QTreeWidgetItem * m_item;
      }; 

    } // end namespace widgets
  }   // end namespace qt
}     // end namespace te

#endif  // __TERRALIB_QT_WIDGETS_QUERY_INTERNAL_QUERYCONDITIONWIDGET_H