/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/widgets/query/QueryConnectorWidget.h

  \brief This file has the Query Condition Widget class.
*/

#ifndef __TERRALIB_QT_WIDGETS_QUERY_INTERNAL_QUERYCONNECTORWIDGET_H
#define __TERRALIB_QT_WIDGETS_QUERY_INTERNAL_QUERYCONNECTORWIDGET_H

// TerraLib
#ifndef Q_MOC_RUN
#include "../../../dataaccess/datasource/DataSource.h"
#include "../../../dataaccess/query/Function.h"
#include "../Config.h"
#endif

// STL
#include <memory>

// Qt
#include <QTreeWidgetItem>
#include <QWidget>

namespace Ui { class QueryConnectorWidgetForm; }

namespace te
{
  namespace qt
  {
    namespace widgets
    {
      class QueryConditionWidget;
      /*!
        \class QueryConnectorWidget

        \brief This class is used to define the Query Connector for a where clause.
      */
      class TEQTWIDGETSEXPORT QueryConnectorWidget : public QWidget
      {
        Q_OBJECT

        public:

          QueryConnectorWidget(te::da::DataSourcePtr dataSource, std::string dataSetName, QWidget* parent = 0, Qt::WindowFlags f = 0);

          ~QueryConnectorWidget();

          void setCurrentConnector(te::da::Function* func);

          te::da::Expression* getExpression();

          void setConnectorParent(te::qt::widgets::QueryConnectorWidget* connectorParent);

          te::qt::widgets::QueryConnectorWidget* getConnectorParent();

          void setConnectorLeft(te::qt::widgets::QueryConnectorWidget* connectorChild);

          te::qt::widgets::QueryConnectorWidget* getConnectorLeft();

          void setConnectorRight(te::qt::widgets::QueryConnectorWidget* connectorChild);

          te::qt::widgets::QueryConnectorWidget* getConnectorRight();

          void setConditionLeft(te::qt::widgets::QueryConditionWidget* condition);

          te::qt::widgets::QueryConditionWidget* getConditionLeft();

          void setConditionRight(te::qt::widgets::QueryConditionWidget* condition);

          te::qt::widgets::QueryConditionWidget* getConditionRight();

        protected:

          void setConnectorsList();

        signals:

          void connectorChanged();

        private:

          std::unique_ptr<Ui::QueryConnectorWidgetForm> m_ui;

          te::qt::widgets::QueryConnectorWidget* m_connectorParent;
          te::qt::widgets::QueryConnectorWidget* m_connectorLeft;
          te::qt::widgets::QueryConnectorWidget* m_connectorRight;
          te::qt::widgets::QueryConditionWidget* m_conditionLeft;
          te::qt::widgets::QueryConditionWidget* m_conditionRight;

          te::da::DataSourcePtr m_dataSource;

          std::string m_dataSetName;

        public:
          
          QTreeWidgetItem * m_item;
      }; 

    } // end namespace widgets
  }   // end namespace qt
}     // end namespace te

#endif  // __TERRALIB_QT_WIDGETS_QUERY_INTERNAL_QUERYCONNECTORWIDGET_H