/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/widgets/query/QuickSelectDialog.h

  \brief This file defines a class for a Select Dialog.
*/

#ifndef __TERRALIB_QT_WIDGETS_QUERY_INTERNAL_QUICKSELECT_H
#define __TERRALIB_QT_WIDGETS_QUERY_INTERNAL_QUICKSELECT_H

// TerraLib
#ifndef Q_MOC_RUN
#include "../../../dataaccess/datasource/DataSource.h"
#include "../../../dataaccess/query/Select.h"
#include "../../../dataaccess/query/Where.h"
#include "QueryBuilderWidget.h"
#endif

#include "../Config.h"


// STL
#include <memory>
#include <string>
#include <set>

// Qt
#include <QColor>
#include <QDialog>
#include <QGridLayout>

namespace Ui { class QuickSelectDialogForm; }

namespace te
{
// Forward declaration

  namespace qt
  {
    namespace widgets
    {
      /*!
        \class QuickSelectDialog

        \brief This file defines a class for a Select Dialog.
      */
      class TEQTWIDGETSEXPORT QuickSelectDialog : public QDialog
      {
        Q_OBJECT

      public:

        QuickSelectDialog(QWidget* parent = 0, Qt::WindowFlags f = 0);

        ~QuickSelectDialog();

      public:

        void setInputDataSource(te::da::DataSourcePtr dataSource);

        std::set<std::string> getFunctionsNames();

        te::da::Select getSelect();

        std::string getSelectString();

        protected slots:

        void onDataSetComboBoxSelected(int index);

        void onAddAttrToolButtonClicked();

        void onRemoveAttrToolButtonClicked();

        void onCheckToolButtonClicked();

        void onOkPushButtonClicked();

      protected:

        std::string setAliasName(std::string value);

      private:

        std::unique_ptr<Ui::QuickSelectDialogForm> m_ui;

        QGridLayout* m_queryLayout;

        te::qt::widgets::QueryBuilderWidget* m_queryWidget;

        te::da::DataSourcePtr m_dataSource;
      };
    } // end namespace widgets
  }   // end namespace qt
}     // end namespace te

#endif  // __TERRALIB_QT_WIDGETS_QUERY_INTERNAL_QUICKSELECT_H
