/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/widgets/query/SpatialWhereClauseWidget.h

  \brief This file has the DataSetWidget class.
*/

#ifndef __TERRALIB_QT_WIDGETS_QUERY_INTERNAL_SPATIALWHERECLAUSEWIDGET_H
#define __TERRALIB_QT_WIDGETS_QUERY_INTERNAL_SPATIALWHERECLAUSEWIDGET_H

// TerraLib
#include "../Config.h"
#ifndef Q_MOC_RUN
#include "../../../dataaccess/datasource/DataSource.h"
#include "../../../dataaccess/query/Expression.h"
#include "../../../dataaccess/query/Where.h"
#include "../../../maptools/AbstractLayer.h"
#include "WhereClauseWidget.h"
#endif

// STL
#include <memory>
#include <map>
#include <set>

// Qt
#include <QComboBox>
#include <QWidget>


namespace Ui { class SpatialWhereClauseWidgetForm; }

namespace te
{
  namespace qt
  {
    namespace widgets
    {
      /*!
        \class WhereClauseWidget

        \brief This class is used to define the WHERE object of select operation.
      */
      class TEQTWIDGETSEXPORT SpatialWhereClauseWidget : public QWidget
      {
        Q_OBJECT

        public:

          SpatialWhereClauseWidget(QWidget* parent = 0, Qt::WindowFlags f = 0);

          ~SpatialWhereClauseWidget();

          te::da::Where* getWhere();

          std::string getWhereString();

        public:

          void addLayer(te::map::AbstractLayerPtr layer);

          void removeLayer(te::map::AbstractLayerPtr layer);

          void setDataSource(const te::da::DataSourcePtr& ds);

          void setLayerList(std::list<te::map::AbstractLayerPtr>& layerList);

          void setFromItems(std::vector<std::pair<std::string, std::string> > vec);

          void setGeomAttributeList(const std::vector<std::string>& vec, int srid);

          void setSpatialOperatorsList(const std::vector<std::string>& vec);

          void setConnectorsList(const std::vector<std::string>& vec);

          void clear();

          void resetInterface();

        public slots:

          void onAddWhereClausePushButtonClicked();

          void onRemoveWhereClausePushButtonClicked();

          void onClearAllPushButtonClicked();

          void onComboBoxActivated(QString value);

        signals:

          void clausesChanged();

        protected:

          te::da::Expression* getExpression(const QString& value, const std::string& propName);

          void copyCombo(QComboBox* input, QComboBox* output, std::string curValue = "");

          QStringList getPropertyValues(std::string propertyName);


        private:

          std::unique_ptr<Ui::SpatialWhereClauseWidgetForm> m_ui;

          te::da::DataSourcePtr m_ds;

          std::vector<std::pair<std::string, std::string> > m_fromItems;

          std::map<int, ExpressionProperty*> m_mapExp;

          std::map< QComboBox*, std::pair<int, int> > m_comboMap;

          QStringList m_connectorsList;

          int m_count;
          int m_srid;
      }; 

    } // end namespace widgets
  }   // end namespace qt
}     // end namespace te

#endif  // __TERRALIB_QT_WIDGETS_QUERY_INTERNAL_SPATIALWHERECLAUSEWIDGET_H
