/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/widgets/query/WhereClauseDialog.cpp

  \brief This file defines a class for a Where Clause Dialog.
*/

// TerraLib
#include "WhereClauseDialog.h"
#include "WhereClauseWidget.h"
#include "ui/ui_WhereClauseDialogForm.h"

// Qt
#include <QMessageBox>

// STL
#include <memory>

te::qt::widgets::WhereClauseDialog::WhereClauseDialog(QWidget* parent, Qt::WindowFlags f)
  : QDialog(parent, f),
    m_ui(new Ui::WhereClauseDialogForm)
{
  m_ui->setupUi(this);

  // Build form
  QGridLayout* layout = new QGridLayout(m_ui->m_widget);
  m_whereClauseWidget.reset( new te::qt::widgets::WhereClauseWidget(m_ui->m_widget));
  layout->addWidget(m_whereClauseWidget.get());
  layout->setContentsMargins(0, 0, 0, 0);

  //
  m_ui->m_helpPushButton->setPageReference("widgets/query/whereclause_dialog.html");
}

te::qt::widgets::WhereClauseDialog::~WhereClauseDialog() = default;

te::da::Where* te::qt::widgets::WhereClauseDialog::getWhere()
{
  return m_whereClauseWidget->getWhere();
}

std::string te::qt::widgets::WhereClauseDialog::getWhereString()
{
  return m_whereClauseWidget->getWhereString();
}

te::qt::widgets::WhereClauseWidget* te::qt::widgets::WhereClauseDialog::getWidget()
{
  return m_whereClauseWidget.get();
}

void te::qt::widgets::WhereClauseDialog::removeSpatialRestriction()
{
  m_whereClauseWidget->removeSpatialRestriction();
}
