/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/widgets/query/WhereClauseDialog.h

  \brief This file defines a class for a Where Clause Dialog.
*/

#ifndef __TERRALIB_QT_WIDGETS_QUERY_INTERNAL_WHERECLAUSEDIALOG_H
#define __TERRALIB_QT_WIDGETS_QUERY_INTERNAL_WHERECLAUSEDIALOG_H

// TerraLib
#ifndef Q_MOC_RUN
#include "../../../dataaccess/query/Where.h"
#include "../Config.h"
#endif

// STL
#include <memory>

// Qt
#include <QDialog>

namespace Ui { class WhereClauseDialogForm; }

namespace te
{
  namespace qt
  {
    namespace widgets
    {
      class WhereClauseWidget;

      /*!
        \class WhereClauseDialog

        \brief This file defines a class for a Where Clause Dialog.
      */
      class TEQTWIDGETSEXPORT WhereClauseDialog : public QDialog
      {
        Q_OBJECT

        public:

          WhereClauseDialog(QWidget* parent = 0, Qt::WindowFlags f = 0);

          ~WhereClauseDialog();

        public:

          te::da::Where* getWhere();

          std::string getWhereString();

          te::qt::widgets::WhereClauseWidget* getWidget();

          void removeSpatialRestriction();

        private:

          std::unique_ptr<Ui::WhereClauseDialogForm> m_ui;
          std::unique_ptr<te::qt::widgets::WhereClauseWidget> m_whereClauseWidget;
      };

    } // end namespace widgets
  }   // end namespace qt
}     // end namespace te

#endif  // __TERRALIB_QT_WIDGETS_QUERY_INTERNAL_WHERECLAUSEDIALOG_H
