/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/widgets/rp/CloudDetectionDialog.h

  \brief ....
*/

#ifndef __TERRALIB_QT_WIDGETS_RP_INTERNAL_CLOUDDETECTIONDIALOG_H
#define __TERRALIB_QT_WIDGETS_RP_INTERNAL_CLOUDDETECTIONDIALOG_H

// TerraLib
#include "../Config.h"
#ifndef Q_MOC_RUN
#include "../../../dataaccess/datasource/DataSource.h"
#include "../../../dataaccess/datasource/DataSourceInfo.h"
#include "../../../maptools/AbstractLayer.h"
#include "../../../qt/widgets/rp/RpToolsWidget.h"
#include "../se/PolygonSymbolizerWidget.h"
#endif

// STL
#include <memory>

// Qt
#include <QDialog>

namespace Ui { class CloudDetectionDialogForm; }

namespace te
{
  //forward declarations
  namespace da  { class DataSetType; }
  namespace gm { class Envelope; }
  namespace mem { class DataSet; }

  namespace qt
  {
    namespace widgets
    {
      class Canvas;
      class MapDisplay;
      class RasterHistogramWidget;

      /*!
        \class CloudDetectionDialog

        \brief ....
      */
      class TEQTWIDGETSEXPORT CloudDetectionDialog : public QDialog
      {
        Q_OBJECT

        public:

          CloudDetectionDialog(QWidget* parent = 0, Qt::WindowFlags f = 0);
            
          ~CloudDetectionDialog();

          void setLayers(std::list<map::AbstractLayerPtr> &layers);

          void setMapDisplay(te::qt::widgets::MapDisplay* mapDisplay);

          void setActionGroup(QActionGroup* actionGroup);

          void setUserPath(std::string path);
          
        protected slots:
          
          void onCloudLayerComboBoxActivated(int index);

          void onShadowLayerComboBoxActivated(int index);

          void onSearchCloudLayerToolButtonClicked();

          void onSearchShadowLayerToolButtonClicked();

          void onUpToolButtonClicked();

          void onDownToolButtonClicked();

          void onFilterCellClicked(int row, int column);

          void onFilterCellDoubleClicked(int row, int column);

          void onFilterCellChanged(int row, int column);

          void onTargetDatasourceToolButtonClicked();
            
          void onTargetFileToolButtonClicked();

          void onEnvelopeAcquired(te::gm::Envelope env);

          void onMapDisplayExtentChanged();

          void onOkPushButtonClicked();

          void onStyleToolButtonClicked();

          void onDrawCloudHistogramClicked();

          void onResetCloudHistogramClicked();

          void onDrawShadowHistogramClicked();

          void onResetShadowHistogramClicked();

          void onCloudMinValueSelected(int, int);

          void onCloudMaxValueSelected(int, int);

          void onCloudMinValueSelected(double, int);

          void onCloudMaxValueSelected(double, int);

          void onShadowMinValueSelected(int, int);

          void onShadowMaxValueSelected(int, int);

          void onShadowMinValueSelected(double, int);

          void onShadowMaxValueSelected(double, int);

        signals:

          void addLayer(te::map::AbstractLayerPtr layer);

          void closeTool();

        protected:

          bool execute();

          bool vectorize(te::rst::Raster* raster, std::vector<te::gm::Geometry*>& geomVec, std::vector< double >& geomsValues);

          void setCloudBands();

          void setShadowBands();

          void drawCloudHistogram();

          void drawShadowHistogram();

          void applyPreview();

          void drawGeom();

          void getRasterFromROI();

          void writeStyle();

          void readStyle();

          /*! Function used to create the output dataset type */
          std::unique_ptr<te::da::DataSetType> createDataSetType(std::string dataSetName, int srid);

          /*! Function used to create the output data */
          std::unique_ptr<te::mem::DataSet> createDataSet(te::da::DataSetType* dsType, std::vector<te::gm::Geometry*>& geoms, std::vector< double >& geomsValues);

          /*! Function used to save the output dataset */
          void saveDataSet(te::mem::DataSet* dataSet, te::da::DataSetType* dsType, te::da::DataSourcePtr ds, std::string dataSetName);

          void clearCanvas();

          void closeEvent(QCloseEvent* e);

		  void rasterFilter(std::list<map::AbstractLayerPtr>& layers);

        private:

          std::unique_ptr<Ui::CloudDetectionDialogForm> m_ui;

          std::unique_ptr<te::qt::widgets::RasterHistogramWidget> m_cloudHistogramWidget;
          std::unique_ptr<te::qt::widgets::RasterHistogramWidget> m_shadowHistogramWidget;
          std::unique_ptr<te::qt::widgets::RpToolsWidget> m_navigator;

          std::list<te::map::AbstractLayerPtr> m_layers;
          std::list<te::map::AbstractLayerPtr> m_layersRasterFiltered;

          te::map::AbstractLayerPtr m_cloudLayer;
          te::map::AbstractLayerPtr m_shadowLayer;
          te::map::AbstractLayerPtr m_outputLayer;

          std::unique_ptr<te::rst::Raster> m_cloudRasterPreview;
          std::unique_ptr<te::rst::Raster> m_shadowRasterPreview;

          std::unique_ptr<te::gm::Geometry> m_geom;
          te::qt::widgets::MapDisplay* m_mapDisplay;

          std::string m_lastFilterTableCellText;

          std::vector<double> m_cloudMinValue;
          std::vector<double> m_cloudMaxValue;
          std::vector<double> m_shadowMinValue;
          std::vector<double> m_shadowMaxValue;

          std::string m_userPath;

          std::unique_ptr<te::qt::widgets::PolygonSymbolizerWidget> m_polygonSymbolizerWidget;
          te::se::Symbolizer* m_symbolizer;
      }; 
    }     // end namespace widgets
  }       // end namespace qt
}         // end namespace te

#endif  // __TERRALIB_QT_PLUGINS_RP_INTERNAL_CLOUDDETECTIONDIALOG_H

