/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/widgets/rp/OutputRasterWidget.cpp

  \brief This file has the OutputRasterWidget class.
*/

// TerraLib
#include "OutputRasterWidget.h"
#include "../../../core/filesystem/FileSystem.h"
#include "../../../dataaccess/datasource/DataSourceInfo.h"
#include "../datasource/selector/DataSourceSelectorDialog.h"
#include "../utils/FileDialog.h"
#include "../Utils.h"
#include "ui/ui_OutputRasterWidgetForm.h"

// BOOST Includes
#include <boost/filesystem.hpp>

// QT
#include <QFileDialog>
#include <QMessageBox>


te::qt::widgets::OutputRasterWidget::OutputRasterWidget(QWidget* parent, Qt::WindowFlags f)
  : QWidget(parent, f),
  m_ui(new Ui::OutputRasterWidgetForm)
{
  m_ui->setupUi(this);

  connect(m_ui->m_targetFileToolButton, SIGNAL(clicked()), this, SLOT(onTargetFileToolButtonClicked()));
}

te::qt::widgets::OutputRasterWidget::~OutputRasterWidget() = default;

Ui::OutputRasterWidgetForm* te::qt::widgets::OutputRasterWidget::getForm() const
{
  return m_ui.get();
}

std::string te::qt::widgets::OutputRasterWidget::getOutputRaster()
{
  std::string outputRaster;
  if (!m_ui->m_repositoryLineEdit->text().isEmpty())
    outputRaster = m_ui->m_repositoryLineEdit->text().toUtf8().data();

  return outputRaster;
}

bool te::qt::widgets::OutputRasterWidget::fileExists() const
{
  boost::filesystem::path path(m_ui->m_repositoryLineEdit->text().toUtf8().data());
  return te::core::FileSystem::exists(path.string());
}

std::string te::qt::widgets::OutputRasterWidget::getType() const
{
  return "GDAL";
}

std::map<std::string, std::string> te::qt::widgets::OutputRasterWidget::getInfo() const
{
  std::map<std::string, std::string> rinfo;

  rinfo["URI"] = m_ui->m_repositoryLineEdit->text().toUtf8().data();

  return rinfo;
}

std::string te::qt::widgets::OutputRasterWidget::getFilePath()
{
  if (m_ui->m_repositoryLineEdit->text().isEmpty())
    return "";

  QFileInfo info(m_ui->m_repositoryLineEdit->text());

  std::string filePath = info.path().toUtf8().data();
  std::string baseName = info.baseName().toUtf8().data();
  std::string fileName = filePath + "/" + baseName;

  return fileName;
}

std::string te::qt::widgets::OutputRasterWidget::getFileExtension()
{
  if (m_ui->m_repositoryLineEdit->text().isEmpty())
    return "";

  QFileInfo info(m_ui->m_repositoryLineEdit->text());

  return info.suffix().toUtf8().data();
}

void te::qt::widgets::OutputRasterWidget::onTargetFileToolButtonClicked()
{
  m_ui->m_newLayerNameLineEdit->clear();
  m_ui->m_repositoryLineEdit->clear();

  te::qt::widgets::FileDialog fileDialog(this, te::qt::widgets::FileDialog::RASTER, 
    "rp_raster_info", te::qt::widgets::FileDialog::SAVE_FILE_MODE);

  try {
    fileDialog.exec();
  }
  catch (te::common::Exception& ex) {
    QMessageBox::warning(this, tr("File information"), ex.what());
    return;
  }

  m_ui->m_newLayerNameLineEdit->setText(fileDialog.getFileName().c_str());
  m_ui->m_repositoryLineEdit->setText(fileDialog.getPath().c_str());

  m_ui->m_newLayerNameLineEdit->setEnabled(false);
}
