/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.
  
    This file is part of the TerraLib - a Framework for building GIS enabled applications.
    
    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.
    
    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.
    
    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */


#ifndef TE_QT_WIDGETS_PCAWIZARD_H
#define TE_QT_WIDGETS_PCAWIZARD_H

// TerraLib
#ifndef Q_MOC_RUN
#include "../../../maptools/AbstractLayer.h"
#endif
#include "../Config.h"
//Qt
#include <QWizard>

//STL
#include <memory>

namespace te 
{
  namespace qt 
  {
    namespace widgets 
    {
      //Forward declarations
      class PCAWizardPage;
      class LayerSearchWizardPage;
      class RasterInfoWizardPage;

      class TEQTWIDGETSEXPORT PCAWizard : public QWizard
      {
        Q_OBJECT

        public:

          /*! \brief Constructor. */
          PCAWizard(QWidget* parent=0);

          /*! \brief Destructor. */
          ~PCAWizard();

          /*!
          \brief Overridden function that checks if the current page's configuration is done in order to proceed to the next page.

          \return A boolean that indicates if the current page's configuration is finished.
          */
          bool validateCurrentPage();

          /*!
          \brief This function sets the list of layers from which the raster that will be sliced will come from.

          \param layerList The list of AbstractLayerPtrs that will be used to get a raster from.
          */
          void setList(std::list<te::map::AbstractLayerPtr>& layerList);

          /*!
          \brief This function returns the layer that has been created by the wizard with the sliced raster.

          \retunrn A new AbstractLayerPtr containing the sliced raster.
          */
          te::map::AbstractLayerPtr getOutputLayer();

          /*!
          \brief This function sets the layer that contains the raster that will be sliced.

          \param layer The layer from which the raster will be obtained.
          */
          void setLayer(te::map::AbstractLayerPtr layer);

        signals:

          void addLayer(te::map::AbstractLayerPtr layer);

        protected:

          void addPages();

          bool execute();

        protected slots:

          void onPageChanged(int index);

        private:

          //No copy allowed
          PCAWizard(const PCAWizard& rhs);
          PCAWizard& operator=(const PCAWizard& rhs);


          std::unique_ptr<te::qt::widgets::PCAWizardPage>  m_wizardPage;       //!< The wizard page used to define the PCA parameters
          std::unique_ptr<te::qt::widgets::LayerSearchWizardPage>    m_layerSearchPage;  //!< The wizard page used to select an input layer
          std::unique_ptr<te::qt::widgets::RasterInfoWizardPage>     m_rasterInfoPage;   //!< The wizard page used to define the output layer parameters


          te::map::AbstractLayerPtr m_outputLayer;
      };      
    } // namespace widgets
  } // namespace qt
} // namespace te

#endif // TE_QT_WIDGETS_PCAWIZARD_H
