/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

This file is part of the TerraLib - a Framework for building GIS enabled applications.

TerraLib is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License,
or (at your option) any later version.

TerraLib is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with TerraLib. See COPYING. If not, write to
TerraLib Team at <terralib-team@terralib.org>.
*/


#ifndef TE_QT_WIDGETS_PCAWIZARDPAGE_H
#define TE_QT_WIDGETS_PCAWIZARDPAGE_H

//Terralib
#ifndef Q_MOC_RUN
#include "../../../dataaccess/datasource/DataSourceInfo.h"
#include "../../../maptools/AbstractLayer.h"
#endif
#include "../Config.h"

//Qt
#include <QWizardPage>

//STL
#include <memory>

//Forward declarations
namespace Ui
{
  class PCAWizardPageForm;
}

namespace te
{
  namespace se
  {
    class ColorMap;
  }

  namespace qt
  {
    namespace widgets
    {
      class TEQTWIDGETSEXPORT PCAWizardPage : public QWizardPage
      {
        Q_OBJECT

      public:

        /*! \brief Constructor. */
        PCAWizardPage(QWidget* parent = 0);

        /*! \brief Destructor. */
        ~PCAWizardPage();

        /*!
        \brief This function returns the bands from the raster taht will be used.

        \return A vector of band indexes.
        */
        std::vector< unsigned int > getSelectedBands() const;

        /*!
        \brief This function returns the bands from the raster taht will be used.

        \return A vector of band indexes.
        */
        std::string getMatrixFileName() const;

        /*!
        \brief This function returns the output raster data type.

        \return The output raster data type.
        */
        int getOutputRasterDataType() const;

        /*!
        \brief This function sets the layer that contains the raster that will be sliced.

        \param layer The layer from which the raster will be obtained.
        */
        void setLayer(te::map::AbstractLayerPtr layer);

        /*!
        In order for this function to return true:

        The color map taht will be used on the output layer can not be empty.

        \param A boolean that indicates if the page is complete.
        */
        bool isComplete() const;

        /*!
        \brief Returnes true the inverse PCA option is selected.

        \return Returnes true the inverse PCA option is selected.

        */
        bool inversePCASelected() const;

      protected:

        std::unique_ptr<Ui::PCAWizardPageForm>  m_ui;              //!< The wizard page form
        std::unique_ptr<te::rst::Raster>                  m_raster;          //!< The raster that will be processed
        te::map::AbstractLayerPtr                       m_rasterLayer;        //!< Testing a better way to show the chart

        protected slots:

         void inversePCACheckBoxToggled(bool checked);
         void loadPCAMatrixFilePushButtonClicked( bool checked );

      private:

        //No copy allowed
        PCAWizardPage(const PCAWizardPage& rhs);
        PCAWizardPage& operator=(const PCAWizardPage& rhs);
      };
    } // namespace widgets
  } // namespace qt
} // namespace te

#endif // TE_QT_WIDGETS_PCAWIZARDPAGE_H

