/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/widgets/rp/PaletteDialog.h

  \brief This interface is used to create palette raster from rgb raster.
*/

#ifndef __TERRALIB_QT_WIDGETS_INTERNAL_PALETTEDIALOG_H
#define __TERRALIB_QT_WIDGETS_INTERNAL_PALETTEDIALOG_H

//TerraLib
#ifndef Q_MOC_RUN
#include "../../../maptools/AbstractLayer.h"
#endif
#include "../Config.h"
#include "OutputRasterWidget.h"

// Qt
#include <QDialog>

//STL
#include <memory>

namespace Ui { class PaletteDialogForm; }

namespace te
{

  namespace qt
  {
    namespace widgets
    {

    /*!
        \class PaletteDialog

        \brief This interface is used to create palette raster from rgb raster.
      */
      class TEQTWIDGETSEXPORT PaletteDialog : public QDialog
      {

        Q_OBJECT

        public:

          /*!
            \brief Constructor
          */
          PaletteDialog(QWidget* parent = 0, Qt::WindowFlags f = 0);

          /*!
            \brief Destructor
          */
          ~PaletteDialog();

      public:

        te::map::AbstractLayerPtr getLayer();

        /*! \brief Set a layer. */
        void setParameters(te::map::AbstractLayerPtr layer);

        protected slots:

        void onOkPushButtonClicked();

        void onCancelPushButtonClicked();

        private:

          std::unique_ptr<Ui::PaletteDialogForm> m_ui;
          te::qt::widgets::OutputRasterWidget* m_outputWidget;
          te::map::AbstractLayerPtr m_layer;                 //!< TerraLib layer auto ptr

          te::map::AbstractLayerPtr m_layerResult;
      };
    } // end namespace widgets
  }   // end namespace qt
}     // end namespace te

#endif  // __TERRALIB_QT_WIDGETS_INTERNAL_PALETTEDIALOG_H