/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/widgets/rp/PostClassificationDialogForm.h

  \brief This file has the PostClassificationDialogForm class.
*/

// TerraLib
#include "../../../common/progress/ProgressManager.h"
#include "../../../common/StringUtils.h"
#include "../../../dataaccess/dataset/DataSet.h"
#include "../../../dataaccess/dataset/DataSetType.h"
#include "../../../dataaccess/utils/Utils.h"
#include "../../../geometry/Utils.h"
#include "../../../maptools/Utils.h"
#include "../../../raster/RasterProperty.h"
#include "../../../se/CoverageStyle.h"
#include "../../../se/Utils.h"
#include "../progress/ProgressViewerDialog.h"
#include "PostClassificationDialogForm.h"
#include "Utils.h"
#include "ui/ui_PostClassificationDialogForm.h"

//QT
#include <QApplication>
#include <QGridLayout>
#include <QMessageBox>
#include <QCheckBox>

te::qt::widgets::PostClassificationDialogForm::PostClassificationDialogForm(QWidget* parent)
  : QDialog(parent),
  m_ui(new Ui::PostClassificationDialogForm)
{
//build form
  m_ui->setupUi(this);

  //build input layer
  QGridLayout* postClassLayout = new QGridLayout(m_ui->m_postClassWidget);
  postClassLayout->setContentsMargins(0, 0, 0, 0);
  m_postClassWidget = new te::qt::widgets::PostClassificationWidget(m_ui->m_postClassWidget);
  postClassLayout->addWidget(m_postClassWidget);

  m_ui->m_helpPushButton->setNameSpace("dpi.inpe.br.plugins");
  m_ui->m_helpPushButton->setPageReference("plugins/rp/rp_postClassification.html");

  //connects
  connect(m_ui->m_okPushButton, SIGNAL(clicked()), SLOT(onOkPushButtonClicked()));
  connect(m_postClassWidget, SIGNAL(addLayer(te::map::AbstractLayerPtr)), this, SLOT(addOuputLayer(te::map::AbstractLayerPtr)));
}

te::qt::widgets::PostClassificationDialogForm::~PostClassificationDialogForm() =
    default;

void te::qt::widgets::PostClassificationDialogForm::setList(std::list<te::map::AbstractLayerPtr>& layerList)
{
  m_postClassWidget->setList(layerList);
}

void te::qt::widgets::PostClassificationDialogForm::onOkPushButtonClicked()
{
  if (!m_postClassWidget->execute())
    return;

  accept();
}

void te::qt::widgets::PostClassificationDialogForm::addOuputLayer(te::map::AbstractLayerPtr layer)
{
  emit addLayer(layer);
}

void te::qt::widgets::PostClassificationDialogForm::closeEvent(
    QCloseEvent* /*e*/)
{
  delete m_postClassWidget;

  emit closeTool();
}
