/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/widgets/rp/PostClassificationDialogForm.h

  \brief This file has the PostClassificationDialogForm class.
*/

#ifndef __TERRALIB_QT_WIDGETS_RP_INTERNAL_POSTCLASSIFICATIONDIALOGFORM_H
#define __TERRALIB_QT_WIDGETS_RP_INTERNAL_POSTCLASSIFICATIONDIALOGFORM_H

// TerraLib
#ifndef Q_MOC_RUN
#include "../../../maptools/AbstractLayer.h"
#endif
#include "../Config.h"
#include "PostClassificationWidget.h"
#include "InputLayerWidget.h"
#include "OutputRasterWidget.h"

// Qt
#include <QDialog>

namespace Ui { class PostClassificationDialogForm; }

namespace te
{
  namespace qt
  {
    namespace af
    {
      namespace evt
      {
        struct Event;
      }
    }

    namespace widgets
    {

      class TEQTWIDGETSEXPORT PostClassificationDialogForm : public QDialog
      {
        Q_OBJECT

        public:

          PostClassificationDialogForm(QWidget* parent = 0);

          ~PostClassificationDialogForm();

        public:

          void setList(std::list<te::map::AbstractLayerPtr>& layerList);

        public slots:

          void onOkPushButtonClicked();

          void addOuputLayer(te::map::AbstractLayerPtr outputLayer);

        signals:

          void addLayer(te::map::AbstractLayerPtr layer);

          void closeTool();

        protected:

          void closeEvent(QCloseEvent* e);

        private:

          std::unique_ptr<Ui::PostClassificationDialogForm> m_ui;

          te::map::AbstractLayerPtr m_layer;
          te::qt::widgets::PostClassificationWidget* m_postClassWidget;
      };
    } // end namespace widgets
  }   // end namespace qt
}     // end namespace te

#endif  // __TERRALIB_QT_WIDGETS_RP_INTERNAL_POSTCLASSIFICATIONDIALOGFORM_H
