/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/widgets/rp/PostClassificationWidget.h

  \brief This file has the PostClassificationWidget class.
*/

#ifndef __TERRALIB_QT_WIDGETS_CANVAS_INTERNAL_POSTCLASSIFICATIONWIDGET_H
#define __TERRALIB_QT_WIDGETS_CANVAS_INTERNAL_POSTCLASSIFICATIONWIDGET_H

// TerraLib
#ifndef Q_MOC_RUN
#include "../../../maptools/AbstractLayer.h"
#endif
#include "../Config.h"
#include "InputLayerWidget.h"
#include "OutputRasterWidget.h"

// STL
#include <memory>

// Qt
#include <QWidget>

namespace Ui { class PostClassificationWidgetForm; }

namespace te
{
  namespace rst { class Raster; }

  namespace qt
  {
    namespace widgets
    {
      /*!
        \class PostClassificationWidget

        \brief This class is used to make post classification.
      */
      class TEQTWIDGETSEXPORT PostClassificationWidget : public QWidget
      {
        Q_OBJECT

        public:

          PostClassificationWidget(QWidget* parent = 0, Qt::WindowFlags f = 0);

          ~PostClassificationWidget();

          Ui::PostClassificationWidgetForm* getForm() const;

          void setList(std::list<te::map::AbstractLayerPtr>& layerList);

          bool execute();

        protected slots:

          void weightSliderValueChanged(int value);

          void thresholdSliderValueChanged(int value);

        signals:

          void addLayer(te::map::AbstractLayerPtr layer);

        private:

          std::unique_ptr<Ui::PostClassificationWidgetForm> m_ui;
          te::map::AbstractLayerPtr m_layer;
          te::map::AbstractLayerPtr m_outputLayer;
          te::qt::widgets::InputLayerWidget* m_inputWidget;
          te::qt::widgets::OutputRasterWidget* m_outputWidget;
      }; 

    } // end namespace widgets
  }   // end namespace qt
}     // end namespace te

#endif  // __TERRALIB_QT_WIDGETS_CANVAS_INTERNAL_POSTCLASSIFICATIONWIDGET_H

