/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/widgets/rp/RasterEditWizard.h

  \brief A Qt dialog that allows raster data edition.
*/

#ifndef __TERRALIB_QT_WIDGETS_RP_INTERNAL_RASTEREDIT_H
#define __TERRALIB_QT_WIDGETS_RP_INTERNAL_RASTEREDIT_H

// TerraLib
#ifndef Q_MOC_RUN
#include "../../../maptools/AbstractLayer.h"
#include "../../../qt/widgets/canvas/MapDisplay.h"
#include "../../../qt/widgets/layer/explorer/LayerItemView.h"
#endif
#include "../Config.h"

// STL
#include <memory>

// Qt
#include <QWizard>
#include <QActionGroup>

namespace te
{
  namespace qt
  {
    namespace widgets
    {
      class RasterEditWizardPage;
      class LayerSearchWizardPage;
      class RasterInfoWizardPage;

      /*!
        \class RasterEditWizard

        \brief This class is GUI used to define the raster editi operation.
      */
      class TEQTWIDGETSEXPORT RasterEditWizard : public QWizard
      {
        Q_OBJECT

        public:

          RasterEditWizard(QWidget* parent, te::qt::widgets::MapDisplay* mainMapDispPtr,
            te::qt::widgets::LayerItemView* mainLayerExplorerPtr );

          ~RasterEditWizard();

        public:

          virtual bool validateCurrentPage();

          void setList(std::list<te::map::AbstractLayerPtr>& layerList);

          void setLayer(te::map::AbstractLayerPtr layer);

          void setActionGroup(QActionGroup* actionGroup);

          te::map::AbstractLayerPtr getOutputLayer();

          void onMapDisplayExtentChanged();

        protected:

          void addPages();

          bool execute();

        protected slots:

          void onPageChanged(int index);

        signals:

          void addLayer(te::map::AbstractLayerPtr layer);

      private:

          te::qt::widgets::MapDisplay* m_mainMapDispPtr;
          te::qt::widgets::LayerItemView* m_mainLayerExplorerPtr;
          std::unique_ptr<te::qt::widgets::RasterEditWizardPage> m_rasterEditPage;
          std::unique_ptr<te::qt::widgets::LayerSearchWizardPage> m_layerSearchPage;
          std::unique_ptr<te::qt::widgets::RasterInfoWizardPage> m_rasterInfoPage;
          te::map::AbstractLayerPtr m_outputLayer;
      };
    }   // end namespace widgets
  }     // end namespace qt
}       // end namespace te

#endif  // __TERRALIB_QT_WIDGETS_RP_INTERNAL_RASTEREDIT_H
