/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.
  
    This file is part of the TerraLib - a Framework for building GIS enabled applications.
    
    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.
    
    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.
    
    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

//Terralib

#include "../help/HelpPushButton.h"
#include "../layer/search/LayerSearchWidget.h"
#include "../layer/search/LayerSearchWizardPage.h"
#include "RasterStatisticsWizard.h"
#include "RasterStatisticsWizardPage.h"

te::qt::widgets::RasterStatisticsWizard::RasterStatisticsWizard(QWidget* parent) :
QWizard(parent)
{
  this->setWizardStyle(QWizard::ModernStyle);
  this->setWindowTitle(tr("Raster Statistics"));

  this->setOption(QWizard::HaveHelpButton, true);
  this->setOption(QWizard::HelpButtonOnRight, false);

  te::qt::widgets::HelpPushButton* helpButton = 
    new te::qt::widgets::HelpPushButton(this);
  this->setButton(QWizard::HelpButton, helpButton);

  helpButton->setPageReference("plugins/rp/rp_rasterstatistics.html");

  connect(this, SIGNAL(currentIdChanged(int)), SLOT(onPageChanged(int)));

  addPages();
}

te::qt::widgets::RasterStatisticsWizard::~RasterStatisticsWizard() = default;

bool te::qt::widgets::RasterStatisticsWizard::validateCurrentPage()
{
  if(currentPage() ==  m_layerSearchPage.get())
  {
    std::list<te::map::AbstractLayerPtr> list = 
      m_layerSearchPage->getSearchWidget()->getSelecteds();

    if(list.size() == 1)
    {
      te::map::AbstractLayerPtr l = *list.begin();

      m_statsPage->setLayer(l);
      
      return m_layerSearchPage->isComplete();
    }
  }
  else if(currentPage() ==  m_statsPage.get())
  {
    return execute();
  }

  return false;
}

void te::qt::widgets::RasterStatisticsWizard::setList(std::list<te::map::AbstractLayerPtr> &layerList)
{
  m_layerSearchPage->getSearchWidget()->setList(layerList);
  m_layerSearchPage->getSearchWidget()->filterOnlyByRaster();
  m_layerSearchPage->getSearchWidget()->enableMultiSelection(
    false );
  m_layerSearchPage->useSkipButton( false );
}

void te::qt::widgets::RasterStatisticsWizard::addPages()
{
  m_layerSearchPage.reset(new LayerSearchWizardPage(this));
  addPage(m_layerSearchPage.get());

  m_statsPage.reset(new RasterStatisticsWizardPage(this));
  addPage(m_statsPage.get());
}

bool te::qt::widgets::RasterStatisticsWizard::execute()
{
  return true;
}

void te::qt::widgets::RasterStatisticsWizard::onPageChanged(int /*index*/)
{
  this->adjustSize();
}

void te::qt::widgets::RasterStatisticsWizard::setMapDisplay(te::qt::widgets::MapDisplay* mapDisplay)
{
  m_statsPage->setMapDisplay(mapDisplay);
}

void te::qt::widgets::RasterStatisticsWizard::setActionGroup(QActionGroup* actionGroup)
{
  m_statsPage->setActionGroup(actionGroup);
}