/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.
  
    This file is part of the TerraLib - a Framework for building GIS enabled applications.
    
    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.
    
    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.
    
    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */


#ifndef TE_QT_WIDGETS_RASTERSTATISTICSWIZARD_H
#define TE_QT_WIDGETS_RASTERSTATISTICSWIZARD_H

//TerraLib
#ifndef Q_MOC_RUN
#include "../../../dataaccess/datasource/DataSource.h"
#include "../../../maptools/AbstractLayer.h"
#include "../../../qt/widgets/canvas/MapDisplay.h"
#endif
#include "../Config.h"

//Qt
#include <QWizard>
#include <QActionGroup>

//STL
#include <memory>

namespace te 
{
  namespace qt 
  {
    namespace widgets 
    {
      //Forward declarations
      class RasterStatisticsWizardPage;
      class LayerSearchWizardPage;

      class TEQTWIDGETSEXPORT RasterStatisticsWizard : public QWizard
      {
        Q_OBJECT

        public:

          RasterStatisticsWizard(QWidget* parent=0);

          ~RasterStatisticsWizard();

          bool validateCurrentPage();

          void setList(std::list<te::map::AbstractLayerPtr>& layerList);
          
          void setMapDisplay(te::qt::widgets::MapDisplay* mapDisplay);
          
          void setActionGroup(QActionGroup* actionGroup);

        protected:
          
          std::unique_ptr<te::qt::widgets::RasterStatisticsWizardPage> 
            m_statsPage;
          std::unique_ptr<te::qt::widgets::LayerSearchWizardPage> 
            m_layerSearchPage;          

          void addPages();

          bool execute();

        protected slots:

          void onPageChanged(int index);

      };
      
    } // namespace widgets
  } // namespace qt
} // namespace te

#endif // TE_QT_WIDGETS_RASTERSTATISTICSWIZARD_H
