/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.
  
    This file is part of the TerraLib - a Framework for building GIS enabled applications.
    
    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.
    
    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.
    
    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */


//Terralib
#include "RasterStatisticsWizardLog.h"
#include "ui/ui_RasterStatisticsWizardLogForm.h"

// Qt
#include <QMessageBox>
#include <QFileDialog>
#include <QFile>
#include <QTextStream>

te::qt::widgets::RasterStatisticsWizardLog::RasterStatisticsWizardLog(QWidget *parent) 
  :
  QDialog(parent)
{
  m_uiLogForm.reset(new Ui::RasterStatisticsWizardLogForm);
  m_uiLogForm->setupUi(this);
  
  connect(m_uiLogForm->m_savePushButton, SIGNAL(clicked(bool)), this,
          SLOT(savePushButtonPressed(bool)));  
  connect(m_uiLogForm->m_closePushButton, SIGNAL(clicked(bool)), this,
          SLOT(closePushButtonPressed(bool)));    
}

te::qt::widgets::RasterStatisticsWizardLog::~RasterStatisticsWizardLog()
{
}

void te::qt::widgets::RasterStatisticsWizardLog::savePushButtonPressed(bool checked)
{
  QString fileNameString = QFileDialog::getSaveFileName( this, tr("Save File"),
    "", "Text files (*.txt)" );
  if( fileNameString.isEmpty() )
  {
    QMessageBox::critical( this, tr("Error"), tr("Invalid file name"));
    return;
  }
  
  QFile qFile(fileNameString);
  if (qFile.open(QIODevice::WriteOnly)) 
  {
    QTextStream out(&qFile);
    out << m_uiLogForm->m_logPlainTextEdit->toPlainText();
    qFile.close();
  }
  else
  {
    QMessageBox::critical( this, tr("Error"), tr("File save error"));
    return;
  }
}

void te::qt::widgets::RasterStatisticsWizardLog::closePushButtonPressed(bool checked)
{
  close();
}

void te::qt::widgets::RasterStatisticsWizardLog::clearAllText()
{
  m_uiLogForm->m_logPlainTextEdit->clear();
}

void te::qt::widgets::RasterStatisticsWizardLog::addTextLine( 
  const std::string& textLine )
{
  m_uiLogForm->m_logPlainTextEdit->appendPlainText( QString( textLine.c_str() ) );
}

