/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/widgets/rp/RasterTransformWizard.h

  \brief A Qt dialog that allows raster transform.
*/

#ifndef __TERRALIB_QT_WIDGETS_RP_INTERNAL_RASTERTRANSFORMWIZARD_H
#define __TERRALIB_QT_WIDGETS_RP_INTERNAL_RASTERTRANSFORMWIZARD_H

// TerraLib
#ifndef Q_MOC_RUN
  #include "../../../maptools/AbstractLayer.h"
  #include "../../../qt/widgets/layer/explorer/LayerItemView.h"
#endif
#include "../Config.h"

// STL
#include <memory>

// Qt
#include <QWizard>
#include <QActionGroup>

namespace te
{
  namespace qt
  {
    namespace widgets
    {
      class RasterTransformWizardPage;
      class LayerSearchWizardPage;
      class RasterInfoWizardPage;

      /*!
        \class RasterTransformWizard

        \brief This class is GUI used to define the raster transform operation.
      */
      class TEQTWIDGETSEXPORT RasterTransformWizard : public QWizard
      {
        Q_OBJECT

        public:

          RasterTransformWizard(QWidget* parent );

          ~RasterTransformWizard();

        public:

          virtual bool validateCurrentPage();

          void setList(std::list<te::map::AbstractLayerPtr>& layerList);

          void setLayer(te::map::AbstractLayerPtr layer);

          te::map::AbstractLayerPtr getOutputLayer();

        protected:

          void addPages();

          bool execute();
          
          /*!
            \brief Load the raster data.
            \param rasterPtr A pointer to the loaded memory raster.
            \note Only user selected bands will be loaded.
          */
          bool executeLoad( std::unique_ptr< te::rst::Raster >& rasterPtr );

          /*!
            \brief Data tapye change.
            \param rasterPtr A pointer to the loaded memory raster.
          */
          bool executeDataTypeChange( std::unique_ptr< te::rst::Raster >& rasterPtr );
          
          /*!
            \brief No-data value change.
            \param rasterPtr A pointer to the loaded memory raster.
          */
          bool executeNoDataValueChange( std::unique_ptr< te::rst::Raster >& rasterPtr );          

          /*!
            \brief Raster ressample.
            \param rasterPtr A pointer to the loaded memory raster.
          */
          bool executeResample( std::unique_ptr< te::rst::Raster >& rasterPtr );                    
          
          /*!
            \brief Raster reprojection.
            \param rasterPtr A pointer to the loaded memory raster.
          */
          bool executeReprojection( std::unique_ptr< te::rst::Raster >& rasterPtr );          
          
        protected slots:

          void onPageChanged(int index);

        signals:

          void addLayer(te::map::AbstractLayerPtr layer);

      private:

        std::unique_ptr<te::qt::widgets::RasterTransformWizardPage> m_mainWizardPagePage;
        std::unique_ptr<te::qt::widgets::LayerSearchWizardPage> m_layerSearchPage;
        std::unique_ptr<te::qt::widgets::RasterInfoWizardPage> m_rasterInfoPage;
        te::map::AbstractLayerPtr m_outputLayer;
      };
    }   // end namespace widgets
  }     // end namespace qt
}       // end namespace te

#endif  // __TERRALIB_QT_WIDGETS_RP_INTERNAL_RASTERTRANSFORMWIZARD_H
