/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.
  
    This file is part of the TerraLib - a Framework for building GIS enabled applications.
    
    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.
    
    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.
    
    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */


#ifndef TE_QT_WIDGETS_RASTERTRANSFORMWIZARDPAGE_H
#define TE_QT_WIDGETS_RASTERTRANSFORMWIZARDPAGE_H

#include "ui/ui_RasterTransformWizardPageForm.h"
#include "../../../maptools/AbstractLayer.h"
#include "../Config.h"

#include <QWizardPage>
#include <QActionGroup>

#include <memory>
#include <vector>

namespace te 
{
  namespace rst { class Raster; }

  namespace qt 
  {
    namespace widgets 
    {
      
      class TEQTWIDGETSEXPORT RasterTransformWizardPage : public QWizardPage
      {
        Q_OBJECT

        public:
          
          std::unique_ptr<Ui::RasterTransformWizardPageForm> m_uiPageForm;
          
          RasterTransformWizardPage(QWidget* parent=0);

          ~RasterTransformWizardPage();

          bool isComplete() const;

          /*!
            \brief This method is used to set the selected layer for vectorization operation
            
            \param layer The layer ptr 

            \note This layer MUST HAVE a valid raster object.
          */
          void setLayer(te::map::AbstractLayerPtr layer);

          /*!
            \brief return the current layer pointer or null pointer.
            \return return the current layer pointer or null pointer.
          */
          te::map::AbstractLayer* getLayerPtr();
          
          /*!
            \brief Returns the output raster data type.
            \brief Returns the output raster data type.
          */
          unsigned int getOutDataType() const;
          
          /*!
            \brief Returns the interpolator type.
            \return Returns the interpolator data type.
          */
          unsigned int getInterpolator() const;          
          
          /*!
            \brief Return the current selected band indexes.
            \param bandsIndexes Current selected band indexes.
          */
          void getSelectedInBandIndexes( std::vector< unsigned int >& bandsIndexes ) const;
          
          /*!
            \brief Returns the output raster no-data values.
            \brief Returns the output raster no-data values.
          */
          std::vector< double > getOutNoDataValues() const;          

        protected:

          bool getInRaster( 
            std::unique_ptr<te::da::DataSet>& inDataSet,
            std::unique_ptr<te::rst::Raster >& inRasterPtr ) const;
          
          te::map::AbstractLayerPtr m_layer;
          
        protected slots:
          
          void onSelectOldSRSPushButtonClicked();
          
          void onSelectNewSRSPushButtonClicked();
          
          void onBandsListItemSelectionChanged();

      };
      
    } // namespace widgets
  } // namespace qt
} // namespace te

#endif // TE_QT_WIDGETS_RASTERTRANSFORMWIZARDPAGE_H
