/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.
  
    This file is part of the TerraLib - a Framework for building GIS enabled applications.
    
    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.
    
    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.
    
    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */


#ifndef TE_QT_WIDGETS_RESTORATIONWIZARDPAGE_H
#define TE_QT_WIDGETS_RESTORATIONWIZARDPAGE_H

#include "ui/ui_RestorationWizardPageForm.h"
#include "../../../maptools/AbstractLayer.h"
#include "../Config.h"
#include "../../../rp/Restoration.h"

#include <QWizardPage>

#include <memory>
#include <vector>

namespace te 
{
  namespace rst { class Raster; }

  namespace qt 
  {
    namespace widgets 
    {
      
      class TEQTWIDGETSEXPORT RestorationWizardPage : public QWizardPage
      {
        Q_OBJECT

        public:
          
          std::unique_ptr<Ui::RestorationWizardPageForm> m_uiPageForm;
          
          RestorationWizardPage(QWidget* parent=0);

          ~RestorationWizardPage();

          bool isComplete() const;

          /*!
            \brief This method is used to set the selected layer for vectorization operation
            
            \param layer The layer ptr 

            \note This layer MUST HAVE a valid raster object.
          */
          void setLayer(te::map::AbstractLayerPtr layer);

          /*!
            \brief return the current layer pointer or null pointer.
            \return return the current layer pointer or null pointer.
          */
          te::map::AbstractLayer* getLayerPtr();
          
          /*!
            \brief Return the current selected band designation indexes.
            \param bandsIndexes Return the current selected band designation indexes.
          */
          void getSelectedInBandDesignations( std::vector< unsigned int >& bandsIndexes ) const;
          
          /*!
            \brief Return the current selected sensor type.
          */
          te::rp::Restoration::InputParameters::SensorType getSensorType() const;

          /*!
            \brief Return the current selected sampling factor.
          */
          te::rp::Restoration::InputParameters::SamplingFactorType
            getSamplingFactor() const;

        protected:
          
          te::map::AbstractLayerPtr m_layer;
          
        protected slots:

          void onBandsItemSelectionChanged();
          
          void onSensorsCurrentIndexChanged(int index);
          
          void onSamplingFactorCurrentIndexChanged(int index);
      };
      
    } // namespace widgets
  } // namespace qt
} // namespace te

#endif // TE_QT_WIDGETS_RESTORATIONWIZARDPAGE_H
