﻿/*  Copyright (C) 2011-2012 National Institute For Space Research (INPE) - Brazil.

This file is part of the TerraLib - a Framework for building GIS enabled applications.

TerraLib is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License,
or (at your option) any later version.

TerraLib is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with TerraLib. See COPYING. If not, write to
TerraLib Team at <terralib-team@terralib.org>.
*/

/*!
\file terralib/rp/widgets/rp/MultiSourceClassifierFirstDialog.cpp

\brief A first dialog used to execute MultiSource image classification.
*/

#include "MultiSourceClassifierFirstDialog.h"
#include "MultiSourceClassifierSecondDialog.h"
#include "ui/ui_MultiSourceClassifierFirstDialogForm.h"
//#include "ui_MultiSourceClassifierFirstHelpDialogForm.h"

//#include "../Config.h"
//#include <terralib/maptools/Utils.h>
//#include <terralib/common.h>
//#include <terralib/rp/Classifier.h>
#include "../../../../rp/RasterAttributes.h"
#include "../../../../raster/RasterFactory.h"
//#include <terralib/dataaccess.h>
#include "../../../../datatype/Property.h"
#include "../../../../dataaccess/datasource/DataSourceFactory.h"
#include "../../utils/FileDialog.h"
#include "../../../../rp/radar/RadarFunctions.h"
#include "../../../../rp/radar/PointersVector.h"
#include "../../../../memory/CachedRaster.h"
#include "../Utils.h"
#include "../../Utils.h"

// Qt
#include <QListWidgetItem>
#include <QFileDialog>
#include <QList>
#include <QMessageBox>
#include <QIntValidator>
#include <QDoubleValidator>

// Boost
#include <boost/algorithm/string.hpp>
#include <boost/filesystem.hpp>
#include <boost/uuid/random_generator.hpp>
#include <boost/uuid/uuid_io.hpp>
#include <boost/foreach.hpp>
#include <boost/lexical_cast.hpp>

#include <fstream>

te::qt::widgets::MultiSourceClassifierFirstDialog::MultiSourceClassifierFirstDialog(QWidget* parent, Qt::WindowFlags f)
	: QDialog(parent, f) {

	m_ui = new Ui::MultiSourceClassifierFirstDialogForm;

	m_ui->setupUi(this);

	// Validating input values

	QLocale lo(QLocale::C);
	lo.setNumberOptions(QLocale::RejectGroupSeparator);

	auto validator1 = new QDoubleValidator(0.00, 1000.00, 2, m_ui->m_numImagesLineEdit);
	validator1->setNotation(QDoubleValidator::StandardNotation);
	validator1->setLocale(lo);
	m_ui->m_numImagesLineEdit->setValidator(validator1);

	auto validator2 = new QDoubleValidator(0.00, 1000.00, 2, m_ui->m_horizontalLagNameLineEdit);
	validator2->setNotation(QDoubleValidator::StandardNotation);
	validator2->setLocale(lo);
	m_ui->m_horizontalLagNameLineEdit->setValidator(validator2);

	auto validator3 = new QDoubleValidator(0.00, 1000.00, 2, m_ui->m_verticalLagNameLineEdit);
	validator3->setNotation(QDoubleValidator::StandardNotation);
	validator3->setLocale(lo);
	m_ui->m_verticalLagNameLineEdit->setValidator(validator3);

	auto validator4 = new QDoubleValidator(0.00, 100.00, 2, m_ui->m_significanceLevelLineEdit);
	validator4->setNotation(QDoubleValidator::StandardNotation);
	validator4->setLocale(lo);
	m_ui->m_significanceLevelLineEdit->setValidator(validator4);

	connect(m_ui->m_targetFileSegImageToolButton, SIGNAL(clicked()), this, SLOT(onTargetFileSegImageToolButtonPressed()));

	connect(m_ui->m_targetFileTrainingToolButton, SIGNAL(clicked()), this, SLOT(onTargetFileTrainingToolButtonPressed()));

	connect(m_ui->m_targetFileTestToolButton, SIGNAL(clicked()), this, SLOT(onTargetFileTestToolButtonPressed()));

	connect(m_ui->m_targetFileRepositoryToolButton, SIGNAL(clicked()), this, SLOT(onTargetFileRepositoryToolButtonPressed()));

	connect(m_ui->m_okPushButton, SIGNAL(clicked()), this, SLOT(onOkPushButtonClicked()));

	connect(m_ui->m_cancelPushButton, SIGNAL(clicked()), this, SLOT(onCancelPushButtonClicked()));

	connect(m_ui->m_numImagesLineEdit, SIGNAL(textEdited(QString)), this, SLOT(onOneOrMultiSourceActivated()));

	connect(m_ui->m_opFuzzyRadioButton, SIGNAL(clicked()), this, SLOT(onFuzzyChecked()));

	connect(m_ui->m_opSumRadioButton, SIGNAL(clicked()), this, SLOT(onSumChecked()));

	connect(m_ui->m_opMultiplicationRadioButton, SIGNAL(clicked()), this, SLOT(onMultiplicationChecked()));

	connect(m_ui->m_opMinimumRadioButton, SIGNAL(clicked()), this, SLOT(onMinimumChecked()));

	connect(m_ui->m_opHellingerRadioButton, SIGNAL(clicked()), this, SLOT(onHellingerChecked()));

	m_ui->m_numImagesLineEdit->setEnabled(true);

	m_ui->m_segImageLineEdit->setEnabled(false);

	m_ui->m_trainingLineEdit->setEnabled(false);

	m_ui->m_testLineEdit->setEnabled(false);

	m_ui->m_layerNameLineEdit->setEnabled(false);

  m_ui->m_helpPushButton->setPageReference("plugins/rp/rp_radar_multisourceclassifier.html");
}

te::qt::widgets::MultiSourceClassifierFirstDialog::~MultiSourceClassifierFirstDialog() {
}

bool te::qt::widgets::MultiSourceClassifierFirstDialog::loadTrainingSamples()
{
  return te::rp::radar::LoadSamplesFile(
            m_algoInputParams.m_dirTrainingSamples["URI"],
            m_algoInputParams.m_numberOfClassesOnTraining,
            m_algoInputParams.m_classesIDT,
            m_algoInputParams.m_nameOfSamples,
            m_algoInputParams.m_numberOfSamples,
            m_algoInputParams.m_polygonSamplesTNew,
            m_algoInputParams.m_polygonSamplesT );
}

bool te::qt::widgets::MultiSourceClassifierFirstDialog::loadTestSamples()
{
  return te::rp::radar::LoadSamplesFile(
          m_algoInputParams.m_dirTestSamples["URI"],
          m_algoInputParams.m_numberOfClassesOnTest,
          m_algoInputParams.m_classesIDTest,
          m_algoInputParams.m_nameOfSamplesTest,
          m_algoInputParams.m_numberOfSamplesTest,
          m_algoInputParams.m_polygonSamplesTestNew,
          m_algoInputParams.m_polygonSamplesTest );
}

std::string te::qt::widgets::MultiSourceClassifierFirstDialog::getCombinationType() {
	if (m_ui->m_opSumRadioButton->isChecked())
		return "sum";
	else if (m_ui->m_opMultiplicationRadioButton->isChecked())
		return "multiplication";
	else if (m_ui->m_opMinimumRadioButton->isChecked())
		return "minimum";
	else if (m_ui->m_opHellingerRadioButton->isChecked())
		return "Hellinger";
	else if (m_ui->m_opFuzzyRadioButton->isChecked())
		return "fuzzy";
	else
		return "false";
}

void te::qt::widgets::MultiSourceClassifierFirstDialog::onTargetFileSegImageToolButtonPressed()
{
	m_ui->m_segImageLineEdit->clear();

  te::qt::widgets::FileDialog fileDialog(this,
    te::qt::widgets::FileDialog::RASTER,
    "rp_radar_raster_processing_load_raster",
    te::qt::widgets::FileDialog::OPEN_FILE_MODE );

  try {
    fileDialog.exec();
  }
  catch (te::common::Exception& ex) {
    QMessageBox::warning(this, tr("File information"), ex.what());
    return;
  }

  std::string fileName = fileDialog.getPath();

	if (fileName.empty())
		return;

	boost::filesystem::path outfile(fileName);
	std::string aux = outfile.string();
	m_ui->m_segImageLineEdit->setText(aux.c_str());

	m_toFile = true;
	m_ui->m_segImageLineEdit->setEnabled(false);
}

void te::qt::widgets::MultiSourceClassifierFirstDialog::onTargetFileTrainingToolButtonPressed() {
	/*m_ui->m_trainingLineEdit->clear();

	QString dirname = QFileDialog::getExistingDirectory(this, tr("Select a Directory"),
	QDir::currentPath() );
	if(dirname.isNull())
	return;

	boost::filesystem::path outfile(dirname.toStdString());
	std::string aux = outfile.string();
	m_ui->m_trainingLineEdit->setText(aux.c_str());

	m_toFile = true;*/
	m_ui->m_trainingLineEdit->setEnabled(false);
	m_ui->m_trainingLineEdit->clear();

	QString fileName = QFileDialog::getOpenFileName(this, tr("Open Text..."),
		GetFilePathFromSettings("rp_radar_raster_processing_load_txt"),
    tr("Text files (*.txt)"));

	if (fileName.isEmpty())
		return;

	boost::filesystem::path outfile(fileName.toStdString());
	std::string aux = outfile.string();
	m_ui->m_trainingLineEdit->setText(aux.c_str());

	m_toFile = true;
	m_ui->m_trainingLineEdit->setEnabled(false);
}

void te::qt::widgets::MultiSourceClassifierFirstDialog::onTargetFileTestToolButtonPressed() {
	/*m_ui->m_testLineEdit->clear();

	QString dirname = QFileDialog::getExistingDirectory(this, tr("Select a Directory"),
	QDir::currentPath() );
	if(dirname.isNull())
	return;

	boost::filesystem::path outfile(dirname.toStdString());
	std::string aux = outfile.string();
	m_ui->m_testLineEdit->setText(aux.c_str());

	m_toFile = true;
	m_ui->m_testLineEdit->setEnabled(false);*/
	m_ui->m_testLineEdit->setEnabled(false);
	m_ui->m_testLineEdit->clear();

	QString fileName = QFileDialog::getOpenFileName(this, tr("Open Text..."),
		te::qt::widgets::GetFilePathFromSettings("rp_radar_raster_processing_load_txt"), tr("Text files (*.txt)"));

	if (fileName.isEmpty())
		return;

	boost::filesystem::path outfile(fileName.toStdString());
	std::string aux = outfile.string();
	m_ui->m_testLineEdit->setText(aux.c_str());

	m_toFile = true;
	m_ui->m_testLineEdit->setEnabled(false);
}

void te::qt::widgets::MultiSourceClassifierFirstDialog::onTargetFileRepositoryToolButtonPressed()
{
	m_ui->m_layerNameLineEdit->clear();
	m_ui->m_repositoryLineEdit->clear();

  te::qt::widgets::FileDialog fileDialog(this,
    te::qt::widgets::FileDialog::RASTER,
    "rp_radar_raster_processing_save_raster",
    te::qt::widgets::FileDialog::SAVE_FILE_MODE );

  try {
    fileDialog.exec();
  }
  catch (te::common::Exception& ex) {
    QMessageBox::warning(this, tr("File information"), ex.what());
    return;
  }

  std::string fileName = fileDialog.getPath();

	if (fileName.empty())
		return;

	boost::filesystem::path outfile(fileName);
	std::string aux = outfile.leaf().string();
	m_ui->m_layerNameLineEdit->setText(aux.c_str());
	aux = outfile.string();
	m_ui->m_repositoryLineEdit->setText(aux.c_str());

	m_toFile = true;
	m_ui->m_layerNameLineEdit->setEnabled(false);
}

void te::qt::widgets::MultiSourceClassifierFirstDialog::onOneOrMultiSourceActivated() {
	int numImages = m_ui->m_numImagesLineEdit->text().toUInt();
	if (numImages == 1) {
		m_ui->m_combinationTypeGroupBox->setEnabled(false);
		m_ui->m_statisticsCheckBox->setEnabled(true);
		m_ui->m_pValuesCheckBox->setEnabled(true);
		m_ui->m_perClassStatisticsCheckBox->setEnabled(true);
		m_ui->m_perClassPValuesCheckBox->setEnabled(true);
		m_ui->m_reportCheckBox->setEnabled(true);
		m_algoInputParams.m_combinationType = "invalid";
	}
	else
		m_ui->m_combinationTypeGroupBox->setEnabled(true);

	if (m_ui->m_opSumRadioButton->isChecked() || m_ui->m_opMinimumRadioButton->isChecked()) {
	 m_ui->m_distancesCheckBox->setEnabled(true);
		m_ui->m_perClassDistancesCheckBox->setEnabled(true);
	}
	else {
		m_ui->m_distancesCheckBox->setEnabled(false);
		m_ui->m_perClassDistancesCheckBox->setEnabled(false);
	}
}

void te::qt::widgets::MultiSourceClassifierFirstDialog::onFuzzyChecked() {
	m_ui->m_statisticsCheckBox->setEnabled(true);
	m_ui->m_pValuesCheckBox->setEnabled(true);
	m_ui->m_perClassStatisticsCheckBox->setEnabled(false);
	m_ui->m_perClassPValuesCheckBox->setEnabled(false);
	m_ui->m_reportCheckBox->setEnabled(false);
	m_ui->m_perClassPValuesCheckBox->setEnabled(false);
	m_ui->m_reportCheckBox->setEnabled(false);
}

void te::qt::widgets::MultiSourceClassifierFirstDialog::onSumChecked() {
	m_ui->m_statisticsCheckBox->setEnabled(true);
	m_ui->m_pValuesCheckBox->setEnabled(true);
	m_ui->m_perClassStatisticsCheckBox->setEnabled(true);
	m_ui->m_perClassPValuesCheckBox->setEnabled(true);
	m_ui->m_reportCheckBox->setEnabled(true);
	m_ui->m_distancesCheckBox->setEnabled(true);
	m_ui->m_perClassDistancesCheckBox->setEnabled(true);
}

void te::qt::widgets::MultiSourceClassifierFirstDialog::onMultiplicationChecked() {
	m_ui->m_statisticsCheckBox->setEnabled(true);
	m_ui->m_pValuesCheckBox->setEnabled(true);
	m_ui->m_perClassStatisticsCheckBox->setEnabled(true);
	m_ui->m_perClassPValuesCheckBox->setEnabled(true);
	m_ui->m_reportCheckBox->setEnabled(true);
	m_ui->m_perClassPValuesCheckBox->setEnabled(false);
	m_ui->m_reportCheckBox->setEnabled(false);
}

void te::qt::widgets::MultiSourceClassifierFirstDialog::onMinimumChecked() {
	m_ui->m_statisticsCheckBox->setEnabled(true);
	m_ui->m_pValuesCheckBox->setEnabled(true);
	m_ui->m_perClassStatisticsCheckBox->setEnabled(false);
	m_ui->m_perClassPValuesCheckBox->setEnabled(false);
	m_ui->m_reportCheckBox->setEnabled(true);
	m_ui->m_distancesCheckBox->setEnabled(true);
	m_ui->m_perClassDistancesCheckBox->setEnabled(true);
}

void te::qt::widgets::MultiSourceClassifierFirstDialog::onHellingerChecked() {
	m_ui->m_statisticsCheckBox->setEnabled(true);
	m_ui->m_pValuesCheckBox->setEnabled(false);
	m_ui->m_perClassStatisticsCheckBox->setEnabled(true);
	m_ui->m_perClassPValuesCheckBox->setEnabled(false);
	m_ui->m_reportCheckBox->setEnabled(true);
	m_ui->m_perClassPValuesCheckBox->setEnabled(false);
	m_ui->m_reportCheckBox->setEnabled(false);
}

void te::qt::widgets::MultiSourceClassifierFirstDialog::onOkPushButtonClicked() {
	if (m_ui->m_numImagesLineEdit->text().isEmpty()) {
		QMessageBox::information(this, "MultiSourceClassifier", "Define the number of images.");
		return;
	}

	if (m_ui->m_numImagesLineEdit->text().toUInt() == 0) {
		QMessageBox::information(this, "MultiSourceClassifier", "The number of images must be a natural number and not equal to 0.");
		return;
	}

	if ((m_ui->m_numImagesLineEdit->text().toDouble() - m_ui->m_numImagesLineEdit->text().toInt()) != 0) {
		QMessageBox::information(this, "MultiSourceClassifier",
			"The number of images must be a natural number and not equal to 0.");
		return;
	}

	if (m_ui->m_segImageLineEdit->text().isEmpty()) {
		QMessageBox::information(this, "MultiSourceClassifier", "Choose a segmented image.");
		return;
	}

	if (m_ui->m_trainingLineEdit->text().isEmpty()) {
		QMessageBox::information(this, "MultiSourceClassifier", "Define a training samples directory.");
		return;
	}

	if (m_ui->m_testLineEdit->text().isEmpty()) {
		QMessageBox::information(this, "MultiSourceClassifier", "Define a test samples directory.");
		return;
	}

	if (m_ui->m_layerNameLineEdit->text().isEmpty()) {
		QMessageBox::information(this, "MultiSourceClassifier", "Define a name for the resulting layer.");
		return;
	}

	double significanceLevel = m_ui->m_significanceLevelLineEdit->text().toDouble();
	if ((significanceLevel < 0.0) || (significanceLevel > 100.0)) {
		QMessageBox::information(this, "MultiSourceClassifier",
			"Define a valid significance level value: 0 <= significance level <= 100.");
		return;
	}

	if (m_ui->m_horizontalLagNameLineEdit->text().isEmpty()) {
		QMessageBox::information(this, "MultiSourceClassifier", "Define horizontal lag.");
		return;
	}

	if ((m_ui->m_horizontalLagNameLineEdit->text().toDouble() - m_ui->m_horizontalLagNameLineEdit->text().toInt()) != 0) {
		QMessageBox::information(this, "MultiSourceClassifier",
			"Horizontal lag value must be 0 or a positive integer.");
		return;
	}

	if (m_ui->m_verticalLagNameLineEdit->text().isEmpty()) {
		QMessageBox::information(this, "MultiSourceClassifier", "Define vertical lag.");
		return;
	}
	if ((m_ui->m_verticalLagNameLineEdit->text().toDouble() - m_ui->m_verticalLagNameLineEdit->text().toInt()) != 0) {
		QMessageBox::information(this, "MultiSourceClassifier",
			"Vertical lag value must be 0 or a positive integer.");
		return;
	}

	m_algoInputParams.m_sourcesNumber = m_ui->m_numImagesLineEdit->text().toUInt();
	m_algoInputParams.m_significanceLevel = (double)(100 - m_ui->m_significanceLevelLineEdit->text().toDouble()) / 100;
	m_algoInputParams.m_lagX = m_ui->m_horizontalLagNameLineEdit->text().toUInt();
	m_algoInputParams.m_lagY = m_ui->m_verticalLagNameLineEdit->text().toUInt();
	m_algoInputParams.m_savePerClassDistancesFile = m_ui->m_perClassDistancesCheckBox->isChecked();
	m_algoInputParams.m_savePerClassStatiticsFile = m_ui->m_perClassStatisticsCheckBox->isChecked();
	m_algoInputParams.m_savePerClassPValueFile = m_ui->m_perClassStatisticsCheckBox->isChecked();
	m_algoInputParams.m_saveDistances = m_ui->m_distancesCheckBox->isChecked();
	m_algoInputParams.m_saveStatitics = m_ui->m_statisticsCheckBox->isChecked();
	m_algoInputParams.m_savePValues = m_ui->m_pValuesCheckBox->isChecked();
	m_algoInputParams.m_saveReportFile = m_ui->m_reportCheckBox->isChecked();

//	m_algoInputParams.m_seginfo["URI"] = m_ui->m_segImageLineEdit->text().toStdString();
	m_algoInputParams.m_dirTrainingSamples["URI"] = m_ui->m_trainingLineEdit->text().toStdString();
	m_algoInputParams.m_dirTestSamples["URI"] = m_ui->m_testLineEdit->text().toStdString();

	if (m_algoInputParams.m_sourcesNumber > 1) {
		if (m_ui->m_opSumRadioButton->isChecked())
			m_algoInputParams.m_combinationType = "sum";
		else if (m_ui->m_opMultiplicationRadioButton->isChecked())
			m_algoInputParams.m_combinationType = "multiplication";
		else if (m_ui->m_opMinimumRadioButton->isChecked())
			m_algoInputParams.m_combinationType = "minimum";
		else if (m_ui->m_opHellingerRadioButton->isChecked())
			m_algoInputParams.m_combinationType = "Hellinger";
		else if (m_ui->m_opFuzzyRadioButton->isChecked())
			m_algoInputParams.m_combinationType = "fuzzy";
	}
	else {
		m_algoInputParams.m_combinationType = "invalid";
	}

	te::rp::radar::PointersVector< te::gm::Geometry* > geometries;
  std::map<std::string, std::string> auxRasterSegInfo;
  auxRasterSegInfo["URI"] = m_ui->m_segImageLineEdit->text().toStdString();
	boost::shared_ptr< te::rst::Raster > rasterseg( te::rst::RasterFactory::open(
    auxRasterSegInfo ) );
	m_algoInputParams.m_rasterSegPtr = rasterseg.get();

	m_algoInputParams.m_numberOfRows = rasterseg->getNumberOfRows();
	m_algoInputParams.m_numberOfColumns = rasterseg->getNumberOfColumns();
	m_algoInputParams.m_SRID = rasterseg->getSRID();
	m_algoInputParams.m_extent = rasterseg->getGrid();

	std::vector< unsigned int > inputRastersBands;

	std::vector< int > modelImage;

  std::vector< boost::shared_ptr< te::rst::Raster > > m_inputRasterPointer;

	for (unsigned int i = 0; i < m_algoInputParams.m_sourcesNumber; i++) {

		te::qt::widgets::MultiSourceClassifierSecondDialog ChooseModelDialog(this);
		ChooseModelDialog.m_numberOfImages = i + 1;

		ChooseModelDialog.exec();

		if ((m_algoInputParams.m_combinationType == "Hellinger") && (m_algoInputParams.m_sourcesNumber > 1))
			ChooseModelDialog.m_specialCaseHellinger = 1;
		else
			ChooseModelDialog.m_specialCaseHellinger = 0;

		std::map< std::string, std::string > rasterInfo = ChooseModelDialog.m_rasterInfo;

		boost::shared_ptr< te::rst::Raster > inputRaster(te::rst::RasterFactory::open(rasterInfo));

		m_inputRasterPointer.push_back(inputRaster);

		m_algoInputParams.m_inputRasterPtr.push_back(inputRaster.get());

		for (unsigned int i = 0; i < inputRaster->getNumberOfBands(); ++i)
			inputRastersBands.push_back(i);
		m_algoInputParams.m_inputRastersBands.push_back(inputRastersBands);

		inputRastersBands.clear();

		if (m_algoInputParams.m_SRID != inputRaster->getSRID()) {
				QMessageBox::information(this, "MultiSourceClassifier", "Input image SRID mismatch.");
				return;
		}

		if (m_algoInputParams.m_numberOfRows != inputRaster->getNumberOfRows()) {
				QMessageBox::information(this, "MultiSourceClassifier", "Input size images mismatch.");
				return;
		}

		if (m_algoInputParams.m_numberOfColumns != inputRaster->getNumberOfColumns()) {
				QMessageBox::information(this, "MultiSourceClassifier", "Input size images mismatch.");
				return;
		}

		m_algoInputParams.m_imageType.push_back(ChooseModelDialog.m_model);

		if ((ChooseModelDialog.m_model == "IntensityPair") || (ChooseModelDialog.m_model == "Gamma"))
		  modelImage.push_back(1);
		else if (ChooseModelDialog.m_model == "Wishart")
		  modelImage.push_back(2);
		else
		  modelImage.push_back(0);

		if (ChooseModelDialog.m_model == "IntensityPair") {
			if (inputRaster->getNumberOfBands() != 2) {
				QMessageBox::information(this, "MultiSourceClassifier", "Pair Intensity image must have two bands.");
				return;
			}
		}

		if (ChooseModelDialog.m_model == "Gamma") {
			if (inputRaster->getNumberOfBands() != 1) {
				QMessageBox::information(this, "MultiSourceClassifier", "Gamma image must have one band.");
				return;
			}
		}

		if (ChooseModelDialog.m_model == "Wishart") {
			if (inputRaster->getNumberOfBands() < 2) {
				QMessageBox::information(this, "MultiSourceClassifier", "Wishart image must have at least two bands.");
				return;
			}
		}


		m_algoInputParams.m_stochasticDistanceType.push_back(ChooseModelDialog.m_stochasticDistance);

		m_algoInputParams.m_betaValue.push_back(ChooseModelDialog.m_beta);

		m_algoInputParams.m_ENL.push_back(ChooseModelDialog.m_ENL);

		m_algoInputParams.m_radarDataRepresentationType.push_back("covarianceMatrix");

		m_algoInputParams.m_vectorOrMatrixOrder.push_back(ChooseModelDialog.m_orderDataWishart);

		ChooseModelDialog.hide();
	}

	std::cout << "Classifying..." << std::endl;
	std::cout << std::endl;

	// exporting the segmentation into shapefile

	rasterseg->vectorize(geometries, 0);

	for (unsigned i = 0; i < geometries.size(); i++)
		m_algoInputParams.m_polygonsSegImage.push_back(static_cast<te::gm::Polygon*> (geometries[i]));

	if( ! loadTrainingSamples() )
  {
		QMessageBox::information(this, "MultiSourceClassifier",
      "Error loading training samples.");
		return;
  }

	if( !loadTestSamples() )
  {
		QMessageBox::information(this, "MultiSourceClassifier",
      "Error loading test samples.");
		return;
  }

	if (m_algoInputParams.m_numberOfClassesOnTraining != m_algoInputParams.m_numberOfClassesOnTest) {
		QMessageBox::information(this, "MultiSourceClassifier", "Number of training classes is different of test classes.");
		return;
	}

	m_algoOutputParams.m_rInfoClassImage["URI"] = m_ui->m_repositoryLineEdit->text().toStdString() + "_ClassifiedImage.tif";
	m_algoOutputParams.m_rType = "GDAL";
	m_algoOutputParams.m_rInfoClassImageFuzzy["URI"] = m_ui->m_repositoryLineEdit->text().toStdString() + "_ClassifiedImageFuzzy.tif";

	m_algoOutputParams.m_rInfoMap["URI"] = m_ui->m_repositoryLineEdit->text().toStdString() + "_StatisticalTestMap.tif";

	m_algoOutputParams.m_rInfoMapPValue["URI"] = m_ui->m_repositoryLineEdit->text().toStdString() + "_PValueMap.tif";

//	m_algoOutputParams.m_rInfoConfusionMatrix["URI"] = m_ui->m_repositoryLineEdit->text().toStdString() + "_Report.txt";

//	m_algoOutputParams.m_rInfoAttributes["URI"] = m_ui->m_repositoryLineEdit->text().toStdString() + "_Attributes.txt";

	m_algoOutputParams.m_rInfoDistancePerClass["URI"] =
		m_ui->m_repositoryLineEdit->text().toStdString() + "_DistancePerClass.txt";

	m_algoOutputParams.m_rInfoStatisticPerClass["URI"] =
		m_ui->m_repositoryLineEdit->text().toStdString() + "_StatisticPerClass.txt";

	m_algoOutputParams.m_rInfoPValuePerClass["URI"] =
		m_ui->m_repositoryLineEdit->text().toStdString() + "_PValuePerClass.txt";

	if (m_ui->m_opSumRadioButton->isChecked())
		m_algoInputParams.m_combinationType = "sum";
	else if (m_ui->m_opMultiplicationRadioButton->isChecked())
		m_algoInputParams.m_combinationType = "multiplication";
	else if (m_ui->m_opMinimumRadioButton->isChecked())
		m_algoInputParams.m_combinationType = "minimum";
	else if (m_ui->m_opHellingerRadioButton->isChecked())
		m_algoInputParams.m_combinationType = "Hellinger";
	else if (m_ui->m_opFuzzyRadioButton->isChecked())
		m_algoInputParams.m_combinationType = "fuzzy";
	else
		m_algoInputParams.m_combinationType = "invalid";

	// Executing the algorithm

	int status = 0;

	te::rp::radar::MultiSourceClassifier algorithmInstance;

	if (!algorithmInstance.initialize(m_algoInputParams))
	{
		QMessageBox::information(this, "MultiSourceClassifier",
      QString( "Classifier initialization error:" ) +
      QString( algorithmInstance.getErrorMessage().c_str() ) );
		return;
	}

	if (!algorithmInstance.execute(m_algoOutputParams))
	{
		QMessageBox::information(this, "MultiSourceClassifier",
      QString( "Classifier execution error:" +
      QString( algorithmInstance.getErrorMessage().c_str() ) ) );
		return;
	}
	else
	{
		status = 1;
	}

	// Saving on disk the confusion matrix and Kappa coefficient, variance of Kappa and overall accuracy

	if (m_ui->m_reportCheckBox->isChecked()) {
    if( ! te::rp::radar::SaveOutputReportFile(
      m_algoInputParams, m_algoOutputParams,
      m_ui->m_repositoryLineEdit->text().toStdString() + "_Report.txt" ) )
    {
      QMessageBox::information(this, "MultiSourceClassifier", "Output statistics file creation error");
      return;
    }

// 		std::ofstream outFile(m_ui->m_repositoryLineEdit->text().toStdString() + "_Report.txt");
// 		if (outFile.is_open()) {
//
// 		outFile << "Classification parameters:" << std::endl;
// 		 outFile << "      Number of Input Images: " << algoInputParams.m_sourcesNumber << std::endl;
// 		 if (algoInputParams.m_sourcesNumber > 1)
// 		   outFile << "      Combination type: " << algoInputParams.m_combinationType << std::endl;
// 		 outFile << "      Confidence Level: " << 100 - (algoInputParams.m_significanceLevel * 100) << "%" << std::endl;
// 		 outFile << "      Horizontal lag: " << algoInputParams.m_lagX << std::endl;
// 		 outFile << "      Vertical lag: " << algoInputParams.m_lagY << std::endl;
// 		 outFile << std::endl;
// 			for (int i = 0; i < algoInputParams.m_sourcesNumber; ++i) {
// 			  outFile << "      Input Image " << i + 1 << ": " << std::endl;
// 			  outFile << "           Statistical Model: " << algoInputParams.m_imageType[i] << std::endl;
//      outFile << "           Stochastic distance: " << algoInputParams.m_stochasticDistanceType[i] << std::endl;
//      if (algoInputParams.m_stochasticDistanceType[i] == "Renyi")
//          outFile << "           Beta: " << algoInputParams.m_betaValue[i] << std::endl;
//      if (modelImage[i] != 0) {
//        outFile << "           ENL: " << algoInputParams.m_ENL[i] << std::endl;
//        if (modelImage[i] == 2)
//          outFile << "           Covariance matrix order: " << algoInputParams.m_vectorOrMatrixOrder[i] << std::endl;
//      }
//      outFile << std::endl;
// 			}
// 			outFile << std::endl;
// 			outFile << std::endl;
//
// 			if ((algoInputParams.m_combinationType == "fuzzy") && (algoInputParams.m_sourcesNumber > 1)) {
// 				outFile << "Fuzzy module does not have statistic results." << std::endl;
// 				outFile << std::endl;
// 				outFile << std::endl;
// 			}
//
// 			outFile << "Classes:" << std::endl;
// 			for (int i = 0; i < algoInputParams.m_numberOfClassesOnTraining; i++)
// 			  outFile << algoInputParams.m_nameOfSamples[i] << std::endl;
// 			outFile << std::endl;
// 			outFile << std::endl;
//
// 			outFile << "Confusion matrix:" << std::endl;
// 			outFile << "Rows - actual class; Columns - classification" << std::endl;
// 			for (unsigned int i = 0; i < algoInputParams.m_numberOfClassesOnTraining; ++i) {
// 				for (unsigned int j = 0; j < algoInputParams.m_numberOfClassesOnTraining; ++j)
// 					outFile << m_algoOutputParams.m_confusionMatrix[i][j] << " ";
// 				outFile << std::endl;
// 			}
//
// 			outFile << std::endl;
// 			outFile.precision(4);
// 			outFile << "Kappa coefficient: " << m_algoOutputParams.m_kappaCoefficient << std::endl;
//
// 			outFile << std::endl;
// 			outFile << "Variance of Kappa coefficient: " << m_algoOutputParams.m_varianceOfKappa << std::endl;
//
// 			outFile << std::endl;
// 			outFile.precision(4);
// 			outFile << "Overall accuracy: " << 100 * m_algoOutputParams.m_overallAccuracy << "%" << std::endl;
// 		}
// 		else std::cout << "Unable to open confusion matrix file" << std::endl;
//
// 		outFile.close();
	}

	// sorting regions according to ID segments
	std::vector< double > regionsToID;
	regionsToID = m_algoOutputParams.m_IDSegment;
	std::sort(regionsToID.begin(), regionsToID.end());

	// Saving on disk the attributes file

	if (m_ui->m_distancesCheckBox->isChecked() || m_ui->m_statisticsCheckBox->isChecked()
		|| m_ui->m_pValuesCheckBox->isChecked()) {

    if( ! te::rp::radar::SaveOutputAttFile( m_algoInputParams, m_algoOutputParams,
      m_ui->m_statisticsCheckBox->isChecked(), m_ui->m_pValuesCheckBox->isChecked(),
      m_ui->m_distancesCheckBox->isChecked(),
      m_ui->m_repositoryLineEdit->text().toStdString() + "_Attributes.txt" ) )
    {
      QMessageBox::information(this, "MultiSourceClassifier", "Output attributes file reation error");
      return;
    }
/*
		std::ofstream outFile( m_ui->m_repositoryLineEdit->text().toStdString() + "_Attributes.txt" );
		outFile << "ID SEGMENT";
		if ((m_algoInputParams.m_sourcesNumber == 1)
			||
			(((m_algoInputParams.m_sourcesNumber > 1)) && (algoInputParams.m_combinationType == "sum"))
			||
			((algoInputParams.m_sourcesNumber > 1) && (algoInputParams.m_combinationType == "minimum"))) {
			if (m_ui->m_distancesCheckBox->isChecked())
				outFile << "   " << "STOCHASTIC DISTANCE";
		}
		if (m_ui->m_statisticsCheckBox->isChecked())
			outFile << "   " << "STATISTICAL VALUE";
		if (m_ui->m_pValuesCheckBox->isChecked())
			outFile << "   " << "P-VALUE";
		outFile << std::endl;

		for (unsigned int i = 0; i < m_algoOutputParams.m_valuesS.size(); ++i) {

			if (algoInputParams.m_sourcesNumber == 1) {

				outFile << m_algoOutputParams.m_IDSegment[i];
				if (m_ui->m_distancesCheckBox->isChecked()) {
					outFile.precision(10);
					outFile << " " << m_algoOutputParams.m_distances[i];
				}
				if (m_ui->m_statisticsCheckBox->isChecked()) {
					outFile.precision(10);
					outFile << " " << m_algoOutputParams.m_valuesS[i];
				}
				if (m_ui->m_pValuesCheckBox->isChecked()) {
					outFile.precision(10);
					outFile << " " << m_algoOutputParams.m_valuesP[i];
				}
				outFile << std::endl;
			}

			if ((algoInputParams.m_combinationType == "sum") && (algoInputParams.m_sourcesNumber > 1)) {
				outFile << m_algoOutputParams.m_IDSegment[i];
				if (m_ui->m_distancesCheckBox->isChecked()) {
					outFile.precision(10);
					outFile << " " << m_algoOutputParams.m_distances[i];
				}
				if (m_ui->m_statisticsCheckBox->isChecked()) {
					outFile.precision(10);
					outFile << " " << m_algoOutputParams.m_valuesS[i];
				}
				if (m_ui->m_pValuesCheckBox->isChecked()) {
					outFile.precision(10);
					outFile << " " << m_algoOutputParams.m_pValuesSum[i];
				}
				outFile << std::endl;
			}

			if ((algoInputParams.m_combinationType == "Hellinger") && (algoInputParams.m_sourcesNumber > 1)) {
				outFile << m_algoOutputParams.m_IDSegment[i];
				if (m_ui->m_statisticsCheckBox->isChecked()) {
					outFile.precision(10);
					outFile << " " << m_algoOutputParams.m_valuesS[i];
				}
				outFile << std::endl;
			}

			if ((algoInputParams.m_combinationType == "multiplication") && (algoInputParams.m_sourcesNumber > 1)) {
				outFile << m_algoOutputParams.m_IDSegment[i];
				if (m_ui->m_statisticsCheckBox->isChecked()) {
					outFile.precision(10);
					outFile << " " << m_algoOutputParams.m_valuesS[i];
				}
				if (m_ui->m_pValuesCheckBox->isChecked()) {
					outFile.precision(10);
					outFile << " " << m_algoOutputParams.m_pValuesMultiplication[i];
				}
				outFile << std::endl;
			}

			if ((algoInputParams.m_combinationType == "minimum") && (algoInputParams.m_sourcesNumber > 1)) {
				outFile << m_algoOutputParams.m_IDSegment[i];
				if (m_ui->m_distancesCheckBox->isChecked()) {
					outFile.precision(10);
					outFile << " " << m_algoOutputParams.m_distances[i];
				}
				if (m_ui->m_statisticsCheckBox->isChecked()) {
					outFile.precision(10);
					outFile << " " << m_algoOutputParams.m_valuesS[i];
				}
				if (m_ui->m_pValuesCheckBox->isChecked()) {
					outFile.precision(10);
					outFile << " " << m_algoOutputParams.m_pValuesMinimum[i];
				}
				outFile << std::endl;
			}

			if ((algoInputParams.m_combinationType == "fuzzy") && (algoInputParams.m_sourcesNumber > 1)) {
				outFile << m_algoOutputParams.m_IDSegment[i];
				if (m_ui->m_statisticsCheckBox->isChecked()) {
					outFile.precision(10);
					outFile << " " << m_algoOutputParams.m_valuesS[i];
				}
				if (m_ui->m_pValuesCheckBox->isChecked()) {
					outFile.precision(10);
					outFile << " " << m_algoOutputParams.m_valuesP[i];
				}
				outFile << std::endl;
			}
		}

		outFile.close();*/
	}

	if (m_ui->m_perClassDistancesCheckBox->isChecked() || m_ui->m_perClassStatisticsCheckBox->isChecked()
		|| m_ui->m_perClassPValuesCheckBox->isChecked()) {

		typedef std::vector< std::vector< double > > Container;

		if (((m_ui->m_perClassDistancesCheckBox->isChecked()) && (m_algoInputParams.m_sourcesNumber == 1))
			||
			((m_ui->m_perClassDistancesCheckBox->isChecked()) && (m_algoInputParams.m_sourcesNumber > 1) &&
    (m_algoInputParams.m_combinationType == "sum"))
   ||
			((m_ui->m_perClassDistancesCheckBox->isChecked()) && (m_algoInputParams.m_sourcesNumber > 1) &&
    (m_algoInputParams.m_combinationType == "minimum"))) {
			std::ofstream outFile(m_algoOutputParams.m_rInfoDistancePerClass["URI"]);

			Container::const_iterator regionsIt = m_algoOutputParams.m_outDistancesPerClass.begin();
			const Container::const_iterator regionsItE = m_algoOutputParams.m_outDistancesPerClass.end();

			int region = 0;

			while (regionsIt != regionsItE) {

				outFile << m_algoOutputParams.m_IDSegment[region];

				for (unsigned int i = 0; i < m_algoInputParams.m_numberOfClassesOnTraining; ++i) {
					outFile.precision(10);
					outFile << "  " << regionsIt->operator[](i);
				}

				outFile << std::endl;

				++regionsIt;
				++region;
			}

			outFile.close();
		}

		if (m_ui->m_perClassStatisticsCheckBox->isChecked()) {
			std::ofstream outFile(m_algoOutputParams.m_rInfoStatisticPerClass["URI"]);

			Container::const_iterator regionsIt = m_algoOutputParams.m_outStatisticsPerClass.begin();
			const Container::const_iterator regionsItE = m_algoOutputParams.m_outStatisticsPerClass.end();

			int region = 0;

			while (regionsIt != regionsItE) {

				outFile << m_algoOutputParams.m_IDSegment[region];

				for (unsigned int i = 0; i < m_algoInputParams.m_numberOfClassesOnTraining; ++i) {
					outFile.precision(10);
					outFile << "  " << regionsIt->operator[](i);
				}

				outFile << std::endl;

				++regionsIt;
				++region;
			}

			outFile.close();
		}

		if (m_ui->m_perClassPValuesCheckBox->isChecked()) {
			std::ofstream outFile(m_algoOutputParams.m_rInfoPValuePerClass["URI"]);

			Container::const_iterator regionsIt = m_algoOutputParams.m_outPValuesPerClass.begin();
			const Container::const_iterator regionsItE = m_algoOutputParams.m_outPValuesPerClass.end();

			int region = 0;

			while (regionsIt != regionsItE) {

				outFile << m_algoOutputParams.m_IDSegment[region];

				for (unsigned int i = 0; i < m_algoInputParams.m_numberOfClassesOnTraining; ++i) {
					outFile.precision(10);
					outFile << "  " << regionsIt->operator[](i);
				}

				outFile << std::endl;

				++regionsIt;
				++region;
			}

			outFile.close();
		}

		if ((m_algoInputParams.m_sourcesNumber > 1) && (m_algoInputParams.m_combinationType == "fuzzy")) {

			std::map< std::string, std::string > info;
			info["URI"] = m_ui->m_repositoryLineEdit->text().toStdString() + "_ClassifiedImage.txt";
			std::ofstream outFile(info["URI"]);

			Container::const_iterator regionsIt = m_algoOutputParams.m_bandsFuzzy.begin();
			const Container::const_iterator regionsItE = m_algoOutputParams.m_bandsFuzzy.end();

			int region = 0;

			while (regionsIt != regionsItE) {

				outFile << m_algoOutputParams.m_IDSegment[region];

				for (unsigned int i = 0; i < m_algoInputParams.m_numberOfClassesOnTraining; ++i)
					outFile << "  " << regionsIt->operator[](i);

				outFile << std::endl;

				++regionsIt;
				++region;
			}

			outFile.close();
		}

		if ((m_algoInputParams.m_sourcesNumber > 1) && (m_algoInputParams.m_combinationType != "fuzzy")) {
		 std::map< std::string, std::string > info;
			info["URI"] = m_ui->m_repositoryLineEdit->text().toStdString() + "_ALL_MONO_DISTANCES_DATA.txt";
			std::ofstream outFile(info["URI"]);

			std::map< std::string, std::string > info2;
			info2["URI"] = m_ui->m_repositoryLineEdit->text().toStdString() + "_ALL_MONO_STATISTICS_DATA.txt";
			std::ofstream outFile2(info2["URI"]);

		 outFile << "ID SEGMENT";
		 outFile2 << "ID SEGMENT";

		 for (unsigned int j = 0; j < m_algoInputParams.m_sourcesNumber; ++j) {
		   outFile << "   " << "STOCHASTIC DISTANCES IMAGE " <<  j + 1;
			}
			for (unsigned int j = 0; j < m_algoInputParams.m_sourcesNumber; ++j) {
		  	outFile2 << "   " << "STATISTICAL VALUES IMAGE " <<  j + 1;
			}
		 outFile << std::endl;
		 outFile2 << std::endl;

			Container::const_iterator regionsIt = m_algoOutputParams.m_outAllDistances.begin();
			const Container::const_iterator regionsItE = m_algoOutputParams.m_outAllDistances.end();

			Container::const_iterator regions2It = m_algoOutputParams.m_outAllStatistics.begin();
			const Container::const_iterator regions2ItE = m_algoOutputParams.m_outAllStatistics.end();

			int region = 0;
			int k;

			while (regionsIt != regionsItE) {

				outFile << m_algoOutputParams.m_IDSegment[region];
				outFile2 << m_algoOutputParams.m_IDSegment[region];

				k = 0;
				for (unsigned int j = 0; j < m_algoInputParams.m_sourcesNumber; ++j) {
     for (unsigned int i = 0; i < m_algoInputParams.m_numberOfClassesOnTraining; ++i) {
      outFile.precision(10);
      outFile << "  " << regionsIt->operator[](k);
      ++k;
     }
				}

				k = 0;
				for (unsigned int j = 0; j < m_algoInputParams.m_sourcesNumber; ++j) {
     for (unsigned int i = 0; i < m_algoInputParams.m_numberOfClassesOnTraining; ++i) {
        outFile2.precision(10);
        outFile2 << "  " << regions2It->operator[](k);
        ++k;
				 }
    }

				outFile << std::endl;
				outFile2 << std::endl;

				++regionsIt;
				++regions2It;
				++region;
			}

			outFile.close();
			outFile2.close();
		}
	}

	// signal processing finished OK
	emit( processingFinishedOK() );

	if (status == 1) {
		QMessageBox::information(this, "", tr("Classification ended sucessfully."));
		return;
	}
}

void te::qt::widgets::MultiSourceClassifierFirstDialog::onCancelPushButtonClicked() {
	reject();
}

void te::qt::widgets::MultiSourceClassifierFirstDialog::setList(std::list<te::map::AbstractLayerPtr>& layerList)
{
  m_layerList = layerList;
}

void te::qt::widgets::MultiSourceClassifierFirstDialog::closeEvent(QCloseEvent* /*e*/)
{
  emit closeTool();
}

void te::qt::widgets::MultiSourceClassifierFirstDialog::getOutputLayers(
  te::map::AbstractLayerPtr& classImageLayer,
  te::map::AbstractLayerPtr& mapLayer,
  te::map::AbstractLayerPtr& mapPValueLayer )
{
  std::map< std::string, std::string > rInfoClassImage =
    m_algoOutputParams.m_rInfoClassImage;
  std::map< std::string, std::string > rInfoMap =
    m_algoOutputParams.m_rInfoMap;
  std::map< std::string, std::string > rInfoMapPValue =
    m_algoOutputParams.m_rInfoMapPValue;

  m_algoOutputParams.reset();

  classImageLayer = te::qt::widgets::createLayer("GDAL",
    rInfoClassImage );
  mapLayer = te::qt::widgets::createLayer("GDAL",
    rInfoMap );
  mapPValueLayer = te::qt::widgets::createLayer("GDAL",
    rInfoMapPValue );
}
