/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/widgets/rp/MultiSourceClassifierFirstDialog.h

  \brief A first dialog used to execute MultiSource image classification.
*/

#ifndef __TERRALIB_QT_WIDGETS_RP_INTERNAL_MULTISOURCECLASSIFIERFIRSTDIALOG_H
#define __TERRALIB_QT_WIDGETS_RP_INTERNAL_MULTISOURCECLASSIFIERFIRSTDIALOG_H

#include "../../Config.h"

// TerraLib
#include "../../../../raster/Raster.h"
#include "../../../../rp/radar/MultiSourceClassifier.h"
#include "../../../../maptools/AbstractLayer.h"

// Qt
#include <QDialog>

// STL
#include <map>
#include <string>
#include <vector>

// Boost
#include <boost/shared_ptr.hpp>

// Forward user interface declaration
namespace Ui
{
  class MultiSourceClassifierFirstDialogForm;
}

namespace te
{
  namespace qt
  {
    namespace widgets
    {
      /*!
        \class MultiSourceClassifierFirstDialog

        \brief A first dialog used to execute MultiSource image classification.
      */
      class TEQTWIDGETSEXPORT MultiSourceClassifierFirstDialog : public QDialog
      {
        Q_OBJECT

        public:

          /*!
            \brief Constructs a basic dialog which is a child of parent, with widget flags set to f.

            \param parent              Parent widget pointer.
            \param f                   Widget flags.
          */
          MultiSourceClassifierFirstDialog(
                           QWidget* parent = 0,
                           Qt::WindowFlags f = 0);

          /*! \brief Destructor. */
          ~MultiSourceClassifierFirstDialog();

          /*!
            \brief Returns the output result raster.

            \param outputRasterPtr A pointer to the output result raster

            \return true if OK, false on errors.
          */
          bool getOutputRaster(boost::shared_ptr< te::rst::Raster >& outputRasterPtr);

          void setList(std::list<te::map::AbstractLayerPtr>& layerList);

          void getOutputLayers( te::map::AbstractLayerPtr& classImageLayer,
            te::map::AbstractLayerPtr& mapLayer,
            te::map::AbstractLayerPtr& mapPValueLayer );

        public slots:

          void closeEvent(QCloseEvent* e);

        signals:

          void processingFinishedOK();

          void closeTool();

        protected slots:

          void onTargetFileSegImageToolButtonPressed();

          void onTargetFileTrainingToolButtonPressed();

          void onTargetFileTestToolButtonPressed();

          void onTargetFileRepositoryToolButtonPressed();

          void onOkPushButtonClicked();

          void onCancelPushButtonClicked();

          void onOneOrMultiSourceActivated();

          void onFuzzyChecked();

          void onSumChecked();

          void onMultiplicationChecked();

          void onMinimumChecked();

          void onHellingerChecked();

        private:

          Ui::MultiSourceClassifierFirstDialogForm* m_ui;            //!< User interface.
          bool m_toFile;

          std::vector< std::map<std::string, std::string> > m_rasterInfo;
          std::string m_outpuRasterDSType;                           //!< Output raster data source type (as described in te::rst::RasterFactory).
          std::map<std::string, std::string> m_outpuRasterInfo;      //!< The necessary information to create the raster (as described in te::rst::RasterFactory).
          boost::shared_ptr<te::rst::Raster> m_outputRasterPtr;      //!< Output raster pointer.
          std::list<te::map::AbstractLayerPtr> m_layerList;
          te::rp::radar::MultiSourceClassifier::InputParameters m_algoInputParams;
          te::map::AbstractLayerPtr m_outputLayer;
          te::rp::radar::MultiSourceClassifier::OutputParameters m_algoOutputParams;

          bool loadTrainingSamples();

          bool loadTestSamples();

          std::string getCombinationType();
      };

    } // end namespace widgets
  }   // end namespace qt
}     // end namespace te

#endif
