/*  Copyright (C) 2011-2012 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/rp/MultiSourceClassifierSecondDialog.cpp

  \brief A second dialog for MultiSourceClassifier operation
*/

// TerraLib
//#include "../Config.h"
//#include "../../../maptools/Utils.h"
// #include <terralib/common.h>
// #include <terralib/rp/Classifier.h>
// #include <terralib/rp/RasterAttributes.h>
// #include <terralib/raster.h>
// #include <terralib/dataaccess.h>
// #include <terralib/datatype/Property.h>
// #include <terralib/dataaccess/datasource/DataSourceFactory.h>
#include "../../../../rp/radar/MultiSourceClassifier.h"

#include "MultiSourceClassifierSecondDialog.h"
#include "ui/ui_MultiSourceClassifierSecondDialogForm.h"
//#include "MultiSourceClassifierSecondHelpDialog.h"
//#include <ui_MultiSourceClassifierSecondHelpDialogForm.h>
#include "../../utils/FileDialog.h"

// Qt
#include <QFileDialog>
#include <QList>
#include <QMessageBox>

// Boost
#include <boost/algorithm/string.hpp>
#include <boost/filesystem.hpp>
#include <boost/uuid/random_generator.hpp>
#include <boost/uuid/uuid_io.hpp>
#include <boost/foreach.hpp>
#include <boost/lexical_cast.hpp>

te::qt::widgets::MultiSourceClassifierSecondDialog::MultiSourceClassifierSecondDialog(QWidget* parent, Qt::WindowFlags f)
  : QDialog(parent, f),
    m_ui(new Ui::MultiSourceClassifierSecondDialogForm),
    m_toFile(false) {
// add controls
  m_ui->setupUi(this);

  // Validating input values

  QLocale lo(QLocale::C);
  lo.setNumberOptions(QLocale::RejectGroupSeparator);

  auto validator1 = new QDoubleValidator(0.00, 100000.00, 2, m_ui->m_looksNameLineEdit);
  validator1->setNotation(QDoubleValidator::StandardNotation);
  validator1->setLocale(lo);
  m_ui->m_looksNameLineEdit->setValidator(validator1);

  auto validator2 = new QDoubleValidator(0.00, 100000.00, 2, m_ui->m_orderComboBox);
  validator2->setNotation(QDoubleValidator::StandardNotation);
  validator2->setLocale(lo);
  m_ui->m_orderComboBox->setValidator(validator2);

  auto validator3 = new QDoubleValidator(0.00, 1.00, 2, m_ui->m_betaNameLineEdit);
  validator3->setNotation(QDoubleValidator::StandardNotation);
  validator3->setLocale(lo);
  m_ui->m_betaNameLineEdit->setValidator(validator3);

  m_ui->m_lcdNumber->display(m_numberOfImages);

  connect(m_ui->m_targetDirectoryImageToolButton, SIGNAL(clicked()), this,  SLOT(onTargetDirectoryImageToolButtonPressed()));

  connect(m_ui->m_okPushButton, SIGNAL(clicked()), this, SLOT(onOkPushButtonClicked()));

  connect(m_ui->m_cancelPushButton, SIGNAL(clicked()), this, SLOT(onCancelPushButtonClicked()));

  connect(m_ui->m_op1ModelRadioButton, SIGNAL(toggled(bool)), this, SLOT(onOpGaussianModelToggled()));
  connect(m_ui->m_op2ModelRadioButton, SIGNAL(toggled(bool)), this, SLOT(onOpGammaModelToggled()));
  connect(m_ui->m_op3ModelRadioButton, SIGNAL(toggled(bool)), this, SLOT(onOpWishartModelToggled()));
  connect(m_ui->m_op4ModelRadioButton, SIGNAL(toggled(bool)), this, SLOT(onOpIntesityPairModelToggled()));

  connect(m_ui->m_op1DistanceRadioButton, SIGNAL(toggled(bool)), this, SLOT(onOpBhattDistanceToggled()));
  connect(m_ui->m_op2DistanceRadioButton, SIGNAL(toggled(bool)), this, SLOT(onOpKLDistanceToggled()));
  connect(m_ui->m_op3DistanceRadioButton, SIGNAL(toggled(bool)), this, SLOT(onOpRenyiDIstanceToggled()));
  connect(m_ui->m_op4DistanceRadioButton, SIGNAL(toggled(bool)), this, SLOT(onOpHellingerDistanceToggled()));
  connect(m_ui->m_op5DistanceRadioButton, SIGNAL(toggled(bool)), this, SLOT(onOpChiDistanceToggled()));
  connect(m_ui->m_op6DistanceRadioButton, SIGNAL(toggled(bool)), this, SLOT(onOpTriangularDistanceToggled()));

    if (m_specialCaseHellinger == 1) {
    onSpecialHellingerCase();
  }

	m_ui->m_op1ModelRadioButton->setEnabled(true);
	m_ui->m_op2ModelRadioButton->setEnabled(true);
	m_ui->m_op3ModelRadioButton->setEnabled(true);
	m_ui->m_op4ModelRadioButton->setEnabled(true);

	  onOpGaussianModelToggled();
	  onOpBhattDistanceToggled();


  m_ui->m_orderComboBox->clear();

  m_ui->m_directoryImageNameLineEdit->setEnabled(false);

  m_ui->m_helpPushButton->setPageReference("plugins/rp/rp_radar_multisourceclassifier.html");
}

te::qt::widgets::MultiSourceClassifierSecondDialog::~MultiSourceClassifierSecondDialog() {
}

void te::qt::widgets::MultiSourceClassifierSecondDialog::onOkPushButtonClicked() {

	if (m_ui->m_directoryImageNameLineEdit->text().isEmpty()) {
		QMessageBox::information(this, "MultiSourceClassifier", "Input image is missing.");
		return;
	}

	if ((m_ui->m_orderComboBox->text().toDouble() - m_ui->m_orderComboBox->text().toInt()) != 0) {
		QMessageBox::information(this, "MultiSourceClassifier",
			"The covariance matrix order must be a natural number greater or equal to 2.");
		return;
	}

  if ((!m_ui->m_op1ModelRadioButton->isChecked()) && (!m_ui->m_op2ModelRadioButton->isChecked()) &&
     (!m_ui->m_op3ModelRadioButton->isChecked()) && (!m_ui->m_op4ModelRadioButton->isChecked())) {
    QMessageBox::information(this, "MultiSourceClassifier", "Choose a statistic model.");
    return;
  }

  if (m_ui->m_op1ModelRadioButton->isChecked() && m_ui->m_op2ModelRadioButton->isChecked() &&
     m_ui->m_op3ModelRadioButton->isChecked() && m_ui->m_op4ModelRadioButton->isChecked()) {
    QMessageBox::information(this, "MultiSourceClassifier", "Choose only one statistic model.");
    return;
  }

  m_rasterInfo["URI"] = m_ui->m_directoryImageNameLineEdit->text().toStdString();

  if (m_ui->m_op1ModelRadioButton->isChecked()) {
    m_model = "Gaussian";
    m_dataTypeWishart = "notWishart";
    m_orderDataWishart = 0;
    m_ENL = 0.0;

    if(!m_ui->m_op1DistanceRadioButton->isChecked() && !m_ui->m_op2DistanceRadioButton->isChecked()) {
      QMessageBox::information(this, "MultiSourceClassifier", "Choose a stochastic distance.");
      return;
    }
  }
  else if (m_ui->m_op2ModelRadioButton->isChecked()) {
    m_model = "Gamma";
    m_dataTypeWishart = "notWishart";
    m_orderDataWishart = 0;
    m_ENL = m_ui->m_looksNameLineEdit->text().toDouble();
    if(m_ui->m_looksNameLineEdit->text().isEmpty()) {
      QMessageBox::information(this, "MultiSourceClassifier", "Define the ENL value.");
      return;
    }

    if(!m_ui->m_op1DistanceRadioButton->isChecked() && !m_ui->m_op2DistanceRadioButton->isChecked() &&
       !m_ui->m_op3DistanceRadioButton->isChecked() && !m_ui->m_op4DistanceRadioButton->isChecked()) {
      QMessageBox::information(this, "MultiSourceClassifier", "Choose a stochastic distance.");
      return;
    }
  }
  else if (m_ui->m_op3ModelRadioButton->isChecked()) {
    m_model = "Wishart";

    m_orderDataWishart = m_ui->m_orderComboBox->text().toUInt();

    m_ENL = m_ui->m_looksNameLineEdit->text().toDouble();
    if(m_ui->m_looksNameLineEdit->text().isEmpty()) {
      QMessageBox::information(this, "MultiSourceClassifier", "Define the ENL value.");
      return;
    }

    if(m_ui->m_orderComboBox->text().isEmpty()) {
      QMessageBox::information(this, "MultiSourceClassifier", "Define the order of vector or matrix.");
      return;
    }
  }
  else if (m_ui->m_op4ModelRadioButton->isChecked()) {
    m_model = "IntensityPair";
    m_dataTypeWishart = "notWishart";
    m_orderDataWishart = 0;
    m_ENL = m_ui->m_looksNameLineEdit->text().toDouble();
    if(m_ui->m_looksNameLineEdit->text().isEmpty()) {
      QMessageBox::information(this, "MultiSourceClassifier", "Define the ENL value.");
      return;
    }

    if(!m_ui->m_op1DistanceRadioButton->isChecked() && !m_ui->m_op6DistanceRadioButton->isChecked()) {
      QMessageBox::information(this, "MultiSourceClassifier", "Choose a stochastic distance.");
      return;
    }
  }

  if (m_ui->m_op1DistanceRadioButton->isChecked()) {
    m_stochasticDistance = "Bhattacharyya";
    m_beta = 0;
  }
  else if (m_ui->m_op2DistanceRadioButton->isChecked()) {
    m_stochasticDistance = "KullbackLeibler";
    m_beta = 0;
  }
  else if (m_ui->m_op3DistanceRadioButton->isChecked()) {
    m_stochasticDistance = "Renyi";
    m_beta = m_ui->m_betaNameLineEdit->text().toDouble();

    if(m_ui->m_betaNameLineEdit->text().isEmpty()) {
      QMessageBox::information(this, "MultiSourceClassifier", "Define the beta value.");
      return;
    }

    if(m_beta < 0 || m_beta > 1) {
      QMessageBox::information(this, "MultiSourceClassifier", "Define a valid beta value: 0 <= beta <= 1.");
      return;
    }
  }
  else if (m_ui->m_op4DistanceRadioButton->isChecked()) {
    m_stochasticDistance = "Hellinger";
    m_beta = 0;
  }
  else if (m_ui->m_op5DistanceRadioButton->isChecked()) {
    m_stochasticDistance = "ChiSquare";
    m_beta = 0;
  }
  else if (m_ui->m_op6DistanceRadioButton->isChecked()) {
    m_stochasticDistance = "Triangular";
    m_beta = 0;
  }

  accept();
}

void te::qt::widgets::MultiSourceClassifierSecondDialog::onCancelPushButtonClicked() {
  reject();
}

void te::qt::widgets::MultiSourceClassifierSecondDialog::onTargetDirectoryImageToolButtonPressed()
{
  m_ui->m_directoryImageNameLineEdit->clear();

  te::qt::widgets::FileDialog fileDialog(this,
    te::qt::widgets::FileDialog::RASTER,
    "rp_radar_raster_processing_load_raster",
    te::qt::widgets::FileDialog::OPEN_FILE_MODE );

  try {
    fileDialog.exec();
  }
  catch (te::common::Exception& ex) {
    QMessageBox::warning(this, tr("File information"), ex.what());
    return;
  }

  std::string fileName = fileDialog.getPath();

  if (fileName.empty())
    return;

  boost::filesystem::path outfile(fileName);
  std::string aux = outfile.string();
  m_ui->m_directoryImageNameLineEdit->setText(aux.c_str());

  m_toFile = true;
  m_ui->m_directoryImageNameLineEdit->setEnabled(false);
}

void te::qt::widgets::MultiSourceClassifierSecondDialog::onOpIntesityPairModelToggled() {

  m_ui->m_op1DistanceRadioButton->setEnabled(true);
  m_ui->m_op2DistanceRadioButton->setEnabled(false);
  m_ui->m_op3DistanceRadioButton->setEnabled(false);
  m_ui->m_op4DistanceRadioButton->setEnabled(false);
  m_ui->m_op5DistanceRadioButton->setEnabled(false);
  m_ui->m_op6DistanceRadioButton->setEnabled(true);

  m_ui->m_label_looks->setEnabled(true);
  m_ui->m_looksNameLineEdit->setEnabled(true);

  m_ui->m_label_order->setEnabled(false);
  m_ui->m_orderComboBox->setEnabled(false);

  m_ui->m_targetDirectoryImageToolButton->setEnabled(true);
}

void te::qt::widgets::MultiSourceClassifierSecondDialog::onOpWishartModelToggled() {

  m_ui->m_op1DistanceRadioButton->setEnabled(true);
  m_ui->m_op2DistanceRadioButton->setEnabled(true);
  m_ui->m_op3DistanceRadioButton->setEnabled(true);
  m_ui->m_op4DistanceRadioButton->setEnabled(true);
  m_ui->m_op5DistanceRadioButton->setEnabled(true);
  m_ui->m_op6DistanceRadioButton->setEnabled(false);

  m_ui->m_label_looks->setEnabled(true);
  m_ui->m_looksNameLineEdit->setEnabled(true);

  m_ui->m_label_order->setEnabled(true);
  m_ui->m_orderComboBox->setEnabled(true);
}

void te::qt::widgets::MultiSourceClassifierSecondDialog::onOpGammaModelToggled() {

  m_ui->m_op1DistanceRadioButton->setEnabled(true);
  m_ui->m_op2DistanceRadioButton->setEnabled(true);
  m_ui->m_op3DistanceRadioButton->setEnabled(true);
  m_ui->m_op4DistanceRadioButton->setEnabled(true);
  m_ui->m_op5DistanceRadioButton->setEnabled(false);
  m_ui->m_op6DistanceRadioButton->setEnabled(false);

  m_ui->m_label_looks->setEnabled(true);
  m_ui->m_looksNameLineEdit->setEnabled(true);

  m_ui->m_label_order->setEnabled(false);
  m_ui->m_orderComboBox->setEnabled(false);

  m_ui->m_targetDirectoryImageToolButton->setEnabled(true);
}

void te::qt::widgets::MultiSourceClassifierSecondDialog::onOpGaussianModelToggled() {

  m_ui->m_op1DistanceRadioButton->setEnabled(true);
  m_ui->m_op2DistanceRadioButton->setEnabled(true);
  m_ui->m_op3DistanceRadioButton->setEnabled(false);
  m_ui->m_op4DistanceRadioButton->setEnabled(false);
  m_ui->m_op5DistanceRadioButton->setEnabled(false);
  m_ui->m_op6DistanceRadioButton->setEnabled(false);

  m_ui->m_label_looks->setEnabled(false);
  m_ui->m_looksNameLineEdit->setEnabled(false);

  m_ui->m_label_order->setEnabled(false);
  m_ui->m_orderComboBox->setEnabled(false);

  m_ui->m_targetDirectoryImageToolButton->setEnabled(true);
}

void te::qt::widgets::MultiSourceClassifierSecondDialog::onOpBhattDistanceToggled() {
  m_ui->m_label_beta->setEnabled(false);
  m_ui->m_betaNameLineEdit->setEnabled(false);
}

void te::qt::widgets::MultiSourceClassifierSecondDialog::onOpKLDistanceToggled() {
  m_ui->m_label_beta->setEnabled(false);
  m_ui->m_betaNameLineEdit->setEnabled(false);
}

void te::qt::widgets::MultiSourceClassifierSecondDialog::onOpRenyiDIstanceToggled() {
  m_ui->m_label_beta->setEnabled(true);
  m_ui->m_betaNameLineEdit->setEnabled(true);
}

void te::qt::widgets::MultiSourceClassifierSecondDialog::onOpHellingerDistanceToggled() {
  m_ui->m_label_beta->setEnabled(false);
  m_ui->m_betaNameLineEdit->setEnabled(false);
}

void te::qt::widgets::MultiSourceClassifierSecondDialog::onOpChiDistanceToggled() {
  m_ui->m_label_beta->setEnabled(false);
  m_ui->m_betaNameLineEdit->setEnabled(false);
}

void te::qt::widgets::MultiSourceClassifierSecondDialog::onOpTriangularDistanceToggled() {
  m_ui->m_label_beta->setEnabled(false);
  m_ui->m_betaNameLineEdit->setEnabled(false);
}

void te::qt::widgets::MultiSourceClassifierSecondDialog::onSpecialHellingerCase() {
  onOpWishartModelToggled();
  onOpHellingerDistanceToggled();
  m_ui->m_op1ModelRadioButton->setEnabled(false);
  m_ui->m_op2ModelRadioButton->setEnabled(true);
  m_ui->m_op3ModelRadioButton->setEnabled(true);
  m_ui->m_op4ModelRadioButton->setEnabled(false);
  m_ui->m_op1DistanceRadioButton->setEnabled(false);
  m_ui->m_op2DistanceRadioButton->setEnabled(false);
  m_ui->m_op3DistanceRadioButton->setEnabled(false);
  m_ui->m_op4DistanceRadioButton->setEnabled(true);
  m_ui->m_op5DistanceRadioButton->setEnabled(false);
  m_ui->m_op6DistanceRadioButton->setEnabled(false);
}
