/*  Copyright (C) 2011-2012 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

 /*!
  \file terralib/vp/qt/MultiSourceClassifierSecondDialog.h

  \brief A second dialog for MultiSourceClassifier operation
*/

#ifndef __TERRALIB_QT_WIDGETS_RP_INTERNAL_MULTISOURCECLASSIFIERSECONDDIALOG_H
#define __TERRALIB_QT_WIDGETS_RP_INTERNAL_MULTISOURCECLASSIFIERSECONDDIALOG_H

// TerraLib
#include "../../Config.h"

// STL
#include <list>
#include <map>
#include <memory>

// Qt
#include <QDialog>
#include <QComboBox>
#include <QToolButton>


// Forward user interface declaration
namespace Ui
{
  class MultiSourceClassifierSecondDialogForm;
}

namespace te
{
  namespace qt
  {
		  namespace widgets
		  {
      class TEQTWIDGETSEXPORT MultiSourceClassifierSecondDialog : public QDialog {

        Q_OBJECT

        public:

          enum MatrixComplexTypes
          {
           SCATTER_VECTOR,
           COVARIANCE_MATRIX
          };

          te::rst::Raster* m_inputRasterPointer1;

          std::map<std::string, std::string> m_rasterInfo;

          int m_numberOfImages;

          unsigned int m_parameters;

          std::string m_stochasticDistance;

          std::string m_model;

          std::string m_dataTypeWishart;

          int m_orderDataWishart;

          double m_beta;

          int m_specialCaseHellinger;

          int m_statusDistance;

          double m_ENL;

          std::auto_ptr< Ui::MultiSourceClassifierSecondDialogForm > m_ui;

          MultiSourceClassifierSecondDialog(QWidget* parent = 0, Qt::WindowFlags f = 0);

          ~MultiSourceClassifierSecondDialog();

        protected slots:

            void onTargetDirectoryImageToolButtonPressed();

            void onOkPushButtonClicked();

            void onCancelPushButtonClicked();

            void onOpGaussianModelToggled();

            void onOpGammaModelToggled();

            void onOpWishartModelToggled();

            void onOpIntesityPairModelToggled();

            void onOpBhattDistanceToggled();

            void onOpKLDistanceToggled();

            void onOpRenyiDIstanceToggled();

            void onOpHellingerDistanceToggled();

            void onOpChiDistanceToggled();

            void onOpTriangularDistanceToggled();

            void onSpecialHellingerCase();

        private:

          bool m_toFile;

//          std::list<te::map::AbstractLayerPtr> m_layerList;

          std::map<QComboBox*, int> m_cmbMap;
          std::map<QToolButton*, int> m_buttonMap;
          std::map<QLineEdit*, int> m_lineEditMap;
      };
		  }  // end namespace widgets
	 }  //end namespace qt
}  // end namespace te

#endif


