/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

 /*!
   \file terralib/qt/widgets/se/CheckableStylePropertyItem.cpp

   \brief A checkbox item for the style property tree
 */

#include "CheckboxStylePropertyItem.h"

#include "../propertybrowser/AbstractPropertyManager.h"

#include <cassert>

te::qt::widgets::CheckboxStylePropertyItem::CheckboxStylePropertyItem(QtTreePropertyBrowser* pb, const QString& propertyName, StylePropertyItemListener* listener, QColor c)
  : StylePropertyItem(pb, listener, c)
  , m_checkableProperty(nullptr)
{
  QtProperty* checkableProperty = te::qt::widgets::AbstractPropertyManager::getInstance().m_boolManager->addProperty(propertyName);
  setPropertyColor(checkableProperty, c);
  setCheckableProperty(checkableProperty);

  init(checkableProperty);
}

bool te::qt::widgets::CheckboxStylePropertyItem::isChecked() const
{
  assert(m_checkableProperty);
  
  if (m_checkableProperty == nullptr)
  {
    return false;
  }

  bool isChecked = te::qt::widgets::AbstractPropertyManager::getInstance().m_boolManager->value(m_checkableProperty);
  return isChecked;
}

void te::qt::widgets::CheckboxStylePropertyItem::setChecked(bool checked)
{
  assert(m_checkableProperty);

  if (m_checkableProperty == nullptr)
  {
    return;
  }

  m_enableListeners = false;

  bool currentValue = te::qt::widgets::AbstractPropertyManager::getInstance().m_boolManager->value(m_checkableProperty);
  if (currentValue != checked)
  {
    te::qt::widgets::AbstractPropertyManager::getInstance().m_boolManager->setValue(m_checkableProperty, checked);
  }

  setChildrenEnabled(checked);

  m_enableListeners = true;
}

void te::qt::widgets::CheckboxStylePropertyItem::valueChanged(QtProperty* p, bool value)
{
  if (m_enableListeners == false)
  {
    return;
  }

  if (p == m_checkableProperty)
  {
    setChecked(value);

    emit checkedStateChanged(this);
  }

  StylePropertyItem::valueChanged(p, value);
}

void te::qt::widgets::CheckboxStylePropertyItem::setCheckableProperty(QtProperty* qtProperty)
{
  m_checkableProperty = qtProperty;
  addListenerToProperty(qtProperty);
}
