/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

 /*!
   \file terralib/qt/widgets/se/StylePropertyItem.h

   \brief A bse item to represent the style items in property tree.
 */

#ifndef __TERRALIB_QT_WIDGETS_SE_INTERNAL_CHECKBOXSTYLEPROPERTYITEM_H
#define __TERRALIB_QT_WIDGETS_SE_INTERNAL_CHECKBOXSTYLEPROPERTYITEM_H

 // TerraLib
#include "StylePropertyItem.h"
#include "../Config.h"

namespace te
{
  namespace qt
  {
    namespace widgets
    {
      class StylePropertyItemListener;

      /*!
        \class StylePropertyItem

        \brief A widget used to define the main property items
              that can be used to describe a se object.

      */
      class TEQTWIDGETSEXPORT CheckboxStylePropertyItem : public StylePropertyItem
      {
        Q_OBJECT

      public:
        /** @name Initializer Methods
          *  Methods related to instantiation and destruction.
          */
          //@{

          /*! \brief Constructor */
        CheckboxStylePropertyItem(QtTreePropertyBrowser* pb, const QString& propertyName, StylePropertyItemListener* listener, QColor c = QColor());

        /*! \brief Destructor. */
        virtual ~CheckboxStylePropertyItem() = default;

        bool isChecked() const;

        void setChecked(bool checked);

      public slots:

        virtual void valueChanged(QtProperty* p, bool value);

      signals:

        void checkedStateChanged(CheckboxStylePropertyItem* stylePropertyItem);

      private:

        void setCheckableProperty(QtProperty* qtProperty);

      private:

        QtProperty* m_checkableProperty;

        //@}
      };
    }
  }
}

#endif //__TERRALIB_QT_WIDGETS_SE_INTERNAL_CHECKBOXSTYLEPROPERTYITEM_H
