/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/widgets/se/CreateRuleDialog.h

  \brief A dialog used to create a rule and define the filter restriction.
*/

#ifndef __TERRALIB_QT_WIDGETS_SE_INTERNAL_CREATERULEDIALOG_H
#define __TERRALIB_QT_WIDGETS_SE_INTERNAL_CREATERULEDIALOG_H

// TerraLib
#include "../Config.h"
#include "../query/QueryBuilderWidget.h"

#ifndef Q_MOC_RUN
#include "../../../maptools/AbstractLayer.h"
#include "../../../se/Rule.h"
#endif

// Qt
#include <QDialog>

// STL
#include <memory>

// Forward declaraion
namespace Ui { class CreateRuleDialogForm; }

namespace te
{
// Forward declarations
  namespace se
  {
    class Rule;
  }

  namespace qt
  {
    namespace widgets
    {
// Forward declarations

      /*!
        \class CreateRuleDialog

        \brief A dialog used to create a rule and define the filter restriction.
      */
      class TEQTWIDGETSEXPORT CreateRuleDialog : public QDialog
      {
        Q_OBJECT

        public:

          /** @name Initializer Methods
           *  Methods related to instantiation and destruction.
           */
          //@{

          /*! \brief Constructs a basic fill dialog which is a child of parent, with widget flags set to f. */
          CreateRuleDialog(QWidget* parent = 0, Qt::WindowFlags f = 0);

          /*! \brief Destructor. */
          ~CreateRuleDialog();

          //@}

        public:
          /*!
            \brief This method is used to set current layer

          */
          void setDataInformation(std::string dataSourceId, std::string dataSetName);

          void setRule(te::se::Rule* rule);

          te::se::Rule* getRule();

          bool isFilterGroupBoxCheckable();

          void setFilterGroupBoxCheckable(bool check);

          bool isFilterGroupBoxChecked();

          void setFilterGroupBoxChecked(bool check);

        protected slots:

          void onOkPushButtonClicked();


        private:

          te::qt::widgets::QueryBuilderWidget* m_queryWidget;

          te::gm::GeomType m_layerGeomType;

          std::unique_ptr<Ui::CreateRuleDialogForm> m_ui;    //!< Dialog form.

          te::se::Rule* m_rule;
      }; 

    } // end namespace widgets
  }   // end namespace qt
}     // end namespace te

#endif  // __TERRALIB_QT_WIDGETS_SE_INTERNAL_CREATERULEDIALOG_H
