/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

 /*!
   \file terralib/qt/widgets/se/FillPropertyItem.cpp

   \brief A widget used to define the fill se object.
 */

#include "FillPropertyItem.h"

#include "BasicFillPropertyItem.h"
#include "GraphicFillPropertyItem.h"
#include "GroupPropertyItemHandler.h"

// TerraLib Includes
#include "../propertybrowser/AbstractPropertyManager.h"
#include "../Utils.h"
#include "../../../color/RGBAColor.h"
#include "../../../common/MathUtils.h"
#include "../../../se/Fill.h"
#include "../../../se/ParameterValue.h"
#include "../../../se/SvgParameter.h"
#include "../../../se/Utils.h"

te::qt::widgets::FillPropertyItem::FillPropertyItem(QtTreePropertyBrowser* propertyBrowser, StylePropertyItemListener* listener, QColor c)
  : te::qt::widgets::StylePropertyItem(propertyBrowser, listener, c)
  , m_groupHandler(new GroupPropertyItemHandler())
  , m_graphicFillProperty(nullptr)
{
  //add property to tree propertybrowser
  QtProperty* itemProperty = te::qt::widgets::AbstractPropertyManager::getInstance().m_groupManager->addProperty(tr("Fill Property"));
  addProperty(itemProperty, QColor(255, 255, 125));

  //basic 
  m_basicFillProperty = new te::qt::widgets::BasicFillPropertyItem(itemProperty, propertyBrowser, listener, QColor(255, 255, 175));
  m_groupHandler->addPropertyToGroup(m_basicFillProperty);

  //graphic
  m_graphicFillProperty = new te::qt::widgets::GraphicFillPropertyItem(itemProperty, propertyBrowser, listener, QColor(255, 255, 175), QColor(255, 255, 225));
  m_groupHandler->addPropertyToGroup(m_graphicFillProperty);

  setFill(nullptr);

  init(itemProperty);
}

te::qt::widgets::FillPropertyItem::~FillPropertyItem()
{
  delete m_groupHandler;
  delete m_basicFillProperty;
  delete m_graphicFillProperty;
}

bool te::qt::widgets::FillPropertyItem::setFill(const te::se::Fill* fill)
{
  const te::se::Graphic* graphicFill = nullptr;
  if (fill != nullptr)
  {
    graphicFill = fill->getGraphicFill();
  }

  m_enableListeners = false;

  //Graphic Fill
  bool enableBasicFill = (fill != nullptr) && (graphicFill == nullptr);
  bool enableGraphicFill = (fill != nullptr) && (graphicFill != nullptr);

  m_basicFillProperty->setFill(fill);
  m_graphicFillProperty->setGraphic(graphicFill);

  m_basicFillProperty->setChecked(enableBasicFill);
  m_basicFillProperty->setExpandedStatus(enableBasicFill, false);

  m_graphicFillProperty->setChecked(enableGraphicFill);
  m_graphicFillProperty->setExpandedStatus(enableGraphicFill, false);

  m_enableListeners = true;

  return true;
}

te::se::Fill* te::qt::widgets::FillPropertyItem::getFill() const
{
  //we always return a new instance
  te::se::Fill* newItem = nullptr;
  if (m_basicFillProperty->isChecked() == true)
  {
    newItem = m_basicFillProperty->getFill();
  }
  else if(m_graphicFillProperty->isChecked() == true)
  {
    newItem = new te::se::Fill();
    newItem->setGraphicFill(m_graphicFillProperty->getGraphic());
  }

  return newItem;
}
