/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

 /*!
   \file terralib/qt/widgets/se/FillPropertyItem.h

   \brief A widget used to define the fill se object.
 */

#ifndef __TERRALIB_QT_WIDGETS_SE_INTERNAL_FILLPROPERTYITEM_H
#define __TERRALIB_QT_WIDGETS_SE_INTERNAL_FILLPROPERTYITEM_H

 // TerraLib
#include "../Config.h"
//#include "../propertybrowser/AbstractPropertyItem.h"

#include "StylePropertyItem.h"

#include <QColor>

class QtProperty;
class QtTreePropertyBrowser;

namespace te
{
  // Forward declarations
  namespace se
  {
    class Fill;
  }

  namespace qt
  {
    namespace widgets
    {
      class AbstractPropertyItem;
      class BasicFillPropertyItem;
      class GroupPropertyItemHandler;
      class GraphicFillPropertyItem;

      /*!
        \class FillPropertyItem

        \brief A widget used to define the fill se object.

          - Basic
            - Color
            - Opacity
          - Graphic

      */

      class TEQTWIDGETSEXPORT FillPropertyItem : public te::qt::widgets::StylePropertyItem
      {
        Q_OBJECT

      public:
        /** @name Initializer Methods
          *  Methods related to instantiation and destruction.
          */
        //@{

          /*! \brief Constructor */
        FillPropertyItem(QtTreePropertyBrowser* propertyBrowser, StylePropertyItemListener* listener, QColor c = QColor());

        /*! \brief Destructor. */
        ~FillPropertyItem();

        //@}

        bool setFill(const te::se::Fill* fill);

        te::se::Fill* getFill() const;

      protected:

        GroupPropertyItemHandler* m_groupHandler;
        
        te::qt::widgets::BasicFillPropertyItem* m_basicFillProperty;
        te::qt::widgets::GraphicFillPropertyItem* m_graphicFillProperty;
      };
    }
  }
}

#endif //__TERRALIB_QT_WIDGETS_SE_INTERNAL_FILLPROPERTYITEM_H
