/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

 /*!
   \file terralib/qt/widgets/se/GraphicFillPropertyItem.h

   \brief A widget used to define the graphic fill se object.
 */

#ifndef __TERRALIB_QT_WIDGETS_SE_INTERNAL_GRAPHICFILLPROPERTYITEM_H
#define __TERRALIB_QT_WIDGETS_SE_INTERNAL_GRAPHICFILLPROPERTYITEM_H

 // TerraLib
#include "CheckboxStylePropertyItem.h"
#include "../Config.h"
#include "../propertybrowser/AbstractPropertyItem.h"

namespace te
{
  // Forward declarations
  namespace se
  {
    class Graphic;
  }

  namespace qt
  {
    namespace widgets
    {
      class GroupPropertyItemHandler;
      class MarkWellKnownPropertyItem;
      class MarkGlyphPropertyItem;
      class MarkExternalGraphicPropertyItem;

      /*!
        \class GraphicFillPropertyItem

        \brief A widget used to define the graphic fill se object.

          - opacity
          - size
          - rotation
          - well known
          - glyph
          - external graphic

      */

      class TEQTWIDGETSEXPORT GraphicFillPropertyItem : public te::qt::widgets::CheckboxStylePropertyItem
      {
        Q_OBJECT

      public:
        /** @name Initializer Methods
          *  Methods related to instantiation and destruction.
          */
        //@{

            /*! \brief Constructor */
        GraphicFillPropertyItem(QtProperty* parent, QtTreePropertyBrowser* propertyBrowser, StylePropertyItemListener* listener, QColor color, QColor subColor, QString caption = tr("Graphic Fill"));

        /*! \brief Destructor. */
        virtual ~GraphicFillPropertyItem();

        //@}

        bool setGraphic(const te::se::Graphic* graphic);

        te::se::Graphic* getGraphic() const;

      protected slots:
        void valueChanged(QtProperty* p, double value);

      protected:

        GroupPropertyItemHandler* m_groupHandler;

        QtProperty* m_sizeProperty;
        QtProperty* m_angleProperty;
        QtProperty* m_opacityProperty;

        te::qt::widgets::MarkWellKnownPropertyItem* m_markWellKnownProperty;
        te::qt::widgets::MarkGlyphPropertyItem* m_markGlyphProperty;
        te::qt::widgets::MarkExternalGraphicPropertyItem* m_markExternalGraphicProperty;
      };
    }
  }
}

#endif //__TERRALIB_QT_WIDGETS_SE_INTERNAL_GRAPHICFILLPROPERTYITEM_H
