/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

 /*!
   \file terralib/qt/widgets/se/GraphicStrokePropertyItem.cpp

   \brief A widget used to define the graphic stroke se object.
 */

#include "GraphicStrokePropertyItem.h"

#include "GraphicFillPropertyItem.h"

 // TerraLib Includes
#include "../Utils.h"
#include "../propertybrowser/AbstractPropertyManager.h"
#include "../../../se/Graphic.h"
#include "../../../se/GraphicStroke.h"
#include "../../../se/ParameterValue.h"
#include "../../../se/Utils.h"

te::qt::widgets::GraphicStrokePropertyItem::GraphicStrokePropertyItem(QtProperty* parent, QtTreePropertyBrowser* propertyBrowser, StylePropertyItemListener* listener, QColor color, QColor subColor)
  : te::qt::widgets::GraphicFillPropertyItem(parent, propertyBrowser, listener, color, subColor, tr("Graphic Stroke"))
  , m_initialGapProperty(nullptr)
  , m_gapProperty(nullptr)
{
  //Initial Gap
  m_initialGapProperty = te::qt::widgets::AbstractPropertyManager::getInstance().m_intManager->addProperty(tr("Initial Gap"));
  addListenerToProperty(m_initialGapProperty);

  //Gap  
  m_gapProperty = te::qt::widgets::AbstractPropertyManager::getInstance().m_intManager->addProperty(tr("Gap"));
  addListenerToProperty(m_gapProperty);
  
  QtProperty* graphicPropertyItem = this->getItemProperty();
  graphicPropertyItem->addSubProperty(m_initialGapProperty);
  graphicPropertyItem->addSubProperty(m_gapProperty);

  setGraphicStroke(nullptr);
}

te::qt::widgets::GraphicStrokePropertyItem::~GraphicStrokePropertyItem()
{
  delete m_initialGapProperty;
  delete m_gapProperty;
}

bool te::qt::widgets::GraphicStrokePropertyItem::setGraphicStroke(const te::se::GraphicStroke* graphicStroke)
{
  const te::se::Graphic* graphic = nullptr;
  double initialGap = 0.;
  double gap = 20.;
  if (graphicStroke != nullptr)
  {
    graphic = graphicStroke->getGraphic();

    const te::se::ParameterValue* initialGapParameter = graphicStroke->getInitialGap();
    if (initialGapParameter != nullptr)
    {
      initialGap = te::se::GetDouble(initialGapParameter);
    }

    const te::se::ParameterValue* gapParameter = graphicStroke->getGap();
    if (gapParameter != nullptr)
    {
      gap = te::se::GetDouble(gapParameter);
    }
  }

  //Graphic
  this->setGraphic(graphic);

  m_enableListeners = false;

  //Initial Gap
  te::qt::widgets::AbstractPropertyManager::getInstance().m_intManager->setValue(m_initialGapProperty, initialGap);

  //Gap
  te::qt::widgets::AbstractPropertyManager::getInstance().m_intManager->setValue(m_gapProperty, gap);

  m_enableListeners = true;

  return true;
}

te::se::GraphicStroke* te::qt::widgets::GraphicStrokePropertyItem::getGraphicStroke() const
{
  te::se::Graphic* graphic = this->getGraphic();
  int initialGap = te::qt::widgets::AbstractPropertyManager::getInstance().m_intManager->value(m_initialGapProperty);
  int gap = te::qt::widgets::AbstractPropertyManager::getInstance().m_intManager->value(m_gapProperty);
  
  te::se::GraphicStroke* newItem = new te::se::GraphicStroke();
  newItem->setGraphic(graphic);
  newItem->setInitialGap(new te::se::ParameterValue(Convert2StdString(QString::number(initialGap, 'f', 2))));
  newItem->setGap(new te::se::ParameterValue(Convert2StdString(QString::number(gap, 'f', 2))));

  return newItem;
}
