/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

 /*!
   \file terralib/qt/widgets/se/GraphicStrokePropertyItem.h

   \brief A widget used to define the graphic stroke se object.
 */

#ifndef __TERRALIB_QT_WIDGETS_SE_INTERNAL_GRAPHICSTROKEPROPERTYITEM_H
#define __TERRALIB_QT_WIDGETS_SE_INTERNAL_GRAPHICSTROKEPROPERTYITEM_H

 // TerraLib
#include "GraphicFillPropertyItem.h"
#include "../Config.h"

 // Qt
#include <QObject>

namespace te
{
  // Forward declarations
  namespace se
  {
    class GraphicStroke;
  }

  namespace qt
  {
    namespace widgets
    {
      class GraphicFillPropertyItem;

      /*!
        \class GraphicStrokePropertyItem

        \brief A widget used to define the graphic stroke se object.

          - initial gap
          - gap
          - graphic

      */

      class TEQTWIDGETSEXPORT GraphicStrokePropertyItem : public te::qt::widgets::GraphicFillPropertyItem
      {
        Q_OBJECT

      public:
        /** @name Initializer Methods
          *  Methods related to instantiation and destruction.
          */
          //@{

              /*! \brief Constructor */
        GraphicStrokePropertyItem(QtProperty* parent, QtTreePropertyBrowser* propertyBrowser, StylePropertyItemListener* listener, QColor color, QColor subColor);

        /*! \brief Destructor. */
        ~GraphicStrokePropertyItem();

        //@}

        void setEnabled(bool enable);

        bool setGraphicStroke(const te::se::GraphicStroke* graphic);

        te::se::GraphicStroke* getGraphicStroke() const;

      protected:

        QtProperty* m_gapProperty;
        QtProperty* m_initialGapProperty;
      };
    }
  }
}

#endif //__TERRALIB_QT_WIDGETS_SE_INTERNAL_GRAPHICSTROKEPROPERTYITEM_H
