/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

 /*!
   \file terralib/qt/widgets/se/CheckableStylePropertyItem.cpp

   \brief A checkbox item for the style property tree
 */

#include "GroupPropertyItemHandler.h"
#include "CheckboxStylePropertyItem.h"

#include "../propertybrowser/AbstractPropertyManager.h"

te::qt::widgets::GroupPropertyItemHandler::GroupPropertyItemHandler()
  : QObject()
{
  
}

void te::qt::widgets::GroupPropertyItemHandler::addPropertyToGroup(CheckboxStylePropertyItem* stylePropertyItem)
{
  connect(stylePropertyItem, SIGNAL(checkedStateChanged(CheckboxStylePropertyItem*)), this, SLOT(onCheckedStateChanged(CheckboxStylePropertyItem*)));
  m_vecCheckboxProperties.push_back(stylePropertyItem);
}

void te::qt::widgets::GroupPropertyItemHandler::onCheckedStateChanged(CheckboxStylePropertyItem* stylePropertyItem)
{
  //if the item has been unchecked, we dont need to do anything
  if (stylePropertyItem->isChecked() == false)
  {
    return;
  }

  //if the item was checked, we must uncheck the other items of the group
  for (std::size_t i = 0; i < m_vecCheckboxProperties.size(); ++i)
  {
    CheckboxStylePropertyItem* otherStylePropertyItem = m_vecCheckboxProperties.at(i);
    if (stylePropertyItem == otherStylePropertyItem)
    {
      otherStylePropertyItem->setChecked(true);
    }
    else
    {
      otherStylePropertyItem->setChecked(false);
    }
  }
}
