/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
\file terralib/qt/widgets/se/HaloPropertyItem.cpp

  \brief A widget used to define the halo se object.
*/


#include "../../../common/StringUtils.h"
#include "../../../se/SvgParameter.h"
#include "../../../se/Fill.h"
#include "../../../se/Halo.h"
#include "../../../se/Utils.h"
#include "../propertybrowser/AbstractPropertyManager.h"
#include "../Utils.h"
#include "BasicFillPropertyItem.h"
#include "HaloPropertyItem.h"

// STL
#include <cassert>

// QtPropertyBrowser


te::qt::widgets::HaloPropertyItem::HaloPropertyItem(QtTreePropertyBrowser* pb, StylePropertyItemListener* listener, QColor c)
  : te::qt::widgets::CheckboxStylePropertyItem(pb, tr("Halo Properties"), listener, c)
  , m_basicFillPropertyItem(nullptr)
  , m_radiusProperty(nullptr)
{
  //build halo property browser
  QtProperty* itemProperty = this->getItemProperty();
  addProperty(itemProperty, QColor(152, 252, 36));

  //label
  QStringList emptyList;
  m_radiusProperty = te::qt::widgets::AbstractPropertyManager::getInstance().m_intManager->addProperty(tr("Radius"));
  te::qt::widgets::AbstractPropertyManager::getInstance().m_intManager->setMinimum(m_radiusProperty, 0);
  te::qt::widgets::AbstractPropertyManager::getInstance().m_intManager->setMaximum(m_radiusProperty, 99);
  addListenerToProperty(m_radiusProperty);
  
  itemProperty->addSubProperty(m_radiusProperty);
  m_basicFillPropertyItem = new te::qt::widgets::BasicFillPropertyItem(itemProperty, m_propertyBrowser, listener, QColor(152, 252, 36));

  // Setups initial halo
  setHalo(nullptr);
}

te::qt::widgets::HaloPropertyItem::~HaloPropertyItem()
{
  delete m_radiusProperty;
  delete m_basicFillPropertyItem;  
}

void te::qt::widgets::HaloPropertyItem::setHalo(const te::se::Halo* halo)
{
  int radius = 8;
  const te::se::Fill* fill = nullptr;

  if (halo != nullptr)
  {
    const te::se::ParameterValue* radiusParam = halo->getRadius();
    if (radiusParam != nullptr)
    {
      radius = te::se::GetInt(radiusParam);
    }

    fill = halo->getFill();
  }
  
  m_enableListeners = false;

  //Radius
  te::qt::widgets::AbstractPropertyManager::getInstance().m_intManager->setValue(m_radiusProperty, radius);

  //Fill
  m_basicFillPropertyItem->setFill(fill);

  m_enableListeners = true;
}

te::se::Halo* te::qt::widgets::HaloPropertyItem::getHalo() const
{
  int radius = te::qt::widgets::AbstractPropertyManager::getInstance().m_intManager->value(m_radiusProperty);
  te::se::Fill* fill = nullptr;

  if (m_basicFillPropertyItem->isChecked())
  {
    fill = m_basicFillPropertyItem->getFill();
  }

  te::se::Halo* newItem = new te::se::Halo();
  newItem->setRadius(new te::se::ParameterValue(Convert2StdString(QString::number(radius))));
  newItem->setFill(fill);

  return newItem;
}
