/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
\file terralib/qt/widgets/se/LineGapPropertyItem.cpp

\brief A widget used to define the gap between marks.
*/


#include "../../../common/StringUtils.h"
#include "../../../se/SvgParameter.h"
#include "../../../se/Utils.h"
#include "../propertybrowser/AbstractPropertyManager.h"
#include "LineGapPropertyItem.h"

// STL
#include <cassert>


te::qt::widgets::LineGapPropertyItem::LineGapPropertyItem(QtTreePropertyBrowser* pb, QColor c) 
  : te::qt::widgets::AbstractPropertyItem(pb, c)
  , m_gap(nullptr)
  , m_lineGap(5.0)
  , m_update(false)
{
  //offset
  m_gap = te::qt::widgets::AbstractPropertyManager::getInstance().m_doubleManager->addProperty(tr("Gap"));
  te::qt::widgets::AbstractPropertyManager::getInstance().m_doubleManager->setValue(m_gap, m_lineGap);
  
  addProperty(m_gap, QColor(175, 255, 175));

  m_update = true;
}

te::qt::widgets::LineGapPropertyItem::~LineGapPropertyItem()
{
  delete m_gap;
}

void te::qt::widgets::LineGapPropertyItem::valueChanged(QtProperty* p, double value)
{
  if (!m_update)
    return;

  if (p == m_gap)
  {
    m_update = false;

    m_lineGap = value;
    
    emit lineGapChanged();
    m_update = true;
  }
}

void te::qt::widgets::LineGapPropertyItem::updateUi()
{    
  te::qt::widgets::AbstractPropertyManager::getInstance().m_doubleManager->setValue(m_gap, m_lineGap);
}

void te::qt::widgets::LineGapPropertyItem::setLineGap(double gap)
{
  m_lineGap = gap;

  updateUi();
}
  
double te::qt::widgets::LineGapPropertyItem::getLineGap() const
{
  return m_lineGap;
}

