/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/widgets/se/LineGapPropertyItem.h

  \brief A widget used to define the gap between marks.
*/

#ifndef __TERRALIB_QT_WIDGETS_SE_INTERNAL_LINE_GAP_PROPERTYITEM_H 
#define __TERRALIB_QT_WIDGETS_SE_INTERNAL_LINE_GAP_PROPERTYITEM_H

// TerraLib
#include "../Config.h"
#include "../propertybrowser/AbstractPropertyItem.h"

namespace te
{
// Forward declarations
  namespace qt
  {
    namespace widgets
    {
      /*!
        \class LineGapPropertyItem

        \brief A widget used to define the gap between marks.

      */

      class TEQTWIDGETSEXPORT LineGapPropertyItem : public te::qt::widgets::AbstractPropertyItem
      {
        Q_OBJECT

        public:
          /** @name Initializer Methods
            *  Methods related to instantiation and destruction.
            */
          //@{

          /*! \brief Constructor */
          LineGapPropertyItem(QtTreePropertyBrowser* pb, QColor c = QColor());

          /*! \brief Destructor. */
          ~LineGapPropertyItem();

          /*!
            \brief Updates the widget form based on internal gap element.
            
            \param gap The gap between marks.
          */
          void setLineGap(double gap);

          /*!
            \brief Get internal gap element value.
            
            \return The gap between marks.
          */
          double getLineGap() const;
          
          /*! \brief Updates the widget form based on internal gap element. */
          void updateUi();

        public slots:

          virtual void valueChanged(QtProperty* p, double value);

        signals:

          /*! This signal is emitted when the internal gap element is changed. */
          void lineGapChanged();

        protected:

          QtProperty* m_gap;

          double m_lineGap;
          
          bool m_update;
      };
    }
  }
}

#endif //__TERRALIB_QT_WIDGETS_SE_INTERNAL_LINE_GAP_PROPERTYITEM_H