/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
\file terralib/qt/widgets/se/LineInitialGapPropertyItem.cpp

\brief A widget used to define the Initial Gap to draw marks in a line.
*/


#include "../../../common/StringUtils.h"
#include "../../../se/SvgParameter.h"
#include "../../../se/Utils.h"
#include "../propertybrowser/AbstractPropertyManager.h"
#include "LineInitialGapPropertyItem.h"

// STL
#include <cassert>


te::qt::widgets::LineInitialGapPropertyItem::LineInitialGapPropertyItem(QtTreePropertyBrowser* pb, QColor c) 
  : te::qt::widgets::AbstractPropertyItem(pb, c)
  , m_initialGap(nullptr)
  , m_lineInitialGap(0.0)
  , m_update(false)
{
  //initial gap
  m_initialGap = te::qt::widgets::AbstractPropertyManager::getInstance().m_doubleManager->addProperty(tr("Initial Gap"));
  te::qt::widgets::AbstractPropertyManager::getInstance().m_doubleManager->setValue(m_initialGap, m_lineInitialGap);
  
  addProperty(m_initialGap, QColor(175, 255, 175));

  m_update = true;
}

te::qt::widgets::LineInitialGapPropertyItem::~LineInitialGapPropertyItem()
{
  delete m_initialGap;
}

void te::qt::widgets::LineInitialGapPropertyItem::valueChanged(QtProperty* p, double value)
{
  if (!m_update)
    return;

  if (p == m_initialGap)
  {
    m_update = false;

    m_lineInitialGap = value;
    
    emit lineInitialGapChanged();
    m_update = true;
  }
}

void te::qt::widgets::LineInitialGapPropertyItem::updateUi()
{    
  te::qt::widgets::AbstractPropertyManager::getInstance().m_doubleManager->setValue(m_initialGap, m_lineInitialGap);
}

void te::qt::widgets::LineInitialGapPropertyItem::setLineInitialGap(double initialGap)
{
  m_lineInitialGap = initialGap;

  updateUi();
}
  
double te::qt::widgets::LineInitialGapPropertyItem::getLineInitialGap() const
{
  return m_lineInitialGap;
}

