/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/widgets/se/LineInitialGapPropertyItem.h

  \brief A widget used to define the Initial Gap to draw marks in a line.
*/

#ifndef __TERRALIB_QT_WIDGETS_SE_INTERNAL_LINE_INITIALGAP_PROPERTYITEM_H 
#define __TERRALIB_QT_WIDGETS_SE_INTERNAL_LINE_INITIALGAP_PROPERTYITEM_H

// TerraLib
#include "../Config.h"
#include "../propertybrowser/AbstractPropertyItem.h"

namespace te
{
  namespace qt
  {
    namespace widgets
    {
      /*!
        \class LineInitialGapPropertyItem

        \brief A widget used to define the Initial Gap to draw marks in a line.

      */

      class TEQTWIDGETSEXPORT LineInitialGapPropertyItem : public te::qt::widgets::AbstractPropertyItem
      {
        Q_OBJECT

        public:
          /** @name Initializer Methods
            *  Methods related to instantiation and destruction.
            */
          //@{

          /*! \brief Constructor */
          LineInitialGapPropertyItem(QtTreePropertyBrowser* pb, QColor c = QColor());

          /*! \brief Destructor. */
          ~LineInitialGapPropertyItem();

          //@}

          /*!
            \brief Updates the widget form based on internal initial gap element.
            
            \param initialGap The initial gap to draw marks in a line.
          */
          void setLineInitialGap(double initialGap);

          /*!
            \brief Get internal initial gap element value.
            
            \return The initial gap to draw marks in a line.
          */
          double getLineInitialGap() const;
          
          /*! \brief Updates the widget form based on internal initial gap element. */
          void updateUi();

        public slots:

          virtual void valueChanged(QtProperty* p, double value);

        signals:

          /*! This signal is emitted when the internal initial gap element is changed. */
          void lineInitialGapChanged();

        protected:

          QtProperty* m_initialGap;

          double m_lineInitialGap;
          
          bool m_update;
      };
    }
  }
}

#endif //__TERRALIB_QT_WIDGETS_SE_INTERNAL_LINE_INITIALGAP_PROPERTYITEM_H