/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
\file terralib/qt/widgets/se/LinePerpendicularOffsetPropertyItem.cpp

\brief A widget used to define the Perpendicular Offset between a line and marks.
*/


#include "../../../common/StringUtils.h"
#include "../../../se/SvgParameter.h"
#include "../../../se/Utils.h"
#include "../propertybrowser/AbstractPropertyManager.h"
#include "LinePerpendicularOffsetPropertyItem.h"

// STL
#include <cassert>


te::qt::widgets::LinePerpendicularOffsetPropertyItem::LinePerpendicularOffsetPropertyItem(QtTreePropertyBrowser* pb, StylePropertyItemListener* listener, QColor c)
  : te::qt::widgets::StylePropertyItem(pb, listener, c)
{
  //offset
  QtProperty* perpendicularOffset = te::qt::widgets::AbstractPropertyManager::getInstance().m_doubleManager->addProperty(tr("Offset")); 
  addProperty(perpendicularOffset, QColor(175, 255, 175));

  init(perpendicularOffset);
}

te::qt::widgets::LinePerpendicularOffsetPropertyItem::~LinePerpendicularOffsetPropertyItem()
{
}

void te::qt::widgets::LinePerpendicularOffsetPropertyItem::setLinePerpendicularOffset(double offset)
{
  m_enableListeners = false;

  te::qt::widgets::AbstractPropertyManager::getInstance().m_doubleManager->setValue(m_itemProperty, offset);

  m_enableListeners = true;
}
  
double te::qt::widgets::LinePerpendicularOffsetPropertyItem::getLinePerpendicularOffset() const
{
  double offset = te::qt::widgets::AbstractPropertyManager::getInstance().m_doubleManager->value(m_itemProperty);
  return offset;
}
