/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
\file terralib/qt/widgets/se/LinePlacementPropertyItem.cpp

\brief A widget used to define the LinePlacement se object.
*/


#include "../../../common/StringUtils.h"
#include "../../../se/SvgParameter.h"
#include "../../../se/LinePlacement.h"
#include "../../../se/Utils.h"
#include "../propertybrowser/AbstractPropertyManager.h"
#include "../Utils.h"
#include "LinePlacementPropertyItem.h"

te::qt::widgets::LinePlacementPropertyItem::LinePlacementPropertyItem(QtTreePropertyBrowser* pb, StylePropertyItemListener* listener, QColor c)
  : te::qt::widgets::CheckboxStylePropertyItem(pb, tr("Line Placement"), listener, c)
  , m_isAligned(nullptr)
  , m_perpendicularOffset(nullptr)
{
  //add property to tree propertybrowser
  QtProperty* itemProperty = this->getItemProperty();
  addProperty(itemProperty, QColor(241, 165, 61));

  //offset
  m_perpendicularOffset = te::qt::widgets::AbstractPropertyManager::getInstance().m_doubleManager->addProperty(tr("Offset"));
  addListenerToProperty(m_perpendicularOffset);

  //isAligned
  m_isAligned = te::qt::widgets::AbstractPropertyManager::getInstance().m_boolManager->addProperty(tr("Is Aligned"));
  addListenerToProperty(m_isAligned);

  itemProperty->addSubProperty(m_perpendicularOffset);
  itemProperty->addSubProperty(m_isAligned);

  setLinePlacement(nullptr);
}

te::qt::widgets::LinePlacementPropertyItem::~LinePlacementPropertyItem()
{
  delete m_perpendicularOffset;
  delete m_isAligned;
}

void te::qt::widgets::LinePlacementPropertyItem::setLinePlacement(const te::se::LinePlacement* linePlacement)
{
  double perpendicularOffset = 10.;
  bool isAligned = false;

  if (linePlacement != nullptr)
  {
    const te::se::ParameterValue* perpendicularParam = linePlacement->getPerpendicularOffset();
    if (perpendicularParam != nullptr)
    {
      perpendicularOffset = te::se::GetDouble(perpendicularParam);
    }

    isAligned = linePlacement->isAligned();
  }

  m_enableListeners = false;

  te::qt::widgets::AbstractPropertyManager::getInstance().m_doubleManager->setValue(m_perpendicularOffset, perpendicularOffset);
  te::qt::widgets::AbstractPropertyManager::getInstance().m_boolManager->setValue(m_isAligned, isAligned);

  m_enableListeners = true;
}

te::se::LinePlacement* te::qt::widgets::LinePlacementPropertyItem::getLinePlacement() const
{
  double perpendicularOffset = te::qt::widgets::AbstractPropertyManager::getInstance().m_doubleManager->value(m_perpendicularOffset);
  bool isAligned = te::qt::widgets::AbstractPropertyManager::getInstance().m_boolManager->value(m_isAligned);

  te::se::LinePlacement* newItem = new te::se::LinePlacement();
  newItem->setPerpendicularOffset(new te::se::ParameterValue(Convert2StdString(QString::number(perpendicularOffset))));
  newItem->setIsAligned(isAligned);

  return newItem;
}
