/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

 /*!
   \file terralib/qt/widgets/se/MarkExternalGraphicPropertyItem.h

   \brief A widget used to define the graphic mark external graphic se object.
 */

#ifndef __TERRALIB_QT_WIDGETS_SE_INTERNAL_MARKEXTERNALGRAPHICPROPERTYITEM_H
#define __TERRALIB_QT_WIDGETS_SE_INTERNAL_MARKEXTERNALGRAPHICPROPERTYITEM_H

 // TerraLib
#include "CheckboxStylePropertyItem.h"
#include "../Config.h"
#include "../propertybrowser/AbstractPropertyItem.h"

#include <string>

class QWidget;
class QtProperty;

namespace te
{
  // Forward declarations
  namespace se
  {
    class ExternalGraphic;
    class InlineContent;
  }

  namespace qt
  {
    namespace widgets
    {
      /*!
        \class MarkExternalGraphicPropertyItem

        \brief A widget used to define the graphic mark external graphic se object.

          - online resource
          - inline contents

      */

      class TEQTWIDGETSEXPORT MarkExternalGraphicPropertyItem : public te::qt::widgets::CheckboxStylePropertyItem
      {
        Q_OBJECT

      public:
        /** @name Initializer Methods
          *  Methods related to instantiation and destruction.
          */
        //@{

            /*! \brief Constructor */
        MarkExternalGraphicPropertyItem(QtProperty* parent, QtTreePropertyBrowser* propertyBrowser, StylePropertyItemListener* listener, QColor c);

        /*! \brief Destructor. */
        ~MarkExternalGraphicPropertyItem();

        //@}

        bool setExternalGraphic(const te::se::ExternalGraphic* externalGraphic);

        te::se::ExternalGraphic* getExternalGraphic() const;

        void setGraphicSize(int size);

      public slots:

        virtual void setDlg(QWidget* parent, QtProperty* prop);
        virtual void showOnLineResourceDlg();
        virtual void showInLineContentDlg();

      protected:

        QtProperty* m_onLineResourceProperty;
        QtProperty* m_inLineContentProperty;

        te::se::InlineContent* m_inlineContent;
        std::string m_format;
        int m_size;
      };
    }
  }
}

#endif //__TERRALIB_QT_WIDGETS_SE_INTERNAL_MARKEXTERNALGRAPHICPROPERTYITEM_H
