/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

 /*!
   \file terralib/qt/widgets/se/MarkWellKnownPropertyItem.cpp

   \brief A widget used to define the mark well known se object.
 */

#include "MarkWellKnownPropertyItem.h"

#include "BasicFillPropertyItem.h"
#include "BasicStrokePropertyItem.h"

 // TerraLib Includes
#include "../propertybrowser/AbstractPropertyManager.h"
#include "../Utils.h"
#include "../../../maptools/MarkRendererManager.h"
#include "../../../se/Mark.h"

te::qt::widgets::MarkWellKnownPropertyItem::MarkWellKnownPropertyItem(QtProperty* parent, QtTreePropertyBrowser* propertyBrowser, StylePropertyItemListener* listener, QColor c)
  : te::qt::widgets::CheckboxStylePropertyItem(propertyBrowser, tr("Well Known Mark"), listener, c)
  , m_typeProperty(nullptr)
  , m_fillPropertyItem(nullptr)
  , m_strokePropertyItem(nullptr)
{
  //add property to parent
  QtProperty* itemProperty = this->getItemProperty();
  addSubProperty(parent, itemProperty, c);

  //Type
  m_typeProperty = te::qt::widgets::AbstractPropertyManager::getInstance().m_enumManager->addProperty(tr("Type"));
  te::qt::widgets::AbstractPropertyManager::getInstance().m_enumManager->setEnumNames(m_typeProperty, getMarkTypeNames());
  addListenerToProperty(m_typeProperty);
  itemProperty->addSubProperty(m_typeProperty);

  //fill
  m_fillPropertyItem = new te::qt::widgets::BasicFillPropertyItem(itemProperty, propertyBrowser, listener, c);

  //stroke
  m_strokePropertyItem = new te::qt::widgets::BasicStrokePropertyItem(itemProperty, propertyBrowser, listener, c);

  setMark(nullptr);
}

te::qt::widgets::MarkWellKnownPropertyItem::~MarkWellKnownPropertyItem()
{
  delete m_fillPropertyItem;
  delete m_strokePropertyItem;
}

bool te::qt::widgets::MarkWellKnownPropertyItem::setMark(const te::se::Mark* mark)
{
  std::string markWellKnownName = "square";
  const te::se::Fill* fill = nullptr;
  const te::se::Stroke* stroke = nullptr;

  if (mark != nullptr)
  {
    if (mark->getWellKnownName() != nullptr)
    {
      markWellKnownName = *mark->getWellKnownName();
    }
    
    fill = mark->getFill();
    stroke = mark->getStroke();
  }

  m_enableListeners = false;

  //mark well known name
  AbstractPropertyItem::setValueToEnumProperty(m_typeProperty, Convert2QString(markWellKnownName));

  //fill
  m_fillPropertyItem->setFill(fill);
  m_fillPropertyItem->setChecked(fill != nullptr);

  //stroke
  m_strokePropertyItem->setStroke(stroke);
  m_strokePropertyItem->setChecked(stroke != nullptr);


  m_enableListeners = true;

  return true;
}

te::se::Mark* te::qt::widgets::MarkWellKnownPropertyItem::getMark() const
{
  te::se::Mark* newItem = new te::se::Mark();
  
  std::string* markName = nullptr;
  QString qMarkName = AbstractPropertyItem::getValueFromEnumProperty(m_typeProperty);
  if (qMarkName.isEmpty() == false)
  {
    markName = new std::string(Convert2StdString(qMarkName));
  }

  newItem->setWellKnownName(markName);
  if (m_fillPropertyItem->isChecked() == true)
  {
    newItem->setFill(m_fillPropertyItem->getFill());
  }
  
  if (m_strokePropertyItem->isChecked() == true)
  {
    newItem->setStroke(m_strokePropertyItem->getStroke());
  }

  return newItem;
}

QStringList te::qt::widgets::MarkWellKnownPropertyItem::getMarkTypeNames() const
{
  QStringList qMarkNameList;

  std::vector<std::string> markNamesVec;
  te::map::MarkRendererManager::getInstance().getAllSupportedMarks(markNamesVec);
  for (std::size_t i = 0; i < markNamesVec.size(); ++i)
  {
    QString qMarkName = te::qt::widgets::Convert2QString(markNamesVec.at(i));
    qMarkNameList.append(qMarkName);
  }

  return qMarkNameList;
}