/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

 /*!
   \file terralib/qt/widgets/se/MarkWellKnownPropertyItem.h

   \brief A widget used to define the mark well known se object.
 */

#ifndef __TERRALIB_QT_WIDGETS_SE_INTERNAL_MARKWELLKNOWNPROPERTYITEM_H
#define __TERRALIB_QT_WIDGETS_SE_INTERNAL_MARKWELLKNOWNPROPERTYITEM_H

 // TerraLib
#include "CheckboxStylePropertyItem.h"
#include "../Config.h"

class QColor;
class QStringList;
class QtProperty;

namespace te
{
  // Forward declarations
  namespace se
  {
    class Mark;
  }

  namespace qt
  {
    namespace widgets
    {
      class AbstractPropertyItem;
      class BasicFillPropertyItem;
      class BasicStrokePropertyItem;

      /*!
        \class GraphicFillPropertyItem

        \brief A widget used to define the mark well known se object.

          - type
          - fill
          - stroke
      */

      class TEQTWIDGETSEXPORT MarkWellKnownPropertyItem : public te::qt::widgets::CheckboxStylePropertyItem
      {
        Q_OBJECT

      public:
        /** @name Initializer Methods
          *  Methods related to instantiation and destruction.
          */
        //@{

            /*! \brief Constructor */
        MarkWellKnownPropertyItem(QtProperty* parent, QtTreePropertyBrowser* propertyBrowser, StylePropertyItemListener* listener, QColor c);

        /*! \brief Destructor. */
        ~MarkWellKnownPropertyItem();

        //@}

        bool setMark(const te::se::Mark* mark);

        te::se::Mark* getMark() const;

      protected:

        QStringList getMarkTypeNames() const;

      protected:

        QtProperty* m_typeProperty;

        te::qt::widgets::BasicFillPropertyItem* m_fillPropertyItem;
        te::qt::widgets::BasicStrokePropertyItem* m_strokePropertyItem;
      };
    }
  }
}

#endif //__TERRALIB_QT_WIDGETS_SE_INTERNAL_MARKGLYPHPROPERTYITEM_H
