/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/widgets/se/PolygonSymbolizerProperty.cpp

  \brief A widget used to define the point symbolizer
         properties, such as:
           - general
           - mark
              - basic fill
              - basic stroke
*/

#include "PointSymbolizerProperty.h"


#include "../../../se/PointSymbolizer.h"
#include "../../../se/Graphic.h"
#include "../propertybrowser/AbstractPropertyManager.h"
#include "GraphicFillPropertyItem.h"


// Qt
#include <QGridLayout>
#include <QToolBox>

// QtPropertyBrowser
#include <QtPropertyBrowser/QtTreePropertyBrowser>
#include <QtPropertyBrowser/QtVariantPropertyManager>

// STL
#include <cassert>

te::qt::widgets::PointSymbolizerProperty::PointSymbolizerProperty(QWidget* parent)
  : AbstractSymbolizerProperty(parent)
  , StylePropertyItemListener()
  , m_graphProp(nullptr)
{
  QGridLayout* layout = new QGridLayout(this);
  this->setLayout(layout);

  QToolBox* tb = new QToolBox(this);
  layout->addWidget(tb);

  QtTreePropertyBrowser* basicPropBrowser = new QtTreePropertyBrowser(this);
  basicPropBrowser->setIndentation(10);
  basicPropBrowser->setPropertiesWithoutValueMarked(true);
  basicPropBrowser->setRootIsDecorated(false);
  basicPropBrowser->setResizeMode(QtTreePropertyBrowser::ResizeToContents);

  layout->addWidget(basicPropBrowser);

  //Graphic property
  QtProperty* pointProperty = te::qt::widgets::AbstractPropertyManager::getInstance().m_groupManager->addProperty(tr("Point Property"));
  basicPropBrowser->addProperty(pointProperty);

  m_graphProp = new te::qt::widgets::GraphicFillPropertyItem(pointProperty, basicPropBrowser, this, QColor(175, 255, 175), QColor(225, 255, 225));

  setSymbolizer(nullptr);
}

te::qt::widgets::PointSymbolizerProperty::~PointSymbolizerProperty()
{
  delete m_graphProp;
}

void te::qt::widgets::PointSymbolizerProperty::setSymbolizer(const te::se::Symbolizer* symb)
{
  const te::se::Graphic* graphic = nullptr;
  if (symb != nullptr)
  {
    const te::se::PointSymbolizer* pointSymbolizer = dynamic_cast<const te::se::PointSymbolizer*>(symb);
    if (pointSymbolizer != nullptr)
    {
      graphic = pointSymbolizer->getGraphic();
    }
  }

  m_graphProp->setGraphic(graphic);
  
  bool enableGraphicFillProperty = (graphic != nullptr);

  m_graphProp->setChecked(enableGraphicFillProperty);
  m_graphProp->setExpandedStatus(enableGraphicFillProperty, false);
}

te::se::Symbolizer* te::qt::widgets::PointSymbolizerProperty::getSymbolizer() const
{
  //we always return a new instance
  te::se::PointSymbolizer* newItem = new te::se::PointSymbolizer();;
  newItem->setGraphic(m_graphProp->getGraphic());

  return newItem;
}

void te::qt::widgets::PointSymbolizerProperty::handleValueChanged(QtProperty* /*qProperty*/, const QVariant& /*qVariant*/)
{
  emit symbolizerChanged();
}
