/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

 /*!
   \file terralib/qt/widgets/se/StrokePropertyItem.cpp

   \brief A widget used to define the stroke se object.
 */

#include "StrokePropertyItem.h"

#include "BasicStrokePropertyItem.h"
#include "GraphicFillPropertyItem.h"
#include "GraphicStrokePropertyItem.h"
#include "GroupPropertyItemHandler.h"

 // TerraLib Includes
#include "../propertybrowser/AbstractPropertyManager.h"
#include "../../../se/Stroke.h"

te::qt::widgets::StrokePropertyItem::StrokePropertyItem(QtTreePropertyBrowser* propertyBrowser, StylePropertyItemListener* listener, QColor c)
  : te::qt::widgets::StylePropertyItem(propertyBrowser, listener, c)
  , m_groupHandler(new GroupPropertyItemHandler())
  , m_basicStrokeProperty(nullptr)
  , m_graphicFillProperty(nullptr)
  , m_graphicStrokeProperty(nullptr)
{
  //add to tree propertybrowser
  QtProperty* itemProperty = te::qt::widgets::AbstractPropertyManager::getInstance().m_groupManager->addProperty(tr("Stroke Property"));
  addProperty(itemProperty, QColor(125, 255, 125));

  //basic stroke
  m_basicStrokeProperty = new te::qt::widgets::BasicStrokePropertyItem(itemProperty, propertyBrowser, listener, QColor(175, 255, 175));
  m_groupHandler->addPropertyToGroup(m_basicStrokeProperty);

  //graphic fill
  m_graphicFillProperty = new te::qt::widgets::GraphicFillPropertyItem(itemProperty, propertyBrowser, listener, QColor(175, 255, 175), QColor(225, 255, 225));
  m_groupHandler->addPropertyToGroup(m_graphicFillProperty);

  //graphic stroke
  m_graphicStrokeProperty = new te::qt::widgets::GraphicStrokePropertyItem(itemProperty, propertyBrowser, listener, QColor(175, 255, 175), QColor(225, 255, 225));
  m_groupHandler->addPropertyToGroup(m_graphicStrokeProperty);

  setStroke(nullptr);

  init(itemProperty);
}

te::qt::widgets::StrokePropertyItem::~StrokePropertyItem()
{
  delete m_groupHandler;
  delete m_basicStrokeProperty;
  delete m_graphicFillProperty;
  delete m_graphicStrokeProperty;
}

bool te::qt::widgets::StrokePropertyItem::setStroke(const te::se::Stroke* stroke)
{
  const te::se::GraphicStroke* graphicStroke = nullptr;
  const te::se::Graphic* graphicFill = nullptr;
  if (stroke != nullptr)
  {
    graphicStroke = stroke->getGraphicStroke();
    graphicFill = stroke->getGraphicFill();
  }

  m_enableListeners = false;

  bool hasBasic = (stroke != nullptr) && (graphicFill == nullptr) && (graphicStroke == nullptr);
  bool hasFill = (graphicFill != nullptr);
  bool hasStroke = (graphicStroke != nullptr);

  m_basicStrokeProperty->setStroke(stroke);
  m_graphicFillProperty->setGraphic(graphicFill);
  m_graphicStrokeProperty->setGraphicStroke(graphicStroke);

  m_basicStrokeProperty->setChecked(hasBasic);
  m_basicStrokeProperty->setExpandedStatus(hasBasic, false);

  m_graphicFillProperty->setChecked(hasFill);
  m_graphicFillProperty->setExpandedStatus(hasFill, false);

  m_graphicStrokeProperty->setChecked(hasStroke);
  m_graphicStrokeProperty->setExpandedStatus(hasStroke, false);

  m_enableListeners = true;

  return true;
}

te::se::Stroke* te::qt::widgets::StrokePropertyItem::getStroke() const
{
  te::se::Stroke* newItem = nullptr;
  if(m_basicStrokeProperty->isChecked() == true)
  {
    newItem = m_basicStrokeProperty->getStroke();
  }
  else if (m_graphicFillProperty->isChecked() == true)
  {
    newItem = new te::se::Stroke();
    newItem->setGraphicFill(m_graphicFillProperty->getGraphic());
  }
  else if (m_graphicStrokeProperty->isChecked() == true)
  {
    newItem = new te::se::Stroke();
    newItem->setGraphicStroke(m_graphicStrokeProperty->getGraphicStroke());
  }

  return newItem;
}
