/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

 /*!
   \file terralib/qt/widgets/se/StrokePropertyItem.h

   \brief A widget used to define the Stroke se object.
 */

#ifndef __TERRALIB_QT_WIDGETS_SE_INTERNAL_STROKEPROPERTYITEM_H
#define __TERRALIB_QT_WIDGETS_SE_INTERNAL_STROKEPROPERTYITEM_H

 // TerraLib
#include "StylePropertyItem.h"
#include "../Config.h"

namespace te
{
  // Forward declarations
  namespace se
  {
    class Stroke;
  }

  namespace qt
  {
    namespace widgets
    {
      class BasicStrokePropertyItem;
      class GraphicFillPropertyItem;
      class GraphicStrokePropertyItem;
      class GroupPropertyItemHandler;

      /*!
        \class StrokePropertyItem

        \brief A widget used to define the stroke se object.

          - stroke
          - opacity
          - width
          - linejoin
          - linecap
          - dasharray
          - dashoffset
          - GraphicFill
          - GraphicStroke

      */

      class TEQTWIDGETSEXPORT StrokePropertyItem : public te::qt::widgets::StylePropertyItem
      {
        Q_OBJECT

      public:
        /** @name Initializer Methods
          *  Methods related to instantiation and destruction.
          */
          //@{

            /*! \brief Constructor */
        StrokePropertyItem(QtTreePropertyBrowser* propertyBrowser, StylePropertyItemListener* listener, QColor c = QColor());

        /*! \brief Destructor. */
        ~StrokePropertyItem();

        //@}

        bool setStroke(const te::se::Stroke* stroke);

        te::se::Stroke* getStroke() const;

      protected:

        GroupPropertyItemHandler* m_groupHandler;

        te::qt::widgets::BasicStrokePropertyItem* m_basicStrokeProperty;
        te::qt::widgets::GraphicFillPropertyItem* m_graphicFillProperty;
        te::qt::widgets::GraphicStrokePropertyItem* m_graphicStrokeProperty;
      };
    }
  }
}

#endif //__TERRALIB_QT_WIDGETS_SE_INTERNAL_STROKEPROPERTYITEM_H
