/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

 /*!
   \file terralib/qt/widgets/se/StylePropertyItem.cpp

   \brief A base item for the style property tree
 */

#include "StylePropertyItem.h"
#include "StylePropertyItemListener.h"

te::qt::widgets::StylePropertyItem::StylePropertyItem(QtTreePropertyBrowser* pb, StylePropertyItemListener* listener, QColor c)
  : te::qt::widgets::AbstractPropertyItem(pb, c)
  , m_listener(listener)
{
}

void te::qt::widgets::StylePropertyItem::addListenerToProperty(QtProperty* p)
{
  m_vecProperties.push_back(p);
}

bool te::qt::widgets::StylePropertyItem::propertyHasListener(QtProperty* p)
{
  std::vector<QtProperty*>::iterator it = std::find(m_vecProperties.begin(), m_vecProperties.end(), p);
  if (it == m_vecProperties.end())
  {
    return false;
  }

  return true;
}

void te::qt::widgets::StylePropertyItem::valueChanged(QtProperty* qProperty, const QVariant& qVariant)
{
  if (m_listener == nullptr)
  {
    return;
  }

  if (m_enableListeners == false)
  {
    return;
  }

  if (propertyHasListener(qProperty) == false)
  {
    return;
  }

  m_listener->handleValueChanged(qProperty, qVariant);
}
