/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled
   applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/widgets/se/SymbolSelectorDialog.cpp

  \brief A dialog used to select a specific symbol.
*/

// TerraLib
#include "../../../common/Exception.h"
#include "SymbolGroupDialog.h"
#include "ui/ui_SymbolGroupDialogForm.h"

// Qt
#include <QFileDialog>
#include <QMessageBox>
#include <QTableWidgetItem>
#include <QCloseEvent>

// STL
#include <cassert>

// Boost
#include <boost/uuid/random_generator.hpp>
#include <boost/uuid/uuid_io.hpp>

te::qt::widgets::SymbolGroupDialog::SymbolGroupDialog(QWidget* parent, Qt::WindowFlags f)
  : QDialog(parent, f), 
  m_ui(new Ui::SymbolGroupDialogForm),
  m_groupName("")
{
  // Setup
  m_ui->setupUi(this);

  initialize();
}

te::qt::widgets::SymbolGroupDialog::SymbolGroupDialog(const std::string& groupName, QWidget* parent, Qt::WindowFlags f)
  : QDialog(parent, f), 
  m_ui(new Ui::SymbolGroupDialogForm),
  m_groupName(groupName)
{
  // Setup
  m_ui->setupUi(this);

  initialize();
}

void te::qt::widgets::SymbolGroupDialog::initialize()
{
  // connect signal/slot
  connect(m_ui->m_okPushButton, SIGNAL(clicked(bool)), this, SLOT(onOkPushButtonClicked()));
  connect(m_ui->m_cancelPushButton, SIGNAL(clicked(bool)), this, SLOT(onCancelPushButtonClicked()));
  connect(m_ui->m_helpPushButton, SIGNAL(clicked(bool)), this, SLOT(onHelpPushButtonClicked()));

  QString name(m_groupName.c_str());

  m_ui->m_lneName->setText(name);
}

te::qt::widgets::SymbolGroupDialog::~SymbolGroupDialog()
{
  
}

void te::qt::widgets::SymbolGroupDialog::onOkPushButtonClicked()
{
  QString name = m_ui->m_lneName->text();

  if (name.compare("") == 0)
  {
    QMessageBox::information(this, "Symbol Group", "Blank text field.");
    return;
  }

  m_groupName = m_ui->m_lneName->text().toUtf8().constData();

  accept();
}

void te::qt::widgets::SymbolGroupDialog::onCancelPushButtonClicked()
{
  reject();
}

void te::qt::widgets::SymbolGroupDialog::onHelpPushButtonClicked()
{

}

std::string te::qt::widgets::SymbolGroupDialog::getGroupName()
{
  return m_groupName;
}

void te::qt::widgets::SymbolGroupDialog::setGroupName(const std::string& groupName)
{
  m_groupName = groupName;
}
