/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/widgets/se/SymbolSelectorDialog.h

  \brief A dialog used to create or edit a specific symbol.
*/

#ifndef __TERRALIB_QT_WIDGETS_SE_INTERNAL_SYMBOL_GROUP_DIALOG_H
#define __TERRALIB_QT_WIDGETS_SE_INTERNAL_SYMBOL_GROUP_DIALOG_H

// TerraLib
#include "../Config.h"

// Qt
#include <QDialog>

// STL
#include <string>
#include <memory>

namespace Ui
{
  class SymbolGroupDialogForm;
}

namespace te
{
  namespace qt
  {
    namespace widgets
    {

      /*!
        \class SymbolSelectorDialog

        \brief A dialog used to create or edit a specific symbol.
      */
      class TEQTWIDGETSEXPORT SymbolGroupDialog : public QDialog
      {
        Q_OBJECT

        public:

          /** @name Initializer Methods
           *  Methods related to instantiation and destruction.
           */
          //@{

          /*! 
          \brief Constructs 

          \note 
          */
         SymbolGroupDialog(QWidget* parent = nullptr, Qt::WindowFlags f = nullptr);

         /*! 
          \brief Constructs 

          \note 
          */
         SymbolGroupDialog(const std::string& groupName, QWidget* parent = nullptr, Qt::WindowFlags f = nullptr);

          /*! \brief Destructor. */
          virtual ~SymbolGroupDialog();

          /*!
          \brief Gets the selected symbol.

          \return The selected fill element.       

            \note The caller will take the ownership of the returned symbol.
            \note It return a NULL pointer if there is not a symbol yet created.
          */
          virtual std::string getGroupName();

          virtual void setGroupName(const std::string& groupName);
                              
        protected slots:
          
          void onOkPushButtonClicked();

          void onCancelPushButtonClicked();

          void onHelpPushButtonClicked();

        protected:

          void initialize();

        private:

          std::unique_ptr<Ui::SymbolGroupDialogForm>          m_ui; //!< Dialog form.
          std::string                                         m_groupName; //!< Group name
      }; 

    } // end namespace widgets
  }   // end namespace qt
}     // end namespace te

#endif  // __TERRALIB_QT_WIDGETS_SE_INTERNAL_SYMBOL_EDITINFO_WIDGET_H
