/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/widgets/se/SymbolLibraryManager.h

  \brief The SymbolLibraryManager is a singleton that can be used to manage all loaded symbol libraries in TerraLib.
*/

#ifndef __TERRALIB_QT_WIDGETS_SE_INTERNAL_SYMBOLLIBRARYMANAGER_H
#define __TERRALIB_QT_WIDGETS_SE_INTERNAL_SYMBOLLIBRARYMANAGER_H

// TerraLib
#include "../../../common/Singleton.h"
#include "../Config.h"

// STL 
#include <map>
#include <string>
#include <memory>

namespace te
{
  namespace qt
  {
    namespace widgets
    {

      // Forward declarations
      class SymbolLibrary;

      /*!
        \class SymbolLibraryManager

        \brief The SymbolLibraryManager is a singleton that can be used to manage all loaded symbol libraries in TerraLib.
      
        \sa Symbol, SymbolLibrary, Singleton
      */
      class TEQTWIDGETSEXPORT SymbolLibraryManager : public te::common::Singleton<SymbolLibraryManager>
      {
        friend class te::common::Singleton<SymbolLibraryManager>;

        public:

          //@}

          /*! \brief Destructor. */
          virtual ~SymbolLibraryManager() = default;
          
          /*!
            \brief It returns the symbol library identified by a given name or NULL if none is found.

            \param name The name of the symbol library we are looking for.

            \return A pointer to a symbol library or NULL if none is found.
          */
          SymbolLibrary* getLibrary() const;
          
        protected:

          /*! \brief It initializes the Singleton. */
          SymbolLibraryManager();

        private:

          std::unique_ptr<SymbolLibrary> m_symbolLibrary; //!< The set of symbol libraries.
    };

    } // end namespace widgets
  }   // end namespace qt
}     // end namespace te

#endif  // __TERRALIB_QT_WIDGETS_SE_INTERNAL_SYMBOLLIBRARYMANAGER_H
