/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

#ifndef __TERRALIB_QT_WIDGETS_SE_INTERNAL_SYMBOL_TREE_FILTER_H
#define __TERRALIB_QT_WIDGETS_SE_INTERNAL_SYMBOL_TREE_FILTER_H

// TerraLib
#include "../Config.h"

// Boost
#include <boost/noncopyable.hpp>

// Qt
#include <QtCore/QObject>

// Forward declarations
class QDragEnterEvent;
class QDropEvent;

namespace te
{
  namespace qt
  {
    namespace widgets
    {

      class TEQTWIDGETSEXPORT SymbolTreeFilter : public QObject, public boost::noncopyable
      {
        Q_OBJECT

      public:

        /** @name Initializer Methods
         *  Methods related to instantiation and destruction.
         */
         //@{

         /*!
           \brief It constructs a tool associated with the given map display and with the specified cursor.

           \note The tool will NOT take the ownership of the given pointers.
           \note If the given cursor is different of Qt::BlankCursor, it will be setted on map display.
         */
        SymbolTreeFilter(QObject* parent = 0);

        /*! \brief Destructor. */
        virtual ~SymbolTreeFilter();

        //@}

        /** @name AbstractTool Methods
         *  Methods related with tool behavior.
         */
         //@{

         /*!
           \brief Filters events if this object has been installed as an event filter for the watched object.

           \param watched The watched object.
           \param event The last event sent to watched object.

           \note In your reimplementation of this function, if you want to filter the event out,
                 i.e. stop it being handled further, return true; otherwise return false.

           \note Here, the default implementation looks for QMouseEvents and call the pure virutal methods
                 defined on AbstractTool class, e.g. mousePressEvent().
         */
        virtual bool eventFilter(QObject* watched, QEvent* e);

        /*!
          \brief This event handler can be reimplemented in a concrete tool class
                 to receive mouse press events for the watched object.

          \param e The mouse event.

          \return If you want to filter the event out,  i.e. stop it being handled further, return true; otherwise return false.
        */
        virtual bool drag(QDragEnterEvent* e);

        /*!
          \brief This event handler can be reimplemented in a concrete tool class
                 to receive mouse press events for the watched object.

          \param e The mouse event.

          \return If you want to filter the event out,  i.e. stop it being handled further, return true; otherwise return false.
        */
        virtual bool drop(QDropEvent* e);

      signals:

        void drag();

        void drop();
      };
    }
  }
}


#endif  // __TERRALIB_LAYOUT_INTERNAL_ABSTRACTLAYOUT_TOOL_H
