/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/widgets/tools/CoordTracking.h

  \brief This class implements a concrete tool to geographic coordinate tracking on mouse move operation.
*/

#ifndef __TERRALIB_QT_WIDGETS_INTERNAL_COORDTRACKING_H
#define __TERRALIB_QT_WIDGETS_INTERNAL_COORDTRACKING_H

// TerraLib
#include "../Config.h"
#include "AbstractTool.h"

// Forward declarations
class QPointF;

namespace te
{
  namespace srs
  {
    class Converter;
  }

  namespace qt
  {
    namespace widgets
    {
// Forward declarations
    class MapDisplay;

      /*!
        \class CoordTracking

        \brief This class implements a concrete tool to geographic coordinate tracking on mouse move operation.

        \ingroup widgets
      */
      class TEQTWIDGETSEXPORT CoordTracking : public AbstractTool
      {
        Q_OBJECT

        public:

          /** @name Initializer Methods
           *  Methods related to instantiation and destruction.
           */
          //@{

          /*!
            \brief It constructs a coordinate tracking tool associated with the given map display.

            \param display The map display associated with the tool.
            \param parent The tool's parent.

            \note The tool will NOT take the ownership of the given pointers.
          */
          CoordTracking(MapDisplay* display, QObject* parent = 0);

          /*! \brief Destructor. */
          ~CoordTracking();

          void setDisplaySRID(int srid);

          //@}

          /** @name AbstractTool Methods
           *  Methods related with tool behavior.
           */
          //@{

          bool mouseMoveEvent(QMouseEvent* e);

          bool keyPressEvent(QKeyEvent* e);

          //@}

        signals:

          /*! This signal is emitted when a coordinate was tracked. */
          void coordTracked(QPointF& coordinate);

          /*! This signal is emitted when a coordinate was tracked and converted to LL. */
          void coordLLTracked(QPointF& coordinate, QPointF& llCoordinate);

        protected:

          QString m_currentPos;

          te::srs::Converter* m_conveter;
      };

    } // end namespace widgets
  }   // end namespace qt
}     // end namespace te

#endif  // __TERRALIB_QT_WIDGETS_INTERNAL_COORDTRACKING_H
