/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/widgets/utils/CoordInfoWidget.cpp

  \brief ...
*/

// TerraLib
#include "ui/ui_CoordInfoWidgetForm.h"
#include "CoordInfoWidget.h"

#include "../Utils.h"
#include "../canvas/MapDisplay.h"

// Qt
#include <QSettings>


te::qt::widgets::CoordInfoWidget::CoordInfoWidget(QWidget* parent, Qt::WindowFlags f)
  : QWidget(parent, f),
    m_display(nullptr),
    m_ui(new Ui::CoordInfoWidgetForm)
{
  m_ui->setupUi(this);

  connect(m_ui->m_dmsToolButton, SIGNAL(toggled(bool)), this, SLOT(onDMSToolButtonToggled(bool)));

  m_ui->m_dmsToolButton->setChecked(getDisplayCoordMode());
  onDMSToolButtonToggled(getDisplayCoordMode());
}

te::qt::widgets::CoordInfoWidget::~CoordInfoWidget() = default;

void te::qt::widgets::CoordInfoWidget::setDisplay(te::qt::widgets::MapDisplay* display)
{
  m_display = display;
}

void te::qt::widgets::CoordInfoWidget::setCoordInfo(double ex, double ny, double llx, double lly)
{
  if (m_display)
  {
    m_ui->m_exLineEdit->setText(te::qt::widgets::FormatCoord(ex, m_display));
    m_ui->m_nyLineEdit->setText(te::qt::widgets::FormatCoord(ny, m_display));
  }
  else
  {
    m_ui->m_exLineEdit->setText(QString::number(ex));
    m_ui->m_nyLineEdit->setText(QString::number(ny));
  }


  if (lly == 0.)
    m_ui->m_latLineEdit->clear();
  else
    m_ui->m_latLineEdit->setText(te::qt::widgets::ConvertDecimalToDegree(lly, 2));

  if (llx == 0.)
    m_ui->m_longLineEdit->clear();
  else
    m_ui->m_longLineEdit->setText(te::qt::widgets::ConvertDecimalToDegree(llx, 2));
}

void te::qt::widgets::CoordInfoWidget::onDMSToolButtonToggled(bool status)
{
  m_ui->m_latLabel->setVisible(status);
  m_ui->m_latLineEdit->setVisible(status);
  m_ui->m_longLabel->setVisible(status);
  m_ui->m_longLineEdit->setVisible(status);

  setDisplayCoordMode(status);
}

bool te::qt::widgets::CoordInfoWidget::getDisplayCoordMode()
{
  QSettings sett(QSettings::IniFormat, QSettings::UserScope, qApp->organizationName(), qApp->applicationName());

  QString key = "Display Coord Mode:";

  return sett.value(key).toBool();
}

void te::qt::widgets::CoordInfoWidget::setDisplayCoordMode(bool mode)
{
  QSettings sett(QSettings::IniFormat, QSettings::UserScope, qApp->organizationName(), qApp->applicationName());

  QString key = "Display Coord Mode:";

  sett.setValue(key, mode);
}
