/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/widgets/utils/DoubleInputListWidget.cpp

  \brief ...
*/

// TerraLib
#include "ui/ui_DoubleInputListWidgetForm.h"
#include "DoubleInputListWidget.h"

// Qt
#include <QIcon>

te::qt::widgets::DoubleInputListWidget::DoubleInputListWidget(QWidget* parent,
                                                              Qt::WindowFlags f)
    : QWidget(parent, f), m_ui(new Ui::DoubleInputListWidgetForm)
{
  m_ui->setupUi(this);

// set icons
  m_ui->m_addToolButton->setIcon(QIcon::fromTheme("go-next"));
  m_ui->m_addAllToolButton->setIcon(QIcon::fromTheme("go-last"));
  m_ui->m_removeToolButton->setIcon(QIcon::fromTheme("go-previous"));
  m_ui->m_removeAllToolButton->setIcon(QIcon::fromTheme("go-first"));
  m_ui->m_upToolButton->setIcon(QIcon::fromTheme("go-up"));
  m_ui->m_downToolButton->setIcon(QIcon::fromTheme("go-down"));

// set selection mode
  m_ui->m_firstInputListWidget->setSelectionMode(
      QAbstractItemView::ExtendedSelection);
  m_ui->m_secondInputListWidget->setSelectionMode(
      QAbstractItemView::ExtendedSelection);
  m_ui->m_outputListWidget->setSelectionMode(
      QAbstractItemView::ExtendedSelection);

// connect signals and slots
  connect(m_ui->m_addToolButton, SIGNAL(pressed()), this,
          SLOT(onAddToolButtonPressed()));
  connect(m_ui->m_addAllToolButton, SIGNAL(pressed()), this,
          SLOT(onAddAllToolButtonPressed()));
  connect(m_ui->m_removeToolButton, SIGNAL(pressed()), this,
          SLOT(onRemoveToolButtonPressed()));
  connect(m_ui->m_removeAllToolButton, SIGNAL(pressed()), this,
          SLOT(onRemoveAllToolButtonPressed()));
  connect(m_ui->m_firstInputListWidget, SIGNAL(itemSelectionChanged()), this,
          SLOT(onFirstInputListItemSelectionChanged()));
  connect(m_ui->m_secondInputListWidget, SIGNAL(itemSelectionChanged()), this,
          SLOT(onSecondInputListItemSelectionChanged()));
  connect(m_ui->m_outputListWidget, SIGNAL(itemSelectionChanged()), this,
          SLOT(onOutputListItemSelectionChanged()));
  connect(m_ui->m_upToolButton, SIGNAL(pressed()), this,
          SLOT(onUpToolButtonPressed()));
  connect(m_ui->m_downToolButton, SIGNAL(pressed()), this,
          SLOT(onDownToolButtonPressed()));
}

te::qt::widgets::DoubleInputListWidget::~DoubleInputListWidget() = default;

Ui::DoubleInputListWidgetForm* te::qt::widgets::DoubleInputListWidget::getForm()
    const
{
  return m_ui.get();
}

void te::qt::widgets::DoubleInputListWidget::setFirstLayerLabel(
    const QString& value)
{
  m_ui->m_firstItemsLabel->setText(value);
}

void te::qt::widgets::DoubleInputListWidget::setSecondLayerLabel(
    const QString& value)
{
  m_ui->m_secondItemsLabel->setText(value);
}

void te::qt::widgets::DoubleInputListWidget::setOutputLabel(
    const QString& value)
{
  m_ui->m_outputItemsLabel->setText(value);
}

void te::qt::widgets::DoubleInputListWidget::setFirstInputValues(
    const std::vector<std::string>& values)
{
  m_ui->m_firstInputListWidget->clear();

  if(!values.empty())
    m_ui->m_addAllToolButton->setEnabled(true);
  else
    m_ui->m_addAllToolButton->setEnabled(false);

  for(size_t i = 0; i < values.size(); ++i)
  {
    QListWidgetItem* item = new QListWidgetItem(m_ui->m_firstInputListWidget);
    item->setText(QString::fromUtf8(values[i].c_str()));
    item->setData(Qt::UserRole, QVariant(0));

    m_ui->m_outputListWidget->addItem(item);
  }
}

void te::qt::widgets::DoubleInputListWidget::setSecondInputValues(
    const std::vector<std::string>& values)
{
  m_ui->m_secondInputListWidget->clear();

  if(!values.empty())
    m_ui->m_addAllToolButton->setEnabled(true);
  else
    m_ui->m_addAllToolButton->setEnabled(false);

  for(size_t i = 0; i < values.size(); ++i)
  {
    QListWidgetItem* item = new QListWidgetItem(m_ui->m_secondInputListWidget);
    item->setText(QString::fromUtf8(values[i].c_str()));
    item->setData(Qt::UserRole, QVariant(1));

    m_ui->m_outputListWidget->addItem(item);
  }
}

void te::qt::widgets::DoubleInputListWidget::setOutputValues(
    const std::vector<std::string>& values, const int& id)
{
  m_ui->m_outputListWidget->clear();

  if(!values.empty())
    m_ui->m_removeAllToolButton->setEnabled(true);
  else
    m_ui->m_removeAllToolButton->setEnabled(false);

  for(size_t i = 0; i < values.size(); ++i)
  {
    QListWidgetItem* item = new QListWidgetItem(m_ui->m_outputListWidget);
    item->setText(QString::fromUtf8(values[i].c_str()));
    item->setData(Qt::UserRole, QVariant(id));

    m_ui->m_outputListWidget->addItem(item);
  }
}

std::vector<std::string>
te::qt::widgets::DoubleInputListWidget::getOutputValues()
{
  std::vector<std::string> vec;

  int count = m_ui->m_outputListWidget->count();

  for(int i = 0; i < count; ++i)
  {
    vec.push_back(m_ui->m_outputListWidget->item(i)->text().toUtf8().data());
  }

  return vec;
}

std::vector<int> te::qt::widgets::DoubleInputListWidget::getOutputDataValues()
{
  std::vector<int> vec;

  int count = m_ui->m_outputListWidget->count();

  for (int i = 0; i < count; ++i)
  {
    QVariant v = m_ui->m_outputListWidget->item(i)->data(Qt::UserRole);

    if (v.isValid())
      vec.push_back(v.toInt());
  }

  return vec;
}

void te::qt::widgets::DoubleInputListWidget::clearInputValues()
{
  m_ui->m_firstInputListWidget->clear();
  m_ui->m_secondInputListWidget->clear();

  m_ui->m_addToolButton->setEnabled(false);
  m_ui->m_addAllToolButton->setEnabled(false);
}

void te::qt::widgets::DoubleInputListWidget::clearOutputValues()
{
  m_ui->m_outputListWidget->clear();

  m_ui->m_removeToolButton->setEnabled(false);
  m_ui->m_removeAllToolButton->setEnabled(false);
}

void te::qt::widgets::DoubleInputListWidget::setVisibledOrderButtons(bool state)
{
  m_ui->m_upToolButton->setVisible(state);
  m_ui->m_downToolButton->setVisible(state);
}

void te::qt::widgets::DoubleInputListWidget::onAddToolButtonPressed()
{
  QList<QListWidgetItem*> firstInputList =
      m_ui->m_firstInputListWidget->selectedItems();

  QList<QListWidgetItem*> secondInputList =
      m_ui->m_secondInputListWidget->selectedItems();

  if(!firstInputList.empty() || !secondInputList.empty())
    m_ui->m_removeAllToolButton->setEnabled(true);

  for(int i = 0; i < firstInputList.size(); ++i)
  {
    int row = m_ui->m_firstInputListWidget->row(firstInputList[i]);

    QListWidgetItem* item = m_ui->m_firstInputListWidget->takeItem(row);

    m_ui->m_outputListWidget->addItem(item);
  }

  for(int i = 0; i < secondInputList.size(); ++i)
  {
    int row = m_ui->m_secondInputListWidget->row(secondInputList[i]);

    QListWidgetItem* item = m_ui->m_secondInputListWidget->takeItem(row);

    m_ui->m_outputListWidget->addItem(item);
  }

  if(m_ui->m_firstInputListWidget->count() == 0 &&
     m_ui->m_secondInputListWidget->count() == 0)
  {
    m_ui->m_addToolButton->setEnabled(false);
    m_ui->m_addAllToolButton->setEnabled(false);
  }

  emit itemChanged();
}

void te::qt::widgets::DoubleInputListWidget::onAddAllToolButtonPressed()
{
  m_ui->m_firstInputListWidget->selectAll();
  m_ui->m_secondInputListWidget->selectAll();

  QList<QListWidgetItem*> firstList =
      m_ui->m_firstInputListWidget->selectedItems();
  QList<QListWidgetItem*> secondList =
      m_ui->m_secondInputListWidget->selectedItems();

  if(!firstList.empty() || !secondList.empty())
    m_ui->m_removeAllToolButton->setEnabled(true);

  for(int i = 0; i < firstList.size(); ++i)
  {
    int row = m_ui->m_firstInputListWidget->row(firstList[i]);

    QListWidgetItem* item = m_ui->m_firstInputListWidget->takeItem(row);

    m_ui->m_outputListWidget->addItem(item);
  }

  for(int i = 0; i < secondList.size(); ++i)
  {
    int row = m_ui->m_secondInputListWidget->row(secondList[i]);

    QListWidgetItem* item = m_ui->m_secondInputListWidget->takeItem(row);

    m_ui->m_outputListWidget->addItem(item);
  }

  m_ui->m_addToolButton->setEnabled(false);
  m_ui->m_addAllToolButton->setEnabled(false);

  emit itemChanged();
}

void te::qt::widgets::DoubleInputListWidget::onRemoveToolButtonPressed()
{
  QList<QListWidgetItem*> list = m_ui->m_outputListWidget->selectedItems();

  if(list.empty() == false)
    m_ui->m_addAllToolButton->setEnabled(true);

  for(int i = 0; i < list.size(); ++i)
  {
    int row = m_ui->m_outputListWidget->row(list[i]);

    QListWidgetItem* item = m_ui->m_outputListWidget->takeItem(row);

    QVariant v = item->data(Qt::UserRole);

    if(v.isValid())
    {
      int id = v.toInt();

      if(id == 0)
        m_ui->m_firstInputListWidget->addItem(item);
      else
        m_ui->m_secondInputListWidget->addItem(item);
    }
  }

  if(m_ui->m_outputListWidget->count() == 0)
  {
    m_ui->m_removeToolButton->setEnabled(false);
    m_ui->m_removeAllToolButton->setEnabled(false);
    m_ui->m_upToolButton->setEnabled(false);
    m_ui->m_downToolButton->setEnabled(false);
  }

  emit itemChanged();
}

void te::qt::widgets::DoubleInputListWidget::onRemoveAllToolButtonPressed()
{
  m_ui->m_outputListWidget->selectAll();

  QList<QListWidgetItem*> list = m_ui->m_outputListWidget->selectedItems();

  if(list.empty() == false)
    m_ui->m_addAllToolButton->setEnabled(true);

  for(int i = 0; i < list.size(); ++i)
  {
    int row = m_ui->m_outputListWidget->row(list[i]);

    QListWidgetItem* item = m_ui->m_outputListWidget->takeItem(row);

    QVariant v = item->data(Qt::UserRole);

    if(v.isValid())
    {
      int id = v.toInt();

      if(id == 0)
        m_ui->m_firstInputListWidget->addItem(item);
      else
        m_ui->m_secondInputListWidget->addItem(item);
    }
  }

  m_ui->m_removeToolButton->setEnabled(false);
  m_ui->m_removeAllToolButton->setEnabled(false);
  m_ui->m_upToolButton->setEnabled(false);
  m_ui->m_downToolButton->setEnabled(false);

  emit itemChanged();
}

void te::qt::widgets::DoubleInputListWidget::onDownToolButtonPressed()
{
  int count = m_ui->m_outputListWidget->count();

  if(count == m_ui->m_outputListWidget->selectedItems().size())
    return;

  for(int i = count-1; i >= 0; --i)
  {
    QListWidgetItem* currentItem = m_ui->m_outputListWidget->item(i);

    if(i == count-1)
      continue;

    if(currentItem->isSelected())
    {
      QListWidgetItem* itemAfter = m_ui->m_outputListWidget->item(i+1);

      bool wasSelected = itemAfter->isSelected();

      m_ui->m_outputListWidget->takeItem(i);
      m_ui->m_outputListWidget->takeItem(i);

      m_ui->m_outputListWidget->insertItem(i, itemAfter);
      m_ui->m_outputListWidget->insertItem(i+1, currentItem);

      currentItem->setSelected(true);

      if(wasSelected)
        itemAfter->setSelected(true);
    }
  }
}

void te::qt::widgets::DoubleInputListWidget::onUpToolButtonPressed()
{
  int count = m_ui->m_outputListWidget->count();

  if(count == m_ui->m_outputListWidget->selectedItems().size())
    return;

  for(int i = 0; i < count; ++i)
  {
    QListWidgetItem* currentItem = m_ui->m_outputListWidget->item(i);

    if(i == 0)
      continue;

    if(currentItem->isSelected())
    {
      QListWidgetItem* itemBefore = m_ui->m_outputListWidget->item(i-1);

      bool wasSelected = itemBefore->isSelected();

      m_ui->m_outputListWidget->takeItem(i);
      m_ui->m_outputListWidget->takeItem(i-1);

      m_ui->m_outputListWidget->insertItem(i-1, currentItem);
      m_ui->m_outputListWidget->insertItem(i, itemBefore);

      currentItem->setSelected(true);

      if(wasSelected)
        itemBefore->setSelected(true);
    }
  }
}

void te::qt::widgets::DoubleInputListWidget::
    onFirstInputListItemSelectionChanged()
{
  if(m_ui->m_firstInputListWidget->selectedItems().size() > 0)
  {
    m_ui->m_addToolButton->setEnabled(true);
    m_ui->m_addAllToolButton->setEnabled(true);
  }
  else
  {
    m_ui->m_addToolButton->setEnabled(false);
  }
}

void te::qt::widgets::DoubleInputListWidget::
    onSecondInputListItemSelectionChanged()
{
  if(m_ui->m_secondInputListWidget->selectedItems().size() > 0)
  {
    m_ui->m_addToolButton->setEnabled(true);
    m_ui->m_addAllToolButton->setEnabled(true);
  }
  else
  {
    m_ui->m_addToolButton->setEnabled(false);
  }
}

void te::qt::widgets::DoubleInputListWidget::onOutputListItemSelectionChanged()
{
  if(m_ui->m_outputListWidget->selectedItems().size() > 0)
  {
    m_ui->m_removeToolButton->setEnabled(true);
    m_ui->m_removeAllToolButton->setEnabled(true);
    m_ui->m_upToolButton->setEnabled(true);
    m_ui->m_downToolButton->setEnabled(true);
  }
  else
  {
    m_ui->m_removeToolButton->setEnabled(false);
    m_ui->m_upToolButton->setEnabled(false);
    m_ui->m_downToolButton->setEnabled(false);
  }
}

