/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/widgets/utils/DoubleInputListWidget.h

  \brief ...
*/

#ifndef __TERRALIB_QT_WIDGETS_UTILS_INTERNAL_DOUBLEINPUTLISTWIDGET_H
#define __TERRALIB_QT_WIDGETS_UTILS_INTERNAL_DOUBLEINPUTLISTWIDGET_H

// TerraLib
#include "../Config.h"

// STL
#include <memory>

// Qt
#include <QWidget>

class QListWidgetItem;

namespace Ui { class DoubleInputListWidgetForm; }

namespace te
{
  namespace qt
  {
    namespace widgets
    {
      /*!
        \class DoubleInputListWidget

        \brief ...
      */
      class TEQTWIDGETSEXPORT DoubleInputListWidget : public QWidget
      {
        Q_OBJECT

        public:

          DoubleInputListWidget(QWidget* parent = 0, Qt::WindowFlags f = 0);

          ~DoubleInputListWidget();

          Ui::DoubleInputListWidgetForm* getForm() const;

          void setFirstLayerLabel(const QString& value);

          void setSecondLayerLabel(const QString& value);

          void setOutputLabel(const QString& value);

          void setFirstInputValues(const std::vector<std::string>& values);

          void setSecondInputValues(const std::vector<std::string>& values);

          void setOutputValues(const std::vector<std::string>& values,
                               const int& id);

          std::vector<std::string> getOutputValues();

          std::vector<int> getOutputDataValues();

          void clearInputValues();

          void clearOutputValues();

          void setVisibledOrderButtons(bool state);

        protected slots:

          void onAddToolButtonPressed();

          void onAddAllToolButtonPressed();

          void onRemoveToolButtonPressed();

          void onRemoveAllToolButtonPressed();

          void onFirstInputListItemSelectionChanged();

          void onSecondInputListItemSelectionChanged();

          void onOutputListItemSelectionChanged();

          void onUpToolButtonPressed();

          void onDownToolButtonPressed();

        signals:

          void itemChanged();

        private:

          std::unique_ptr<Ui::DoubleInputListWidgetForm> m_ui;
      }; 

    } // end namespace widgets
  }   // end namespace qt
}     // end namespace te

#endif  // TERRALIB_QT_WIDGETS_UTILS_INTERNAL_DOUBLEINPUTLISTWIDGET_H


