/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

This file is part of the TerraLib - a Framework for building GIS enabled applications.

TerraLib is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License,
or (at your option) any later version.

TerraLib is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with TerraLib. See COPYING. If not, write to
TerraLib Team at <terralib-team@terralib.org>.
*/

/*!
 * \file InputCoordinateDialog.h
 *
 * \brief Implements a dialog used to centralize the area of visualization in some coordinate.
 */

#ifndef __TERRALIB_QT_WIDGETS_INTERNAL_INPUTCOORDINATEDIALOG_H
#define __TERRALIB_QT_WIDGETS_INTERNAL_INPUTCOORDINATEDIALOG_H

#include "../Config.h"

// Qt
#include <QDialog>

namespace Ui
{
  // Forward declaration
  class InputCoordinateForm;
}

// Forward declarations
class QValidator;

namespace te
{
  namespace gm
  {
    // Forward declaration.
    struct Coord2D;
  }

  namespace qt
  {
    namespace widgets
    {
      // Forward declaration
      class MapDisplay;

      /*!
       * \class InputCoordinateDialog
       *
       * \brief A dialog that gets a coordinate ad centralize the area of visualization at it.
       */
      class TEQTWIDGETSEXPORT InputCoordinateDialog : public QDialog
      {
        Q_OBJECT

      public:

        /*!
         * \brief Constructor.
         *
         * \param display The display to be used.
         */
        InputCoordinateDialog(MapDisplay* display);

        /*!
         * \brief Destructor.
         */
        ~InputCoordinateDialog();

      protected slots:

        /*!
         * \brief Called when the degree-minute-second button was selected.
         */
        void DMS_triggered();

        /*!
         * \brief Called when the decimal-degree button was selected.
         */
        void DD_triggered();

        /*!
         * \brief Called when the \a Ok button was pressed.
         */
        void ok_triggered();

        /*!
         * \brief Called when the \a Cancel button was pressed.
         */
        void cancel_triggered();

        /*!
         * \brief Called when the line-edits changed its valid state.
         */
        void validatorStateChanged();

      protected:

        /*!
         * \brief Updates the validator and the mask being used on coordinate line edit.
         *
         * \param isDMS If \a true mask and validator for the degree-minute-second input type, and false for decimal-degree validators.
         */
        void updateValidator(const bool& isDMS);

        /*!
         * \brief Returns the coordinate of the input.
         *
         * \return Coordinate captured by the dialog.
         */
        te::gm::Coord2D* getCoordinate();

        Ui::InputCoordinateForm* m_ui;  //!< Pointer for Qt structure

        QValidator* m_validator;        //!< Validator being used.

        MapDisplay* m_display;          //!< Map display to show the result.
      };
    }
  }
}

#endif // __TERRALIB_QT_WIDGETS_INTERNAL_INPUTCOORDINATEDIALOG_H
