/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/qt/widgets/utils/ParameterDialog.cpp

  \brief A dialog to be used as input of key/value pairs.
*/

// TerraLib
#include "ui/ui_ParameterDialogForm.h"
#include "ParameterDialog.h"


te::qt::widgets::ParameterDialog::ParameterDialog(QWidget* parent, Qt::WindowFlags f)
  : QDialog(parent, f),
    m_ui(new Ui::ParameterDialogForm)
{
  m_ui->setupUi(this);

  connect(m_ui->m_value1LineEdit, SIGNAL(textEdited(const QString&)), this, SLOT(onValue1TextEdited(const QString&)));
  connect(m_ui->m_value2LineEdit, SIGNAL(textEdited(const QString&)), this, SLOT(onValue2TextEdited(const QString&)));
}

te::qt::widgets::ParameterDialog::~ParameterDialog() = default;

QString te::qt::widgets::ParameterDialog::getLabel1() const
{
  return m_ui->m_value1Label->text().trimmed();
}

void te::qt::widgets::ParameterDialog::setLabel1(const QString& label)
{
  m_ui->m_value1Label->setText(label);
}

QString te::qt::widgets::ParameterDialog::getValue1() const
{
  return m_ui->m_value1LineEdit->text().trimmed();
}

void te::qt::widgets::ParameterDialog::setValue1(const QString& value)
{
  m_pvalue1 = value;
  m_ui->m_value1LineEdit->setText(value);
}

void te::qt::widgets::ParameterDialog::setValue1EchoMode(QLineEdit::EchoMode echoMode)
{
  m_ui->m_value1LineEdit->setEchoMode(echoMode);
}

QString te::qt::widgets::ParameterDialog::getLabel2() const
{
  return m_ui->m_value2Label->text().trimmed();
}

void te::qt::widgets::ParameterDialog::setLabel2(const QString& label)
{
  m_ui->m_value2Label->setText(label);
}

QString te::qt::widgets::ParameterDialog::getValue2() const
{
  return m_ui->m_value2LineEdit->text().trimmed();
}

void te::qt::widgets::ParameterDialog::setValue2(const QString& value)
{
  m_pvalue2 = value;
  m_ui->m_value2LineEdit->setText(value);
}

void te::qt::widgets::ParameterDialog::setValue2EchoMode(QLineEdit::EchoMode echoMode)
{
  m_ui->m_value2LineEdit->setEchoMode(echoMode);
}

void te::qt::widgets::ParameterDialog::onValue1TextEdited(const QString& nextTxt)
{
  if(nextTxt != m_pvalue1)
    m_ui->m_okPushButton->setEnabled(true);
  else if(m_pvalue2 == m_ui->m_value2LineEdit->text())
    m_ui->m_okPushButton->setEnabled(false);
}

void te::qt::widgets::ParameterDialog::onValue2TextEdited(const QString& nextTxt)
{
  if(nextTxt != m_pvalue2)
    m_ui->m_okPushButton->setEnabled(true);
  else if(m_pvalue1 == m_ui->m_value1LineEdit->text())
    m_ui->m_okPushButton->setEnabled(false);
}

