
#include "SearchLineEditWidget.h"

// Qt Includes
#include <QGridLayout>
#include <QMessageBox>

te::qt::widgets::CustomLineEdit::CustomLineEdit(QWidget *parent)
  : QLineEdit(parent)
{
}

te::qt::widgets::CustomLineEdit::~CustomLineEdit() = default;

void te::qt::widgets::CustomLineEdit::focusInEvent(QFocusEvent *e)
{
  QLineEdit::focusInEvent(e);
  emit(focussed(true));
}

void te::qt::widgets::CustomLineEdit::focusOutEvent(QFocusEvent *e)
{
  QLineEdit::focusOutEvent(e);
  emit(focussed(false));
}

te::qt::widgets::SearchLineEditWidget::SearchLineEditWidget(QWidget* parent) :
QWidget(parent),
m_lineEditBackgroundText(QString(""))
{
  //build ui
  QGridLayout* layout = new QGridLayout(this);
  layout->setContentsMargins(0, 0, 0, 0);
  layout->setHorizontalSpacing(2);
  layout->setVerticalSpacing(2);

  m_lineEdit = new te::qt::widgets::CustomLineEdit(this);
  m_searchToolButton = new QToolButton(this);
  m_clearToolButton = new QToolButton(this);

  layout->addWidget(m_lineEdit, 0, 0);
  layout->addWidget(m_searchToolButton, 0, 1);
  layout->addWidget(m_clearToolButton, 0, 2);

  m_searchToolButton->setIcon(QIcon::fromTheme("zoom"));
  m_searchToolButton->setToolTip(tr("Search"));
  m_clearToolButton->setIcon(QIcon::fromTheme("edit-clear"));
  m_clearToolButton->setToolTip(tr("Clear search"));

  //connects
  connect(m_lineEdit, SIGNAL(focussed(bool)), this, SLOT(onLineEditFocussed(bool)));
  connect(m_searchToolButton, SIGNAL(released()), this, SLOT(onSearchToolButtonTriggered()));
  connect(m_clearToolButton, SIGNAL(released()), this, SLOT(onClearToolButtonTriggered()));
}

te::qt::widgets::SearchLineEditWidget::~SearchLineEditWidget() = default;

void te::qt::widgets::SearchLineEditWidget::setSearchFieldTextBackground(QString str)
{
  m_lineEditBackgroundText = str;

  onLineEditFocussed(false);
}

void te::qt::widgets::SearchLineEditWidget::setSearchFieldValidator(QValidator* validator)
{
  m_lineEdit->setValidator(validator);
}

void te::qt::widgets::SearchLineEditWidget::onLineEditFocussed(bool hasFocus)
{
  if (hasFocus)
  {
    if (m_lineEdit->text() == m_lineEditBackgroundText)
    {
      m_lineEdit->clear();

      QPalette palette = m_lineEdit->palette();
      palette.setColor(QPalette::Text, Qt::black);

      QFont font = m_lineEdit->font();
      font.setItalic(false);

      m_lineEdit->setPalette(palette);
      m_lineEdit->setFont(font);
    }
  }
  else
  {
    if (m_lineEdit->text().isEmpty())
    {
      QPalette palette = m_lineEdit->palette();
      palette.setColor(QPalette::Text, Qt::gray);

      QFont font = m_lineEdit->font();
      font.setItalic(true);

      m_lineEdit->setPalette(palette);
      m_lineEdit->setFont(font);

      m_lineEdit->setText(m_lineEditBackgroundText);
    }
  }
}

void te::qt::widgets::SearchLineEditWidget::onSearchToolButtonTriggered()
{
  if (m_lineEdit->text().isEmpty() || m_lineEdit->text() == m_lineEditBackgroundText)
  {
    QMessageBox::warning(this, tr("Warning"), tr("Search field not defined."));
    return;
  }

  emit search(m_lineEdit->text());
}

void te::qt::widgets::SearchLineEditWidget::onClearToolButtonTriggered()
{
  m_lineEdit->clear();

  m_lineEdit->clearFocus();

  onLineEditFocussed(false);

  emit clearSearch();
}
