/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file SearchLineEditWidget.h
  
  \brief Defines a component used as a search field.
*/
#ifndef __TERRALIB_QT_WIDGETS_INTERNAL_SEARCHLINEEDITWIDGET_H
#define __TERRALIB_QT_WIDGETS_INTERNAL_SEARCHLINEEDITWIDGET_H

#include <terralib/qt/widgets/Config.h>

//! Qt include files
#include <QLineEdit>
#include <QToolButton>
#include <QValidator>
#include <QWidget>

namespace te
{
  namespace qt
  {
    namespace widgets
    {

      /*!
        \class CustomLineEdit

        \brief Reimplements QLineEdit to get focus event
      */
      class CustomLineEdit : public QLineEdit
      {
        Q_OBJECT

      public:
        CustomLineEdit(QWidget* parent = 0);
        ~CustomLineEdit();

      signals:
        void focussed(bool hasFocus);

      protected:
        virtual void focusInEvent(QFocusEvent* e);
        virtual void focusOutEvent(QFocusEvent* e);
      };

      /*!
        \class SearchLineEditWidget
        
        \brief Defines a component used as a search field.
      */
      class TEQTWIDGETSEXPORT SearchLineEditWidget : public QWidget
      {
        Q_OBJECT
        
        public:
        
          /*!
            \brief Constructor.
          */
          SearchLineEditWidget(QWidget* parent=0);

          /*!
            \brief Destructor.
          */
          ~SearchLineEditWidget();

        public:

          void setSearchFieldTextBackground(QString str);

          void setSearchFieldValidator(QValidator* validator);

        protected slots:

          void onLineEditFocussed(bool hasFocus);

          void onSearchToolButtonTriggered();

          void onClearToolButtonTriggered();

      signals:

          void search(QString str);

          void clearSearch();

        protected:

          te::qt::widgets::CustomLineEdit* m_lineEdit;
          QToolButton* m_searchToolButton;
          QToolButton* m_clearToolButton;
          QString m_lineEditBackgroundText;

      };
    }
  }
}

#endif //! __TERRALIB_QT_WIDGETS_INTERNAL_SEARCHLINEEDITWIDGET_H