/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/raster/BandSummary.cpp
 
  \brief A struct that can be used to keep the summary of a raster band (most statistics).
*/

// TerraLib
#include "BandSummary.h"

te::rst::BandSummary::BandSummary()
  : m_minVal(nullptr),
    m_maxVal(nullptr),
    m_stdVal(nullptr),
    m_meanVal(nullptr),
    m_histogramR(nullptr),
    m_histogramI(nullptr)
{
}

te::rst::BandSummary::BandSummary(const BandSummary& rhs)
  : m_minVal(nullptr),
    m_maxVal(nullptr),
    m_stdVal(nullptr),
    m_meanVal(nullptr),
    m_histogramR(nullptr),
    m_histogramI(nullptr)
{
  m_minVal = rhs.m_minVal ? new std::complex<double>(*rhs.m_minVal) : nullptr;
  m_maxVal = rhs.m_maxVal ? new std::complex<double>(*rhs.m_maxVal) : nullptr;
  m_stdVal = rhs.m_stdVal ? new std::complex<double>(*rhs.m_stdVal) : nullptr;
  m_meanVal = rhs.m_meanVal ? new std::complex<double>(*rhs.m_meanVal) : nullptr;
  m_histogramR = rhs.m_histogramR ? new std::map<double, unsigned int>(*rhs.m_histogramR) : nullptr;
  m_histogramI = rhs.m_histogramI ? new std::map<double, unsigned int>(*rhs.m_histogramI) : nullptr;
}

te::rst::BandSummary::~BandSummary()
{
  clear();
}

te::rst::BandSummary& te::rst::BandSummary::operator=(const BandSummary& rhs)
{
  if(this != &rhs)
  {
    clear();

    m_minVal = rhs.m_minVal ? new std::complex<double>(*rhs.m_minVal) : nullptr;
    m_maxVal = rhs.m_maxVal ? new std::complex<double>(*rhs.m_maxVal) : nullptr;
    m_stdVal = rhs.m_stdVal ? new std::complex<double>(*rhs.m_stdVal) : nullptr;
    m_meanVal = rhs.m_meanVal ? new std::complex<double>(*rhs.m_meanVal) : nullptr;
    m_histogramR = rhs.m_histogramR ? new std::map<double, unsigned int>(*rhs.m_histogramR) : nullptr;
    m_histogramI = rhs.m_histogramI ? new std::map<double, unsigned int>(*rhs.m_histogramI) : nullptr;
  }

  return *this;
}

void te::rst::BandSummary::clear()
{
  delete m_minVal;
  m_minVal = nullptr;

  delete m_maxVal;
  m_maxVal = nullptr;

  delete m_stdVal;
  m_stdVal = nullptr;

  delete m_meanVal;
  m_meanVal = nullptr;

  delete m_histogramR;
  m_histogramR = nullptr;

  delete m_histogramI;
  m_histogramI = nullptr;
}

