/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/clouddetection/CloudDetection.h

  \brief Class to provide cloud detection.
*/

#ifndef __TERRALIB_RP_INTERNAL_CLOUDDETECTION_H
#define __TERRALIB_RP_INTERNAL_CLOUDDETECTION_H

// TerraLib
#include "../rp/Filter.h"
#include "../dataaccess/datasource/DataSourceInfo.h"
#include "../maptools/AbstractLayer.h"
#include "Config.h"

namespace te
{

  namespace ds
  {
    class DataSetType;
    class DataSet;
  }

  namespace gm
  {
    class Envelope;
  }

  namespace mem
  {
    class DataSet;
  }

  namespace rst
  {
    class Raster;
  }

  namespace rp
  {
    /*!
      \class CloudDetection

      \brief This class provide cloud detection.
    */
    class TERPEXPORT CloudDetection
    {
      public:

        CloudDetection();

        ~CloudDetection();

        /*!
        \brief This method is used to run cloud detection operation
        */
        te::rst::Raster* executeCloudDetection(te::rst::Raster* cloudRaster, te::rst::Raster* shadowRaster, int cloudBand, int shadowBand, double minCloud, double maxCloud,
          double minShadow, double maxShadow, std::vector<te::rp::Filter::InputParameters::FilterType> vecFilter,
          std::vector<int> vecIt);

      protected:

        te::rst::Raster* generateCloudRaster(te::rst::Raster* raster, int cloudBand, double minCloud, double maxCloud);

        te::rst::Raster* generateShadowRaster(te::rst::Raster* raster, int shadowBand, double minShadow, double maxShadow);
        
        te::rst::Raster* generateSumRasters(te::rst::Raster* cloudRaster, te::rst::Raster* shadowRaster);
    };
  }
}


#endif // __TERRALIB_RP_INTERNAL_ClOUDDETECTION_H
