/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

#ifndef __TERRALIB_RP_INTERNAL_COMPOSER_RULE_VALID_PIXEL_H
#define __TERRALIB_RP_INTERNAL_COMPOSER_RULE_VALID_PIXEL_H

#include "ComposerRule.h"
#include "../srs/Converter.h"
#include "../raster/Interpolator.h"

#include <memory>

namespace te
{
  namespace rp
  {
    /*!
      \class ComposerRuleValidPixel A basic composer rule capable of choosing the valid pixels (all bands with no no-data values).
      \brief A basic composer rule capable of choosing the valid pixels (all bands with no no-data values).
      \ingroup rp_optical
     */
    class TERPEXPORT ComposerRuleValidPixel : public ComposerRule
    {
      public:
        
        ComposerRuleValidPixel();

        ~ComposerRuleValidPixel();

      protected:
        
        // overload
        void getComposedValues( const double& line, const double& col, 
          double* const composedValuesPtr ) const;

        // overload
        ComposerRule* clone() const;
        
        // overload
        bool initialize( 
          const te::rst::Raster& raster1, 
          const std::vector< unsigned int >& raster1Bands, 
          const te::rst::Raster& raster2, 
          const std::vector< unsigned int >& raster2Bands,
          te::rst::Raster& raster3, 
          const std::vector< unsigned int >& raster3Bands,
          const te::rst::Interpolator::Method& interpMethod1,
          const te::rst::Interpolator::Method& interpMethod2,
          const double& noDataValue,
          const bool forceRaster1NoDataValue,
          const bool forceRaster2NoDataValue,
          const std::vector< double >& pixelOffsets1,
          const std::vector< double >& pixelScales1,
          const std::vector< double >& pixelOffsets2,
          const std::vector< double >& pixelScales2 );
        
      private :
        
        std::size_t m_raster1BandsSize;
        
        te::srs::Converter m_converter1;  //<! Raster 1 converter.
        
        te::srs::Converter m_converter2;  //<! Raster 2 converter.
        
        std::unique_ptr< te::rst::Interpolator > m_interpolator1Ptr;  //<! Raster 1 interpolator.
        
        std::unique_ptr< te::rst::Interpolator > m_interpolator2Ptr;  //<! Raster 1 interpolator.
        
        std::vector< std::complex< double > > m_noDataValues1;
        
        std::vector< std::complex< double > > m_noDataValues2;        
        
        // variables used by getComposedValues method
        mutable bool m_getComposedValues_pixelIsValid;
        mutable std::size_t m_getComposedValues_rasterBandsIdx;
        mutable std::complex< double > m_getComposedValues_PixelValue;
        mutable double m_getComposedValues_raster3X;
        mutable double m_getComposedValues_raster3Y;
        mutable double m_getComposedValues_rasterX;
        mutable double m_getComposedValues_rasterY;        
        mutable double m_getComposedValues_rasterRow;
        mutable double m_getComposedValues_rasterCol;
        
        ComposerRuleValidPixel( const ComposerRuleValidPixel& other );
        
        const ComposerRuleValidPixel& operator=( const ComposerRuleValidPixel& other );        
        
        void initState();
        
        void clear();

    };

  } // end namespace rp
}   // end namespace te

#endif

