/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/rp/Module.cpp
   
  \brief This singleton defines the TerraLib Raster Processing module entry.
 */

// TerraLib
#include "../core/logger/Logger.h"
#include "../common/TerraLib.h"
#include "../core/translator/Translator.h"
#include "Config.h"
#include "Module.h"
#include "Functions.h"

const te::rp::Module& sm_module = te::rp::Module::getInstance();
std::string te::rp::Module::m_lastLogString;
std::mutex te::rp::Module::m_mutex;

te::rp::Module::Module()
{
  m_module.reset(new TerraLib::Module(TE_RP_MODULE_NAME,
                         te::rp::Module::initialize,
                         te::rp::Module::finalize
                       ));

// initialize TerraLib singleton
  TerraLib::getInstance().add(m_module);
}

te::rp::Module::~Module() = default;

void te::rp::Module::initialize()
{
  std::map<std::string, struct SpectralSensorParams > specBandInfos;
  te::rp::GetAllSpectralBandInfos( specBandInfos );
  
  TE_LOG_TRACE(TE_TR("TerraLib Raster Processing module initialized!"));
}

void te::rp::Module::finalize()
{
  TE_LOG_TRACE(TE_TR("TerraLib Raster Processing module finalized!"));

  TerraLib::getInstance().remove(TE_RP_MODULE_NAME);
}

const std::string te::rp::Module::getLastLogStr()
{
  std::lock_guard<std::mutex> lock(m_mutex);
  return m_lastLogString;
}

void te::rp::Module::setLastLogStr( const std::string& errorStr )
{
  std::lock_guard<std::mutex> lock(m_mutex);
  m_lastLogString = errorStr;
}

