#include "PaletteRaster.h"

#include "../raster/BandProperty.h"

#include <assert.h>

#include <fstream> 

namespace te
{
  namespace rp
  {

    CubeColor::CubeColor(int ir, int ig, int ib)
    {
      m_ibr = ir;
      m_ibg = ig;
      m_ibb = ib;
      m_pop = 0.;
      m_igc = 0;
      m_mean[0] = m_mean[1] = m_mean[2] = (float)0.;
    }

    void
    CubeColor::InsertTripleRGB(int r, int g, int b)
    {
      double	newpop;

      newpop = m_pop + 1;
      m_mean[0] = (m_mean[0] * m_pop + r) / newpop;
      m_mean[1] = (m_mean[1] * m_pop + g) / newpop;
      m_mean[2] = (m_mean[2] * m_pop + b) / newpop;
      m_pop = newpop;
    }
    
    GroupColor::GroupColor(int ident)
    {
      m_id = ident;
      m_varmax = 0.;
      m_pop = 0.;
      for (int i = 0; i < 3; i++)
      {
        m_Ex[i] = m_Ex2[i] = 0.;
        m_var[i] = (double)0;
      }
      m_itCC = m_CClist.begin();
    }

    GroupColor :: ~GroupColor()
    {
    }

    void GroupColor::InsertCubeColor(CubeColor* cc)
    {
      double	r, g, b;

      cc->SetGroupColorIndex(m_id);
      m_CClist.push_back(cc);

      m_varmax = (double)0;

      r = (double)cc->Mean(0);
      g = (double)cc->Mean(1);
      b = (double)cc->Mean(2);

      m_Ex[0] = m_Ex[0] + r * cc->Pop();
      m_Ex[1] = m_Ex[1] + g * cc->Pop();
      m_Ex[2] = m_Ex[2] + b * cc->Pop();

      m_Ex2[0] = m_Ex2[0] + r * r * cc->Pop();
      m_Ex2[1] = m_Ex2[1] + g * g * cc->Pop();
      m_Ex2[2] = m_Ex2[2] + b * b * cc->Pop();

      m_pop += cc->Pop();

      m_var[0] = (m_Ex2[0] / m_pop) - (m_Ex[0] / m_pop * m_Ex[0] / m_pop);
      m_var[1] = (m_Ex2[1] / m_pop) - (m_Ex[1] / m_pop * m_Ex[1] / m_pop);
      m_var[2] = (m_Ex2[2] / m_pop) - (m_Ex[2] / m_pop * m_Ex[2] / m_pop);

      m_varmax = m_var[0];
      if (m_varmax < m_var[1]) m_varmax = m_var[1];
      if (m_varmax < m_var[2]) m_varmax = m_var[2];
    }

    CubeColor* GroupColor::RemoveCubeColor()
    {
      CubeColor	*cc = NULL;
      double		r, g, b;

      cc = *m_itCC;
      m_itCC = m_CClist.erase(m_itCC);

      m_varmax = (double)0;
      m_pop -= cc->Pop();
      if (m_pop <= 0.)
      {
        m_Ex[0] = m_Ex[1] = m_Ex[2] = 0.;
        m_Ex2[0] = m_Ex2[1] = m_Ex2[2] = 0.;
        m_var[0] = m_var[1] = m_var[2] = 0.;
        return cc;
      }

      r = (double)cc->Mean(0);
      g = (double)cc->Mean(1);
      b = (double)cc->Mean(2);

      m_Ex[0] = m_Ex[0] - r * cc->Pop();
      m_Ex[1] = m_Ex[1] - g * cc->Pop();
      m_Ex[2] = m_Ex[2] - b * cc->Pop();

      m_Ex2[0] = m_Ex2[0] - r * r * cc->Pop();
      m_Ex2[1] = m_Ex2[1] - g * g * cc->Pop();
      m_Ex2[2] = m_Ex2[2] - b * b * cc->Pop();

      m_var[0] = (m_Ex2[0] / m_pop) - (m_Ex[0] / m_pop * m_Ex[0] / m_pop);
      m_var[1] = (m_Ex2[1] / m_pop) - (m_Ex[1] / m_pop * m_Ex[1] / m_pop);
      m_var[2] = (m_Ex2[2] / m_pop) - (m_Ex[2] / m_pop * m_Ex[2] / m_pop);

      m_varmax = m_var[0];
      if (m_varmax < m_var[1]) m_varmax = m_var[1];
      if (m_varmax < m_var[2]) m_varmax = m_var[2];

      return cc;
    }

    GroupColor* GroupColor::Split(int newgcid)
    {
      GroupColor	*lc = NULL;
      CubeColor	*cc = NULL;
      int		dimvarmax;
      double		center;

      dimvarmax = 0;
      if (m_var[dimvarmax] < m_var[1]) dimvarmax = 1;
      if (m_var[dimvarmax] < m_var[2]) dimvarmax = 2;

      if ((lc = new GroupColor(newgcid)) == NULL)
      {
         return NULL;
      }

      center = Mean(dimvarmax);
      for (m_itCC = m_CClist.begin(); m_itCC != m_CClist.end(); )
      {
        cc = *m_itCC;
        if ((double)cc->Mean(dimvarmax) < center)
        {
          if (RemoveCubeColor() == NULL)
          {
            lc->InsertCubeColor(cc);
            break;
          }
          lc->InsertCubeColor(cc);

          if(m_itCC == m_CClist.end())
            break;
        }
        else
          ++m_itCC;
      } 

      return lc;
    }

    Codifima::Codifima(const te::rst::Raster& inputRaster,
      te::map::RasterTransform& rasterTransform,
      te::rst::Raster& outRasterPtr,
      int palettesize)
    {
      m_inputRaster = &inputRaster;
      m_rasterTransform = &rasterTransform;
      m_outputRaster = &outRasterPtr;
      m_outBand = outRasterPtr.getBand(0);
      m_outNCols = outRasterPtr.getNumberOfColumns();
      m_outNRows = outRasterPtr.getNumberOfRows();
      m_ngc = palettesize;
    }

    bool Codifima::Apply()
    {
      int i;

      try
      {
        m_GC = new GroupColor*[m_ngc];

        for (i = 0; i < m_ngc; i++)
          m_GC[i] = NULL;

        histogram_3d();

        if ((m_finalcolors = color_generation()) <= 0)
        {
          Clear();
          return false;
        }

        m_lut = new double*[m_finalcolors];
        for (i = 0; i < m_finalcolors; i++)
          m_lut[i] = new double[3];

        for (i = 0; i < 10; i++)
        {
          if (k_means() == false)
          {
            Clear();
            return false;
          }
        }

        if (image_codifying() == false)
        {
          Clear();

          return false;
        }

        for (i = 0; i < m_finalcolors; i++)
        {
          te::rst::BandProperty::ColorEntry color;
          color.c1 = (short)m_lut[i][0];
          color.c2 = (short)m_lut[i][1];
          color.c3 = (short)m_lut[i][2];
          color.c4 = 255;

          m_palette.push_back(color);
        }
      }
      catch (std::exception &e)
      {
        throw e;
      }

      return true;
    }
    
     void Codifima::Clear()
    {
      if (m_GC != NULL)
      {
        for (int i = 0; i < m_ngc; i++)
          if (m_GC[i] != NULL)
          { 
            delete m_GC[i];
            m_GC[i] = NULL;
          }

        delete[]m_GC;
        m_GC = NULL;
      }
      DeallocateHist();

      for (int i = 0; i < m_finalcolors; ++i)
      {
        delete m_lut[i];
      }
      delete m_lut;
    }

    bool Codifima::AllocateMatclass()
    {
      int	i, j, k;

      try
      {
        matclass = new int**[NCUBESDIM];
 
        for (i = 0; i < NCUBESDIM; i++)
        {
          matclass[i] = new int*[NCUBESDIM];

          for (j = 0; j < NCUBESDIM; j++)
          {
            matclass[i][j] = new int[NCUBESDIM];
 
            for (k = 0; k < NCUBESDIM; k++)
              matclass[i][j][k] = 0;
          }
        }
      }
      catch (std::exception &e)
      {
        throw e;
      }

      return true;
    }

    void Codifima::DeallocateMatclass()
    {
      int	i, j;

      if (matclass == NULL) return;
      for (i = 0; i < NCUBESDIM; i++)
      {
        if (matclass[i] != NULL)
        {
          for (j = 0; j < NCUBESDIM; j++)
            if (matclass[i][j] != NULL)
            {
              delete matclass[i][j];
              matclass[i][j] = NULL;
            }
          delete[]matclass[i];
          matclass[i] = NULL;
        }
      }
      delete[]matclass;
      matclass = NULL;
    }

    bool Codifima::AllocateHist()
    {
      int 	i, j, k;

      hist = new CubeColor***[NCUBESDIM];
 
      for (i = 0; i < NCUBESDIM; i++)
      {
        hist[i] = new CubeColor**[NCUBESDIM];

        for (j = 0; j < NCUBESDIM; j++)
        {
          hist[i][j] = new CubeColor*[NCUBESDIM];

          for (k = 0; k < NCUBESDIM; k++)
            hist[i][j][k] = NULL;
        }
      }
      return true;
    }


    void Codifima::DeallocateHist()
    {
      int	i, j;

      if (hist == NULL) return;
      for (i = 0; i < NCUBESDIM; i++)
      {
        if (hist[i] != NULL)
        {
          for (j = 0; j < NCUBESDIM; j++)
            if (hist[i][j] != NULL)
            {
              delete hist[i][j];
              hist[i][j] = NULL;
            }
          delete[]hist[i];
          hist[i] = NULL;
        }
      }
      delete[]hist;
      hist = NULL;
    }

    bool Codifima::histogram_3d()
    {
      double hr[256],
        hg[256],
        hb[256];
      unsigned int outRow = 0;
      unsigned int outCol = 0;
      int		i, j, k, nc;

      std::fill(hr, hr + 256, 0);
      std::fill(hg, hg + 256, 0);
      std::fill(hb, hb + 256, 0);

      try
      {
        // Allocates space for hist
        AllocateHist();

        // Computes the histogram for each band and the initial and final gray levels
        for (outRow = 0; outRow < m_outNRows; ++outRow)
        {
          for (outCol = 0; outCol < m_outNCols; ++outCol)
          {
            te::color::RGBAColor rgbacolor(0, 0, 0, 0);
            rgbacolor = m_rasterTransform->apply(outCol, outRow);

            int red_int = rgbacolor.getRed();
            int green_int = rgbacolor.getGreen();
            int blue_int = rgbacolor.getBlue();
            if (red_int >= 0 && red_int < 256)
              hr[red_int]++;
            if (green_int >= 0 && green_int < 256)
              hg[green_int]++;
            if (blue_int >= 0 && blue_int < 256)
              hb[blue_int]++;
          }
        }

        for (nc = 0; nc < 256; nc++)
          if (hr[nc] > 0) {
            m_firstvr = nc;
            break;
          }
        for (nc = 0; nc < 256; nc++)
          if (hg[nc] > 0) {
            m_firstvg = nc;
            break;
          }
        for (nc = 0; nc < 256; nc++)
          if (hb[nc] > 0) {
            m_firstvb = nc;
            break;
          }
        for (nc = 255; nc >= 0; nc--)
          if (hr[nc] > 0) {
            m_lastvr = nc;
            break;
          }
        for (nc = 255; nc >= 0; nc--)
          if (hg[nc] > 0) {
            m_lastvg = nc;
            break;
          }
        for (nc = 255; nc >= 0; nc--)
          if (hb[nc] > 0) {
            m_lastvb = nc;
            break;
          }

        float sider = ((float)(m_lastvr - m_firstvr + 1)) / NCUBESDIM;
        if (sider == 0.0)
          sider = (float)1.0;
        float sideg = ((float)(m_lastvg - m_firstvg + 1)) / NCUBESDIM;
        if (sideg == 0.0)
          sideg = (float)1.0;
        float sideb = ((float)(m_lastvb - m_firstvb + 1)) / NCUBESDIM;
        if (sideb == 0.0)
          sideb = (float)1.0;

        for (nc = 0; nc < 256; nc++)
        {
          m_blockr[nc] = (nc < m_firstvr) ? (0) : ((nc <= m_lastvr) ? ((short)((nc - m_firstvr) / sider)) : (NCUBESDIM - 1));
          m_blockg[nc] = (nc < m_firstvg) ? (0) : ((nc <= m_lastvg) ? ((short)((nc - m_firstvg) / sideg)) : (NCUBESDIM - 1));
          m_blockb[nc] = (nc < m_firstvb) ? (0) : ((nc <= m_lastvb) ? ((short)((nc - m_firstvb) / sideb)) : (NCUBESDIM - 1));
        }

        // Generates Histogram 3-d. All CubeColors created here are...
        // ... appended in the CClist.

        for (outRow = 0; outRow < m_outNRows; ++outRow)
        {
          for (outCol = 0; outCol < m_outNCols; ++outCol)
          {
            te::color::RGBAColor rgbacolor(0, 0, 0, 0);
            rgbacolor = m_rasterTransform->apply(outCol, outRow);

            int R = rgbacolor.getRed();
            int G = rgbacolor.getGreen();
            int B = rgbacolor.getBlue();

            if (R < 0 || R > 256 
              || G < 0 || G > 256 
              || B < 0 || B > 256)
              continue;

            i = (int)m_blockr[R];
            j = (int)m_blockg[G];
            k = (int)m_blockb[B];
            if (hist[i][j][k] == NULL) 
            {
              hist[i][j][k] = new CubeColor(i, j, k);
              m_CClist.push_back(hist[i][j][k]);
            }
            hist[i][j][k]->InsertTripleRGB(R, G, B);
          }
        }

        DeallocateHist();
      }
      catch (std::exception &e)
      {
        DeallocateHist();
        throw e;
      }
      return true;

    }
    
    int Codifima::color_generation()
    {
      double		maxvargroup;
      int		splitgroup;
      int		nextgcid;

      try {
        m_GC[0] = new GroupColor(0);

        // Creates the GroupColor 1
        m_GC[1] = new GroupColor(1);

        // Inserts all CubeColors in GroupColor 1
        for (std::list<CubeColor*>::iterator it = m_CClist.begin(); it != m_CClist.end(); ++it)
        {
          m_GC[1]->InsertCubeColor(*it);
        }
  
        // Generates the all others GroupColors
        for (nextgcid = 2; nextgcid < m_ngc; nextgcid++)
        {
          splitgroup = -1;
          maxvargroup = -1.;

          // Selects the GroupColor with biggest variance
          for (int indlut = 1; indlut < nextgcid; indlut++)
            if (maxvargroup < m_GC[indlut]->MaxVar() && m_GC[indlut]->GetNCubeColors() > 1)
            {
              maxvargroup = m_GC[indlut]->MaxVar();
              splitgroup = indlut;
            }

          if (splitgroup < 0 && maxvargroup <= 0.)  // There is no more colors to be generated
            return nextgcid;

          // Splits the GroupColor with biggest variance
          m_GC[nextgcid] = m_GC[splitgroup]->Split(nextgcid);
          if (m_GC[nextgcid] == NULL)
            return 0;
        }
      }
      catch(std::exception &e)
      {
        throw e;
      }

      return nextgcid;
    }

    bool Codifima::k_means()
    {
      int	quad[256];
      double			*histlut;
      double			**center;
      double			*histcenter;
      CubeColor	*cc;
      int		i, ind;
      int	dist;
      int	distmin;
      int	square;
      double 	r, g, b;
      int		cmin;

      try
      {
        histlut = new double[m_finalcolors];
        center = new double*[m_finalcolors];
        histcenter = new double[m_finalcolors];

        for (i = 0; i < m_finalcolors; i++) {
          center[i] = new double[3];
          m_lut[i][0] = m_lut[i][1] = m_lut[i][2] = 0.;
          center[i][0] = center[i][1] = center[i][2] = 0.;
          histcenter[i] = 0.;
          histlut[i] = 0.;
        }

        for (i = 0; i < 256; ++i)
          quad[i] = i * i;

        for (std::list<CubeColor*> ::iterator it = m_CClist.begin(); it != m_CClist.end(); ++it)
        {
          cc = *it;
          ind = cc->GroupColorIndex();
          center[ind][0] += cc->Mean(0) * cc->Pop();
          center[ind][1] += cc->Mean(1) * cc->Pop();
          center[ind][2] += cc->Mean(2) * cc->Pop();
          histcenter[ind] += cc->Pop();
        }

        for (i = 1; i < m_finalcolors; i++) {
          if (histcenter[i] <= 0.) 
            continue;;
          center[i][0] /= histcenter[i];
          center[i][1] /= histcenter[i];
          center[i][2] /= histcenter[i];
        }

        // Classification
        for (std::list<CubeColor*> ::iterator it = m_CClist.begin(); it != m_CClist.end(); ++it)
        {
          cc = *it;
          r = cc->Mean(0);
          g = cc->Mean(1);
          b = cc->Mean(2);
          cmin = cc->GroupColorIndex();
          square = (int)((center[cmin][0] > r) ? (center[cmin][0] - r) : (r - center[cmin][0]));
          distmin = quad[square];
          square = (int)((center[cmin][1] > g) ? (center[cmin][1] - g) : (g - center[cmin][1]));
          distmin += quad[square];
          square = (int)((center[cmin][2] > b) ? (center[cmin][2] - b) : (b - center[cmin][2]));
          distmin += quad[square];
          for (i = 1; i < m_finalcolors; i++) {
            square = (int)((center[i][0] > r) ? (center[i][0] - r) : (r - center[i][0]));
            dist = quad[square];
            if (dist > distmin)
              continue;
            square = (int)((center[i][1] > g) ? (center[i][1] - g) : (g - center[i][1]));
            dist += quad[square];
            if (dist > distmin)
              continue;
            square = (int)((center[i][2] > b) ? (center[i][2] - b) : (b - center[i][2]));
            dist += quad[square];
            if (dist < distmin) {
              distmin = dist;
              cmin = i;
            }
          }
          cc->SetGroupColorIndex(cmin);
          m_lut[cmin][0] += (r * cc->Pop());
          m_lut[cmin][1] += (g * cc->Pop());
          m_lut[cmin][2] += (b * cc->Pop());
          histlut[cmin] += cc->Pop();
        }

        for (i = 1; i < m_finalcolors; i++)
        {
          if (histlut[i] <= 0.)
            continue;
          if (m_lut[i][0] < 0.)
            continue;
          if (m_lut[i][1] < 0.)
            continue;
          if (m_lut[i][2] < 0.)
            continue;
          m_lut[i][0] /= histlut[i];
          m_lut[i][1] /= histlut[i];
          m_lut[i][2] /= histlut[i];
        }

        for (i = 0; i < m_finalcolors; ++i)
          delete center[i];

      }
      catch (std::exception &)
      {
        delete histlut;
        delete center;
        delete histcenter;
        return false;

      }

      delete histlut;
      delete center;
      delete histcenter;

      return true;
    }

    bool  Codifima::image_codifying()
    {
      // Allocates space for matclass
      if (AllocateMatclass() == false)
        return false;

      // Generates Classification matrix;
      for (std::list<CubeColor*> ::iterator it = m_CClist.begin(); it != m_CClist.end(); ++it)
      {
        matclass[(*it)->IndexR()][(*it)->IndexG()][(*it)->IndexB()] = (*it)->GroupColorIndex();
      } 

      // Classification process
      std::complex<double> valuer, valueg, valueb;

      for (unsigned int outRow = 0; outRow < m_outNRows; ++outRow)
      {
        for (unsigned int outCol = 0; outCol < m_outNCols; ++outCol)
        {
          te::color::RGBAColor rgbacolor(0, 0, 0, 0);
          rgbacolor = m_rasterTransform->apply(outCol, outRow);

          int R = rgbacolor.getRed();
          int G = rgbacolor.getGreen();
          int B = rgbacolor.getBlue();

          if (R < 0 || R > 256
            || G < 0 || G > 256
            || B < 0 || B > 256)
            continue;

          int i = (int)m_blockr[R];
          int j = (int)m_blockg[G];
          int k = (int)m_blockb[B];
          m_outBand->setValue(outCol, outRow, matclass[i][j][k]);
        }
      }
      DeallocateMatclass();
      return true;
    }
  } // end namespace rp
}   // end namespace te

