#include "SegmenterIdsManager.h"
#include "Macros.h"

namespace te
{
  namespace rp
  { 
    
    SegmenterIdsManager::SegmenterIdsManager()
    {
      m_segmentsIdsCounter = 0;
    }

    SegmenterIdsManager::~SegmenterIdsManager() = default;

    SegmenterSegmentsBlock::SegmentIdDataType SegmenterIdsManager::getNewID()
    {
      SegmenterSegmentsBlock::SegmentIdDataType newId = 0;
      
      m_mutex.lock();
      
      if( m_segmentsIdsCounter < std::numeric_limits<
        SegmenterSegmentsBlock::SegmentIdDataType>::max() )
      {
        ++m_segmentsIdsCounter;
        newId = m_segmentsIdsCounter;
      }
      else
      {
        m_mutex.unlock();
        TERP_LOG_AND_THROW( "Segments IDs limit reached" );
      }

      m_mutex.unlock();
      
      return newId;
    }
    
    bool SegmenterIdsManager::getNewIDs( 
      const unsigned int& idsNumber,
      std::vector< SegmenterSegmentsBlock::SegmentIdDataType >& ids )
    {
      ids.resize( idsNumber );
      
      m_mutex.lock();

      const SegmenterSegmentsBlock::SegmentIdDataType segmentsIdsCounterBegin =
        m_segmentsIdsCounter;
      
      for( unsigned int genIds = 0 ; genIds < idsNumber ; ++genIds )
      {
        if( m_segmentsIdsCounter == std::numeric_limits<
          SegmenterSegmentsBlock::SegmentIdDataType>::max() )
        {
          m_segmentsIdsCounter = segmentsIdsCounterBegin;
          m_mutex.unlock();
          return false;
        }
        else
        {
          ++m_segmentsIdsCounter;
          ids[ genIds ] = m_segmentsIdsCounter;
        }
      }
      
      m_mutex.unlock();
      
      return true;
    }    

  } // namespace rp
} // namespace te
