/*  Copyright (C) 2015 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

#include "SegmenterRegionGrowingStrategyParameters.h"
#include "AlgorithmParametersSerializer.h"
#include "Macros.h"

namespace te
{
  namespace rp
  {
    //-------------------------------------------------------------------------
    
    SegmenterRegionGrowingStrategyParameters::SegmenterRegionGrowingStrategyParameters()
    {
      reset();
    }

    SegmenterRegionGrowingStrategyParameters::~SegmenterRegionGrowingStrategyParameters() = default;

    const SegmenterRegionGrowingStrategyParameters& 
      SegmenterRegionGrowingStrategyParameters::operator=( 
      const SegmenterRegionGrowingStrategyParameters& params )
    {
      reset();

      SegmenterStrategyParameters::operator=( params );

      m_minSegmentSize = params.m_minSegmentSize;
      m_segmentsSimilarityThreshold = params.m_segmentsSimilarityThreshold;
      m_segmentsSimIncreaseSteps = params.m_segmentsSimIncreaseSteps;
      m_enableLocalMutualBestFitting = params.m_enableLocalMutualBestFitting;

      return *this;
    }

    void SegmenterRegionGrowingStrategyParameters::reset()
    {
      m_minSegmentSize = 1;
      m_segmentsSimilarityThreshold = 0;
      m_segmentsSimIncreaseSteps = 2;
      m_enableLocalMutualBestFitting = false;
    }

    bool SegmenterRegionGrowingStrategyParameters::serialize(
      AlgorithmParametersSerializer& serializer ) const
    {
      TERP_TRUE_OR_RETURN_FALSE( SegmenterStrategyParameters::serialize( serializer ),
        "Serialization error" );

      TERP_TRUE_OR_RETURN_FALSE( serializer.addParameter( "minSegmentSize",
        m_minSegmentSize ), "Error adding parameter minSegmentSize" );

      TERP_TRUE_OR_RETURN_FALSE( serializer.addParameter( "segmentsSimilarityThreshold",
        m_segmentsSimilarityThreshold ), "Error adding parameter segmentsSimilarityThreshold" );

      TERP_TRUE_OR_RETURN_FALSE( serializer.addParameter( "segmentsSimIncreaseSteps",
        m_segmentsSimIncreaseSteps ), "Error adding parameter segmentsSimIncreaseSteps" );

      TERP_TRUE_OR_RETURN_FALSE( serializer.addParameter( "enableLocalMutualBestFitting",
        m_enableLocalMutualBestFitting ), "Error adding parameter enableLocalMutualBestFitting" );

      return true;
    }
  } // end namespace rp
}   // end namespace te    

