/*  Copyright (C) 2015 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/rp/SegmenterRegionGrowingMeanStrategy.h
  \brief Raster region growing segmenter Mean strategy.
 */

#ifndef __TERRALIB_RP_INTERNAL_SEGMENTERREGIONGROWINGSTRATEGYPARAMETERS_H
#define __TERRALIB_RP_INTERNAL_SEGMENTERREGIONGROWINGSTRATEGYPARAMETERS_H

#include "SegmenterStrategyParameters.h"
#include "Config.h"

namespace te
{
  namespace rp
  {
    /*!
      \class SegmenterRegionGrowingStrategyParameters
      \brief Segmenter Region Growing Strategy Parameters
     */        
    class TERPEXPORT SegmenterRegionGrowingStrategyParameters 
    : public SegmenterStrategyParameters
    {
      public:

        unsigned int m_minSegmentSize; //!< A positive minimum segment size (pixels number - default: 1).

        double m_segmentsSimilarityThreshold; //!< Segments similarity treshold - Use lower values to merge only those segments that are more similar - Higher values will allow more segments to be merged - valid values range: positive values - default: 0 ).

        unsigned int m_segmentsSimIncreaseSteps; //!< The maximum number of steps to increment the similarity threshold value for the cases where no segment merge occurred - zero will disable segment similarity threshold increments - defaul: 2.

        bool m_enableLocalMutualBestFitting; //!< If enabled, a merge only occurs between two segments if the minimum dissimilarity criteria is best fulfilled mutually (default: false).

        virtual ~SegmenterRegionGrowingStrategyParameters();
        
        //overload 
        const SegmenterRegionGrowingStrategyParameters& operator=( 
          const SegmenterRegionGrowingStrategyParameters& params );
        
        //overload
        virtual void reset() _NOEXCEPT_OP(false);

        //overload
        virtual bool serialize( AlgorithmParametersSerializer& serializer ) const;
        
      protected :  
        
        SegmenterRegionGrowingStrategyParameters();
    };
  } // end namespace rp
}   // end namespace te

#endif  // __TERRALIB_RP_INTERNAL_SEGMENTERREGIONGROWINGSTRATEGYPARAMETERS_H
