/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

This file is part of the TerraLib - a Framework for building GIS enabled applications.

TerraLib is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License,
or (at your option) any later version.

TerraLib is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with TerraLib. See COPYING. If not, write to
TerraLib Team at <terralib-team@terralib.org>.
*/

/*!
\file terralib/rp/radar/GammaModel.cpp
\brief Gamma model data manipulation.
*/

// TerraLib includes
#include "GammaModel.h"
#include "../Macros.h"
#include "../RasterAttributes.h"
#include "../../raster/PositionIterator.h"
#include "../../common/MatrixUtils.h"
#include "../../common/progress/TaskProgress.h"

#include <fstream>

namespace te
{
	namespace rp
	{
		namespace radar
		{

			GammaModel::GammaModel() {
			}

			GammaModel::~GammaModel() {
			}

			// Loading samples
			bool GammaModel::LoadingSamples() {

				/*MClassesSamplesCTPtr allClassesSamples(new MClassesSamplesCT());
				ClassSamplesContainerT classSamples;
				ClassSampleT samples;

				for(unsigned int npol = 0; npol < m_polygonSamplesT.size(); npol++) {
				te::gm::Polygon* polygon = &m_polygonSamplesT[npol];

				// create iterators for band and polygon
				te::rst::PolygonIterator< double > it = te::rst::PolygonIterator< double >::
				begin(m_inputRasterPtr, polygon);
				const te::rst::PolygonIterator< double > itend = te::rst::PolygonIterator< double >::
				end(m_inputRasterPtr, polygon);

				while (it != itend) {

				unsigned int column = it.getColumn();
				unsigned int row = it.getRow();

				if (((row % (m_lagY + 1)) == 0) && ((column % (m_lagX + 1)) == 0)) {  // De-correlating samples input

				for(unsigned int band = 0; band < m_inputRastersBands.size(); band++)  //algoInputParams.m_inputRasterPtr[j]->getNumberOfBands()
				samples.push_back((*it)[band]);

				classSamples.push_back(samples);
				samples.clear();
				}

				++it;
				}

				if((npol + 1) == m_polygonSamplesT.size()) {
				allClassesSamples->insert(MClassesSamplesCT::value_type(m_classesIDT[npol], classSamples));
				continue;
				}

				else if(m_classesIDT[npol] != m_classesIDT[npol + 1]) {
				allClassesSamples->insert(MClassesSamplesCT::value_type(m_classesIDT[npol], classSamples));
				classSamples.clear();
				}
				}

				m_trainSamplesPtr = allClassesSamples;

				TERP_TRUE_OR_RETURN_FALSE(m_trainSamplesPtr,
				"Invalid Gamma classes training samples pointer")
				TERP_TRUE_OR_RETURN_FALSE(m_trainSamplesPtr->size() > 0,
				"Invalid Gamma classes training samples number")
				TERP_TRUE_OR_RETURN_FALSE(m_trainSamplesPtr->begin()->second.size() > 0,
				"Invalid Gamma classes training samples number")*/

				MClassesSamplesCTPtr allClassesSamples(new MClassesSamplesCT());
				ClassSamplesContainerT classSamples;
				ClassSampleT samples;

				ClassSamplesContainerT::const_iterator samplesIt =
					m_polygonSamplesTNew.begin();
				const ClassSamplesContainerT::const_iterator samplesItE =
					m_polygonSamplesTNew.end();

				int classID = 0;

				while (samplesIt != samplesItE) {

					for (int i = 0; i < m_numberOfSamples[classID]; ++i) {

						int row = (int)(samplesIt->operator[](i) / m_inputRasterPtr->getNumberOfColumns());
						int column = ((int)samplesIt->operator[](i) % m_inputRasterPtr->getNumberOfColumns());

						if (((row % (m_lagY + 1)) == 0) && ((column % (m_lagX + 1)) == 0)) {  // De-correlating samples input
							double value = 0.0;

							for (unsigned int band = 0; band < m_inputRastersBands.size(); ++band) {
								m_inputRasterPtr->getBand(band)->getValue(column, row, value);

								samples.push_back(value);
							}

							classSamples.push_back(samples);
							samples.clear();
						}
					}

					allClassesSamples->insert(MClassesSamplesCT::value_type(classID, classSamples));
					classSamples.clear();

					++classID;
					++samplesIt;
				}

				m_trainSamplesPtr = allClassesSamples;

				TERP_TRUE_OR_RETURN_FALSE(m_trainSamplesPtr,
					"Invalid Gamma classes training samples pointer")
					TERP_TRUE_OR_RETURN_FALSE(m_trainSamplesPtr->size() > 0,
					"Invalid Gamma classes training samples number")
					TERP_TRUE_OR_RETURN_FALSE(m_trainSamplesPtr->begin()->second.size() > 0,
					"Invalid Gamma classes training samples number")

					return true;
			}

			// Calculating Classes Lambda parameter
			bool GammaModel::CalculatingLambdaClass() {

				const unsigned int dimsNumber = (unsigned int)
					m_trainSamplesPtr->begin()->second.operator[](0).size();
				TERP_TRUE_OR_RETURN_FALSE(dimsNumber == 1, "Invalid band number");
//				std::cout << "          Number of bands: " << dimsNumber << std::endl;  // --->>>>> TESTE DO NUMERO DE CLASSES
				MClassesSamplesCT::const_iterator classesIt =
					m_trainSamplesPtr->begin();
				const MClassesSamplesCT::const_iterator classesItE =
					m_trainSamplesPtr->end();
				int classesNum = 0;  // --->>>>> TESTE DO NUMERO DE CLASSES
				while (classesIt != classesItE) {
					classesNum++;  // --->>>>> TESTE DO NUMERO DE CLASSES
					unsigned int samplesNum = 0;

					const ClassIDT& classID = classesIt->first;

					const ClassSamplesContainerT& classSamples = classesIt->second;
					TERP_TRUE_OR_RETURN_FALSE(classSamples.size() > 0,
						"Invalid class samples number");

					ClassSampleT classMeans(1, 0.0);

					unsigned int dimIdx = 0;

					for (dimIdx = 0; dimIdx < 1; ++dimIdx) {
						double& dimMean = classMeans[dimIdx];
						dimMean = 0.0;

						ClassSamplesContainerT::const_iterator samplesIt =
							classSamples.begin();
						const ClassSamplesContainerT::const_iterator samplesItE =
							classSamples.end();

						while (samplesIt != samplesItE) {
							TERP_TRUE_OR_RETURN_FALSE(samplesIt->size() == 1,
								"Sample size mismatch")

								if ((samplesIt->operator[](dimIdx) / 1) == samplesIt->operator[](dimIdx))  // avoiding nan
									dimMean += samplesIt->operator[](dimIdx);

							++samplesIt;
							++samplesNum;
						}

						dimMean /= static_cast<double>(classSamples.size());
					}

					m_lambdaClassesParameters.push_back(classMeans);
					m_classesIndex2IDLambda.push_back(classID);
					m_samplesSize.push_back(samplesNum);

					++classesIt;
				}
//				std::cout << "          Number of classes: " << classesNum << std::endl;  // --->>>>> TESTE DO NUMERO DE CLASSES

				return true;
			}

			// Calculating Regions Lambda parameter
			bool GammaModel::CalculatingLambdaRegion() {

				TERP_TRUE_OR_RETURN_FALSE(m_polygonsSegImage.size() > 0, "GammaModel algorithm needs polygons")

					TERP_TRUE_OR_RETURN_FALSE(m_inputRastersBands.size() == 1, "Invalid band number");

//				std::cout << "          Regions " << m_polygonsSegImage.size() << std::endl;

				for (unsigned int i = 0; i < m_polygonsSegImage.size(); i++) {

					unsigned int regionsNum = 0;

					m_lambdaRegionsParameters.push_back(0.0);

					te::rst::PolygonIterator<double> polygonsIt = te::rst::PolygonIterator<double>::
						begin(m_inputRasterPtr, m_polygonsSegImage[i]);
					te::rst::PolygonIterator<double> polygonsItE = te::rst::PolygonIterator<double>::
						end(m_inputRasterPtr, m_polygonsSegImage[i]);

					while (polygonsIt != polygonsItE) {

						unsigned int column = polygonsIt.getColumn();
						unsigned int row = polygonsIt.getRow();

						if (((row % (m_lagY + 1)) == 0) && ((column % (m_lagX + 1)) == 0)) {  // De-correlating regions

							if (((*polygonsIt)[0] / 1) == (*polygonsIt)[0])  // avoiding nan
								m_lambdaRegionsParameters[i] += (*polygonsIt)[0];

							++regionsNum;
						}

						++polygonsIt;
					}

					m_regionsSize.push_back(regionsNum);

					m_lambdaRegionsParameters[i] /= (double)regionsNum;
				}

				return true;
			}

			bool GammaModel::GettingStochasticDistances() {

				// Calculating Stochastic Distances

				double numerator = 0.0;
				double denominator = 0.0;
				std::vector<double> distances;

				for (unsigned int i = 0; i < m_lambdaRegionsParameters.size(); ++i) {

					ClassSamplesContainerT::const_iterator samplesIt =
						m_lambdaClassesParameters.begin();
					ClassSamplesContainerT::const_iterator samplesItE =
						m_lambdaClassesParameters.end();

					if (m_stochasticDistanceType == "Bhattacharyya") {  // Bhattacharyya distance

						while (samplesIt != samplesItE) {

							numerator = pow(m_lambdaRegionsParameters[i] + (*samplesIt)[0], m_ENL);
							double aux1 = m_lambdaRegionsParameters[i] * (*samplesIt)[0];
							double aux2 = m_ENL / 2.0;
							denominator = pow(2.0, m_ENL)*(pow(aux1, aux2));
							distances.push_back(log(numerator / denominator));

							++samplesIt;
						}

						m_stochasticDistance.push_back(distances);
						distances.clear();
					}

					if (m_stochasticDistanceType == "KullbackLeibler") {  // Kullback-Leibler distance

						while (samplesIt != samplesItE) {
							numerator = pow(m_lambdaRegionsParameters[i], 2) + pow((*samplesIt)[0], 2);
							denominator = 2 * m_lambdaRegionsParameters[i] * (*samplesIt)[0];
							distances.push_back(m_ENL * ((numerator / denominator) - 1));
							++samplesIt;
						}

						m_stochasticDistance.push_back(distances);
						distances.clear();
					}

					if (m_stochasticDistanceType == "Hellinger") {  // Hellinger distance

						while (samplesIt != samplesItE) {
							double aux1 = m_lambdaRegionsParameters[i] * (*samplesIt)[0];
							double aux2 = m_ENL / 2.0;
							numerator = pow(2, m_ENL) * (pow(aux1, aux2));
							denominator = pow(m_lambdaRegionsParameters[i] + (*samplesIt)[0], m_ENL);
							distances.push_back(1 - (numerator / denominator));
							++samplesIt;
						}

						m_stochasticDistance.push_back(distances);
						distances.clear();
					}

					if (m_stochasticDistanceType == "Renyi") {  // Renyi distance
						while (samplesIt != samplesItE) {
							numerator = m_lambdaRegionsParameters[i] * (*samplesIt)[0];
							denominator = (m_betaValue * (*samplesIt)[0] + (1 - m_betaValue)* m_lambdaRegionsParameters[i])
								*
								(m_betaValue * m_lambdaRegionsParameters[i] + (1 - m_betaValue) * (*samplesIt)[0]);
							distances.push_back((m_ENL / (2 * (m_betaValue - 1))) * log(numerator / denominator));
							++samplesIt;
						}

						m_stochasticDistance.push_back(distances);
						distances.clear();
					}
				}

				return true;
			}

			// Calculating Statistical Tests and P-Values

			bool GammaModel::GettingStatisticalTests() {

				double numerator = 0.0;
				double denominator = 0.0;
				std::vector< double > tests;
				std::vector< double > pValues;

				for (unsigned int i = 0; i < m_lambdaRegionsParameters.size(); ++i) {

					double degreesOfFreedom = 1;

					boost::math::chi_squared distChiSquared(degreesOfFreedom);

					ClassSamplesContainerT::const_iterator samples2It =
						m_lambdaClassesParameters.begin();
					ClassSamplesContainerT::const_iterator samples2ItE =
						m_lambdaClassesParameters.end();

					unsigned int samplesNum = 0;

					if (m_stochasticDistanceType == "Bhattacharyya") {  // Bhattacharyya distance
						while (samples2It != samples2ItE) {
							double a = (4 * m_regionsSize[i] * m_samplesSize[samplesNum]) / (m_regionsSize[i] + m_samplesSize[samplesNum]);
							numerator = m_lambdaRegionsParameters[i] + (*samples2It)[0];
							double aux = m_lambdaRegionsParameters[i] * (*samples2It)[0];
							denominator = 2 * sqrt(aux);
							double result = a * 2 * m_ENL * log(numerator / denominator);
							tests.push_back(result);

							try {
								pValues.push_back(boost::math::cdf(complement(distChiSquared, result)));
							}
							catch (...) {
								pValues.push_back(NAN);
							}

							++samples2It;
							++samplesNum;
						}

						m_statisticalTest.push_back(tests);
						m_pValue.push_back(pValues);
						tests.clear();
						pValues.clear();
					}

					if (m_stochasticDistanceType == "KullbackLeibler") {  // Kullback-Leibler distance
						while (samples2It != samples2ItE) {
							double a = (m_regionsSize[i] * m_samplesSize[samplesNum] * 2 * m_ENL) / (m_regionsSize[i] + m_samplesSize[samplesNum]);
							numerator = (m_lambdaRegionsParameters[i] * m_lambdaRegionsParameters[i]) + ((*samples2It)[0] * (*samples2It)[0]);
							denominator = 2 * m_lambdaRegionsParameters[i] * (*samples2It)[0];
							double result = a * ((numerator / denominator) - 1);
							tests.push_back(result);

							try {
								pValues.push_back(boost::math::cdf(complement(distChiSquared, result)));
							}
							catch (...) {
								pValues.push_back(NAN);
							}

							++samples2It;
							++samplesNum;
						}

						m_statisticalTest.push_back(tests);
						m_pValue.push_back(pValues);
						tests.clear();
						pValues.clear();
					}

					if (m_stochasticDistanceType == "Hellinger") {  // Hellinger distance
						while (samples2It != samples2ItE) {
							double a = (8.0 * m_regionsSize[i] * m_samplesSize[samplesNum]) / (double)(m_regionsSize[i] + m_samplesSize[samplesNum]);
							double aux1 = 2.0 * m_ENL / 2.0;
							double aux2 = 2.0 * m_ENL / 4.0;
							double aux3 = m_lambdaRegionsParameters[i] * (*samples2It)[0];
							numerator = (pow(2, aux1) * pow(aux3, aux2));
							denominator = pow(m_lambdaRegionsParameters[i] + (*samples2It)[0], aux1);
							double result = a * (1.0 - (numerator / denominator));
							tests.push_back(result);

							try {
								pValues.push_back(boost::math::cdf(complement(distChiSquared, result)));
							}
							catch (...) {
								pValues.push_back(NAN);
							}

							++samples2It;
							++samplesNum;
						}

						m_statisticalTest.push_back(tests);
						m_pValue.push_back(pValues);
						tests.clear();
						pValues.clear();
					}

					if (m_stochasticDistanceType == "Renyi") {  // Renyi distance
						while (samples2It != samples2ItE) {

							double a = (2 * m_regionsSize[i] * m_samplesSize[samplesNum] * 2 * m_ENL * m_betaValue
								*
								(m_betaValue - 1)) / (m_regionsSize[i] + m_samplesSize[samplesNum]);
							numerator = m_lambdaRegionsParameters[i] * (*samples2It)[0];
							denominator = (m_betaValue * (*samples2It)[0] + (1 - m_betaValue) * m_lambdaRegionsParameters[i])
								*
								(m_betaValue * m_lambdaRegionsParameters[i] + (1 - m_betaValue) * (*samples2It)[0]);
							double result = a * log(numerator / denominator);
							tests.push_back(result);

							try {
								pValues.push_back(boost::math::cdf(complement(distChiSquared, result)));
							}
							catch (...) {
								pValues.push_back(NAN);
							}

							++samples2It;
							++samplesNum;
						}

						m_statisticalTest.push_back(tests);
						m_pValue.push_back(pValues);
						tests.clear();
						pValues.clear();
					}
				}

				return true;
			}

			bool GammaModel::execute() {

				TERP_TRUE_OR_RETURN_FALSE(m_inputRastersBands.size() == 1,  //m_inputRasterPtr->getNumberOfBands()
					"Invalid m_inputRasterPtr Gamma Model number bands")

					m_samplesSize.clear();
				m_regionsSize.clear();
				m_lambdaClassesParameters.clear();
				m_lambdaRegionsParameters.clear();
				m_stochasticDistance.clear();
				m_statisticalTest.clear();
				m_pValue.clear();
				m_trainSamplesPtr = 0;

				if (!LoadingSamples())
				{
          TERP_LOG_AND_RETURN_FALSE( "LoadingSamples execution error" );
				}
				else
				{
//					std::cout << "LoadingSamples execution OK." << std::endl;
				}

				if (!CalculatingLambdaClass())
				{
					TERP_LOG_AND_RETURN_FALSE( "CalculatingLambdaClass execution error." );
				}
				else
				{
//					std::cout << "CalculatingLambdaClass execution OK." << std::endl;
				}

				if (!CalculatingLambdaRegion())
				{
					TERP_LOG_AND_RETURN_FALSE( "CalculatingLambdaRegion execution error." );
				}
				else
				{
//					std::cout << "CalculatingLambdaRegion execution OK." << std::endl;
				}

				if (!GettingStatisticalTests())
				{
					TERP_LOG_AND_RETURN_FALSE( "GettingStatisticalTests execution error." );
				}
				else
				{
//					std::cout << "GettingStatisticalTests execution OK." << std::endl;
				}

				if (!GettingStochasticDistances())
				{
					TERP_LOG_AND_RETURN_FALSE( "GettingStochasticDistances execution error." );
				}
				else
				{
//					std::cout << "GettingStochasticDistances execution OK." << std::endl;
				}

				return true;
			}

		} // end namespace radar
	} // end namespace rp
}   // end namespace msc
