
#include "MultiSourceClassifierInputParameters.h"
#include "../AlgorithmParametersSerializer.h"
#include "../Macros.h"

namespace te
{
	namespace rp
	{
		namespace radar
		{

			MultiSourceClassifierInputParameters::MultiSourceClassifierInputParameters() {
				reset();
			}

			MultiSourceClassifierInputParameters::MultiSourceClassifierInputParameters(const MultiSourceClassifierInputParameters& other) {
				reset();
				operator=(other);
			}

			MultiSourceClassifierInputParameters::~MultiSourceClassifierInputParameters() {
				reset();
			}

			void MultiSourceClassifierInputParameters::reset() {
				// General input parameters
				m_inputRasterPtr.clear();
				m_inputRastersBands.clear();
				m_rasterSegPtr = 0;
				m_numberOfColumns = 0;
				m_numberOfRows = 0;
				m_SRID = 0;
				m_extent = 0;
				m_polygonsSegImage.clear();
				m_sourcesNumber = 0;
				m_imageType.clear();
				m_savePerClassDistancesFile = false;
				m_savePerClassStatiticsFile = false;
				m_savePerClassPValueFile = false;
				m_saveDistances = false;
				m_saveStatitics = false;
				m_savePValues = false;
				m_saveReportFile = false;
				m_stochasticDistanceType.clear();
				m_lagX = 0;
				m_lagY = 0;
				m_significanceLevel = 0.0;
				// Radar input parameters
				m_radarDataRepresentationType.clear();
				m_vectorOrMatrixOrder.clear();
				m_ENL.clear();
				m_betaValue.clear();
				// Samples input
				m_polygonSamplesT.clear();
				m_polygonSamplesTNew.clear();
				m_polygonSamplesTestNew.clear();
				m_numberOfSamples.clear();
				m_numberOfSamplesTest.clear();
				m_classesIDT.clear();
				m_classesIDT2.clear();
				m_polygonSamplesTest.clear();
				m_classesIDTest.clear();
				m_numberOfClassesOnTraining = 0;
				m_numberOfClassesOnTest = 0;
//				m_datasets.clear();
			}

			const MultiSourceClassifierInputParameters& MultiSourceClassifierInputParameters::operator=(
				const MultiSourceClassifierInputParameters& params) {
				reset();

				// General input parameters
				m_inputRasterPtr = params.m_inputRasterPtr;
				m_inputRastersBands = params.m_inputRastersBands;
				m_rasterSegPtr = params.m_rasterSegPtr;
				m_numberOfColumns = params.m_numberOfColumns;
				m_numberOfRows = params.m_numberOfRows;
				m_SRID = params.m_SRID;
				m_extent = params.m_extent;
				m_polygonsSegImage = params.m_polygonsSegImage;
				m_sourcesNumber = params.m_sourcesNumber;
				m_imageType = params.m_imageType;
				m_stochasticDistanceType = params.m_stochasticDistanceType;
				m_savePerClassDistancesFile = params.m_savePerClassDistancesFile;
				m_savePerClassStatiticsFile = params.m_savePerClassStatiticsFile;
				m_savePerClassPValueFile = params.m_savePerClassPValueFile;
				m_saveDistances = params.m_saveDistances;
				m_saveStatitics = params.m_saveStatitics;
				m_savePValues = params.m_savePValues;
				m_saveReportFile = params.m_saveReportFile;
				m_combinationType = params.m_combinationType;
				m_lagX = params.m_lagX;
				m_lagY = params.m_lagY;
				m_significanceLevel = params.m_significanceLevel;
				// Radar input parameters
				m_radarDataRepresentationType = params.m_radarDataRepresentationType;
				m_vectorOrMatrixOrder = params.m_vectorOrMatrixOrder;
				m_ENL = params.m_ENL;
				m_betaValue = params.m_betaValue;
				m_dirTrainingSamples = params.m_dirTrainingSamples;
				m_dirTestSamples = params.m_dirTestSamples;
				// Samples input
				m_polygonSamplesT = params.m_polygonSamplesT;
				m_polygonSamplesTNew = params.m_polygonSamplesTNew;
				m_polygonSamplesTestNew = params.m_polygonSamplesTestNew;
				m_numberOfSamples = params.m_numberOfSamples;
				m_numberOfSamplesTest = params.m_numberOfSamplesTest;
				m_nameOfSamples = params.m_nameOfSamples;
				m_nameOfSamplesTest = params.m_nameOfSamplesTest;
				m_classesIDT = params.m_classesIDT;
				m_classesIDT2 = params.m_classesIDT2;
				m_polygonSamplesTest = params.m_polygonSamplesTest;
				m_classesIDTest = params.m_classesIDTest;
				m_numberOfClassesOnTraining = params.m_numberOfClassesOnTraining;
				m_numberOfClassesOnTest = params.m_numberOfClassesOnTest;
//				m_datasets = params.m_datasets;

				return *this;
			}

			te::common::AbstractParameters* MultiSourceClassifierInputParameters::clone() const {
				return new MultiSourceClassifierInputParameters(*this);
			}

			bool MultiSourceClassifierInputParameters::serialize(
        te::rp::AlgorithmParametersSerializer& serializer ) const
			{
        TERP_TRUE_OR_RETURN_FALSE( AlgorithmInputParameters::serialize( serializer ),
          "Serializarion error" );

        serializer.setAlgorithmName( "MultiSourceClassifier" );

        for (auto inRasterPtr: m_inputRasterPtr)
        {
          TERP_TRUE_OR_RETURN_FALSE( serializer.addParameter( "inputRaster",
            inRasterPtr->getName() ),
            "Error adding parameter inputRaster" );
        }

        TERP_TRUE_OR_RETURN_FALSE( serializer.addMultivaluedParameters(
          "inputRastersBands", m_inputRastersBands ),
          "Error adding parameter inputRastersBands" );

        TERP_TRUE_OR_RETURN_FALSE( serializer.addParameter( "numberOfColumns",
          m_numberOfColumns ), "Error adding parameter numberOfColumns " );

        TERP_TRUE_OR_RETURN_FALSE( serializer.addParameter( "numberOfRows",
          m_numberOfRows ), "Error adding parameter m_numberOfRows " );

        TERP_TRUE_OR_RETURN_FALSE( serializer.addParameter( "SRID",
          m_SRID ), "Error adding parameter SRID" );

        TERP_TRUE_OR_RETURN_FALSE( serializer.addParameter( "rasterSeg",
          m_rasterSegPtr->getName() ), "Error adding parameter rasterSeg" );

        TERP_TRUE_OR_RETURN_FALSE( serializer.addParameter(
          "sourcesNumber", m_sourcesNumber ), "Error adding parameter sourcesNumber" );

        TERP_TRUE_OR_RETURN_FALSE( serializer.addMultivaluedParameter(
          "imageType", m_imageType ),
          "Error adding parameter imageType" );

        TERP_TRUE_OR_RETURN_FALSE( serializer.addMultivaluedParameter(
          "stochasticDistanceType", m_stochasticDistanceType ),
          "Error adding parameter stochasticDistanceType" );

        TERP_TRUE_OR_RETURN_FALSE( serializer.addParameter(
          "savePerClassDistancesFile", m_savePerClassDistancesFile ),
          "Error adding parameter savePerClassDistancesFile" );

        TERP_TRUE_OR_RETURN_FALSE( serializer.addParameter(
          "savePerClassStatiticsFile", m_savePerClassStatiticsFile ),
          "Error adding parameter savePerClassStatiticsFile" );

        TERP_TRUE_OR_RETURN_FALSE( serializer.addParameter(
          "savePerClassPValueFile", m_savePerClassPValueFile ),
          "Error adding parameter savePerClassPValueFile" );

        TERP_TRUE_OR_RETURN_FALSE( serializer.addParameter(
          "saveDistances", m_saveDistances ),
          "Error adding parameter saveDistances" );

        TERP_TRUE_OR_RETURN_FALSE( serializer.addParameter(
          "saveStatitics", m_saveStatitics ),
          "Error adding parameter saveStatitics" );

        TERP_TRUE_OR_RETURN_FALSE( serializer.addParameter(
          "savePValues", m_savePValues ),
          "Error adding parameter savePValues" );

        TERP_TRUE_OR_RETURN_FALSE( serializer.addParameter(
          "saveReportFile", m_saveReportFile ),
          "Error adding parameter saveReportFile" );

        TERP_TRUE_OR_RETURN_FALSE( serializer.addParameter(
          "combinationType", m_combinationType ),
          "Error adding parameter m_combinationType" );

        TERP_TRUE_OR_RETURN_FALSE( serializer.addParameter(
          "lagX", m_lagX ),
          "Error adding parameter lagX" );

        TERP_TRUE_OR_RETURN_FALSE( serializer.addParameter(
          "lagY", m_lagY ),
          "Error adding parameter lagY" );

        TERP_TRUE_OR_RETURN_FALSE( serializer.addParameter(
          "significanceLevel", m_significanceLevel ),
          "Error adding parameter significanceLevel" );

        TERP_TRUE_OR_RETURN_FALSE( serializer.addMultivaluedParameter(
          "radarDataRepresentationType", m_radarDataRepresentationType ),
          "Error adding parameter radarDataRepresentationType" );

        TERP_TRUE_OR_RETURN_FALSE( serializer.addMultivaluedParameter(
          "vectorOrMatrixOrder", m_vectorOrMatrixOrder ),
          "Error adding parameter vectorOrMatrixOrder" );

        TERP_TRUE_OR_RETURN_FALSE( serializer.addMultivaluedParameter(
          "ENL", m_ENL ),
          "Error adding parameter ENL" );

        TERP_TRUE_OR_RETURN_FALSE( serializer.addMultivaluedParameter(
          "betaValue", m_betaValue ),
          "Error adding parameter betaValue" );

        TERP_TRUE_OR_RETURN_FALSE( serializer.addMultivaluedParameter(
          "dirTrainingSamples", m_dirTrainingSamples ),
          "Error adding parameter dirTrainingSamples" );

        TERP_TRUE_OR_RETURN_FALSE( serializer.addMultivaluedParameter(
          "dirTestSamples", m_dirTestSamples ),
          "Error adding parameter dirTestSamples" );

        TERP_TRUE_OR_RETURN_FALSE( serializer.addMultivaluedParameters(
          "polygonSamplesTNew", m_polygonSamplesTNew ),
          "Error adding parameter polygonSamplesTNew" );

        TERP_TRUE_OR_RETURN_FALSE( serializer.addMultivaluedParameters(
          "polygonSamplesTestNew", m_polygonSamplesTestNew ),
          "Error adding parameter polygonSamplesTestNew" );

        TERP_TRUE_OR_RETURN_FALSE( serializer.addMultivaluedParameter(
          "numberOfSamples", m_numberOfSamples ),
          "Error adding parameter numberOfSamples" );

        TERP_TRUE_OR_RETURN_FALSE( serializer.addMultivaluedParameter(
          "numberOfSamplesTest", m_numberOfSamplesTest ),
          "Error adding parameter numberOfSamplesTest" );

        TERP_TRUE_OR_RETURN_FALSE( serializer.addMultivaluedParameter(
          "nameOfSamples", m_nameOfSamples ),
          "Error adding parameter nameOfSamples" );

        TERP_TRUE_OR_RETURN_FALSE( serializer.addMultivaluedParameter(
          "nameOfSamplesTest", m_nameOfSamplesTest ),
          "Error adding parameter nameOfSamplesTest" );

        TERP_TRUE_OR_RETURN_FALSE( serializer.addMultivaluedParameter(
          "classesIDT", m_classesIDT ),
          "Error adding parameter classesIDT" );

        TERP_TRUE_OR_RETURN_FALSE( serializer.addMultivaluedParameter(
          "classesIDTest", m_classesIDTest ),
          "Error adding parameter classesIDTest" );

        TERP_TRUE_OR_RETURN_FALSE( serializer.addMultivaluedParameter(
          "classesIDT2", m_classesIDT2 ),
          "Error adding parameter classesIDT2" );

        TERP_TRUE_OR_RETURN_FALSE( serializer.addParameter(
          "numberOfClassesOnTraining", m_numberOfClassesOnTraining ),
          "Error adding parameter numberOfClassesOnTraining" );

        TERP_TRUE_OR_RETURN_FALSE( serializer.addParameter(
          "numberOfClassesOnTest", m_numberOfClassesOnTest ),
          "Error adding parameter numberOfClassesOnTest" );

        return true;
      }
		} // end namespace radar
	} // end namespace rp
}   // end namespace msc
