/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

#ifndef __TERRALIB_RP_RADAR_INTERNAL_MULTISOURCECLASSIFIER_OUTPUT_PARAMETERS_H
#define __TERRALIB_RP_RADAR_INTERNAL_MULTISOURCECLASSIFIER_OUTPUT_PARAMETERS_H

// TerraLib includes

#include "../Config.h"
#include "../../raster/Raster.h"
#include "../AlgorithmOutputParameters.h"

// Boost includes
#include <boost/numeric/ublas/matrix.hpp>

#include <vector>
#include <string>
#include <map>

namespace te
{
  namespace rp
  {
    namespace radar
    {
      /*!
        \class MultiSourceClassifierOutputParameters
        \brief MultiSourceClassifier output parameters
      */
      class TERPEXPORT MultiSourceClassifierOutputParameters : public te::rp::AlgorithmOutputParameters {
        public:
          std::string m_rType;  //!< Output raster data source type (as described in te::raster::RasterFactory).

          std::map< std::string, std::string > m_outinfo;  //!< The necessary information to create the output.

          std::map< std::string, std::string > m_rInfoClassImage;  //!< The necessary information to create the output raster for classified image.

          std::unique_ptr< te::rst::Raster > m_outputRasterClassImagePtr;  //!< The generated output raster for classified iamge.
          std::unique_ptr< te::rst::Raster > newRaster;  //!< The generated output raster for classified iamge.

          std::map< std::string, std::string > m_rInfoClassImageFuzzy;  //!< The necessary information to create the output raster for classified image for Fuzzy.

          std::unique_ptr< te::rst::Raster > m_outputRasterClassImageFuzzyPtr;  //!< The generated output raster for classified iamge for Fuzzy.

          std::map< std::string, std::string > m_rInfoMap;  //!< The necessary information to create the output raster for reliability map (statistical test).

          std::unique_ptr< te::rst::Raster > m_outputRasterMapPtr;  //!< The generated output raster for reliability map (statistical test).

          std::map< std::string, std::string > m_rInfoMapPValue;  //!< The necessary information to create the output raster for reliability map (p-value).

          std::unique_ptr< te::rst::Raster > m_outputRasterMapPValuePtr;  //!< The generated output raster for reliability map (p-value).

  //        std::map< std::string, std::string > m_rInfoConfusionMatrix;  //!< The necessary information to create the output statistics file.

  //        std::map< std::string, std::string > m_rInfoAttributes;  //!< The necessary information to create the output attribute file.

          std::map< std::string, std::string > m_rInfoDistancePerClass;  //!< The necessary information to create the output stochastic distance per class file.

          std::map< std::string, std::string > m_rInfoStatisticPerClass;  //!< The necessary information to create the output statistic per class file.

          std::map< std::string, std::string > m_rInfoPValuePerClass;  //!< The necessary information to create the output p-value per class file.

          unsigned int m_outputRasterBand;

          std::vector< std::vector< unsigned int > > m_confusionMatrix;  //!< The confusion matrix.

          double m_kappaCoefficient;  //!< The Kappa coefficient.

          double m_varianceOfKappa;  //!< The variance of Kappa.

          double m_overallAccuracy;  //!< The overall accuracy.

          std::vector< double > m_IDSegment;  //!< ID of segments.

          std::vector< double > m_valuesS;  //!< Statiscal test values of classification result.

          std::vector< double > m_valuesP;  //!< P-values of single source classification result.

          std::vector< double > m_pValuesSum;  //!< P-values of sum combination multi-source classification result.

          std::vector< double > m_pValuesMultiplication;  //!< P-values of multiplication combination multi-source classification result.

          std::vector< double > m_pValuesMinimum;  //!< P-values of minimum combination multi-source classification result.

          std::vector< double > m_distances;  //!< Stochastic distance values of classification result.

          std::vector< std::vector< double > > m_outDistancesPerClass;  //!< Stochastic distance per class values of classification result.

          std::vector< std::vector< double > > m_outStatisticsPerClass;  //!< Statistics per class values of classification result.

          std::vector< std::vector< double > > m_outPValuesPerClass;  //!< P-values per class values of classification result.

          std::vector< std::vector< double > > m_bandsFuzzy;  //!< The bands values in Fuzzy.

          std::vector< std::vector< double > > m_outAllDistances;
                  std::vector< std::vector< double > > m_outAllStatistics;

          MultiSourceClassifierOutputParameters();

          MultiSourceClassifierOutputParameters(const MultiSourceClassifierOutputParameters&);

          ~MultiSourceClassifierOutputParameters();

          // overload
          void reset();

          // overload
          const  MultiSourceClassifierOutputParameters& operator=(const MultiSourceClassifierOutputParameters& params);

          // overload
          AbstractParameters* clone() const;
      };

    } // end namespace radar
  } // end namespace rp
}   // end namespace msc

#endif
