/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

#ifndef __TERRALIB_RP_RADAR_INTERNAL_POINTERS_VECTOR_H
#define __TERRALIB_RP_RADAR_INTERNAL_POINTERS_VECTOR_H

#include "../Config.h"

#include <vector>

namespace te
{
  namespace rp
  {
    namespace radar
    {
      /*!
        \class PointersVector
        \brief A pointers vector.
       */
      template< typename PointerType >
      class PointersVector : public std::vector< PointerType >
      {
        public :

          PointersVector() {};

          ~PointersVector()
          {
            clear();
          };

          void clear()
          {
            std::size_t size = std::vector< PointerType >::size();
            for( std::size_t idx = 0 ; idx < size ; ++idx )
            {
              delete std::vector< PointerType >::operator[]( idx );
            }

            std::vector< PointerType >::clear();
          }

          void copyPointersTo( std::vector< PointerType >& outVec )
          {
            outVec.clear();

            std::size_t size = std::vector< PointerType >::size();
            for( std::size_t idx = 0 ; idx < size ; ++idx )
            {
              outVec[ idx ] =
                std::vector< PointerType >::operator[]( idx );
            }
          }

      };
      
    } // end namespace radar
  } // end namespace rp
}   // end namespace msc

#endif  // MSC_INTERNAL_FUNCTIONS_H

