/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/rp/radar/RadarFunctions.h
  \brief Radar Raster Processing functions.
 */

#ifndef __TERRALIB_RP_RADAR_INTERNAL_FUNCTIONS_H
#define __TERRALIB_RP_RADAR_INTERNAL_FUNCTIONS_H

#include "../Config.h"
#include "MultiSourceClassifierInputParameters.h"
#include "MultiSourceClassifierOutputParameters.h"

#include "../../geometry/Polygon.h"

#include <string>
#include <vector>

namespace te
{
  namespace rp
  {
    namespace radar
    {
      /*!
        \brief Load a sample file (SHP or TXT).
        \param loadedNumberOfClasses Number of classes.
        \param loadedClassesID Classes IDs.
        \param loadedNameOfSamples Sample names.
        \param loadedNumberOfSamples Samples namber.
        \param loadedSamples Samples.
        \param loadedPolygons Loaded Polygons.
        \return true if OK, false on errors.
        \ingroup rp_radar_func
       */
      bool TERPEXPORT LoadSamplesFile(
        const std::string& fileName,
        unsigned int& loadedNumberOfClasses,
        std::vector< unsigned int > loadedClassesID,
        std::vector< std::string >& loadedNameOfSamples,
        std::vector< int >& loadedNumberOfSamples,
        std::vector< std::vector< double > >& loadedSamples,
        std::vector< te::gm::Polygon >& loadedPolygons );

      /*!
        \brief Save all output files.
        \param inputParameters Input parameters used by the executed algorithm.
        \param outputParameters Output parameters generated by the executed algorithm.
        \param reportFileName Report file name (or an empty string) (confusion matrix and Kappa coefficient, i.e. ).
        \param modelImage ???
        \return true if OK, false on errors.
        \ingroup rp_radar_func
       */
      bool TERPEXPORT SaveOutputReportFile(
        const MultiSourceClassifierInputParameters& inputParameters,
        const MultiSourceClassifierOutputParameters& outputParameters,
        const std::string& reportFileName
      );

      /*!
        \brief Save all output files.
        \param inputParameters Input parameters used by the executed algorithm.
        \param outputParameters Output parameters generated by the executed algorithm.
        \param reportFileName Report file name (or an empty string) (confusion matrix and Kappa coefficient, i.e. ).
        \param exportStatData If true Stats data will be generated.
        \param exportPValuesData If true PValues data will be generated.
        \param exportDistData If true Dist data will be generated.
        \return true if OK, false on errors.
        \ingroup rp_radar_func
       */
      bool TERPEXPORT SaveOutputAttFile(
        const MultiSourceClassifierInputParameters& inputParameters,
        const MultiSourceClassifierOutputParameters& outputParameters,
        const bool exportStatData,
        const bool exportPValuesData,
        const bool exportDistData,
        const std::string& attributesFileName );
    } // end namespace radar
  } // end namespace rp
}   // end namespace msc

#endif  // __TERRALIB_RP_RADAR_INTERNAL_FUNCTIONS_H

