/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file Module.cpp
   
  \brief This singleton defines the TerraLib Spatial Reference System module entry.
*/

// TerraLib
#include "../core/logger/Logger.h"
#include "../common/TerraLib.h"
#include "../core/translator/Translator.h"
#include "Config.h"
#include "Module.h"
#include "SpatialReferenceSystemManager.h"

#ifdef TERRALIB_LEAK_DETECTOR_ENABLED
  #include <vld.h>
#endif

const te::srs::Module& sm_module = te::srs::Module::getInstance();

te::srs::Module::Module()
{
  m_module.reset(new TerraLib::Module(TE_SRS_MODULE_NAME,
                         te::srs::Module::initialize,
                         te::srs::Module::finalize
                       ));

// initialize TerraLib singleton
  TerraLib::getInstance().add(m_module);

  // initializing the static mutex
  getStaticMutex();
}

te::srs::Module::~Module() = default;

void te::srs::Module::initialize()
{
  std::string projLibDir = SpatialReferenceSystemManager::getProjLibDir();
  if (projLibDir.empty() == false)
  {
    te::core::SetEnv("PROJ_LIB", projLibDir);
    TE_LOG_TRACE(TE_TR("PROJ_LIB found in: ") + projLibDir);
  }
  else
  {
    TE_LOG_TRACE(TE_TR("PROJ_LIB NOT found!"));
  }

#ifdef TERRALIB_AUTOMATIC_INITIALIZATION
  te::srs::SpatialReferenceSystemManager::getInstance().init();
#endif

  TE_LOG_TRACE(TE_TR("TerraLib SRS initialized!"));
}

void te::srs::Module::finalize()
{
  te::srs::SpatialReferenceSystemManager::getInstance().clear();
  TE_LOG_TRACE(TE_TR("TerraLib SRS Finalized!"));

  TerraLib::getInstance().remove(TE_SRS_MODULE_NAME);
}

boost::mutex& te::srs::getStaticMutex()
{
  static boost::mutex getStaticMutexStaticMutex;
  return getStaticMutexStaticMutex;
}




