/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/vp.h

  \brief This file contains include headers for the TerraLib Vector Processing module.
*/

#ifndef __TERRALIB_INTERNAL_VP_H
#define __TERRALIB_INTERNAL_VP_H

#include "vp/AbstractDataHandler.h"
#include "vp/Dissolve.h"
#include "vp/IntersectionMemory.h"     
#include "vp/OperationThreadManagerListener.h"
#include "vp/AbstractOperation.h"  
#include "vp/DissolveOperation.h"  
#include "vp/IntersectionOperation.h"  
#include "vp/OverlapFixer.h"
#include "vp/AbstractTillingStrategy.h"      
#include "vp/DissolveSubdivideDataHandler.h" 
#include "vp/IntersectionOp.h"         
#include "vp/PolygonSubdivider.h"
#include "vp/Algorithm.h"          
#include "vp/DissolveSubdivisionOperation.h" 
#include "vp/IntersectionQuery.h"      
#include "vp/PolygonToLineMemory.h"
#include "vp/AlgorithmParams.h"    
#include "vp/Enums.h"   
#include "vp/KDTreeMultiIndex.h"       
#include "vp/PolygonToLineOp.h"
#include "vp/AttributeGroupingDataHandler.h" 
#include "vp/Exception.h"          
#include "vp/LineToPolygonMemory.h"    
#include "vp/PolygonToLineQuery.h"
#include "vp/BufferMemory.h"       
#include "vp/Feature.h" 
#include "vp/LineToPolygonOp.h"        
#include "vp/RTreeIndex.h"
#include "vp/BufferOp.h"
#include "vp/GeometricOp.h"        
#include "vp/LineToPolygonQuery.h"     
#include "vp/SubdivideByBorderTillingStrategy.h"
#include "vp/BufferQuery.h"        
#include "vp/GeometricOpMemory.h"  
#include "vp/MakeGeometryValid.h"      
#include "vp/SubdivideDataHandler.h"
#include "vp/CommonDataStructures.h"         
#include "vp/GeometricOpQuery.h"   
#include "vp/MakeValidOperation.h"     
#include "vp/SubdivideOperation.h"
#include "vp/CommonFunctions.h"    
#include "vp/GeometrySnapper.h"    
#include "vp/Merge.h"        
#include "vp/SubdividerTilling.h"
#include "vp/ComplexData.h"        
#include "vp/GroupThreadManager.h" 
#include "vp/MultipartToSinglepart.h"  
#include "vp/Union.h"
#include "vp/Config.h"  
#include "vp/Identity.h"
#include "vp/NoGroupingDataHandler.h"  
#include "vp/UnionOperation.h"
#include "vp/Difference.h"         
#include "vp/IdentityOperation.h"  
#include "vp/OperationReport.h"        
#include "vp/UnorderedGroupingDataHandler.h"
#include "vp/DifferenceOperation.h"
#include "vp/InputParams.h"        
#include "vp/OperationService.h"       
#include "vp/Utils.h"
#include "vp/DissolveCellSubdivision.h"      
#include "vp/Intersection.h"       
#include "vp/OperationThreadManager.h"
#include "vp/TiePointsLocator.h"
#include "vp/Register.h"

/*!
  \defgroup vp Vector Processing

  \brief This module provides classes and functions with vector processing capabilities.

    Basically, the TerraLib Vector Processing module contains several processing operations applied to vectorial data and it is organized as follows:

    <ul>
      <li>Aggregation Geographical Operation</li>
      <li>Buffer Geographical Operation</li>
      <li>Intersection Geographical Operation</li>
    </ul>

  The main classes/concepts in this module are listed here. The namespace associated to the Vector Processing module is te::vp.
  To know more about it, see the te::vp namespace documentation.
*/

namespace te
{
  /*!
    \brief Namespace for Vector Processing module of TerraLib.
  */
  namespace vp
  {
  } // end namespace vp
}   // end namespace te

#endif  // __TERRALIB_INTERNAL_VP_H
