/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

 /*!
   \file terralin/vp/AbstractDataHandler.cpp

   \brief This class represents handler to help reading and writing data from/to a dataSource by dividing it into smaller groups
 */

#include "AbstractDataHandler.h"

#include "../common/progress/TaskProgress.h"
#include "../dataaccess/utils/DataAccess.h"
#include "../geometry/Envelope.h"
#include "AbstractOperation.h"
#include "Feature.h"


te::vp::AbstractDataHandler::AbstractDataHandler()
  : m_maxCoordinates(0)
{

}

void te::vp::AbstractDataHandler::initialize(te::vp::AbstractParameters* absParams, te::common::TaskProgress* taskProgress)
{
  m_vecInputData = absParams->getInputDataAccessVector();
  m_vecOutputData = absParams->getOutputDataAccessVector();
  m_maxCoordinates = absParams->getMaxCoordinates();
}

void te::vp::AbstractDataHandler::finalize(te::common::TaskProgress* taskProgress)
{

}

const te::da::DataSetType* te::vp::AbstractDataHandler::getOutputDataType() const
{
  return m_vecOutputData.at(0)->getDataSetType();
}

std::size_t te::vp::AbstractDataHandler::getInputCount() const
{
  return m_vecInputData.size();
}

std::size_t te::vp::AbstractDataHandler::getOutputCount() const
{
  return m_vecOutputData.size();
}

te::vp::FeatureSet te::vp::AbstractDataHandler::getFeatureSet(std::size_t groupIndex, std::size_t inputIndex)
{
  return std::move(getFeatureSet(groupIndex, inputIndex, te::gm::Envelope()));
}

const std::vector<te::da::DataAccess*>& te::vp::AbstractDataHandler::getInputDataAccessVector() const
{
  return m_vecInputData;
}

const std::vector<te::da::DataAccess*>& te::vp::AbstractDataHandler::getOutputDataAccessVector() const
{
  return m_vecOutputData;
}


te::vp::AbstractDataHandlerFactory::AbstractDataHandlerFactory(const std::string& factoryKey)
  : te::common::AbstractFactory<te::vp::AbstractDataHandler, std::string>(factoryKey)
{

}

te::vp::AbstractDataHandlerFactory::~AbstractDataHandlerFactory() = default;