/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

 /*!
   \file terralib/vp/AbstractDataLoaderStrategy.h

   \brief Abstract class to represent a data loader strategy to load features from datasets using a tilling strategy
 */

#ifndef __TERRALIB_VP_INTERNAL_ABSTRACTDATALOADERSTRATEGY_H
#define __TERRALIB_VP_INTERNAL_ABSTRACTDATALOADERSTRATEGY_H

 //Terralib include files
#include "Config.h"

//STL include files
#include <string>
#include <vector>

namespace te
{
  namespace da
  {
    class DataAccess;
    class DataSetType;
  }

  namespace vp
  {
    /*!
      \class AbstractDataLoaderStrategy

      \brief Abstract class to represent a data loader strategy to load features from datasets using a tilling strategy
    */
    class TEVPEXPORT AbstractTillingStrategy
    {
      public:

        //!< Constructor
        AbstractTillingStrategy() = default;
        
        //!< Destructor
        virtual ~AbstractTillingStrategy() = default;

        virtual te::da::DataAccess* subdivide(const te::da::DataAccess* inputDataAccess, std::size_t maxCoordinates) = 0;

        virtual std::vector<te::da::DataAccess*> subdivide(const std::vector<te::da::DataAccess*>& vecDataAccess, std::size_t maxCoordinates) = 0;

        virtual te::da::DataAccess* merge(te::da::DataAccess* inputDataAccess, const std::vector<std::string>& vecMergeColumns, bool dissolveGeometryCoordinateSubdivision) = 0;

        virtual std::vector<te::da::DataAccess*> merge(const std::vector<te::da::DataAccess*>& vecDataAccess, const std::vector< std::vector<std::string> >& vecMergeColumns, bool dissolveGeometryCoordinateSubdivision) = 0;
    };
  }
}

#endif //__TERRALIB_VP_INTERNAL_ABSTRACTDATALOADERSTRATEGY_H
