/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

 /*!
   \file terralib/vp/DissolveCellSubdivision.h

   \brief Algorithm dedicated to dissolve the borders of a cell subdivision
 */

#ifndef __TERRALIB_VP_INTERNAL_DISSOLVECELLSUBDIVISION_H
#define __TERRALIB_VP_INTERNAL_DISSOLVECELLSUBDIVISION_H

 //Terralib include files
#include "Config.h"

#include "../geometry/CommonDataStructures.h"
#include "../geometry/Enums.h"

#include <cstdio>

namespace te
{
  namespace da
  {
    class DataSetType;
  }

  namespace vp
  {
    class FeatureSet;

    /*!
      \class DissolveCellSubdivision

      \brief Algorithm dedicated to dissolve the borders of a cell subdivision
    */
    class TEVPEXPORT DissolveCellSubdivision
    {
    public:

      //!< Constructor
      DissolveCellSubdivision() = delete;

      //!< Destructor
      ~DissolveCellSubdivision() = delete;

      static te::gm::GeometryVector GetGeometryUnionNoOverlaps(const te::gm::GeometryVectorConst& vecGeometries);

      static te::vp::FeatureSet GetGeometryUnionNoOverlaps(const te::vp::FeatureSet& fs);

      static te::vp::FeatureSet Dissolve(const te::vp::FeatureSet& fs, std::size_t rowIndex, std::size_t columnIndex, bool dissolveCoordinateSubdivision);
    };
  }
}

#endif //__TERRALIB_VP_INTERNAL_DISSOLVECELLSUBDIVISION_H
