/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

 /*!
   \file terralib/vp/DissolveOperation.h

   \brief Represents an dissolve operation
 */

#ifndef __TERRALIB_VP_INTERNAL_DISSOLVEOPERATION_H
#define __TERRALIB_VP_INTERNAL_DISSOLVEOPERATION_H

 //Terralib include files
#include "Config.h"

#include "AbstractOperation.h"

#include <vector>

namespace te
{
  namespace vp
  {
    class FeatureSet;

    class TEVPEXPORT DissolveOperationParameters : public te::vp::AbstractParameters
    {
    public:
      DissolveOperationParameters();
      virtual ~DissolveOperationParameters();

      virtual bool isValid(std::string& errorMessage) override;
    };

    class TEVPEXPORT DissolveOperationCapabilities : public te::vp::AbstractOperationCapabilities
    {
      public:

        DissolveOperationCapabilities(const std::string& operationName, std::size_t numberOfInputs, std::size_t numberOfOutputs, const std::string& dataHandlerName, bool snapGeometries, bool allowCoordinateSubdivision);

        //!< Creates the parameters for this operation. If the concrete operation has specific parameters, it must override this class and instantiate the correct parameters
        std::unique_ptr<AbstractParameters> createParameters() const override;
    };


    /*!
      \class DissolveOperation

      \brief Represents an dissolve operation
    */
    class TEVPEXPORT DissolveOperation : public te::vp::AbstractOperation
    {
    public:

      /*!
        * \brief Constructor
        */
      DissolveOperation();

      //!< Destructor
      virtual ~DissolveOperation() = default;

      //!< Get the capabilities of the operation. Basically, operation will have 1 input, 1 output, and support subdivision
      virtual std::unique_ptr<te::vp::AbstractOperationCapabilities> getCapabilities() const;

      std::vector<te::vp::FeatureSet> executeImpl(const std::vector<te::vp::FeatureSet>& vecInput);
    };

    class TEVPEXPORT DissolveOperationFactory : public te::vp::OperationFactory
    {
    public:

      DissolveOperationFactory();

      virtual ~DissolveOperationFactory();

    protected:

      virtual te::vp::AbstractOperation* build() override;
    };

  }//end namespace vp
}// end namespace te

#endif // __TERRALIB_VP_INTERNAL_DISSOLVEOPERATION_H
