#include "DissolveSubdivideDataHandler.h"

#include "../common/StringUtils.h"
#include "../dataaccess/dataset/DataSetType.h"
#include "../dataaccess/utils/DataAccess.h"
#include "SubdivideByBorderTillingStrategy.h"

namespace
{
  static te::vp::DissolveSubdivideDataHandlerFactory SubdivideDataHandlerFactoryInstanceDissolve;
}

te::vp::DissolveSubdivideDataHandler::DissolveSubdivideDataHandler()
  : SubdivideDataHandler()
{
}

//declaration just for the unique_ptr to be deleted on the cpp file
te::vp::DissolveSubdivideDataHandler::~DissolveSubdivideDataHandler() = default;

void te::vp::DissolveSubdivideDataHandler::initGroups()
{
  if (m_vecInputData.size() != 1)
  {
    return;
  }

  te::da::DataAccess* dataAccess = m_vecInputData.at(0);
  const te::da::DataSetType* dataSetType = dataAccess->getDataSetType();

  std::vector<std::string> vecCurrentMergeColumns;

  std::string rowPropertyName = te::vp::SubdivideByBorderTillingStrategy::getRowPropertyName();
  std::string columnPropertyName = te::vp::SubdivideByBorderTillingStrategy::getColumnPropertyName();
  std::string parentPropertyName = te::vp::SubdivideByBorderTillingStrategy::getFullPropertyName(dataSetType->getName(), te::vp::SubdivideByBorderTillingStrategy::getParentPropertyName());

  
  const te::da::PrimaryKey* primaryKey = dataSetType->getPrimaryKey();
  for (std::size_t i = 0; i < dataSetType->size(); ++i)
  {
    te::dt::Property* property = dataSetType->getProperty(i);
    if (property->getType() == te::dt::GEOMETRY_TYPE)
    {
      continue;
    }

    std::string propertyName = property->getName();
    if (primaryKey != nullptr && primaryKey->has(propertyName) == true)
    {
      continue;
    }

    if(te::common::IsStringEqual(propertyName, rowPropertyName, false))
    {
      continue;
    }
    if (te::common::IsStringEqual(propertyName, columnPropertyName, false))
    {
      continue;
    }
    if (te::common::IsStringEqual(propertyName, parentPropertyName, false))
    {
      continue;
    }

    vecCurrentMergeColumns.push_back(propertyName);
  }

  std::vector<std::string> vecCurrentSubdivideColumns(vecCurrentMergeColumns);
  vecCurrentSubdivideColumns.push_back(rowPropertyName);
  vecCurrentSubdivideColumns.push_back(columnPropertyName);

  std::vector< std::vector<std::string> > vecSubdivideColumns;
  vecSubdivideColumns.push_back(vecCurrentSubdivideColumns);

  std::vector<te::vp::SubdivideKey> vecGroups;

  std::string query = "SELECT DISTINCT " + te::common::Convert2String(vecCurrentSubdivideColumns, ",") + " FROM " + dataSetType->getName();
  std::unique_ptr<te::da::DataSet> dataSet = dataAccess->getDataSource()->query(query);
  dataSet->moveBeforeFirst();
  while (dataSet->moveNext())
  {
    std::vector<te::dt::AbstractData*> vecKeys;
    for (std::size_t j = 0; j < vecCurrentSubdivideColumns.size(); ++j)
    {
      te::dt::AbstractData* absData = nullptr;
      if (dataSet->isNull(j) == false)
      {
        absData = dataSet->getValue(j).release();
      }

      vecKeys.push_back(absData);
    }

    te::vp::SubdivideKey key(vecKeys);
    vecGroups.push_back(key);
  }
  
  std::vector< std::vector<std::string> > vecMergeColumns;
  vecMergeColumns.push_back(vecCurrentMergeColumns);

  setGroupInfo(vecGroups, vecSubdivideColumns, vecMergeColumns);
}

te::vp::DissolveSubdivideDataHandlerFactory::DissolveSubdivideDataHandlerFactory()
  : te::vp::AbstractDataHandlerFactory("DissolveSubdivideDataHandler")
{

}

te::vp::DissolveSubdivideDataHandlerFactory::~DissolveSubdivideDataHandlerFactory() = default;

te::vp::AbstractDataHandler* te::vp::DissolveSubdivideDataHandlerFactory::build()
{
  return new te::vp::DissolveSubdivideDataHandler();
}
