/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

 /*!
   \file terralin/vp/DissolveSubdivideDataHandler.h

   \brief This class represents handler to help reading and writing data from/to a dataSource by dividing it into smaller groups
 */

#ifndef __TERRALIB_VP_INTERNAL_DISSOLVESUBDIVIDEDATAHANDLER_H
#define __TERRALIB_VP_INTERNAL_DISSOLVESUBDIVIDEDATAHANDLER_H

#include "SubdivideDataHandler.h"
#include "Config.h"

#include <memory>
#include <vector>

namespace te
{
  namespace da
  {
    class DataAccess;
  }

  namespace gm
  {
    class Envelope;
  }

  namespace vp
  {
    class FeatureSet;

    class TEVPEXPORT DissolveSubdivideDataHandler : public te::vp::SubdivideDataHandler
    {
    public:
      /*! \brief Default constructor. */
      DissolveSubdivideDataHandler();

      /*! \brief Virtual destructor. */
      virtual ~DissolveSubdivideDataHandler();

    protected:

      virtual void initGroups() override;
    };

    class TEVPEXPORT DissolveSubdivideDataHandlerFactory : public te::vp::AbstractDataHandlerFactory
    {
    public:

      DissolveSubdivideDataHandlerFactory();

      virtual ~DissolveSubdivideDataHandlerFactory();

    protected:

      virtual te::vp::AbstractDataHandler* build() override;
    };
  }
}

#endif //__TERRALIB_VP_INTERNAL_DISSOLVESUBDIVIDEDATAHANDLER_H
