/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

 /*!
   \file terralib/vp/MakeValidOperation.h

   \brief Represents a make valid operation which analyses and fixes features of a set
 */

#ifndef __TERRALIB_VP_INTERNAL_MAKEVALIDOPERATION_H
#define __TERRALIB_VP_INTERNAL_MAKEVALIDOPERATION_H

 //Terralib include files
#include "Config.h"

#include "AbstractOperation.h"

#include <vector>

namespace te
{
  namespace vp
  {
    class FeatureSet;
    
    /*!
      \class MakeValidOperation
      \brief Represents a make valid operation which analyses and fixes features of a set
    */
    class TEVPEXPORT MakeValidOperation : public te::vp::AbstractOperation
    {
    public:

      /*!
        * \brief Constructor
        */
      MakeValidOperation();

      //!< Destructor
      virtual ~MakeValidOperation();

      virtual std::unique_ptr<te::vp::AbstractOperationCapabilities> getCapabilities() const override;

      std::vector<te::vp::FeatureSet> executeImpl(const std::vector<te::vp::FeatureSet>& vecInput) override;
    };

    class TEVPEXPORT MakeValidOperationFactory : public te::vp::OperationFactory
    {
    public:

      MakeValidOperationFactory();

      virtual ~MakeValidOperationFactory();

    protected:

      virtual te::vp::AbstractOperation* build() override;
    };

  }//end namespace vp
}// end namespace te

#endif // __TERRALIB_VP_INTERNAL_MAKEVALIDOPERATION_H
