/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file MultipartToSinglepart.h
   
  \brief 
 
  \ingroup vp
*/

#ifndef __TERRALIB_VP_INTERNAL_MULTIPARTTOSINGLEPART_H
#define __TERRALIB_VP_INTERNAL_MULTIPARTTOSINGLEPART_H

//Terralib
#include "Config.h"
#include "../dataaccess/datasource/DataSource.h"
#include "../dataaccess/utils/DataAccess.h"


// STL
#include <memory>
#include <string>

namespace te
{

  namespace vp
  {
    class TEVPEXPORT MultipartToSinglepart
    {

      public:

        MultipartToSinglepart();

        ~MultipartToSinglepart();

        bool run();

        //!< Sets the input dataAccess. This function will take the ownership of the dataAccess
        void setInput(te::da::DataAccessPtr inputDataAccess);

        void setOutput(te::da::DataSourcePtr outDsrc, const std::string& outDsName);

      protected:

        std::unique_ptr<te::da::DataSetType> getOutDst(bool onlySelected = false);

        bool runAll();

      private:

        te::da::DataAccessPtr m_inputDataAccess;
        te::da::DataSourcePtr m_outDsrc;
        std::string m_outDsName;
    }; // end class
  } // end namespace vp
}   // end namespace te

#endif  // __TERRALIB_VP_INTERNAL_MULTIPARTTOSINGLEPART_H

