/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

 /*!
   \file terralin/vp/NoGroupingDataHandler.h

   \brief This class represents handler to help reading and writing data from/to a dataSource using one group only (all data will be returned)
 */

#ifndef __TERRALIB_VP_INTERNAL_NOGROUPINGDATAHANDLER_H
#define __TERRALIB_VP_INTERNAL_NOGROUPINGDATAHANDLER_H


#include "AbstractDataHandler.h"
#include "Config.h"

#include <vector>

namespace te
{
  namespace da
  {
    class DataAccess;
  }

  namespace gm
  {
    class Envelope;
  }

  namespace vp
  {
    class FeatureSet;

    /*!
      \class NoGroupingDataHandler

      \brief This class represents handler to help reading and writing data from/to a dataSource using one group only (all data will be returned)
     */
    class TEVPEXPORT NoGroupingDataHandler : public AbstractDataHandler
    {
    public:
      /*! \brief Default constructor. */
      NoGroupingDataHandler();

      /*! \brief Virtual destructor. */
      virtual ~NoGroupingDataHandler()= default;

      //!< Returns the number of groups of the data handler
      virtual std::size_t getGroupCount() const override;

      //!< Returns the inputIndex-th featureSet og the groupIndex-th group considering the given filter
      virtual te::vp::FeatureSet getFeatureSet(std::size_t groupIndex, std::size_t inputIndex, const te::gm::Envelope& filter) override;

      //!< Adds a featureSet related to groupIndex-th group the to be added to the outputIndex-th output data access
      virtual void addFeatureSet(std::size_t groupIndex, std::size_t outputIndex, const te::vp::FeatureSet& featureSet) override;
    };

    class TEVPEXPORT NoGroupingDataHandlerFactory : public te::vp::AbstractDataHandlerFactory
    {
    public:

      NoGroupingDataHandlerFactory();

      virtual ~NoGroupingDataHandlerFactory();

    protected:

      virtual te::vp::AbstractDataHandler* build() override;
    };
  }
}

#endif //__TERRALIB_VP_INTERNAL_NOGROUPINGDATAHANDLER_H
