/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralin/vp/OperationReport.h

  \brief Class used as a report logger for Operations.
*/

#ifndef __TERRALIB_VP_INTERNAL_OPERATIONREPORT_H
#define __TERRALIB_VP_INTERNAL_OPERATIONREPORT_H

#include "Config.h"

// STL
#include <cstdio>
#include <ctime>
#include <map>
#include <string>
#include <sstream>

namespace te
{
  namespace vp
  {
    /*!
       \class OperationReport

       \brief Class used as a report logger for Operations.
    */
    class TEVPEXPORT OperationReport
    {
      public:

        /*! \brief Default constructor. */
        OperationReport();

        /*! \brief Virtual destructor. */
        ~OperationReport();

      public:

        void addTitle(const std::string& message);

        void addSubTitle(const std::string& message);

        void addMessage(const std::string& message, const std::string& severity = "", bool appendTime = false);

        void addStartMessage(const std::string& message, int id);

        void addEndMessage(const std::string& message, int id);

        void addKeyValueMessage(const std::string& key, const std::string& value);

        void addEmptyLine();

        void toFile(const std::string& reportPath);

        std::string toString();


      protected:

        std::stringstream m_reportStr;

        std::map<int, std::clock_t> m_clockTime;
    };
  }
}

#endif // __TERRALIB_VP_INTERNAL_OPERATIONREPORT_H
