/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

 /*!
   \file terralin/vp/OperationService.h

   \brief Represents a service containing useful functions related to an operation
 */

#ifndef __TERRALIB_VP_INTERNAL_OPERATIONSERVICE_H
#define __TERRALIB_VP_INTERNAL_OPERATIONSERVICE_H

#include "Config.h"

#include "../geometry/CommonDataStructures.h"
#include "../geometry/Enums.h"
#include "../sam/rtree/Index.h"

#include <memory>
#include <vector>

namespace te
{
  namespace gm
  {
    class Geometry;
    class GeometryPtr;
  }

  namespace vp
  {
    class FeatureSet;

    using GeometryPair = std::pair<te::gm::GeometryPtr, te::gm::GeometryPtr>;

    /*!
      \class OperationService

      \brief Represents a service containing useful functions related to an operation
    */
    class TEVPEXPORT OperationService
    {
    public:

      //!< No Constructor      
      OperationService() = delete;

      //!< No Destructor
      ~OperationService() = delete;

      //!< Snaps the given featureSets to themselves using the appropriated precison
      static std::vector<te::vp::FeatureSet> SnapGeometries(const std::vector<te::vp::FeatureSet>& vecInput);

      //!< Creates an index based on the given featureSet
      static std::unique_ptr< te::sam::rtree::Index<std::size_t> > CreateIndex(const te::vp::FeatureSet& inputFeatureSet);
    };
  } //end namespace vp
} //end namespace te


#endif // __TERRALIB_VP_INTERNAL_OPERATIONSERVICE_H
