/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

 /*!
   \file terralin/vp/OperationThreadManager.h

   \brief This class represents a vector processing engine to help the processing of a large number of geometries by dividing it into smallers groups
 */

#ifndef __TERRALIB_VP_INTERNAL_OPERATIONTHREADMANAGER_H
#define __TERRALIB_VP_INTERNAL_OPERATIONTHREADMANAGER_H

#include "Config.h"

#include <memory>
#include <thread>

namespace te
{
  namespace common
  {
    class TaskProgress;
  }

  namespace vp
  {
    class AbstractDataHandler;
    class AbstractOperation;
    class OperationThreadManagerListener;

    class TEVPEXPORT OperationThreadManager
    {
      public:
        /*! \brief Default constructor. */
        OperationThreadManager(AbstractOperation* operation, AbstractDataHandler* dataHandler);

        /*! \brief Virtual destructor. */
        virtual ~OperationThreadManager();

        virtual void start( const bool useThreads = true );

        virtual void startAsync(OperationThreadManagerListener* listener);

        virtual void stop();

        virtual void setTask(te::common::TaskProgress* task);

        virtual AbstractOperation* getOperation() const;

    protected:

      void startImpl(te::vp::OperationThreadManagerListener* listener, const bool useThreads );

      void stopThread();

      protected:

        std::unique_ptr<AbstractOperation> m_operation;
        std::unique_ptr<AbstractDataHandler> m_dataHandler;
        std::thread* m_thread;
        te::common::TaskProgress* m_task;
    };
  }
}

#endif //__TERRALIB_VP_INTERNAL_OPERATIONTHREADMANAGER_H
