/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

 /*!
   \file terralin/vp/OperationThreadManagerListener.h

   \brief This class represents a vector processing engine to help the processing of a large number of geometries by dividing it into smallers groups
 */

#ifndef __TERRALIB_VP_INTERNAL_OPERATIONTHREADMANAGERLISTENER_H
#define __TERRALIB_VP_INTERNAL_OPERATIONTHREADMANAGERLISTENER_H

#include "Config.h"

namespace te
{
  namespace vp
  {
    enum class OperationResult;

    enum class ProgressState 
    { 
      PRE_PROCESSING_STARTED, PRE_PROCESSING_FINISHED, PROCESSING_STARTED, PROCESSING_FINISHED, POST_PROCESSING_STARTED, POST_PROCESSING_FINISHED, CANCEL_REQUESTED
    };

    class TEVPEXPORT OperationThreadManagerListener
    {
      public:

        virtual void processingStateChanged(const te::vp::ProgressState& newState) = 0;

        virtual void processingFinished(const te::vp::OperationResult& result) = 0;
    };
  }
}

#endif //__TERRALIB_VP_INTERNAL_OPERATIONTHREADMANAGERLISTENER_H
