/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

 /*!
   \file terralib/vp/SequencialDataLoaderStrategy.h

   \brief Represent a data loader strategy to load features considering a seq
 */

#ifndef __TERRALIB_VP_INTERNAL_SUBDIVIDEDATALOADERSTRATEGY_H
#define __TERRALIB_VP_INTERNAL_SUBDIVIDEDATALOADERSTRATEGY_H

 //Terralib include files
#include "Config.h"

#include "AbstractTillingStrategy.h"

//STL include files
#include <string>
#include <vector>

//Forward declarations
class SubdivideByBorderTillingStrategyImpl;

namespace te
{
  namespace common
  {
    class TaskProgress;
  }

  namespace da
  {
    class DataSetType;
    class DataSource;
  }

  namespace gm
  {
    class Envelope;
  }

  namespace vp
  {
    class SubdividerTilling;

    /*!
      \class AbstractDataLoaderStrategy

      \brief Abstract class to represent a data loader strategy to load features from datasets using a tilling strategy
    */
    class TEVPEXPORT SubdivideByBorderTillingStrategy : public AbstractTillingStrategy
    {
      public:

        //!< Constructor
        SubdivideByBorderTillingStrategy(const te::gm::Envelope& mbr, int srid, double resolutionX, double resolutionY, te::common::TaskProgress* taskProgress);
        
        //!< Destructor
        virtual ~SubdivideByBorderTillingStrategy();

        //!< Executes the subdivide of this tilling strategy
        virtual te::da::DataAccess* subdivide(const te::da::DataAccess* inputDataAccess, std::size_t maxCoordinates) override;

        //!< Executes the subdivide of this tilling strategy
        virtual std::vector<te::da::DataAccess*> subdivide(const std::vector<te::da::DataAccess*>& vecInputDataAccess, std::size_t maxCoordinates) override;

        //!< Executes the merge of this tilling strategy
        virtual te::da::DataAccess* merge(te::da::DataAccess* inputDataAccess, const std::vector<std::string>& vecMergeColumns, bool dissolveGeometryCoordinateSubdivision) override;

        //!< Executes the merge of this tilling strategy
        virtual std::vector<te::da::DataAccess*> merge(const std::vector<te::da::DataAccess*>& vecInputDataAccess, const std::vector< std::vector<std::string> >& vecMergeColumns, bool dissolveGeometryCoordinateSubdivision) override;

        virtual te::da::DataAccess* createInputDataAccessForSubdivision(const te::da::DataSetType* baseOutputDataSetType, const std::string& dataSetName);

        virtual te::da::DataAccess* createOutputDataAccessForSubdivision(const te::da::DataSetType* baseOutputDataSetType, const std::vector<te::da::DataAccess*>& vecInputDataAccess, const std::string& dataSetName);

        virtual void createIndexes(te::da::DataAccess* dataAccess, const std::string& baseDataSetName);

        static std::string getColumnPropertyName();

        static std::string getRowPropertyName();

        static std::string getParentPropertyName();

        static std::string getFullPropertyName(const std::string& dataSetName, const std::string& propertyName);

        virtual void appendSubdivideColumns(te::da::DataSetType* dataSetType, const std::string& baseDataSetName);

      protected:

        void init();

    protected:
        SubdivideByBorderTillingStrategyImpl* m_impl;
    };
  }
}

#endif //__TERRALIB_VP_INTERNAL_SUBDIVIDEDATALOADERSTRATEGY_H
