/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

 /*!
   \file terralib/vp/UnionOperation.h

   \brief Represents an union operation between two sets of features
 */

#ifndef __TERRALIB_VP_INTERNAL_UNIONOPERATION_H
#define __TERRALIB_VP_INTERNAL_UNIONOPERATION_H

 //Terralib include files
#include "Config.h"

#include "AbstractOperation.h"

#include <memory>
#include <vector>

namespace te
{
  namespace vp
  {
    class AbstractParameters;
    class DifferenceOperation;
    class FeatureSet;
    class IdentityOperation;

    /*!
      \class UnionOperation
      \brief Represents an union operation between two sets of features. 

      It executes 3 operations (A - B), (B - A) and (A & B)
    */
    class TEVPEXPORT UnionOperation : public te::vp::AbstractOperation
    {
    public:

      /*!
        * \brief Constructor
        */
      UnionOperation();

      //!< Destructor
      virtual ~UnionOperation();

      virtual void initialize() override;

      std::vector<te::vp::FeatureSet> executeImpl(const std::vector<te::vp::FeatureSet>& vecInput);

    protected:
      std::unique_ptr<te::vp::IdentityOperation> m_identityOperation;
      std::unique_ptr<te::vp::DifferenceOperation> m_differenceOperation;
    };

    class TEVPEXPORT UnionOperationFactory : public te::vp::OperationFactory
    {
    public:

      UnionOperationFactory();

      virtual ~UnionOperationFactory();

    protected:

      virtual te::vp::AbstractOperation* build() override;
    };

  }//end namespace vp
}// end namespace te

#endif // __TERRALIB_VP_INTERNAL_UNIONOPERATION_H
