/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

This file is part of the TerraLib - a Framework for building GIS enabled applications.

TerraLib is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License,
or (at your option) any later version.

TerraLib is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with TerraLib. See COPYING. If not, write to
TerraLib Team at <terralib-team@terralib.org>.
*/

/*!
 * \file src/terralib/vp/qt/DifferenceDialogCapabilities.h
 *
 * \brief Implements a dialog used to execute a vector processing difference operation
 */

#ifndef __TERRALIB_VP_QT_INTERNAL_DIFFERENCEDIALOGCAPABILITIES_H
#define __TERRALIB_VP_QT_INTERNAL_DIFFERENCEDIALOGCAPABILITIES_H

#include "../Config.h"

#include "VectorProcessingDialogCapabilities.h"

//STL includes
#include <memory>
#include <string>

class QWidget;

namespace te
{
  namespace vp
  {
    namespace qt
    {
      /*!
        \class DifferenceDialogCapabilities

        \brief A class used to define the capabilities of the difference operation
      */
      class TEVPEXPORT DifferenceDialogCapabilities : public VectorProcessingDialogCapabilities
      {
      public:
        DifferenceDialogCapabilities();

        //!< Destructor
        virtual ~DifferenceDialogCapabilities();
      };


      /*!
        \class DifferenceDialogCapabilitiesFactory

        \brief Abstract factory used to create VectorProcessingDialogCapabilities.
      */
      class TEVPEXPORT DifferenceDialogCapabilitiesFactory : public VectorProcessingDialogCapabilitiesFactory
      {
      public:

        /*!
          \brief Constructor.

          \param factoryKey The key that identifies the factory.
        */
        DifferenceDialogCapabilitiesFactory();

        //!< Destructor
        virtual ~DifferenceDialogCapabilitiesFactory();

      protected:

        virtual VectorProcessingDialogCapabilities* build() override;
      };

    } //end namespace qt
  }//end namespace vp
}//end namespace te


#endif //__TERRALIB_VP_QT_INTERNAL_INTERSECTIONDIALOGCAPABILITIES_H
