/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

This file is part of the TerraLib - a Framework for building GIS enabled applications.

TerraLib is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License,
or (at your option) any later version.

TerraLib is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with TerraLib. See COPYING. If not, write to
TerraLib Team at <terralib-team@terralib.org>.
*/

/*!
 * \file src/terralib/vp/qt/DissolveDialogCapabilities.cpp
 *
 * \brief A class used to define the capabilities of the dissolve operation
 */


#include "DissolveDialogCapabilities.h"

#include "DissolveParametersWidget.h"

#include "../../core/translator/Translator.h"

namespace
{
  static te::vp::qt::DissolveDialogCapabilitiesFactory
    DissolveDialogCapabilitiesFactoryInstance;
}

te::vp::qt::DissolveDialogCapabilities::DissolveDialogCapabilities()
  : VectorProcessingDialogCapabilities(
      "Dissolve"
    , TE_TR("Simple Dissolve")
    , TE_TR("Dissolves the borders between geometries of the same attribute group")
    , "vector-processing-dissolve"
    , "vp-dissolve-hint"
    , "dpi.inpe.br.plugins"
    , "share/terraview/help/plugins/vp/vp_dissolveSimple.html"
    , false
  )
{

}

te::vp::qt::DissolveDialogCapabilities::~DissolveDialogCapabilities() = default;

te::vp::qt::DissolveDialogCapabilitiesFactory::DissolveDialogCapabilitiesFactory()
  : VectorProcessingDialogCapabilitiesFactory("Dissolve")
{

}

te::vp::qt::SpecificParametersWidget* te::vp::qt::DissolveDialogCapabilities::getParametersWidget(QWidget* parent) const
{
  return new te::vp::qt::DissolveParametersWidget(parent);
}

te::vp::qt::DissolveDialogCapabilitiesFactory::~DissolveDialogCapabilitiesFactory() = default;

te::vp::qt::VectorProcessingDialogCapabilities* te::vp::qt::DissolveDialogCapabilitiesFactory::build()
{
  return new te::vp::qt::DissolveDialogCapabilities();
}