/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

This file is part of the TerraLib - a Framework for building GIS enabled applications.

TerraLib is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License,
or (at your option) any later version.

TerraLib is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with TerraLib. See COPYING. If not, write to
TerraLib Team at <terralib-team@terralib.org>.
*/

/*!
 * \file src/terralib/vp/qt/DissolveParametersWidget.cpp
 *
 * \brief Implements a widget used to return the specific parameters for dissolve operation.
 */

#include "DissolveParametersWidget.h"
#include "ui/ui_DissolveParametersWidgetForm.h"

#include "../../common/Exception.h"
#include "../../dataaccess/dataset/DataSetType.h"
#include "../../dataaccess/utils/DataAccess.h"
#include "../../qt/widgets/dataset/utils/DataSetTypeFilter.h"
#include "../../qt/widgets/Utils.h"
#include "../SubdivideByBorderTillingStrategy.h"
#include "../DissolveOperation.h"

#include <QLayout>


te::vp::qt::DissolveParametersWidget::DissolveParametersWidget(QWidget* parent, Qt::WindowFlags f)
  : QWidget(parent, f)
  , te::vp::qt::SpecificParametersWidget()
  , m_ui(new Ui::DissolveParametersWidgetForm)
  , m_dataSetTypeFilter(nullptr)
{
  m_ui->setupUi(this);

  QGridLayout* gridLayout = new QGridLayout(m_ui->m_groupBox);

  m_dataSetTypeFilter = new te::qt::widgets::DataSetTypeFilter(m_ui->m_groupBox);
  gridLayout->setContentsMargins(0, 0, 0, 0);
  gridLayout->addWidget(m_dataSetTypeFilter);

  m_ui->m_groupBox->setLayout(gridLayout);
}

te::vp::qt::DissolveParametersWidget::~DissolveParametersWidget() = default;

void te::vp::qt::DissolveParametersWidget::setInputData(const std::vector<te::da::DataAccessPtr>& vecDataAccess)
{
  if (vecDataAccess.size() != 1)
  {
    return;
  }

  const te::da::DataSetType* dataSetType = vecDataAccess.at(0)->getDataSetType();

  //we clone the dataSetType so we can remove all the floating pointing attributes
  std::unique_ptr<te::da::DataSetType> filteredDataSetType(static_cast<te::da::DataSetType*>(dataSetType->clone()));
  std::vector<te::dt::Property*> vecProperties(filteredDataSetType->getProperties());

  for (std::size_t i = 0; i < vecProperties.size(); ++i)
  {
    te::dt::Property* property = vecProperties.at(i);
    if (property->getType() == te::dt::FLOAT_TYPE || property->getType() == te::dt::DOUBLE_TYPE)
    {
      filteredDataSetType->remove(property);
      continue;
    }
  }

  m_dataSetTypeFilter->init(filteredDataSetType.get(), false);
}

QWidget* te::vp::qt::DissolveParametersWidget::getWidget()
{
  return this;
}

void te::vp::qt::DissolveParametersWidget::setValuesToParameters(te::vp::AbstractParameters* parameters)
{
  //if the converter already exists, we just append the new attribute convertions
  m_dataSetTypeFilter->applyFilterToDataAccess(parameters->getInputDataAccessVector().at(0));
}
