/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

This file is part of the TerraLib - a Framework for building GIS enabled applications.

TerraLib is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License,
or (at your option) any later version.

TerraLib is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with TerraLib. See COPYING. If not, write to
TerraLib Team at <terralib-team@terralib.org>.
*/

/*!
 * \file src/terralib/vp/qt/DissolveParametersWidget.h
 *
 * \brief Implements a widget used to return the specific parameters for dissolve operation.
 */

#ifndef __TERRALIB_VP_QT_INTERNAL_DISSOLVEPARAMETERSWIDGET_H
#define __TERRALIB_VP_QT_INTERNAL_DISSOLVEPARAMETERSWIDGET_H

#include "SpecificParametersWidget.h"
#include "../Config.h"

 // STL Includes
#include <memory>

 // Qt Includes
#include <QWidget>

namespace Ui { class DissolveParametersWidgetForm; }

namespace te
{
  namespace da
  {
    class DataAccessPtr;
  }

  namespace qt
  {
    namespace widgets
    {
      class DataSetTypeFilter;
    }
  }

  namespace vp
  {
    namespace qt
    {
      /*!
        \class DissolveParametersWidget

        \brief  Implements a widget used to return the specific parameters for dissolve operation.
      */
      class TEVPEXPORT DissolveParametersWidget : public QWidget, public te::vp::qt::SpecificParametersWidget
      {
        Q_OBJECT

      public:

        /*!
          \brief Constructor.
        */
        DissolveParametersWidget(QWidget* parent = 0, Qt::WindowFlags f = 0);

        /*!
          \brief Destructor.
        */
        ~DissolveParametersWidget();

        virtual void setInputData(const std::vector<te::da::DataAccessPtr>& vecDataAccess) override;

        public:

          virtual QWidget* getWidget() override;

          virtual void setValuesToParameters(te::vp::AbstractParameters* parameters) override;

      private:

        std::unique_ptr<Ui::DissolveParametersWidgetForm> m_ui;  //!< Pointer to Qt widget
        te::qt::widgets::DataSetTypeFilter* m_dataSetTypeFilter; //!< Component used to filter a dataSetType
      };
    } // end namespace qt
  }  // end namespace qt
}  // end namespace qt


#endif // __TERRALIB_VP_QT_INTERNAL_DISSOLVEPARAMETERSWIDGET_H

