/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

This file is part of the TerraLib - a Framework for building GIS enabled applications.

TerraLib is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License,
or (at your option) any later version.

TerraLib is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with TerraLib. See COPYING. If not, write to
TerraLib Team at <terralib-team@terralib.org>.
*/

/*!
 * \file src/terralib/vp/qt/DissolveSubdivisionDialogCapabilities.cpp
 *
 * \brief A class used to define the capabilities of the dissolve subdivision operation
 */


#include "DissolveSubdivisionDialogCapabilities.h"

#include "DissolveSubdivisionParametersWidget.h"

#include "../../core/translator/Translator.h"

namespace
{
  static te::vp::qt::DissolveSubdivisionDialogCapabilitiesFactory
    DissolveDialogCapabilitiesFactoryInstance;
}

te::vp::qt::DissolveSubdivisionDialogCapabilities::DissolveSubdivisionDialogCapabilities()
  : VectorProcessingDialogCapabilities(
      "DissolveSubdivision"
    , TE_TR("Dissolve Subdivision")
    , TE_TR("Dissolves the subdivision borders between geometries of the same attribute group")
    , "vector-processing-dissolve"
    , "vp-dissolve-hint"
    , "dpi.inpe.br.plugins"
    , "share/terraview/help/plugins/vp/vp_dissolve.html"
  )
{

}

te::vp::qt::DissolveSubdivisionDialogCapabilities::~DissolveSubdivisionDialogCapabilities() = default;

te::vp::qt::SpecificParametersWidget* te::vp::qt::DissolveSubdivisionDialogCapabilities::getParametersWidget(QWidget* parent) const
{
  return new te::vp::qt::DissolveSubdivisionParametersWidget();
}

te::vp::qt::DissolveSubdivisionDialogCapabilitiesFactory::DissolveSubdivisionDialogCapabilitiesFactory()
  : VectorProcessingDialogCapabilitiesFactory("DissolveSubdivision")
{

}

te::vp::qt::DissolveSubdivisionDialogCapabilitiesFactory::~DissolveSubdivisionDialogCapabilitiesFactory() = default;

te::vp::qt::VectorProcessingDialogCapabilities* te::vp::qt::DissolveSubdivisionDialogCapabilitiesFactory::build()
{
  return new te::vp::qt::DissolveSubdivisionDialogCapabilities();
}