/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/vp/qt/IdentityDialog.h

  \brief Identity operation dialog.
*/

#ifndef __TERRALIB_VP_QT_INTERNAL_IDENTITYDIALOG_H
#define __TERRALIB_VP_QT_INTERNAL_IDENTITYDIALOG_H

// TerraLib
#ifndef Q_MOC_RUN
#include "../../dataaccess/datasource/DataSourceInfo.h"
#include "../../maptools/AbstractLayer.h"
#include "../AlgorithmParams.h"
#endif
#include "../Config.h"

// STL
#include <map>
#include <memory>

// Qt
#include <QDialog>

namespace Ui { class IdentityDialogForm; }

namespace te
{
  namespace qt
  {
    namespace widgets
    {
      class DoubleInputListWidget;
    }
  }

  namespace vp
  {
    /*!
      \class IdentityDialog

      \brief A dialog used to execute vector identity.
    */
    class TEVPEXPORT IdentityDialog : public QDialog
    {
      Q_OBJECT

      public:

        IdentityDialog(QWidget* parent = 0, Qt::WindowFlags f = 0);

        ~IdentityDialog();

        struct DataStruct
        {
          DataStruct()
          {
            m_dataSetType = 0;
            m_dataSet = 0;
          }

          te::da::DataSetType* m_dataSetType;
          te::da::DataSet* m_dataSet;
        };

        /*!
          \brief Set the layer that can be used

          \param layers   List of AbstractLayerPtr
        */
        void setLayers(std::list<te::map::AbstractLayerPtr> layers);

        te::map::AbstractLayerPtr getLayer();

        std::vector<std::string> getWarnings();

      protected:

        std::map<int, std::vector<std::string> > getSelectedProperties();

        void updateInputLayerComboBox();

        void updateIntersectionLayerComboBox();

        void updateDoubleListWidget();

      protected slots:

        void onInputLayerComboBoxChanged(int index);

        void onIntersectionLayerComboBoxChanged(int index);

        void onTargetDatasourceToolButtonPressed();

        void onTargetFileToolButtonPressed();

        void onOkPushButtonClicked();
        
        void onCancelPushButtonClicked();

      private:

        std::unique_ptr<Ui::IdentityDialogForm> m_ui;
        std::list<te::map::AbstractLayerPtr> m_layers;          //!< The vector layers in Layer Explorer.
        te::map::AbstractLayerPtr m_inputSelectedLayer;         //!< Input layer selected.
        te::map::AbstractLayerPtr m_intersectionSelectedLayer;  //!< Identity layer selected.

        std::vector<te::vp::InputParams> m_inputParams;         //!< A vector of input parameters.
        te::vp::AlgorithmParams* m_params;                      //!< Algorithm parameters.

        te::da::DataSourceInfoPtr m_outputDatasource;           //!< DataSource information.
        te::map::AbstractLayerPtr m_layerResult;                //!< Generated Layer.
        
        bool m_toFile;                                          //!< The result is in a file?
        std::vector<std::string> m_warnings;                    //!< Warnings during the operation.

        std::unique_ptr<te::qt::widgets::DoubleInputListWidget>
            m_inputDoubleInputListWidget;
    };
  }   // end namespace vp
}     // end namespace te

#endif  // __TERRALIB_VP_QT_INTERNAL_IDENTITYDIALOG_H
