/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

This file is part of the TerraLib - a Framework for building GIS enabled applications.

TerraLib is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License,
or (at your option) any later version.

TerraLib is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with TerraLib. See COPYING. If not, write to
TerraLib Team at <terralib-team@terralib.org>.
*/

/*!
 * \file src/terralib/vp/qt/InputDataAccessItemWidget.h
 *
 * \brief Implements a widget used to return a Data Access object.
 */

#ifndef __TERRALIB_VP_QT_INTERNAL_INPUTDATAACCESSITEMWIDGET_H
#define __TERRALIB_VP_QT_INTERNAL_INPUTDATAACCESSITEMWIDGET_H

#include "../Config.h"

// TerraLib Includes
#include "../../maptools/AbstractLayer.h"

// STL Includes
#include <memory>

// Qt Includes
#include <QWidget>

namespace Ui { class InputDataAccessItemWidgetForm; }

namespace te
{
  namespace da
  {
    class DataAccessPtr;
  }

  namespace qt
  {
    namespace widgets
    {
      class DataSetTypeFilter;
    }
  }

  namespace vp
  {
    namespace qt
    {
      /*!
        \class InputDataAccessItemWidget

        \brief  Implements a widget used to return a Data Access object.
      */
      class TEVPEXPORT InputDataAccessItemWidget : public QWidget
      {
        Q_OBJECT

        public:

          /*!
            \brief Constructor.
          */
          InputDataAccessItemWidget(QWidget* parent = 0, Qt::WindowFlags f = 0);

          /*!
            \brief Destructor.
          */
          ~InputDataAccessItemWidget();

        public:

          /*!
            \brief Returns the internal widget
          
            \return Pointer to Qt Widget InputDataAccessItemWidgetForm
          */
          Ui::InputDataAccessItemWidgetForm* getForm() const;

          /*!
            \brief Set the list of layers to be selected

            \param layers List of layers
          */
          void setLayerList(const std::list<te::map::AbstractLayerPtr>& layers);

          /*!
            \brief Get DataAccess given the user defined parameters in the interface

            \return TerraLib DataAccess object
          */
          te::da::DataAccessPtr getDataAccess();

          //!< Configures the widget to allow the filter of properties
          void setFilterPropertiesEnabled(bool enabled);

        protected slots:

          /*!
            \brief Slot for layer selection

            \param index  Index of combo box item selected
          */
          void onInputLayerComboBoxActivated(int index);

          void onConverterToolButtonClicked();

      signals:

        void dataAccessSelectionChanged();

        private:

          std::unique_ptr<Ui::InputDataAccessItemWidgetForm> m_ui;                    //!< Pointer to Qt widget

          std::unique_ptr<te::qt::widgets::DataSetTypeFilter> m_dsTypeFiltertWidget;  //!< Qt Component to filter dataset type
      };
    } // end namespace qt
  }  // end namespace qt
}  // end namespace qt


#endif // __TERRALIB_VP_QT_INTERNAL_INPUTDATAACCESSITEMWIDGET_H

