/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

This file is part of the TerraLib - a Framework for building GIS enabled applications.

TerraLib is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License,
or (at your option) any later version.

TerraLib is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with TerraLib. See COPYING. If not, write to
TerraLib Team at <terralib-team@terralib.org>.
*/

/*!
 * \file src/terralib/vp/qt/InputDataAccessWidget.cpp
 *
 * \brief Implements a widget used to return a set of Data Access object.
 */

#include "InputDataAccessWidget.h"

#include "../../dataaccess/utils/DataAccess.h"

#include "InputDataAccessItemWidget.h"
#include "ui/ui_InputDataAccessWidgetForm.h"

// Qt Includes
#include <QGridLayout>

te::vp::qt::InputDataAccessWidget::InputDataAccessWidget(QWidget* parent, Qt::WindowFlags f)
  : QWidget(parent, f),
  m_ui(new Ui::InputDataAccessWidgetForm)
{
  m_ui->setupUi(this);
}

te::vp::qt::InputDataAccessWidget::~InputDataAccessWidget() = default;

Ui::InputDataAccessWidgetForm* te::vp::qt::InputDataAccessWidget::getForm() const
{
  return m_ui.get();
}

void te::vp::qt::InputDataAccessWidget::setNumbersOfInputDataAccess(const std::size_t& nDataAccess, const std::list<te::map::AbstractLayerPtr>& layers)
{
  for (std::size_t i = 0; i < nDataAccess; ++i)
  {
    te::vp::qt::InputDataAccessItemWidget* item = new te::vp::qt::InputDataAccessItemWidget(m_ui->m_groupBox);
    connect(item, SIGNAL(dataAccessSelectionChanged()), this, SLOT(onDataAccessSelectionChanged()));
    m_ui->m_groupBox->layout()->addWidget(item);

    inputDataAccessItemWidgetVec.push_back(item);

    item->setLayerList(layers);
  }
}

std::vector<te::da::DataAccessPtr> te::vp::qt::InputDataAccessWidget::getDataAccessVector()
{
  std::vector<te::da::DataAccessPtr> dataAccessVec;

  for (std::size_t i = 0; i < inputDataAccessItemWidgetVec.size(); ++i)
  {
    te::vp::qt::InputDataAccessItemWidget* item = inputDataAccessItemWidgetVec[i];

    if (item != nullptr)
    {
      dataAccessVec.push_back(std::move(item->getDataAccess()));
    }
  }

  return dataAccessVec;
}

//!< Configures the widget to allow the filter of properties
void te::vp::qt::InputDataAccessWidget::setFilterPropertiesEnabled(bool enabled)
{
  for (std::size_t i = 0; i < inputDataAccessItemWidgetVec.size(); ++i)
  {
    te::vp::qt::InputDataAccessItemWidget* item = inputDataAccessItemWidgetVec[i];
    if (item != nullptr)
    {
      item->setFilterPropertiesEnabled(enabled);
    }
  }
}

void te::vp::qt::InputDataAccessWidget::onDataAccessSelectionChanged()
{
  emit dataAccessSelectionChanged();
}
