/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

This file is part of the TerraLib - a Framework for building GIS enabled applications.

TerraLib is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License,
or (at your option) any later version.

TerraLib is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with TerraLib. See COPYING. If not, write to
TerraLib Team at <terralib-team@terralib.org>.
*/

/*!
 * \file src/terralib/vp/qt/InputDataAccessWidget.h
 *
 * \brief Implements a widget used to return a set of Data Access object.
 */

#ifndef __TERRALIB_VP_QT_INTERNAL_INPUTDATAACCESSWIDGET_H
#define __TERRALIB_VP_QT_INTERNAL_INPUTDATAACCESSWIDGET_H

#include "../Config.h"

 // TerraLib Includes
#include "../../maptools/AbstractLayer.h"

// STL Includes
#include <memory>

 // Qt Includes
#include <QWidget>

namespace Ui { class InputDataAccessWidgetForm; }

namespace te
{
  namespace da
  {
    class DataAccessPtr;
  }

  namespace vp
  {
    namespace qt
    {
      class InputDataAccessItemWidget;

      /*!
        \class InputDataAccessWidget

        \brief  Implements a widget used to return a set of Data Access object.
      */
      class TEVPEXPORT InputDataAccessWidget : public QWidget
      {
        Q_OBJECT

        public:

          /*!
            \brief Constructor.
          */
          InputDataAccessWidget(QWidget* parent = 0, Qt::WindowFlags f = 0);

          /*!
            \brief Destructor.
          */
          ~InputDataAccessWidget();

      public:

          /*!
            \brief Returns the internal widget

            \return Pointer to Qt Widget InputDataAccessWidgetForm
          */
          Ui::InputDataAccessWidgetForm* getForm() const;

          /*!
            \brief Set the number of input data access

            \param nDataAccess Number of input data access
          */
          void setNumbersOfInputDataAccess(const std::size_t& nDataAccess, const std::list<te::map::AbstractLayerPtr>& layers);

          /*!
            \brief Get a vector of DataAccess given the user defined parameters in the interface

            \return a vector of TerraLib DataAccess object
          */
          std::vector<te::da::DataAccessPtr> getDataAccessVector();

          //!< Configures the widget to allow the filter of properties
          void setFilterPropertiesEnabled(bool enabled);

        protected slots:

          void onDataAccessSelectionChanged();

        signals:

          void dataAccessSelectionChanged();

        private:

          std::vector<te::vp::qt::InputDataAccessItemWidget*> inputDataAccessItemWidgetVec;

          std::unique_ptr<Ui::InputDataAccessWidgetForm> m_ui;  //!< Pointer to Qt widget
      };
    } // end namespace qt
  }  // end namespace qt
}  // end namespace qt


#endif // __TERRALIB_VP_QT_INTERNAL_INPUTDATAACCESSWIDGET_H

