/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

This file is part of the TerraLib - a Framework for building GIS enabled applications.

TerraLib is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License,
or (at your option) any later version.

TerraLib is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with TerraLib. See COPYING. If not, write to
TerraLib Team at <terralib-team@terralib.org>.
*/

/*!
 * \file src/terralib/vp/qt/OutputDataAccessItemWidget.h
 *
 * \brief Implements a widget used to return a Data Access object.
 */

#ifndef __TERRALIB_VP_QT_INTERNAL_OUTPUTDATAACCESSITEMWIDGET_H
#define __TERRALIB_VP_QT_INTERNAL_OUTPUTDATAACCESSITEMWIDGET_H

#include "../Config.h"

// TerraLib Includes
#include "../../dataaccess/datasource/DataSource.h"
#include "../../dataaccess/datasource/DataSourceInfo.h"

// STL Includes
#include <memory>

 // Qt Includes
#include <QWidget>

namespace Ui { class OutputDataAccessItemWidgetForm; }

namespace te
{
  namespace da
  {
    class DataAccessPtr;
  }

  namespace vp
  {
    namespace qt
    {
      /*!
        \class OutputDataAccessItemWidget

        \brief  Implements a widget used to return a Data Access object.
      */
      class TEVPEXPORT OutputDataAccessItemWidget : public QWidget
      {
        Q_OBJECT

        public:

          /*!
            \brief Constructor.
          */
          OutputDataAccessItemWidget(QWidget* parent = 0, Qt::WindowFlags f = 0);

          /*!
            \brief Destructor.
          */
          ~OutputDataAccessItemWidget();

        public:

          /*!
            \brief Returns the internal widget

            \return Pointer to Qt Widget OutputDataAccessItemWidget
          */
          Ui::OutputDataAccessItemWidgetForm* getForm() const;

          /*!
            \brief Used to get the output datasource given the user information

            \return Pointer to terraLib DataSource
          */
          te::da::DataSourcePtr getOutputDataSorce();

          /*!
            \brief Get DataAccess given the user defined parameters in the interface

            \return TerraLib DataAccess object
          */
          te::da::DataAccessPtr getDataAccess();

          //!< Resets the widget to its default clear state
          void reset();

        protected:

          /*!
            \brief Function used to register the new datasource into terralib datasource manager

            \param filePath     path of new file data source
            \param dataSetName  dataset name for the new data to be created

            \return Return the terralib object with datasource metadata
          */
          te::da::DataSourceInfoPtr registerFileDataSource(const std::string& filePath, const std::string& dataSetName);

        protected slots:

          /*!
            \brief Slot for target datasource tool button pressed
          */
          void onTargetDatasourceToolButtonPressed();

          /*!
            \brief Slot for target file tool button pressed
          */
          void onTargetFileToolButtonPressed();

        private:

          std::unique_ptr<Ui::OutputDataAccessItemWidgetForm> m_ui;   //!< Pointer to a internal Qt widget

          bool m_toFile;                                              //!< The result is in a file

          te::da::DataSourceInfoPtr m_outputDatasourceInfo;           //!< DataSource information metadata

          te::da::DataSourcePtr m_outputDatasource;                   //!< DataSource information
      };
    } // end namespace qt
  }  // end namespace qt
}  // end namespace qt


#endif // __TERRALIB_VP_QT_INTERNAL_OUTPUTDATAACCESSITEMWIDGET_H

