/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

This file is part of the TerraLib - a Framework for building GIS enabled applications.

TerraLib is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License,
or (at your option) any later version.

TerraLib is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with TerraLib. See COPYING. If not, write to
TerraLib Team at <terralib-team@terralib.org>.
*/

/*!
 * \file src/terralib/vp/qt/OutputDataAccessWidget.cpp
 *
 * \brief Implements a widget used to return a set of Data Access object.
 */

#include "OutputDataAccessWidget.h"

#include "../../dataaccess/utils/DataAccess.h"

#include "OutputDataAccessItemWidget.h"
#include "ui/ui_OutputDataAccessWidgetForm.h"

 // Qt Includes
#include <QTabWidget>

te::vp::qt::OutputDataAccessWidget::OutputDataAccessWidget(QWidget* parent, Qt::WindowFlags f)
  : QWidget(parent, f),
  m_ui(new Ui::OutputDataAccessWidgetForm)
{
  m_ui->setupUi(this);
}

te::vp::qt::OutputDataAccessWidget::~OutputDataAccessWidget() = default;

Ui::OutputDataAccessWidgetForm* te::vp::qt::OutputDataAccessWidget::getForm() const
{
  return m_ui.get();
}

void te::vp::qt::OutputDataAccessWidget::setNumbersOfOutputDataAccess(const std::size_t& nDataAccess)
{
  for (std::size_t i = 0; i < nDataAccess; ++i)
  {
    te::vp::qt::OutputDataAccessItemWidget* item = new te::vp::qt::OutputDataAccessItemWidget(m_ui->m_groupBox);

    outputDataAccessItemWidgetVec.push_back(item);

    m_ui->m_groupBox->layout()->addWidget(item);
  }
}

std::vector<te::da::DataAccessPtr> te::vp::qt::OutputDataAccessWidget::getDataAccessVector()
{
  std::vector<te::da::DataAccessPtr> dataAccessVec;

  for (std::size_t i = 0; i < outputDataAccessItemWidgetVec.size(); ++i)
  {
    te::vp::qt::OutputDataAccessItemWidget* item = outputDataAccessItemWidgetVec[i];

    if (item != nullptr)
    {
      dataAccessVec.push_back(std::move(item->getDataAccess()));
    }
  }

  return dataAccessVec;
}

void te::vp::qt::OutputDataAccessWidget::reset()
{
  for (std::size_t i = 0; i < outputDataAccessItemWidgetVec.size(); ++i)
  {
    te::vp::qt::OutputDataAccessItemWidget* item = outputDataAccessItemWidgetVec[i];

    if (item != nullptr)
    {
      item->reset();
    }
  }
}

