/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

This file is part of the TerraLib - a Framework for building GIS enabled applications.

TerraLib is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License,
or (at your option) any later version.

TerraLib is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with TerraLib. See COPYING. If not, write to
TerraLib Team at <terralib-team@terralib.org>.
*/

/*!
 * \file src/terralib/vp/qt/OutputDataAccessWidget.h
 *
 * \brief Implements a widget used to return a set of Data Access object.
 */

#ifndef __TERRALIB_VP_QT_INTERNAL_OUTPUTDATAACCESSWIDGET_H
#define __TERRALIB_VP_QT_INTERNAL_OUTPUTDATAACCESSWIDGET_H

#include "../Config.h"

 // STL Includes
#include <memory>

 // Qt Includes
#include <QWidget>

namespace Ui { class OutputDataAccessWidgetForm; }

namespace te
{
  namespace da
  {
    class DataAccessPtr;
  }

  namespace vp
  {
    namespace qt
    {
      class OutputDataAccessItemWidget;

      /*!
        \class OutputDataAccessItemWidget

        \brief  Implements a widget used to return a set of Data Access object.
      */
      class TEVPEXPORT OutputDataAccessWidget : public QWidget
      {
        Q_OBJECT

      public:

        /*!
          \brief Constructor.
        */
        OutputDataAccessWidget(QWidget* parent = 0, Qt::WindowFlags f = 0);

        /*!
          \brief Destructor.
        */
        ~OutputDataAccessWidget();

      public:

        /*!
          \brief Returns the internal widget

          \return Pointer to Qt Widget OutputDataAccessWidget
        */
        Ui::OutputDataAccessWidgetForm* getForm() const;

        /*!
          \brief Set the number of input data access

          \param nDataAccess Number of input data access
        */
        void setNumbersOfOutputDataAccess(const std::size_t& nDataAccess);

        /*!
          \brief Get a vector of DataAccess given the user defined parameters in the interface

          \return a vector of TerraLib DataAccess object
        */
        std::vector<te::da::DataAccessPtr> getDataAccessVector();

        //!< Resets the widget to its original clear state
        void reset();

      private:

        std::vector<te::vp::qt::OutputDataAccessItemWidget*> outputDataAccessItemWidgetVec;

        std::unique_ptr<Ui::OutputDataAccessWidgetForm> m_ui;  //!< Pointer to Qt widget
      };
    } // end namespace qt
  }  // end namespace qt
}  // end namespace qt


#endif // __TERRALIB_VP_QT_INTERNAL_OUTPUTDATAACCESSWIDGET_H

