/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

#ifndef __TERRALIB_VP_INTERNAL_REGISTERDIALOG_H
#define __TERRALIB_VP_INTERNAL_REGISTERDIALOG_H

// TerraLib
#ifndef Q_MOC_RUN
  #include "../../dataaccess/datasource/DataSourceInfo.h"
  #include "../../maptools/AbstractLayer.h"
#endif
#include "../Config.h"
#include "../TiePointsLocator.h"

// STL
#include <list>
#include <vector>
#include <map>
#include <memory>

// Qt
#include <QDialog>

namespace Ui { class RegisterDialogForm; }

// Forward declarations
class QListWidgetItem;
class QModelIndex;

namespace te
{
  namespace vp
  {
// Forward declarations
    class LayerTreeModel;

    class TEVPEXPORT RegisterDialog : public QDialog
    {
      Q_OBJECT

      public:

        RegisterDialog(QWidget* parent = 0, Qt::WindowFlags f = 0);

        ~RegisterDialog();

        /*!
          \brief Set the layer that can be used

          \param layers   List of AbstractLayerPtr
        */
        void setLayers(std::list<te::map::AbstractLayerPtr> layers);

        te::map::AbstractLayerPtr getLayer();

      protected slots:

        void onLayerComboBoxChanged(int index);

        void onTargetDatasourceToolButtonPressed();

        void onTargetFileToolButtonPressed();

        void onOkPushButtonClicked();

        void onCancelPushButtonClicked();

        void onAcquirePushButtonClicked();

      private:

        std::unique_ptr<Ui::RegisterDialogForm> m_ui;
        te::da::DataSourceInfoPtr m_outputDatasource;   //!< DataSource information.
        std::string m_outputArchive;                    //!< Archive information.
        std::list<te::map::AbstractLayerPtr> m_layers;  //!< List of layers.
        te::map::AbstractLayerPtr m_selectedLayer;      //!< Layer used for buffer
        te::map::AbstractLayerPtr m_outputLayer;        //!< Generated Layer.
//        std::vector<te::dt::Property*> m_properties;    //!< Properties related to the selected Layer
//        int m_newSRID;                                  //!< newSRID to compure buffer Area.
        bool m_toFile;
        std::vector< te::vp::TiePointsLocator::TiePointT > m_tiePoints;
    };
  }   // end namespace vp
}     // end namespace te

#endif  // __TERRALIB_VP_INTERNAL_REGISTERDIALOG_H
