/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

This file is part of the TerraLib - a Framework for building GIS enabled applications.

TerraLib is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License,
or (at your option) any later version.

TerraLib is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with TerraLib. See COPYING. If not, write to
TerraLib Team at <terralib-team@terralib.org>.
*/

/*!
 * \file src/terralib/vp/qt/SpecificParametersWidget.h
 *
 * \brief Abstract class used do define a component to set specific parameters for each operation.
 */

#ifndef __TERRALIB_VP_QT_INTERNAL_SPECIFICPARAMETERSWIDGET_H
#define __TERRALIB_VP_QT_INTERNAL_SPECIFICPARAMETERSWIDGET_H

#include "../Config.h"

// Qt
#include <QWidget>

//STL include files
#include <memory>
#include <vector>

namespace te
{
  namespace da
  {
    class DataAccessPtr;
  }

  namespace vp
  {
    class AbstractParameters;

    namespace qt
    {
      /*!
        \class SpecificParametersWidget

        \brief Abstract class used do define a component to set specific parameters for each operation.
      */
      class TEVPEXPORT SpecificParametersWidget
      {
        public:

          //!< Constructor
            SpecificParametersWidget() = default;

          //!< Destructor
          virtual ~SpecificParametersWidget() = default;

          virtual void setInputData(const std::vector<te::da::DataAccessPtr>& vecDataAccess) {}

        public:

          virtual QWidget* getWidget() = 0;

          virtual void setValuesToParameters(te::vp::AbstractParameters* parameters) = 0;

      };
    } //end namespace qt
  }//end namespace vp
}//end namespace te


#endif //__TERRALIB_VP_QT_INTERNAL_SPECIFICPARAMETERSWIDGET_H
