/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

This file is part of the TerraLib - a Framework for building GIS enabled applications.

TerraLib is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License,
or (at your option) any later version.

TerraLib is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with TerraLib. See COPYING. If not, write to
TerraLib Team at <terralib-team@terralib.org>.
*/

/*!
 * \file src/terralib/vp/qt/SubdivideParametersWidget.cpp
 *
 * \brief Implements a widget used to return the specific parameters for subdivide operation.
 */

#include "SubdivideParametersWidget.h"
#include "ui/ui_SubdivideParametersWidgetForm.h"

#include "../../common/Exception.h"
#include "../../core/translator/Translator.h"
#include "../SubdivideOperation.h"


te::vp::qt::SubdivideParametersWidget::SubdivideParametersWidget(QWidget* parent, Qt::WindowFlags f)
  : QWidget(parent, f), te::vp::qt::SpecificParametersWidget(),
  m_ui(new Ui::SubdivideParametersWidgetForm)
{
  m_ui->setupUi(this);
}

te::vp::qt::SubdivideParametersWidget::~SubdivideParametersWidget() = default;

QWidget* te::vp::qt::SubdivideParametersWidget::getWidget()
{
  return this;
}

void te::vp::qt::SubdivideParametersWidget::setValuesToParameters(te::vp::AbstractParameters* parameters)
{
  te::vp::SubdivideOperationParameters* specificParameters = dynamic_cast<te::vp::SubdivideOperationParameters*>(parameters);
  if (specificParameters == nullptr)
  {
    throw te::common::Exception(TE_TR("Invalid parameter type. Call the developer"));
  }

  bool ok = false;
  std::size_t maxCoordinates = static_cast<std::size_t>(m_ui->lineEdit->text().toUInt(&ok));

  if (ok == false)
  {
    throw te::common::Exception(TE_TR("Please type a valid value for Max Coordinates parameter"));
  }

  specificParameters->setMaxCoordinates(maxCoordinates);
}