/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

#ifndef __TERRALIB_VP_INTERNAL_TIEPOINTSLOCATORDIALOG_H
#define __TERRALIB_VP_INTERNAL_TIEPOINTSLOCATORDIALOG_H

// TerraLib
#ifndef Q_MOC_RUN
  #include "../../dataaccess/datasource/DataSourceInfo.h"
  #include "../../maptools/AbstractLayer.h"
#endif
#include "../Config.h"
#include "../TiePointsLocator.h"

// STL
#include <list>
#include <map>
#include <memory>

// Qt
#include <QDialog>

namespace Ui { class TiePointsLocatorDialogForm; }

// Forward declarations
class QListWidgetItem;
class QModelIndex;

namespace te
{
  namespace vp
  {
// Forward declarations
    class LayerTreeModel;

    class TEVPEXPORT TiePointsLocatorDialog : public QDialog
    {
      Q_OBJECT

      public:

        TiePointsLocatorDialog(QWidget* parent = 0, Qt::WindowFlags f = 0);

        ~TiePointsLocatorDialog();

        /*!
          \brief Set the layer that can be used

          \param layers   List of AbstractLayerPtr
        */
        void setLayers(std::list<te::map::AbstractLayerPtr> layers);

        void setRefLayer( te::map::AbstractLayerPtr newReflayer);
        
        void setAdjLayer( te::map::AbstractLayerPtr newAdjlayer);

        /*!
          \brief Return the current tie-points.
          \param tiePoints The current tie-points.
          \note Tie poit type definition [ SRID_adj, x2_adj, y2_adj, SRID_ref, x1_ref, y2_ref ].
        */
        void getTiePoints( std::vector< te::vp::TiePointsLocator::TiePointT >& tiePoints ) const;

      protected slots:

        void onRefLayerComboBoxChanged(int index);

        void onAdjLayerComboBoxChanged(int index);

        void onLocatePushButtonClicked();

        void onClearPushButtonClicked();

        void onLoadPushButtonClicked();

        void onSavePushButtonClicked();

        void onClosePushButtonClicked();

      private:

        std::unique_ptr<Ui::TiePointsLocatorDialogForm> m_ui;
        std::list<te::map::AbstractLayerPtr> m_layers;  //!< List of layers.
        te::map::AbstractLayerPtr m_selectedRefLayer;
        te::map::AbstractLayerPtr m_selectedAdjLayer;
        std::vector< te::vp::TiePointsLocator::TiePointT > m_tiePoints;
    };
  }   // end namespace vp
}     // end namespace te

#endif  // __TERRALIB_VP_INTERNAL_TIEPOINTSLOCATORDIALOG_H
