/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

This file is part of the TerraLib - a Framework for building GIS enabled applications.

TerraLib is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License,
or (at your option) any later version.

TerraLib is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with TerraLib. See COPYING. If not, write to
TerraLib Team at <terralib-team@terralib.org>.
*/

/*!
 * \file src/terralib/vp/qt/UnionDialogCapabilities.h
 *
 * \brief Implements a dialog used to execute a vector processing operation
 */

#ifndef __TERRALIB_VP_QT_INTERNAL_UNIONDIALOGCAPABILITIES_H
#define __TERRALIB_VP_QT_INTERNAL_UNIONDIALOGCAPABILITIES_H

#include "../Config.h"

#include "VectorProcessingDialogCapabilities.h"

//STL includes
#include <memory>
#include <string>

class QWidget;

namespace te
{
  namespace vp
  {
    namespace qt
    {
      /*!
        \class UnionDialogCapabilities

        \brief A class used to define the capabilities of the intersection operation
      */
      class TEVPEXPORT UnionDialogCapabilities : public VectorProcessingDialogCapabilities
      {
      public:
        UnionDialogCapabilities();

        //!< Destructor
        virtual ~UnionDialogCapabilities();

      };


      /*!
        \class UnionDialogCapabilitiesFactory

        \brief Factory used to create VectorProcessingDialogCapabilities.
      */
      class TEVPEXPORT UnionDialogCapabilitiesFactory : public VectorProcessingDialogCapabilitiesFactory
      {
      public:

        /*!
          \brief Constructor.
        */
        UnionDialogCapabilitiesFactory();

        //!< Destructor
        virtual ~UnionDialogCapabilitiesFactory();

      protected:

        virtual VectorProcessingDialogCapabilities* build() override;
      };

    } //end namespace qt
  }//end namespace vp
}//end namespace te


#endif //__TERRALIB_VP_QT_INTERNAL_UNIONDIALOGCAPABILITIES_H
