/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/wfs/Utils.cpp
   
  \brief Utility functions for WFS driver.
*/

// TerraLib
#include "Utils.h"

#include "../dataaccess/datasource/DataSourceInfoManager.h"
#include "../dataaccess/query/BinaryOpEncoder.h"
#include "../dataaccess/query/FunctionEncoder.h"
#include "../dataaccess/query/SQLDialect.h"
#include "../dataaccess/query/UnaryOpEncoder.h"
#include "../maptools/AbstractLayer.h"


// Boost
#include <boost/lexical_cast.hpp>

// STL
#include <cassert>

bool te::wfs::IsWFSLayer(const te::map::AbstractLayer& layer)
{
 te::da::DataSourceInfoPtr info = te::da::DataSourceInfoManager::getInstance().get(layer.getDataSourceId());

 return (info.get() != nullptr && info->getType() == "WFS");
}

void te::wfs::DefineDataSourceDialect(te::da::SQLDialect* dialect)
{
  dialect->insert("+", new te::da::BinaryOpEncoder("+"));
  dialect->insert("-", new te::da::BinaryOpEncoder("-"));
  dialect->insert("*", new te::da::BinaryOpEncoder("*"));
  dialect->insert("/", new te::da::BinaryOpEncoder("/"));
  dialect->insert("=", new te::da::BinaryOpEncoder("="));
  dialect->insert("<>", new te::da::BinaryOpEncoder("<>"));
  dialect->insert(">", new te::da::BinaryOpEncoder(">"));
  dialect->insert("<", new te::da::BinaryOpEncoder("<"));
  dialect->insert(">=", new te::da::BinaryOpEncoder(">="));
  dialect->insert("<=", new te::da::BinaryOpEncoder("<="));
  dialect->insert("and", new te::da::BinaryOpEncoder("AND"));
  dialect->insert("or", new te::da::BinaryOpEncoder("OR"));
  dialect->insert("not", new te::da::UnaryOpEncoder("NOT"));
  dialect->insert("st_envelopeintersects", new te::da::FunctionEncoder("Intersection"));
}

