#include "WCSLayerSelector.h"

// TerraLib
#include "../../../../core/translator/Translator.h"
#include "../../../../dataaccess/datasource/DataSource.h"
#include "../../../../dataaccess/datasource/DataSourceManager.h"
#include "../../../../dataaccess/datasource/DataSourceInfoManager.h"
#include "../../../../maptools/DataSetLayer.h"
#include "../../../../qt/widgets/dataset/selector/DataSetSelectorDialog.h"
#include "../../../../qt/widgets/layer/utils/DataSet2Layer.h"
#include "../dataaccess/DataSource.h"
#include "../client/WCSClient.h"
#include "../client/DataTypes.h"

#include <memory>

// BOOST
#include <boost/uuid/random_generator.hpp>
#include <boost/uuid/uuid_io.hpp>

te::ws::ogc::wcs::qt::WCSLayerSelector::WCSLayerSelector(QWidget *parent, Qt::WindowFlags f)
  : te::qt::widgets::AbstractLayerSelector(parent, f)
{

}

te::ws::ogc::wcs::qt::WCSLayerSelector::~WCSLayerSelector() = default;

void te::ws::ogc::wcs::qt::WCSLayerSelector::set(const std::list<te::da::DataSourceInfoPtr> &datasources)
{
  m_datasources = datasources;
}

std::list<te::map::AbstractLayerPtr> te::ws::ogc::wcs::qt::WCSLayerSelector::getLayers()
{
  std::list<te::map::AbstractLayerPtr> layers;
  for(std::list<te::da::DataSourceInfoPtr>::iterator it = m_datasources.begin(); it != m_datasources.end(); ++it)
  {
    te::da::DataSourcePtr datasource = te::da::DataSourceManager::getInstance().find((*it)->getId());
    if(datasource.get() == nullptr)
    {
      datasource = te::da::DataSourceManager::getInstance().get((*it)->getId(), (*it)->getAccessDriver(), (*it)->getConnInfo());
      //if(datasource.get() == 0)
        //throw Exception(TE_TR("Could not retrieve the data source instance!"));
    }
    if(!datasource->isOpened())
      datasource->open();

    te::ws::ogc::wcs::da::DataSource* wcsDataSource = dynamic_cast<te::ws::ogc::wcs::da::DataSource*>(datasource.get());

    std::shared_ptr<te::ws::ogc::WCSClient> client = wcsDataSource->getWCSClient();

    std::unique_ptr<te::qt::widgets::DataSetSelectorDialog> ldialog(new te::qt::widgets::DataSetSelectorDialog(static_cast<QWidget*>(parent())));
    ldialog->set(*it, true);
    int retval = ldialog->exec();
    if(retval == QDialog::Rejected)
      continue;
    std::list<te::da::DataSetTypePtr> datasets = ldialog->getCheckedDataSets();

    std::list<te::da::DataSetTypePtr>::iterator itDataset = datasets.begin();

    for(;itDataset != datasets.end(); ++itDataset)
    {
      te::ws::ogc::wcs::CoverageRequest request;
      request.coverageID = (*itDataset)->getName();
      request.format = "image/tiff";

      //create data source for coverageDiskPath
      boost::uuids::basic_random_generator<boost::mt19937> gen;
      boost::uuids::uuid u = gen();
      std::string id = boost::uuids::to_string(u);

      te::common::TaskProgress task ("Getting Coverage", te::common::TaskProgress::UNDEFINED);
      task.setTotalSteps(0);

      std::string coverageDiskPath = client->getCoverage(request, &task);

      task.setMessage("Download Done.");

      te::da::DataSourceInfoPtr dsInfo(new te::da::DataSourceInfo);
      dsInfo->setConnInfo("file://" + coverageDiskPath);
      dsInfo->setTitle(coverageDiskPath);
      dsInfo->setAccessDriver("GDAL");
      dsInfo->setType("GDAL");
      dsInfo->setDescription(coverageDiskPath);
      dsInfo->setId(id);

      te::da::DataSourceInfoManager::getInstance().add(dsInfo);
      te::da::DataSourcePtr ds = te::da::DataSourceManager::getInstance().get(dsInfo->getId(), dsInfo->getType(), dsInfo->getConnInfo());
      std::string dataSetName = ds->getDataSetNames()[0];

      te::qt::widgets::DataSet2Layer converter(ds->getId());
      te::da::DataSetTypePtr dt(ds->getDataSetType(dataSetName).release());
      te::map::AbstractLayerPtr layer = converter(dt);

      layers.push_back(layer);
    }
  }

  return layers;
}
