/*  Copyright (C) 2017 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */


#ifndef __TERRALIB_WS_OGC_WMS_111_XMLPARSER_WMS_H
#define __TERRALIB_WS_OGC_WMS_111_XMLPARSER_WMS_H


#include "../Config.h"
#include "../XMLParser.h"

#include "../DataTypes.h"

namespace te
{
  namespace ws
  {
    namespace ogc
    {
      namespace wms
      {
        namespace version_1_1_1
        {

        /*!
          \class XMLParser

          \brief A class responsible to parse WMS 1.1.1 XML files to WMS TerraLib data types.
        */
        class TEOGCWMSEXPORT XMLParser : public te::ws::ogc::wms::XMLParser
        {

        public:

          XMLParser();

          virtual ~XMLParser();

          /*!
            \brief Method responsible to parse a WMS 1.1.1 XML GetCapabilities file and
            create an WMSCapabilities instance with document metadata.

            \param xmlPath path of the file that will be parsed.

            \return Return the WMS Server Capabilities
          */
          virtual WMSCapabilities parseCapabilities(const std::string& xmlPath);

          /*!
          \brief Method responsible to parse a WMS 1.1.1 XML GetCapabilities content
          and create an WMSCapabilities instance with document metadata.

          \param cotent The content that will be parsed.

          \return Return the WMS Server Capabilities
         */
          virtual WMSCapabilities parseCapabilitiesFromContent(const std::string& content);

          virtual Layer parseLayers(const std::unique_ptr<te::xml::Reader>& reader);

          virtual GeographicBoundingBox parseGeographicBoundingBox(const std::unique_ptr<te::xml::Reader>& reader);

          virtual Dimension parseDimension(const std::unique_ptr<te::xml::Reader>& reader);

          virtual BoundingBox parseBoundingBox(const std::unique_ptr<te::xml::Reader>& reader);

        };

        }
      }
    }
  }
}

#endif //__TERRALIB_WS_OGC_WMS_111_XMLPARSER_WMS_H
