/*  Copyright (C) 2017 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/ws/ogc/wms/client/AbstractWMSXMLParserFactory.h

  \brief This is the abstraction factory for WMS XMLParser.
*/

#ifndef __TERRALIB_WS_OGC_ABSTRACTWMSXMLPARSERFACTORY_WMS_H
#define __TERRALIB_WS_OGC_ABSTRACTWMSXMLPARSERFACTORY_WMS_H

#include "Config.h"

//TerraLib
#include "../../../../common/AbstractFactory.h"
#include "XMLParser.h"

namespace te
{
  namespace ws
  {
    namespace ogc
    {
      namespace wms
      {
      /*!
        \class AbstractWMSXMLParserFactory

        \brief This is the abstraction factory for WMS XMLParser.
        */
      class TEOGCWMSEXPORT AbstractWMSXMLParserFactory : public te::common::AbstractFactory<XMLParser, std::string>
      {

      public:

        /*!
          \brief Virtual destructor.

          \note It will automatically unregister the factory from the dictionary.
        */
        virtual ~AbstractWMSXMLParserFactory();

      protected:

        /*!
          \brief It creates the factory.

          \param factoryKey The key that identifies the factory.
        */
        AbstractWMSXMLParserFactory(const std::string& factoryKey);
      };
      }
    }
  }
}

#endif //__TERRALIB_WS_OGC_ABSTRACTWMSXMLPARSERFACTORY_WMS_H
