/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */


#ifndef __TERRALIB_WS_OGC_WMS_XMLPARSER_WMS_H
#define __TERRALIB_WS_OGC_WMS_XMLPARSER_WMS_H

#include "DataTypes.h"

// TerraLib
#include "../../../../xml.h"

namespace te
{
  namespace ws
  {
    namespace ogc
    {
      namespace wms
      {
      /*!
        \class XMLParser

        \brief A class responsible to parse WMS XML files to WMS TerraLib data types.
      */
      class TEOGCWMSEXPORT XMLParser
      {
      public:

        XMLParser();

        virtual ~XMLParser();

        /*!
          \brief Method responsible to parse a WMS XML GetCapabilities file and
          create an WMSCapabilities instance with document metadata.

          \param xmlPath path of the file that will be parsed.

          \return Return the WMS Server Capabilities
        */
        virtual WMSCapabilities parseCapabilities(const std::string& xmlPath);

        /*!
          \brief Method responsible to parse a WMS XML GetCapabilities content and
          create an WMSCapabilities instance with document metadata.

          \param cotent The content that will be parsed.

          \return Return the WMS Server Capabilities
        */
        virtual WMSCapabilities parseCapabilitiesFromContent(const std::string& content);

        virtual Capability parseCapability(const std::unique_ptr<te::xml::Reader>& reader);

        virtual Request parseRequest(const std::unique_ptr<te::xml::Reader>& reader);

        virtual Layer parseLayers(const std::unique_ptr<te::xml::Reader>& reader);

        virtual GeographicBoundingBox parseGeographicBoundingBox(const std::unique_ptr<te::xml::Reader>& reader);

        virtual BoundingBox parseBoundingBox(const std::unique_ptr<te::xml::Reader>& reader);

        virtual Dimension parseDimension(const std::unique_ptr<te::xml::Reader>& reader);

        virtual Style parseStyle(const std::unique_ptr<te::xml::Reader>& reader);

        virtual LegendURL parseLegendURL(const std::unique_ptr<te::xml::Reader>& reader);

        virtual OnlineResource parseOnlineResource(const std::unique_ptr<te::xml::Reader>& reader);

        virtual StyleSheetURL parseStyleSheetURL(const std::unique_ptr<te::xml::Reader>& reader);

        virtual StyleURL parseStyleURL(const std::unique_ptr<te::xml::Reader>& reader);

        virtual Operation parseOperation(const std::unique_ptr<te::xml::Reader>& reader, const std::string& operationName);

        virtual HTTP parseHTTP(const std::unique_ptr<te::xml::Reader>& reader);

        std::string getVersion();

      protected:

        std::string m_version; //!< WMS version that XMLParser instance could read. Default implementation is 1.3.0.

      };
      }
    }
  }
}

#endif //__TERRALIB_WS_OGC_WMS_XMLPARSER_WMS_H
