/*  Copyright (C) 2017 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/ws/ogc/wms/qt/WMSGetFeatureInfoMaker.h

  \brief WMSGetFeatureInfoMaker is a WMS implementation of AbstractFeatureInfoMaker
         that could be used to fill a QTreeWidgetItem with GetFeatureInfo WMS operation based information.
*/


#ifndef __TERRALIB_WS_OGC_WMS_QT_WMSGETFEATUREINFOMAKER_H
#define __TERRALIB_WS_OGC_WMS_QT_WMSGETFEATUREINFOMAKER_H

#include "Config.h"
#include "../../../../maptools/AbstractLayer.h"
#include "../../../../qt/widgets/layer/info/AbstractFeatureInfoMaker.h"

// Qt
#include <QTreeWidgetItem>

namespace te
{
  namespace da
  {
    class DataSet;
  }

  namespace gm
  {
    class Envelope;
  }

  namespace qt
  {
    namespace widgets
    {
    //Foward declaration
    class MapDisplay;
    }
  }

  namespace ws
  {
    namespace ogc
    {
      namespace wms
      {
        namespace qt
        {

        class TEOGCWMSQTEXPORT WMSGetFeatureInfoMaker : public te::qt::widgets::AbstractFeatureInfoMaker
        {

        public:

          WMSGetFeatureInfoMaker();

          virtual ~WMSGetFeatureInfoMaker();

          /*!
           * \brief This function is an implementation to get feature information.
           *        The information based on WMS GetFeatureInfo of the layer will be embedded in QTreeWidgetItem.
           *
           * \param layerItem QTreeWidgetItem that will be populated.
           * \param display MapDisplay that will be passed by Info tool.
           * \param layer AbstractLayerPtr to retrieve the feature information.
           * \param e Envelope with the box based on user's click.
           */
          void getInfo(QTreeWidgetItem* layerItem, te::qt::widgets::MapDisplay* display, const te::map::AbstractLayerPtr& layer, const te::gm::Envelope& e);

        };

        }
      }
    }
  }
}

#endif // __TERRALIB_WS_OGC_WMS_QT_WMSGETFEATUREINFOMAKER_H
