/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled
   applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

#include <terralib/datatype/Exception.h>
#include <terralib/datatype/TimeInstantTZ.h>

// Boost
#include <boost/algorithm/string/predicate.hpp>
#include <boost/date_time/local_time/local_time.hpp>
#include <boost/test/unit_test.hpp>

BOOST_AUTO_TEST_SUITE(time_instant_tz_test_case)

BOOST_AUTO_TEST_CASE(test_ok)
{
  boost::local_time::time_zone_ptr tz_ptr(
      new boost::local_time::posix_time_zone("UTC"));

  te::dt::TimeInstantTZ ti_tz(boost::local_time::local_date_time(
      boost::gregorian::date(2016, boost::gregorian::Dec, 16),
      boost::posix_time::hours(0), tz_ptr,
      boost::local_time::local_date_time::NOT_DATE_TIME_ON_ERROR));

  BOOST_CHECK(ti_tz.toString() != "");
}

BOOST_AUTO_TEST_CASE(test_no_tz)
{
  te::dt::TimeInstantTZ ti_tz(boost::local_time::local_date_time(
      boost::gregorian::date(2016, boost::gregorian::Dec, 16),
      boost::posix_time::hours(0), nullptr,
      boost::local_time::local_date_time::NOT_DATE_TIME_ON_ERROR));

  BOOST_CHECK(!boost::algorithm::ends_with(ti_tz.toString(), "UTC+00"));
}

BOOST_AUTO_TEST_CASE(test_special_date)
{
  boost::local_time::time_zone_ptr tz_ptr(
      new boost::local_time::posix_time_zone("UTC"));

  te::dt::TimeInstantTZ ti_tz(boost::local_time::local_date_time(
      boost::posix_time::not_a_date_time, tz_ptr));

  BOOST_CHECK(ti_tz.toString() == "");
}
BOOST_AUTO_TEST_SUITE_END()
