/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

#ifndef __TERRALIB_UNITTEST_GEOMETRY_INTERNAL_VALIDATION_H
#define __TERRALIB_UNITTEST_GEOMETRY_INTERNAL_VALIDATION_H

#include "../../src/terralib/geometry.h"

// cppUnit
#include <cppunit/extensions/HelperMacros.h>

#include <string>

/*!
  \class TsValidation

  \brief Test suite for the te::gm::Validation class
 */
class TsValidation : public CPPUNIT_NS::TestFixture
{
  CPPUNIT_TEST_SUITE( TsValidation );

  CPPUNIT_TEST( tcMakeValid_hole_lies_outside_shell );
  
  CPPUNIT_TEST( tcValidate_hole_lies_outside_shell );
  
  CPPUNIT_TEST( tcMakeValidWithSnap_hole_lies_outside_shell );
  
  CPPUNIT_TEST_SUITE_END();    
  
  public:

    void setUp();

    void tearDown();

  protected:
    
    bool loadSHPFile( const std::string& shpFileName,
      std::vector< te::gm::Geometry* >& geomPtrs );
    
    void saveToShp( std::vector<te::gm::Polygon*> polygons, 
      std::vector< double >& polygonsValues,
      const std::string& shpBaseFileName );
    
    void tcMakeValid_hole_lies_outside_shell();
    
    void tcValidate_hole_lies_outside_shell();
    
    void tcMakeValidWithSnap_hole_lies_outside_shell();
};

#endif

