/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

#include "TsMemRaster.h"

CPPUNIT_TEST_SUITE_REGISTRATION( TsMemRaster );

void TsMemRaster::TestRasterDatatype( unsigned int nBands, unsigned int nLines,
  unsigned int nCols, int dataType )
{
  std::vector< te::rst::BandProperty * > bandsProps;
  for( unsigned int bandsPropsIdx = 0 ; bandsPropsIdx < nBands ; ++bandsPropsIdx )
  {
    bandsProps.push_back( new te::rst::BandProperty( bandsPropsIdx, 
      dataType ) );
    bandsProps[ bandsPropsIdx ]->m_blkh = 1;
    bandsProps[ bandsPropsIdx ]->m_blkw = nCols;
    bandsProps[ bandsPropsIdx ]->m_nblocksx = 1;
    bandsProps[ bandsPropsIdx ]->m_nblocksy = nLines;
  }
  
  std::unique_ptr< te::rst::Raster > rasterPointer;
  try
  {
    rasterPointer.reset( te::rst::RasterFactory::make( "MEM",
      new te::rst::Grid( nCols, nLines ), bandsProps,
      std::map< std::string, std::string >(), 0, 0 ) );
  }
  catch(...)
  {
    rasterPointer.reset();
  }
  CPPUNIT_ASSERT( rasterPointer.get() != nullptr );

  unsigned int band = 0;
  unsigned int line = 0;
  unsigned int col = 0;
  std::complex< double > pixelValue( 0, 0 );
  
  for( band = 0 ; band < nBands ; ++band )
    for( line = 0 ; line < nLines ; ++line )
      for( col = 0 ; col < nCols ; ++col )
      {
        rasterPointer->setValue( col, line, pixelValue, band );
        pixelValue.real( pixelValue.real() + 1 );
        pixelValue.imag( pixelValue.imag() + 1 );
      }

  std::complex< double > testValue( 0, 0 );

  for( band = 0 ; band < nBands ; ++band )
    for( line = 0 ; line < nLines ; ++line )
      for( col = 0 ; col < nCols ; ++col )
      {
        rasterPointer->getValue( col, line, pixelValue, band );

        CPPUNIT_ASSERT_DOUBLES_EQUAL( pixelValue.real(), testValue.real(), 0.0000001 );

        testValue.real( testValue.real() + 1 );
        testValue.imag( testValue.imag() + 1 );
      }
}

void TsMemRaster::Test8BitsU()
{
  TestRasterDatatype( 5, 5, 2, te::dt::CHAR_TYPE );
}

void TsMemRaster::Test8Bits()
{
  TestRasterDatatype( 10, 10, 2, te::dt::UCHAR_TYPE );
}

void TsMemRaster::Test16Bits()
{
  TestRasterDatatype( 10, 10, 10, te::dt::INT16_TYPE );
}

void TsMemRaster::Test16BitsU()
{
  TestRasterDatatype( 10, 10, 10, te::dt::UINT16_TYPE );
}

void TsMemRaster::Test32Bits()
{
  TestRasterDatatype( 10, 10, 10, te::dt::INT32_TYPE );
}

void TsMemRaster::Test32BitsU()
{
  TestRasterDatatype( 10, 10, 10, te::dt::UINT32_TYPE );
}

void TsMemRaster::Test64Bits()
{
  TestRasterDatatype( 10, 10, 10, te::dt::INT64_TYPE );
}

void TsMemRaster::Test64BitsU()
{
  TestRasterDatatype( 10, 10, 10, te::dt::UINT64_TYPE );
}

void TsMemRaster::TestFloat()
{
  TestRasterDatatype( 10, 10, 10, te::dt::FLOAT_TYPE );
}

void TsMemRaster::TestDouble()
{
  TestRasterDatatype( 10, 10, 10, te::dt::DOUBLE_TYPE );
}

void TsMemRaster::Test16BitsC()
{
  TestRasterDatatype( 10, 10, 10, te::dt::CINT16_TYPE );
}

void TsMemRaster::Test32BitsC()
{
  TestRasterDatatype( 10, 10, 10, te::dt::CINT32_TYPE );
}

void TsMemRaster::TestFloatC()
{
  TestRasterDatatype( 10, 10, 10, te::dt::CFLOAT_TYPE );
}

void TsMemRaster::TestDoubleC()
{
  TestRasterDatatype( 10, 10, 10, te::dt::CDOUBLE_TYPE );
}
