/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/unittest/postgis/Utils.cpp

  \brief A Utils functions for  postgis unit test.

 */

// TerraLib
//#include <terralib/dataaccess/datasource/DataSource.h>
//#include <terralib/dataaccess/datasource/DataSourceFactory.h>
#include <terralib/dataaccess.h>
//#include <terralib/postgis/Transactor.h>

#include "Utils.h"

// STL
#include <iostream>

#define PGIS_UTEST_UTILS_HOST "127.0.0.1"
#define PGIS_UTEST_UTILS_PORT "5432"
#define PGIS_UTEST_UTILS_USER "postgres"
#define PGIS_UTEST_UTILS_PASS "postgres"
#define PGIS_UTEST_UTILS_PATH "postgres"
#define PGIS_UTEST_UTILS_DSID "PGIS_UTEST_UTILS_DSID"
#define PGIS_UTEST_UTILS_TYPE "POSTGIS"

std::unique_ptr<te::da::DataSource> GetPostGISConnection()
{
  // let's give the minimal server connection information needed to connect to
  // the database server
  std::string user, password, host, port, path, query;
  std::string strURI = "pgsql://"; // The base of the URI
  host = PGIS_UTEST_UTILS_HOST;
  port = PGIS_UTEST_UTILS_PORT;
  user = PGIS_UTEST_UTILS_USER;
  password = PGIS_UTEST_UTILS_PASS;
  path = PGIS_UTEST_UTILS_PATH;
  query = "PG_CLIENT_ENCODING=" +
          te::core::CharEncoding::getEncodingName(te::core::EncodingType::UTF8);
  query += "&PG_CONNECT_TIMEOUT=4";

  strURI += user + ":";
  strURI += password + "@";
  strURI += host + ":";
  strURI += port + "/";
  strURI += path + "?";
  strURI += query;

  // create a data source using the data source factory
  std::unique_ptr<te::da::DataSource> ds = te::da::DataSourceFactory::make(PGIS_UTEST_UTILS_TYPE, strURI);

  try
  {
    ds->open();
  }
  catch(const std::exception& e)
  {
    std::cout << "Datasource " << host << "/" << path
              << " can not be used!\nMake sure to have the correct connection "
                 "parameters\n";
    std::cout << "Error: " << e.what() << std::endl;
    ds.reset();
    return ds;
  }
  catch(...)
  {
    std::cout << "Datasource " << host << "/" << path
              << " can not be used!\nMake sure to have the correct connection "
                 "parameters\n";
    ds.reset();
    return ds;
  }

  std::cout << "Using datasource " << host << "/" <<  path << std::endl;
  ds->close();
  return ds;
}

te::da::DataSourcePtr GetPostGISConnectionFromDSManager()
{
  te::da::DataSourcePtr datasource = te::da::DataSourceManager::getInstance().find(
    PGIS_UTEST_UTILS_DSID);

  if( datasource.get() == nullptr )
  {
    std::string strURI = "pgsql://";
    strURI += PGIS_UTEST_UTILS_USER ":";
    strURI += PGIS_UTEST_UTILS_PASS "@";
    strURI += PGIS_UTEST_UTILS_HOST ":";
    strURI += PGIS_UTEST_UTILS_PORT "/";
    strURI += PGIS_UTEST_UTILS_PATH "?";
    strURI += "PG_CLIENT_ENCODING=" +
      te::core::CharEncoding::getEncodingName(te::core::EncodingType::UTF8)
      + "&PG_CONNECT_TIMEOUT=4";

    try
    {
      datasource = te::da::DataSourceManager::getInstance().make(PGIS_UTEST_UTILS_DSID,
        PGIS_UTEST_UTILS_TYPE, strURI);
    }
    catch(const std::exception& e)
    {
      std::cout << "Datasource open error:" << e.what();
      throw e;
    }
  }

  if( !datasource->isOpened() ) datasource->open();

  return datasource;
}