/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/unittest/postgis/Utils.h

  \brief Utility functions for Postgis UnitTest.
*/

#ifndef __TERRALIB_UNITTEST_POSTGIS_UTILS_H
#define __TERRALIB_UNITTEST_POSTGIS_UTILS_H

// Terralib
#include <terralib/dataaccess.h>

//STL
#include <memory>

/*!
  \brief It returns the union of a geometry vector.

  \param items  Vector of itens that represents a group.

  \return Union of the geometry.
*/
std::unique_ptr<te::da::DataSource> GetPostGISConnection();

/*!
  \brief Returns a pointer to the required datasource using the datasource maanager.

  \param datasourceId DataSource ID or an empty string to open a new datasource.

  \return A pointer to the datasource or a null pointer if errors ocurred.
*/
te::da::DataSourcePtr GetPostGISConnectionFromDSManager();


#endif  // __TERRALIB_UNITTEST_POSTGIS_UTILS_H
