/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/unittest/raster/TsBand.cpp
 
  \brief A test suit for the Raster Band class.
 */

// TerraLib
#include <terralib_buildconfig.h>

#include <terralib/raster.h>
#include "../Config.h"

// Boost
#include <boost/test/unit_test.hpp>

#include <memory>

BOOST_AUTO_TEST_SUITE ( band_tests )

BOOST_AUTO_TEST_CASE (getHistograms_test)
{
  /* Openning input raster */

  std::map<std::string, std::string> auxRasterInfo;

  auxRasterInfo["URI"] = TERRALIB_DATA_DIR "/geotiff/cbers_rgb342_crop3_EPSG_22522.tif";
  std::unique_ptr< te::rst::Raster > inputRasterPtr ( te::rst::RasterFactory::open(
    auxRasterInfo ) );
  BOOST_REQUIRE( inputRasterPtr.get() );
  
  std::map<double, unsigned> histogramR;
  inputRasterPtr->getBand( 0 )->getHistogramR( 0, 0, 
    inputRasterPtr->getNumberOfRows() -1, inputRasterPtr->getNumberOfColumns() -1,
    0, 1, histogramR );
  BOOST_REQUIRE( !histogramR.empty() );
  
  std::map<double, unsigned> histogramI =
    inputRasterPtr->getBand( 0 )->getHistogramI( 0, 0, 
      inputRasterPtr->getNumberOfRows() -1, inputRasterPtr->getNumberOfColumns() -1 );
  BOOST_REQUIRE( histogramI.size() == 1 );
}

BOOST_AUTO_TEST_CASE (getStats_test)
{
  /* Openning input raster */

  std::map<std::string, std::string> auxRasterInfo;

  auxRasterInfo["URI"] = TERRALIB_DATA_DIR "/geotiff/cbers_rgb342_crop3_EPSG_22522.tif";
  std::unique_ptr< te::rst::Raster > inputRasterPtr ( te::rst::RasterFactory::open(
    auxRasterInfo ) );
  BOOST_REQUIRE( inputRasterPtr.get() );
  
  std::complex<double> value;
  
  value = inputRasterPtr->getBand( 0 )->getMinValue( true, 0, 0, 
    inputRasterPtr->getNumberOfRows() -1, inputRasterPtr->getNumberOfColumns() -1 );
  BOOST_REQUIRE_CLOSE( value.real(), 24, 0.01 );  
  
  value = inputRasterPtr->getBand( 0 )->getMaxValue( true, 0, 0, 
    inputRasterPtr->getNumberOfRows() -1, inputRasterPtr->getNumberOfColumns() -1 );
  BOOST_REQUIRE_CLOSE( value.real(), 184, 0.01 );
  
  value = inputRasterPtr->getBand( 0 )->getMeanValue( 0, 0, 
    inputRasterPtr->getNumberOfRows() -1, inputRasterPtr->getNumberOfColumns() -1 );
  BOOST_REQUIRE_CLOSE( value.real(), 42.163915616593513, 0.01 ); 
  
  value = inputRasterPtr->getBand( 0 )->getStdValue( 0, 0, 
    inputRasterPtr->getNumberOfRows() -1, inputRasterPtr->getNumberOfColumns() -1 );
  BOOST_REQUIRE_CLOSE( value.real(), 7.1024926078435477, 0.01 );   
}

BOOST_AUTO_TEST_SUITE_END()
