/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

This file is part of the TerraLib - a Framework for building GIS enabled applications.

TerraLib is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License,
or (at your option) any later version.

TerraLib is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with TerraLib. See COPYING. If not, write to
TerraLib Team at <terralib-team@terralib.org>.
*/

/*!
\file terralib/unittest/rp/cloud_detection/TsCloudDetection.cpp

\brief A test suit for the Cloud Detection interface.
*/

// TerraLib
#include "../Config.h"
#include <terralib/rp/PostClassification.h>
#include <terralib/rp.h>
#include <terralib/raster.h>

// Boost
#define BOOST_TEST_NO_MAIN
#include <boost/test/unit_test.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/uuid/random_generator.hpp>
#include <boost/uuid/uuid_io.hpp>

BOOST_AUTO_TEST_CASE(postClassification_test)
{
  /* Openning input raster */

  std::map<std::string, std::string> rasterInfo;

  rasterInfo["URI"] = "terralib_unittest_rp_Classifier_ED_Test.tif";//TERRALIB_DATA_DIR "/geotiff/terralib_unittest_rp_Classifier_ED_Test.tif";
  te::rst::Raster* inputRaster = te::rst::RasterFactory::open(rasterInfo);

  /* Create output raster info */

  std::map<std::string, std::string> outputInfo;
  outputInfo["URI"] = "terralib_unittest_rp_PostClassification.tif";

  /* Creating the algorithm parameters */

  te::rp::PostClassification algorithmInstance;

  te::rp::PostClassification::InputParameters algoInputParams;
  algoInputParams.m_inRasterPtr = inputRaster;
  algoInputParams.m_weight = 1;
  algoInputParams.m_threshold = 1;

  te::rp::PostClassification::OutputParameters algoOutputParams;
  algoOutputParams.m_rInfo = outputInfo;
  algoOutputParams.m_rType = "GDAL";

  /* Run post classification */

  BOOST_CHECK(algorithmInstance.initialize(algoInputParams));
  BOOST_CHECK(algorithmInstance.execute(algoOutputParams));

  delete inputRaster;
}
