/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file terralib/unittest/rp/filter/TsFilter.cpp

  \brief A test suit for the Filter interface.
*/

// TerraLib
#include "../Config.h"
#include <terralib/rp.h>
#include <terralib/raster.h>

// Boost
#define BOOST_TEST_NO_MAIN
#include <boost/test/unit_test.hpp>
#include <boost/shared_ptr.hpp>

BOOST_AUTO_TEST_SUITE (restoration_tests)

BOOST_AUTO_TEST_CASE(restoration_test_CBERS2_CCD_SAMPLING_FACTOR_1_BY_2)
{
  /* Openning input raster */

  std::map<std::string, std::string> auxRasterInfo;

  auxRasterInfo["URI"] = TERRALIB_DATA_DIR "/geotiff/cbers_rgb342_crop1.tif";
  boost::shared_ptr< te::rst::Raster > inputRasterPtrPointer ( te::rst::RasterFactory::open(
    auxRasterInfo ) );
  BOOST_CHECK( inputRasterPtrPointer.get() );

  /* Creating the algorithm parameters */

  te::rp::Restoration::InputParameters algoInputParams;

  algoInputParams.m_sensorType = te::rp::Restoration::InputParameters::CBERS2_CCD;
  algoInputParams.m_sensorBandDesignations.push_back( 3 );
  algoInputParams.m_sensorBandDesignations.push_back( 4 );
  algoInputParams.m_sensorBandDesignations.push_back( 2 );
  algoInputParams.m_inRasterPtr = inputRasterPtrPointer.get();
  algoInputParams.m_inRasterBands.push_back( 0 );
  algoInputParams.m_inRasterBands.push_back( 1 );
  algoInputParams.m_inRasterBands.push_back( 2 );  
  algoInputParams.m_samplingFactor = te::rp::Restoration::InputParameters::SAMPLING_FACTOR_1_BY_2;
  algoInputParams.m_enableProgress = true;
  algoInputParams.m_enableRasterCache = true;

  te::rp::Restoration::OutputParameters algoOutputParams;

  algoOutputParams.m_rInfo["URI"] = "terralib_unittest_rp_restoration_test_CBERS2_CCD_SAMPLING_FACTOR_1_BY_2.tif";
  algoOutputParams.m_rType = "GDAL";

  /* Executing the algorithm */

  te::rp::Restoration algorithmInstance;

  BOOST_CHECK( algorithmInstance.initialize( algoInputParams ) );
  BOOST_CHECK( algorithmInstance.execute( algoOutputParams ) );
}

BOOST_AUTO_TEST_CASE(restoration_test_CBERS2_CCD_SAMPLING_FACTOR_2_BY_3)
{
  /* Openning input raster */

  std::map<std::string, std::string> auxRasterInfo;

  auxRasterInfo["URI"] = TERRALIB_DATA_DIR "/geotiff/cbers_rgb342_crop1.tif";
  boost::shared_ptr< te::rst::Raster > inputRasterPtrPointer ( te::rst::RasterFactory::open(
    auxRasterInfo ) );
  BOOST_CHECK( inputRasterPtrPointer.get() );

  /* Creating the algorithm parameters */

  te::rp::Restoration::InputParameters algoInputParams;

  algoInputParams.m_sensorType = te::rp::Restoration::InputParameters::CBERS2_CCD;
  algoInputParams.m_sensorBandDesignations.push_back( 3 );
  algoInputParams.m_sensorBandDesignations.push_back( 4 );
  algoInputParams.m_sensorBandDesignations.push_back( 2 );
  algoInputParams.m_inRasterPtr = inputRasterPtrPointer.get();
  algoInputParams.m_inRasterBands.push_back( 0 );
  algoInputParams.m_inRasterBands.push_back( 1 );
  algoInputParams.m_inRasterBands.push_back( 2 );  
  algoInputParams.m_samplingFactor = te::rp::Restoration::InputParameters::SAMPLING_FACTOR_2_BY_3;
  algoInputParams.m_enableProgress = true;
  algoInputParams.m_enableRasterCache = true;

  te::rp::Restoration::OutputParameters algoOutputParams;

  algoOutputParams.m_rInfo["URI"] = "terralib_unittest_rp_restoration_test_CBERS2_CCD_SAMPLING_FACTOR_2_BY_3.tif";
  algoOutputParams.m_rType = "GDAL";

  /* Executing the algorithm */

  te::rp::Restoration algorithmInstance;

  BOOST_CHECK( algorithmInstance.initialize( algoInputParams ) );
  BOOST_CHECK( algorithmInstance.execute( algoOutputParams ) );
}

BOOST_AUTO_TEST_SUITE_END()
