/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

//TerraLib
#include <terralib_buildconfig.h>
#include <terralib/common.h>
#include <terralib/vp.h>
#include <terralib/geometry.h>
#include <terralib/dataaccess.h>
#include <terralib/dataaccess/utils/DataAccess.h>

// Boost
#include <boost/test/unit_test.hpp>
#include <boost/uuid/uuid_io.hpp>

#include <cstdio>

bool MatchDataSetsTest( bool useThreads )
{
  te::common::ConsoleProgressViewer progressViewerInstance;

  std::unique_ptr< te::da::DataSource > inDataSource1Ptr(
    te::da::DataSourceFactory::make( "OGR", "file://" TERRALIB_DATA_DIR
    "/shape/munic_goias.shp" ) );
  BOOST_REQUIRE( inDataSource1Ptr.get() != 0 );

  inDataSource1Ptr->open();

  BOOST_REQUIRE( inDataSource1Ptr->isOpened() );

  std::vector< std::string > dataSetNames1 = inDataSource1Ptr->getDataSetNames();
  BOOST_REQUIRE( !dataSetNames1.empty() );

  std::unique_ptr< te::da::DataSourceTransactor > inTransactor1Ptr =
    inDataSource1Ptr->getTransactor();

  std::unique_ptr<te::da::DataSet> inDataSet1Ptr = inTransactor1Ptr->getDataSet(
    dataSetNames1[ 0 ] );
  BOOST_REQUIRE( inDataSet1Ptr.get() != 0 );

  std::unique_ptr<te::da::DataSetType> inDsType1 =
    inTransactor1Ptr->getDataSetType(	dataSetNames1[ 0 ] );

  std::unique_ptr< te::da::DataSource > inDataSource2Ptr(
    te::da::DataSourceFactory::make( "OGR", "file://" TERRALIB_DATA_DIR
    "/shape/munic_2001.shp" ) );
  BOOST_REQUIRE( inDataSource2Ptr.get() != 0 );

  inDataSource2Ptr->open();

  BOOST_REQUIRE( inDataSource2Ptr->isOpened() );

  std::vector< std::string > dataSetNames2 = inDataSource2Ptr->getDataSetNames();
  BOOST_REQUIRE( !dataSetNames2.empty() );

  std::unique_ptr< te::da::DataSourceTransactor > inTransactor2Ptr =
    inDataSource2Ptr->getTransactor();

  std::unique_ptr<te::da::DataSet> inDataSet2Ptr = inTransactor2Ptr->getDataSet(
    dataSetNames2[ 0 ] );
  BOOST_REQUIRE( inDataSet2Ptr.get() != 0 );

  std::unique_ptr<te::da::DataSetType> inDsType2 =
    inTransactor2Ptr->getDataSetType(	dataSetNames2[ 0 ] );

  std::vector< te::vp::TiePointsLocator::TiePointT > tiePoints;
  BOOST_REQUIRE( te::vp::TiePointsLocator::MatchDataSets( *inDataSet1Ptr,
    *inDsType1, *inDataSet2Ptr, *inDsType2, useThreads, 0.001, 0.001, 0.1,
    tiePoints ) );
  BOOST_REQUIRE( tiePoints.size() == 102 );

  return true;
}

BOOST_AUTO_TEST_SUITE( tiepointslocator_tests )

BOOST_AUTO_TEST_CASE( MatchDataSets_no_threads_test )
{
  BOOST_REQUIRE( MatchDataSetsTest( false ) );
}

BOOST_AUTO_TEST_CASE( MatchDataSets_with_threads_test )
{
  BOOST_REQUIRE( MatchDataSetsTest( true ) );
}

BOOST_AUTO_TEST_SUITE_END()
