/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

//TerraLib
#include <terralib_buildconfig.h>
#include <terralib/common.h>
#include <terralib/vp.h>
#include <terralib/geometry.h>
#include <terralib/dataaccess.h>
#include <terralib/dataaccess/utils/DataAccess.h>

// Boost
#include <boost/test/unit_test.hpp>
#include <boost/uuid/uuid_io.hpp>

#include <cstdio>

bool RegisterDatasetTest( bool useThreads )
{
  te::common::ConsoleProgressViewer progressViewerInstance;

  std::unique_ptr< te::da::DataSource > inDataSource1Ptr(
    te::da::DataSourceFactory::make( "OGR", "file://" TERRALIB_DATA_DIR
    "/shape/munic_2001.shp" ) );
  BOOST_REQUIRE( inDataSource1Ptr.get() != 0 );

  inDataSource1Ptr->open();

  BOOST_REQUIRE( inDataSource1Ptr->isOpened() );

  std::vector< std::string > dataSetNames1 = inDataSource1Ptr->getDataSetNames();
  BOOST_REQUIRE( !dataSetNames1.empty() );

  std::unique_ptr< te::da::DataSourceTransactor > inTransactor1Ptr =
    inDataSource1Ptr->getTransactor();

  std::unique_ptr<te::da::DataSet> inDataSet1Ptr = inTransactor1Ptr->getDataSet(
    dataSetNames1[ 0 ] );
  BOOST_REQUIRE( inDataSet1Ptr.get() != 0 );

  std::unique_ptr<te::da::DataSetType> inDsType1 =
    inTransactor1Ptr->getDataSetType(	dataSetNames1[ 0 ] );

  std::vector< te::vp::Register::TiePointT > tiePoints( 2 );
  std::get< 0 >( tiePoints[ 0 ] ) = 4326;
  std::get< 1 >( tiePoints[ 0 ] ) = -51.513929;
  std::get< 2 >( tiePoints[ 0 ] ) = 4.436487;
  std::get< 3 >( tiePoints[ 0 ] ) = 4326;
  std::get< 4 >( tiePoints[ 0 ] ) = -51.613929;
  std::get< 5 >( tiePoints[ 0 ] ) = 4.536487;
  std::get< 0 >( tiePoints[ 1 ] ) = 4326;
  std::get< 1 >( tiePoints[ 1 ] ) = -53.423175;
  std::get< 2 >( tiePoints[ 1 ] ) = -33.744662;
  std::get< 3 >( tiePoints[ 1 ] ) = 4326;
  std::get< 4 >( tiePoints[ 1 ] ) = -53.523175;
  std::get< 5 >( tiePoints[ 1 ] ) = -33.844662;
  
  const std::string outDataSetName( "RegisterDatasetTest" +
    std::to_string( useThreads ) );
  
  remove( ( outDataSetName + ".shp" ).c_str() );
  remove( ( outDataSetName + ".shx" ).c_str() );
  remove( ( outDataSetName + ".dbf" ).c_str() );
  remove( ( outDataSetName + ".prj" ).c_str() );  
  remove( ( outDataSetName + ".qix" ).c_str() );  
  
  te::core::URI outURI( "file://" + outDataSetName + ".shp" );
  
  te::da::DataAccess outDataAccess( outURI, "OGR", "" );

  BOOST_REQUIRE( 
    te::vp::Register::RegisterDataset( 
      te::vp::Register::TransByNMethod,
      tiePoints,
      2, // minNeighborTiePoints
      useThreads,
      *inDataSet1Ptr,
      *inDsType1, 
      outDataSetName,
      outDataAccess 
    )
  );

  return true;
}

BOOST_AUTO_TEST_SUITE( register_tests )

BOOST_AUTO_TEST_CASE( register_no_threads_test )
{
  BOOST_REQUIRE( RegisterDatasetTest( false ) );
}

BOOST_AUTO_TEST_CASE( register_with_threads_test )
{
  BOOST_REQUIRE( RegisterDatasetTest( true ) );
}

BOOST_AUTO_TEST_SUITE_END()
